CategoryListBox = class( Turbine.UI.Control )
function CategoryListBox:Constructor(items, name, parent)
	Turbine.UI.Control.Constructor( self )

	if not(items) or items == 0 then items = 1 end
	self.parent = parent
	self.width = 35 * 5
	self.height = math.ceil(items / 5) * 35 + 25

	self.items = 0
	--ChatOutput(name .. " " .. items .. " " .. self.height)
	self:SetSize(self.width + 10, self.height + 10)

	self.itemSlotIDs = {}

	local HeaderLabel = Turbine.UI.Label()
    HeaderLabel:SetParent(self)
    HeaderLabel:SetPosition(0, 0)
    HeaderLabel:SetSize(self.width, 25)
    HeaderLabel:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft)
    HeaderLabel:SetFont(Turbine.UI.Lotro.Font.TrajanPro14)
    HeaderLabel:SetFontStyle(Turbine.UI.FontStyle.Outline)
    HeaderLabel:SetOutlineColor(Turbine.UI.Color(0, 0, 0))
    --HeaderLabel:SetBackColor(Turbine.UI.Color(0.250, 0.250, 0.250))
    HeaderLabel:SetText(name)
    HeaderLabel:SetMouseVisible(false)

	self.itemListBox = Turbine.UI.ListBox()
	self.itemListBox:SetParent(self)
	self.itemListBox:SetMaxColumns( 5 )
	self.itemListBox:SetPosition(0, 25)
	self.itemListBox:SetSize(self.width, self.height - 25)
	--self.itemListBox:SetBackColor(Turbine.UI.Color(1,0,0))
	self.itemListBox:SetOrientation( Turbine.UI.Orientation.Horizontal )
	self.itemListBox:SetAllowDrop(true)

	self.itemListBox.DragDrop = function(sender, args)
		local shortcut = args.DragDropInfo:GetShortcut()

		if shortcut ~= nil then
			local item = self.itemListBox:GetItemAt(args.X, args.Y)
			if item then
				local index = item:GetIndex()
			
				self.parent:GetBackpack():PerformShortcutDrop(shortcut, index, Turbine.UI.Control.IsShiftKeyDown())
			else

				self.parent:PerformDragDrop(args)
			end
		end
	end
end

function CategoryListBox:ContainsItem(item)
	return self.itemListBox:ContainsItem(item)
end

function CategoryListBox:AddItem(item)
	self.items = self.items + 1
	self.itemListBox:AddItem(item)
	self:PerformLayout()
	self.itemSlotIDs[item:GetIndex()] = true
	self:Sort()
end

function CategoryListBox:ClearItems()
	self.itemListBox:ClearItems()
	self.items = 0
end

function CategoryListBox:RemoveItem(item)
	self.items = self.items - 1
	self.itemSlotIDs[item:GetIndex()] = nil
	if self.itemListBox:ContainsItem(item) then self.itemListBox:RemoveItem(item) end
	self:PerformLayout()
end

function CategoryListBox:GetItemCount()
	return(self.itemListBox:GetItemCount())
end

function CategoryListBox:GetItemIDs()
	return self.itemSlotIDs
end

function CategoryListBox:PerformLayout()
	self.height = math.ceil(self.items / 5) * 35 + 25
	self:SetSize(self.width + 10, self.height + 10)
	self.itemListBox:SetSize(self.width, self.height - 25)
	self:Sort()
end

function CategoryListBox:Sort()
	self.itemListBox:Sort(function(a, b) return a:GetName() < b:GetName() end)
end

------------------------------------------------------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------------------------------------------------------

EmptyListBox = class( Turbine.UI.Control )
function EmptyListBox:Constructor(items, name, parent)
	Turbine.UI.Control.Constructor( self )

	self.parent = parent
	self.width = 35 * 5
	self.height = 35 + 25

	self.items = 0

	self.name = name
	--ChatOutput(name .. " " .. items .. " " .. self.height)
	self:SetSize(self.width + 10, self.height + 10)

	self.shownSlot = nil
	self.itemSlots = {}

	self.HeaderLabel = Turbine.UI.Label()
    self.HeaderLabel:SetParent(self)
    self.HeaderLabel:SetPosition(0, 0)
    self.HeaderLabel:SetSize(self.width, 25)
    self.HeaderLabel:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft)
    self.HeaderLabel:SetFont(Turbine.UI.Lotro.Font.TrajanPro14)
    self.HeaderLabel:SetFontStyle(Turbine.UI.FontStyle.Outline)
    self.HeaderLabel:SetOutlineColor(Turbine.UI.Color(0, 0, 0))
    --self.HeaderLabel:SetBackColor(Turbine.UI.Color(0.250, 0.250, 0.250))
    self.HeaderLabel:SetText(self.name .. " - " .. self.items )
    self.HeaderLabel:SetMouseVisible(false)

	self.itemListBox = Turbine.UI.ListBox()
	self.itemListBox:SetParent(self)
	self.itemListBox:SetMaxColumns( 5 )
	self.itemListBox:SetPosition(0, 25)
	self.itemListBox:SetSize(self.width, self.height - 25)
	--self.itemListBox:SetBackColor(Turbine.UI.Color(1,0,0))
	self.itemListBox:SetOrientation( Turbine.UI.Orientation.Horizontal )
	self.itemListBox:SetAllowDrop(true)

	self.itemListBox.DragDrop = function(sender, args)
		local shortcut = args.DragDropInfo:GetShortcut()

		if shortcut ~= nil then
			local item = self.itemListBox:GetItemAt(args.X, args.Y)
			if item then
				local index = item:GetIndex()
			
				self.parent:GetBackpack():PerformShortcutDrop(shortcut, index, Turbine.UI.Control.IsShiftKeyDown())
			else

				self.parent:PerformDragDrop(args)
			end
		end
	end
end

function EmptyListBox:AddItem(item)
	if self.shownSlot == nil then
		self.itemListBox:AddItem(item)
		self.shownSlot = item
	end
	self.itemSlots[item] = true

	self.items = self.items + 1
	self:PerformLayout()
end

function EmptyListBox:ClearItems()
	self.itemListBox:ClearItems()
end

function EmptyListBox:ContainsItem(item)
	return self.itemSlots[item] ~= nil
end

function EmptyListBox:RemoveItem(item)
	self.items = self.items - 1
	self.itemSlots[item] = nil
	if self.shownSlot == item then
		if self.itemListBox:ContainsItem(item) then self.itemListBox:RemoveItem(item) end
		self.shownSlot = nil
		for key, value in pairs(self.itemSlots) do
			self.itemListBox:AddItem(key)
			self.shownSlot = key
			break
		end
	end
	self:PerformLayout()
end

function EmptyListBox:GetItemCount()
	return 0
end

function EmptyListBox:PerformLayout()
	if self.items ~= 1 then
		self.HeaderLabel:SetText("Empty Slots - " .. self.items )
	else
		self.HeaderLabel:SetText("Empty Slot - " .. self.items )
	end
end

function EmptyListBox:Sort()
	self.itemListBox:Sort(function(a, b) return a:GetName() < b:GetName() end)
end

function EmptyListBox:GetTopSlotID()
	if self.shownSlot then return self.shownSlot:GetIndex() end
end

function EmptyListBox:GetAllSlots()
	local result = {}
	for key, value in pairs(self.itemSlots) do
		table.insert(result, key:GetIndex())
	end
	return result, tablelength(result)
end