function ChatOutput(line)
    Turbine.Shell.WriteLine(PluginString .. tostring(line))
end

function GetRandomColour() return(Turbine.UI.Color(math.random(), math.random(), math.random())) end

function spairs(t, order)
    -- collect the keys
    local keys = {}
    for k in pairs(t) do keys[#keys+1] = k end

    -- if order function given, sort by it by passing the table and keys a, b,
    -- otherwise just sort the keys 
    if order then
        table.sort(keys, function(a,b) return order(t, a, b) end)
    else
        table.sort(keys)
    end

    -- return the iterator function
    local i = 0
    return function()
        i = i + 1
        if keys[i] then
            return keys[i], t[keys[i]]
        end
    end
end

function Round(num, numDecimalPlaces)
    if num then
        local mult = 10^(numDecimalPlaces or 0)
        local number = math.floor(num * mult + 0.5) / mult
        if (number % 1) == 0 and numDecimalPlaces ~= 0 then
            return tostring(number .. ".0")
        else
            return number
        end
    end
end

function FormatBigNumber(number)
    if number == nil then return 0 end
    local returnString = nil
    if number < 1000 then
        returnString = Round(number, 0)
    elseif number >= 1000 and number < 1000000 then
        returnString = Round((number / 1000), 1) .. "k"
    elseif number >= 1000000 and number < 1000000000 then
        returnString = Round((number / 1000000),1) .. "m"
    elseif number >= 1000000000 and number < 1000000000000 then 
        returnString = Round((number / 1000000000), 1) .. "b"
    elseif number >= 1000000000000 then
        returnString = Round((number / 1000000000000), 1) .. "t"
    else
        returnString = number
    end

    return returnString
end

function FormatTimeString(time)
    local minutes = math.floor(time / 60)
    local seconds = string.format("%02.f", math.floor(time % 60))
    local milliseconds = string.format("%02.f", math.floor(time * 100 % 100))

    return(minutes .. ":" .. seconds .. "." .. milliseconds)
end

function AddCallback(object, event, callback)
    if (object[event] == nil) then
        object[event] = callback
    else
        if (type(object[event]) == "table") then
            table.insert(object[event], callback)
        else
            object[event] = {object[event], callback}
        end
    end
    return callback
end

function RemoveCallback(object, event, callback)
    if (object[event] == callback) then
        object[event] = nil
    else
        if (type(object[event]) == "table") then
            for i = 1, #object[event] do
                if (object[event][i] == callback) then
                    table.remove(object[event], i)
                    break
                end
            end
        end
    end
end

function tablelength(T)
    local count = 0
    if T ~= nil then
        for _ in pairs(T) do count = count + 1 end
    end
    return count
end