FrameGroup = class(Turbine.UI.Window)
function FrameGroup:Constructor(x, y, width, height)
    
    Turbine.UI.Window.Constructor(self)
    self:SetSize(Turbine.UI.Display.GetWidth(), height * 6)
    self:SetPosition(x, y)
    self:SetVisible(true)
    self:SetMouseVisible(false)

    self.Frames = {}

    self.selected = nil

    self.NameToFrame = {}

    self.callback = {}

    self.player = Turbine.Gameplay.LocalPlayer.GetInstance()

    for i = 0, 5, 1 do
        self.Frames[i] = TargetFrame(self, 0, height * i, width, height, Turbine.UI.Color(0,0,0.7))
        --self:SetPlayer(i, self.player)
    end

    for i = 6, 11, 1 do
        self.Frames[i] = TargetFrame(self, Turbine.UI.Display.GetWidth() - width, height * (i - 6), width, height, Turbine.UI.Color(0.7,0,0))
        --self:SetPlayer(i, self.player)
    end

    self.HandleTarget = function(player)
        local target = player:GetTarget()

        if target and target.GetName then 
            self:SetSelected(target:GetName()) 
        elseif self.selected then
            self.Frames[self.NameToFrame[self.selected]]:ChangeState(false)
        end
    end

    self.callback["TargetChanged"] = AddCallback(self.player, "TargetChanged", function(sender, args)
        self.HandleTarget(sender)
    end)

    self.HandleTarget(self.player)
end

function FrameGroup:SetPlayer(player, class, i)

    if player == nil or player.GetName == nil then return end

    self.Frames[i]:SetTarget(player)
    self.Frames[i]:SetName(player:GetName())
    self.Frames[i]:RefreshScore()
    if class then self.Frames[i]:SetIcon(Icons[class]) end

    self.NameToFrame[player:GetName()] = i

end

function FrameGroup:SetSelected(name)

    if self.selected then self.Frames[self.NameToFrame[self.selected]]:ChangeState(false) end

    if self.NameToFrame[name] then 
        self.Frames[self.NameToFrame[name]]:ChangeState(true) 
        self.selected = name
    end

end

function FrameGroup:RemoveCallbacks()
    RemoveCallback(self.player, "TargetChanged", self.callback["TargetChanged"])
end

function FrameGroup:AddKill(name)
    if self.NameToFrame[name] then 
        self.Frames[self.NameToFrame[name]]:AddKill()
    end
end

function FrameGroup:AddDeath(name)
    if self.NameToFrame[name] then 
        self.Frames[self.NameToFrame[name]]:AddDeath()
    end
end

function FrameGroup:Revive(name)
    if self.NameToFrame[name] then 
        self.Frames[self.NameToFrame[name]]:Revive()
    end
end

function FrameGroup:ResetScore()
    for key, value in pairs(self.Frames) do
        value:ResetScore()
    end
end

function FrameGroup:TotalReset()
    for key, value in pairs(self.Frames) do
        value:TotalReset()
    end
    self.Frames = nil
    self.NameToFrame = nil

    collectgarbage()

    self.Frames = {}
    self.NameToFrame = {}

end

function FrameGroup:ResetFrames()
    for key, value in pairs(self.Frames) do
        value:ResetScore()
        value:Revive()
        value:SetTarget(nil)
        value:SetName("")
        value:SetSelected(false)
    end

    self.NameToFrame = {}
end