CombatDataHandler = class(Turbine.UI.Control)
function CombatDataHandler:Constructor()
    Turbine.UI.Control.Constructor(self)

    -- Deliver combat data somewhere
    self.CombatEvent = function(sender, event, args)
        if type(event)=="function" then
            event(args)
        else
            if type(event)=="table" then
                for i=1,#event do
                    if type(event[i])=="function" then
                        event[i](args)
                    end
                end
            end
        end
    end

    -- Saves data if there is a curren encounter going on
    self.currentEncounterData = nil
    self.encounterTable = {}

    self.oldGameTime = Turbine.Engine.GetGameTime()
    self.updateDelta = 1

    self.player = Turbine.Gameplay.LocalPlayer.GetInstance()
    self.isInCombat = self.player:IsInCombat()

    self.combatDelay = 3
    self.combatLeaveTime = 0
    self.combatEnterTime = 0

    self.currentID = 0
    self.currentViewingID = 0

    self.combatCallBack = AddCallback(self.player, "InCombatChanged", function(sender, args)
        self:CombatChangeHandling()
    end)

    self.chatHandler = ChatHandler(self)

    self.lastUpdate = 0

    self.Update = function()
        local time = Turbine.Engine.GetGameTime()
        if self.lastUpdate + 1 < time then
            if self.currentID == self.currentViewingID then
                self:UpdateCombatWindow()
            end
            self.lastUpdate = time
        end

        if self.afterCombat and self.OOCTimer < time then
            self:UpdateHistory()
            self:SetWantsUpdates(false)
            self.afterCombat = false
            --Update history
        end
    end

    self:CombatChangeHandling()
end

-- Handle the switch of combat
function CombatDataHandler:CombatChangeHandling()
    -- We got into combat
    if self.player:IsInCombat() then
        self.currentID = self.currentID + 1
        if self.currentEncounterData then
            self:UpdateHistory()
        end
        --Update history
        -- Set new data object
        self.currentEncounterData = Encounter()
        self.encounterTable[self.currentID] = self.currentEncounterData
        CombatWindowHandler:AddHistory(self.currentEncounterData, self.currentID)
        self.currentViewingID = self.currentID

        self.afterCombat = false
        self:SetWantsUpdates(true)
    else
        if self.currentEncounterData then
            self.OOCTimer = Turbine.Engine.GetGameTime() + 3
            self.afterCombat = true

            self.currentEncounterData:Stop()
        end

        --(parent, data, x, y, title, width, height, borderWidth, titleHeight, itemWidth, dotWidth, lineWidth, itemColour, dotColour, dotSelectedColour, lineColour)
        --x = PrimePlugins.RaidTools.UI.GraphWindow(nil, self.currentEncounterData:GetData()["DamageMapped"], 100, 500, "Damage Graph", 1500, 400, 3, 30, 10, 5, 3)

        --self.CombatEvent(self, self.CombatData, self.currentEncounterData:GetData())
        --self.currentEncounterData:Unload()
        --self.currentEncounterData = nil
    end
end

function CombatDataHandler:UpdateHistory()
    if self.currentEncounterData then
        CombatWindowHandler:UpdateHistory(self.currentEncounterData)
    end
end

function CombatDataHandler:HandleDamage(damage, damageType, hitType, skillName, target)
    if self.currentEncounterData then
        self.currentEncounterData:UpdateDamage(damage, damageType, hitType, skillName, target)
    end
end

function CombatDataHandler:HandleDamageTaken(damage, damageType, hitType, skillName, target)
    if self.currentEncounterData then
        self.currentEncounterData:UpdateTakenDamage(damage, damageType, hitType, skillName, target)
    end
end

function CombatDataHandler:HandleCorruption(name, target)
    if self.currentEncounterData then
        self.currentEncounterData:HandleCorruption(name, target)
    end
end

function CombatDataHandler:HandleKB(Source, Target)
    if self.currentEncounterData then
        self.currentEncounterData:UpdateKB(Source, Target)
    end
end

function CombatDataHandler:AddTemporaryMorale(Amount)
    if self.currentEncounterData then
        self.currentEncounterData:AddTemporaryMorale(Amount)
    end
end

function CombatDataHandler:HandleBuffSource(Source, SkillName, Target)
    if self.currentEncounterData then
        self.currentEncounterData:HandleBuffsSource(Source, SkillName, Target)
    end
end

function CombatDataHandler:HandleBuffTarget(Source, SkillName, Target)
    if self.currentEncounterData then
        self.currentEncounterData:HandleBuffsTarget(Source, SkillName, Target)
    end
end


function CombatDataHandler:HandleHealingReceived(HitType, SkillName, Healing, Type, Source)
    if self.currentEncounterData then
        self.currentEncounterData:HandleHealingReceived(HitType, SkillName, Healing, Type, Source)
    end
end

function CombatDataHandler:HandleHealingApplied(HitType, SkillName, Healing, Type, Source)
    if self.currentEncounterData then
        self.currentEncounterData:HandleHealingApplied(HitType, SkillName, Healing, Type, Source)
    end
end

function CombatDataHandler:GetEncounterData()
    if self.encounterTable[self.currentViewingID] then 
        return self.encounterTable[self.currentViewingID]
    end
    

end

function CombatDataHandler:UpdateCombatWindow()
    --CombatWindow
    if self.encounterTable[self.currentViewingID] then
        CombatWindowHandler:UpdateWindows(self.encounterTable[self.currentViewingID])
    end
end


function CombatDataHandler:SetViewingID(id)
    self.currentViewingID = id
    self:UpdateCombatWindow()
end