CombatWindowHandler = class(Turbine.UI.Control)
function CombatWindowHandler:Constructor()
    self.windowData = LoadTable(Turbine.DataScope.Character, "PrimeParseWindows")
    self.windows = {}
    if self.windowData == nil then
        self.windowData = {}
        self:AddWindow()
    end
    Turbine.UI.Control.Constructor(self)
    self:SetWantsKeyEvents(true)

    self.player = Turbine.Gameplay.LocalPlayer.GetInstance()
    self.icons = GetIconLUT(self.player)
    self.classIcons = GetClassLUT()

    -- DPS colour
    self.itemBarColour = Turbine.UI.Color.DarkRed
    self.itemHoverColour = Turbine.UI.Color.IndianRed

    -- Heal colour
    self.itemBarColourHeal = Turbine.UI.Color.DarkGreen
    self.itemHoverColourHeal = Turbine.UI.Color.SeaGreen

    -- Power colour
    self.itemBarColourPower = Turbine.UI.Color.DarkBlue
    self.itemHoverColourPower = Turbine.UI.Color.Blue

    -- Buff colour
    self.itemBarColourBuff = Turbine.UI.Color.Teal
    self.itemHoverColourBuff = Turbine.UI.Color(0.26,0.67,0.67)

    -- Taken colour
    self.itemBarColourTaken = Turbine.UI.Color(0.15,0.15,0.15)
    self.itemHoverColourTaken = Turbine.UI.Color(0.28,0.28,0.28)

    -- Temporary morale
    self.itemBarColourTemporary = Turbine.UI.Color.Purple
    self.itemHoverColourTemporary = Turbine.UI.Color.MediumPurple

    for key, value in pairs(self.windowData) do
        if not(self.windows[key]) then
            self.windows[key] = UI.CombatWindow(value, key)
        end
    end

    self.KeyDown = function(sender, args)
        if args.Action == 268435635 then
            for key, value in pairs(self.windows) do
                value:SetVisible(not(value:IsVisible()))
            end
        end
    end
end

function CombatWindowHandler:Unload()
    for key, value in pairs(self.windows) do
        self.windows[key] = value:Unload()
    end
    SaveTable(Turbine.DataScope.Character, "PrimeParseWindows", self.windowData)
end

function CombatWindowHandler:UpdateWindows(currentEncounterData)
    for _, combatWindow in pairs(self.windows) do
        combatWindow:Clear()
        combatWindow:UpdateTitleLabel(FormatTimeStringNoMilli((currentEncounterData.CombatEndTime or currentEncounterData.oldGameTime) - currentEncounterData.CombatStartTime))
    
        if combatWindow.mode == "Damage" then
            local DPS = FormatBigNumber(currentEncounterData.CurrentTotalDamage / ((currentEncounterData.CombatEndTime or currentEncounterData.oldGameTime) - currentEncounterData.CombatStartTime))
            combatWindow:AddListBoxItem(self.classIcons[self.player:GetClass()] or 0x410081D4, "Total Damage", string.format("%s | %s", DPS, FormatBigNumber(currentEncounterData.CurrentTotalDamage)), "", 1, self.itemBarColour, self.itemHoverColour)
            local highestValue = -1
            for name, value in spairs(currentEncounterData.SkillDamage, function(t, a, b) return t[a] > t[b] end) do
                if highestValue == -1 then
                    highestValue = value
                end
                combatWindow:AddListBoxItem(self.icons[name] or 0x410081D4, name, FormatBigNumber(value), string.format("(%s)", currentEncounterData.SkillTotalHits[name]), value / highestValue, self.itemBarColour, self.itemHoverColour)
            end
        elseif combatWindow.mode == "Targets" then
            local DPS = FormatBigNumber(currentEncounterData.CurrentTotalDamage / ((currentEncounterData.CombatEndTime or currentEncounterData.oldGameTime) - currentEncounterData.CombatStartTime))
            combatWindow:AddListBoxItem(self.classIcons[self.player:GetClass()] or 0x410081D4, "Total Damage", string.format("%s | %s", DPS, FormatBigNumber(currentEncounterData.CurrentTotalDamage)), "", 1, self.itemBarColour, self.itemHoverColour)
            local highestValue = -1
            for name, value in spairs(currentEncounterData.TargetDamage, function(t, a, b) return t[a] > t[b] end) do
                if highestValue == -1 then
                    highestValue = value
                end
                --string.format("(%s)", self.currentEncounterData.TargetHits[name])
                combatWindow:AddListBoxItem(self.icons[name] or 0x410081D4, name, FormatBigNumber(value), "", value / highestValue, self.itemBarColour, self.itemHoverColour)
            end
        elseif combatWindow.mode == "Corruptions" then
            combatWindow:AddListBoxItem(self.classIcons[self.player:GetClass()] or 0x410081D4, "Total Corruptions", FormatBigNumber(currentEncounterData.CorruptionTotal), "", 1, self.itemBarColour, self.itemHoverColour)
            local highestValue = -1
            for targetName, targetTable in spairs(currentEncounterData.TargetCorruptionBreakdown, function(t, a, b) return currentEncounterData.CorruptionsTargetTotal[a] > currentEncounterData.CorruptionsTargetTotal[b] end) do
                
                local value = currentEncounterData.CorruptionsTargetTotal[targetName]
                if highestValue == -1 then
                    highestValue = value
                end
    
                --string.format("(%s)", self.currentEncounterData.TargetHits[name])
                combatWindow:AddListBoxItem(self.icons[targetName] or 0x410081D4, targetName, value, "", value / highestValue, self.itemBarColour, self.itemHoverColour)
            end
        elseif combatWindow.mode == "Killing Blows" then
            local highestValue = -1

            for targetName, targetTable in spairs(currentEncounterData.KillingBlows, function(t, a, b) return currentEncounterData.TotalKillingBlows[a] > currentEncounterData.TotalKillingBlows[b] end) do
                
                local value = currentEncounterData.TotalKillingBlows[targetName]
                if highestValue == -1 then
                    highestValue = value
                end
    
                --string.format("(%s)", self.currentEncounterData.TargetHits[name])
                combatWindow:AddListBoxItem(self.icons[targetName] or 0x41116214, targetName, value, "", value / highestValue, self.itemBarColour, self.itemHoverColour)
            end
        elseif combatWindow.mode == "Kills" then
            local highestValue = -1

            for targetName, value in spairs(currentEncounterData.Kills, function(t, a, b) return t[a] > t[b] end) do
                if highestValue == -1 then
                    highestValue = value
                end
    
                --string.format("(%s)", self.currentEncounterData.TargetHits[name])
                --0x41116214
                combatWindow:AddListBoxItem(self.icons[targetName] or 0x41116214, targetName, value, "", value / highestValue, self.itemBarColour, self.itemHoverColour)
            end
        elseif combatWindow.mode == "Taken Damage" then
            local TPS = FormatBigNumber(currentEncounterData.DamageTaken / ((currentEncounterData.CombatEndTime or currentEncounterData.oldGameTime) - currentEncounterData.CombatStartTime))
            combatWindow:AddListBoxItem(self.classIcons[self.player:GetClass()] or 0x410081D4, "Total Taken", string.format("%s | %s", TPS, FormatBigNumber(currentEncounterData.DamageTaken)), "", 1, self.itemBarColourTaken, self.itemHoverColourTaken)
            local highestValue = -1
            for name, value in spairs(currentEncounterData.DamageTakenSkills, function(t, a, b) return t[a] > t[b] end) do
                if highestValue == -1 then
                    highestValue = value
                end
                combatWindow:AddListBoxItem(self.icons[name] or 0x410081D4, name, FormatBigNumber(value), string.format("(%s)", currentEncounterData.DamageTakenSkillTotalHits[name]), value / highestValue, self.itemBarColourTaken, self.itemHoverColourTaken)
            end
        elseif combatWindow.mode == "Taken Damage Targets" then
            local TPS = FormatBigNumber(currentEncounterData.DamageTaken / ((currentEncounterData.CombatEndTime or currentEncounterData.oldGameTime) - currentEncounterData.CombatStartTime))
            combatWindow:AddListBoxItem(self.classIcons[self.player:GetClass()] or 0x410081D4, "Total Taken", string.format("%s | %s", TPS, FormatBigNumber(currentEncounterData.DamageTaken)), "", 1, self.itemBarColourTaken, self.itemHoverColourTaken)
            local highestValue = -1
            for name, value in spairs(currentEncounterData.DamageTakenTargets, function(t, a, b) return t[a] > t[b] end) do
                if highestValue == -1 then
                    highestValue = value
                end
                --string.format("(%s)", self.currentEncounterData.TargetHits[name])
                combatWindow:AddListBoxItem(self.icons[name] or 0x410081D4, name, FormatBigNumber(value), "", value / highestValue, self.itemBarColourTaken, self.itemHoverColourTaken)
            end
        elseif combatWindow.mode == "Buffs Uptime" then
            local highestValue = -1
            local combatTime = (currentEncounterData.CombatEndTime or currentEncounterData.oldGameTime) - currentEncounterData.CombatStartTime
            for combinedName, uptime in spairs(currentEncounterData.EffectData, function(t, a, b) return t[a] > t[b] end) do
                local name, icon, stack = string.match(combinedName, "(.*)@@(.*)@@(.*)")

                if highestValue == -1 then
                    highestValue = uptime
                end
                stack = tonumber(stack)

                if uptime > 0 then
                    combatWindow:AddListBoxItem(tonumber(icon), name, string.format("%.1f%%", uptime/combatTime*100), stack > 1 and string.format("(%d)", stack) or "", uptime / highestValue, self.itemBarColourBuff, self.itemHoverColourBuff, combinedName)
                end
            end
        elseif combatWindow.mode == "Debuffs Uptime" then
            local highestValue = -1
            local combatTime = (currentEncounterData.CombatEndTime or currentEncounterData.oldGameTime) - currentEncounterData.CombatStartTime
            for combinedName, uptime in spairs(currentEncounterData.DebuffEffectData, function(t, a, b) return t[a] > t[b] end) do
                local name, icon, stack = string.match(combinedName, "(.*)@@(.*)@@(.*)")

                if highestValue == -1 then
                    highestValue = uptime
                end
                stack = tonumber(stack)

                if uptime > 0 then
                    combatWindow:AddListBoxItem(tonumber(icon), name, string.format("%.1f%%", uptime/combatTime*100), stack > 1 and string.format("(%d)", stack) or "", uptime / highestValue, self.itemBarColour, self.itemHoverColour, combinedName)
                end
            end
        elseif combatWindow.mode == "Buffs Received" then
            local highestValue = -1
            for name, value in spairs(currentEncounterData.Buffs, function(t, a, b) return t[a] > t[b] end) do
                if highestValue == -1 then
                    highestValue = value
                end
                --string.format("(%s)", self.currentEncounterData.TargetHits[name])
                combatWindow:AddListBoxItem(self.icons[name] or 0x4115B5B3, name, FormatBigNumber(value), "", value / highestValue, self.itemBarColourBuff, self.itemHoverColourBuff)
            end
        elseif combatWindow.mode == "Buffs Applied" then
            local highestValue = -1
            for name, value in spairs(currentEncounterData.BuffsApplied, function(t, a, b) return t[a] > t[b] end) do
                if highestValue == -1 then
                    highestValue = value
                end
                --string.format("(%s)", self.currentEncounterData.TargetHits[name])
                combatWindow:AddListBoxItem(self.icons[name] or 0x4115B5B3, name, FormatBigNumber(value), "", value / highestValue, self.itemBarColourBuff, self.itemHoverColourBuff)
            end
        elseif combatWindow.mode == "Healing Received" then
            local HPS = FormatBigNumber(currentEncounterData.TotalHealing / ((currentEncounterData.CombatEndTime or currentEncounterData.oldGameTime) - currentEncounterData.CombatStartTime))
            combatWindow:AddListBoxItem(self.classIcons[self.player:GetClass()] or 0x410081D4, "Total Healing Received", string.format("%s | %s", HPS, FormatBigNumber(currentEncounterData.TotalHealing)), "", 1, self.itemBarColourHeal, self.itemHoverColourHeal)
            local highestValue = -1
            for name, value in spairs(currentEncounterData.HealingReceived, function(t, a, b) return t[a] > t[b] end) do
                if highestValue == -1 then
                    highestValue = value
                end
                --string.format("(%s)", self.currentEncounterData.TargetHits[name])
                combatWindow:AddListBoxItem(self.icons[name] or 0x41019E79, name, FormatBigNumber(value), string.format("(%d)", currentEncounterData.HealingRecievedSkillHits[name]), value / highestValue, self.itemBarColourHeal, self.itemHoverColourHeal)
            end
        elseif combatWindow.mode == "Healing Received Source" then
            local HPS = FormatBigNumber(currentEncounterData.TotalHealing / ((currentEncounterData.CombatEndTime or currentEncounterData.oldGameTime) - currentEncounterData.CombatStartTime))
            combatWindow:AddListBoxItem(self.classIcons[self.player:GetClass()] or 0x410081D4, "Total Healing Received", string.format("%s | %s", HPS, FormatBigNumber(currentEncounterData.TotalHealing)), "", 1, self.itemBarColourHeal, self.itemHoverColourHeal)
            local highestValue = -1
            for name, value in spairs(currentEncounterData.HealingReceivedTotalTarget, function(t, a, b) return t[a] > t[b] end) do
                if highestValue == -1 then
                    highestValue = value
                end
                --string.format("(%s)", self.currentEncounterData.TargetHits[name])
                combatWindow:AddListBoxItem(self.icons[name] or 0x41019E79, name, FormatBigNumber(value), string.format("(%d)", currentEncounterData.HealingRecievedTargetHits[name]), value / highestValue, self.itemBarColourHeal, self.itemHoverColourHeal)
            end
        elseif combatWindow.mode == "Healing Received (Power)" then
            local HPS = FormatBigNumber(currentEncounterData.PowerTotalHealing / ((currentEncounterData.CombatEndTime or currentEncounterData.oldGameTime) - currentEncounterData.CombatStartTime))
            combatWindow:AddListBoxItem(self.classIcons[self.player:GetClass()] or 0x410081D4, "Total Power Received", string.format("%s | %s", HPS, FormatBigNumber(currentEncounterData.PowerTotalHealing)), "", 1, self.itemBarColourPower, self.itemHoverColourPower)
            local highestValue = -1
            for name, value in spairs(currentEncounterData.PowerHealingReceived, function(t, a, b) return t[a] > t[b] end) do
                if highestValue == -1 then
                    highestValue = value
                end
                --string.format("(%s)", self.currentEncounterData.TargetHits[name])
                combatWindow:AddListBoxItem(self.icons[name] or 0x41019E79, name, FormatBigNumber(value), string.format("(%d)", currentEncounterData.PowerHealingRecievedSkillHits[name]), value / highestValue, self.itemBarColourPower, self.itemHoverColourPower)
            end
        elseif combatWindow.mode == "Healing Applied" then
            local HPS = FormatBigNumber(currentEncounterData.TotalHealingApplied / ((currentEncounterData.CombatEndTime or currentEncounterData.oldGameTime) - currentEncounterData.CombatStartTime))
            combatWindow:AddListBoxItem(self.classIcons[self.player:GetClass()] or 0x410081D4, "Total Healing Applied", string.format("%s | %s", HPS, FormatBigNumber(currentEncounterData.TotalHealingApplied)), "", 1, self.itemBarColourHeal, self.itemHoverColourHeal)
            local highestValue = -1
            for name, value in spairs(currentEncounterData.HealingReceivedApplied, function(t, a, b) return t[a] > t[b] end) do
                if highestValue == -1 then
                    highestValue = value
                end
                --string.format("(%s)", self.currentEncounterData.TargetHits[name])
                combatWindow:AddListBoxItem(self.icons[name] or 0x41019E79, name, FormatBigNumber(value), string.format("(%d)", currentEncounterData.HealingRecievedSkillHitsApplied[name]), value / highestValue, self.itemBarColourHeal, self.itemHoverColourHeal)
            end
        elseif combatWindow.mode == "Healing Applied Target" then
            local HPS = FormatBigNumber(currentEncounterData.TotalHealingApplied / ((currentEncounterData.CombatEndTime or currentEncounterData.oldGameTime) - currentEncounterData.CombatStartTime))
            combatWindow:AddListBoxItem(self.classIcons[self.player:GetClass()] or 0x410081D4, "Total Healing Applied", string.format("%s | %s", HPS, FormatBigNumber(currentEncounterData.TotalHealingApplied)), "", 1, self.itemBarColourHeal, self.itemHoverColourHeal)
            local highestValue = -1
            for name, value in spairs(currentEncounterData.HealingReceivedTotalTargetApplied, function(t, a, b) return t[a] > t[b] end) do
                if highestValue == -1 then
                    highestValue = value
                end
                --string.format("(%s)", self.currentEncounterData.TargetHits[name])
                combatWindow:AddListBoxItem(self.icons[name] or 0x41019E79, name, FormatBigNumber(value), string.format("(%d)", currentEncounterData.HealingRecievedTargetHitsApplied[name]), value / highestValue, self.itemBarColourHeal, self.itemHoverColourHeal)
            end
        elseif combatWindow.mode == "Healing Received (Temporary)" then
            combatWindow:AddListBoxItem(self.classIcons[self.player:GetClass()] or 0x410081D4, "Temporary Healing Received", string.format("%s", FormatBigNumber(currentEncounterData.TemporaryMorale)), "", 1, self.itemBarColourTemporary, self.itemHoverColourTemporary)
        end
    end
end

function CombatWindowHandler:AddHistory(data, id)
    if self.windows then
        for key, value in pairs(self.windows) do
            value.historyWindow:CreateListBoxItem(data, id)
        end
    end
end

function CombatWindowHandler:UpdateHistory(data)
    if self.windows then
        for key, value in pairs(self.windows) do
            value.historyWindow:UpdateHistory(data)
        end
    end
end

function CombatWindowHandler:AddWindow()
    table.insert(self.windowData, {
        ["WindowWidth"] = 300,
        ["WindowHeight"] = 200,
    
        ["WindowX"] = Turbine.UI.Display.GetWidth() / 2 - Settings.SmallCombatWindowWidth / 2,
        ["WindowY"] = Turbine.UI.Display.GetHeight() / 2 - Settings.SmallCombatWindowHeight / 2,

        ["Mode"] = "Damage",
    })

    for key, value in pairs(self.windowData) do
        if not(self.windows[key]) then
            self.windows[key] = UI.CombatWindow(value, key)
        end
    end
end

function CombatWindowHandler:RemoveWindow(id)
    self.windows[id]:Unload()
    self.windows[id]:Remove()
    self.windows[id] = nil
    self.windowData[id] = nil
end
