CombatWindow = class(Turbine.UI.Window)
function CombatWindow:Constructor(settingsTable, id)
    self.settingsTable = settingsTable
    self.width = self.settingsTable.WindowWidth
    self.height = self.settingsTable.WindowHeight
    self.x = self.settingsTable.WindowX
    self.y = self.settingsTable.WindowY
    self.mode = self.settingsTable.Mode

    self.borderSize = 2
    self.titleHeight = 15

    self.resizeAnchorSize = 8

    self.windowID = id

    self.id = 0

    self.itemHeight = 22
    self.itemBorderColour = Turbine.UI.Color.Black
    self.itemBorderSize = 1
    
    self.borderColour = Turbine.UI.Color.Black
    self.outlineColour = Turbine.UI.Color.Black
    self.innerColour = Turbine.UI.Color(0.5, 0, 0, 0)

    self.innerWidth = self.width - 2 * self.borderSize
    self.innerHeight = self.height - 2 * self.borderSize - self.titleHeight

    self.handler = CombatHandlerObject

    Turbine.UI.Window.Constructor(self)
    self:SetSize(self.width, self.height)
    self:SetPosition(self.x, self.y)
    self:SetText("Data")
    self:SetVisible(true)
    self:SetBackColor(self.borderColour)

    self.popupWindow = PopupWindow(self)
    self.dataTypeWindow = DataTypeWindow(self)
    self.historyWindow = HistoryWindow(self)
    self.outputWindow = OutputWindow(self)

    self.titleBar = Turbine.UI.Control()
    self.titleBar:SetParent(self)
    self.titleBar:SetSize(self.width, self.titleHeight)
    self.titleBar:SetPosition(0, 0)
    self.titleBar:SetVisible(true)
    self.titleBar:SetBackColor(self.borderColour)

    self.titleLabel = Turbine.UI.Label()
    self.titleLabel:SetParent(self.titleBar)
    self.titleLabel:SetSize(self.width - self.titleHeight, self.titleHeight)
    self.titleLabel:SetPosition(19, 0)
    self.titleLabel:SetVisible(true)
    self.titleLabel:SetBackColor(self.borderColour)
    self.titleLabel:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft)
    self.titleLabel:SetFontStyle(Turbine.UI.FontStyle.Outline)
    self.titleLabel:SetOutlineColor(self.outlineColour)
    self.titleLabel:SetFont(Turbine.UI.Lotro.Font.Verdana14)
    self.titleLabel:SetText(self.mode)

    self.updateTitleLabel = Turbine.UI.Label()
    self.updateTitleLabel:SetParent(self.titleBar)
    self.updateTitleLabel:SetSize(self.width - 2, self.titleHeight)
    self.updateTitleLabel:SetPosition(0, 2)
    self.updateTitleLabel:SetVisible(true)
    self.updateTitleLabel:SetMouseVisible(false)
    self.updateTitleLabel:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleRight)
    self.updateTitleLabel:SetFontStyle(Turbine.UI.FontStyle.Outline)
    self.updateTitleLabel:SetOutlineColor(self.outlineColour)
    self.updateTitleLabel:SetFont(Turbine.UI.Lotro.Font.Verdana14)

    self.historyButton = Turbine.UI.Control()
    self.historyButton:SetParent(self)
    self.historyButton:SetSize(19, 15)
    self.historyButton:SetPosition(0, 1)
    self.historyButton:SetVisible(true)
    --self.historyButton:SetBackColor(Turbine.UI.Color.Green)
    self.historyButton:SetBackground(0x4112BE9A)
    self.historyButton:SetBlendMode(Turbine.UI.BlendMode.Overlay)

    self.historyButton.MouseClick = function()
        -- History window
        self.dataTypeWindow:SetVisible(false)
        self.outputWindow:SetVisible(false)
        self.historyWindow:Show()
    end

    
    self.outputButton = Turbine.UI.Window()
    self.outputButton:SetParent(self)
    self.outputButton:SetSize(16, 16)
    self.outputButton:SetPosition(self.width - 90, 1)
    self.outputButton:SetVisible(true)
    self.outputButton:SetMouseVisible(true)
    self.outputButton:SetOpacity(0.7)
    --self.outputButton:SetBackColor(Turbine.UI.Color.Green)
    self.outputButton:SetBackground(0x41005E95)

    self.outputButton.MouseClick = function()
        -- History window
        self.dataTypeWindow:SetVisible(false)
        self.historyWindow:SetVisible(false)

        local encounterData = self.handler:GetEncounterData()

        if encounterData then

            local highestDamTarget = ""
    
            for key, value in spairs(encounterData.TargetDamage, function(t, a, b) return t[a] > t[b] end) do
                highestDamTarget = key
                break
            end
    
            local text = string.format("<rgb=#FFFFFF>[%d:%02d:%02d] %s - (%s)</rgb> \n", encounterData.date.Hour, encounterData.date.Minute, encounterData.date.Second, highestDamTarget, FormatTimeString(encounterData.currentGameTime - encounterData.CombatStartTime))

            text = string.format("%s%s", text, self:GetOutput(encounterData))
            
            self.outputWindow:Show(text)
        end

    end

    self.innerWindow = Turbine.UI.Control()
    self.innerWindow:SetParent(self)
    self.innerWindow:SetSize(self.innerWidth, self.innerHeight)
    self.innerWindow:SetPosition(self.borderSize, self.borderSize + self.titleHeight)
    self.innerWindow:SetVisible(true)
    self.innerWindow:SetBackColor(self.innerColour)

    self.itemListBox = Turbine.UI.ListBox()
    self.itemListBox:SetParent(self.innerWindow)
    self.itemListBox:SetSize(self.innerWidth, self.innerHeight)
    self.itemListBox:SetPosition(0, 0)
    self.itemListBox:SetVisible(true)
    self.itemListBox:SetBackColor(self.innerColour)

    self.listBoxScrollbar = Turbine.UI.Lotro.ScrollBar()
    self.listBoxScrollbar:SetOrientation(Turbine.UI.Orientation.Vertical)
    self.listBoxScrollbar:SetParent(self)
    self.listBoxScrollbar:SetPosition(-100, -100)
    self.listBoxScrollbar:SetSize(0, 0)
    self.itemListBox:SetVerticalScrollBar(self.listBoxScrollbar)

    self.resizeAnchor = Turbine.UI.Window()
    self.resizeAnchor:SetParent(self)
    self.resizeAnchor:SetSize(self.resizeAnchorSize, self.resizeAnchorSize)
    self.resizeAnchor:SetPosition(self.width - self.resizeAnchorSize, self.height - self.resizeAnchorSize)
    self.resizeAnchor:SetVisible(true)
    self.resizeAnchor:SetOpacity(0.5)
    --self.resizeAnchor:SetBackColor(Turbine.UI.Color.Green)
    self.resizeAnchor:SetBackground(0x41165831)

    self.resizeAnchor.MouseDown = function(sender, args)
        sender.dragStartX = args.X
        sender.dragStartY = args.Y
        sender.dragging = true
    end
    
    self.resizeAnchor.MouseMove = function( sender, args )
        local width, height = self:GetSize()
    
        if sender.dragging then
            local newWidth = math.max(90, width + (args.X - sender.dragStartX))
            local newHeight = math.max(90, height + (args.Y - sender.dragStartY))
            self:SetSize(newWidth, newHeight)
            self.resizeAnchor:SetPosition(self:GetWidth() - self.resizeAnchor:GetWidth() - self.borderSize, self:GetHeight() - self.resizeAnchor:GetHeight() - self.borderSize)
            self.width, self.height = self:GetSize()
            self:SetLayout()
        end
    end
    
    self.resizeAnchor.MouseUp = function( sender, args )
        sender.dragging = false
    end

    self.resizeAnchor.MouseEnter = function()
        self.resizeAnchor:SetOpacity(1)
    end

    self.resizeAnchor.MouseLeave = function()
        self.resizeAnchor:SetOpacity(0.5)
    end

    self.Moving = false

    self.titleLabel.MouseClick = function(sender, args)
        if args.Button == Turbine.UI.MouseButton.Right then
            self.historyWindow:SetVisible(false)
            self.dataTypeWindow:SetVisible(not(self.dataTypeWindow:IsVisible()))
            self.outputWindow:SetVisible(false)
            self.dataTypeWindow:Reformat()
        end
    end

    self.titleLabel.MouseDown = function(sender,args)
        self.MoveX = args.X
        self.MoveY = args.Y
        self.Moving = true
    end

    self.titleLabel.MouseUp = function()
        self.Moving = false
    end

    self.titleLabel.MouseMove = function(sender,args)
        if self.Moving then
          local newLeft = self:GetLeft() - (self.MoveX - args.X)
          local newTop = self:GetTop() - (self.MoveY - args.Y)
          if newLeft < 0 then newLeft = 0 end
          if newTop < 0 then newTop = 0 end
          if newLeft > (Turbine.UI.Display.GetWidth() - self:GetWidth()) then
            newLeft = Turbine.UI.Display.GetWidth() - self:GetWidth() end
          if newTop > (Turbine.UI.Display.GetHeight() - self:GetHeight()) then
            newTop = Turbine.UI.Display.GetHeight() - self:GetHeight() end
          self:SetPosition(newLeft,newTop)
        end
    end

--0x410081D4 swords
end

function CombatWindow:AddListBoxItem(icon, name, numberString, countString, proportion, colourBar, colourHover, extraToken)
    local itemControl = self:CreateListBoxItem(icon, name, numberString, countString, proportion, colourBar, colourHover, extraToken)

    self.itemListBox:AddItem(itemControl)
end

function CombatWindow:CreateListBoxItem(icon, name, numberString, countString, proportion, colourBar, colourHover, extraToken)
    local itemControl = Turbine.UI.Control()
    itemControl:SetSize(self.innerWidth, self.itemHeight)
    itemControl:SetBackColor(self.itemBorderColour)

    local innerItemControl = Turbine.UI.Control()
    innerItemControl:SetParent(itemControl)
    innerItemControl:SetSize(self.innerWidth - self.itemBorderSize * 2, self.itemHeight - self.itemBorderSize * 2)
    innerItemControl:SetPosition(self.itemBorderSize, self.itemBorderSize)
    innerItemControl:SetBackColor(self.itemBorderColour)
    innerItemControl:SetMouseVisible(false)

    local iconControl = Turbine.UI.Control()
    iconControl:SetParent(innerItemControl)
    iconControl:SetBackground(icon)
    iconControl:SetStretchMode(2)
    iconControl:SetStretchMode(1)
    iconControl:SetSize(self.itemHeight - self.itemBorderSize * 2, self.itemHeight - self.itemBorderSize * 2)
    iconControl:SetPosition(0, 0)
    iconControl:SetMouseVisible(false)


    local barControl = Turbine.UI.Control()
    barControl:SetParent(innerItemControl)
    barControl:SetSize((self.innerWidth - self.itemBorderSize * 2) * proportion, self.itemHeight - self.itemBorderSize * 2)
    barControl:SetPosition(self.itemHeight, 0)
    barControl:SetBackColor(colourBar or Turbine.UI.Color.DarkRed)
    barControl:SetMouseVisible(false)

    local nameControl = Turbine.UI.Label()
    nameControl:SetParent(innerItemControl)
    nameControl:SetSize(self.innerWidth - self.itemBorderSize * 2 - self.itemHeight, self.itemHeight - self.itemBorderSize * 2)
    nameControl:SetPosition(self.itemHeight, 0)
    nameControl:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft)
    nameControl:SetFontStyle(Turbine.UI.FontStyle.Outline)
    nameControl:SetOutlineColor(self.outlineColour)
    nameControl:SetFont(Turbine.UI.Lotro.Font.Verdana16)
    nameControl:SetText(name)
    nameControl:SetMouseVisible(false)

    local countControl = Turbine.UI.Label()
    countControl:SetParent(innerItemControl)
    countControl:SetSize(self.innerWidth - self.itemBorderSize * 2 - self.itemHeight - 60, self.itemHeight - self.itemBorderSize * 2)
    countControl:SetPosition(self.itemHeight, 0)
    countControl:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleRight)
    countControl:SetFontStyle(Turbine.UI.FontStyle.Outline)
    countControl:SetOutlineColor(self.outlineColour)
    countControl:SetFont(Turbine.UI.Lotro.Font.Verdana16)
    countControl:SetText(countString)
    countControl:SetMouseVisible(false)

    local numberControl = Turbine.UI.Label()
    numberControl:SetParent(innerItemControl)
    numberControl:SetSize(self.innerWidth - self.itemBorderSize * 2 - self.itemHeight, self.itemHeight - self.itemBorderSize * 2)
    numberControl:SetPosition(self.itemHeight, 0)
    numberControl:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleRight)
    numberControl:SetFontStyle(Turbine.UI.FontStyle.Outline)
    numberControl:SetOutlineColor(self.outlineColour)
    numberControl:SetFont(Turbine.UI.Lotro.Font.Verdana16)
    numberControl:SetText(numberString)
    numberControl:SetMouseVisible(false)

    itemControl.MouseEnter = function()
        barControl:SetBackColor(colourHover or Turbine.UI.Color.IndianRed)
        self.popupWindow:SetVisible(true)
        self.popupWindow:HanldePopup(self.handler:GetEncounterData(), name, self.mode, extraToken)
    end

    itemControl.MouseLeave = function()
        barControl:SetBackColor(colourBar or Turbine.UI.Color.DarkRed)
        self.popupWindow:SetVisible(false)
    end

    itemControl.PositionChanged = function()
        if itemControl:GetTop() < 0 or itemControl:GetTop() + self.itemHeight > self.itemListBox:GetHeight() then
            iconControl:SetVisible(false)
        else
            iconControl:SetVisible(true)
        end 
    end

    return itemControl
end

function CombatWindow:Clear()
    for i = 1, self.itemListBox:GetItemCount() do
        self.itemListBox:GetItem(i):SetVisible(false)
    end
    self.itemListBox:ClearItems()
end

function CombatWindow:UpdateTitleLabel(text)
    self.updateTitleLabel:SetText(text)
end

function CombatWindow:Unload()
    self.settingsTable.WindowWidth, self.settingsTable.WindowHeight = self:GetSize()
    self.settingsTable.WindowX,  self.settingsTable.WindowY = self:GetPosition()
    self.settingsTable.Mode = self.mode
    return self.settingsTable
end

function CombatWindow:SetMode(mode)
    self.mode = mode
    self.titleLabel:SetText(self.mode)
    self.handler:UpdateCombatWindow()
end

function CombatWindow:GetOutput(encounterData)
    if self.mode == "Damage" then
        local DPS = FormatBigNumber(encounterData.CurrentTotalDamage / ((encounterData.CombatEndTime or encounterData.oldGameTime) - encounterData.CombatStartTime))
        return string.format("<rgb=#FF0000>Total Damage</rgb> <rgb=#FFFFFF>%s</rgb> | <rgb=#FFFFFF>%s</rgb> <rgb=#FF0000>DPS</rgb> \n", FormatBigNumber(encounterData.CurrentTotalDamage), DPS)
    elseif self.mode == "Targets" then
        local DPS = FormatBigNumber(encounterData.CurrentTotalDamage / ((encounterData.CombatEndTime or encounterData.oldGameTime) - encounterData.CombatStartTime))
        local text = string.format("<rgb=#FF0000>Total Damage</rgb> <rgb=#FFFFFF>%s</rgb> | <rgb=#FFFFFF>%s</rgb> <rgb=#FF0000>DPS</rgb> \n", FormatBigNumber(encounterData.CurrentTotalDamage), DPS)
        for name, value in spairs(encounterData.TargetDamage, function(t, a, b) return t[a] > t[b] end) do
            text = string.format("%s%s - %s \n", text, name, FormatBigNumber(value))
        end
        return text
    elseif self.mode == "Corruptions" then
        local text = string.format("Total Corruptions %s \n", FormatBigNumber(encounterData.CorruptionTotal))
        for targetName, targetTable in spairs(encounterData.TargetCorruptionBreakdown, function(t, a, b) return encounterData.CorruptionsTargetTotal[a] > encounterData.CorruptionsTargetTotal[b] end) do
            text = string.format("%s%s - %s \n", text, targetName, encounterData.CorruptionsTargetTotal[targetName])
        end
        return text
    elseif self.mode == "Killing Blows" then
        local text = ""
        for targetName, targetTable in spairs(encounterData.KillingBlows, function(t, a, b) return encounterData.TotalKillingBlows[a] > encounterData.TotalKillingBlows[b] end) do
                
            local value = encounterData.TotalKillingBlows[targetName]
            text = string.format("%s%s - %s \n", text, targetName, value)
        end
        return text
    elseif self.mode == "Kills" then
        local text = ""
        for targetName, value in spairs(encounterData.Kills, function(t, a, b) return t[a] > t[b] end) do
            text = string.format("%s%s - %s \n", text, targetName, value)
        end
        return text
    elseif self.mode == "Taken Damage" then
        local TPS = FormatBigNumber(encounterData.DamageTaken / ((encounterData.CombatEndTime or encounterData.oldGameTime) - encounterData.CombatStartTime))
        local text = string.format("<rgb=#7d7d7d>Total Taken</rgb> <rgb=#FFFFFF>%s</rgb> | <rgb=#FFFFFF>%s</rgb> <rgb=#7d7d7d>TPS</rgb> \n", FormatBigNumber(encounterData.DamageTaken), TPS)
        local highestValue = -1
        for name, value in spairs(encounterData.DamageTakenSkills, function(t, a, b) return t[a] > t[b] end) do
            if highestValue == -1 then
                highestValue = value
            end
            text = string.format("%s%s - %s \n", text, name,  FormatBigNumber(value))
        end
        return text
    elseif self.mode == "Taken Damage Targets" then
        local TPS = FormatBigNumber(encounterData.DamageTaken / ((encounterData.CombatEndTime or encounterData.oldGameTime) - encounterData.CombatStartTime))
        local text = string.format("<rgb=#7d7d7d>Total Taken</rgb> <rgb=#FFFFFF>%s</rgb> | <rgb=#FFFFFF>%s</rgb> <rgb=#7d7d7d>TPS</rgb> \n", FormatBigNumber(encounterData.DamageTaken), TPS)
        for name, value in spairs(encounterData.DamageTakenTargets, function(t, a, b) return t[a] > t[b] end) do
            text = string.format("%s%s - %s \n", text, name,  FormatBigNumber(value))
        end
        return text
    elseif self.mode == "Buffs Applied" then
        local text = ""
        for name, value in spairs(encounterData.BuffsApplied, function(t, a, b) return t[a] > t[b] end) do
            text = string.format("%s%s - %s \n", text, name, value)
        end
        return text
    elseif self.mode == "Buffs Uptime" then
        local text = ""
        local combatTime = (encounterData.CombatEndTime or encounterData.oldGameTime) - encounterData.CombatStartTime
        for combinedName, uptime in spairs(encounterData.EffectData, function(t, a, b) return t[a] > t[b] end) do
            local name, icon, stack = string.match(combinedName, "(.*)@@(.*)@@(.*)")

            if highestValue == -1 then
                highestValue = uptime
            end
            stack = tonumber(stack)

            text = string.format("%s%s (%s) - %s \n", text, name, stack, string.format("%.1f%%", uptime/combatTime*100))
        end
        return text
    elseif self.mode == "Debuffs Uptime" then
        local text = ""
        local combatTime = (encounterData.CombatEndTime or encounterData.oldGameTime) - encounterData.CombatStartTime
        for combinedName, uptime in spairs(encounterData.DebuffEffectData, function(t, a, b) return t[a] > t[b] end) do
            local name, icon, stack = string.match(combinedName, "(.*)@@(.*)@@(.*)")

            if highestValue == -1 then
                highestValue = uptime
            end
            stack = tonumber(stack)

            text = string.format("%s%s (%s) - %s \n", text, name, stack, string.format("%.1f%%", uptime/combatTime*100))
        end
        return text
    elseif self.mode == "Buffs Received" then
        local text = ""
        for name, value in spairs(encounterData.Buffs, function(t, a, b) return t[a] > t[b] end) do
            text = string.format("%s%s - %s \n", text, name, value)
        end
        return text
    elseif self.mode == "Healing Received" then
        local HPS = FormatBigNumber(encounterData.TotalHealing / ((encounterData.CombatEndTime or encounterData.oldGameTime) - encounterData.CombatStartTime))
        local text = string.format("<rgb=#00FF00>Total Healing Received</rgb> <rgb=#FFFFFF>%s</rgb> | <rgb=#FFFFFF>%s</rgb> <rgb=#00FF00>HPS</rgb> \n", FormatBigNumber(encounterData.TotalHealing), HPS)
        local highestValue = -1
        for name, value in spairs(encounterData.HealingReceived, function(t, a, b) return t[a] > t[b] end) do
            if highestValue == -1 then
                highestValue = value
            end
            text = string.format("%s%s - %s \n", text, name, FormatBigNumber(value))
        end
        return text
    elseif self.mode == "Healing Received Source" then
        local HPS = FormatBigNumber(encounterData.TotalHealing / ((encounterData.CombatEndTime or encounterData.oldGameTime) - encounterData.CombatStartTime))
        local text = string.format("<rgb=#00FF00>Total Healing Received</rgb> <rgb=#FFFFFF>%s</rgb> | <rgb=#FFFFFF>%s</rgb> <rgb=#00FF00>HPS</rgb> \n", FormatBigNumber(encounterData.TotalHealing), HPS)
        for name, value in spairs(encounterData.HealingReceivedTotalTarget, function(t, a, b) return t[a] > t[b] end) do
            text = string.format("%s%s - %s \n", text, name, FormatBigNumber(value))
        end
        return text
    elseif self.mode == "Healing Received (Power)" then
        local HPS = FormatBigNumber(encounterData.PowerTotalHealing / ((encounterData.CombatEndTime or encounterData.oldGameTime) - encounterData.CombatStartTime))
        local text = string.format("<rgb=#0000FF>Total Healing Received (Power)</rgb> <rgb=#FFFFFF>%s</rgb> | <rgb=#FFFFFF>%s</rgb> <rgb=#0000FF>HPS</rgb> \n", FormatBigNumber(encounterData.PowerTotalHealing), HPS)
        local highestValue = -1
        for name, value in spairs(encounterData.PowerHealingReceived, function(t, a, b) return t[a] > t[b] end) do
            text = string.format("%s%s - %s \n", text, name, FormatBigNumber(value))
        end
        return text
    elseif self.mode == "Healing Applied" then
        local HPS = FormatBigNumber(encounterData.TotalHealingApplied / ((encounterData.CombatEndTime or encounterData.oldGameTime) - encounterData.CombatStartTime))
        local text = string.format("<rgb=#00FF00>Total Healing Applied</rgb> <rgb=#FFFFFF>%s</rgb> | <rgb=#FFFFFF>%s</rgb> <rgb=#00FF00>HPS</rgb> \n", FormatBigNumber(encounterData.TotalHealingApplied), HPS)
        for name, value in spairs(encounterData.HealingReceivedApplied, function(t, a, b) return t[a] > t[b] end) do
            text = string.format("%s%s - %s \n", text, name, FormatBigNumber(value))
        end
        return text
    elseif self.mode == "Healing Applied Target" then
        local HPS = FormatBigNumber(encounterData.TotalHealingApplied / ((encounterData.CombatEndTime or encounterData.oldGameTime) - encounterData.CombatStartTime))
        local text = string.format("<rgb=#00FF00>Total Healing Applied</rgb> <rgb=#FFFFFF>%s</rgb> | <rgb=#FFFFFF>%s</rgb> <rgb=#00FF00>HPS</rgb> \n", FormatBigNumber(encounterData.TotalHealingApplied), HPS)
        for name, value in spairs(encounterData.HealingReceivedTotalTargetApplied, function(t, a, b) return t[a] > t[b] end) do
            text = string.format("%s%s - %s \n", text, name, FormatBigNumber(value))
        end
        return text
    elseif self.mode == "Healing Received (Temporary)" then
        return string.format("<rgb=#b700d4>Temporary Healing Received</rgb> <rgb=#FFFFFF>%s</rgb>", FormatBigNumber(encounterData.TemporaryMorale))
    end
end

function CombatWindow:Remove()
    
    self.popupWindow:Remove()
    self.dataTypeWindow:Remove()
    self.historyWindow:Remove() 

    self:SetParent(nil)
    local controlList = self:GetControls()
    for i = 1, controlList:GetCount() do
        controlList:RemoveAt(1)
    end
    for key, _ in pairs(self) do
        self[key] = nil
    end
    collectgarbage()
end

function CombatWindow:SetLayout()
    self.innerWidth = self.width - 2 * self.borderSize
    self.innerHeight = self.height - 2 * self.borderSize - self.titleHeight

    self.titleBar:SetSize(self.width, self.titleHeight)
    self.titleBar:SetPosition(0, 0)

    self.titleLabel:SetSize(self.width - 5, self.titleHeight)
    self.titleLabel:SetPosition(19, 0)

    self.updateTitleLabel:SetSize(self.width - 5, self.titleHeight)
    self.updateTitleLabel:SetPosition(0, 2)

    self.innerWindow:SetSize(self.innerWidth, self.innerHeight)
    self.innerWindow:SetPosition(self.borderSize, self.borderSize + self.titleHeight)

    self.itemListBox:SetSize(self.innerWidth, self.innerHeight)
    self.itemListBox:SetPosition(0, 0)

    self.outputButton:SetPosition(self.width - 90, 1)
end