DataTypeWindow = class(Turbine.UI.Window)
function DataTypeWindow:Constructor(parent)
    self.width = parent:GetWidth()
    self.height = parent:GetHeight()
    self.x = Settings.SmallCombatWindowX
    self.y = Settings.SmallCombatWindowY
    self.borderSize = 2
    self.titleHeight = 15

    self.innerWidth = self.width - 2 * self.borderSize
    self.innerHeight = self.height - 2 * self.borderSize - self.titleHeight

    self.itemHeight = 22
    self.itemBorderColour = Turbine.UI.Color.Black
    self.itemBorderSize = 1
    self.itemBarColour = Turbine.UI.Color.DarkRed
    self.itemHoverColour = Turbine.UI.Color.IndianRed
    
    self.borderColour = Turbine.UI.Color.Black
    self.outlineColour = Turbine.UI.Color.Black
    self.innerColour = Turbine.UI.Color(0.85, 0, 0, 0)

    self.parent = parent

    Turbine.UI.Window.Constructor(self)
    self:SetSize(self.width, self.height)
    self:SetPosition(self.x, self.y)
    self:SetText("Data")
    self:SetVisible(false)
    self:SetBackColor(self.borderColour)

    self.itemListBox = Turbine.UI.ListBox()
    self.itemListBox:SetParent(self)
    self.itemListBox:SetSize(self.width - 2 * self.borderSize, self.height - 2 * self.borderSize)
    self.itemListBox:SetPosition(self.borderSize, self.borderSize)
    self.itemListBox:SetVisible(true)
    self.itemListBox:SetBackColor(self.innerColour)

    self:CreateListBoxItem("Damage", 0x410081D4)
    self:CreateListBoxItem("Targets", 0x410081D4)
    self:CreateListBoxItem("Corruptions", 0x410081D4)
    self:CreateListBoxItem("Kills", 0x41116214)
    self:CreateListBoxItem("Killing Blows", 0x41116214)
    self:CreateListBoxItem("Taken Damage", 0x410053E6)
    self:CreateListBoxItem("Taken Damage Targets", 0x410053E6)
    self:CreateListBoxItem("Buffs Uptime", 0x4115B5B3)
    self:CreateListBoxItem("Debuffs Uptime", 0x4115B5B3)
    self:CreateListBoxItem("Buffs Received", 0x4115B5B3)
    self:CreateListBoxItem("Buffs Applied", 0x4115B5B3)
    self:CreateListBoxItem("Healing Received", 0x41019E79)
    self:CreateListBoxItem("Healing Received Source", 0x41019E79)
    self:CreateListBoxItem("Healing Applied", 0x41019E79)
    self:CreateListBoxItem("Healing Applied Target", 0x41019E79)
    self:CreateListBoxItem("Healing Received (Temporary)", 0x411188D9)
    self:CreateListBoxItem("Healing Received (Power)", 0x410EA4FC)

    -- 0x4101F8A2 tod

end

function DataTypeWindow:CreateListBoxItem(labelString, icon)
    local itemControl = Turbine.UI.Control()
    itemControl:SetSize(self.innerWidth, self.itemHeight)
    itemControl:SetBackColor(self.itemBorderColour)

    local innerItemControl = Turbine.UI.Control()
    innerItemControl:SetParent(itemControl)
    innerItemControl:SetSize(self.innerWidth - self.itemBorderSize * 2, self.itemHeight - self.itemBorderSize * 2)
    innerItemControl:SetPosition(self.itemBorderSize, self.itemBorderSize)
    innerItemControl:SetBackColor(self.innerColour)
    innerItemControl:SetMouseVisible(false)

    local iconControl = Turbine.UI.Control()
    iconControl:SetParent(innerItemControl)
    iconControl:SetBackground(icon)
    iconControl:SetStretchMode(2)
    iconControl:SetStretchMode(1)
    iconControl:SetSize(self.itemHeight - self.itemBorderSize * 2, self.itemHeight - self.itemBorderSize * 2)
    iconControl:SetPosition(0, 0)
    iconControl:SetMouseVisible(false)

    local nameControl = Turbine.UI.Label()
    nameControl:SetParent(innerItemControl)
    nameControl:SetSize(self.innerWidth - self.itemBorderSize * 2, self.itemHeight - self.itemBorderSize * 2)
    nameControl:SetPosition(self.itemHeight, 0)
    nameControl:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft)
    nameControl:SetFontStyle(Turbine.UI.FontStyle.Outline)
    nameControl:SetOutlineColor(self.outlineColour)
    nameControl:SetFont(Turbine.UI.Lotro.Font.Verdana16)
    nameControl:SetText(labelString)
    nameControl:SetMouseVisible(false)

    --[[
    local numberControl = Turbine.UI.Label()
    numberControl:SetParent(innerItemControl)
    numberControl:SetSize(self.innerWidth - self.itemBorderSize * 2 - self.itemHeight, self.itemHeight - self.itemBorderSize * 2)
    numberControl:SetPosition(self.itemHeight, 0)
    numberControl:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleRight)
    numberControl:SetFontStyle(Turbine.UI.FontStyle.Outline)
    numberControl:SetOutlineColor(self.outlineColour)
    numberControl:SetFont(Turbine.UI.Lotro.Font.Verdana16)
    numberControl:SetText(numberString)
    numberControl:SetMouseVisible(false)
    --]]

    itemControl.MouseClick = function()
        self:SetVisible(false)
        self.parent:SetMode(labelString)
    end

    self.itemListBox:AddItem(itemControl)
end

function DataTypeWindow:Reformat()
    self:SetWidth(self.parent:GetWidth())
    self.itemListBox:SetHeight(self.itemListBox:GetItemCount() * self.itemHeight)
    self:SetHeight(self.itemListBox:GetItemCount() * self.itemHeight + 2 * self.borderSize)
    self:SetPosition(self.parent:GetLeft(), self.parent:GetTop() - (self.itemListBox:GetItemCount() * self.itemHeight + 2 * self.borderSize))
    self:SetZOrder(100)
end

function DataTypeWindow:Remove()
    self:SetParent(nil)
    local controlList = self:GetControls()
    for i = 1, controlList:GetCount() do
        controlList:RemoveAt(1)
    end
    for key, _ in pairs(self) do
        self[key] = nil
    end
    collectgarbage()
end