DataWindow = class(Turbine.UI.Window)
function DataWindow:Constructor()
    self.width = 1900
    self.height = 400
    local x = Turbine.UI.Display.GetWidth() / 2 - self.width / 2
    local y = Turbine.UI.Display.GetHeight() / 2 - self.height / 2
    local colour = Turbine.UI.Color(1, 1, 1)
    Turbine.UI.Window.Constructor(self)
    self:SetSize(self.width, self.height)
    self:SetPosition(x,y)
    self:SetText("Data")
    self:SetVisible(false)

    self.data = {}

    self.currentGraph = nil
    self.currentDate = 0

    -- PrimeUI Options treeview
    self.DataTreeview = Turbine.UI.TreeView()
    self.DataTreeview:SetParent(self)
    self.DataTreeview:SetSize(200 - 10, self.height)
    self.DataTreeview:SetIndentationWidth(0)

    self.PrimeUIGeneralOptionsScrollbar = Turbine.UI.Lotro.ScrollBar()
    self.PrimeUIGeneralOptionsScrollbar:SetOrientation(Turbine.UI.Orientation.Vertical)
    self.PrimeUIGeneralOptionsScrollbar:SetParent(self)
    self.PrimeUIGeneralOptionsScrollbar:SetPosition(self.DataTreeview:GetWidth(), 27)
    self.PrimeUIGeneralOptionsScrollbar:SetSize(10, self.height - 27)
    self.PrimeUIGeneralOptionsScrollbar:SetBackColor(Turbine.UI.Color.Black)
    self.DataTreeview:SetVerticalScrollBar(self.PrimeUIGeneralOptionsScrollbar)

    self.PrimeUIGeneralOptionsTreeNodesInControl = self.DataTreeview:GetNodes()
    self.dataHeader = PrimePlugins.RaidTools.PrimeUITools.PrimeUIOptions.HeaderNode(self.DataTreeview:GetWidth(), "Combat Data", Turbine.UI.ContentAlignment.MiddleCenter)
    self.PrimeUIGeneralOptionsTreeNodesInControl:Add(self.dataHeader)

    self.combatCallback = AddCallback(CombatHandlerObject, "CombatData", function(args)
        self.data[args.CombatStartTime] = args["PowerMapped"]
        self:AddOverviewTab(args.Date, args.CombatStartTime)
    end)

    -- Move logic
    self.Moving = false

    self.dataHeader.MouseDown = function(sender,args)
        self.MoveX = args.X
        self.MoveY = args.Y
        self.Moving = true
    end
    
    self.dataHeader.MouseUp = function()
        self.Moving = false
    end
    
    self.dataHeader.MouseMove = function(sender,args)
        if self.Moving then
            local newLeft = self:GetLeft() - (self.MoveX - args.X)
            local newTop = self:GetTop() - (self.MoveY - args.Y)
            self:SetPosition(newLeft,newTop)
        end
    end
end

function DataWindow:AddOverviewTab(date, startTime)
    local overviewTab = PrimePlugins.RaidTools.PrimeUITools.PrimeUIOptions.HeaderNode(self.DataTreeview:GetWidth(), string.format("%d/%d %d:%d.%d", date.Day, date.Month, date.Hour, date.Minute, date.Second), nil, Turbine.UI.Lotro.Font.VerdanaBold16)

    overviewTab.MouseClick = function()
        if self.currentDate == startTime then return end
        if self.currentGraph ~= nil then self.currentGraph:Unload(); self.currentGraph = nil end
        self.currentGraph = PrimePlugins.RaidTools.UI.GraphWindow(self, self.data[startTime], 200, 0, string.format("Damage Graph from %d/%d %d:%d.%d", date.Day, date.Month, date.Hour, date.Minute, date.Second), 1800, 400, 3, 30, 10, 5, 3)
        self.currentDate = startTime
        if self.selectedButton ~= nil then
            self.selectedButton.innerBlock:SetBackColor(Turbine.UI.Color(0.01,0.31,0.31))
        end
        self.selectedButton = overviewTab
        overviewTab.innerBlock:SetBackColor(Turbine.UI.Color(0,0.1,0.1))
    end

    self.PrimeUIGeneralOptionsTreeNodesInControl:Add(overviewTab)
end