HistoryWindow = class(Turbine.UI.Window)
function HistoryWindow:Constructor(parent)
    self.width = parent:GetWidth()
    self.height = parent:GetHeight()
    self.x = Settings.SmallCombatWindowX
    self.y = Settings.SmallCombatWindowY
    self.borderSize = 2
    self.titleHeight = 15

    self.historyItems = {}

    self.innerWidth = self.width - 2 * self.borderSize
    self.innerHeight = self.height - 2 * self.borderSize - self.titleHeight

    self.itemHeight = 22
    self.itemBorderColour = Turbine.UI.Color.Black
    self.itemBorderSize = 1
    self.itemBarColour = Turbine.UI.Color.DarkRed
    self.itemHoverColour = Turbine.UI.Color.IndianRed
    
    self.borderColour = Turbine.UI.Color.Black
    self.outlineColour = Turbine.UI.Color.Black
    self.innerColour = Turbine.UI.Color(0.85, 0, 0, 0)

    self.parent = parent

    Turbine.UI.Window.Constructor(self)
    self:SetSize(self.width, self.height)
    self:SetPosition(self.x, self.y)
    self:SetText("Data")
    self:SetVisible(false)
    self:SetBackColor(self.borderColour)

    self.itemListBox = Turbine.UI.ListBox()
    self.itemListBox:SetParent(self)
    self.itemListBox:SetSize(self.width - 2 * self.borderSize, self.height - 2 * self.borderSize)
    self.itemListBox:SetPosition(self.borderSize, self.borderSize)
    self.itemListBox:SetVisible(true)
    self.itemListBox:SetBackColor(self.innerColour)

    self.listBoxScrollbar = Turbine.UI.Lotro.ScrollBar()
    self.listBoxScrollbar:SetOrientation(Turbine.UI.Orientation.Vertical)
    self.listBoxScrollbar:SetParent(self)
    self.listBoxScrollbar:SetPosition(-100, -100)
    self.listBoxScrollbar:SetSize(0, 0)
    self.itemListBox:SetVerticalScrollBar(self.listBoxScrollbar)

    self:CreateFunctionListboxItem("Delete Window", function() CombatWindowHandler:RemoveWindow(self.parent.windowID) end)
end

function HistoryWindow:CreateListBoxItem(encounterData, id)
    local itemControl = Turbine.UI.Control()
    itemControl:SetSize(self.innerWidth, self.itemHeight)
    itemControl:SetBackColor(self.itemBorderColour)

    itemControl.innerItemControl = Turbine.UI.Control()
    itemControl.innerItemControl:SetParent(itemControl)
    itemControl.innerItemControl:SetSize(self.innerWidth - self.itemBorderSize * 2, self.itemHeight - self.itemBorderSize * 2)
    itemControl.innerItemControl:SetPosition(self.itemBorderSize, self.itemBorderSize)
    itemControl.innerItemControl:SetBackColor(self.innerColour)
    itemControl.innerItemControl:SetMouseVisible(false)

    itemControl.nameControl = Turbine.UI.Label()
    itemControl.nameControl:SetParent(itemControl.innerItemControl)
    itemControl.nameControl:SetSize(self.innerWidth - self.itemBorderSize * 2, self.itemHeight - self.itemBorderSize * 2)
    itemControl.nameControl:SetPosition(0, 0)
    itemControl.nameControl:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft)
    itemControl.nameControl:SetFontStyle(Turbine.UI.FontStyle.Outline)
    itemControl.nameControl:SetOutlineColor(self.outlineColour)
    itemControl.nameControl:SetFont(Turbine.UI.Lotro.Font.Verdana16)
    itemControl.nameControl:SetText(string.format("[%d:%02d:%02d]", encounterData.date.Hour, encounterData.date.Minute, encounterData.date.Second))
    itemControl.nameControl:SetMouseVisible(false)

    --[[
    local numberControl = Turbine.UI.Label()
    numberControl:SetParent(innerItemControl)
    numberControl:SetSize(self.innerWidth - self.itemBorderSize * 2 - self.itemHeight, self.itemHeight - self.itemBorderSize * 2)
    numberControl:SetPosition(self.itemHeight, 0)
    numberControl:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleRight)
    numberControl:SetFontStyle(Turbine.UI.FontStyle.Outline)
    numberControl:SetOutlineColor(self.outlineColour)
    numberControl:SetFont(Turbine.UI.Lotro.Font.Verdana16)
    numberControl:SetText(numberString)
    numberControl:SetMouseVisible(false)
    --]]

    itemControl.MouseClick = function()
        self:SetVisible(false)
        CombatHandlerObject:SetViewingID(id)
    end

    self.historyItems[encounterData.CombatStartTime] = itemControl

    self.itemListBox:AddItem(itemControl)
end

function HistoryWindow:UpdateHistory(encounterData)
    if self.historyItems[encounterData.CombatStartTime] then

        local highestDamTarget = ""

        for key, value in spairs(encounterData.TargetDamage, function(t, a, b) return t[a] > t[b] end) do
            highestDamTarget = key
            break
        end

        local text = string.format("[%d:%02d:%02d] (%s) - %s", encounterData.date.Hour, encounterData.date.Minute, encounterData.date.Second, FormatTimeString((encounterData.currentGameTime or Turbine.Engine.GetGameTime()) - encounterData.CombatStartTime), highestDamTarget)
        self.historyItems[encounterData.CombatStartTime].nameControl:SetText(text)
    end
end

function HistoryWindow:CreateFunctionListboxItem(name, func)
    local itemControl = Turbine.UI.Control()
    itemControl:SetSize(self.innerWidth, self.itemHeight)
    itemControl:SetBackColor(self.itemBorderColour)

    local innerItemControl = Turbine.UI.Control()
    innerItemControl:SetParent(itemControl)
    innerItemControl:SetSize(self.innerWidth - self.itemBorderSize * 2, self.itemHeight - self.itemBorderSize * 2)
    innerItemControl:SetPosition(self.itemBorderSize, self.itemBorderSize)
    innerItemControl:SetBackColor(self.innerColour)
    innerItemControl:SetMouseVisible(false)

    local nameControl = Turbine.UI.Label()
    nameControl:SetParent(innerItemControl)
    nameControl:SetSize(self.innerWidth - self.itemBorderSize * 2, self.itemHeight - self.itemBorderSize * 2)
    nameControl:SetPosition(0, 0)
    nameControl:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft)
    nameControl:SetFontStyle(Turbine.UI.FontStyle.Outline)
    nameControl:SetOutlineColor(self.outlineColour)
    nameControl:SetFont(Turbine.UI.Lotro.Font.Verdana16)
    nameControl:SetText(name)
    nameControl:SetMouseVisible(false)

    --[[
    local numberControl = Turbine.UI.Label()
    numberControl:SetParent(innerItemControl)
    numberControl:SetSize(self.innerWidth - self.itemBorderSize * 2 - self.itemHeight, self.itemHeight - self.itemBorderSize * 2)
    numberControl:SetPosition(self.itemHeight, 0)
    numberControl:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleRight)
    numberControl:SetFontStyle(Turbine.UI.FontStyle.Outline)
    numberControl:SetOutlineColor(self.outlineColour)
    numberControl:SetFont(Turbine.UI.Lotro.Font.Verdana16)
    numberControl:SetText(numberString)
    numberControl:SetMouseVisible(false)
    --]]

    itemControl.MouseClick = function()
        func()
    end

    self.itemListBox:AddItem(itemControl)
end


function HistoryWindow:Reformat()
    self:SetWidth(self.parent:GetWidth())
    self.itemListBox:SetHeight(math.min(self.itemListBox:GetItemCount(), 20) * self.parent.itemHeight)
    self:SetHeight(math.min(self.itemListBox:GetItemCount(), 20) * self.parent.itemHeight + 2 * self.parent.borderSize)
    self:SetPosition(self.parent:GetLeft(), self.parent:GetTop() - (math.min(self.itemListBox:GetItemCount(), 20) * self.parent.itemHeight + 2 * self.parent.borderSize))
    self:SetZOrder(100)
end

function HistoryWindow:Show()
    if self:IsVisible() then
        self:SetVisible(false)
    else
        self:Reformat()
        self:SetVisible(true)
    end
end

function HistoryWindow:Remove()
    self:SetParent(nil)
    local controlList = self:GetControls()
    for i = 1, controlList:GetCount() do
        controlList:RemoveAt(1)
    end
    for key, _ in pairs(self) do
        self[key] = nil
    end
    collectgarbage()
end