OutputWindow = class(Turbine.UI.Window)
function OutputWindow:Constructor(parent)
    self.width = parent:GetWidth()
    self.height = parent:GetHeight()
    self.x = Settings.SmallCombatWindowX
    self.y = Settings.SmallCombatWindowY
    self.borderSize = 2
    self.titleHeight = 15

    self.historyItems = {}

    self.innerWidth = self.width - 2 * self.borderSize
    self.innerHeight = self.height - 2 * self.borderSize - self.titleHeight

    self.itemHeight = 22
    self.itemBorderColour = Turbine.UI.Color.Black
    self.itemBorderSize = 1
    self.itemBarColour = Turbine.UI.Color.DarkRed
    self.itemHoverColour = Turbine.UI.Color.IndianRed
    
    self.borderColour = Turbine.UI.Color.Black
    self.outlineColour = Turbine.UI.Color.Black
    self.innerColour = Turbine.UI.Color(0.85, 0, 0, 0)

    self.parent = parent

    Turbine.UI.Window.Constructor(self)
    self:SetSize(self.width, self.height)
    self:SetPosition(self.x, self.y)
    self:SetText("Data")
    self:SetVisible(false)
    self:SetBackColor(self.borderColour)

    self.itemListBox = Turbine.UI.ListBox()
    self.itemListBox:SetParent(self)
    self.itemListBox:SetSize(self.width - 2 * self.borderSize, self.height - 2 * self.borderSize)
    self.itemListBox:SetPosition(self.borderSize, self.borderSize)
    self.itemListBox:SetVisible(true)
    self.itemListBox:SetBackColor(self.innerColour)

    self.listBoxScrollbar = Turbine.UI.Lotro.ScrollBar()
    self.listBoxScrollbar:SetOrientation(Turbine.UI.Orientation.Vertical)
    self.listBoxScrollbar:SetParent(self)
    self.listBoxScrollbar:SetPosition(-100, -100)
    self.listBoxScrollbar:SetSize(0, 0)
    self.itemListBox:SetVerticalScrollBar(self.listBoxScrollbar)

    self.chatTypes = {
        ["Say"] = "/say",
        ["Fellowship"] = "/f",
        ["Raid"] = "/ra",
        ["Kinship"] = "/k",
    }
    self.outputText = ""
    self.shortcutItem = self:AddListboxItem(string.format("Channel (%s)", Settings.Output))
end

function OutputWindow:AddListboxItem(name)
    local itemControl = Turbine.UI.Control()
    itemControl:SetSize(self.innerWidth, self.itemHeight)
    itemControl:SetBackColor(self.itemBorderColour)

    local innerItemControl = Turbine.UI.Control()
    innerItemControl:SetParent(itemControl)
    innerItemControl:SetSize(self.innerWidth - self.itemBorderSize * 2, self.itemHeight - self.itemBorderSize * 2)
    innerItemControl:SetPosition(self.itemBorderSize, self.itemBorderSize)
    innerItemControl:SetBackColor(self.innerColour)
    innerItemControl:SetMouseVisible(false)

    itemControl.nameControl = Turbine.UI.Label()
    itemControl.nameControl:SetParent(innerItemControl)
    itemControl.nameControl:SetSize(150 - self.itemBorderSize * 2, self.itemHeight - self.itemBorderSize * 2)
    itemControl.nameControl:SetPosition(0, 0)
    itemControl.nameControl:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft)
    itemControl.nameControl:SetFontStyle(Turbine.UI.FontStyle.Outline)
    itemControl.nameControl:SetOutlineColor(self.outlineColour)
    itemControl.nameControl:SetFont(Turbine.UI.Lotro.Font.Verdana16)
    itemControl.nameControl:SetText(name)

    itemControl.menu = Turbine.UI.ContextMenu()
    itemControl.menuItems = itemControl.menu:GetItems()



    for key, value in pairs(self.chatTypes) do
        local menuItem = Turbine.UI.MenuItem(key)
        menuItem.Click = function()
            Settings.Output = key
            itemControl.nameControl:SetText(string.format("Channel (%s)", Settings.Output))
            self.shortcutItem.quickslot:SetShortcut(Turbine.UI.Lotro.Shortcut(Turbine.UI.Lotro.ShortcutType.Alias, string.format("%s %s", self.chatTypes[Settings.Output], self.outputText)))
        end

        itemControl.menuItems:Add(menuItem)
    end

    itemControl.quickslot = Turbine.UI.Lotro.Quickslot()
    itemControl.quickslot:SetParent(innerItemControl)
    itemControl.quickslot:SetSize(self.innerWidth - self.itemBorderSize * 2 - 150, self.itemHeight - self.itemBorderSize * 2)
    itemControl.quickslot:SetPosition(150, 0)

    itemControl.nameControl.MouseClick = function()
        itemControl.menu:ShowMenu()
    end

    self.itemListBox:AddItem(itemControl)

    return itemControl
end


function OutputWindow:Reformat()
    self:SetWidth(self.parent:GetWidth())
    self.itemListBox:SetHeight(self.parent.itemHeight)
    self:SetHeight(self.parent.itemHeight + 2 * self.parent.borderSize)
    self:SetPosition(self.parent:GetLeft(), self.parent:GetTop() - ( self.parent.itemHeight + 2 * self.parent.borderSize))
    self:SetZOrder(100)
end

function OutputWindow:Show(output)
    if self:IsVisible() then
        self:SetVisible(false)
    else
        output = output:sub(1, 256)
        self.outputText = output
        self.shortcutItem.quickslot:SetShortcut(Turbine.UI.Lotro.Shortcut(Turbine.UI.Lotro.ShortcutType.Alias, string.format("%s %s", self.chatTypes[Settings.Output], self.outputText)))
        self:Reformat()
        self:SetVisible(true)
    end
end

function OutputWindow:Remove()
    self:SetParent(nil)
    local controlList = self:GetControls()
    for i = 1, controlList:GetCount() do
        controlList:RemoveAt(1)
    end
    for key, _ in pairs(self) do
        self[key] = nil
    end
    collectgarbage()
end