PopupWindow = class(Turbine.UI.Window)
function PopupWindow:Constructor(parent)
    self.width = parent:GetWidth()
    self.height = parent:GetHeight()
    self.x = Turbine.UI.Display.GetWidth() / 2 - self.width / 2
    self.y = Turbine.UI.Display.GetHeight() / 2 - self.height / 2
    self.borderSize = 2
    self.titleHeight = 15

    self.innerWidth = self.width - 2 * self.borderSize
    self.innerHeight = self.height - 2 * self.borderSize - self.titleHeight

    self.itemHeight = 22
    self.itemBorderColour = Turbine.UI.Color.Black
    self.itemBorderSize = 1
    self.itemBarColour = Turbine.UI.Color.DarkRed
    self.itemHoverColour = Turbine.UI.Color.IndianRed
    
    self.borderColour = Turbine.UI.Color.Black
    self.outlineColour = Turbine.UI.Color.Black
    self.innerColour = Turbine.UI.Color(1, 0, 0, 0)

    self.parent = parent

    self.hitTypeTable = {
        [""] = "Hit",
        ["critical "] = "Critical",
        ["devastating "] = "Devastating",
        ["partially blocked "] = "Block (partial)",
        ["partially parried "] = "Parry (partial)",
        ["partially evaded "] = "Evade (partial)",
        ["blocked"] = "Block",
        ["parried"] = "Parry",
        ["evaded"] = "Evade",
        ["resisted"] = "Resist",
        ["immune"] = "Immune",
    }

    self.hitTypeTableOrder = {
        [""] = 1,
        ["critical "] = 2,
        ["devastating "] = 3,
        ["blocked"] = 4,
        ["partially blocked "] = 5,
        ["parried"] = 6,
        ["partially parried "] = 7,
        ["evaded"] = 8,
        ["partially evaded "] = 9,



        ["resisted"] = 10,
        ["immune"] = 11,
    }


    Turbine.UI.Window.Constructor(self)
    self:SetSize(self.width, self.height)
    self:SetPosition(self.x, self.y - self.height - 10)

    self:SetText("Data")
    self:SetVisible(false)
    self:SetBackColor(self.borderColour)

    self.itemListBox = Turbine.UI.ListBox()
    self.itemListBox:SetParent(self)
    self.itemListBox:SetSize(self.width - 2 * self.borderSize, self.height - 2 * self.borderSize)
    self.itemListBox:SetPosition(self.borderSize, self.borderSize)
    self.itemListBox:SetVisible(true)
    self.itemListBox:SetBackColor(self.innerColour)

    -- [[
    self.specialPopupAction = {
        ["Total Damage"] = "SetupWindow",
        ["Total Corruptions"] = "SetupTotalCorruptionWindow",
        ["Total Taken"] = "SetupTotalTakenWindow",
        ["Total Healing Received"] = "TotalHealingReceivedWindow",
        ["Total Power Received"] = "PowerTotalHealingReceivedWindow",
        ["Total Healing Applied"] = "TotalHealingAppliedWindow",
    }

    self.popupAction = {
        ["Damage"]  = "SetupSkillWindow",
        ["Target"] = "SetupTargetSkillWindow",
        ["Corruptions"] = "SetupCorruptionWindow",
        ["Killing Blows"] = "SetupKillingBlowsWindow",
        ["Kills"] = "SetupKillsWindow",
        ["Taken Damage"] = "SetupSkillTakenWindow",
        ["Taken Damage Targets"] = "SetupTargetTakenWindow",
        ["Buffs Applied"] = "SetBuffsAppliedWindow",
        ["Buffs Received"] = "SetBuffsReceivedWindow",
        ["Healing Received"] = "HealingReceivedWindow",
        ["Healing Received Source"] = "HealingReceivedTargetWindow",
        ["Healing Received (Power)"] = "PowerHealingReceivedWindow",
        ["Healing Applied"] = "HealingAppliedWindow",
        ["Healing Applied Target"] = "HealingAppliedTargetWindow",
    }
    --]]
end

function PopupWindow:CreateListBoxItem(labelString, numberString)
    local itemControl = Turbine.UI.Control()
    itemControl:SetSize(self.innerWidth, self.itemHeight)
    itemControl:SetBackColor(self.itemBorderColour)

    local innerItemControl = Turbine.UI.Control()
    innerItemControl:SetParent(itemControl)
    innerItemControl:SetSize(self.innerWidth - self.itemBorderSize * 2, self.itemHeight - self.itemBorderSize * 2)
    innerItemControl:SetPosition(self.itemBorderSize, self.itemBorderSize)
    innerItemControl:SetBackColor(self.innerColour)
    innerItemControl:SetMouseVisible(false)

    local nameControl = Turbine.UI.Label()
    nameControl:SetParent(innerItemControl)
    nameControl:SetSize(self.innerWidth - self.itemBorderSize * 2, self.itemHeight - self.itemBorderSize * 2)
    nameControl:SetPosition(0, 0)
    nameControl:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft)
    nameControl:SetFontStyle(Turbine.UI.FontStyle.Outline)
    nameControl:SetOutlineColor(self.outlineColour)
    nameControl:SetFont(Turbine.UI.Lotro.Font.Verdana16)
    nameControl:SetText(labelString)
    nameControl:SetMouseVisible(false)

    local numberControl = Turbine.UI.Label()
    numberControl:SetParent(innerItemControl)
    numberControl:SetSize(self.innerWidth - self.itemBorderSize * 2 - self.itemHeight, self.itemHeight - self.itemBorderSize * 2)
    numberControl:SetPosition(self.itemHeight, 0)
    numberControl:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleRight)
    numberControl:SetFontStyle(Turbine.UI.FontStyle.Outline)
    numberControl:SetOutlineColor(self.outlineColour)
    numberControl:SetFont(Turbine.UI.Lotro.Font.Verdana16)
    numberControl:SetText(numberString)
    numberControl:SetMouseVisible(false)

    self.itemListBox:AddItem(itemControl)
end

function PopupWindow:HanldePopup(encounterData, name, mode, extraToken)
    --[[
    if self.specialPopupAction[name] then
        self[self.specialPopupAction[name] ](encounterData)
    elseif self.popupAction[mode] then
        self[self.popupAction[mode] ](encounterData, name)
    end
    --]]

    if name == "Total Damage" then
        self:SetupWindow(encounterData)
    elseif mode == "Damage" then
        self:SetupSkillWindow(encounterData, name)
    elseif mode == "Targets" then
        self:SetupTargetSkillWindow(encounterData, name)
    elseif name == "Total Corruptions" then
        self:SetupTotalCorruptionWindow(encounterData)
    elseif mode == "Corruptions" then
        self:SetupCorruptionWindow(encounterData, name)
    elseif mode == "Killing Blows" then
        self:SetupKillingBlowsWindow(encounterData, name)
    elseif mode == "Kills" then
        self:SetupKillsWindow(encounterData, name)
    elseif name == "Total Taken" then
        self:SetupTotalTakenWindow(encounterData)
    elseif mode == "Taken Damage" then
        self:SetupSkillTakenWindow(encounterData, name)
    elseif mode == "Taken Damage Targets" then
        self:SetupTargetTakenWindow(encounterData, name)
    elseif mode == "Buffs Applied" then
        self:SetBuffsAppliedWindow(encounterData, name)
    elseif mode == "Buffs Uptime" then
        self:SetBuffsUptimeWindow(encounterData, name, extraToken)
    elseif mode == "Debuffs Uptime" then
        self:SetDebuffsUptimeWindow(encounterData, name, extraToken)
    elseif mode == "Buffs Received" then
        self:SetBuffsReceivedWindow(encounterData, name)
    elseif name == "Total Healing Received" then
        self:TotalHealingReceivedWindow(encounterData)
    elseif mode == "Healing Received" then
        self:HealingReceivedWindow(encounterData, name)
    elseif mode == "Healing Received Source" then
        self:HealingReceivedTargetWindow(encounterData, name)
    elseif name == "Total Power Received" then
        self:PowerTotalHealingReceivedWindow(encounterData)
    elseif mode == "Healing Received (Power)" then
        self:PowerHealingReceivedWindow(encounterData, name)
    elseif name == "Total Healing Applied" then
        self:TotalHealingAppliedWindow(encounterData)
    elseif mode == "Healing Applied" then
        self:HealingAppliedWindow(encounterData, name)
    elseif mode == "Healing Applied Target" then
        self:HealingAppliedTargetWindow(encounterData, name)
    elseif mode == "Healing Received (Temporary)" then
        self:TemporaryMoraleWindow(encounterData, name)
    end
end

function PopupWindow:SetupSkillWindow(encounterData, skillname)
    if not(encounterData) or not(encounterData.SkillTotalHits[skillname]) then
        self:SetVisible(false)
        return 
    end
    self.itemListBox:ClearItems()

    self:CreateListBoxItem(skillname, "")
    self:CreateListBoxItem("Highest hit", FormatBigNumber(encounterData.SkillMaxDamage[skillname]))
    self:CreateListBoxItem("Total hits", encounterData.SkillTotalHits[skillname])
    local missedItems = encounterData.SkillTotalHits[skillname] - (encounterData.SkillHits[skillname][""] + encounterData.SkillHits[skillname]["critical "] + encounterData.SkillHits[skillname]["devastating "] + encounterData.SkillHits[skillname]["partially evaded "] + encounterData.SkillHits[skillname]["partially parried "] + encounterData.SkillHits[skillname]["partially blocked "])
    self:CreateListBoxItem("Missed", string.format("(%d) %.1f%%",missedItems, missedItems/encounterData.SkillTotalHits[skillname]*100))

    self:CreateListBoxItem("Critical hits", string.format("(%d) %.1f%%", encounterData.SkillHits[skillname]["critical "], encounterData.SkillHits[skillname]["critical "]/encounterData.SkillTotalHits[skillname]*100))
    self:CreateListBoxItem("Devastating hits", string.format("(%d) %.1f%%", encounterData.SkillHits[skillname]["devastating "], encounterData.SkillHits[skillname]["devastating "]/encounterData.SkillTotalHits[skillname]*100))
    self:CreateListBoxItem("Total Damage", FormatBigNumber(encounterData.SkillDamage[skillname]))

    for key, value in pairs(encounterData.SkillDamageTypes[skillname]) do
        if value > 0 then
            self:CreateListBoxItem(key == "" and "True" or key, string.format("(%s) %.1f%%", FormatBigNumber(encounterData.SkillDamageTypes[skillname][key]), encounterData.SkillDamageTypes[skillname][key]/encounterData.SkillDamage[skillname]*100))
        end
    end

    self:Reformat()
end

function PopupWindow:SetupWindow(encounterData)
    if not(encounterData) then
        self:SetVisible(false)
        return 
    end
    self.itemListBox:ClearItems()

    self:CreateListBoxItem("Highest hit", FormatBigNumber(encounterData.MaxDamage))
    self:CreateListBoxItem("Total hits", encounterData.TotalHits)
    local missedItems = encounterData.TotalHits - (encounterData.HitTypeData[""] + encounterData.HitTypeData["critical "] + encounterData.HitTypeData["devastating "] + encounterData.HitTypeData["partially evaded "] + encounterData.HitTypeData["partially parried "] + encounterData.HitTypeData["partially blocked "])
    self:CreateListBoxItem("Missed", string.format("(%d) %.1f%%", missedItems, missedItems/encounterData.TotalHits*100))

    self:CreateListBoxItem("Critical hits", string.format("(%d) %.1f%%", encounterData.HitTypeData["critical "], encounterData.HitTypeData["critical "]/encounterData.TotalHits*100))
    self:CreateListBoxItem("Devastating hits", string.format("(%d) %.1f%%", encounterData.HitTypeData["devastating "], encounterData.HitTypeData["devastating "]/encounterData.TotalHits*100))
    self:CreateListBoxItem("Total Damage", FormatBigNumber(encounterData.CurrentTotalDamage))

    for key, value in pairs(encounterData.DamagetTypeData) do
        if value > 0 then
            self:CreateListBoxItem(key == "" and "True" or key, string.format("(%s) %.1f%%", FormatBigNumber(encounterData.DamagetTypeData[key]), encounterData.DamagetTypeData[key]/encounterData.CurrentTotalDamage*100))
        end
    end

    self:Reformat()
end


function PopupWindow:SetupTargetSkillWindow(encounterData, name)
    if not(encounterData) then
        self:SetVisible(false)
        return 
    end
    self.itemListBox:ClearItems()

    --self:CreateListBoxItem("Total hits", encounterData.TotalHits)
    --self:CreateListBoxItem("Highest hit", FormatBigNumber(encounterData.MaxDamage))
    local missedItems = encounterData.TotalHits - (encounterData.TargetHits[name][""] + encounterData.TargetHits[name]["critical "] + encounterData.TargetHits[name]["devastating "] + encounterData.TargetHits[name]["partially evaded "] + encounterData.TargetHits[name]["partially parried "] + encounterData.TargetHits[name]["partially blocked "])
    self:CreateListBoxItem("Missed", string.format("(%d) %.1f%%", missedItems, missedItems/encounterData.TotalHits*100))

    self:CreateListBoxItem("Critical hits", string.format("(%d) %.1f%%", encounterData.TargetHits[name]["critical "], encounterData.TargetHits[name]["critical "]/encounterData.TotalHits*100))
    self:CreateListBoxItem("Devastating hits", string.format("(%d) %.1f%%", encounterData.TargetHits[name]["devastating "], encounterData.TargetHits[name]["devastating "]/encounterData.TotalHits*100))
    self:CreateListBoxItem("Total Damage", FormatBigNumber(encounterData.CurrentTotalDamage))

    for key, value in pairs(encounterData.TargetDamageTypes[name]) do
        if value > 0 then
            self:CreateListBoxItem(key == "" and "True" or key, string.format("(%s) %.1f%%", FormatBigNumber(encounterData.TargetDamageTypes[name][key]), encounterData.DamagetTypeData[key]/encounterData.TargetDamage[name]*100))
        end
    end

    self:Reformat()
end

function PopupWindow:SetupCorruptionWindow(encounterData, name)
    self.itemListBox:ClearItems()
    for key, value in spairs(encounterData.TargetCorruptionBreakdown[name], function (t, a, b) return a > b end) do
        if value > 0 then
            self:CreateListBoxItem(key == "" and "True" or key, string.format("(%s) %.1f%%", encounterData.TargetCorruptionBreakdown[name][key], encounterData.TargetCorruptionBreakdown[name][key]/encounterData.CorruptionsTargetTotal[name]*100))
        end
    end
    self:Reformat()
end

function PopupWindow:SetupTotalCorruptionWindow(encounterData)
    self.itemListBox:ClearItems()
    for key, value in spairs(encounterData.CorruptionBreakdown, function (t, a, b) return a > b end) do
        if value > 0 then
            self:CreateListBoxItem(key == "" and "True" or key, string.format("(%s) %.1f%%", value, value/encounterData.CorruptionTotal*100))
        end
    end
    self:Reformat()
end

function PopupWindow:SetupKillingBlowsWindow(encounterData, name)
    self.itemListBox:ClearItems()
    for key, value in spairs(encounterData.KillingBlows[name], function (t, a, b) return a > b end) do
        if value > 0 then
            self:CreateListBoxItem(key == "" and "True" or key, string.format("(%s) %.1f%%", value, value/encounterData.TotalKillingBlows[name]*100))
        end
    end
    self:Reformat()
end

function PopupWindow:SetupKillsWindow(encounterData, name)
    self.itemListBox:ClearItems()
    for charName, kbData in spairs(encounterData.KillingBlows, function (t, a, b) return a > b end) do
        for kbName, kbValue in pairs(kbData) do
            if kbName == name then
                self:CreateListBoxItem(charName == "" and "True" or charName, string.format("(%s) %.1f%%", kbValue, kbValue/encounterData.Kills[kbName]*100))
            end
        end
    end
    self:Reformat()
end

function PopupWindow:SetupSkillTakenWindow(encounterData, skillname)
    if not(encounterData) or not(encounterData.DamageTakenSkillTotalHits[skillname]) then
        self:SetVisible(false)
        return 
    end
    self.itemListBox:ClearItems()

    self:CreateListBoxItem(skillname, "")
    self:CreateListBoxItem("Highest hit", FormatBigNumber(encounterData.DamageTakenSkillMaxDamage[skillname]))
    self:CreateListBoxItem("Total hits", encounterData.DamageTakenSkillTotalHits[skillname])
    local missedItems = encounterData.DamageTakenSkillTotalHits[skillname] - (encounterData.DamageTakenSkillsHitType[skillname][""] + encounterData.DamageTakenSkillsHitType[skillname]["critical "] + encounterData.DamageTakenSkillsHitType[skillname]["devastating "] + encounterData.DamageTakenSkillsHitType[skillname]["partially evaded "] + encounterData.DamageTakenSkillsHitType[skillname]["partially parried "] + encounterData.DamageTakenSkillsHitType[skillname]["partially blocked "])
    self:CreateListBoxItem("Missed", string.format("(%d) %.1f%%",missedItems, missedItems/encounterData.DamageTakenSkillTotalHits[skillname]*100))

    for key, value in spairs(self.hitTypeTable, function(t, a, b) return self.hitTypeTableOrder[a] < self.hitTypeTableOrder[b] end) do
        self:CreateListBoxItem(self.hitTypeTable[key], string.format("(%s) %.1f%%", FormatBigNumber(encounterData.DamageTakenSkillsHitType[skillname][key]), encounterData.DamageTakenSkillsHitType[skillname][key]/encounterData.DamageTakenSkillTotalHits[skillname]*100))
    end
    self:CreateListBoxItem("Total Damage", FormatBigNumber(encounterData.DamageTakenSkills[skillname]))

    for key, value in pairs(encounterData.DamageTakenSkillsType[skillname]) do
        if value > 0 then
            self:CreateListBoxItem(key == "" and "True" or key, string.format("(%s) %.1f%%", FormatBigNumber(encounterData.DamageTakenSkillsType[skillname][key]), encounterData.DamageTakenSkillsType[skillname][key]/encounterData.DamageTakenSkills[skillname]*100))
        end
    end

    self:Reformat()
end

function PopupWindow:SetupTargetTakenWindow(encounterData, target)
    if not(encounterData) or not(encounterData.DamageTakenTargetsTotalHits[target]) then
        self:SetVisible(false)
        return 
    end
    self.itemListBox:ClearItems()

    self:CreateListBoxItem(target, "")
    self:CreateListBoxItem("Highest hit", FormatBigNumber(encounterData.DamageTakenTargetsMaxDamage[target]))
    self:CreateListBoxItem("Total hits", encounterData.DamageTakenTargetsTotalHits[target])
    local missedItems = encounterData.DamageTakenTargetsTotalHits[target] - (encounterData.DamageTakenTargetHitType[target][""] + encounterData.DamageTakenTargetHitType[target]["critical "] + encounterData.DamageTakenTargetHitType[target]["devastating "] + encounterData.DamageTakenTargetHitType[target]["partially evaded "] + encounterData.DamageTakenTargetHitType[target]["partially parried "] + encounterData.DamageTakenTargetHitType[target]["partially blocked "])
    self:CreateListBoxItem("Missed", string.format("(%d) %.1f%%",missedItems, missedItems/encounterData.DamageTakenTargetsTotalHits[target]*100))

    for key, value in spairs(self.hitTypeTable, function(t, a, b) return self.hitTypeTableOrder[a] < self.hitTypeTableOrder[b] end) do
        self:CreateListBoxItem(self.hitTypeTable[key], string.format("(%s) %.1f%%", FormatBigNumber(encounterData.DamageTakenTargetHitType[target][key]), encounterData.DamageTakenTargetHitType[target][key]/encounterData.DamageTakenTargetsTotalHits[target]*100))
    end
    self:CreateListBoxItem("Total Damage", FormatBigNumber(encounterData.DamageTakenTargets[target]))

    for key, value in pairs(encounterData.DamageTakenTargetType[target]) do
        if value > 0 then
            self:CreateListBoxItem(key == "" and "True" or key, string.format("(%s) %.1f%%", FormatBigNumber(encounterData.DamageTakenTargetType[target][key]), encounterData.DamageTakenTargetType[target][key]/encounterData.DamageTakenTargets[target]*100))
        end
    end

    self:Reformat()
end

function PopupWindow:SetupTotalTakenWindow(encounterData, skillname)
    if not(encounterData) or not(encounterData.TotalTakenHits) then
        self:SetVisible(false)
        return 
    end
    self.itemListBox:ClearItems()

    self:CreateListBoxItem("Highest hit", FormatBigNumber(encounterData.MaxDamageTaken))
    self:CreateListBoxItem("Total hits", encounterData.TotalTakenHits)
    
    for key, value in spairs(self.hitTypeTable, function(t, a, b) return self.hitTypeTableOrder[a] < self.hitTypeTableOrder[b] end) do
        self:CreateListBoxItem(self.hitTypeTable[key], string.format("(%s) %.1f%%", FormatBigNumber(encounterData.DamageTakenHitType[key]), encounterData.DamageTakenHitType[key]/encounterData.TotalTakenHits*100))
    end
    self:CreateListBoxItem("Total Damage", FormatBigNumber(encounterData.DamageTaken))

    for key, value in pairs(encounterData.DamageTakenType) do
        if value > 0 then
            self:CreateListBoxItem(key == "" and "True" or key, string.format("(%s) %.1f%%", FormatBigNumber(encounterData.DamageTakenType[key]), encounterData.DamageTakenType[key]/encounterData.DamageTaken*100))
        end
    end

    self:Reformat()
end

function PopupWindow:SetBuffsReceivedWindow(encounterData, skillName)
    self.itemListBox:ClearItems()
    if not(encounterData) or not(encounterData.Buffs[skillName]) then
        self:SetVisible(false)
        return 
    end
    for key, value in pairs(encounterData.BuffsSource[skillName]) do
        if value > 0 then
            self:CreateListBoxItem(key == "" and "True" or key, string.format("(%s) %.1f%%", encounterData.BuffsSource[skillName][key], encounterData.BuffsSource[skillName][key]/encounterData.Buffs[skillName]*100))
        end
    end
    self:Reformat()
end

function PopupWindow:SetBuffsUptimeWindow(encounterData, skillName, combinedName)
    self.itemListBox:ClearItems()

    if not(encounterData) or not(encounterData.EffectData[combinedName]) then
        self:SetVisible(false)
        return 
    end

    self:CreateListBoxItem(skillName, string.format("%s", FormatTimeString(encounterData.EffectData[combinedName])))
    self:Reformat()
end

function PopupWindow:SetDebuffsUptimeWindow(encounterData, skillName, combinedName)
    self.itemListBox:ClearItems()

    if not(encounterData) or not(encounterData.DebuffEffectData[combinedName]) then
        self:SetVisible(false)
        return 
    end

    self:CreateListBoxItem(skillName, string.format("%s", FormatTimeString(encounterData.DebuffEffectData[combinedName])))
    self:Reformat()
end

function PopupWindow:SetBuffsAppliedWindow(encounterData, skillName)
    self.itemListBox:ClearItems()
    if not(encounterData) or not(encounterData.BuffsApplied[skillName]) then
        self:SetVisible(false)
        return 
    end
    for key, value in pairs(encounterData.BuffsTarget[skillName]) do
        if value > 0 then
            self:CreateListBoxItem(key == "" and "True" or key, string.format("(%s) %.1f%%", encounterData.BuffsTarget[skillName][key], encounterData.BuffsTarget[skillName][key]/encounterData.BuffsApplied[skillName]*100))
        end
    end
    self:Reformat()
end

function PopupWindow:TotalHealingReceivedWindow(encounterData)
    --HealingReceivedSkillHitType
    self.itemListBox:ClearItems()
    for key, value in spairs(self.hitTypeTable, function(t, a, b) return self.hitTypeTableOrder[a] < self.hitTypeTableOrder[b] end) do
        if encounterData.HealingReceivedHitType[key] then
            self:CreateListBoxItem(self.hitTypeTable[key] == "Hit" and "Normal" or self.hitTypeTable[key], string.format("(%s) %.1f%%", FormatBigNumber(encounterData.HealingReceivedHitType[key]), encounterData.HealingReceivedHitType[key]/encounterData.HealingHits*100))
        end
    end
    self:Reformat()
end

function PopupWindow:HealingReceivedWindow(encounterData, skillName)
    self.itemListBox:ClearItems()
    for key, value in spairs(self.hitTypeTable, function(t, a, b) return self.hitTypeTableOrder[a] < self.hitTypeTableOrder[b] end) do
        if encounterData.HealingReceivedSkillHitType[skillName][key] then
            self:CreateListBoxItem(self.hitTypeTable[key] == "Hit" and "Normal" or self.hitTypeTable[key], string.format("(%s) %.1f%%", FormatBigNumber(encounterData.HealingReceivedSkillHitType[skillName][key]), encounterData.HealingReceivedSkillHitType[skillName][key]/encounterData.HealingRecievedSkillHits[skillName]*100))
        end
    end
    self:Reformat()
end

function PopupWindow:HealingReceivedTargetWindow(encounterData, targetName)
    self.itemListBox:ClearItems()
    for key, value in spairs(self.hitTypeTable, function(t, a, b) return self.hitTypeTableOrder[a] < self.hitTypeTableOrder[b] end) do
        if encounterData.HealingReceivedHitTypeTarget[targetName][key] then
            self:CreateListBoxItem(self.hitTypeTable[key] == "Hit" and "Normal" or self.hitTypeTable[key], string.format("(%s) %.1f%%", FormatBigNumber(encounterData.HealingReceivedHitTypeTarget[targetName][key]), encounterData.HealingReceivedHitTypeTarget[targetName][key]/encounterData.HealingRecievedTargetHits[targetName]*100))
        end
    end

    for skill, value in spairs(encounterData.HealingReceivedSkillsTarget[targetName], function(t, a, b) return t[a] > t[b] end) do
        self:CreateListBoxItem(skill, string.format("(%s) %.1f%%", FormatBigNumber(value), value/encounterData.HealingReceivedTotalTarget[targetName]*100))
    end
    self:Reformat()
end

function PopupWindow:TotalHealingAppliedWindow(encounterData)
    --HealingReceivedSkillHitType
    self.itemListBox:ClearItems()
    for key, value in spairs(self.hitTypeTable, function(t, a, b) return self.hitTypeTableOrder[a] < self.hitTypeTableOrder[b] end) do
        if encounterData.HealingReceivedHitTypeApplied[key] then
            self:CreateListBoxItem(self.hitTypeTable[key] == "Hit" and "Normal" or self.hitTypeTable[key], string.format("(%s) %.1f%%", FormatBigNumber(encounterData.HealingReceivedHitTypeApplied[key]), encounterData.HealingReceivedHitTypeApplied[key]/encounterData.HealingHitsApplied*100))
        end
    end
    self:Reformat()
end

function PopupWindow:HealingAppliedWindow(encounterData, skillName)
    self.itemListBox:ClearItems()
    for key, value in spairs(self.hitTypeTable, function(t, a, b) return self.hitTypeTableOrder[a] < self.hitTypeTableOrder[b] end) do
        if encounterData.HealingReceivedSkillHitTypeApplied[skillName][key] then
            self:CreateListBoxItem(self.hitTypeTable[key] == "Hit" and "Normal" or self.hitTypeTable[key], string.format("(%s) %.1f%%", FormatBigNumber(encounterData.HealingReceivedSkillHitTypeApplied[skillName][key]), encounterData.HealingReceivedSkillHitTypeApplied[skillName][key]/encounterData.HealingRecievedSkillHitsApplied[skillName]*100))
        end
    end
    self:Reformat()
end

function PopupWindow:HealingAppliedTargetWindow(encounterData, targetName)
    self.itemListBox:ClearItems()
    for key, value in spairs(self.hitTypeTable, function(t, a, b) return self.hitTypeTableOrder[a] < self.hitTypeTableOrder[b] end) do
        if encounterData.HealingReceivedHitTypeTargetApplied[targetName][key] then
            self:CreateListBoxItem(self.hitTypeTable[key] == "Hit" and "Normal" or self.hitTypeTable[key], string.format("(%s) %.1f%%", FormatBigNumber(encounterData.HealingReceivedHitTypeTargetApplied[targetName][key]), encounterData.HealingReceivedHitTypeTargetApplied[targetName][key]/encounterData.HealingRecievedTargetHitsApplied[targetName]*100))
        end
    end

    for skill, value in spairs(encounterData.HealingReceivedSkillsTargetApplied[targetName], function(t, a, b) return t[a] > t[b] end) do
        self:CreateListBoxItem(skill, string.format("(%s) %.1f%%", FormatBigNumber(value), value/encounterData.HealingReceivedTotalTargetApplied[targetName]*100))
    end
    self:Reformat()
end

function PopupWindow:PowerTotalHealingReceivedWindow(encounterData)
    --HealingReceivedSkillHitType
    self.itemListBox:ClearItems()
    for key, value in spairs(self.hitTypeTable, function(t, a, b) return self.hitTypeTableOrder[a] < self.hitTypeTableOrder[b] end) do
        if encounterData.PowerHealingReceivedHitType[key] then
            self:CreateListBoxItem(self.hitTypeTable[key] == "Hit" and "Normal" or self.hitTypeTable[key], string.format("(%s) %.1f%%", FormatBigNumber(encounterData.PowerHealingReceivedHitType[key]), encounterData.PowerHealingReceivedHitType[key]/encounterData.PowerHealingHits*100))
        end
    end
    self:Reformat()
end

function PopupWindow:PowerHealingReceivedWindow(encounterData, skillName)
    self.itemListBox:ClearItems()
    for key, value in spairs(self.hitTypeTable, function(t, a, b) return self.hitTypeTableOrder[a] < self.hitTypeTableOrder[b] end) do
        if encounterData.PowerHealingReceivedSkillHitType[skillName][key] then
            self:CreateListBoxItem(self.hitTypeTable[key] == "Hit" and "Normal" or self.hitTypeTable[key], string.format("(%s) %.1f%%", FormatBigNumber(encounterData.PowerHealingReceivedSkillHitType[skillName][key]), encounterData.PowerHealingReceivedSkillHitType[skillName][key]/encounterData.PowerHealingRecievedSkillHits[skillName]*100))
        end
    end
    self:Reformat()
end

function PopupWindow:TemporaryMoraleWindow(encounterData, skillName)
    self.itemListBox:ClearItems()
    self:Reformat()
end

function PopupWindow:UptimeWindow(encounterData, skillName)
    self.itemListBox:ClearItems()
    --self:CreateListBoxItem(, FormatTimeStringNoMilli()))
    self:Reformat()
end

function PopupWindow:Reformat()
    self:SetWidth(self.parent:GetWidth())
    self.itemListBox:SetHeight(self.itemListBox:GetItemCount() * self.parent.itemHeight)
    self:SetHeight(self.itemListBox:GetItemCount() * self.itemHeight + 2 * self.parent.borderSize)
    self:SetPosition(self.parent:GetLeft(), self.parent:GetTop() - (self.itemListBox:GetItemCount() * self.parent.itemHeight + 2 * self.parent.borderSize))
    self:SetZOrder(100)
end

function PopupWindow:Remove()
    self:SetParent(nil)
    local controlList = self:GetControls()
    for i = 1, controlList:GetCount() do
        controlList:RemoveAt(1)
    end
    for key, _ in pairs(self) do
        self[key] = nil
    end
    collectgarbage()
end
