GroupHandler = class(Turbine.UI.Control)
function GroupHandler:Constructor()
    Turbine.UI.Control.Constructor(self)
    self.Player = Turbine.Gameplay.LocalPlayer.GetInstance()

    -- Information tables
    self.Group = {}
    self.GroupNameLUT = {}
    self.GroupRats = {}

    self.GroupSize = 0

    self.callbacks = {}
    self.addedCallbacks = {}

    self.EffectUpdate = function(sender, event, args)
        if type(event)=="function" then
            event(args)
        else
            if type(event)=="table" then
                for i=1,#event do
                    if type(event[i])=="function" then
                        event[i](args)
                    end
                end
            end
        end
    end

    -- Set Group Rat Infor
    self.SetGroupRats = function()
        self.GroupRats = {}
        if Party and Party:GetAssistTargetCount() > 0 then
            for i = 1, Party:GetAssistTargetCount(), 1 do
                self.GroupRats[Party:GetAssistTarget(i):GetName()] = true
            end
        end
    end

    -- Set Group Info
    self.SetGroupInformation = function()
        self.Group = {}
        self.GroupNameLUT = {}
        self.GroupRats = nil
        if Party then
            self.GroupSize = Party:GetMemberCount()
            for i = 1, GroupSize, 1 do
                self.Group[i] = {}
                self.Group[i].player = Party:GetMember(i)
                self.Group[i].effects = self.Group[i].player:GetEffects()
                self.Group[i].currentEffects = {}
                self.GroupNameLUT[Party:GetMember(i):GetName()] = true
            end
        else
            self.Group[1] = {}
            self.Group[1].player = Player
            self.Group[1].effects = self.Group[1].player:GetEffects()
            self.Group[1].currentEffects = {}
        end

        self.SetGroupRats()
    end

    -- Set Group Triggers
    self.SetGroupReloads = function()
        self.Player.PartyChanged = function()
            ReloadPlugin()
        end
        self.Player.RaidChanged = function()
            ReloadPlugin()
        end
        
        if Party ~= nil then
            Party.MemberAdded = function()
                ReloadPlugin()
            end
            Party.MemberRemoved = function()
                ReloadPlugin()
            end
            Party.LeaderChanged = function()
                ReloadPlugin()
            end

            for i = 1, Party:GetMemberCount(), 1 do
                Party:GetMember(i).IsLinkDeadChanged = function()
                    ReloadPlugin()
                end
            end
        end
    end

    self.SetGroupInformation()
    self.SetGroupReloads()

    -- Go through all players
    for i = 1, Party:GetMemberCount(), 1 do

        -- Return if localplayer -> handled differently
        if self.Group[i].player:GetName() ~= self.Player:GetName() then
            self.callbacks[i] = {}
            self.callbacks[i]["EffectAdded"] = AddCallback(self.Group[i].effects, "EffectAdded", function(sender, args)
                local effect = self.Group[i].effects:Get(args.Index)
                if not(self.Group[i].currentEffects[effect:GetID()]) then
                    self.Group[i].currentEffects[effect:GetID()] = true
                    self.EffectUpdate(self, self.EffectAdded, {["effect"] = effect, ["groupID"] = i})
                end
            end)
        end
    end
end

function GroupHandler:RemoveCallbacks()
    if Party then
        for i = 1, self.GroupSize, 1 do
            if self.Group[i].player:GetName() ~= self.Player:GetName() then 
                for key, value in pairs(self.callbacks[i]) do
                    RemoveCallback(self.Group[i].effects, key, value)
                end
            end
        end
    end
end

function GroupHandler:CheckGroupNameLUT(name)
    return(self.GroupNameLUT[name])
end

function GroupHandler:CheckGroupRat(name)
    return(self.GroupRats[name])
end

function GroupHandler:GetGroup()
    return self.Group
end