AlertWindow = class(Turbine.UI.Window)
function AlertWindow:Constructor(width, height, x, y, alertHeight, allAlerts)
    self.width = width
    self.height = height
    local x = x
    local y = y
    self.Alerts = {}
    self.TieringAlerts = {}
    self.resort = false
    self.AlertHeight = alertHeight

    local MaxItems = math.floor(self.height / self.AlertHeight)

    Turbine.UI.Window.Constructor(self)
    self:SetSize(self.width,self.height)
    self:SetPosition(x,y)
    self:SetZOrder(1)

    self:SetWantsKeyEvents(false)
    self:SetMouseVisible(false)
    self:SetVisible(true)
    --self:SetWantsUpdates(true)


    self.Clear = function()
        for key, value in pairs(self.Alerts) do 
            value[1]:SetVisible(nil)
            value[1]:Destroy()
            self.Alerts[key] = nil
        end
        
        self.TieringAlerts = {}
        if Tablelength(self.Alerts) > 0 then
            self.Clear()
        end
        self.Alerts = nil
        collectgarbage()
        self.Alerts = {}
    end

    self.ReUpdateAlertPositions = function()

        local counter = 1
        for _,v in spairs(self.Alerts, function(t,a,b) return t[b][1]:GetDuration() > t[a][1]:GetDuration() end) do
            v[1]:SetPosition(0, MaxItems * (self.AlertHeight) - (counter * self.AlertHeight)) 
            counter = counter + 1
        end

        if Tablelength(self.Alerts) == 0 then 
            RemoveCallback(Updater, "ExecuteUpdate", self.callback)
            self.callback = nil
        end

        self.resort = false
    end

    local moveBox = MoveBox(self, self.width, self.height, "Alert Window")
    
    self.ToggleMove = function()
        moveBox:ToggleMove()
    end

    self.ExecuteUpdate = function(delta)
        for key, value in pairs(self.Alerts) do
            if value ~= nil then
                value[1]:UpdateText(delta)
                if value[1]:GetDuration() <= 0 then
                    value[1]:Destroy()
                    if value[2] and value[3] then
                        self.TieringAlerts[string.gsub(value[2]:GetName(), "%d", "") .. value[3]] = nil
                    end
                    self.Alerts[key] = nil
                    self.resort = true
                end
            end
        end
        if self.resort then
            self.ReUpdateAlertPositions()
        end
    end
end

function AlertWindow:Message(duration, message, icon, cdVisible, tieringEffect, effect, playerName)
    if tieringEffect and effect then
        if self.TieringAlerts[string.gsub(effect:GetName(), "%d", "") .. playerName] then
            local tieringID = self.TieringAlerts[string.gsub(effect:GetName(), "%d", "") .. playerName]
            self.Alerts[tieringID][1]:ChangeDuration(duration)
            self.Alerts[tieringID][1]:ChangeName(message)
            return
        end
    end

    local ID = #self.Alerts + 1
    self.Alerts[ID] = {Alert(self, duration, message, icon, cdVisible), effect, playerName}
    self.resort = true

    if tieringEffect and effect then
        if not(self.TieringAlerts[string.gsub(effect:GetName(), "%d", "") .. playerName]) then
            self.TieringAlerts[string.gsub(effect:GetName(), "%d", "") .. playerName] = ID
        end
    end

    if not(self.callback) then self.callback = AddCallback(Updater, "ExecuteUpdate", self.ExecuteUpdate) end
    ChatOutput(message)
end

function AlertWindow:Unload()
    RemoveCallback(Updater, "ExecuteUpdate", self.callback)
    --if self.chatCallback then RemoveCallback(ChatEventHandler, "StandardChat", self.chatCallback) end
    self:Clear()
    self:SetParent(nil)
end

Alert = class(Turbine.UI.Window)
function Alert:Constructor(parent, duration, message, icon, cdVisible)
    --ChatOutput("Printing:")
    --ChatOutput("Message: " .. message)
    --ChatOutput("Duration: " .. duration)
    --ChatOutput("Icon: " .. icon)
    --ChatOutput("CdVisible: " .. cdVisible)

    Turbine.UI.Window.Constructor(self)
    self.timer = duration
    if not(self.timer) then self.timer = 0 end
    self.showTimers = cdVisible
    self.oldtimer = duration
    self.colourEvent = 2
    self.showLeft = Settings.AL
    self.showRight = Settings.AR
    self.colourChange = true
    self:SetParent(parent)
    self:SetSize(Settings.AWidth, Settings.ABaseHeight)
    self:SetPosition(0, 0)
    self:SetMouseVisible(false)
    self:SetVisible(true)

    self.messageBox = Turbine.UI.Label()
    self.messageBox:SetParent(self)
    self.messageBox:SetSize(600, Settings.ABaseHeight)
    self.messageBox:SetPosition(Settings.ABaseHeight + 5,0)
    self.messageBox:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft)
    self.messageBox:SetFontStyle(Turbine.UI.FontStyle.Outline)
    self.messageBox:SetMouseVisible(false)
    self.messageBox:SetVisible(true)
    self.messageBox:SetMarkupEnabled(true)
    self.messageBox:SetOutlineColor(Turbine.UI.Color(0, 0, 0))
    self.messageBox:SetFont(Settings.AFont)
    self.messageBox:SetText(message)
    self.messageBox:SetMultiline(false)
    --self.messageBox:SetBackColor(Turbine.UI.Color(0,0,0))

    self.imageBox1 = Turbine.UI.Control()
    self.imageBox1:SetParent(self)
    self.imageBox1:SetSize(Settings.ABaseHeight, Settings.ABaseHeight)
    self.imageBox1:SetPosition(0, 0)
    self.imageBox1:SetMouseVisible(false)

    if icon ~= nil then
        self.imageBox1:SetBackground(BackgroundHandler:GetBackground(icon))
        self.imageBox1:SetStretchMode(2)
        self.imageBox1:SetStretchMode(1)
        self.imageBox1:SetSize(Settings.ABaseHeight, Settings.ABaseHeight)
        self.imageBox1:SetVisible(true)
    else
        self.imageBox1:SetBackground("")
        self.imageBox1:SetVisible(true)
    end

    self.durationLabel = Turbine.UI.Label()
    self.durationLabel:SetParent(self)
    self.durationLabel:SetVisible(true)
    self.durationLabel:SetSize(Settings.ABaseHeight, Settings.ABaseHeight)
    self.durationLabel:SetPosition(0, 0)
    self.durationLabel:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
    self.durationLabel:SetFont(Settings.AFont)
    self.durationLabel:SetFontStyle(Turbine.UI.FontStyle.Outline)
    self.durationLabel:SetOutlineColor(Turbine.UI.Color(0, 0, 0))
    self.durationLabel:SetForeColor(Turbine.UI.Color(1, 1, 1))
    self.durationLabel:SetMouseVisible(false)
    self.durationLabel:SetStretchMode(1)
    
    if cdVisible then
        self.durationLabel:SetText(duration)
    else 
        self.durationLabel:SetText("")
    end

    self.Destroy = function()
        self:SetVisible(false)
        self:SetParent(nil)
    end

    self.GetDuration = function()
        return(self.timer)
    end
end

function Alert:UpdateText(delta)
    self.timer = self.timer - delta

    if self.timer < 1 then
        self:SetOpacity(self.timer + 0.3)
    end
    
    if self.oldtimer - self.timer > 0.05 then
        if self.showTimers then
            if self.timer <= self.colourEvent then
                self.durationLabel:SetStretchMode(nil)
                self.durationLabel:SetStretchMode(1)
                self.durationLabel:SetText(string.format("%.1f", self.timer))
                if self.colourChange then
                    self.durationLabel:SetForeColor(Turbine.UI.Color(1, 0.75, 0))
                    --self.imageBox1:SetOutlineColor(Turbine.UI.Color(1, 0, 0))
                    self.colourChange = false
                end
                --self.colourEvent = -1
            else
                self.durationLabel:SetStretchMode(nil)
                self.durationLabel:SetStretchMode(1)
                self.durationLabel:SetText(math.floor(self.timer))
            end
        end
        self.oldtimer = self.timer
    end
end

function Alert:ChangeDuration(newDuration)
    self.timer = newDuration
    self.oldtimer = self.timer
end

function Alert:ChangeName(newName)
    self.messageBox:SetText(newName)
end