EventWindowsHandler = class(Turbine.UI.Control)
function EventWindowsHandler:Constructor()
    Turbine.UI.Control.Constructor(self)

    -- Initialize all data
    self:LoadChatData()
    self:LoadSelfEffectData()
    self:LoadGroupEffectData()
    self:LoadChatPlayerCombatData()
    self:LoadChatEnemyCombatData()

    self.playerName = Turbine.Gameplay.LocalPlayer.GetInstance():GetName()

    self.Role = 0

    self.AlertWindows = {}
    self.EffectWindows = {}
    self.TimerWindows = {}
    self.addedCallbacks = {}

    self.chatCallback = {}

    self.Group = GroupEffectsHandler:GetGroup()

    self.eventToken = 2

    if not(AccountData["Alert Token"]) or AccountData["Alert Token"] < self.eventToken then
        ChatOutput("Automatically updating alerts due to new alerts detected")
        self:UpdateBase()
        ChatOutput("Succesfully updated alerts")
        AccountData["Alert Token"] = self.eventToken
    end

    -- Intialize base windows
    if Settings.AEnabled then 
        self.messageWindow = AlertWindow(Settings.AWidth, Settings.AHeight, Settings.AX, Settings.AY, Settings.ABaseHeight)
        self.AlertWindows[1] = self.messageWindow
        --Windows["Alert Window"] = messageWindow

        self.alertsEnabled = true
    end 

    if Settings.EEnabled then 
        self.effectsWindow = EffectsWindow() 
        self.EffectWindows[1] = self.effectsWindow
        --Windows["Effect Window"] = effectsWindow

        self.bigAlertsEnabled = true
    end 

    if Settings.SEnabled then 
        self.skillCounters = Timers.SkillCounters()
        self.TimerWindows[1] = self.skillCounters
        --Windows["Boss Timers"] = skillCounters

        self.bossCooldownsEnbabled = true
    end 

    self.nameFunction = function(text, name)
        return string.gsub(text, "<name>", NameHandlerObject:GetName(name, true, "EventWindowsHandler", ""))
    end

    -- Initialize additional windows

    -- INitialize chat callbacks
    self.ChatAlert = function(message)
        local name, cleanedLine = self:ParseTextLine(message)    

        -- Chat alerts {1 duration, 2 message, 3 icon, 4 countdown, 5 soundeffect, 6 instanceID, 7 role, 8 timerduration, 9 timer text, 10 timer icon, 11 unique, 12 loop, 13 timerWindowID}
        local phraseTable = self.AllChatData[name]
        if phraseTable then 
            for phrase, alertTable in pairs(phraseTable) do
                if string.find(cleanedLine, phrase) then
                    local restrict = alertTable["Restrictions"]
                    -- Double check if we want the alert
                    if restrict and (not(restrict[1]) or restrict[1] == Settings.InstanceVersion) and (not(restrict[2]) or restrict[2] == self.Role) then
                        local alert = alertTable["Alerts"][1]
                        local timer = alertTable["Timers"][1]
                        local timerID = timer[6]
                        if not(timer[6]) then timerID = 1 end
                        -- Alert specific
                        if self.alertsEnabled and alert[1] then
                            self.AlertWindows[1]:Message(alert[1], alert[2], alert[3], alert[4])
                        end

                        -- Sound handling
                        if alert[5] then Sounds:PlaySoundEffect(alert[5]) end

                        -- Timer Handling
                        if self.bossCooldownsEnbabled and timer[1] then
                            self.TimerWindows[timerID]:NewTimer(timer[1], timer[2], alert[4], nil, timer[5], timer[3])
                        end
                    end
                end
            end
        end
    end

    self.ChatPlayerCombatAlert = function(args)
        -- Chat alerts {1 duration, 2 message, 3 icon, 4 countdown, 5 soundeffect, 6 instanceID, 7 role, 8 timerduration, 9 timer text, 10 timer icon, 11 unique, 12 loop, 13 timerWindowID}
        local alertTable = self.AllChatPlayerCombatData[args.SkillName]
        if alertTable then 
            local restrict = alertTable["Restrictions"]
            -- Double check if we want the alert
            if restrict and (not(restrict[1]) or restrict[1] == Settings.InstanceVersion) and (not(restrict[2]) or restrict[2] == self.Role) then
                local alert = alertTable["Alerts"][1]
                local timer = alertTable["Timers"][1]
                local timerID = timer[6]
                if not(timer[6]) then timerID = 1 end
                -- Alert specific
                if self.alertsEnabled and alert[1] then
                    self.AlertWindows[1]:Message(alert[1], alert[2], alert[3], alert[4])
                end

                -- Sound handling
                if alert[5] then Sounds:PlaySoundEffect(alert[5]) end

                -- Timer Handling
                if self.bossCooldownsEnbabled and timer[1] then
                    self.TimerWindows[timerID]:NewTimer(timer[1], timer[2], timer[4], nil, timer[5], timer[3])
                end
            end
        end
    end

    self.ChatEnemyCombatAlert = function(args)
        -- Chat alerts {1 duration, 2 message, 3 icon, 4 countdown, 5 soundeffect, 6 instanceID, 7 role, 8 timerduration, 9 timer text, 10 timer icon, 11 unique, 12 loop, 13 timerWindowID}
        local alertTable = self.AllChatEnemyCombatData[args.SkillName]
        if alertTable then 
            local restrict = alertTable["Restrictions"]
            -- Double check if we want the alert
            if restrict and (not(restrict[1]) or restrict[1] == Settings.InstanceVersion) and (not(restrict[2]) or restrict[2] == self.Role) then
                local alert = alertTable["Alerts"][1]
                local timer = alertTable["Timers"][1]
                local timerID = timer[6]
                if not(timer[6]) then timerID = 1 end
                -- Alert specific
                if self.alertsEnabled and alert[1] then
                    self.AlertWindows[1]:Message(alert[1], alert[2], alert[3], alert[4])
                end

                -- Sound handling
                if alert[5] then Sounds:PlaySoundEffect(alert[5]) end

                -- Timer Handling
                if self.bossCooldownsEnbabled and timer[1] then
                    self.TimerWindows[timerID]:NewTimer(timer[1], timer[2], timer[4], nil, timer[5], timer[3])
                end
            end
        end
    end

    self.chatCallback["Say"] = AddCallback(ChatEventHandler, "Say", self.ChatAlert)
    self.chatCallback["EnemyCombat"] = AddCallback(ChatEventHandler, "EnemyCombat", self.ChatEnemyCombatAlert)
    self.chatCallback["PlayerCombat"] = AddCallback(ChatEventHandler, "PlayerCombat", self.ChatPlayerCombatAlert)

     -- All effect added alerts {1 duration, 2 message, 3 icon, 4 countdown, 5 soundeffect, 6 5_countdown, 7 instanceID, 8 role 9 timer duration 10 timer text, 11 timer icon, 12 unique, 13 loop, 14 tiering effect}

    self.PlayerEffectWindowHandling = function(effect)
        local alertTable = nil
        
        if string.match(effect:GetName(), "%d") then
            alertTable = self.AllSelfEffectData[effect:GetName()]
            if not(alertTable) then
                alertTable = self.AllSelfEffectData[string.gsub(effect:GetName(), "%d", "")]
                if alertTable and not(alertTable["Restrictions"][3]) then return end
            end
        else
            alertTable = self.AllSelfEffectData[effect:GetName()]
        end


        if alertTable then
            local restrict = alertTable["Restrictions"]
            if restrict and (not(restrict[1]) or restrict[1] == Settings.InstanceVersion) and (not(restrict[2]) or restrict[2] == self.Role) then

                local alert = alertTable["Alerts"][1]
                local timer = alertTable["Timers"][1]

                if self.bigAlertsEnabled and alert[1] and self.EffectWindows[1] then
                    if alert[5] and Sounds then Sounds:PlaySoundEffect(alert[5]) end
                    self.EffectWindows[1]:Message(alert[1], alert[2], alert[3], alert[4], effect:GetName(), alert[6])
                end

                if self.bossCooldownsEnbabled and timer[1] then

                    local effectInput = nil
                    local icon = timer[3]
                    if timer[3] == true then 
                        if effect then 
                            effectInput = effect 
                            icon = nil
                        else
                            icon = effect:GetIcon()
                        end
                    end

                    self.skillCounters:NewTimer(timer[1], self.nameFunction(timer[2] or "", self.playerName), timer[4], effectInput, timer[5], icon, restrict[3], self.playerName)
                end
            end
        end
    end

    self.PlayerGroupEffectWindowHandling = function(effect)
        local alertTable = nil
        
        if string.match(effect:GetName(), "%d") then
            alertTable = self.AllGroupEffectData[effect:GetName()]
            if not(alertTable) then
                alertTable = self.AllGroupEffectData[string.gsub(effect:GetName(), "%d", "")]
                if alertTable and not(alertTable["Restrictions"][3]) then return end
            end
        else
            alertTable = self.AllGroupEffectData[effect:GetName()]
        end

        if alertTable then
            local restrict = alertTable["Restrictions"]
            if restrict and (not(restrict[1]) or restrict[1] == Settings.InstanceVersion) and (not(restrict[2]) or restrict[2] == self.Role) then

                local alert = alertTable["Alerts"][1]
                local timer = alertTable["Timers"][1]

                local name = NameHandlerObject:GetName(self.playerName, true, "EventWindowsHandler", "")
                
                if self.alertsEnabled and alert[1] then
                    if alert[5] and Sounds then Sounds:PlaySoundEffect(alert[5]) end
                    local icon = alert[3]
                    if alert[3] == true and effect then
                        icon = effect:GetIcon()
                    end
                    local stackName = ""
                    if string.match(effect:GetName(), "%d") then
                        stackName = string.match(effect:GetName(), "%d") .. " - "
                    end
                    self.messageWindow:Message(alert[1], stackName.. alert[2] .. "<rgb=#FF0000>" .. name .. "</rgb>", icon, alert[4], restrict[3], effect, name)
                end

                if self.bossCooldownsEnbabled and timer[1] then

                    local effectInput = nil
                    local icon = timer[3]
                    if timer[3] == true then 
                        if effect then 
                            effectInput = effect 
                            icon = nil
                        else
                            icon = effect:GetIcon()
                        end
                    end

                    self.skillCounters:NewTimer(timer[1], self.nameFunction(timer[2], self.playerName), timer[4], effectInput, timer[5], icon, restrict[3], name)
                end
            end
        end
    end

    self.addedCallbacks["effectsWindow"] = AddCallback(PlayerEffectsHandler, "EffectAdded", self.PlayerEffectWindowHandling)
    self.addedCallbacks["groupWindow"] = AddCallback(PlayerEffectsHandler, "EffectAdded", self.PlayerGroupEffectWindowHandling)


    -----------------------------------
    -- Group
    -----------------------------------
    self.GroupEffectHandling = function(args)
        local effectArgs = args["effect"]
        local alertTable = nil
        
        if string.match(effectArgs.Name, "%d") then
            alertTable = self.AllGroupEffectData[effectArgs.Name]
            if not(alertTable) then
                alertTable = self.AllGroupEffectData[string.gsub(effectArgs.Name, "%d", "")]
                if alertTable and not(alertTable["Restrictions"][3]) then return end
            end
        else
            alertTable = self.AllGroupEffectData[effectArgs.Name]
        end

        if alertTable then
            local restrict = alertTable["Restrictions"]
            if (not(restrict[1]) or restrict[1] == Settings.InstanceVersion) and (not(restrict[2]) or restrict[2] == self.Role) then

                local alert = alertTable["Alerts"][1]
                local timer = alertTable["Timers"][1]

                local name = NameHandlerObject:GetName(self.Group[args["groupID"]].player:GetName(), true, "EventWindowsHandler", "")

                if self.alertsEnabled and alert[1] then
                    if alert[5] and Sounds then Sounds:PlaySoundEffect(alert[5]) end

                    local icon = alert[3]
                    if alert[3] == true then 
                        icon = effectArgs.Icon
                    end

                    local stackName = ""
                    if string.match(effectArgs.Name, "%d") then
                        stackName = string.match(effectArgs.Name, "%d") .. " - "
                    end
                    self.messageWindow:Message(alert[1], stackName  .. alert[2] .. "<rgb=#FF0000>" .. name .. "</rgb>", icon, alert[4], restrict[3], nil, name)
                end

                if self.bossCooldownsEnbabled and timer[1] then

                    local effectInput = nil
                    local icon = timer[3]
                    if timer[3] == true then 
                        if effect then 
                            effectInput = effect 
                            icon = nil
                        else
                            icon = effectArgs.Icon
                        end
                    end

                    self.skillCounters:NewTimer(timer[1], self.nameFunction(timer[2], self.Group[args["groupID"]].player:GetName()), timer[4], nil, timer[5], icon, restrict[3], name)
                end
            end
        end

        -- [1] Duration [2] Message [3] Icon (Special/Game/None) [4] Countdown [5] SoundEffect [6] instanceID [7] role [8] timerduration, [9] timer text, [10] timer icon, [11] unique, [12] loop, [13] tiering effect}
    end
    self.groupCallbacks = {}
    self.groupCallbacks["groupeffectsWindow"] = AddCallback(GroupEffectsHandler, "EffectAdded", self.GroupEffectHandling)
end

function EventWindowsHandler:GetEffectWindow()
    return self.EffectWindows[1]
end

function EventWindowsHandler:GetAlertWindow()
    return self.AlertWindows[1]
end

function EventWindowsHandler:GetTimerWindow()
    return self.TimerWindows[1]
end

function EventWindowsHandler:ParseTextLine(message)
    local name, _, line = string.match(message, ">(.*)<\\Select> (.*), ''(.*)''")
    return name, line
end

function EventWindowsHandler:Unload()
    SaveTable(Turbine.DataScope.Account, "PrimeChatData", self.AllChatData)
    SaveTable(Turbine.DataScope.Account, "PrimeChatPlayerCombatData", self.AllChatPlayerCombatData)
    SaveTable(Turbine.DataScope.Account, "PrimeChatEnemyCombatData", self.AllChatEnemyCombatData)
    SaveTable(Turbine.DataScope.Account, "PrimeSelfEffectData", self.AllSelfEffectData)
    SaveTable(Turbine.DataScope.Account, "PrimeGroupEffectData", self.AllGroupEffectData)
    for key, value in pairs(self.addedCallbacks) do
        RemoveCallback(PlayerEffectsHandler, "EffectAdded", value)
    end
    for key, value in pairs(self.groupCallbacks) do
        RemoveCallback(GroupEffectsHandler, "EffectAdded", value)
    end
    for key, value in pairs(self.chatCallback) do
        RemoveCallback(ChatEventHandler, key, value)
    end
end

function EventWindowsHandler:ResetData()
    ChatOutput("Resetting Alert & Timer Data")
    self.AllChatData = self:GetBaseChatData()
    ChatOutput("Reset Say Chat data")
    self.AllChatPlayerCombatData = self:GetBaseChatPlayerCombatData()
    ChatOutput("Reset Player Combat Chat data")
    self.AllChatEnemyCombatData = self:GetBaseChatEnemyCombatData()
    ChatOutput("Reset Player Enemy Chat data")
    self.AllSelfEffectData = self:GetBaseSelfEffectData()
    ChatOutput("Reset Self Effect data")
    self.AllGroupEffectData = self:GetBaseGroupEffectData()
    ChatOutput("Reset Group Effect data")
    ChatOutput("Succesfully Reset Alert & Timer Data")
    ReloadPlugin()
end

function EventWindowsHandler:UpdateBase()
    ChatOutput("Updating Alert & Timer Data")
    self:UpdateTable(self.AllChatData, self:GetBaseChatData())
    ChatOutput("Updating Say Chat data")
    self:UpdateTable(self.AllChatPlayerCombatData, self:GetBaseChatPlayerCombatData())
    ChatOutput("Updating Player Combat Chat data")
    self:UpdateTable(self.AllChatEnemyCombatData, self:GetBaseChatEnemyCombatData())
    ChatOutput("Updating Player Enemy Chat data")
    self:UpdateTable(self.AllSelfEffectData, self:GetBaseSelfEffectData())
    ChatOutput("Updating Self Effect data")
    self:UpdateTable(self.AllGroupEffectData, self:GetBaseGroupEffectData())
    ChatOutput("Updating Group Effect data")
    ChatOutput("Succesfully Updated Alert & Timer Data")
end

function EventWindowsHandler:UpdateTable(base, updating)
    for key, value in pairs(updating) do
        base[key] = value
    end
end

function EventWindowsHandler:SetRole(role)
    self.Role = role
end

-- Data loading functions
function EventWindowsHandler:LoadChatData()
    self.AllChatData = LoadTable(Turbine.DataScope.Account, "PrimeChatData")

    if not(self.AllChatData) then
        self.AllChatData = self:GetBaseChatData()
    end
end

function EventWindowsHandler:LoadChatPlayerCombatData()
    self.AllChatPlayerCombatData = LoadTable(Turbine.DataScope.Account, "PrimeChatPlayerCombatData")

    if not(self.AllChatPlayerCombatData) then
        self.AllChatPlayerCombatData = self:GetBaseChatPlayerCombatData()
    end
end

function EventWindowsHandler:LoadChatEnemyCombatData()
    self.AllChatEnemyCombatData = LoadTable(Turbine.DataScope.Account, "PrimeChatEnemyCombatData")

    if not(self.AllChatEnemyCombatData) then
        self.AllChatEnemyCombatData = self:GetBaseChatEnemyCombatData()
    end
end

function EventWindowsHandler:LoadSelfEffectData()
    self.AllSelfEffectData = LoadTable(Turbine.DataScope.Account, "PrimeSelfEffectData")
    if not(self.AllSelfEffectData) then
        self.AllSelfEffectData = self:GetBaseSelfEffectData()
    end
end

function EventWindowsHandler:LoadGroupEffectData()
    self.AllGroupEffectData = LoadTable(Turbine.DataScope.Account, "PrimeGroupEffectData")
    if not(self.AllGroupEffectData) then
        self.AllGroupEffectData = self:GetBaseGroupEffectData()
    end
end

function EventWindowsHandler:NoteAlert(message, duration, icon)
    if self.EffectWindows[1] then
        self.EffectWindows[1]:Message(duration, message, icon, false, message)
    end
end

function EventWindowsHandler:NoteSmallAlert(message, duration, icon)
    if self.AlertWindows[1] then
        self.AlertWindows[1]:Message(duration, message, icon, true)
    end
end

function EventWindowsHandler:NoteTimer(message, duration, icon, playerName)
    if self.TimerWindows[1] then
        self.TimerWindows[1]:NewTimer(duration, message, nil, nil, nil, icon, nil, playerName)
    end
end

--------------------------------------------------------------------------------------------------------------
--------------------------------------------------------------------------------------------------------------
--------------------------------------------------------------------------------------------------------------
--------------------------------------------------------------------------------------------------------------

function EventWindowsHandler:AddChatAlert(source, trigger, alert)
    if not(self.AllChatData[source]) then self.AllChatData[source] = {} end
    self.AllChatData[source][trigger] = alert
    if alert == nil and #self.AllChatData[source] == 0 then self.AllChatData[source] = nil end
end

function EventWindowsHandler:AddSelfEffectAlert(key, value)
    self.AllSelfEffectData[key] = value
end

function EventWindowsHandler:AddGroupEffectAlert(key, value)
    self.AllGroupEffectData[key] = value
end

function EventWindowsHandler:AddChatPlayerCombatAlert(trigger, alert)
    self.AllChatPlayerCombatData[trigger] = alert
end

function EventWindowsHandler:AddChatEnemyCombatAlert(trigger, alert)
    self.AllChatEnemyCombatData[trigger] = alert
end

function EventWindowsHandler:GetCurrentChatData()
    return self.AllChatData
end

function EventWindowsHandler:GetCurrentSelfEffectData()
    return self.AllSelfEffectData
end

function EventWindowsHandler:GetCurrentGroupEffectData()
    return self.AllGroupEffectData
end

function EventWindowsHandler:GetCurrentChatPlayerCombatData()
    return self.AllChatPlayerCombatData
end

function EventWindowsHandler:GetCurrentChatEnemyCombatData()
    return self.AllChatEnemyCombatData
end

function EventWindowsHandler:GetBaseChatData()
    import "PrimePlugins.RaidTools.Alerts.BaseChatData"
    return GetBaseChatData()
end

function EventWindowsHandler:GetBaseChatPlayerCombatData()
    import "PrimePlugins.RaidTools.Alerts.BaseChatPlayerCombat"
    return GetBaseChatPlayerCombatData()
end

function EventWindowsHandler:GetBaseChatEnemyCombatData()
    import "PrimePlugins.RaidTools.Alerts.BaseChatEnemyCombat"
    return GetBaseChatEnemyCombatData()
end

function EventWindowsHandler:GetBaseSelfEffectData()
    import "PrimePlugins.RaidTools.Alerts.BaseSelfEffect"
    return GetBaseSelfEffectData()
end

function EventWindowsHandler:GetBaseGroupEffectData()
    import "PrimePlugins.RaidTools.Alerts.BaseGroupEffect"
    return GetBaseGroupEffectData()
end
 