AuraHandler = class(Turbine.UI.Control)
function AuraHandler:Constructor()
    Turbine.UI.Control.Constructor(self)

    self.data = LoadTable(Turbine.DataScope.Character, "PrimeAuras")

    self.player = Turbine.Gameplay.LocalPlayer.GetInstance()

    self.ItemTools = AuraCollection.ItemToolHandler()



    -- 0 = never load | 1 = always load | 2 = instance load | 3 = pvp load

    self.auras = {
        ["Portents"] = 1,
        ["Revive Tracker"] = 2,
        ["Captain Pet Tracker"] = Turbine.Gameplay.Class.Captain == self.player:GetClass() and 2 or 0,
        ["Captain Motivating Speech Tracker"] = Turbine.Gameplay.Class.Captain == self.player:GetClass() and 1 or 0,
        ["Loremaster Pet Debuff Tracker"] = Turbine.Gameplay.Class.LoreMaster == self.player:GetClass() and 2 or 0,
        ["Loremaster Air Lore Tracker"] = Turbine.Gameplay.Class.LoreMaster == self.player:GetClass() and 2 or 0,
        ["Bear Blue Mark Tracker"] = Turbine.Gameplay.Class.Beorning == self.player:GetClass() and 1 or 0,
        ["Bear Red Mark Tracker"] = Turbine.Gameplay.Class.Beorning == self.player:GetClass() and 1 or 0,
        ["Scrope status"] = 2,
        ["Shadow Zone Tracker"] = 2,
        ["Burglar Trick Tracker"] = Turbine.Gameplay.Class.Burglar == self.player:GetClass() and 2 or 0,
        ["Mouse Tracker"] = 0,
        ["Tooltip Detach"] = 0,
    }

    self.reloadableAuras = {
        ["Loremaster Air Lore Tracker"] = true,
        ["Bear Blue Mark Tracker"] = true,
        ["Bear Red Mark Tracker"] = true,
        ["Burglar Trick Tracker"] = true,
    }

    self.itemAuras = {
        ["Portents"] = true,
        ["Threshold Jewellery"] = true,
    }

    self.otherAuras = {
        ["Revive Tracker"] =  AuraCollection.ReviveTracker,
        ["Captain Motivating Speech Tracker"] = AuraCollection.CaptainMotivated,
        ["Captain Pet Tracker"] = AuraCollection.CappyPetBanner,
        ["Loremaster Pet Debuff Tracker"] = AuraCollection.LMPetDebuffs,
        ["Loremaster Air Lore Tracker"] = AuraCollection.LMAirLore,
        ["Bear Blue Mark Tracker"] = AuraCollection.BearMarkBlueTracker,
        ["Bear Red Mark Tracker"] = AuraCollection.BearMarkRedTracker,
        ["Scrope status"] = AuraCollection.ScropeStatus,
        ["Shadow Zone Tracker"] = AuraCollection.ShadowPuddleTracker,
        ["Burglar Trick Tracker"] = AuraCollection.BurglarDebuffs,
        ["Mouse Tracker"] = AuraCollection.MouseWindow,
        ["Tooltip Detach"] = AuraCollection.TooltipDetachHandler
    }

    self.status = 1

    self.activeAuras = {}

    if self.data == nil then self.data = {} end

    for key, value in pairs(self.auras) do
        if self.data[key] == nil then
            self.data[key] = value
        end
    end

    for key, value in pairs(self.data) do
        if value == 1 and self.auras[key] then
            self:Load(key, true)
        end
    end

    self.VisibleChanged = function()
        self:Hide(self:IsVisible())
    end

    self.visibleAuras = {}
end

function AuraHandler:SetCallbacks(Name, value)
    if Name == "Portents" then
        self.ItemTools:LoadPortents(value)
    else
        if value then
            self.activeAuras[Name]:SetCallbacks()
        else
            self.activeAuras[Name]:RemoveCallbacks()
        end
        
    end


end

function AuraHandler:Load(Name, bool)
    if not(self.auras[Name]) then return end
    if self.itemAuras[Name] then
        self:SetCallbacks(Name, bool)
    elseif bool then
        if not(self.activeAuras[Name]) and self.otherAuras[Name] then
            self.activeAuras[Name] = self.otherAuras[Name]()
        end
    else
        if self.activeAuras[Name] then
            self.activeAuras[Name]:Unload()
            self.activeAuras[Name] = nil
        end
    end
end

function AuraHandler:HandleRaidLoads(bool)
    for key, value in pairs(self.data) do
        if value == 2 then
            self:Load(key, bool)
        end
    end

    if bool then self.status = 2 else self.status = 1 end
end

function AuraHandler:HandlePvpLoads(bool)
    for key, value in pairs(self.data) do
        if value == 3 then
            self:Load(key, bool)
        end
    end
    if bool then self.status = 3 else self.status = 1 end
end

function AuraHandler:GetData()
    return self.data
end

function AuraHandler:GetBase()
    return self.auras
end

function AuraHandler:ChangeData(key, value)

    self.data[key] = value
end

function AuraHandler:ReloadAurasWithDelay()
    self.reloadTime = Turbine.Engine.GetGameTime() + 5
    self:SetWantsUpdates(true)
end

function AuraHandler:ReloadAuras()
    for key, value in pairs(self.reloadableAuras) do
        if self.activeAuras[key] then
            self.activeAuras[key]:RemoveCallbacks()
            self.activeAuras[key]:SetCallbacks()
        end
    end
end

function AuraHandler:Update()
    if Turbine.Engine.GetGameTime() > self.reloadTime then
        self:ReloadAuras()
        self:SetWantsUpdates(false)
    end
end

function AuraHandler:Close()
    self.ItemTools:Close()

    for key, value in pairs(self.activeAuras) do
        value:Unload()
    end

    SaveTable(Turbine.DataScope.Character, "PrimeAuras", self.data)
end

function AuraHandler:Hide(hide)
    for key, value in pairs(self.activeAuras) do
        if not(hide) then
            self.visibleAuras[key] = value:IsVisible()
            value:SetVisible(hide)
        elseif self.visibleAuras[key] then
            value:SetVisible(self.visibleAuras[key])
        end
    end
end