BleedTimerLabel = class(Turbine.UI.Window)
function BleedTimerLabel:Constructor(parent, x, y, base, normal, baseIcon, visible, reverse, duration, normalOpacity, usageOpacity, hide, shade, text, grayOut, hideWhileOff, visibitilityThreshold, font, hideWhileOn)
    Turbine.UI.Window.Constructor(self)

    self.base = base
    self.normal = normal
    self.reverse = reverse
    self.normalOpacity = normalOpacity
    self.usageOpacity = usageOpacity
    self.hide = hide
    self.antiHide = hideWhileOff
    self.hideWhileActive = hideWhileOn
    self.shade = shade
    self.text = text
    self.grayOut = grayOut
    self.font = font or Turbine.UI.Lotro.Font.FixedSys15
    self.baseIcon = baseIcon

    self.visibitilityThreshold = visibitilityThreshold or 999999

    self.textFunction = StringifyDuration

    self:SetParent(parent)
    self:SetPosition(x, y)
    self:SetSize(self.normal, self.normal)
    self:SetVisible(visible)
    --self:SetOpacity(self.normalOpacity)
    self:SetMouseVisible(false)

    self.currentNumber = -1
    self.currentPercentage = 0

    self.baseImageID = 0x41007E35

    self.IconLabel = Turbine.UI.Control()
    self.IconLabel:SetParent(self)
    self.IconLabel:SetMouseVisible(false)
    self.IconLabel:SetSize(self.base, self.base)
    self.IconLabel:SetBackground(BackgroundHandler:GetBackground(tonumber(baseIcon) or baseIcon))
    self.IconLabel:SetStretchMode(1)
    self.IconLabel:SetSize(self.normal, self.normal)
    --self.IconLabel:SetBlendMode(Turbine.UI.BlendMode.GrayScale)
    --self.IconLabel:SetVisible(false)

    self.backgroundLabel = Turbine.UI.Label()
    self.backgroundLabel:SetParent(self)
    self.backgroundLabel:SetMouseVisible(false)
    --self.backgroundLabel:SetBackColorBlendMode(Turbine.UI.BlendMode.GrayScale)
    if self.grayOut then
        self.backgroundLabel:SetBackColor(Turbine.UI.Color(0.6, 0.3, 0.3, 0.3))
    end
    self.backgroundLabel:SetStretchMode(1)
    self.backgroundLabel:SetSize(self.normal, self.normal)

    --self.backgroundLabel:SetBlendMode(Turbine.UI.BlendMode.Overlay)


    if self.shade then
        self.elapsedOverlay = Turbine.UI.Control()
        self.elapsedOverlay:SetStretchMode( 1 )
        self.elapsedOverlay:SetMouseVisible( false )
        --self.elapsedOverlay:SetZOrder( 99 )
        self.elapsedOverlay:SetBackColor( Turbine.UI.Color( 1, 0, 0, 0 ) )
        self.elapsedOverlay:SetBackColorBlendMode( Turbine.UI.BlendMode.Overlay )
        self.elapsedOverlay:SetParent( self )
        self.elapsedOverlay:SetPosition(-1, -1)
        self.elapsedOverlay:SetSize(self.normal + 2, self.normal + 2)
    end
    
    if self.text then
        self.textLabel = Turbine.UI.Label()
        self.textLabel:SetParent(self)
        self.textLabel:SetSize(self.normal, self.normal)
        self.textLabel:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
        --self.textLabel:SetFont(Turbine.UI.Lotro.Font.TrajanProBold16)
        self.textLabel:SetFont(self.font)
        self.textLabel:SetOutlineColor(Turbine.UI.Color(0,0,0))
        self.textLabel:SetFontStyle(Turbine.UI.FontStyle.Outline)
        self.textLabel:SetMouseVisible(false)
        --self.textLabel:SetStretchMode(1)
        --self.textLabel:SetSize(self.normal, self.normal)
    end

    if self.hide then self:SetVisible(false) end
    if self.antiHide then self:SetVisible(true) end
    -- Add timer overlay

    self.startTime = Turbine.Engine.GetGameTime()
end

function BleedTimerLabel:SetTimer(timer, maxTimer, callbackFunction)
    self:SetVisible(timer <= self.visibitilityThreshold and not(self.hideWhileActive))
    self.callbackFunction = callbackFunction
    if maxTimer then
        self.overlaydDuration = maxTimer
        self.startTime = Turbine.Engine.GetGameTime() - (maxTimer - timer)
    else
        self.overlaydDuration = timer
        self.startTime = Turbine.Engine.GetGameTime()
    end

    self.duration = timer
    self:ToggleVisible(true)

    --Updater:Subscribe(self)
end

function BleedTimerLabel:ExecuteUpdate(delta)
    self.duration = self.duration - delta
    if self.duration < 0 then 
        self:ToggleVisible(false)

        if self.hideWhileActive and not(self.hide) then self:SetVisible(true) end
        if self.hide then self:SetVisible(false) end 
        if self.shade then self.elapsedOverlay:SetVisible(false) end

        if self.text then
            self.textLabel:SetStretchMode(nil)
            self.textLabel:SetStretchMode(1)
            self.textLabel:SetText("")
        end

        if self.callbackFunction then
            self.callbackFunction()
        end
        return true
    end

    if self.shade then
        self.elapsedOverlay:SetVisible(true)
        local percentComplete = math.ceil(60 * (Turbine.Engine.GetGameTime() - self.startTime) / self.overlaydDuration)
        if self.reverse then
            percentComplete = 60 - percentComplete
        end

        if self.currentPercentage ~= percentComplete then 

            self.currentPercentage = percentComplete

            local imageId = self.baseImageID + self.currentPercentage            

            self.elapsedOverlay:SetStretchMode(3)
            self.elapsedOverlay:SetSize(32, 32)
            self.elapsedOverlay:SetBackground(BackgroundHandler:GetBackground(imageId))
            self.elapsedOverlay:SetStretchMode(1)
            self.elapsedOverlay:SetSize(self.normal + 2, self.normal + 2)
        end
    end

    local newNumber = math.round(self.duration)
    if newNumber == self.currentNumber then return end

    self:SetVisible(self.duration <= self.visibitilityThreshold + 1.2 and not(self.hideWhileActive))

    self.currentNumber = newNumber

    if self.text then
        self.textLabel:SetStretchMode(nil)
        self.textLabel:SetStretchMode(1)
        self.textLabel:SetText(self.textFunction(self.currentNumber))
    end
end

function BleedTimerLabel:ToggleVisible(value)
    if not(value) then
        self.backgroundLabel:SetVisible(true)
        if self.text then
            self.textLabel:SetVisible(false)
        end
    else
        self.backgroundLabel:SetVisible(false)
        if self.text then
            self.textLabel:SetVisible(true)
        end
    end

    --self:SetVisible(value)
    --self.IconLabel:SetVisible(value)
end

function BleedTimerLabel:Unload()
    self:SetParent(nil)
end

function BleedTimerLabel:AddBackground(icon)
    self.IconLabel:SetStretchMode(nil)
    self.IconLabel:SetSize(self.base, self.base)
    self.IconLabel:SetStretchMode(1)
    self.IconLabel:SetBackground(BackgroundHandler:GetBackground(icon))
    self.IconLabel:SetSize(self.normal, self.normal)
end