DynamicSkillBar = class(Turbine.UI.Control)
function DynamicSkillBar:Constructor(parent, x, y, width, height, labelSize, sortMethod, hide, selfManagement, offset, sortSource)
    Turbine.UI.Control.Constructor(self)
    self:SetParent(parent)
    self:SetPosition(x, y)
    self:SetSize(width, height)
    self:SetMouseVisible(false)
    --self:SetBackColor(GetRandomColour())

    self.width = width
    self.selfManagement = selfManagement
    self.offset = offset or 1

    self.barHeight = 0
    self.barWidth = 0

    self.xCounts = {}

    self.width = width
    self.labelSize = labelSize + self.offset

    self.skills = {}
    self.classLabels = {}
    self.activeClassLabels = {}
    self.timerLabels = {}
    self.skillLUT = {}
    self.yIndex = {}

    self.activeTimerCount = 0

    self.sortTable = {}

    self.activeCount = 0

    self.hide = not(hide)

    self.sortSource = sortSource
    self.sortMethod = sortMethod

    self:SetSortMethod()

    self.ExecuteUpdate = function(delta)
        for key, value in pairs(self.timerLabels) do
            if self.classLabels[key]:ExecuteUpdate(delta) then
                if not(self.selfManagement) then self:ToggleLabel(key, nil) end
            end
        end
    
        if self.activeTimerCount <= 0 then
            if self.updateCallback then if self.updateCallback then RemoveCallback(Updater, "ExecuteUpdate", self.updateCallback) end end
            self.updateCallback = nil
        end
    end

    self.scheduleReload = function()
        if self.activeClassLabels then self:SortLabels() end
        if self.reloadCallback then if self.reloadCallback then RemoveCallback(Updater, "ExecuteUpdate", self.reloadCallback) end end
        self.reloadCallback = nil
    end
end

function DynamicSkillBar:TimeSort(time1, time2, key1, key2)
    if time1 > 99999 and time2 > 99999 then
        return key1 > key2
    else
        return time1 > time2
    end
end

function DynamicSkillBar:SetSortMethod()
    -- Make efficient sorting
    if self.sortSource then
        if self.fancySort then
            if self.sortMethod == 1 then -- Left
                self.SortLabels = function()
                    local counter = 0
                    for key,_ in spairs(self.activeClassLabels, function(t,a,b) return self:TimeSort(self.classLabels[b]:GetRemainingTime(), self.classLabels[a]:GetRemainingTime(), a, b) end) do
                        self.classLabels[key]:MoveLabelFancy(counter * self.labelSize, 0)
                        counter = counter + 1
                    end
                end
            elseif self.sortMethod == 2 then -- Center
                self.SortLabels = function()
                    --ChatOutput("DD")
                    local counter = 0
                    for key,_ in spairs(self.activeClassLabels, function(t,a,b) return self:TimeSort(self.classLabels[b]:GetRemainingTime(), self.classLabels[a]:GetRemainingTime(), a, b) end) do
                        self.classLabels[key]:MoveLabelFancy(self.width / 2 - self.labelSize / 2 * self.activeCount + counter * self.labelSize + self.offset, 0)
                        counter = counter + 1
                    end
                end
            elseif self.sortMethod == 3 then -- Right
                self.SortLabels = function()
                    local counter = 0
                    for key,_ in spairs(self.activeClassLabels, function(t,a,b) return self:TimeSort(self.classLabels[b]:GetRemainingTime(), self.classLabels[a]:GetRemainingTime(), a, b) end) do
                        self.classLabels[key]:MoveLabelFancy(self.width - (counter + 1) * self.labelSize, 0)
                        counter = counter + 1
                    end
                end
            elseif self.sortMethod == 4 then
                self.SortLabels = function()
    
                end
            end
        else
            if self.sortMethod == 1 then -- Left
                self.SortLabels = function()
                    local counter = 0
                    for key,_ in spairs(self.activeClassLabels, function(t,a,b) return self:TimeSort(self.classLabels[b]:GetRemainingTime(), self.classLabels[a]:GetRemainingTime(), a, b) end) do
                        self.classLabels[key]:SetPosition(counter * self.labelSize, 0)
                        counter = counter + 1
                    end
                end
            elseif self.sortMethod == 2 then -- Center
                self.SortLabels = function()
                    --ChatOutput("DD")
                    local counter = 0
                    for key,_ in spairs(self.activeClassLabels, function(t,a,b) return self:TimeSort(self.classLabels[b]:GetRemainingTime(), self.classLabels[a]:GetRemainingTime(), a, b) end) do
                        self.classLabels[key]:SetPosition(self.width / 2 - self.labelSize / 2 * self.activeCount + counter * self.labelSize + self.offset, 0)
                        counter = counter + 1
                    end
                end
            elseif self.sortMethod == 3 then -- Right
                self.SortLabels = function()
                    local counter = 0
                    for key,_ in spairs(self.activeClassLabels, function(t,a,b) return self:TimeSort(self.classLabels[b]:GetRemainingTime(), self.classLabels[a]:GetRemainingTime(), a, b) end) do
                        self.classLabels[key]:SetPosition(self.width - (counter + 1) * self.labelSize, 0)
                        counter = counter + 1
                    end
                end
            elseif self.sortMethod == 4 then
                self.SortLabels = function()
    
                end
            end
        end
    else
        if self.sortMethod == 1 then -- Left
            self.SortLabels = function()
                local counter = 0
                for key,_ in spairs(self.activeClassLabels, function(t,a,b) return self.sortTable[b] > self.sortTable[a] end) do
                    self.classLabels[key]:SetPosition(counter * self.labelSize, 0)
                    counter = counter + 1
                end
            end
        elseif self.sortMethod == 2 then -- Center
            self.SortLabels = function()
                local counter = 0
                for key,_ in spairs(self.activeClassLabels, function(t,a,b) return self.sortTable[b] > self.sortTable[a] end) do
                    self.classLabels[key]:SetPosition(self.width / 2 - self.labelSize / 2 * self.activeCount + counter * self.labelSize, 0)
                    counter = counter + 1
                end
            end
        elseif self.sortMethod == 3 then -- Right
            self.SortLabels = function()
                local counter = 0
                for key,_ in spairs(self.activeClassLabels, function(t,a,b) return self.sortTable[b] > self.sortTable[a] end) do
                    self.classLabels[key]:SetPosition(self.width - (counter + 1) * self.labelSize, 0)
                    counter = counter + 1
                end
            end
        elseif self.sortMethod == 4 then
            self.SortLabels = function()

            end
        end
    end
end

function DynamicSkillBar:SetFancySort(value)
    self.fancySort = value
    self:SetSortMethod()
end

function DynamicSkillBar:SetLabel(label)
    local counter = 0
    for key,_ in spairs(self.activeClassLabels, function(t,a,b) return self.classLabels[b]:GetRemainingTime() > self.classLabels[a]:GetRemainingTime() end) do
        if self.classLabels[key] == label then
            self.classLabels[key]:SetPosition(counter * self.labelSize, 0)
        end
        counter = counter + 1
    end

    if not(self.reloadCallback) then self.reloadCallback = AddCallback(Updater, "ExecuteUpdate", self.scheduleReload) end
end

function DynamicSkillBar:LabelExists(skill)
    return self.classLabels[skill] ~= nil
end

function DynamicSkillBar:AddLabel(skill, label, priority, skillObject, y)
    if self.classLabels[skill] then
        return
    end
    self.sortTable[skill] = priority

    self.classLabels[skill] = label
    self.skillLUT[skill] = skillObject

    if y == nil then y = 1 end

    if not(self.yIndex[y]) then self.yIndex[y] = {} end

    self.yIndex[y][skill] = true

    -- Update counts
    if self.xCounts[y] then
        self.xCounts[y] = self.xCounts[y] + 1
    else
        self.xCounts[y] = 1
    end

    if self.barHeight < y then 
        self.barHeight = y 
        self:SetHeight(y * self.labelSize)
    end
end

function DynamicSkillBar:ExpandHeight(y)
    self:SetHeight(y)

    if y / self.labelSize > self.barHeight then
        self.barHeight = y / self.labelSize
    end
end

function DynamicSkillBar:MaxHeight()
    return self.barHeight
end

function DynamicSkillBar:SetStaticPositions(alignment, usePriorityAsIndex)
    for yIndex, skills in pairs(self.yIndex) do
        local positionCounter = 0
        if alignment == "Left" then
            for skillName, _ in spairs(skills, function(t,a,b) return self.sortTable[b] > self.sortTable[a] end) do
                if usePriorityAsIndex then
                    self.classLabels[skillName]:SetPosition(self.sortTable[skillName] * self.labelSize, (yIndex - 1) * self.labelSize)
                else
                    self.classLabels[skillName]:SetPosition(positionCounter * self.labelSize, (yIndex - 1) * self.labelSize)
                    positionCounter = positionCounter + 1
                end
            end
        elseif alignment == "Right" then
            local padding = self.width - self.xCounts[yIndex] * self.labelSize
            for skillName, _ in spairs(skills, function(t,a,b) return self.sortTable[b] > self.sortTable[a] end) do
                if usePriorityAsIndex then
                    self.classLabels[skillName]:SetPosition(padding + self.sortTable[skillName] * self.labelSize, (yIndex - 1) * self.labelSize)
                else
                    self.classLabels[skillName]:SetPosition(padding + positionCounter * self.labelSize, (yIndex - 1) * self.labelSize)
                    positionCounter = positionCounter + 1
                end
            end
        else
            local padding = self.width / 2 - self.xCounts[yIndex] * (self.labelSize / 2)
            for skillName, _ in spairs(skills, function(t,a,b) return self.sortTable[b] > self.sortTable[a] end) do
                if usePriorityAsIndex then
                    self.classLabels[skillName]:SetPosition(padding + self.sortTable[skillName] * self.labelSize, (yIndex - 1) * self.labelSize)
                else
                    self.classLabels[skillName]:SetPosition(padding + positionCounter * self.labelSize, (yIndex - 1) * self.labelSize)
                    positionCounter = positionCounter + 1
                end
            end
        end

    end
end

function DynamicSkillBar:MaxWidth()
    local maxWidth = 0 
    for key, value in pairs(self.xCounts) do
        if value > maxWidth then maxWidth = value end
    end
    return maxWidth
end

function DynamicSkillBar:ToggleHighlight(id, toggle)
    if self.classLabels[id] then
        self.classLabels[id]:ToggleHighlight(toggle)
    end
end

function DynamicSkillBar:ToggleVisible(id, value)
    if self.classLabels[id] then
        self.classLabels[id]:ToggleVisible(value)
    end
end

function DynamicSkillBar:ToggleLabel(skill, value, time, icon)

    -- Set visible
    if value and time > 0 then 
       
        -- Check if already visible
        if self.activeClassLabels[skill] then
            -- Correct timer and return
            if time then self:SetLabelTimer(skill, time) end
            return
        end

        -- Otherwise set visible and update book keeping
        if icon then self.classLabels[skill]:AddBackground(icon) end
        self.activeClassLabels[skill] = true

        self.classLabels[skill]:ToggleVisible(true)
        self.activeCount = self.activeCount + 1
        if time then self:SetLabelTimer(skill, time) end

        if self.fancySort then
            if self.sortMethod == 2 then -- Center
                self.classLabels[skill]:SetPosition(self.width / 2 - self.labelSize / 2 * 1 + self.offset, 0)
            elseif self.sortMethod == 3 then -- Right
                self.classLabels[skill]:SetPosition(self.width - (1) * self.labelSize, 0)
            end
        end

    else -- Disable label

        -- Check if already disabled
        if self.activeClassLabels[skill] == nil then 
            return 
        end

        --ChatOutput(skill .. "Disable")
        self.classLabels[skill]:ToggleVisible(self.hide)
        self.activeCount = self.activeCount - 1
        self.activeClassLabels[skill] = nil

        if self.timerLabels[skill] then
            self.activeTimerCount = self.activeTimerCount - 1
            self.timerLabels[skill] = nil
            if self.activeTimerCount <= 0 then
                if self.updateCallback then if self.updateCallback then RemoveCallback(Updater, "ExecuteUpdate", self.updateCallback) end end
                self.updateCallback = nil
            end
        end
    end

    if not(self.reloadCallback) then self.reloadCallback = AddCallback(Updater, "ExecuteUpdate", self.scheduleReload) end

    --[[
    if self.activeClassLabels[skill] ~= value then
        self.classLabels[skill]:ToggleVisible(value)
        if value then
            self.activeCount = self.activeCount + 1
            self.activeClassLabels[skill] = value
        else
            self.activeCount = self.activeCount - 1 
            self.activeClassLabels[skill] = nil
        end
    end

    if time == true and self.skillLUT[skill]:GetResetTime() > 0 then
        self:SetLabelTimer(skill, self.skillLUT[skill]:GetResetTime() - Turbine.Engine.GetGameTime(), self.skillLUT[skill]:GetCooldown())
    elseif time and time ~= true and time > 0 then
        self:SetLabelTimer(skill, time)
    end
    --]]
end

function DynamicSkillBar:RemoveLabel(skill)
    if self.classLabels[skill] then
        self.sortTable[skill] = nil
        self.classLabels[skill]:SetVisible(false)
        self.classLabels[skill]:Unload()
        self.classLabels[skill] = nil
        self.skillLUT[skill] = nil
        self.timerLabels[skill] = nil
        if self.activeClassLabels[skill] then
            self.activeCount = self.activeCount - 1
        end
        self.activeClassLabels[skill] = nil
        if not(self.reloadCallback) then self.reloadCallback = AddCallback(Updater, "ExecuteUpdate", self.scheduleReload) end
    end
end

function DynamicSkillBar:SetLabelTimer(skill, time, maxTime)

    if time <= 0 then return end

    if not(self.timerLabels[skill]) then
        self.activeTimerCount = self.activeTimerCount + 1
    end

    self.classLabels[skill]:SetTimer(time, maxTime)
    self.activeClassLabels[skill] = true
    self.timerLabels[skill] = true

    if not(self.updateCallback) then self.updateCallback = AddCallback(Updater, "ExecuteUpdate", self.ExecuteUpdate) end
end

function DynamicSkillBar:Unload(preseveSelf)

    for key, value in spairs(self.classLabels, function(t, a, b) return tostring(a) < tostring(b) end) do
        self.sortTable[key] = nil
        self.classLabels[key]:SetVisible(false)
        self.classLabels[key]:Unload()
        self.classLabels[key] = nil
        self.skillLUT[key] = nil
        self.timerLabels[key] = nil
        self.activeClassLabels[key] = nil
    end

    if tablelength(self.classLabels) > 0 then self:Unload(preseveSelf) end

    if self.updateCallback then if self.updateCallback then RemoveCallback(Updater, "ExecuteUpdate", self.updateCallback) end end
    self.updateCallback = nil

    self.sortTable = nil
    self.skillLUT = nil
    self.yIndex = nil
    self.skills = nil
    self.classLabels = nil
    self.activeClassLabels = nil
    self.timerLabels = nil
    self.skillLUT = nil
    self.sortTable = nil
    self.xCounts = nil

    if preseveSelf then
        self.sortTable = {}
        self.skillLUT = {}
        self.yIndex = {}
        self.skills = {}
        self.classLabels = {}
        self.activeClassLabels = {}
        self.timerLabels = {}
        self.skillLUT = {}
        self.sortTable = {}
        self.xCounts = {}
        self.activeTimerCount = 0
        self.activeCount = 0
        self.barHeight = 0
        self.barWidth = 0
    else
        self:SetParent(nil)
    end
end

function DynamicSkillBar:Clear()
    if not(self.selfManagement) then
        for key, value in spairs(self.classLabels, function(t, a, b) return a < b end) do
            self.sortTable[key] = nil
            self.classLabels[key]:SetVisible(false)
            self.classLabels[key]:Unload()
            self.classLabels[key] = nil
            self.skillLUT[key] = nil
            self.timerLabels[key] = nil
            self.activeClassLabels[key] = nil
        end

        if tablelength(self.classLabels) > 0 then self:Clear() end

        if self.updateCallback then if self.updateCallback then RemoveCallback(Updater, "ExecuteUpdate", self.updateCallback) end end
        self.updateCallback = nil

        self.sortTable = nil
        self.skillLUT = nil
        self.yIndex = nil
        self.skills = nil
        self.classLabels = nil
        self.activeClassLabels = nil
        self.timerLabels = nil
        self.skillLUT = nil
        self.sortTable = nil
        self.xCounts = nil

        collectgarbage()

        self.sortTable = {}
        self.skillLUT = {}
        self.yIndex = {}
        self.skills = {}
        self.classLabels = {}
        self.activeClassLabels = {}
        self.timerLabels = {}
        self.skillLUT = {}
        self.sortTable = {}
        self.xCounts = {}
        self.activeTimerCount = 0
        self.activeCount = 0
        self.barHeight = 0
        self.barWidth = 0
    end
end