
GuardianTools = class(Turbine.UI.Window)
function GuardianTools:Constructor(data, buffData)
    Turbine.UI.Window.Constructor(self)
    self.width = 500
    self.height = 500
    self:SetPosition(Settings.ClassToolsX, Settings.ClassToolsY)
    self:SetSize(self.width, self.height)
    --self:SetBackColor(GetRandomColour())
    self:SetVisible(true)
    self:SetMouseVisible(false)

    self.data = data
    self.buffData = buffData

    self.moveBox = MoveBox(self, self.width, self.height, "Guardian Tools")

    self.ReactionBar = DynamicSkillBar(self, 0, 50, self.width, 32, 32, 2, true)
    self.SmallCDBar = DynamicSkillBar(self, 0, 18, self.width / 2 - 25, 32, 32, 3, true)
    self.BigCDBar = DynamicSkillBar(self, self.width / 2 + 25, 18, self.width / 2 - 25, 32, 32, 1, true)

    -- Add small health bar

    -- Add skill quickslots
    self.normalSkillBar = DynamicSkillBar(self, 0, 82, self.width, 64, 32, 4, false)

    -- Proper cleaning
    self.effectCallbacks = {}
    self.skillCallbacks = {}
    self.skillsLUT = {}
    self.playerCallbacks = {}

    self.skillLUT = {}

    self.normalSkills = {}

    self.baseSize = 32
    self.reactionSize = 32
    self.fortSize = 50

    self.specialSkillCount = 0
    self.skillCount = 0

    self.noSkills = Settings.ClassToolsNoSkills
    self.breakRankEnabled = Settings.ClassToolsGuardianBreakRankLabel


    self:LoadData()

    self.fortification = 0
    self.breakRanksAvailable = false
    self.breakRanksTime = 0

    -- 0 base | 1 after re | 2 after sh | 3 after both
    self.smashingStabeState = 0

    self.duration = 0

    -- Cooldown table

    self.cooldownTable = {
        ["Juggernaut"] = 1091805299,
        ["Guardian's Pledge"] = 1090541178,
        ["Redirect"] = 1091805300,
        ["Warrior's Heart"] = 1090541182,
        ["Litany of Defiance"] = 1091444826,
    }

    -- small coodlowns
    self.smallcooldownTable = {
        ["Charge"] = 1090553762,
        ["Damage Reflection"] = 1091805300,
        ["Bring on the Pain"] = 1090527705,
        ["Stoic"] = 1091668185,
    }

    self.specialSmallcooldownTable = {
        ["Damage Reflection"] = 1091805300,
        ["Bring on the Pain"] = 1090527705,
        ["Stoic"] = 1091668185,
        ["Break Ranks"] = 1091805247,
    }

    -- big cooldowns
    self.bigcooldownTable = {
        ["Juggernaut"] = 1091805299,
        ["Guardian's Pledge"] = 1090541178,
        ["Warrior's Heart"] = 1090541182,
        ["Litany of Defiance"] = 1091444826,
    }

    if self.buffData == nil or tablelength(self.buffData) < 1  then
        self.buffData = {}
        for key, value in pairs(self.smallcooldownTable) do
            self.buffData[key] = {value, 1, false, false, true}
        end
    else
        for key, value in pairs(self.buffData) do
            if value[5] == false then
                self.smallcooldownTable[key] = nil
                --self.specialEffectTable[key] = nil
                --self.removeEffectTable[key] = nil
            elseif self.smallcooldownTable[key] == nil then
                self.smallcooldownTable[key] = value[1]

                if value[3] then
                    --self.specialEffectTable[key] = value[1]
                end

                if value[4] then
                    --self.removeEffectTable[key] = true
                end
            end
        end
    end

    self.cooldownLabels = {}
    self.activeLabels = {}

    -- Normal skills - 20

        --[[

        Hitting skills - 9

            Guardian's Ward Tactics **
            Sweeping Cut
            Improved Sting
            War-chant **
            Vexing Blow
            Shield-blow
            Smashing Stab **
            Stagger
            Stamp **

        Taunt skills - 3

            Challenge
            Engage
            Fray the Edge

        Utility - 2

            Ignore the Pain
            Turn the Tables

        Cooldowns - 6

            Juggernaut
            Guardian's Pledge
            Warrior's Heart
            Litany of Defiance

            Thrill of Danger
            Catch a Breath **

    ]]

    -- Add break ranks timer
    self.SkillBarSkills = {
        -- Bar 1
        ["Guardian's Ward"] = {3, "0x70003066", 1},
        ["Improved Sting"] ={2, "0x7000F1E1", 1},
        ["Shield-blow"] = {1,"0x7000306F", 1},
        ["Sweeping Cut"] = {6, "0x70003EDE", 1},
        ["Vexing Blow"] = {5, "0x7000308D", 1},
        ["Catch a Breath"] = {10, "0x7000307D", 1},
        ["Stamp"] = {4, "0x70003EE7", 1},
        ["War-chant"] = {8, "0x70036947", 1},
        ["Smashing Stab"] = {9, "0x70036DCD", 1},
        ["Stagger"] = {7, "0x7000D4B3", 1},
        ["Ignore the Pain"] = {11, "0x70021663", 1},
        ["Thrill of Danger"] = {12, "0x700030B4", 1},
        ["Turn the Tables"] = {14, " 0x70003EE8", 1},
        ["Charge"] = {13, "0x7000DD14", 1},

        -- Bar 2

        ["Redirect"] = {7, "0x70036945", 2},
        ["Challenge"] = {4, "0x70003097", 2},
        ["Engage"] = {5, "0x7000D4B2", 2},
        ["Fray the Edge"] = {6, "0x7000D4B5", 2},
        ["Guardian's Pledge"] = {9, "0x700030A4", 2},
        ["Juggernaut"] = {10, "0x70036946", 3},
        ["Warrior's Heart"] = {11, "0x700030AC", 2},
        ["Litany of Defiance"] = {12, "0x7000F095", 2},
        --break ranks

    }

    if self.data == nil then
        self.data = {}
        for key, value in pairs(self.SkillBarSkills) do
            self.data[key] = {true, value[1], value[3]}
        end
    else
        for key, value in pairs(self.data) do
            --ChatOutput(value[1])
            if value[1] == false then
                self.SkillBarSkills[key] = nil
            else
                self.SkillBarSkills[key] = {value[2], self.SkillBarSkills[key][2], value[3]}
            end
        end
    end

    -- forticon IDs  
    self.FortificationTable = {
        --[1090527700] = 0,
        [1091840994] = 1,
        [1091840981] = 2,
        [1091840991] = 3,
        [1091840993] = 4,
        [1091840986] = 5
    }

    -- Skill table
    self.SkillTable = {
        ["Shield-swipe"] = {1, "0x7000305D"},
        ["Shield-taunt"] = {2, "0x70003EE6"},
        ["Bash"] = {3, "0x70003EE1"},
        ["Shield-smash"] = {4, "0x70002EC4"},
        ["Break Ranks"] = {5, nil},

        ["Retaliation"] = {6, "0x70003EE0"},
        ["Whirling Retaliation"] = {7, "0x70003EE4"},
        ["Redirect"] = {8, "0x70036945"},

        ["Catch a Breath"] = {9, "0x7000307D"},
    }

    -- [1] 1 = block 2 = parry [2] tier
    self.ResponseTable = {
        ["Block Response"] = {1, 1},
        ["Shield Swipe"] = {1, 2},
        ["Parry Response"] = {2, 1},
        ["Retaliation"] = {2, 2}
    }

    self.SkillUsageTable = {
        ["Shield-swipe"] = {1, 1},
        ["Shield-taunt"] = {1, 2},
        ["Bash"] = {1, 2},
        ["Shield-smash"] = {1, 3},

        ["Retaliation"] = {2, 1},
        ["Whirling Retaliation"] = {2, 1},
        ["Redirect"] = {2, 2},
    }

    self.SkillIcons = {}

    self.ExecuteUpdate = function(delta)
        self.duration = self.duration - delta
        --ChatOutput(self.duration)
        if self.duration < 0 then
            self:SmashingStabUpdate(0)
            self.fortification = 0
            self.FortLabel2:ChangeStacks(0)
            if self.updateCallback then if self.updateCallback then RemoveCallback(Updater, "ExecuteUpdate", self.updateCallback) end end
            self.updateCallback = nil
        end
    end

    --self.FortLabel = ClassSkillLabel(self, 0, 0, self.baseSize, self.fortSize, 1090527700, false)


    --Bleed 0x41000083
    -- Color 0x41007E13
    -- Shield  0x41007E90
    -- Knockdown shield 0x4100823E
    -- Stun 0x4100823F
    -- Man line 0x410087B8
    -- SHield and sword 0x4112E312
    -- Yellow break ranks 0x411F5271

    self:Load()
end

function GuardianTools:SmashingStabUpdate(action)
    --ChatOutput(action)
    if self.normalSkills["Smashing Stab"] then self.normalSkills["Smashing Stab"]:SetMode(action) end
    ----ChatOutput("SM" .. self.smashingStabeState)
end

function GuardianTools:SetSkillObjects()

    self.FortLabel2 = FortificationIndicator(self, self.width / 2 - 25, 0, self.baseSize, self.fortSize, true) 

    if self.breakRankEnabled and not(self.noSkills) then
        self.BreakRanksLabel = ClassTimerLabel(self.normalSkillBar, 9 + 32 * 7, 32, self.baseSize, self.reactionSize, 0x41139EFF, true, true, 0, 1, 1, false, true, true)
        self.normalSkillBar:AddLabel("Break Ranks", self.BreakRanksLabel, Settings.ClassToolsGuardianBreakRankXPrio, nil, Settings.ClassToolsGuardianBreakRankYBar)
        self.normalSkills["Break Ranks"] = self.BreakRanksLabel
    end

    self.SkillIcons["Break Ranks"] = ClassTimerLabel(self.ReactionBar, 0, 0, self.baseSize, self.reactionSize, 1091805247, false, true, 0, 1, 1, false, false, true)
    self.ReactionBar:AddLabel("Break Ranks", self.SkillIcons["Break Ranks"], self.SkillTable["Break Ranks"][1], nil)

    self.OutOfCombatLabel = ClassTimerLabel(self.ReactionBar, 0, 0, self.baseSize, self.reactionSize, 0x410D3DE9, false, true, 0, 1, 1, false, false, true)
    self.ReactionBar:AddLabel("Leaving Combat", self.OutOfCombatLabel, 1, nil)

    for key, value in pairs(self.specialSmallcooldownTable) do
        --ChatOutput(key)
        self.SmallCDBar:AddLabel(key, ClassTimerLabel(self.SmallCDBar, 0, 0, self.baseSize, self.reactionSize, value, false, false, 0, 1, 1, true, false, true), 1)
    end

    local index = 1
    for i = 1, self.trainedSkills:GetCount(), 1 do
        local item = self.trainedSkills:GetItem(i)--:GetSkillInfo()
        local name = item:GetSkillInfo():GetName()
        ----ChatOutput(name .. item:GetSkillInfo():GetIconImageID())

        self.skillCallbacks[name] = {}

        if self.SkillTable[name] then
            
            self.SkillIcons[name] = ClassSkillSlot(self.ReactionBar, 50 * index, 0, self.baseSize, self.reactionSize, self.SkillTable[name][2], true)
            self.SkillIcons[name]:SetTimer(item:GetResetTime() - Turbine.Engine.GetGameTime(), item:GetCooldown())
            self.ReactionBar:AddLabel(name, self.SkillIcons[name], self.SkillTable[name][1], item)
            index = index + 1
        end

        if self.SkillBarSkills[name] and not(self.noSkills) then
            local offset = 9
            if self.SkillBarSkills[name][3] == 1 then offset = 25 end
            --self.normalSkills[name] = ClassTimerLabel(self.normalSkillBar, offset + 32 * (self.SkillBarSkills[name][1] - 1), 32 * (self.SkillBarSkills[name][3] - 1), self.baseSize, self.reactionSize, item:GetSkillInfo():GetIconImageID(), true, true, 0, 1, 1, false)
            --self.normalSkillBar:AddLabel(name, self.normalSkills[name], nil, item)
            if name == "Smashing Stab" then
                self.normalSkills[name] = SmashingStabSlot(self.normalSkillBar, offset + 32 * (self.SkillBarSkills[name][1] - 1), 32 * (self.SkillBarSkills[name][3] - 1), self.baseSize, self.reactionSize, self.SkillBarSkills[name][2])
            else
                self.normalSkills[name] = ClassSkillSlot(self.normalSkillBar, offset + 32 * (self.SkillBarSkills[name][1] - 1), 32 * (self.SkillBarSkills[name][3] - 1), self.baseSize, self.reactionSize, self.SkillBarSkills[name][2])
            end
            self.normalSkillBar:AddLabel(name, self.normalSkills[name], self.SkillBarSkills[name][1], item, self.SkillBarSkills[name][3])
            self.normalSkillBar:SetLabelTimer(name, item:GetResetTime() - Turbine.Engine.GetGameTime(), item:GetCooldown())

            if self.SkillBarSkills[name][3] == 1 then
                self.skillCount = self.skillCount + 1
            elseif self.SkillBarSkills[name][3] == 2 then
                self.specialSkillCount = self.specialSkillCount + 1
            end
        end
    
        if self.smallcooldownTable[name] then
            self.SmallCDBar:AddLabel(name, ClassTimerLabel(self.SmallCDBar, 0, 0, self.baseSize, self.reactionSize, item:GetSkillInfo():GetIconImageID(), false, false, 0, 1, 1, true, false, true), 1)
        end

        if self.bigcooldownTable[name] then
            self.BigCDBar:AddLabel(name, ClassTimerLabel(self.BigCDBar, 0, 0, self.baseSize, self.reactionSize, item:GetSkillInfo():GetIconImageID(), false, false, 0, 1, 1, true, false, true), 1)
        end
    end

    if Settings.ClassToolsGuardianBreakRankYBar == 1 then
        self.skillCount = self.skillCount + 1
    elseif Settings.ClassToolsGuardianBreakRankYBar == 2 then
        self.specialSkillCount = self.specialSkillCount + 1
    end

    self.SkillBarSkills["Break Ranks"] = {Settings.ClassToolsGuardianBreakRankXPrio, nil, Settings.ClassToolsGuardianBreakRankYBar}

    self.normalSkillBar:SetStaticPositions()

    -- Handle initial state
    for i = 1, self.effectList:GetCount(), 1 do
        local name = self.effectList:Get(i):GetName()

        local response = self.ResponseTable[name]
        if response then
            if response[1] == 1 then self:HandleBlockUpdate(response[2]) end
            if response[1] == 2 then self:HandleParryUpdate(response[2]) end
        end

        local effectIcon = self.effectList:Get(i):GetIcon()

        if self.FortificationTable[effectIcon] then
            self.fortification = self.FortificationTable[effectIcon]
            self.FortLabel2:ChangeStacks( self.fortification)
            --self.FortLabel:ChangeIcon(effectIcon)
        end
    end
end

function GuardianTools:HandleBlockUpdate(state)
    if state == 1 then
        --ChatOutput("Shield-Swipe")
        self.ReactionBar:ToggleLabel("Shield-swipe", true, 6)
        --self.SkillIcons["Shield-swipe"]:SetVisible(true)
        -- Refresh timer
    elseif state == 2 then
        --self.SkillIcons["Shield-swipe"]:SetVisible(false)
        self.ReactionBar:ToggleLabel("Bash", true, 6)
        self.ReactionBar:ToggleLabel("Shield-taunt", true, 6)
        if self.fortification == 5 and Turbine.Engine.GetGameTime() - self.breakRanksTime > 60 then
            self.ReactionBar:ToggleLabel("Break Ranks", true, 6)
        end
        --self.SkillIcons["Bash"]:SetVisible(true)
        --self.SkillIcons["Shield-taunt"]:SetVisible(true)
        --ChatOutput("Bash")
        --ChatOutput("Taunt")
        -- Refresh timer

        -- Update stab
        self:SmashingStabUpdate(2)
    elseif state == 3 then
        --ChatOutput("Smash")
        self.ReactionBar:ToggleLabel("Shield-smash", true, 6)
        --self.SkillIcons["Shield-smash"]:SetVisible(true)
        -- Refresh timer
    end
end

function GuardianTools:HandleBlockRemovalUpdate(state)
    if state == 1 then
        self.ReactionBar:ToggleLabel("Shield-swipe", nil)
    elseif state == 2 then
        self.ReactionBar:ToggleLabel("Bash", nil)
        self.ReactionBar:ToggleLabel("Shield-taunt", nil)
    elseif state == 3 then
        -- Add timeout
        self.ReactionBar:ToggleLabel("Shield-smash", nil)
    end
end

function GuardianTools:HandleParryRemovalUpdate(state)
    if state == 1 then
        self.ReactionBar:ToggleLabel("Retaliation", nil)
        self.ReactionBar:ToggleLabel("Whirling Retaliation", nil)
    elseif state == 2 then
        self.ReactionBar:ToggleLabel("Redirect", nil)
    elseif state == 3 then
    end
end

function GuardianTools:HandleParryUpdate(state)
    if state == 1 then
        self.ReactionBar:ToggleLabel("Retaliation", true, 6)
        self.ReactionBar:ToggleLabel("Whirling Retaliation", true, 6)
        -- Refresh timer
    elseif state == 2 then
        self.ReactionBar:ToggleLabel("Redirect", true, 6)
        -- Refresh timer

        -- Update stab
        self:SmashingStabUpdate(1)
    elseif state == 3 then
    end
end

function GuardianTools:RemoveCallbacks()
    --player callbacks
    for key, value in pairs(self.playerCallbacks) do
        RemoveCallback(self.player, key, value)
    end

    for key, value in pairs(self.effectCallbacks) do
        RemoveCallback(PlayerEffectsHandler, key, value)
    end

    for key, value in pairs(self.skillCallbacks) do
        for _, callback in pairs(value) do
            RemoveCallback(self.skillsLUT[key], "ResetTimeChanged", callback)
        end
    end

    collectgarbage()
end

function GuardianTools:SetCallbacks()

    -- broken
    self.ClassAttr.IsBlockTier1AvailableChanged = function()
        --ChatOutput("1")
    end

    self.ClassAttr.IsBlockTier2AvailableChanged = function()
        --ChatOutput("2")
        if self.ClassAttr:IsBlockTier2Available() then self:HandleBlockUpdate(2) end
    end

    self.ClassAttr.IsBlockTier3AvailableChanged = function()
        --ChatOutput("3")
        if self.ClassAttr:IsBlockTier3Available() then self:HandleBlockUpdate(3) end
    end

    -- broken
    self.ClassAttr.IsParryTier1AvailableChanged = function()
        --ChatOutput("p1")
        if self.ClassAttr:IsParryTier1Available() then self:HandleParryUpdate(1) end
    end

    self.ClassAttr.IsParryTier2AvailableChanged = function()
        --ChatOutput("p2")
        if self.ClassAttr:IsParryTier2Available() then self:HandleParryUpdate(2) end
    end

    -- Red memes
    self.ClassAttr.IsParryTier3AvailableChanged = function()
        --ChatOutput("p3")
    end

    self.playerCallbacks["InCombatChanged"] = AddCallback(self.player, "InCombatChanged",
    function(sender, args)
        if not(self.player:IsInCombat()) then
            self.duration = 9
            self.ReactionBar:ToggleLabel("Leaving Combat", true, 9)
            if not(self.updateCallback) then self.updateCallback = AddCallback(Updater, "ExecuteUpdate", self.ExecuteUpdate) end
        end
    end)

    
    -- Handle effect updates
    self.effectCallbacks["EffectAdded"] = AddCallback(PlayerEffectsHandler, "EffectAdded", 
    function(effect)
        local effectName = effect:GetName()
        if effectName == "Block Response" then
            self:HandleBlockUpdate(1)
        elseif effectName == "Parry Response" then
            self:HandleParryUpdate(1)
        end

        if effectName == "Break Ranks" then
            --ChatOutput("BR")
            if not(self.noSkills) then self.normalSkillBar:SetLabelTimer("Break Ranks", 60, 60) end
            self.breakRanksTime = Turbine.Engine.GetGameTime()
            self.ReactionBar:ToggleLabel("Break Ranks", nil)
            self.SmallCDBar:ToggleLabel("Break Ranks", true, 15)
        end

        local effectIcon = effect:GetIcon()

        --ChatOutput(effect:GetName() .. effect:GetIcon())

        if self.FortificationTable[effectIcon] then
            self.fortification = self.FortificationTable[effectIcon]
            self.FortLabel2:ChangeStacks( self.FortificationTable[effectIcon])
            --self.FortLabel:ChangeIcon(effectIcon)
        elseif effectName == "Litany of Defiance" or (effectName == "Break Ranks" and self.fortification == 5) then
            --self.FortLabel:ChangeIcon(1090527700)
            self.fortification = 0
            self.FortLabel2:ChangeStacks(0)
        end

        if self.smallcooldownTable[effectName] and effect:GetDuration() < 99999 then
            self.SmallCDBar:ToggleLabel(effectName, true, effect:GetDuration(), self.smallcooldownTable[effectName] == true and effect:GetIcon())
        end

        if self.bigcooldownTable[effectName] and self.bigcooldownTable[effectName] == effectIcon then
            self.BigCDBar:ToggleLabel(effectName, true, effect:GetDuration())
        end
    end)

    -- [[
    self.effectCallbacks["EffectRemoved"] = AddCallback(PlayerEffectsHandler, "EffectRemoved", 
    function(effect)
        --[[
        --local effect = sender:Get(args.Index) 
        local effectName = args.Effect:GetName()

        local response = self.ResponseTable[effectName]
        if response then
            if response[1] == 1 then
                self:HandleBlockRemovalUpdate(response[2])
            elseif response[1] == 2 then
                self:HandleParryRemovalUpdate(response[2])
                -- parry
            end
        end
        --]]

        if effect:GetName() == "Stoic" then
            self.SmallCDBar:ToggleLabel("Stoic", nil)
        end

        if self.FortificationTable[effect:GetIcon()] and not(self.player:IsInCombat()) then
            --:ChangeIcon(1090527700)
            self.FortLabel2:ChangeStacks( 0)
        end
    end)

    -- skill callbacks
    local index = 1
    for i = 1, self.trainedSkills:GetCount(), 1 do
        local item = self.trainedSkills:GetItem(i)--:GetSkillInfo()
        local name = item:GetSkillInfo():GetName()
        ----ChatOutput(name .. item:GetSkillInfo():GetIconImageID())

        self.skillCallbacks[name] = {}

        if self.SkillTable[name] and not(self.noSkills) then
            self.skillsLUT[name] = item
           table.insert(self.skillCallbacks[name], AddCallback(item, "ResetTimeChanged", function(sender, args) 
                self.SkillIcons[name]:SetTimer(item:GetResetTime() - Turbine.Engine.GetGameTime(), item:GetCooldown())
            end))
        end

        if self.SkillBarSkills[name] and not(self.noSkills) then
            self.skillsLUT[name] = item
           table.insert(self.skillCallbacks[name], AddCallback(item, "ResetTimeChanged", function(sender, args) 
                self.normalSkillBar:SetLabelTimer(name, item:GetResetTime() - Turbine.Engine.GetGameTime(), item:GetCooldown())
            end))
        end

        if name == "Catch a Breath" then
            self.skillsLUT[name] = item
           table.insert(self.skillCallbacks[name], AddCallback(item, "ResetTimeChanged", function(sender, args) 
                if item:GetResetTime() - Turbine.Engine.GetGameTime() > item:GetCooldown() - 0.5 then
                    self:HandleBlockRemovalUpdate(1)
                    self:HandleParryRemovalUpdate(1) 
                end 
            end))
        end

        if self.SkillUsageTable[name] then
            local value = self.SkillUsageTable[name]
            if value[1] == 1 then
                self.skillsLUT[name] = item
               table.insert(self.skillCallbacks[name], AddCallback(item, "ResetTimeChanged", function(sender, args) 
                    --ChatOutput("SkillRemoval")
                    if item:GetResetTime() - Turbine.Engine.GetGameTime() > item:GetCooldown() - 0.5 then
                        self:HandleBlockRemovalUpdate(value[2]) 
                    end 
                end))
            elseif value[1] == 2 then
                self.skillsLUT[name] = item
               table.insert(self.skillCallbacks[name], AddCallback(item, "ResetTimeChanged", function(sender, args) 
                    --ChatOutput("SkillRemoval")
                    if item:GetResetTime() - Turbine.Engine.GetGameTime() > item:GetCooldown() - 0.5 then
                        self:HandleParryRemovalUpdate(value[2]) 
                    end 
                end))
            end
        end

        if name == "Smashing Stab" then
            self.skillsLUT[name] = item
           table.insert(self.skillCallbacks[name], AddCallback(item, "ResetTimeChanged", function(sender, args) 
                --ChatOutput("SkillRemoval")
                --ChatOutput(item:GetResetTime())
                if item:GetResetTime() - Turbine.Engine.GetGameTime() > item:GetCooldown() - 0.5 then
                    self:SmashingStabUpdate(0) 
                end 
            end))
        end
    end
    --]]
end

function GuardianTools:LoadData()
    self.player = Turbine.Gameplay.LocalPlayer.GetInstance()
    self.effectList = self.player:GetEffects()
    self.ClassAttr = self.player:GetClassAttributes()
    self.trainedSkills = self.player:GetTrainedSkills()
end

function GuardianTools:RemoveSkillObjects()
    self.ReactionBar:Unload(true)
    self.SmallCDBar:Unload(true)
    self.BigCDBar:Unload(true)
    self.normalSkillBar:Unload(true)
    self.FortLabel2:Unload()

    self.SkillIcons = {}
    self.normalSkills = {}

    self.specialSkillCount = 0
    self.skillCount = 0

    collectgarbage()


end

function GuardianTools:ParseUpdate(message)
end

function GuardianTools:ToggleMove()
    self.moveBox:ToggleMove()
end

function GuardianTools:GetOptions(parent, x, y)
    local container = Turbine.UI.Control()
    container:SetParent(parent)
    container:SetPosition(x, y)
    container:SetSize(530 + 4, 100 + 4)
    container:SetBackColor(Turbine.UI.Color(0, 0, 0))

    local SingleBackground = Turbine.UI.Control()
    SingleBackground:SetParent(container)
    SingleBackground:SetPosition(2, 2)
    SingleBackground:SetSize(530, 100)
    SingleBackground:SetBackColor(Turbine.UI.Color(0.250, 0.250, 0.250))

    local EffectBox2 = Turbine.UI.Lotro.CheckBox()
    EffectBox2:SetParent(SingleBackground)
    EffectBox2:SetPosition(10, 50)
    EffectBox2:SetSize(200,20)
    EffectBox2:SetChecked(Settings.ClassToolsNoSkills)
    EffectBox2:SetText("Disable skill bars")
    EffectBox2:SetVisible(true)

    EffectBox2.CheckedChanged = function()
        Settings.ClassToolsNoSkills = EffectBox2:IsChecked()
    end

    local ScaleLabel = Turbine.UI.Label()
    ScaleLabel:SetParent(SingleBackground)
    ScaleLabel:SetSize(300, 30)
    ScaleLabel:SetTextAlignment(Turbine.UI.ContentAlignment.LeftCenter)
    ScaleLabel:SetPosition(10, 10)
    ScaleLabel:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
    ScaleLabel:SetText("Break Ranks: X-axis priority / Y bar") 

    local BarScaleBar = Turbine.UI.Lotro.TextBox()
    BarScaleBar:SetParent(SingleBackground)
    BarScaleBar:SetSize(50, 30)
    BarScaleBar:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
    BarScaleBar:SetPosition(280, 5)
    BarScaleBar:SetText(Settings.ClassToolsGuardianBreakRankXPrio)

    local BarScaleBar1 = Turbine.UI.Lotro.TextBox()
    BarScaleBar1:SetParent(SingleBackground)
    BarScaleBar1:SetSize(50, 30)
    BarScaleBar1:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
    BarScaleBar1:SetPosition(340, 5)
    BarScaleBar1:SetText(Settings.ClassToolsGuardianBreakRankYBar)

    local Button2 = Turbine.UI.Lotro.Button()
    Button2:SetParent(SingleBackground)
    Button2:SetSize(80, 20)
    Button2:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
    Button2:SetMouseVisible(true)
    Button2:SetPosition(400, 10)
    Button2:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
    Button2:SetText("Save")

    Button2.Click = function()
        Settings.ClassToolsGuardianBreakRankXPrio = tonumber(BarScaleBar:GetText())
        Settings.ClassToolsGuardianBreakRankYBar = tonumber(BarScaleBar1:GetText())
    end

    local EffectBox1 = Turbine.UI.Lotro.CheckBox()
    EffectBox1:SetParent(SingleBackground)
    EffectBox1:SetPosition(10, 30)
    EffectBox1:SetSize(200,20)
    EffectBox1:SetChecked(Settings.ClassToolsGuardianBreakRankLabel)
    EffectBox1:SetText("Break ranks skill box")
    EffectBox1:SetVisible(true)

    EffectBox1.CheckedChanged = function()
        Settings.ClassToolsGuardianBreakRankLabel = EffectBox1:IsChecked()
    end

end

function GuardianTools:GetData()
    return self.data
end

function GuardianTools:Load()
    self:SetSkillObjects()
    self:SetCallbacks()
end

function GuardianTools:Unload()
    self:RemoveCallbacks()
    self:RemoveSkillObjects()
end

function GuardianTools:Reload()
    self:Unload()
    self:Load()
end

function GuardianTools:ChangeData(key, value)
    self.data[key] = value
end

function GuardianTools:GetBuffData()
    return self.buffData
end

function GuardianTools:ChangeBuffData(key, stack, remove, enable)
    self.buffData[key] = {self.buffData[key][1], self.buffData[key][2], stack, remove, enable}
end

function GuardianTools:AddBuffData(key, icon, priority, stack, remove, enable)
    self.buffData[key] = {icon, priority, stack, remove, enable}
end
