HunterTools = class(Turbine.UI.Window)
function HunterTools:Constructor(data, buffData)
    Turbine.UI.Window.Constructor(self)
    self.width = 500
    self.height = 500
    self:SetPosition(Settings.ClassToolsX, Settings.ClassToolsY)
    self:SetSize(self.width, self.height)
    --self:SetBackColor(GetRandomColour())
    self:SetVisible(true)
    self:SetMouseVisible(false)

    self.data = data
    self.buffData = buffData

    self.moveBox = MoveBox(self, self.width, self.height, "Hunter Tools")

    self.buffSkillBar = DynamicSkillBar(self, 0, 0, self.width, 32, 32, 2, true)
    self.normalSkillBar = DynamicSkillBar(self, 0, 57, self.width, 64, 32, 4, false)
    self.stanceSkillBar = DynamicSkillBar(self, self.width / 2 - 16 - 32, 121, 96, 64, 32, 4, false)

    self:LoadData()

    self.noSkills = Settings.ClassToolsNoSkills
    --self.noSkills = true

    self.playerCallbacks = {}
    self.effectCallbacks = {}
    self.attrCallbacks = {}

    self.skillCallbacks = {}
    self.skillsLUT = {}

    self.skillCount = 0 
    self.specialSkillCount = 0
    self.stance = 1

    self.formSlots = {}
    
    self.skillSlots = {}
    self.specialSkillSlots = {}

    self.currentForm = "Man-form"
    self.ferocious = false
    self.slashBleed = true

    self.colour1 = {
        [1] = Turbine.UI.Color(0.8, 0, 0.4, 0),
        [2] = Turbine.UI.Color(0.8, 0, 0.4, 0),
        [3] = Turbine.UI.Color(0.8, 0, 0.4, 0),
        [4] = Turbine.UI.Color(0.8, 0.4, 0.4, 0),
        [5] = Turbine.UI.Color(0.8, 0.4, 0.4, 0),
        [6] = Turbine.UI.Color(0.8, 0.4, 0.4, 0),
        [7] = Turbine.UI.Color(0.8, 0.4, 0, 0),
        [8] = Turbine.UI.Color(0.8, 0.4, 0, 0),
        [9] = Turbine.UI.Color(0.8, 0.4, 0, 0),
    }

    self.colour2 = {
        [1] = Turbine.UI.Color(0.8, 0, 0.8, 0),
        [2] = Turbine.UI.Color(0.8, 0, 0.8, 0),
        [3] = Turbine.UI.Color(0.8, 0, 0.8, 0),
        [4] = Turbine.UI.Color(0.8, 0.8, 0.8, 0),
        [5] = Turbine.UI.Color(0.8, 0.8, 0.8, 0),
        [6] = Turbine.UI.Color(0.8, 0.8, 0.8, 0),
        [7] = Turbine.UI.Color(0.8, 0.8, 0, 0),
        [8] = Turbine.UI.Color(0.8, 0.8, 0, 0),
        [9] = Turbine.UI.Color(0.8, 0.8, 0, 0),
    }

    self.skills = {
        ["Barbed Arrow"] = {1, "0x70003F33", 1},
        ["Improved Quick Shot"] = {2, "0x70029856", 1},
        ["Improved Swift Bow"] = {3, "0x700153E2", 1},
        ["Improved Penetrating Shot"] = {4, "0x7002984F", 1},
        ["Blood Arrow"] = {5, "0x70021D0D", 1},
        ["Exsanguinate"] = {6, "0x700364A9", 1},
        ["Barrage"] = {7, "0x7003646E", 1},
        ["Upshot"] = {8, "0x70035F62", 1},
        ["Lingering Wound"] = {9, "0x70035CC1", 1},
        ["Heart Seeker"] = {10, "0x7000A2D3", 1},
        ["Explosive Arrow"] = {11, "0x70033AEA", 1},
        ["Split Shot"] = {12, "0x700292DC", 1},
        ["Rain of Arrows"] = {13, "0x70003F36", 1},
        ["Rain of Thorns"] = {14, "0x700010F4", 1},
        ["Improved Merciful Shot"] = {15, "0x700153E5", 1},
        ["Blindside"] = {16, "0x70003F3B", 1},

        ["Piercing Trap"] = {18, "0x7000A6B1", 1},
        ["The One Trap"] = {19, "0x70035F61", 1},
        ["Decoy"] = {20, "0x70035CA7", 1},
        ["Tripwire"] = {21, "0x70037DC8", 1},



        ["Improved Focus"] = {1, "0x70029854", 2},
        ["Camouflage"] = {2, "0x70003F48", 2},
        ["Burn Hot"] = {3, "0x7000F835", 2},
        ["Intent Concentration"] = {4, "0x700030DE", 2},
        ["Rapid Fire"] = {5, "0x70036424", 2},
        ["Low Cut"] = {6, "0x70003F39", 2},
        ["Improved Dazing Blow"] = {7, "0x70021D18", 2},
        ["Scourging Blow"] = {8, "0x70003F3A", 2},
        ["Bard's Arrow"] = {9, "0x700010D3", 2},
        ["Distracting Shot"] = {10, "0x7000F9AB", 2},
        ["Purge Poison"] = {11, "0x70003F3E", 2},
        ["Cry of the Hunter"] = {12, "0x7003200C", 2},
        ["Press Onward"] = {13, "0x70010004", 2},
        ["Find the Path"] = {14, "0x70003F3D", 2},


        --["Desperate Flight"] = {1, "0x70003F40", 1},

        --["Pinning Shot"] = {1, "0x70036497", 1},
        --["Strength of the Earth"] = {1, "0x7000A2D4", 1},
        --["Beneath Notice"] = {1, "0x70020DF8", 1},


    }

    self.skillNameTimers = {
        ["Improved Penetrating Shot"] = true,
        ["Imp. Penetrating Shot: Burn Through"] = true,
        ["Imp. Penetrating Shot: Pierce the Darkness"] = true
    }

    if self.data == nil then
        self.data = {}
        for key, value in pairs(self.skills) do
            self.data[key] = {true, value[1], value[3]}
        end
    else
        for key, value in pairs(self.data) do
            --ChatOutput(value[1])
            if value[1] == false then
                self.skills[key] = nil
            else
                self.skills[key] = {value[2], self.skills[key][2], value[3]}
            end
        end
    end

    self.dynamicSkillTable = {
        --["Encouraging Roar"] = {15, 1091940904, 2},
        --1091940904
        ["Improved Penetrating Shot"] = {10, 1090553933, 4},
        ["Barbed Arrow"] = {12, 1090553904, 1},
        ["Exsanguinate"] = {20, 1091804879, 2},
        ["Lingering Wound"] = {10, 1091804872, 3},

    }

    self.dynamicEffectTable = {
        -- Yellow + Base
        --["Recuperate"] = {1091940911, 3},
        ["Improved Focus"] = {1090553915, 10},
        ["Fast Draw - Tier 1"] = {1090532013, 16},
        ["Fast Draw - Tier 2"] = {1090532013, 16},
        ["Fast Draw - Tier 3"] = {1090532013, 16},
        ["Swift and True"] = {1090541188, 17},
        ["Needful Haste"] = {1090541202, 18},
        ["Burn Hot"] = {1091456474, 20},
        ["Strength Stance Bonus"] = {1090553959, 14},
        ["Precision Stance Bonus"] = {1090553958, 14},
        ["Endurance Stance Bonus"] = {1090553960, 14},
        ["Rapid Fire"] = {1091804881, 19},
        ["Volley"] = {1090598919, 15},
        ["Strong Draw"] = {1091798514, 15},
        ["Improved Fleetness"] = {1091463726, 13},

    }

    self.specialEffectTable = {
        ["Fast Draw - Tier 1"] = {1090532013, 16, 1},
        ["Fast Draw - Tier 2"] = {1090532013, 16, 2},
        ["Fast Draw - Tier 3"] = {1090532013, 16, 3},
    }

    self.removeEffectTable = {
        --["Aiding Strike - Tier 1"] = true,
        ["Fast Draw - Tier 1"] = true,
        ["Fast Draw - Tier 2"] = true,
        ["Fast Draw - Tier 3"] = true,
        ["Volley"] = true,
        ["Strong Draw"] = true,
    }

    if self.buffData == nil or tablelength(self.buffData) < 1  then
        self.buffData = {}
        for key, value in pairs(self.dynamicEffectTable) do
            self.buffData[key] = {value[1], value[2], self.specialEffectTable[key] ~= nil, self.removeEffectTable[key] ~= nil, true}
        end
    else
        for key, value in pairs(self.buffData) do
            if value[5] == false then
                self.dynamicEffectTable[key] = nil
                self.specialEffectTable[key] = nil
                self.removeEffectTable[key] = nil
            elseif self.dynamicEffectTable[key] == nil then
                self.dynamicEffectTable[key] = {value[1], value[2]}

                if value[3] then
                    self.specialEffectTable[key] = {value[1], value[2], true}
                end

                if value[4] then
                    self.removeEffectTable[key] = true
                end
            end
        end
    end

    self.stanceSkills = {
        ["Stance: Precision"] = 1,
        ["Stance: Strength"] = 2,
        ["Stance: Endurance"] = 3,
    }
    self:Load()
end

function HunterTools:SetSkillObjects()

    ChatEventHandler:SubscribeTools(self)
    ChatEventHandler:TogglePlayerCombat(true)
    
    if Roles:GetLine() == 3 then
        self.skills["Set Trap"] = {17, "0x700030BB", 1}
    else
        self.skills["Set Trap"] = nil
    end

    local index = 1
    for i = 1, self.trainedSkills:GetCount(), 1 do
        local item = self.trainedSkills:GetItem(i)--:GetSkillInfo()
        local name = item:GetSkillInfo():GetName()


        if self.skills[name] and not(self.noSkills) then
            self.skillSlots[name] = ClassSkillSlot(self.normalSkillBar, self.width / 2 - 16, 0, 32, 32, self.skills[name][2], false)
            self.normalSkillBar:AddLabel(name, self.skillSlots[name], self.skills[name][1], nil, self.skills[name][3])
        end
    end

    if not(self.noSkills) then
        local barHeight = self.normalSkillBar:MaxHeight() * 32 + 57
        self.formSlots[1] = ClassSkillSlot(self, self.width / 2 - 16 - 32, barHeight, 32, 32, "0x70003F4D", false) -- "Stance: Precision"
        self.formSlots[2] = ClassSkillSlot(self, self.width / 2 - 16, barHeight, 32, 32, "0x70003F4E", false) -- "Stance: Strength"
        self.formSlots[3] = ClassSkillSlot(self, self.width / 2 - 16 + 32, barHeight, 32, 32, "0x70003F4F", false) -- "Stance: Endurance"
        self.stanceSkillBar:AddLabel("Stance: Precision", self.formSlots[1], 1)
        self.stanceSkillBar:AddLabel("Stance: Strength", self.formSlots[2], 2)
        self.stanceSkillBar:AddLabel("Stance: Endurance", self.formSlots[3], 3)
        self.stance = self.ClassAttr:GetStance()
    end

    local padding = (self.width - self.skillCount * 32 ) / 2

    local positionCounter = 0
    for key, value in spairs(self.skillSlots, function(t,a,b) return self.skills[b][1] > self.skills[a][1] end) do
        value:SetPosition(padding + positionCounter * 32, 0)
        positionCounter = positionCounter + 1
    end

    local specialPadding = (self.width - self.specialSkillCount * 32 ) / 2

    positionCounter = 0
    for key, value in spairs(self.specialSkillSlots, function(t,a,b) return self.skills[b][1] > self.skills[a][1] end) do
        value:SetPosition(specialPadding + positionCounter * 32, 0)
        positionCounter = positionCounter + 1
    end

    for key, value in pairs(self.dynamicSkillTable) do
        self.buffSkillBar:AddLabel(key, ClassTimerLabel(self.buffSkillBar, 0, 0, 32, 32, value[2], false, false, nil, 1, 1, true, false, true), value[3])
    end

    for key, value in pairs(self.dynamicEffectTable) do
        if not(self.specialEffectTable[key]) then
            self.buffSkillBar:AddLabel(key, ClassTimerLabel(self.buffSkillBar, 0, 0, 32, 32, value[1], false, false, nil, 1, 1, true, false, true), value[2])
        end
    end

    for key, value in pairs(self.specialEffectTable) do
        local stack = value[3]
        if value [3] == true then
            stack = string.match(key, "%d+")
            if stack == nil then stack = 1 end
        end
        self.buffSkillBar:AddLabel(key, StackingClassTimerLabel(self.buffSkillBar, 0, 0, 32, 32, value[1], false, false, nil, 1, 1, true, false, false, stack), value[2])
    end

    local barWidth = self.skillCount * 32
    if barWidth == 0 then barWidth = 320 end

    self.normalSkillBar:SetStaticPositions()

    -- Focus bar
    --ResourceBar:Constructor(parent, width, height, boxes, text, y, borderColour)
    self.focusBar = ResourceBar(self, barWidth, 20, 0, 0, 9, self.colour1, self.colour2, true, 3, 0, nil)
    --self.focusBar:SetParent(self)
    self.focusBar:SetPosition(1 + (self.width - 320) / 2, 32)
    --self.focusBar:SetColours(nil, Turbine.UI.Color(0.6, 0.5, 0, 0), nil, nil)
    --self.focusBar:SetMaxValue(100)
    self.focusBar:ChangeResource(self.ClassAttr:GetFocus())

    -- Power bar
    self.powerBar = VitalBar(barWidth - 1, 4, true, Turbine.UI.ContentAlignment.MiddleCenter, 1)
    self.powerBar:SetParent(self)
    self.powerBar:SetPosition(1 + (self.width - barWidth) / 2, 52)
    self.powerBar:SetColours(nil, Turbine.UI.Color(0.6, 0, 0, 1), nil, nil)
    self.powerBar:SetMaxValue(self.player:GetMaxPower())
    self.powerBar:SetCurrentValue(self.player:GetPower())
end

function HunterTools:SetCallbacks()
    self.effectCallbacks["EffectAdded"] = AddCallback(PlayerEffectsHandler, "EffectAdded", 
    function(effect)
        local effectName = effect:GetName()

        --ChatOutput(effectName .. effect:GetIcon())

        if self.dynamicEffectTable[effectName] then 
            if effectName == "Volley" then
                self.buffSkillBar:ToggleLabel(effectName, true)
            else
                self.buffSkillBar:ToggleLabel(effectName, true, effect:GetDuration(), self.dynamicEffectTable[effectName][1] == true and effect:GetIcon())
            end
        end
    end)
    
    self.effectCallbacks["EffectRemoved"] = AddCallback(PlayerEffectsHandler, "EffectRemoved", 
    function(effect)

        if effect and self.removeEffectTable[effect:GetName()] then
            self.buffSkillBar:ToggleLabel(effect:GetName(), false)
        end

        --[[
        if self.dynamicEffectTable[effect:GetName()] and 
        self.dynamicEffectTable[effect:GetName()][1] == effect:GetIcon() and 
        not(self.exceptionEffects[effectName]) then
            self.buffSkillBar:ToggleLabel(effect:GetName(), false)
        end
        --]]
    end)

    self.attrCallbacks["FocusChanged"] = AddCallback(self.ClassAttr, "FocusChanged", function(sender, args) self.focusBar:ChangeResource( self.ClassAttr:GetFocus() ) end)
    self.playerCallbacks["PowerChanged"] = AddCallback(self.player, "PowerChanged", function(sender, args) self.powerBar:SetCurrentValue( self.player:GetPower() ) end)

    --[[
    self.attrCallbacks = AddCallback(self.ClassAttr, "StanceChanged", function(sender, args) 
        if self.stance > 0 then
            self.formSlots[self.stance]:ToggleVisible(false)
        end
        self.stance = sender:GetStance()
        if self.stance > 0 then
            self.formSlots[self.stance]:ToggleVisible(true)
        end
    end)
    --]]


    -- skill callbacks
    local index = 1
    for i = 1, self.trainedSkills:GetCount(), 1 do
        local item = self.trainedSkills:GetItem(i)--:GetSkillInfo()
        local name = item:GetSkillInfo():GetName()
        --ChatOutput(name .. item:GetSkillInfo():GetIconImageID())

        self.skillCallbacks[name] = {}

        if self.skills[name] and not(self.noSkills) then
            self.skillsLUT[name] = item
            table.insert(self.skillCallbacks[name], AddCallback(item, "ResetTimeChanged", function(sender, args) 
                self.normalSkillBar:SetLabelTimer(name, item:GetResetTime() - Turbine.Engine.GetGameTime(), item:GetCooldown())
            end))
        end

        if self.dynamicSkillTable[name] then
            self.skillsLUT[name] = item
            table.insert(self.skillCallbacks[name], AddCallback(item, "ResetTimeChanged", function(sender, args) 
                self.buffSkillBar:ToggleLabel(name, true, self.dynamicSkillTable[name][1])
            end))
        end
        
        if self.stanceSkills[name] and not(self.noSkills) then
            self.skillsLUT[name] = item
            table.insert(self.skillCallbacks[name], AddCallback(item, "ResetTimeChanged", function(sender, args) 
                if self.stanceSkills[name] ~= self.ClassAttr:GetStance() then
                    self.stanceSkillBar:SetLabelTimer(name, item:GetResetTime() - Turbine.Engine.GetGameTime(), item:GetCooldown()) 
                end
            end))
        end
    end

    --[[
    self.playerCallbacks["InCombatChanged"] = AddCallback(self.player, "InCombatChanged",
    function(sender, args)
        if self.ferocious and not(self.player:IsInCombat()) then
            self.buffSkillBar:ToggleLabel("Ferocious Free Heal", true, 9)
        end
    end)
    --]]
end

function HunterTools:RemoveCallbacks()
    for key, value in pairs(self.effectCallbacks) do
        RemoveCallback(PlayerEffectsHandler, key, value)
    end

    RemoveCallback(self.ClassAttr, "FocusChanged", self.attrCallbacks["FocusChanged"])

    --player callbacks
    for key, value in pairs(self.playerCallbacks) do
        RemoveCallback(self.player, key, value)
    end

    for key, value in pairs(self.skillCallbacks) do
        for _, callback in pairs(value) do
            RemoveCallback(self.skillsLUT[key], "ResetTimeChanged", callback)
        end
    end

    collectgarbage()
end

function HunterTools:RemoveSkillObjects()
    self.buffSkillBar:Unload(true)
    self.normalSkillBar:Unload(true)
    self.stanceSkillBar:Unload(true)
    self.focusBar:SetParent(nil)
    self.powerBar:SetParent(nil)

    ChatEventHandler:SubscribeTools(nil)
    ChatEventHandler:TogglePlayerCombat(false)

    for _, value in pairs(self.formSlots) do
        value:Unload()
    end

    self.formSlots = {}
    self.skillSlots = {}
    self.specialSkillSlots = {}

    self.skillCount = 0
    self.specialSkillCount = 0

    collectgarbage()
end

function HunterTools:ParseUpdate(message)
    local source,_,skillName,target,_ = string.match(message,"^(.*) scored a (.*)hit with (.*) on (.*) for (.-) (.*)%.$")
    if self.skillNameTimers[skillName] then
        self.buffSkillBar:ToggleLabel("Improved Penetrating Shot", true, self.dynamicSkillTable["Improved Penetrating Shot"][1])
    end
end

function HunterTools:ToggleMove()
    self.moveBox:ToggleMove()
end

function HunterTools:GetOptions(parent, x, y)
    local container = Turbine.UI.Control()
    container:SetParent(parent)
    container:SetPosition(x, y)
    container:SetSize(530 + 4, 100 + 4)
    container:SetBackColor(Turbine.UI.Color(0, 0, 0))

    local SingleBackground = Turbine.UI.Control()
    SingleBackground:SetParent(container)
    SingleBackground:SetPosition(2, 2)
    SingleBackground:SetSize(530, 100)
    SingleBackground:SetBackColor(Turbine.UI.Color(0.250, 0.250, 0.250))

    local EffectBox2 = Turbine.UI.Lotro.CheckBox()
    EffectBox2:SetParent(SingleBackground)
    EffectBox2:SetPosition(10, 50)
    EffectBox2:SetSize(200,20)
    EffectBox2:SetChecked(Settings.ClassToolsNoSkills)
    EffectBox2:SetText("Disable skill bars")
    EffectBox2:SetVisible(true)

    EffectBox2.CheckedChanged = function()
        Settings.ClassToolsNoSkills = EffectBox2:IsChecked()
    end

    
end

function HunterTools:GetData()
    return self.data
end

function HunterTools:LoadData()
    self.player = Turbine.Gameplay.LocalPlayer.GetInstance()
    self.effectList = self.player:GetEffects()
    self.ClassAttr = self.player:GetClassAttributes()
    self.trainedSkills = self.player:GetTrainedSkills()
end

function HunterTools:Load()
    self:SetSkillObjects()
    self:SetCallbacks()
end

function HunterTools:Unload()
    self:RemoveCallbacks()
    self:RemoveSkillObjects()
end

function HunterTools:Reload()
    self:Unload()
    self:LoadData()
    self:Load()
end

function HunterTools:ChangeData(key, value)
    self.data[key] = value
end

function HunterTools:GetBuffData()
    return self.buffData
end

function HunterTools:ChangeBuffData(key, stack, remove, enable)
    self.buffData[key] = {self.buffData[key][1], self.buffData[key][2], stack, remove, enable}
end

function HunterTools:AddBuffData(key, icon, priority, stack, remove, enable)
    self.buffData[key] = {icon, priority, stack, remove, enable}
end
