ResourceBar = class( Turbine.UI.Control )
function ResourceBar:Constructor(parent, width, height, x, y, boxes, colourtable1, colourtable2, ShowText, boxOffset, borderSize, borderColour, boxBackgroundColour, fullBorder, animate, fontColour, font, fullMode, fullColour)
    Turbine.UI.Control.Constructor( self )

    self.width = width
    self.height = height
    self.showText = ShowText

    self:SetSize(self.width + borderSize * 2, self.height + borderSize * 2)
    self:SetZOrder(100)
    self:SetMouseVisible(false)
    self:SetBackColor(borderColour)
    self:SetParent(parent)
    self:SetPosition(x, y)

    if not(fontColour) or fontColour.A == 0 then
        self.fontColour = Turbine.UI.Color( 1, 1, 1, 1 )
    else
        self.fontColour = fontColour
    end
    self.font = font or Turbine.UI.Lotro.Font.TrajanProBold24

    self.bar = Turbine.UI.Control()
    self.bar:SetParent(self)
    self.bar:SetPosition(borderSize, borderSize)
    self.bar:SetSize(width, height)
    self.bar:SetMouseVisible(false)

    self.adjustment = fullBorder and 1 or 0
    self.animate = animate
    self.fullMode = fullMode
    self.fullColour = fullColour
    self.boxBackgroundColour = boxBackgroundColour

    self.boxes = boxes

    self.BarDictionary = {}
    self.BarBackgroundDictionary = {}

    if colourtable1 and type(colourtable1[1]) == "table" then
        if self.boxes > tablelength(colourtable1) then
            self.colourTable = {}
            for i=1, self.boxes, 1 do
                self.colourTable[i] = colourtable1[1]
            end
        else
            self.colourTable = colourtable1
        end
    else
        self.colourTable = {}
        for i=1, self.boxes, 1 do
            self.colourTable[i] = colourtable1
        end
    end

    if colourtable2 and type(colourtable2[1]) == "table" then
        if self.boxes > tablelength(colourtable2) then
            self.colourTable2 = {}
            for i=1, self.boxes, 1 do
                self.colourTable2[i] = colourtable2[1]
            end
        else
            self.colourTable2 = colourtable2
        end
    else
        self.colourTable2 = {}
        for i=1, self.boxes, 1 do
            self.colourTable2[i] = colourtable2
        end
    end

    local boxWidth = math.ceil((width - boxOffset) / (boxes))

    for i = 1, self.boxes, 1 do
        if boxBackgroundColour and boxBackgroundColour.A > 0 then
            self.BarBackgroundDictionary[i] = ResourceBox(self.bar, boxOffset / 2 + math.floor(boxOffset / 2) + (i - 1) * boxWidth, boxWidth - boxOffset, self.height, boxBackgroundColour, boxBackgroundColour, 0, boxBackgroundColour)
            self.BarBackgroundDictionary[i]:SetVisible(true)   
        end
        self.BarDictionary[i] = ResourceBox(self.bar, boxOffset / 2 + math.floor(boxOffset / 2) + (i - 1) * boxWidth, boxWidth - boxOffset, self.height, self.colourTable[i], self.colourTable2[i], self.adjustment, boxBackgroundColour, self.animate)
    end  

    self.textBoxViewWindow = Turbine.UI.Window()
    self.textBoxViewWindow:SetParent(self.bar)
    self.textBoxViewWindow:SetSize(20, 20)
    self.textBoxViewWindow:SetPosition((self.width) / 2 - 7 , (height - 23) / 2)
    self.textBoxViewWindow:SetVisible(true)
    self.textBoxViewWindow:SetMouseVisible(false)

    self.textBox = Turbine.UI.Label()
    self.textBox:SetParent(self.textBoxViewWindow)
    self.textBox:SetSize(20, 20)
    self.textBox:SetPosition(0, 0)
    self.textBox:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
    self.textBox:SetOutlineColor(Turbine.UI.Color.Black )
    self.textBox:SetFontStyle( Turbine.UI.FontStyle.Outline )
    self.textBox:SetFont(self.font)
    self.textBox:SetForeColor(self.fontColour)
    self.textBox:SetText("0")
    --self.textBox:SetBackColor(Turbine.UI.Color(0, 0, 0))
    self.textBox:SetMouseVisible(false)
    self.textBox:SetVisible(false)
end

function ResourceBar:ChangeResource(NewNumber, Colour1, Colour2)
    self.textBox:SetText(NewNumber)
    self.textBox:SetVisible(NewNumber > 0 and self.showText)
    for i = 1, self.boxes, 1 do
        if i <= NewNumber then
            self.BarDictionary[i]:SetVisible(true)
            if self.fullMode then
                if NewNumber == self.boxes then
                    self.BarDictionary[i]:TurnOn(self.boxBackgroundColour, self.fullColour)
                 else
                    self.BarDictionary[i]:TurnOn( self.colourTable[i], self.colourTable2[i])
                 end
             end
             if Colour1 and Colour2 then
                self.BarDictionary[i]:TurnOn(Colour1, Colour2)
            end
        else
            self.BarDictionary[i]:SetVisible(false)
        end
    end  
end

function ResourceBar:ChangeRKResource(NewNumber, Colour1, Colour2)
    self.textBox:SetText(NewNumber)
    self.textBox:SetVisible(NewNumber > 0 and self.showText)
    for i = 1, self.boxes, 1 do
        if i <= NewNumber then
            self.BarDictionary[i]:SetVisible(true)
            if Colour1 and Colour2 then
                self.BarDictionary[i]:TurnOn( Colour1, Colour2)
            end
            if self.fullMode then
               if NewNumber == self.boxes then
                    self.BarDictionary[i]:TurnOn(self.boxBackgroundColour, self.fullColour)
                else
                    self.BarDictionary[i]:TurnOn( self.colourTable[i], self.colourTable2[i])
                end
            end
        else
            self.BarDictionary[i]:SetVisible(false)
        end
    end  
end

function ResourceBar:Unload()
    self:SetVisible(false)
    self:SetParent(nil)
end

ResourceBox = class( Turbine.UI.Control )
function ResourceBox:Constructor(parent, x, width, height, colour, colour2, adjustment, parentBackground, animate)

    Turbine.UI.Control.Constructor( self )
    self:SetParent(parent)
    self:SetBackColor(colour2)
    --self:SetBackground("PrimePlugins/Vitals/Images/FittedSmooth.tga")
    self:SetPosition(x, 0)
    self:SetSize(width, height)
    self:SetMouseVisible(false)
    self:SetVisible(false)

    self.updateTicks = 5
    self.animate = animate

    self.backgroundColour = colour
    self.foregroundColour = colour2
    self.currentForegroundColour = Turbine.UI.Color(0,0,0,0)
    self.parentBackground = parentBackground

    self.InnerBarBackground = Turbine.UI.Control()
    self.InnerBarBackground:SetParent(self)
    self.InnerBarBackground:SetBackColor(Turbine.UI.Color(0.250,0.250,0.250))
    --self.InnerBarBackground:SetBackColorBlendMode(Turbine.UI.BlendMode.Overlay)
    self.InnerBarBackground:SetPosition(1, 1)
    self.InnerBarBackground:SetSize(width - 1 - adjustment, height - 1 - adjustment)
    self.InnerBarBackground:SetMouseVisible(false)
    self.InnerBarBackground:SetBackColor(colour)

    self.InnerBar = Turbine.UI.Control()
    self.InnerBar:SetParent(self)
    --self.InnerBar:SetBackColor(Turbine.UI.Color(1,0.250,0.250))
    self.InnerBar:SetPosition(1, 1)
    self.InnerBar:SetSize(width - 2, height - 2)
    self.InnerBar:SetMouseVisible(false)

    self.colourUpdates = 0
    self.lastUpdate = 0

    self.ExecuteUpdate = function()
        if Turbine.Engine.GetGameTime() - self.lastUpdate > 0.03 then
            self.currentBackgroundColour = Turbine.UI.Color(self.currentBackgroundColour.A + self.backgroundColourA, 
                                                            self.currentBackgroundColour.R + self.backgroundColourR,
                                                            self.currentBackgroundColour.G + self.backgroundColourG,
                                                            self.currentBackgroundColour.B + self.backgroundColourB)
            self:SetBackColor(self.currentBackgroundColour)

            self.currentForegroundColour = Turbine.UI.Color(self.currentForegroundColour.A + self.foregroundColourA, 
                                                            self.currentForegroundColour.R + self.foregroundColourR,
                                                            self.currentForegroundColour.G + self.foregroundColourG,
                                                            self.currentForegroundColour.B + self.foregroundColourB)
        
            self.InnerBarBackground:SetBackColor(self.currentForegroundColour)

            self.colourUpdates = self.colourUpdates + 1

            if self.colourUpdates >= self.updateTicks then
                self:SetBackColor(self.backgroundColour)
                self.InnerBarBackground:SetBackColor(self.foregroundColour)
                self.foregroundColour = self.currentForegroundColour
                self.backgroundColour = self.currentBackgroundColour
                if self.updateCallback then RemoveCallback(Updater, "ExecuteUpdate", self.updateCallback) end
                self.updateCallback = nil
            end
            self.lastUpdate = Turbine.Engine.GetGameTime()
        end
    end


    self.VisibleChanged = function()
        if self:IsVisible() then
            self:TurnOn(self.backgroundColour, self.foregroundColour)
        else
            if self.updateCallback then RemoveCallback(Updater, "ExecuteUpdate", self.updateCallback) end
            self.updateCallback = nil
        end
    end
end

function ResourceBox:TurnOn(Colour1, Colour2)
    if self.animate then
        if self.currentBackgroundColour == Colour1 and self.currentForegroundColour == Colour2 then return end

        if self:IsVisible() then
            self.backgroundColour = Colour1
            self.foregroundColour = Colour2
            self:SetBackColor(self.backgroundColour)
            self.InnerBarBackground:SetBackColor(self.foregroundColour)
            return
        end
        self.backgroundColour = Colour1
        self.foregroundColour = Colour2

        self.currentBackgroundColour = self.parentBackground
        self.currentForegroundColour = self.parentBackground                                                

        self.colourUpdates = 0

        self:SetBackColor(self.currentBackgroundColour)
        self.InnerBarBackground:SetBackColor(self.currentForegroundColour)

        self.backgroundColourA = (self.backgroundColour.A - self.parentBackground.A) / self.updateTicks
        self.backgroundColourR = (self.backgroundColour.R - self.parentBackground.R) / self.updateTicks
        self.backgroundColourG = (self.backgroundColour.G - self.parentBackground.G) / self.updateTicks
        self.backgroundColourB = (self.backgroundColour.B - self.parentBackground.B) / self.updateTicks

        self.foregroundColourA = (self.foregroundColour.A - self.parentBackground.A) / self.updateTicks
        self.foregroundColourR = (self.foregroundColour.R - self.parentBackground.R) / self.updateTicks
        self.foregroundColourG = (self.foregroundColour.G - self.parentBackground.G) / self.updateTicks
        self.foregroundColourB = (self.foregroundColour.B - self.parentBackground.B) / self.updateTicks

        if not(self.updateCallback) then self.updateCallback = AddCallback(Updater, "ExecuteUpdate", self.ExecuteUpdate) end
    else
        self:SetBackColor(Colour1)
        self.InnerBarBackground:SetBackColor(Colour2)
    end
end

function ResourceBox:TurnOff()
    self.InnerBar:SetBackColor()
end