
StackingClassTimerLabel = class(Turbine.UI.Window)
function StackingClassTimerLabel:Constructor(parent, x, y, base, normal, baseIcon, visible, reverse, duration, normalOpacity, usageOpacity, hide, shade, text, stack)
    Turbine.UI.Window.Constructor(self)

    self.base = base
    self.normal = normal
    self.reverse = reverse
    self.normalOpacity = normalOpacity
    self.usageOpacity = usageOpacity
    self.hide = hide
    self.shade = shade
    self.text = text

    self:SetParent(parent)
    self:SetPosition(x, y)
    self:SetSize(self.normal, self.normal)
    self:SetVisible(visible)
    self:SetOpacity(normalOpacity)

    self.currentNumber = -1
    self.currentPercentage = 0

    self.baseImageID = 0x41007E35

    self.IconLabel = Turbine.UI.Label()
    self.IconLabel:SetParent(self)
    self.IconLabel:SetSize(self.base, self.base)
    self.IconLabel:SetStretchMode(1)
    self.IconLabel:SetBackground(BackgroundHandler:GetBackground(baseIcon))
    self.IconLabel:SetSize(self.normal, self.normal)

    self.stackLabel = Turbine.UI.Label()
    self.stackLabel:SetParent(self)
    self.stackLabel:SetSize(self.base - 3, self.base)
    self.stackLabel:SetTextAlignment(Turbine.UI.ContentAlignment.TopRight)
    --self.stackLabel:SetFont(Turbine.UI.Lotro.Font.TrajanProBold16)
    self.stackLabel:SetFont(Turbine.UI.Lotro.Font.TrajanProBold16)
    self.stackLabel:SetOutlineColor(Turbine.UI.Color(0,0,0))
    self.stackLabel:SetFontStyle(Turbine.UI.FontStyle.Outline)
    self.stackLabel:SetMouseVisible(false)
    self.stackLabel:SetStretchMode(1)
    self.stackLabel:SetText(stack)

    if self.shade then
        self.elapsedOverlay = Turbine.UI.Control()
        self.elapsedOverlay:SetStretchMode( 1 )
        self.elapsedOverlay:SetMouseVisible( false )
        --self.elapsedOverlay:SetZOrder( 99 )
        self.elapsedOverlay:SetBackColor( Turbine.UI.Color( 1, 0, 0, 0 ) )
        self.elapsedOverlay:SetBackColorBlendMode( Turbine.UI.BlendMode.Overlay )
        self.elapsedOverlay:SetParent( self )
        self.elapsedOverlay:SetPosition(-1, -1)
        self.elapsedOverlay:SetSize(self.normal + 2, self.normal + 2)
    end
    
    --if self.text then
        self.textLabel = Turbine.UI.Label()
        self.textLabel:SetParent(self)
        self.textLabel:SetSize(self.normal, self.normal)
        self.textLabel:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
        --self.textLabel:SetFont(Turbine.UI.Lotro.Font.TrajanProBold16)
        self.textLabel:SetFont(Turbine.UI.Lotro.Font.FixedSys15)
        self.textLabel:SetOutlineColor(Turbine.UI.Color(0,0,0))
        self.textLabel:SetFontStyle(Turbine.UI.FontStyle.Outline)
        self.textLabel:SetMouseVisible(false)
    --end

    -- Add timer overlay

    self.startTime = Turbine.Engine.GetGameTime()
end

function StackingClassTimerLabel:SetTimer(timer, maxTimer)
    if timer > 99999 then self.textLabel:SetVisible(false) else self.textLabel:SetVisible(true) end
    if maxTimer then
        self.overlaydDuration = maxTimer
        self.startTime = Turbine.Engine.GetGameTime() - (maxTimer - timer)
    else
        self.overlaydDuration = timer
        self.startTime = Turbine.Engine.GetGameTime()
    end
    self.duration = timer
    self:SetOpacity(self.usageOpacity)

    --Updater:Subscribe(self)
end

function StackingClassTimerLabel:ExecuteUpdate(delta)

    self.duration = self.duration - delta
    if self.duration < 0 then 
        --Updater:UnSubscribe(self)
        self:SetOpacity(self.normalOpacity)
        if self.hide then self:SetVisible(false) end 

        if self.text then
            self.textLabel:SetText("")
        end
        return true
    end

    if self.shade then

        local percentComplete = math.floor(60 * (Turbine.Engine.GetGameTime() - self.startTime) / self.overlaydDuration)
        if self.reverse then
            percentComplete = 60 - percentComplete
        end

        if self.currentPercentage ~= percentComplete then 

            self.currentPercentage = percentComplete

            local imageId = self.baseImageID + self.currentPercentage            

            self.elapsedOverlay:SetStretchMode(3)
            self.elapsedOverlay:SetSize(32, 32)
            self.elapsedOverlay:SetBackground(BackgroundHandler:GetBackground(imageId))
            self.elapsedOverlay:SetStretchMode(1)
            self.elapsedOverlay:SetSize(self.normal + 2, self.normal + 2)
        end
    end

    local newNumber = math.round(self.duration)
    if newNumber == self.currentNumber then return end

    self.currentNumber = newNumber

    if self.currentNumber < 99999 then
        self.textLabel:SetStretchMode(nil)
        self.textLabel:SetStretchMode(1)
        self.textLabel:SetText(self.currentNumber)
    end
end

function StackingClassTimerLabel:ToggleVisible(value)
    self:SetVisible(value)
    self.IconLabel:SetVisible(value)
end

function StackingClassTimerLabel:Unload()
    self.IconLabel:SetParent(nil)
    if self.textLabel then self.textLabel:SetParent(nil) end
    if self.elapsedOverlay then self.elapsedOverlay:SetParent(nil) end
    self:SetParent(nil)
    self = nil
end

function StackingClassTimerLabel:AddBackground(icon)
    self.IconLabel:SetStretchMode(nil)
    self.IconLabel:SetSize(self.base, self.base)
    self.IconLabel:SetStretchMode(1)
    self.IconLabel:SetBackground(BackgroundHandler:GetBackground(icon))
    self.IconLabel:SetSize(self.normal, self.normal)
end