WardenTools = class(Turbine.UI.Window)
function WardenTools:Constructor(data, buffData)
    Turbine.UI.Window.Constructor(self)
    self.width = 500
    self.height = 500
    self:SetPosition(Settings.ClassToolsX, Settings.ClassToolsY)
    self:SetSize(self.width, self.height)
    --self:SetBackColor(GetRandomColour())
    self:SetVisible(true)
    self:SetMouseVisible(false)

    self.data = data
    self.buffData = buffData

    self.moveBox = MoveBox(self, self.width, self.height, "Warden Tools")

    self.buffSkillBar = DynamicSkillBar(self, 0, 0, self.width, 32, 32, 2, true)
    self.bleedBar = DynamicSkillBar(self, 0, 32, self.width, 32, 32, 4, true)
    self.normalSkillBar = DynamicSkillBar(self, 0, 64, self.width, 64, 32, 4, false)

    self:LoadData()

    self.noSkills = Settings.ClassToolsNoSkills
    --self.noSkills = false

    self.timeSinceLastUpdate = 0
    self.updateDelay = 0.3

    self.playerCallbacks = {}
    self.effectCallbacks = {}
    self.attrCallbacks = {}

    self.skillCallbacks = {}
    self.skillsLUT = {}

    self.skillCount = 0 
    self.specialSkillCount = 0
    self.stance = 1

    self.formSlots = {}
    
    self.skillSlots = {}
    self.specialSkillSlots = {} 

    self.qsa = Settings.ClassToolsWardenQS

    self.skills = {
        ["Quick Thrust"] = {1, "0x7000E867", 5},
        ["Shield-bash"] = {2, " 0x7000ED0E", 5},
        ["Warden's Taunt"] = {3, "0x7000EECF", 5},
        ["Gambit Default"] = {4, " 0x70015F56", 5},

        ["Spear and Spear"] = {1, "0x7002850A", 2},
        ["Spear and Shield"] = {3, "0x70015099", 2},
        ["Spear and Fist"] = {2, "0x70015098", 2},

        ["Shield and Shield"] = {1, "0x7002850B", 4},
        ["Shield and Spear"] = {3, "0x7001509D", 4},
        ["Shield and Fist"] = {2, "0x7001509B", 4},

        ["Fist and Fist"] = {1, "0x7002850C", 3},
        ["Fist and Spear"] = {3, "0x7001509A", 3},
        ["Fist and Shield"] = {2, "0x7001509C", 3},

        -- [[
        ["Recovery"] = {5, "0x70016A0A", 7},
        ["Quick Recovery"] = {6, "0x70016A09", 7},

        ["Marked Target"] = {16, "0x70036550", 7},

        ["Hampering Javelin"] = {18, "0x70014833", 7},
        ["Ambush"] = {19, "0x7000EEC4", 7},
        ["Diminished Target"] = {20, "0x70036551", 7},
        ["Wages of Fear"] = {21, "0x70014831", 7},

        ["First Aid"] = {5, "0x7003A654", 7},
        
        ["Steadfast"] = {7, "0x7003A395", 7},
        ["Battle Preparation"] = {8, "0x7002BEEC", 7},
        ["Never Surrender"] = {9, "0x7003B619", 7},
        ["Battle Memory I"] = {10, "0x70028906", 7},

        ["Shield-slam"] = {1, "0x7004E1FF", 6},
        ["In the Fray - Stance"] = {2, "0x7003639B", 6},
        ["Assailment - Stance"] = {3, "0x7003639C", 6},
        ["Recklessness"] = {4, "0x700363A8", 6},
        
        --]]

    }

    self.skillNameTimers = {
    }

    if self.data == nil then
        self.data = {}
        for key, value in pairs(self.skills) do
            self.data[key] = {true, value[1], value[3]}
        end
    else
        for key, value in pairs(self.data) do
            if value[1] == false then
                self.skills[key] = nil
            else
                self.skills[key] = {value[2], self.skills[key][2], value[3]}
            end
        end
    end

    self.dynamicSkillTable = {
        --["Encouraging Roar"] = {15, 1091940904, 2},
        --109194090
    }

    self.dynamicEffectTable = {
        -- Yellow + Base
        --["Recuperate"] = {1091940911, 3},
        ["Preparing for Battle"] = {1091692906, 100},
        ["Reckless"] = {1091659503, 15},

        -- Yellow
        ["Seize the Moment"] = {1091916302, 1},
        ["Momentum"] = {1091830134, 2},
        ["Adroit Manoeuvre"] = {1091478097, 16},
        ["Adroit Manoeuvre - Induction Reduction"] = {1091478097, 17},
        ["Warden's Triumph- Ranged"] = {1091478187, 19},
        ["Warden's Triumph"] = {1091478187, 18},

        --Blue
        --
        ["Tier 1 Heal over Time"] = {1091469960, 40},
        ["Tier 2 Heal over Time"] = {1091471259, 41},
        ["Tier 3 Heal over Time"] = {1091471258, 42},
        ["Tier 4 Heal over Time"] = {1091471247, 43},

        ["Enduring Shieldwork"] = {1091469964, 50},
        ["Enduring Advanced Shieldwork"] = {1091471259, 51},
        ["Enduring Expert Shieldwork"] = {1091471258, 52},

        ["Defiance"] = {1091830177, 60},
        ["Conviction"] = {1091478183, 61},

        ["Enduring Armour Use"] = {1091469960, 70},
        ["Enduring Advanced Armour Use"] = {1091471254, 71},
        ["Enduring Expert Armour Use"] = {1091471266, 72},
        ["Enduring Tactical Armour Use"] = {1091478183, 73},

        ["Shield Mastery"] = {1091471248, 30},
        ["Tactically Sound"] = {1090541176, 20},
        ["Shield Tactics"] = {1091659503, 31},
        ["Darkest Before Dawn"] = {1091719861, 21},

        ["For the Free Peoples"] = {1091830171, 80},

        ["Fellowship Protector - Tier 1"] = {1091916300, 90, 1},
        ["Fellowship Protector - Tier 2"] = {1091916300, 90, 2},
        ["Fellowship Protector - Tier 3"] = {1091916300, 90, 3},
        ["Fellowship Protector - Tier 4"] = {1091916300, 90, 4},
        ["Fellowship Protector - Tier 5"] = {1091916300, 90, 5},
        ["Fellowship Protector - Tier 6"] = {1091916300, 90, 6},
        ["Fellowship Protector - Tier 7"] = {1091916300, 90, 7},
        ["Fellowship Protector - Tier 8"] = {1091916300, 90, 8},
        ["Fellowship Protector - Tier 9"] = {1091916300, 90, 9},
        ["Fellowship Protector - Tier 10"] = {1091916300, 90, 10},

    }

    self.specialEffectTable = {
        ["Fellowship Protector - Tier 1"] = {1091916300, 90, 1},
        ["Fellowship Protector - Tier 2"] = {1091916300, 90, 2},
        ["Fellowship Protector - Tier 3"] = {1091916300, 90, 3},
        ["Fellowship Protector - Tier 4"] = {1091916300, 90, 4},
        ["Fellowship Protector - Tier 5"] = {1091916300, 90, 5},
        ["Fellowship Protector - Tier 6"] = {1091916300, 90, 6},
        ["Fellowship Protector - Tier 7"] = {1091916300, 90, 7},
        ["Fellowship Protector - Tier 8"] = {1091916300, 90, 8},
        ["Fellowship Protector - Tier 9"] = {1091916300, 90, 9},
        ["Fellowship Protector - Tier 10"] = {1091916300, 90, 10},
    }

    self.removeEffectTable = {
        --["Aiding Strike - Tier 1"] = true,
        ["Preparing for Battle"] = true,
        ["Fellowship Protector - Tier 1"] = true,
        ["Fellowship Protector - Tier 2"] = true,
        ["Fellowship Protector - Tier 3"] = true,
        ["Fellowship Protector - Tier 4"] = true,
        ["Fellowship Protector - Tier 5"] = true,
        ["Fellowship Protector - Tier 6"] = true,
        ["Fellowship Protector - Tier 7"] = true,
        ["Fellowship Protector - Tier 8"] = true,
        ["Fellowship Protector - Tier 9"] = true,
        ["Fellowship Protector - Tier 10"] = true,
    }

    if self.buffData == nil or tablelength(self.buffData) < 1  then
        self.buffData = {}
        for key, value in pairs(self.dynamicEffectTable) do
            self.buffData[key] = {value[1], value[2], self.specialEffectTable[key] ~= nil, self.removeEffectTable[key] ~= nil, true}
        end
    else
        for key, value in pairs(self.buffData) do
            if value[5] == false then
                self.dynamicEffectTable[key] = nil
                self.specialEffectTable[key] = nil
                self.removeEffectTable[key] = nil
            elseif self.dynamicEffectTable[key] == nil then
                self.dynamicEffectTable[key] = {value[1], value[2]}

                if value[3] then
                    self.specialEffectTable[key] = {value[1], value[2], true}
                end

                if value[4] then
                    self.removeEffectTable[key] = true
                end
            end
        end
    end



    self.stanceSkills = {
        ["Assailment - Stance"] = 1,
        ["In the Fray - Stance"] = 2,
    }

    self.stanceBarSkills = {
        ["Assailment - Stance"] = true,
        ["In the Fray - Stance"] = true,
        ["Recklessness"] = true,
        ["Shield-slam"] = true,
    }

    -- Small QS 0x41140177
    -- Medium QS 0x4114017E
    -- War cry 0x410E8782
    -- Brink 0x410E8793
    -- Surety 0x410E8785
    -- Desolation 0x410EA2AA
    -- Precise blow 0x410E879E
    -- Piercing strike 0x410E878D
    -- Spear of Virtue 0x410E878C
    -- Power Attack 0x410E8798 LB
    -- Mighty Blow 0x410E879F MB
    -- Unerring Strike 0x41104562
    -- Resounding Challenge  0x4112B441

    -- Ranged Power Attack 0x41123894
    -- Ranged Mighty blow 0x41123899
    -- Ranged Unerring Strike 0x4111BF69

    -- Battle prep 0x4111E96A
    -- Never Surrender 0x4110BE84
    -- Never Surrender Effect Icon 0x4111BF69

    -- Melee damage buff 0x4111BF69


    self.bleedSkills = {
        ["Power Attack"] = {1, 0x410E8798},
        ["Mighty Blow"] = {2, 0x410E879F},
        ["Unerring Strike"] = {3, 0x41104562},
        ["Low Bleed - Quick Sweep"] = {4, 0x41140177},
        ["Medium Bleed - Quick Sweep"] = {5, 0x4114017E},
        ["Precise Blow"] = {6, 0x410E879E},
        ["Piercing Strike"] = {7, 0x410E878D},
        ["Spear of Virtue"] = {8, 0x410E878C},
        ["War-cry"] = {9, 0x410E8782},
        ["Brink of Victory"] = {10, 0x410E8793},
        ["Surety of Death"] = {11, 0x410E8785},
        ["Desolation"] = {12, 0x410EA2AA},
        ["Ranged Resounding Challenge"] = {13, 0x4112B441},
        ["Ranged Unerring Strike"] = {14, 0x4112389E}
    }

    self.bleedDuration = {
        ["Power Attack"] = 24,
        ["Mighty Blow"] = 24,
        ["Unerring Strike"] = 24,
        ["Low Bleed - Quick Sweep"] = 20,
        ["Medium Bleed - Quick Sweep"] = 20,
        ["Precise Blow"] = 24,
        ["Piercing Strike"] = 24,
        ["Spear of Virtue"] = 24,
        ["War-cry"] = 24,
        ["Brink of Victory"] = 24,
        ["Surety of Death"] = 24,
        ["Desolation"] = 24,
        ["Ranged Resounding Challenge"] = 24,
        ["Ranged Unerring Strike"] = 28,
    }

    if Settings.ClassToolsWardenExtraTick then
        for key, value in pairs(self.bleedDuration) do
            self.bleedDuration[key] = value + 4
        end
    end

    local bleedtablecounter = 0
    for key, value in pairs(Settings.ClassToolsWardenBleeds) do
        if self.bleedSkills[key] and value == false then
            self.bleedSkills[key] = nil
        end
    end

    if bleedtablecounter == 0 then
        for key, value in pairs(self.bleedSkills) do
            Settings.ClassToolsWardenBleeds[key] = true
        end
    end

    self.specialBleedSkills = {
        ["Low Bleed - Quick Sweep"] = 0,
        ["Medium Bleed - Quick Sweep"] = 0,
        ["Power Attack Bleed"] = 0,
        ["Mighty Blow Bleed"] = 0,
    }

    self.initialToBleed = {
        ["Low Bleed - Quick Sweep"] = "Low Bleed - Quick Sweep",
        ["Medium Bleed - Quick Sweep"] = "Low Bleed - Quick Sweep",
        ["Power Attack"] = "Power Attack Bleed",
        ["Mighty Blow"] = "Mighty Blow Bleed",
    }

    self.bleedToInitial = {
        ["Low Bleed - Quick Sweep"] = "Low Bleed - Quick Sweep",
        ["Medium Bleed - Quick Sweep"] = "Low Bleed - Quick Sweep",
        ["Power Attack Bleed"] = "Power Attack",
        ["Mighty Blow Bleed"] = "Mighty Blow",
    }

    self.maxBleedInterval = {
        ["Low Bleed - Quick Sweep"] = 4.2,
        ["Medium Bleed - Quick Sweep"] = 4.2,
        ["Power Attack Bleed"] = 8.2,
        ["Mighty Blow Bleed"] = 8.2,
    }

    self.qsaTable = {
        ["Low Bleed - Quick Sweep"] = true,
        ["Medium Bleed - Quick Sweep"] = true,
        ["Marked Target"] = true,
        ["Diminished Target"] = true,
    }

    if self.qsa then
        for key, value in pairs(self.specialBleedSkills) do
            if string.find(key, "Quick Sweep") then self.specialBleedSkills[key] = nil end
        end

        for key, value in pairs(self.initialToBleed) do
            if string.find(key, "Quick Sweep") then self.initialToBleed[key] = nil end
        end

        for key, value in pairs(self.bleedToInitial) do
            if string.find(key, "Quick Sweep") then self.bleedToInitial[key] = nil end
        end

        for key, value in pairs(self.maxBleedInterval) do
            if string.find(key, "Quick Sweep") then self.maxBleedInterval[key] = nil end
        end

        self.bleedSkills["Marked Target"] = {15, 1091804897}
        self.bleedSkills["Diminished Target"] = {16, 1091806157}
        
    end

    self.specialBleedsActive = {}

    self.bleedSlots = {}

    self.ExecuteUpdate = function(delta)

        self.timeSinceLastUpdate = self.timeSinceLastUpdate + delta
        if self.timeSinceLastUpdate < self.updateDelay then return end
    
        self.timeSinceLastUpdate = 0
        
    
        for key, value in pairs(self.specialBleedsActive) do 
            if self.specialBleedSkills[key] + self.maxBleedInterval[key] < Turbine.Engine.GetGameTime() then
                self.specialBleedsActive[key] = nil
                self.bleedBar:ToggleLabel(self.bleedToInitial[key], nil)
            end
        end
    end

    self:Load()

    self:TargetEvents(self.player)
end

function WardenTools:SetSkillObjects()

    ChatEventHandler:SubscribeTools(self)
    ChatEventHandler:TogglePlayerCombat(true)

    local bleedSkills = 0
    for key, value in pairs(self.bleedSkills) do
        --(parent, x, y, base, normal, baseIcon, visible, reverse, duration, normalOpacity, usageOpacity, hide, shade, text)
        self.bleedSlots[key] = BleedTimerLabel(self.bleedBar, 0, 0, 32, 32, value[2], true, false, nil, 1, 1, false, false, true, true)
        self.bleedBar:AddLabel(key, self.bleedSlots[key])
        bleedSkills = bleedSkills + 1
    end

    local bleedPosition = 0
    local bleedPadding = self.width / 2 - bleedSkills * 16
    for key, value in spairs(self.bleedSlots, function(t,a,b) return self.bleedSkills[b][1] > self.bleedSkills[a][1] end) do
        value:SetPosition(bleedPadding + bleedPosition * 32, 0)
        bleedPosition = bleedPosition + 1
    end

    if self.qsa then
        for key, value in pairs(self.bleedSkills) do
            if string.find(key, "Quick Sweep") then self.bleedSkills[key] = nil end
        end
        self.bleedSkills["Marked Target"] = nil
        self.bleedSkills["Diminished Target"] = nil
    end
    

    local index = 1
    for i = 1, self.trainedSkills:GetCount(), 1 do
        local item = self.trainedSkills:GetItem(i)--:GetSkillInfo()
        local name = item:GetSkillInfo():GetName()


        if self.skills[name] and not(self.noSkills) then
            self.skillSlots[name] = ClassSkillSlot(self.normalSkillBar, self.width / 2 - 16, 0, 32, 32, self.skills[name][2], false)
            self.normalSkillBar:AddLabel(name, self.skillSlots[name], self.skills[name][1], nil, self.skills[name][3])
        end
    end

    self.normalSkillBar:SetStaticPositions()

    for key, value in pairs(self.dynamicSkillTable) do
        self.buffSkillBar:AddLabel(key, ClassTimerLabel(self.buffSkillBar, 0, 0, 32, 32, value[2], false, false, nil, 1, 1, true, false, true), value[3])
    end

    for key, value in pairs(self.dynamicEffectTable) do
        if not(self.specialEffectTable[key]) then
            self.buffSkillBar:AddLabel(key, ClassTimerLabel(self.buffSkillBar, 0, 0, 32, 32, value[1], false, false, nil, 1, 1, true, false, true), value[2])
        end
    end
    
    for key, value in pairs(self.specialEffectTable) do
        local stack = value[3]
        if value [3] == true then
            stack = string.match(key, "%d+")
            if stack == nil then stack = 1 end
        end
        self.buffSkillBar:AddLabel(key, StackingClassTimerLabel(self.buffSkillBar, 0, 0, 32, 32, value[1], false, false, nil, 1, 1, true, false, false, stack), value[2])
    end


    local barWidth = self.skillCount * 32
    if barWidth == 0 then barWidth = 320 end

end

function WardenTools:QSCheck(effect)
    if self.qsaTable[effect:GetName()] then
        local effectTime = effect:GetDuration() - (Turbine.Engine.GetGameTime() - effect:GetStartTime())
        if effectTime and effectTime > 0 then
            self.bleedBar:ToggleLabel(effect:GetName(), true, effectTime)
        end
    end
end

function WardenTools:TargetEvents(player)

    if self.targetEffectCallback then
        RemoveCallback(self.targetEffects, "EffectAdded", self.targetEffectCallback)
        self.targetEffectCallback = nil
        self.targetEffects = nil
    end
    -- Release memory and force clean it
    self.taget = nil
    collectgarbage()


    -- Start regathering info
    self.target = player:GetTarget()

    if self.target then 

        if self.player:GetName() == player:GetTarget():GetName() then
            return
        end

        -- Check if target can have effects
        if self.target.GetEffects then
            self.targetEffects = self.target:GetEffects()
            self.targetEffectCallback = AddCallback(self.targetEffects, "EffectAdded", function(sender, args) self:QSCheck(self.targetEffects:Get(args.Index)) end)
        end
    end
end 

function WardenTools:SetCallbacks()
    self.effectCallbacks["EffectAdded"] = AddCallback(PlayerEffectsHandler, "EffectAdded", 
    function(effect)
        local effectName = effect:GetName()

        --ChatOutput(effectName .. effect:GetIcon())

        if self.dynamicEffectTable[effectName] then 
            if effectName == "Volley" then
                self.buffSkillBar:ToggleLabel(effectName, true)
            elseif self.dynamicEffectTable[effectName][1] == effect:GetIcon() then
                self.buffSkillBar:ToggleLabel(effectName, true, effect:GetDuration())
            end
        end
    end)

    if self.qsa then
        self.playerCallbacks["TargetChanged"] = AddCallback(self.player, "TargetChanged", function(sender, args) self:TargetEvents(sender) end)
    end


    self.effectCallbacks["EffectRemoved"] = AddCallback(PlayerEffectsHandler, "EffectRemoved", 
    function(effect)

        if effect and self.removeEffectTable[effect:GetName()] then
            self.buffSkillBar:ToggleLabel(effect:GetName(), false)
        end

        --[[
        if self.dynamicEffectTable[effect:GetName()] and 
        self.dynamicEffectTable[effect:GetName()][1] == effect:GetIcon() and 
        not(self.exceptionEffects[effectName]) then
            self.buffSkillBar:ToggleLabel(effect:GetName(), false)
        end
        --]]
    end)

    -- skill callbacks
    local index = 1
    for i = 1, self.trainedSkills:GetCount(), 1 do
        local item = self.trainedSkills:GetItem(i)--:GetSkillInfo()
        local name = item:GetSkillInfo():GetName()
        --ChatOutput(name .. item:GetSkillInfo():GetIconImageID())

        self.skillCallbacks[name] = {}

        if self.skills[name] and not(self.noSkills) then
            self.skillsLUT[name] = item
            table.insert(self.skillCallbacks[name], AddCallback(item, "ResetTimeChanged", function(sender, args) 
                self.normalSkillBar:SetLabelTimer(name, item:GetResetTime() - Turbine.Engine.GetGameTime(), item:GetCooldown())
            end))
        end

        if self.dynamicSkillTable[name] then
            self.skillsLUT[name] = item
            table.insert(self.skillCallbacks[name], AddCallback(item, "ResetTimeChanged", function(sender, args) 
                self.buffSkillBar:ToggleLabel(name, true, self.dynamicSkillTable[name][1])
            end))
        end
    end

    --[[
    self.playerCallbacks["InCombatChanged"] = AddCallback(self.player, "InCombatChanged",
    function(sender, args)
        if self.ferocious and not(self.player:IsInCombat()) then
            self.buffSkillBar:ToggleLabel("Ferocious Free Heal", true, 9)
        end
    end)
    --]]


    --[[
    self.attrCallbacks["GambitChanged"] = AddCallback(self.ClassAttr, "GambitChanged", function(sender, args) 
        for i=1, self.ClassAttr:GetGambitCount(), 1 do
            ChatOutput(self.ClassAttr:GetGambit(i))
        end
    end)
    --]]
end

function WardenTools:RemoveCallbacks()
    for key, value in pairs(self.effectCallbacks) do
        RemoveCallback(PlayerEffectsHandler, key, value)
    end

    --player callbacks
    for key, value in pairs(self.playerCallbacks) do
        RemoveCallback(self.player, key, value)
    end

    for key, value in pairs(self.skillCallbacks) do
        for _, callback in pairs(value) do
            RemoveCallback(self.skillsLUT[key], "ResetTimeChanged", callback)
        end
    end

    collectgarbage()
end

function WardenTools:RemoveSkillObjects()
    self.buffSkillBar:Unload(true)
    self.normalSkillBar:Unload(true)
    self.bleedBar:Unload(true)

    ChatEventHandler:SubscribeTools(nil)
    ChatEventHandler:TogglePlayerCombat(false)

    if self.qsa then
        self.bleedSkills["Low Bleed - Quick Sweep"] = {4, 0x41140177}
        self.bleedSkills["Medium Bleed - Quick Sweep"] = {5, 0x4114017E}
        self.bleedSkills["Marked Target"] = {15, 1091804897}
        self.bleedSkills["Diminished Target"] = {16, 1091806157}
    end

    for _, value in pairs(self.formSlots) do
        value:Unload()
    end

    self.formSlots = {}
    self.skillSlots = {}
    self.specialSkillSlots = {}

    self.skillCount = 0
    self.specialSkillCount = 0

    collectgarbage()
end

function WardenTools:ParseUpdate(message)
    local source,_,skillName = string.match(message,"^(.*) scored a (.*)hit with (.*) on")
    if self.bleedSkills[skillName] then
        if not(self.specialBleedsActive[skillName]) then 
            self.bleedBar:ToggleLabel(skillName, true, self.bleedDuration[skillName])
        end
        if self.initialToBleed[skillName] then 
            self.specialBleedsActive[self.initialToBleed[skillName]] = true
            self.specialBleedSkills[self.initialToBleed[skillName]] = Turbine.Engine.GetGameTime()
        end
    end

    if self.specialBleedSkills[skillName] then
        if string.find(skillName, "Bleed") then
            local interval = Turbine.Engine.GetGameTime() - self.specialBleedSkills[skillName]
            if interval > 4.3 then
                self.bleedBar:ToggleLabel(self.bleedToInitial[skillName], true, self.bleedDuration[self.bleedToInitial[skillName]] - 4)
            end
        end

        self.specialBleedSkills[skillName] = Turbine.Engine.GetGameTime()
    end
end

function WardenTools:ToggleMove()
    self.moveBox:ToggleMove()
end

function WardenTools:GetOptions(parent, x, y)
    local container = Turbine.UI.Control()
    container:SetParent(parent)
    container:SetPosition(x, y)
    container:SetSize(530 + 4, 100 + 4)
    container:SetBackColor(Turbine.UI.Color(0, 0, 0))

    local SingleBackground = Turbine.UI.Control()
    SingleBackground:SetParent(container)
    SingleBackground:SetPosition(2, 2)
    SingleBackground:SetSize(530, 100)
    SingleBackground:SetBackColor(Turbine.UI.Color(0.250, 0.250, 0.250))

    local EffectBox2 = Turbine.UI.Lotro.CheckBox()
    EffectBox2:SetParent(SingleBackground)
    EffectBox2:SetPosition(300, 80)
    EffectBox2:SetSize(400,20)
    EffectBox2:SetChecked(Settings.ClassToolsWardenQS)
    EffectBox2:SetText("Accurate QS and javelin debuff tracking")
    EffectBox2:SetVisible(true)

    EffectBox2.CheckedChanged = function()
        Settings.ClassToolsWardenQS = EffectBox2:IsChecked()
    end

    local EffectBox4 = Turbine.UI.Lotro.CheckBox()
    EffectBox4:SetParent(SingleBackground)
    EffectBox4:SetPosition(450, 60)
    EffectBox4:SetSize(200,20)
    EffectBox4:SetChecked(Settings.ClassToolsWardenExtraTick)
    EffectBox4:SetText("+1 bleed pulse")
    EffectBox4:SetVisible(true)

    EffectBox4.CheckedChanged = function()
        Settings.ClassToolsWardenExtraTick = EffectBox4:IsChecked()
    end

    local EffectBox5 = Turbine.UI.Lotro.CheckBox()
    EffectBox5:SetParent(SingleBackground)
    EffectBox5:SetPosition(450, 20)
    EffectBox5:SetSize(100,40)
    EffectBox5:SetChecked(Settings.ClassToolsNoSkills)
    EffectBox5:SetText("Disable skill bars")
    EffectBox5:SetVisible(true)

    EffectBox5.CheckedChanged = function()
        Settings.ClassToolsNoSkills = EffectBox5:IsChecked()
    end

    

    self.bleedBox = function(key, x, y)
        local EffectBox3 = Turbine.UI.Lotro.CheckBox()
        EffectBox3:SetParent(SingleBackground)
        EffectBox3:SetPosition(x, y)
        EffectBox3:SetSize(150,20)
        EffectBox3:SetMultiline(false)
        EffectBox3:SetChecked(Settings.ClassToolsWardenBleeds[key])
        EffectBox3:SetText(key)
        EffectBox3:SetVisible(true)
    
        EffectBox3.CheckedChanged = function()
            Settings.ClassToolsWardenBleeds[key] = EffectBox3:IsChecked()
        end
    end

    local xCounter = 0
    local yCounter = 0
    for key, value in pairs(Settings.ClassToolsWardenBleeds) do
        self.bleedBox(key, math.floor(xCounter / 5) * 150 , (yCounter % 5) * 20)
        xCounter = xCounter + 1
        yCounter = yCounter + 1
    end
end

function WardenTools:GetData()
    return self.data
end

function WardenTools:LoadData()
    self.player = Turbine.Gameplay.LocalPlayer.GetInstance()
    self.effectList = self.player:GetEffects()
    self.ClassAttr = self.player:GetClassAttributes()
    self.trainedSkills = self.player:GetTrainedSkills()
end

function WardenTools:Load()
    self:SetSkillObjects()
    self:SetCallbacks()
    if not(self.updateCallback) then self.updateCallback = AddCallback(Updater, "ExecuteUpdate", self.ExecuteUpdate) end
end

function WardenTools:Unload()
    self:RemoveCallbacks()
    self:RemoveSkillObjects()
    if self.updateCallback then if self.updateCallback then RemoveCallback(Updater, "ExecuteUpdate", self.updateCallback) end end
    self.updateCallback = nil
end

function WardenTools:Reload()
    self:Unload()
    self:LoadData()
    self:Load()
end

function WardenTools:GetBuffData()
    return self.buffData
end

function WardenTools:ChangeData(key, value)
    self.data[key] = value
end

function WardenTools:ChangeBuffData(key, stack, remove, enable)
    self.buffData[key] = {self.buffData[key][1], self.buffData[key][2], stack, remove, enable}
end

function WardenTools:AddBuffData(key, icon, priority, stack, remove, enable)
    self.buffData[key] = {icon, priority, stack, remove, enable}
end

