DataGrab = class(Turbine.UI.Control)
function DataGrab:Constructor()
    Turbine.UI.Control.Constructor(self)
    self:SetVisible(false)
    self:SetMouseVisible(false)

    --self.x = IconHandler()

    self.player = Turbine.Gameplay.LocalPlayer.GetInstance()
    self.effectList = self.player:GetEffects()
    self.ClassAttr = self.player:GetClassAttributes()
    self.trainedSkills = self.player:GetTrainedSkills()
    self.class = self.player:GetClass()

    self.effectCallbacks = {}

    if Settings.CollectData then
        assert(PlayerEffectsHandler ~= nil, "Player Effects Handler is not loaded")
        
        self.effectData = LoadTable(Turbine.DataScope.Account, "PrimeEffectData")
        self.skillData = LoadTable(Turbine.DataScope.Account, "PrimeSkillData")
        self.iconData = LoadTable(Turbine.DataScope.Account, "PrimeIconData")

        if self.effectData == nil or self.skillData == nil or self.skillData[self.class] == nil or self.iconData == nil then
            import "PrimePlugins.RaidTools.Data.BaseData"
        end
    
        if self.effectData == nil then self.effectData = GetBaseEffects() end
        if self.skillData == nil then self.skillData = GetBaseSkills() end
        if not(self.skillData[self.class]) then self.skillData[self.class] = {} end
        if self.iconData == nil then self.iconData = {} end

        local index = 0
        for i = 1, self.trainedSkills:GetCount(), 1 do
            local item = self.trainedSkills:GetItem(i)--:GetSkillInfo()
            local skillInfo = item:GetSkillInfo()
            local name = item:GetSkillInfo():GetName()
            if not(self.skillData[name] ) and skillInfo then
                -- [name] = {Description, ImageID, Type}
                self.skillData[self.class][name] = {skillInfo:GetDescription(), skillInfo:GetIconImageID(), skillInfo:GetType()}
            end
            if not(self.iconData[skillInfo:GetIconImageID()]) then
                self.iconData[name] = skillInfo:GetIconImageID()
            end
        end

        self:SetCallbacks()
    end
end

function DataGrab:GetIcons()
    if self.skillData == nil or self.effectData == nil or self.iconData == nil then 
        import "PrimePlugins.RaidTools.Data.BaseData"
        local result = {}
        for effect, value in pairs(GetBaseEffects()) do
            result[value[4]] = value[4]
            result[effect] = value[4]
        end

        return result
    end
    local result = {}

    for class, classSkills in pairs(self.skillData) do
        for key, value in pairs(classSkills) do
            result[value[2]] = value[2]
            result[key] = value[2]
        end
    end

    for effect, value in pairs(self.effectData) do
        result[value[4]] = value[4]
        result[effect] = value[4]
    end

    for name, icon in pairs(self.iconData) do
        result[name] = icon
    end

    return result
end

function DataGrab:GetEffects(callback)
    if self.effectData == nil then
        LoadTableCallback(Turbine.DataScope.Account, "PrimeEffectData", callback)
    else
        callback(self.effectData)
    end
end

function DataGrab:SetCallbacks()

    self.effectCallbacks["EffectAdded"] = AddCallback(PlayerEffectsHandler, "EffectAdded", 
    function(effect)
        local effectName = effect:GetName()

        if not(self.effectData[effectName]) then
            self.effectData[effectName] = {effect:GetCategory(), effect:GetDescription(), effect:GetDuration(), effect:GetIcon(), effect:IsCurable(), effect:IsDebuff()}
        end
    end)

end

function DataGrab:RemoveCallbacks()
    --player callbacks
    for key, value in pairs(self.effectCallbacks) do
        RemoveCallback(PlayerEffectsHandler, key, value)
    end

    collectgarbage()

end

function DataGrab:Save()
    if Settings.CollectData then
        SaveTable(Turbine.DataScope.Account, "PrimeEffectData", self.effectData)
        SaveTable(Turbine.DataScope.Account, "PrimeSkillData", self.skillData)
        SaveTable(Turbine.DataScope.Account, "PrimeIconData", self.iconData)
    end

    --Turbine.PluginData.Save(Turbine.DataScope.Account,  "PrimeEffectDataTemp", self.effectData)
    --Turbine.PluginData.Save(Turbine.DataScope.Account, "PrimeSkillDataTemp", self.skillData)
    --Turbine.PluginData.Save(Turbine.DataScope.Account, "PrimeIconDataTemp", self.iconData)

    self:RemoveCallbacks()
end

