LootWindow = class(Turbine.UI.Window)
function LootWindow:Constructor()
    local width = math.max((tablelength(LootLocks.Characters) + 1) * 125, 5 * 125)
    local height = 500
    local x = Settings.LOX
    local y = Settings.LOY
    local font = Settings.LOFont
    local colour = Turbine.UI.Color(1, 1, 1)
    Turbine.UI.Window.Constructor(self)
    self:SetSize(width,height); -- sets the window size to 200 by 200 pixels
    self:SetPosition(x,y)
    self:SetVisible(false)
    self:SetWantsKeyEvents(true)
    self:SetMouseVisible(false)
    --self:SetBackColor(Turbine.UI.Color(0.250, 0.250, 0.250))

    self.InstanceLocks = nil
    self.RaidLocks = nil
    self.QuestLocks = nil

    --BeginningOfPrimeTime = (AccountData["TimeZone"] - 1) * 3600

    local UIparent = nil

    local ButtonDictionary = {}
    local PageDictionary = {}

    local CurrentPage = nil

    self.KeyDown = function(sender, args)
        if args["Action"] == 145 then
            if self:IsVisible() then self:SetVisible(false) end
        end
    end

    self.ChangeButtonState = function(name)

        if CurrentPage == name then
            PageDictionary[name]:SetVisible(false)
            ButtonDictionary[name]:SetBackColor(Turbine.UI.Color(0.250, 0.250, 0.250))
            CurrentPage = nil
            
        else
            for key, value in pairs(PageDictionary) do
                value:SetVisible(false)
            end
    
            for key, value in pairs(ButtonDictionary) do
                value:SetBackColor(Turbine.UI.Color(0.250, 0.250, 0.250))
            end
    
            ButtonDictionary[name]:SetBackColor(Turbine.UI.Color(0.125, 0.125, 0.125))
            
            PageDictionary[name]:SetVisible(true)

            CurrentPage = name
        end
    end

    self.ClearUI = function()
        for key, value in pairs(PageDictionary) do
            value:SetParent(nil)
        end

        for key, value in pairs(ButtonDictionary) do
            value:SetParent(nil)
        end
    end

    self.UpdateUI = function()
        self.ClearUI()
        HandleLootUpdate()

        local SortedCharacters = {}
        for key, value in pairs(LootLocks.Characters) do table.insert( SortedCharacters, key ) end
        table.sort( SortedCharacters )

        local tableWidth = (tablelength(LootLocks.Characters) + 1) * 125

        -- Instance locks
        local InstanceLocks = LootPage(self, tableWidth, 400)
        PageDictionary["Instances"] = InstanceLocks

        local InstanceButton = LootPageButton(self, "Instances", 1)
        ButtonDictionary["Instances"] = InstanceButton
        
        InstanceButton.MouseClick = function()
            self.ChangeButtonState("Instances")
        end

        
        -- Instance locks
        local completeInstanceLocks = {}

        -- T2+ Instance Locks
        if LootLocks.Locks ~= nil then
            for key, value in pairs(LootLocks.Locks) do
                completeInstanceLocks[key] = value
            end
        end

        -- T1 instance Locks

        if tablelength(completeInstanceLocks) > 0 then
            LootOverview(InstanceLocks, completeInstanceLocks, SortedCharacters, 2)
        end

        -- Raid locks
        local completeRaidLocks = {}

        -- T2+ Raid Locks
        if LootLocks.RaidLocks ~= nil then
            for key, value in pairs(LootLocks.RaidLocks) do
                completeRaidLocks[key] = value
            end
        end

        -- T1 Raid Locks
        if LootLocks.T1RaidLocks ~= nil then
            for key, value in pairs(LootLocks.T1RaidLocks) do
                completeRaidLocks[key] = value
            end
        end
        
        -- Raid locks
        local RaidLocks = LootPage(self, tableWidth, 400)
        PageDictionary["Raids"] = RaidLocks

        local RaidButton = LootPageButton(self, "Raids", 0)
        ButtonDictionary["Raids"] = RaidButton

        RaidButton.MouseClick = function()
            self.ChangeButtonState("Raids")
        end

        if tablelength(completeRaidLocks) > 0 then
            LootOverview(RaidLocks, completeRaidLocks, SortedCharacters, 1)
        end

        -- Daily Locks
        local DailyLocks = LootPage(self, tableWidth, 400)
        PageDictionary["Daily"] = DailyLocks

        local DailyButton = LootPageButton(self, "Daily", 2)
        ButtonDictionary["Daily"] = DailyButton

        DailyButton.MouseClick = function()
            self.ChangeButtonState("Daily")
        end

        local dailyLocks = {}

        -- Daily Instance Locks
        if LootLocks.DailyLocks ~= nil then
            for key, value in pairs(LootLocks.DailyLocks) do
                dailyLocks[key] = value
            end
        end

        -- T1 instance Locks
        if LootLocks.T1Locks ~= nil then
            for key, value in pairs(LootLocks.T1Locks) do
                dailyLocks[key] = value
            end
        end

        if tablelength(dailyLocks) > 0 then
            LootOverview(DailyLocks, dailyLocks, SortedCharacters, 1)
        end

        -- Weekly Locks
        local WeeklyLocks = LootPage(self, tableWidth, 400)
        PageDictionary["Weekly"] = WeeklyLocks

        local WeeklyButton = LootPageButton(self, "Weekly", 3)
        ButtonDictionary["Weekly"] = WeeklyButton

        WeeklyButton.MouseClick = function()
            self.ChangeButtonState("Weekly")
        end

        if LootLocks.WeeklyLocks ~= nil and tablelength(LootLocks.WeeklyLocks) > 0 then
            LootOverview(WeeklyLocks, LootLocks.WeeklyLocks, SortedCharacters, 1)
        end

        local MoveButton = SpecialFunctionButton(self, Turbine.UI.Color(0,102/255, 0), "PrimePlugins/RaidTools/Images/Move.tga", false)

        self.Moving = false

        MoveButton.MouseDown = function(sender,args)
            self.MoveX = args.X
            self.MoveY = args.Y
            self.Moving = true
        end
      
        MoveButton.MouseUp = function()
            self.Moving = false
        end
      
        MoveButton.MouseMove = function(sender,args)
            if self.Moving then
              local newLeft = self:GetLeft() - (self.MoveX - args.X)
              local newTop = self:GetTop() - (self.MoveY - args.Y)
              if newLeft > (Turbine.UI.Display.GetWidth() - self:GetWidth()) then
                newLeft = Turbine.UI.Display.GetWidth() - self:GetWidth() end
              if newTop > (Turbine.UI.Display.GetHeight() - self:GetHeight()) then
                newTop = Turbine.UI.Display.GetHeight() - self:GetHeight() end
                self:SetPosition(newLeft,newTop)
            end
        end

        local CloseButton = SpecialFunctionButton(self, Turbine.UI.Color(153/255, 0, 0), "PrimePlugins/RaidTools/Images/X.tga", true)

        CloseButton.MouseClick = function()
            self:SetVisible(false)
        end

    end
    self.UpdateUI()
end

function GetPartialLabel(parent, text, fontColour, font, fill, index, width)
    local tempLabel =  Turbine.UI.Label()
    tempLabel:SetParent(parent)
    tempLabel:SetPosition(125 * index , 0)
    tempLabel:SetTextAlignment(fill) -- Turbine.UI.ContentAlignment.MiddleCenter
    --tempLabel:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter) -- Turbine.UI.ContentAlignment.MiddleCenter
    tempLabel:SetSize(width, 15)
    tempLabel:SetForeColor(fontColour)
    tempLabel:SetFont(font)
    tempLabel:SetVisible(true)
    tempLabel:SetText(text)

    --tempLabel:SetBackColor(fontColour)

    --return tempLabel
end

function SaveLoot()
    if lootWindow then SaveTable(Turbine.DataScope.Server, "PrimeLootLocks", LootLocks); end
end

LootButton = class(Turbine.UI.Window)
function LootButton:Constructor()
    Turbine.UI.Window.Constructor(self)
    self:SetSize(100, 100)
    self:SetPosition(360, Turbine.UI.Display.GetHeight() - 20)
    self:SetVisible(true)
    
    self.toggleButton = Turbine.UI.Lotro.Button()
    self.toggleButton:SetParent(self)
    self.toggleButton:SetText("Loot")
    self.toggleButton:SetVisible(true)
    self.toggleButton:SetWidth(100)
    
    self.toggleButton.Click = function()
        lootWindow:SetVisible(not lootWindow:IsVisible())
    end
end

function LootOverview(parent, locks, characters, maxLocks)
    local font = Settings.LOFont
    local bFont = Settings.LOBFont 
    local colour = Turbine.UI.Color(1, 1, 1)

    local tableWidth = (tablelength(characters) + 1) * 125

    local container = Turbine.UI.ListBox()
    container:SetParent(parent)
    container:SetSize(tableWidth, 400)
    container:SetPosition(0, 0)
    --container:SetBackColor(GoldColor)
    container:SetVisible(true)
    container:SetWantsKeyEvents(true)
    container:SetMouseVisible(false)

    local containerScrollbar = Turbine.UI.Lotro.ScrollBar()
    containerScrollbar:SetOrientation( Turbine.UI.Orientation.Vertical )
    containerScrollbar:SetParent( parent )
    containerScrollbar:SetPosition(tableWidth - 10, 0 )
    containerScrollbar:SetSize( 10, 400 )
    container:SetVerticalScrollBar( containerScrollbar )
    
    local NameBar = Turbine.UI.Control()
    NameBar:SetParent(parent)
    NameBar:SetPosition(10, 0)
    NameBar:SetSize(tableWidth, 15)
    NameBar:SetBackColor(Turbine.UI.Color(0.125, 0.125, 0.125))

    local charIndex = 1
    for _, value in pairs(characters) do
        GetPartialLabel(NameBar, value, colour, bFont, Turbine.UI.ContentAlignment.MiddleCenter, charIndex, 125)
        charIndex = charIndex + 1
    end

    container:AddItem( NameBar )

    local SortedEffects = {}
    for key, value in pairs(locks) do table.insert( SortedEffects, key ) end
    table.sort( SortedEffects )

    local tempIndex = 1

    local chest = nil
    local lastChest = nil

    for _, value in pairs(SortedEffects) do
        if lastChest ~= string.sub(value, 1, string.len(value) - 1) then
            local listItem = Turbine.UI.Control()
            listItem:SetVisible(true)
            listItem:SetSize( tableWidth, 15 )
            listItem:SetMouseVisible(true)
            local colourNumb = tempIndex % 2
            listItem:SetBackColor(Turbine.UI.Color(0.125 + colourNumb * 0.125, 0.125 + colourNumb * 0.125, 0.125 + colourNumb * 0.125))

            tempIndex = tempIndex + 1
            container:AddItem( listItem )
        end

        local listItem = Turbine.UI.Control()
        listItem:SetVisible(true)
        listItem:SetSize( tableWidth, 15 )
        listItem:SetMouseVisible(true)

        local colourNumb = tempIndex % 2
        listItem:SetBackColor(Turbine.UI.Color(0.125 + colourNumb * 0.125, 0.125 + colourNumb * 0.125, 0.125 + colourNumb * 0.125))

        GetPartialLabel(listItem, value, colour, bFont, Turbine.UI.ContentAlignment.MiddleLeft, 0, 200)

        local lockIndex = 1
        for _, name in pairs(characters) do
            local lockAmount = nil
            if locks[value][name] == nil then
                lockAmount = maxLocks
            else
                lockAmount = locks[value][name]
            end
            local labelColour = nil
            if lockAmount > 0 then
                labelColour = Turbine.UI.Color(0, 1, 0)
            else
                labelColour = Turbine.UI.Color(1, 0, 0)
            end
            GetPartialLabel(listItem, lockAmount, labelColour, bFont, Turbine.UI.ContentAlignment.MiddleCenter, lockIndex, 125)
            lockIndex = lockIndex + 1
        end

        lastChest = string.sub(value, 1, string.len(value) - 1)

        tempIndex = tempIndex + 1
        container:AddItem( listItem )
    end

    local listItem = Turbine.UI.Control()
    listItem:SetVisible(true)
    listItem:SetSize( tableWidth, 15 )
    listItem:SetMouseVisible(true)
    local colourNumb = tempIndex % 2
    listItem:SetBackColor(Turbine.UI.Color(0.125 + colourNumb * 0.125, 0.125 + colourNumb * 0.125, 0.125 + colourNumb * 0.125))

    tempIndex = tempIndex + 1
    container:AddItem( listItem )
end

function LootPage(parent, width, height)
    local Page = Turbine.UI.Control()
    Page:SetParent(parent)
    Page:SetPosition((math.max((tablelength(LootLocks.Characters) + 1) * 125, 5 * 125) - ((tablelength(LootLocks.Characters) + 1) * 125)) / 2, 30)
    Page:SetSize(width, height)
    --Page:SetBackColor(Turbine.UI.Color(0.125, 0.125, 0.125))
    Page:SetVisible(false)
    Page:SetMouseVisible(false)
    
    return(Page)
end

function LootPageButton(parent, name, index)
    local Button = Turbine.UI.Label()
    Button:SetParent(parent)
    Button:SetPosition(125 * index + (math.max((tablelength(LootLocks.Characters) + 1) * 125, 5 * 125) - 4 * 125) / 2, 0)
    Button:SetSize(125, 30)
    Button:SetBackColor(Turbine.UI.Color(0.250, 0.250, 0.250))
    Button:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
    Button:SetFont(Turbine.UI.Lotro.Font.BookAntiquaBold24)
    Button:SetText(name)
    Button:SetWantsKeyEvents(true)

    return(Button)
end

function SpecialFunctionButton(parent, colour, image, maxPos)
    local Button = Turbine.UI.Label()
    Button:SetParent(parent)
    if maxPos then
        Button:SetPosition(125 * 4 + (math.max((tablelength(LootLocks.Characters) + 1) * 125, 5 * 125) - 4 * 125) / 2, 0)
    else
        Button:SetPosition((math.max((tablelength(LootLocks.Characters) + 1) * 125, 5 * 125) - 4 * 125) / 2 - 30, 0)
    end
    Button:SetSize(30, 30)
    Button:SetBackColor(colour)
    --Button:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
    --Button:SetFont(Turbine.UI.Lotro.Font.BookAntiquaBold24)
    --Button:SetWantsKeyEvents(true)

    local Image = Turbine.UI.Label()
    Image:SetParent(Button)
    Image:SetPosition(2.5, 2.5)
    Image:SetSize(25, 25)
    Image:SetBackground(image)
    Image:SetBackColorBlendMode(Turbine.UI.BlendMode.AlphaBlend)
    Image:SetBlendMode(Turbine.UI.BlendMode.Overlay)
    Image:SetWantsKeyEvents(false)
    Image:SetMouseVisible(false)

    return(Button)
end