TargetBox = class(Turbine.UI.Window)
function TargetBox:Constructor()
    Turbine.UI.Window.Constructor(self)

    local x = Settings.TBX
    local y = Settings.TBY
    local width = 100
    local height = 5000
    self:SetVisible(true)
    self:SetPosition(x, y)
    self:SetSize(width, height)
    --self:SetBackColor(GetRandomColour())
    self:SetMouseVisible(false)

    self.TargetSelectionsBoxes = {}
    self.TargetSelections = {}
    self.VisibleBox = {}

    self.targettingAllowed = false

    self.ViewingWindow = {}

    self.BoxCount = 0

    self.Shuffle = false

    self.player = Turbine.Gameplay.LocalPlayer.GetInstance()

    self.callback = {}

    self.moveBox = MoveBox(self, width, 50, "TargetBox")    

    if Party ~= nil then
        for i = 1, Party:GetMemberCount(), 1 do

            local E = Turbine.UI.Lotro.EntityControl()
            E:SetParent (self)
            E:SetVisible (false)
            E:SetSize(100, 50)
            E:SetEntity(Party:GetMember(i))
            --E:SetBackColor(GetRandomColour())
            
            self.TargetSelections[Party:GetMember(i):GetName()] = Party:GetMember(i)
            self.TargetSelectionsBoxes[Party:GetMember(i):GetName()] = E

            --self:ShowBox(Party:GetMember(i):GetName(), nil)

        end
    end

    self.TargetChanged = function(sender)
        local PTarget = sender:GetTarget()
        if PTarget then
            local TargetName = PTarget:GetName()
    
            if self.TargetSelectionsBoxes[TargetName] then
                if self.TargetSelectionsBoxes[TargetName]:GetEntity() then
                    --ChatOutput("Already existing and still active")
                    return
                end
    
                if self.TargetSelectionsBoxes[TargetName]:GetEntity() == nil then
                    ChatOutput("Already Existing but needed refresh")
                    self.TargetSelectionsBoxes[TargetName]:SetEntity(sender:GetTarget())
                    return
                end
            end
    
            local E = Turbine.UI.Lotro.EntityControl()
            E:SetParent(self)
            E:SetVisible(false)
            E:SetSize(100, 50)
            E:SetEntity(sender:GetTarget())
            --E:SetBackColor(GetRandomColour())
    
            self.TargetSelections[sender:GetTarget():GetName()] = sender:GetTarget()
            self.TargetSelectionsBoxes[TargetName] = E
        end
    end

    if self.targettingAllowed then
        self.TargetChanged(self.player)

        self.callback["TargetChanged"] = AddCallback(self.player, "TargetChanged", function(sender, args)
            self.TargetChanged(sender)
        end)
    end
end

function TargetBox:RemoveCallbacks()
    RemoveCallback(self.player, "TargetChanged", self.callback["TargetChanged"])
end

function TargetBox:ExecuteUpdate(delta)

    for key, value in pairs(self.VisibleBox) do
        if value:ExecuteUpdate(delta) then
            self.VisibleBox[key] = nil
            self.Shuffle = true
        end
    end

    if self.Shuffle then self:Reshuffle() end

    if tablelength(self.VisibleBox) <= 0 then Updater:UnSubscribe(self) end
end


function TargetBox:ShowBox(name, iconType)
    if not(name) then return end

    -- Entitybox exists
    if self.TargetSelectionsBoxes[name] and 
    self.TargetSelectionsBoxes[name]:GetEntity() then 

        if self.VisibleBox[name] then 
            self.VisibleBox[name]:ResetDuration()
            self.Shuffle = true
            return
        end

        -- Set new box visible
        self.VisibleBox[name] = UITargetBox(0, 50 * tablelength(self.VisibleBox), self.TargetSelectionsBoxes[name], self, name, iconType)

        -- Subscribe to update handler
        Updater:Subscribe(self)

        self.Shuffle = true
    end
end

function TargetBox:ToggleMove()
    self.moveBox:ToggleMove()
end

function TargetBox:Reshuffle()

    if tablelength(self.VisibleBox) <= 0 then return end

    --[[
    local RemainingDurations = {}
    for key, value in pairs(self.VisibleBox) do
        RemainingDurations[key] = value:GetViewingTime()
    end

    table.sort( RemainingDurations, Compare )

    local index = 0
    for key, _ in pairs(RemainingDurations) do
        self.VisibleBox[key]:AdjustPosition(index)
        index = index + 1
    end

    self.Shuffle = false
    --]]

    local counter = 0
    for _,v in spairs(self.VisibleBox, function(t,a,b) return t[b]:GetViewingTime() > t[a]:GetViewingTime() end) do
        v:AdjustPosition(counter)
        counter = counter + 1
    end

    self.Shuffle = false
end

UITargetBox = class(Turbine.UI.Control)
function UITargetBox:Constructor(x, y, entityBox, parent, name, iconType)
    Turbine.UI.Control.Constructor(self)
    local width = 100
    local height = 50
    self:SetParent(parent)
    self:SetVisible(true)
    self:SetPosition(x, y)
    self:SetSize(width, height)
    --self:SetBackColor(GetRandomColour())
    self:SetMouseVisible(false)

    self.ViewingWindow = 15

    self.entityBox = entityBox

    entityBox:SetVisible(true)
    
    self.parent = parent

    self.opacityNameWindow = Turbine.UI.Window()
    self.opacityNameWindow:SetParent(self)
    self.opacityNameWindow:SetSize(width, height)
    self.opacityNameWindow:SetBackColor(Turbine.UI.Color(0, 0, 0))
    self.opacityNameWindow:SetOpacity(0.3)
    self.opacityNameWindow:SetVisible(true)
    self.opacityNameWindow:SetMouseVisible(false)
    self.opacityNameWindow:SetBackColor(Turbine.UI.Color(1, 0, 0))

    self.containerWindow = Turbine.UI.Window()
    self.containerWindow:SetParent(self)
    self.containerWindow:SetSize(width, height)
    --self.opacityNameWindow2:SetBackColor(Turbine.UI.Color(0, 0, 0))
    --self.opacityNameWindow2:SetOpacity(0.3)
    self.containerWindow:SetVisible(true)
    self.containerWindow:SetMouseVisible(false)

    self.NameLabel = Turbine.UI.Label()
    self.NameLabel:SetParent(self.containerWindow)
    self.NameLabel:SetSize(100, 15)
    self.NameLabel:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
    self.NameLabel:SetMouseVisible(false)
    self.NameLabel:SetMarkupEnabled(true)
    self.NameLabel:SetFont(Turbine.UI.Lotro.Font.TrajanProBold16)
    self.NameLabel:SetText(name)

    self.IconLabel = Turbine.UI.Label()
    self.IconLabel:SetParent(self.containerWindow)
    self.IconLabel:SetPosition(34, 15)
    self.IconLabel:SetSize(32, 32)
    self.IconLabel:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
    self.IconLabel:SetMouseVisible(false)
    self.IconLabel:SetBackground(iconType)
end

function UITargetBox:ExecuteUpdate(delta)
    self.ViewingWindow = self.ViewingWindow - delta
    if self.ViewingWindow > 10 then
        self.opacityNameWindow:SetBackColor(Turbine.UI.Color(1 - ((15 - self.ViewingWindow)/5), 0, 0))
        self.opacityNameWindow:SetOpacity(math.max(1 - ((15 - self.ViewingWindow)/5), 0.3))
    end

    if self.ViewingWindow < 0 then
        self.containerWindow:SetVisible(false)
        self.opacityNameWindow:SetVisible(false)
        self.entityBox:SetVisible(false)
        self:SetParent(nil)
        return true
    end
end

function UITargetBox:ResetDuration()
    self.ViewingWindow = 15
end

function UITargetBox:AdjustPosition(i)
    self:SetPosition(0, 50 * i)
    self.entityBox:SetPosition(0, 50 * i)
end

function UITargetBox:GetViewingTime()
    return(self.ViewingWindow)
end