local normalText = Turbine.UI.Lotro.Font.Verdana18 --Turbine.UI.Lotro.Font.TrajanPro16
local headerText = Turbine.UI.Lotro.Font.TrajanProBold22
local bigHeaderText = Turbine.UI.Lotro.Font.TrajanProBold30

DungeonLog = class(Turbine.UI.Lotro.Window)
function DungeonLog:Constructor()
    import "PrimePlugins.RaidTools.DungeonData"
    local width = 1000
    local height = 700
    local x = math.floor(Turbine.UI.Display.GetWidth() / 2 - (width /2))
    local y = math.floor(Turbine.UI.Display.GetHeight() / 2 - (height /2))
    local font = Settings.LOFont
    local colour = Turbine.UI.Color(1, 1, 1)

    local pages = {}

    Turbine.UI.Lotro.Window.Constructor(self)
    self:SetSize(width,height)
    self:SetPosition(x,y)
    self:SetWantsKeyEvents(true)
    self:SetText("Dungeon Log")
    self:SetVisible(false)
    self:SetZOrder(1)

    self.KeyDown = function(sender, args)
        if args["Action"] == 145 then
            if self:IsVisible() then self:SetVisible(false) end
        end
    end

    -- Create the tree view control.
    local treeView = Turbine.UI.TreeView()
    treeView:SetParent( self )
    treeView:SetPosition( 10, 50 )
    treeView:SetSize( 220, 900 )
    --treeView:SetBackColor( Turbine.UI.Color( 0.2, 0.2, 0.2 ) )
    treeView:SetIndentationWidth( 15 )

    -- Populate the test nodes.
    local rootNodes = treeView:GetNodes()

    for key, value in pairs(DungeonInfo) do
        --local node = MyTreeNode("<rgb=#".. DungeonColour[key] .. ">" .. key .. "</rgb>")
        local node = MyTreeNode(key, false)
        rootNodes:Add(node)

        node.MouseClick = function()
            node:ChangeBackground()
        end

        local subRootNodes = node:GetChildNodes()

        local sortedKeys = {}
        for key, value in pairs(DungeonInfo[key]) do table.insert( sortedKeys,key ) end
        table.sort( sortedKeys )

        for subkey, subvalue in pairs(sortedKeys) do
            --local subNode = MyTreeNode("<rgb=#".. DungeonColourSecondary[key] .. ">" .. subkey .. "</rgb>")
            local subNode = MyTreeNode(subvalue, true)
            subRootNodes:Add(subNode)

            local container = Turbine.UI.ListBox()
            container:SetParent(self)
            container:SetSize(780, 600)
            container:SetPosition(220, 50)
            --container:SetBackColor(GoldColor)
            container:SetVisible(false)
            container:SetWantsKeyEvents(true)
            container:SetMouseVisible(true)

            local containerScrollbar = Turbine.UI.Lotro.ScrollBar()
            containerScrollbar:SetOrientation( Turbine.UI.Orientation.Vertical )
            containerScrollbar:SetParent( self )
            containerScrollbar:SetPosition( 980, 50 )
            containerScrollbar:SetSize( 10, 600 )
            containerScrollbar:SetVisible(false)
        
            container:SetVerticalScrollBar( containerScrollbar )

            local description = DungeonDescription(container, key, subvalue)
            container:AddItem(description)
            table.insert( pages,  container)
            subNode.MouseClick = function()
                for key, value in pairs(pages) do
                    value:SetVisible(false)
                    value:GetVerticalScrollBar():SetVisible(false)
                end
                container:SetVisible(true)
                if container:GetItem(1):GetHeight() > 600 then
                    container:GetVerticalScrollBar():SetVisible(true)
                end
            end
        end

    end
end

DungeonDescription = class( Turbine.UI.Control )
function DungeonDescription:Constructor(parent, dungeon, boss)
    Turbine.UI.Control.Constructor( self )
    self:SetParent(parent)
    --self:SetBackColor( Turbine.UI.Color( math.random(), math.random(), math.random() ) )
    self:SetPosition(0, 0)
    self:SetVisible(true)

    local heightPosition = 10

    self.nameLabel = Turbine.UI.Label()
    self.nameLabel:SetParent(self)
    self.nameLabel:SetFont(Turbine.UI.Lotro.Font.TrajanProBold36)
    self.nameLabel:SetSize(800, 40)
    self.nameLabel:SetPosition(20, heightPosition)
    self.nameLabel:SetText(dungeon .. " - " .. boss)
    self.nameLabel:SetMouseVisible(false)

    heightPosition = heightPosition + 40

    self.description = Turbine.UI.Label()
    self.description:SetParent(self)
    self.description:SetFont(normalText)
    self.description:SetSize(700, 20 * TextHeight(DungeonInfo[dungeon][boss]["Description"]))
    self.description:SetPosition(30, heightPosition)
    self.description:SetMarkupEnabled(true)
    self.description:SetText(DungeonInfo[dungeon][boss]["Description"])

    heightPosition = heightPosition + 20 * TextHeight(DungeonInfo[dungeon][boss]["Description"]) + 30

    if tablelength(DungeonInfo[dungeon][boss]["Mechanics"]) > 0 then 
        self.mechanicsLabel = Turbine.UI.Label()
        self.mechanicsLabel:SetParent(self)
        self.mechanicsLabel:SetFont(bigHeaderText)
        self.mechanicsLabel:SetSize(800, 40)
        self.mechanicsLabel:SetPosition(20, heightPosition)
        self.mechanicsLabel:SetText("Mechanics")

        heightPosition = heightPosition + 40

        for key, value in pairs(DungeonInfo[dungeon][boss]["Mechanics"]) do

            local mechanicName = Turbine.UI.Label()
            mechanicName:SetParent(self)
            mechanicName:SetFont(headerText)
            mechanicName:SetSize(700, 20)
            mechanicName:SetPosition(30, heightPosition)
            mechanicName:SetMarkupEnabled(true)
            mechanicName:SetText(key)

            heightPosition = heightPosition + 20

            local mechanic = Turbine.UI.Label()
            mechanic:SetParent(self)
            mechanic:SetFont(normalText)
            mechanic:SetSize(700, 18 * TextHeight(value))
            mechanic:SetMarkupEnabled(true)
            --mechanic:SetBackColor(GoldColor)
            mechanic:SetPosition(40, heightPosition)
            mechanic:SetText(value)

            heightPosition = heightPosition + 18 * TextHeight(value)
        end
    end

    heightPosition = heightPosition + 30

    if tablelength(DungeonInfo[dungeon][boss]["Loot"]) > 0 then
        local lootName = Turbine.UI.Label()
        lootName:SetParent(self)
        lootName:SetFont(bigHeaderText)
        lootName:SetSize(700, 40)
        lootName:SetPosition(20, heightPosition)
        lootName:SetMarkupEnabled(true)
        lootName:SetText("<rgb=#FF9900>" .. "Unique Loot</rgb>")

        heightPosition = heightPosition + 40

        for key, value in pairs(DungeonInfo[dungeon][boss]["Loot"]) do

            local lootName = Turbine.UI.Label()
            lootName:SetParent(self)
            lootName:SetFont(headerText)
            lootName:SetSize(700, 20)
            lootName:SetPosition(30, heightPosition)
            lootName:SetMarkupEnabled(true)
            lootName:SetText(key)
            --ChatOutput(key)
            lootName:SetForeColor(TypeColour[key])

            heightPosition = heightPosition + 20

            local counter = 0

            for subkey, subvalue in pairs(DungeonInfo[dungeon][boss]["Loot"][key]) do

                if counter > 15 then
                    counter = 0
                    heightPosition = heightPosition + 40
                end
                -- Thank you compendium
                local qs = Turbine.UI.Lotro.Quickslot()
                qs:SetSize(40,40)
                qs:SetVisible(true);	        
                qs:SetAllowDrop(false); 
                qs:SetEnabled(false);	
                qs:SetUseOnRightClick(false)
                qs:SetParent(self)
                qs:SetPosition(counter * 40 + 40, heightPosition)


                local shortcut = Turbine.UI.Lotro.Shortcut(Turbine.UI.Lotro.ShortcutType.Item , string.format('0x0000000000000000,0x%s', string.upper(string.format('%x', subvalue)) ))
                qs:SetShortcut( shortcut )
                qs.MouseDown=function(s,a)
                    -- hack to prevent real item from being consumed by a click
                    local sc = qs:GetShortcut()
                    qs:SetShortcut(nil)
                    qs:SetShortcut(shortcut)
                end	
                counter = counter + 1

            end
            heightPosition = heightPosition + 40
        end
    end

    heightPosition = heightPosition + 30
    
    if DungeonInfo[dungeon][boss]["T45Loot"] then
        local lootName = Turbine.UI.Label()
        lootName:SetParent(self)
        lootName:SetFont(bigHeaderText)
        lootName:SetSize(700, 40)
        lootName:SetPosition(20, heightPosition)
        lootName:SetMarkupEnabled(true)
        lootName:SetText("<rgb=#FF9900>" .. "T4 / T5 Loot</rgb>")

        heightPosition = heightPosition + 40

        for key, value in pairs(SpecialLoot) do

            local lootName = Turbine.UI.Label()
            lootName:SetParent(self)
            lootName:SetFont(headerText)
            lootName:SetSize(700, 20)
            lootName:SetPosition(30, heightPosition)
            lootName:SetMarkupEnabled(true)
            lootName:SetText(key)
            lootName:SetForeColor(TypeColour[key])

            heightPosition = heightPosition + 20

            local counter = 0

            for subkey, subvalue in pairs(SpecialLoot[key]) do

                if counter > 15 then
                    counter = 0
                    heightPosition = heightPosition + 40
                end
                -- Thank you compendium
                local qs = Turbine.UI.Lotro.Quickslot()
                qs:SetSize(40,40)
                qs:SetVisible(true);	        
                qs:SetAllowDrop(false); 
                qs:SetEnabled(false);	
                qs:SetUseOnRightClick(false)
                qs:SetParent(self)
                qs:SetPosition(counter * 40 + 40, heightPosition)


                local shortcut = Turbine.UI.Lotro.Shortcut(Turbine.UI.Lotro.ShortcutType.Item , string.format('0x0000000000000000,0x%s', string.upper(string.format('%x', subvalue)) ))
                qs:SetShortcut( shortcut )
                qs.MouseDown=function(s,a)
                    -- hack to prevent real item from being consumed by a click
                    local sc = qs:GetShortcut()
                    qs:SetShortcut(nil)
                    qs:SetShortcut(shortcut)
                end	
                counter = counter + 1

            end
            heightPosition = heightPosition + 40
        end
    end

    heightPosition = heightPosition + 30
    
    if DungeonInfo[dungeon][boss]["BaseLoot"] then
        local lootName = Turbine.UI.Label()
        lootName:SetParent(self)
        lootName:SetFont(bigHeaderText)
        lootName:SetSize(700, 40)
        lootName:SetPosition(20, heightPosition)
        lootName:SetMarkupEnabled(true)
        lootName:SetText("<rgb=#FF9900>" .. "Base Loot</rgb>")

        heightPosition = heightPosition + 40

        for key, value in pairs(BaseLoot) do

            local lootName = Turbine.UI.Label()
            lootName:SetParent(self)
            lootName:SetFont(headerText)
            lootName:SetSize(700, 20)
            lootName:SetPosition(30, heightPosition)
            lootName:SetMarkupEnabled(true)
            lootName:SetText(key)
            lootName:SetForeColor(TypeColour[key])

            heightPosition = heightPosition + 20

            local counter = 0

            for subkey, subvalue in pairs(BaseLoot[key]) do

                if counter > 15 then
                    counter = 0
                    heightPosition = heightPosition + 40
                end
                -- Thank you compendium
                local qs = Turbine.UI.Lotro.Quickslot()
                qs:SetSize(40,40)
                qs:SetVisible(true);	        
                qs:SetAllowDrop(false); 
                qs:SetEnabled(false);	
                qs:SetUseOnRightClick(false)
                qs:SetParent(self)
                qs:SetPosition(counter * 40 + 40, heightPosition)


                local shortcut = Turbine.UI.Lotro.Shortcut(Turbine.UI.Lotro.ShortcutType.Item , string.format('0x0000000000000000,0x%s', string.upper(string.format('%x', subvalue)) ))
                qs:SetShortcut( shortcut )
                qs.MouseDown=function(s,a)
                    -- hack to prevent real item from being consumed by a click
                    local sc = qs:GetShortcut()
                    qs:SetShortcut(nil)
                    qs:SetShortcut(shortcut)
                end	
                counter = counter + 1

            end
            heightPosition = heightPosition + 40
        end
    end

    self:SetSize( 800, heightPosition + 30)
end

function TextHeight(text) 
    return math.ceil( string.len( text ) / 90)
end