
-- Reset button
SettingsSettings = class(Turbine.UI.Control)
function SettingsSettings:Constructor(parent)
    local width = Settings.LOWidth
    local height = 500
    local x = Settings.LOX
    local y = Settings.LOY
    local font = Settings.LOFont
    local colour = Turbine.UI.Color(1, 1, 1)
    Turbine.UI.Control.Constructor(self)
    self:SetSize(width,height); -- sets the window size to 200 by 200 pixels
    self:SetParent(parent)
    self:SetPosition(10,30)
    self:SetWantsKeyEvents(true)
    self:SetVisible(false)
    self:SetBackColor(Turbine.UI.Color(0.125, 0.125, 0.125))
    local height = 0

    -- Reset to default settings
    local ResetLabel = GetLabelUI(self, "Reset to Default Settings:", height)

    local ResetButton = Turbine.UI.Lotro.Button()
    ResetButton:SetParent(self)
    ResetButton:SetPosition(20, height + 30)
    ResetButton:SetSize(100,15)
    ResetButton:SetText("Reset")
    ResetButton:SetVisible(true)

    ResetButton.Click = function()
        Settings.Reload = true
        Profiles:ResetProfile()
        MasterOptions:ChangesMade()
        ReloadPlugin(true)
    end

    -- Setup
    local ResetLabel = GetLabelUI(self, "Redo setup:", height + 60)

    local ResetButton = Turbine.UI.Lotro.Button()
    ResetButton:SetParent(self)
    ResetButton:SetPosition(20, height + 90)
    ResetButton:SetSize(100,15)
    ResetButton:SetText("Setup")
    ResetButton:SetVisible(true)

    ResetButton.Click = function()
        if Setup == nil then
            import "PrimePlugins.RaidTools.NewVersionSetup.FirstTimeSetup"
            Setup = NewVersionSetup.FirstTimerOptions()
        else
            Setup:SetVisible()
        end
    end

    local PatchButton = Turbine.UI.Lotro.Button()
    PatchButton:SetParent(self)
    PatchButton:SetVisible(true)
    PatchButton:SetSize(150, 15)
    PatchButton:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
    PatchButton:SetMouseVisible(true)
    PatchButton:SetPosition(150, height + 90)
    PatchButton:SetText("Check Patch Notes")

    PatchButton.Click = function()
        WindowsHandlerController:CreatePatchWindow()
    end

    local collectionBox = Turbine.UI.Lotro.CheckBox()
    collectionBox:SetParent(self)
    collectionBox:SetPosition(220, height + 120)
    collectionBox:SetSize(200, 20)
    collectionBox:SetChecked(Settings.CollectData)
    collectionBox:SetText("Enable Data Collection")
    collectionBox:SetVisible(true)

    collectionBox.CheckedChanged = function(sender,args)
        Settings.CollectData = collectionBox:IsChecked()
    end
    
    -- Enable advanced settings
    local cBox = Turbine.UI.Lotro.CheckBox()
    cBox:SetParent(self)
    cBox:SetPosition(20, height + 120)
    cBox:SetSize(200, 20)
    cBox:SetChecked(Settings.Advanced)
    cBox:SetText("Enable Advanced Options")
    cBox:SetVisible(true)

    local SettingsMade = false

    cBox.CheckedChanged = function(sender,args)
        Settings.Advanced = cBox:IsChecked()
        if Settings.Advanced and not(SettingsMade) then
            GenerateAdvancedSettings(self)
            SettingsMade = true
        end
    end

    if Settings.Advanced then
        GenerateAdvancedSettings(self)
        SettingsMade = true
    end



    self.menu = Turbine.UI.ContextMenu()
    self.menuItems = self.menu:GetItems()



    for key, value in pairs(Profiles:GetPreferences()) do
        local menuItem = Turbine.UI.MenuItem(key)
        menuItem.Click = function()
            Profiles:ChangePreference(key)
        end
        self.menuItems:Add(menuItem)
    end

    self.button = Turbine.UI.Lotro.Button()
    self.button:SetParent(self)
    self.button:SetPosition(300, 30)
    self.button:SetSize(200,15)
    self.button:SetText("Change profile")
    self.button:SetVisible(true)

    self.button.Click = function()
        self.menu:ShowMenu()
    end

    local EffectBox = Turbine.UI.Lotro.CheckBox()
    EffectBox:SetParent(self)
    EffectBox:SetPosition(310, 10)
    EffectBox:SetSize(200,20)
    EffectBox:SetChecked(Settings.OptionsButton == 2)
    EffectBox:SetText("New Optionsbutton Enabled")
    EffectBox:SetVisible(true)

    EffectBox.CheckedChanged = function()
        if EffectBox:IsChecked() then
            Settings.OptionsButton = 2
        else
            Settings.OptionsButton = 1
        end
        MasterOptions:ChangesMade()
    end

end

function GenerateAdvancedSettings(parent)

    local Player = Turbine.Gameplay.LocalPlayer.GetInstance()

    local ResetButton = Turbine.UI.Lotro.Button()
    ResetButton:SetParent(parent)
    ResetButton:SetPosition(20, 150)
    ResetButton:SetSize(100,15)
    ResetButton:SetText("Print Settings")
    ResetButton:SetVisible(true)

    ResetButton.Click = function()
        PrintSettings()
    end

    local ResetButton = Turbine.UI.Lotro.Button()
    ResetButton:SetParent(parent)
    ResetButton:SetPosition(20, 200)
    ResetButton:SetSize(200,15)
    ResetButton:SetText("Get local time")
    ResetButton:SetVisible(true)

    ResetButton.Click = function()
        ChatOutput(Turbine.Engine.GetLocalTime())
    end

    local ResetButton = Turbine.UI.Lotro.Button()
    ResetButton:SetParent(parent)
    ResetButton:SetPosition(20, 220)
    ResetButton:SetSize(200,15)
    ResetButton:SetText("Get game time")
    ResetButton:SetVisible(true)

    ResetButton.Click = function()
        ChatOutput(Turbine.Engine.GetGameTime())
    end

    local ResetButton = Turbine.UI.Lotro.Button()
    ResetButton:SetParent(parent)
    ResetButton:SetPosition(20, 240)
    ResetButton:SetSize(200,15)
    ResetButton:SetText("Print Swap Data")
    ResetButton:SetVisible(true)

    ResetButton.Click = function()
        SwappyWarner:PrintSwaps()
    end

    local ResetButton = Turbine.UI.Lotro.Button()
    ResetButton:SetParent(parent)
    ResetButton:SetPosition(20, 260)
    ResetButton:SetSize(200,15)
    ResetButton:SetText("Get Instancecontroller state")
    ResetButton:SetVisible(true)

    ResetButton.Click = function()
        if Settings.InstanceState then
            ChatOutput("Instance State: true")
        else
            ChatOutput("Instance State: false")
        end
        ChatOutput("Instance version: " .. Settings.InstanceVersion)
        ChatOutput("Instance tier: " .. Settings.InstanceTier)
    end

    local ResetButton = Turbine.UI.Lotro.Button()
    ResetButton:SetParent(parent)
    ResetButton:SetPosition(20, 280)
    ResetButton:SetSize(200,15)
    ResetButton:SetText("Get memory used")
    ResetButton:SetVisible(true)

    ResetButton.Click = function()
        ChatOutput(collectgarbage("count")*1024 .. " Bytes")
        ChatOutput(collectgarbage("count") .. " KB")
        ChatOutput(collectgarbage("count")/1024 .. " MB")
    end

    local ScaleBox = Turbine.UI.Lotro.TextBox()
    ScaleBox:SetParent(parent)
    ScaleBox:SetSize(50, 30)
    ScaleBox:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
    ScaleBox:SetPosition(20, 300)
    ScaleBox:SetText(Settings.UpdateDelay)

    local ScaleButton = Turbine.UI.Lotro.Button()
    ScaleButton:SetParent(parent)
    ScaleButton:SetSize(150, 30)
    ScaleButton:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
    ScaleButton:SetMouseVisible(false)
    ScaleButton:SetPosition(70, 305)
    ScaleButton:SetText("Set Update Delay")

    ScaleButton.Click = function()
        if ScaleBox:GetText() ~= "" and ScaleBox:GetText() ~= nil and ScaleBox:GetText() ~= Settings.UpdateDelay then
            Settings.UpdateDelay = tonumber(ScaleBox:GetText())
        end
    end

    local ResetButton = Turbine.UI.Lotro.Button()
    ResetButton:SetParent(parent)
    ResetButton:SetPosition(20, 330)
    ResetButton:SetSize(200,15)
    ResetButton:SetText("Get effect info on player")
    ResetButton:SetVisible(true)

    ResetButton.Click = function()
        local effectList = Player:GetEffects()

        for i = 1, effectList:GetCount(), 1 do
            ChatOutput(effectList:Get(i):GetName() .. " - Icon: " .. effectList:Get(i):GetIcon() .. " - Category: " .. effectList:Get(i):GetCategory() .. " - Duration: " .. effectList:Get(i):GetDuration() )
        end 
    end

    local ResetButton = Turbine.UI.Lotro.Button()
    ResetButton:SetParent(parent)
    ResetButton:SetPosition(20, 350)
    ResetButton:SetSize(200,15)
    ResetButton:SetText("Get backpack info")
    ResetButton:SetVisible(true)

    ResetButton.Click = function()
        local backPack = Player:GetBackpack()

        for i = 1, backPack:GetSize(), 1 do
            local item1 = backPack:GetItem(i)
            if item1 ~= nil then 
                local item = item1:GetItemInfo()
                ChatOutput(item:GetName() .. " - Icon: " .. item:GetIconImageID()) 
            end
        end
    end

    local ResetButton = Turbine.UI.Lotro.Button()
    ResetButton:SetParent(parent)
    ResetButton:SetPosition(20, 370)
    ResetButton:SetSize(200,15)
    ResetButton:SetText("Get equipment info")
    ResetButton:SetVisible(true)

    ResetButton.Click = function()
        local backPack = Player:GetEquipment()

        for i = 1, backPack:GetSize(), 1 do
            local item1 = backPack:GetItem(i)
            if item1 ~= nil then 
                local item = item1:GetItemInfo()
                ChatOutput(item:GetName() .. " - Icon: " .. item:GetIconImageID()) 
            end
        end
    end

    local ResetButton = Turbine.UI.Lotro.Button()
    ResetButton:SetParent(parent)
    ResetButton:SetPosition(20, 390)
    ResetButton:SetSize(200,15)
    ResetButton:SetText("Get skill info + Icon")
    ResetButton:SetVisible(true)

    ResetButton.Click = function()
        local tskills = Player:GetTrainedSkills()
        for i = 1, Player:GetTrainedSkills():GetCount(), 1 do
            local item = tskills:GetItem(i)--:GetSkillInfo()
            local name = item:GetSkillInfo():GetName() 
            local description = item:GetSkillInfo():GetIconImageID()

            ChatOutput(name .. " - Icon: " .. description)
        end
    end

    local ResetButton = Turbine.UI.Lotro.Button()
    ResetButton:SetParent(parent)
    ResetButton:SetPosition(20, 410)
    ResetButton:SetSize(200,15)
    ResetButton:SetText("Get skill info + description")
    ResetButton:SetVisible(true)

    ResetButton.Click = function()
        local tskills = Player:GetTrainedSkills()
        for i = 1, Player:GetTrainedSkills():GetCount(), 1 do
            local item = tskills:GetItem(i)--:GetSkillInfo()
            local name = item:GetSkillInfo():GetName() 
            local description = item:GetSkillInfo():GetDescription()

            ChatOutput(name .. " - Description: " .. description)
        end
    end

    local cBox = Turbine.UI.Lotro.CheckBox()
    cBox:SetParent(parent)
    cBox:SetPosition(20, 430)
    cBox:SetSize(200, 20)
    cBox:SetChecked(Settings.DiagnosticsEnabled)
    cBox:SetText("Diagnostic Window")
    cBox:SetVisible(true)

    cBox.CheckedChanged = function(sender,args)
        Settings.DiagnosticsEnabled = cBox:IsChecked()
    end
end

-- Reworked option panels

function PrintSettings()
    local SettingsWindow = LogWindow()
    SettingsWindow:SetVisible(true)
    for key, value in pairs(Settings) do
        local output = nil
        if value == true then
            output = "true"
        elseif value == false then
            output = "false"
        else
            output = value
        end
        SettingsWindow:Message(key .. ":" ..output, false)
    end
end