GroupEffectEvents = class(Turbine.UI.Control)
function GroupEffectEvents:Constructor()
    Turbine.UI.Control.Constructor(self)
    self.Player = Turbine.Gameplay.LocalPlayer.GetInstance()
    self.party = self.Player:GetParty()

    -- Information tables
    self.Group = {}
    self.GroupNameLUT = {}
    self.GroupRats = {}

    self.GroupSize = 0
    self.callbacks = {}
    self.addedCallbacks = {}

    self.EffectUpdate = function(sender, event, args)
        if type(event) == "function" then
            event(args)
        else
            if type(event) == "table" then
                for i = 1, #event do
                    if type(event[i]) == "function" then
                        event[i](args)
                    end
                end
            end
        end
    end

    -- Set Group Rat Infor
    self.SetGroupRats = function()
        self.GroupRats = {}
        if self.party and self.party:GetAssistTargetCount() > 0 then
            for i = 1, self.party:GetAssistTargetCount(), 1 do
                self.GroupRats[self.party:GetAssistTarget(i):GetName()] = true
            end
        end
    end

    -- Set Group Info
    self.SetGroupInformation = function()
        self.Group = {}
        self.GroupNameLUT = {}
        self.GroupRats = nil
        if self.party then
            self.GroupSize = self.party:GetMemberCount()
            for i = 1, self.GroupSize, 1 do
                self.Group[i] = {}
                self.Group[i].player = self.party:GetMember(i)
                self.Group[i].name = self.party:GetMember(i):GetName()
                self.Group[i].effects = self.Group[i].player:GetEffects()
                self.Group[i].currentEffects = {}
                self.GroupNameLUT[self.party:GetMember(i):GetName()] = true
            end
        else
            self.Group[1] = {}
            self.Group[1].player = self.Player
            self.Group[1].name = self.Player:GetName()
            self.Group[1].effects = self.Group[1].player:GetEffects()
            self.Group[1].currentEffects = {}
        end

        self.SetGroupRats()
    end

    -- Set Group Triggers
    self.SetGroupReloads = function()
        self.partyChangedCallback = AddCallback(self.Player, "PartyChanged", function()
            ReloadPlugin()
        end)
        self.raidChangedCallback = AddCallback(self.Player, "RaidChanged", function()
            ReloadPlugin()
        end)

        if self.party ~= nil then
            self.memberAddedCallback = AddCallback(self.party, "MemberAdded", function()
                ReloadPlugin()
            end)

            self.memberRemovedCallback = AddCallback(self.party, "MemberRemoved", function()
                ReloadPlugin()
            end)

            self.leaderChangedCallback = AddCallback(self.party, "LeaderChanged", function()
                ReloadPlugin()
            end)

            self.partyLinkDeadCallback = {}
            for i = 1, self.party:GetMemberCount(), 1 do
                self.partyLinkDeadCallback[i] = AddCallback(self.party:GetMember(i), "IsLinkDeadChanged", function()
                    ReloadPlugin()
                end)
            end
        end
    end

    -- Refresh group info
    self.GroupEffectsAdded = function()

        -- Ignore all group effects
        if Settings.IDKMode then return end

        -- Go through all players
        for i = 1, self.party:GetMemberCount(), 1 do
            -- Return if localplayer -> handled differently
            if self.Group[i].player:GetName() ~= self.Player:GetName() then
                self.callbacks[i] = {}
                self.callbacks[i]["EffectAdded"] = AddCallback(self.Group[i].effects, "EffectAdded", function(sender, args)
                    local effect = self.Group[i].effects:Get(args.Index)
                    if self.Group[i].currentEffects[effect:GetID()] then return end
                    self.Group[i].currentEffects[effect:GetID()] = true
                    self.EffectUpdate(self, self.EffectAdded, { ["effect"] = {
                        Name = effect:GetName(),
                        StartTime = effect:GetStartTime(),
                        Category = effect:GetCategory(), 
                        ID = effect:GetID(),
                        Description = effect:GetDescription(), 
                        Duration = effect:GetDuration(), 
                        Icon = effect:GetIcon(), 
                        IsCurable = effect:IsCurable(), 
                        IsDebuff = effect:IsDebuff(),
                    }, ["groupID"] = i, ["name"] = self.Group[i].name })
                end)
            end
        end
    end

    self.RefreshGroupData = function()
        if self.party ~= nil then
            for i = 1, self.party:GetMemberCount(), 1 do
                self.Group[i].effects = nil
                self.Group[i].currentEffects = nil
            end
            collectgarbage()
            for i = 1, self.party:GetMemberCount(), 1 do
                self.Group[i].effects = self.Group[i].player:GetEffects()
                self.Group[i].currentEffects = {}
            end
        end
        self.GroupRats = {}
        if self.party ~= nil and self.party:GetAssistTargetCount() > 0 then
            for i = 1, self.party:GetAssistTargetCount(), 1 do
                self.GroupRats[self.party:GetAssistTarget(i):GetName()] = true
            end
        end
    end

    self.outOfCombatTime = 0

    self.ExecuteUpdate = function(delta)
        self.outOfCombatTime = self.outOfCombatTime + delta
        if self.outOfCombatTime > 120 then
            self:RefreshGroupData()
            self.outOfCombatTime = 0
            if self.updateCallback then RemoveCallback(Updater, "ExecuteUpdate", self.updateCallback) end
            self.updateCallback = nil
        end
    end

    self.combatCallback = AddCallback(self.Player, "InCombatChanged", function()
        if self.Player:IsInCombat() then
            self.outOfCombatTime = 0
            if self.updateCallback then RemoveCallback(Updater, "ExecuteUpdate", self.updateCallback) end
            self.updateCallback = nil
        else
            self.updateCallback = AddCallback(Updater, "ExecuteUpdate", self.ExecuteUpdate)
        end
    end)

    self.SetGroupInformation()
    self.SetGroupReloads()

    -- Party check
    if self.party then
        self.GroupEffectsAdded()
        ChatEventHandler:SetPartyData()
    end
end

function GroupEffectEvents:RemoveGroupChangeCallbacks()
    RemoveCallback(self.Player, "PartyChanged", self.partyChangedCallback)
    self.partyChangedCallback = nil
    RemoveCallback(self.Player, "RaidChanged", self.raidChangedCallback)
    self.raidChangedCallback = nil

    if self.Party ~= nil then
        RemoveCallback(self.Party, "MemberAdded", self.memberAddedCallback)
        self.memberAddedCallback = nil

        RemoveCallback(self.Party, "MemberRemoved", self.memberRemovedCallback)
        self.memberRemovedCallback = nil

        RemoveCallback(self.Party, "LeaderChanged", self.leaderChangedCallback)
        self.leaderChangedCallback = nil

        for i = 1, self.Party:GetMemberCount(), 1 do
            RemoveCallback(self.Party:GetMember(i), "IsLinkDeadChanged", self.partyLinkDeadCallback[i])
            self.partyLinkDeadCallback[i] = nil
        end
    end
end

function GroupEffectEvents:RemoveCallbacks()
    if self.Party then
        for i = 1, self.GroupSize, 1 do
            if self.Group[i].player:GetName() ~= self.Player:GetName() then
                for key, value in pairs(self.callbacks[i]) do
                    RemoveCallback(self.Group[i].effects, key, value)
                end
            end
        end
        self:RemoveGroupChangeCallbacks()
        collectgarbage()
    end
end

function GroupEffectEvents:CheckGroupNameLUT(name)
    return (self.GroupNameLUT[name])
end

function GroupEffectEvents:CheckGroupRat(name)
    return (self.GroupRats[name])
end

function GroupEffectEvents:GetGroup()
    return self.Group
end
