PlayerHandler = class()
function PlayerHandler:Constructor()
    self.PlayerUpdate = function(sender, event, args)
        if type(event)=="function" then
            event(args)
        else
            if type(event)=="table" then
                for i=1,#event do
                    if type(event[i])=="function" then
                        event[i](args)
                    end
                end
            end
        end
    end

    self.effectLink = {}
    self.effectSatus = {}

    self.player = Turbine.Gameplay.LocalPlayer.GetInstance()
    self.pet = self.player:GetPet()
    self:Load()
end

function PlayerHandler:Load()
    self.callbacks = {}

    -- Morale callbacks
    self.callbacks["MoraleChanged"] = AddCallback(self.player, "MoraleChanged", function(sender, args)
        self.PlayerUpdate(self, self.MoraleChanged, {["Player_Morale"] = math.round(self.player:GetMorale()), ["Player_Max_Morale"] = math.round(self.player:GetMaxMorale()), ["Player_Base_Max_Morale"] = math.round(self.player:GetBaseMaxMorale()), ["Player_Morale_Percentage"] = string.format("%.2f", self.player:GetMorale() / self.player:GetMaxMorale() * 100)})
    end)

    self.callbacks["BaseMaxMoraleChanged"] = AddCallback(self.player, "BaseMaxMoraleChanged", function(sender, args)
        self.PlayerUpdate(self, self.MoraleChanged, {["Player_Morale"] = math.round(self.player:GetMorale()), ["Player_Max_Morale"] = math.round(self.player:GetMaxMorale()), ["Player_Base_Max_Morale"] = math.round(self.player:GetBaseMaxMorale()), ["Player_Morale_Percentage"] = string.format("%.2f", self.player:GetMorale() / self.player:GetMaxMorale() * 100)})
    end)

    self.callbacks["MaxMoraleChanged"] = AddCallback(self.player, "MaxMoraleChanged", function(sender, args)
        self.PlayerUpdate(self, self.MoraleChanged, {["Player_Morale"] = math.round(self.player:GetMorale()), ["Player_Max_Morale"] = math.round(self.player:GetMaxMorale()), ["Player_Base_Max_Morale"] = math.round(self.player:GetBaseMaxMorale()), ["Player_Morale_Percentage"] = string.format("%.2f", self.player:GetMorale() / self.player:GetMaxMorale() * 100)})
    end)
    
    -- Temporary morale callbacks
    self.callbacks["TemporaryMoraleChanged"] = AddCallback(self.player, "TemporaryMoraleChanged", function(sender, args)
        self.PlayerUpdate(self, self.TemporaryMoraleChanged, {["Player_Temporary_Morale"] = math.round(self.player:GetTemporaryMorale()), ["Player_Max_Temporary_Morale"] = math.round(self.player:GetMaxTemporaryMorale())})
    end)

    self.callbacks["MaxTemporaryMoraleChanged"] = AddCallback(self.player, "MaxTemporaryMoraleChanged", function(sender, args)
        self.PlayerUpdate(self, self.TemporaryMoraleChanged, {["Player_Temporary_Morale"] = math.round(self.player:GetTemporaryMorale()), ["Player_Max_Temporary_Morale"] = math.round(self.player:GetMaxTemporaryMorale())})
    end)

    -- Power callbacks
    self.callbacks["PowerChanged"] = AddCallback(self.player, "PowerChanged", function(sender, args)
        self.PlayerUpdate(self, self.PowerChanged, {["Player_Power"] = math.round(self.player:GetPower()), ["Player_Max_Power"] = math.round(self.player:GetMaxPower()), ["Player_Base_Max_Power"] = math.round(self.player:GetBaseMaxPower()), ["Player_Power_Percentage"] = string.format("%.2f", self.player:GetPower() / self.player:GetMaxPower() * 100)})
    end)

    self.callbacks["MaxPowerChanged"] = AddCallback(self.player, "MaxPowerChanged", function(sender, args)
        self.PlayerUpdate(self, self.PowerChanged, {["Player_Power"] = math.round(self.player:GetPower()), ["Player_Max_Power"] = math.round(self.player:GetMaxPower()), ["Player_Base_Max_Power"] = math.round(self.player:GetBaseMaxPower()), ["Player_Power_Percentage"] = string.format("%.2f", self.player:GetPower() / self.player:GetMaxPower() * 100)})
    end)

    self.callbacks["BaseMaxPowerChanged"] = AddCallback(self.player, "BaseMaxPowerChanged", function(sender, args)
        self.PlayerUpdate(self, self.PowerChanged, {["Player_Power"] = math.round(self.player:GetPower()), ["Player_Max_Power"] = math.round(self.player:GetMaxPower()), ["Player_Base_Max_Power"] = math.round(self.player:GetBaseMaxPower()), ["Player_Power_Percentage"] = string.format("%.2f", self.player:GetPower() / self.player:GetMaxPower() * 100)})
    end)

    -- Temporary power callbacks
    self.callbacks["TemporaryPowerChanged"] = AddCallback(self.player, "TemporaryPowerChanged", function(sender, args)
        self.PlayerUpdate(self, self.TemporaryPowerChanged, {["Player_Temporary_Power"] = math.round(self.player:GetTemporaryPower()), ["Player_Max_Temporary_Power"] = math.round(self.player:GetMaxTemporaryPower())})
    end)

    self.callbacks["MaxTemporaryPowerChanged"] = AddCallback(self.player, "MaxTemporaryPowerChanged", function(sender, args)
        self.PlayerUpdate(self, self.TemporaryPowerChanged, {["Player_Temporary_Power"] = math.round(self.player:GetTemporaryPower()), ["Player_Max_Temporary_Power"] = math.round(self.player:GetMaxTemporaryPower())})
    end)

    -- Level
    self.callbacks["LevelChanged"] = AddCallback(self.player, "LevelChanged", function(sender, args)
        self.PlayerUpdate(self, self.LevelChanged, {["Player_Level"] = self.player:GetLevel()})
    end)

    -- Combat
    self.callbacks["InCombatChanged"] = AddCallback(self.player, "InCombatChanged", function(sender, args)
        self.PlayerUpdate(self, self.CombatChanged, {["Player_Is_In_Combat"] = self.player:IsInCombat()})
    end)

    self.callbacks["PetChanged"] = AddCallback(self.player, "PetChanged", function(sender, args)
        if self.pet then
            self:RemovePetCallbacks()
        end
        self.pet = self.player:GetPet()
        if self.pet then
            self.PlayerUpdate(self, self.PetInfo, {["PetStatus"] = true,
            ["Pet_Morale"] = math.round(self.pet:GetMorale()), ["Pet_Max_Morale"] = math.round(self.pet:GetMaxMorale()), ["Pet_Base_Max_Morale"] = math.round(self.pet:GetBaseMaxMorale()), ["Pet_Morale_Percentage"] = string.format("%.2f", self.pet:GetMorale() / self.pet:GetMaxMorale() * 100),
            ["Pet_Temporary_Morale"] = math.round(self.pet:GetTemporaryMorale()), ["Pet_Max_Temporary_Morale"] = math.round(self.pet:GetMaxTemporaryMorale()),
            ["Pet_Name"] = self.pet:GetName() })
            self:SetPetCallbacks()
        else
            self.PlayerUpdate(self, self.PetInfo, {["PetStatus"] = false})
        end
    end)

    -- Wound handling
    for key, value in pairs(Turbine.Gameplay.EffectCategory) do
        self.effectLink[value] = key
        self.effectSatus[value] = 0
    end
    self.effectSatus[0] = 0

    for i = 1, self.player:GetEffects():GetCount() do
        local effect = self.player:GetEffects():Get(i)
        if effect:IsCurable() then
            local category = effect:GetCategory()
            if self.effectSatus[category] then
                self.effectSatus[category] = self.effectSatus[category] + 1
            end
        end
    end

    self.effectAddedCallback = AddCallback(PlayerEffectsHandler, "EffectAdded", 
    function(effect)
        if effect:IsCurable() then
            local category = effect:GetCategory()
            if self.effectSatus[category] then
                if self.effectSatus[category] == 0 then
                    self.PlayerUpdate(self, self.CategoryAdded, {["Type"] = self.effectLink[category]})
                end
                self.effectSatus[category] = self.effectSatus[category] + 1
            end
        end
    end)

    self.effectRemovedCallback = AddCallback(PlayerEffectsHandler, "EffectRemoved", 
    function(effect)
        if effect:IsCurable() then
            local category = effect:GetCategory()
            if self.effectSatus[category] then
                self.effectSatus[category] = self.effectSatus[category] - 1
                if self.effectSatus[category] == 0 then
                    self.PlayerUpdate(self, self.CategoryRemoved, {["Type"] = self.effectLink[category]})
                end
            end
        end
    end)
end

function PlayerHandler:Unload()
    if self.callbacks then
        for key, value in pairs(self.callbacks) do
            RemoveCallback(self.player, key, value)
            self.callbacks[key] = nil
        end
    end

    RemoveCallback(PlayerEffectsHandler, "EffectAdded", self.effectAddedCallback)
    self.effectAddedCallbacks = nil

    RemoveCallback(PlayerEffectsHandler, "EffectRemoved", self.effectRemovedCallback)
    self.effectRemovedCallbacks = nil

    self.effectLink = nil
    self.effectSatus = nil
end

function PlayerHandler:CategoryApplied(category)
    if self.effectSatus and self.effectSatus[category] then return self.effectSatus[category] > 0 else return false end
end

function PlayerHandler:SetPetCallbacks()
    self.petCallbacks = {}

    -- Morale callbacks
    self.petCallbacks["MoraleChanged"] = AddCallback(self.pet, "MoraleChanged", function(sender, args)
        self.PlayerUpdate(self, self.PetMoraleChanged, {["Pet_Morale"] = math.round(self.pet:GetMorale()), ["Pet_Max_Morale"] = math.round(self.pet:GetMaxMorale()), ["Pet_Base_Max_Morale"] = math.round(self.pet:GetBaseMaxMorale()), ["Pet_Morale_Percentage"] = string.format("%.2f", self.pet:GetMorale() / self.pet:GetMaxMorale() * 100)})
    end)

    self.petCallbacks["MaxMoraleChanged"] = AddCallback(self.pet, "MaxMoraleChanged", function(sender, args)
        self.PlayerUpdate(self, self.PetMoraleChanged, {["Pet_Morale"] = math.round(self.pet:GetMorale()), ["Pet_Max_Morale"] = math.round(self.pet:GetMaxMorale()), ["Pet_Base_Max_Morale"] = math.round(self.pet:GetBaseMaxMorale()), ["Pet_Morale_Percentage"] = string.format("%.2f", self.pet:GetMorale() / self.pet:GetMaxMorale() * 100)})
    end)
    
    -- Temporary morale callbacks
    self.petCallbacks["TemporaryMoraleChanged"] = AddCallback(self.pet, "TemporaryMoraleChanged", function(sender, args)
        self.PlayerUpdate(self, self.PetTemporaryMoraleChanged, {["Pet_Temporary_Morale"] = math.round(self.pet:GetTemporaryMorale()), ["Pet_Max_Temporary_Morale"] = math.round(self.pet:GetMaxTemporaryMorale())})
    end)

    self.petCallbacks["MaxTemporaryMoraleChanged"] = AddCallback(self.pet, "MaxTemporaryMoraleChanged", function(sender, args)
        self.PlayerUpdate(self, self.PetTemporaryMoraleChanged, {["Pet_Temporary_Morale"] = math.round(self.pet:GetTemporaryMorale()), ["Pet_Max_Temporary_Morale"] = math.round(self.pet:GetMaxTemporaryMorale())})
    end)

    -- Name changed
    self.petCallbacks["NameChanged"] = AddCallback(self.pet, "NameChanged", function(sender, args)
        self.PlayerUpdate(self, self.PetInfo, {["PetStatus"] = true,
        ["Pet_Morale"] = math.round(self.pet:GetMorale()), ["Pet_Max_Morale"] = math.round(self.pet:GetMaxMorale()), ["Pet_Base_Max_Morale"] = math.round(self.pet:GetBaseMaxMorale()), ["Pet_Morale_Percentage"] = string.format("%.2f", self.pet:GetMorale() / self.pet:GetMaxMorale() * 100),
        ["Pet_Temporary_Morale"] = math.round(self.pet:GetTemporaryMorale()), ["Pet_Max_Temporary_Morale"] = math.round(self.pet:GetMaxTemporaryMorale()),
        ["Pet_Name"] = self.pet:GetName() })
    end)
end

function PlayerHandler:RemovePetCallbacks()
    if self.petCallbacks then
        for key, value in pairs(self.petCallbacks) do
            RemoveCallback(self.pet, key, value)
            self.petCallbacks[key] = nil
        end
    end
    self.petCallbacks = nil
end

function PlayerHandler:GetInfo()
    return {
        ["Player_Morale"] = math.round(self.player:GetMorale()), ["Player_Max_Morale"] = math.round(self.player:GetMaxMorale()), ["Player_Base_Max_Morale"] = math.round(self.player:GetBaseMaxMorale()),
        ["Player_Temporary_Morale"] = math.round(self.player:GetTemporaryMorale()), ["Player_Max_Temporary_Morale"] = math.round(self.player:GetMaxTemporaryMorale()),
        ["Player_Power"] = math.round(self.player:GetPower()), ["Player_Max_Power"] = math.round(self.player:GetMaxPower()), ["Player_Base_Max_Power"] = math.round(self.player:GetBaseMaxPower()),
        ["Player_Temporary_Power"] = math.round(self.player:GetTemporaryPower()), ["Player_Max_Temporary_Power"] = math.round(self.player:GetMaxTemporaryPower()),
        ["Player_Level"] = self.player:GetLevel(), ["Player_Is_In_Combat"] = self.player:IsInCombat(),
        ["Player_Morale_Percentage"] = string.format("%.2f", self.player:GetMorale() / self.player:GetMaxMorale() * 100),
        ["Player_Power_Percentage"] = string.format("%.2f", self.player:GetPower() / self.player:GetMaxPower() * 100),

    }
end

function PlayerHandler:GetPetInfo()
    if self.pet then
        return {["PetStatus"] = true,
        ["Pet_Morale"] = math.round(self.pet:GetMorale()), ["Pet_Max_Morale"] = math.round(self.pet:GetMaxMorale()), ["Pet_Base_Max_Morale"] = math.round(self.pet:GetBaseMaxMorale()), ["Pet_Morale_Percentage"] = string.format("%.2f", self.pet:GetMorale() / self.pet:GetMaxMorale() * 100),
        ["Pet_Temporary_Morale"] = math.round(self.pet:GetTemporaryMorale()), ["Pet_Max_Temporary_Morale"] = math.round(self.pet:GetMaxTemporaryMorale()),
        ["Pet_Name"] = self.pet:GetName() }
    else
        return {["PetStatus"] = false}
    end
end