RoleHandler = class(Turbine.UI.Control)
function RoleHandler:Constructor()
    Turbine.UI.Control.Constructor(self)

    self.lastRoleSwap = 0

    self.player = Turbine.Gameplay.LocalPlayer.GetInstance()

    self.Role = 0

    self.Line = 0

    self.Roles = {
        [0] = "Undefined",
        [1] = "Tank",
        [2] = "Heal",
        [3] = "Support",
        [4] = "DPS",
    }

    self.RoleNames = {
        ["Undefined"] = 0,
        ["Tank"] = 1,
        ["Heal"] = 2,
        ["Support"] = 3,
        ["DPS"] = 4,
    }

    self.Lines = {
        [1] = "Blue",
        [2] = "Red",
        [3] = "Yellow",
    }

    self.LoadStatus = LoadTable(Turbine.DataScope.Character, "PrimeCharacterRoleData")

    if self.LoadStatus == nil then
        self.LoadStatus = {
            ["Role"] = 0,
            ["Line"] = 0,
        }
    else
        self.Role = self.LoadStatus.Role
        self.Line = self.LoadStatus.Line
    end

    -- Roles
    -- 0 undefined
    -- 1 tank
    -- 2 heal
    -- 3 support
    -- 4 dps

    self.playerClass = self.player:GetClass()

    local tskills = self.player:GetTrainedSkills()
    for i = 1, self.player:GetTrainedSkills():GetCount(), 1 do
        self:ParseTraits(tskills:GetItem(i):GetSkillInfo():GetName())
    end

    EventWindows:SetRole(self.Role)
    
    self.Reload = true

    self.lastRoleSwap = 0
end

function RoleHandler:NameToRoleID(name)
    return self.RoleNames[name]
end

function RoleHandler:GetRole()
    return self.Role
end

function RoleHandler:GetRoleText()
    return self.Roles[self.Role]
end

function RoleHandler:GetLine()
    return self.Line
end

function RoleHandler:GetLineText()
    return self.Lines[self.Line]
end


function RoleHandler:GetAvailableRoles()
    return self.Roles
end

function RoleHandler:SetRole(role, line)
    if self.lastRoleSwap + 5 > Turbine.Engine.GetGameTime() then return end

    SkillEventHandler:ReloadWithDelay()

    self:GetRoleSpecificItems(false)

    self.Role = role

    self.Line = line

    if self.Reload then SkillsHandler:ReloadBarsWithDelay() end

    if ClassTools then ClassTools:ReloadToolWithDelay() end

    if Auras then Auras:ReloadAurasWithDelay() end

    self:GetRoleSpecificItems(true)

    self.lastRoleSwap = Turbine.Engine.GetGameTime()

    if PrimeUIHandlerObject then PrimeUIHandlerObject:RecheckPermissions() end

    self.LoadStatus.Role = self.Role
    self.LoadStatus.Line = self.Line

    ChatOutput("Set Role To " .. self.Roles[self.Role]) 
end

function RoleHandler:ParseTraits(message)
    if self.playerClass == Turbine.Gameplay.Class.Beorning then
        
        -- Beorning
        if string.find(message, "Counter") then self:SetRole(1, 1); self.Line = 1 end -- blue
        if string.find(message, "Bash") then self:SetRole(4, 2); self.Line = 2 end -- red
        if string.find(message, "Mark of Grimbeorn") then self:SetRole(2, 3); self.Line = 3 end -- yellow

        -- Last set traits
        if string.find(message, "Animalistic") then self:SetRole(1, 1); self.Line = 1 end -- blue
        if string.find(message, "Wrathful") then self:SetRole(4, 2); self.Line = 2 end -- red
        if string.find(message, "Bond of Trust") then self:SetRole(2, 3); self.Line = 3 end -- yellow

    elseif self.playerClass == Turbine.Gameplay.Class.Brawler then

        -- Brawler
        if string.find(message, "Iron Will") then self:SetRole(1, 1); self.Line = 1 end -- blue
        if string.find(message, "Pummel") then self:SetRole(4, 2); self.Line = 2 end -- red

        -- Last set traits
        if string.find(message, "Innate Strength: Inner Resilience") then self:SetRole(1, 1); self.Line = 1 end -- blue
        if string.find(message, "Innate Strength: Raw Power") then self:SetRole(4, 2); self.Line = 2 end -- red

    elseif self.playerClass == Turbine.Gameplay.Class.Burglar then 

        -- Burglar
        if string.find(message, "Lucky Strike") then self:SetRole(4, 1); self.Line = 1 end -- blue
        if string.find(message, "Knives Out") then self:SetRole(4, 2); self.Line = 2 end -- red
        if string.find(message, "Mischievous Glee") then self:SetRole(3, 3); self.Line = 3 end -- yellow

        -- Last set traits
        if string.find(message, "Loaded Dice") then self:SetRole(4, 1); self.Line = 1 end -- blue
        if string.find(message, "Improved Sneak") then self:SetRole(4, 2); self.Line = 2 end -- red
        if string.find(message, "Confound the Fools") then self:SetRole(3, 3); self.Line = 3 end -- yellow

    elseif self.playerClass == Turbine.Gameplay.Class.Champion then

        -- Champ
        if string.find(message, "Sudden Defence") then self:SetRole(1, 1); self.Line = 1 end -- blue
        if string.find(message, "Devastating Strike") then self:SetRole(4, 2); self.Line = 2 end -- red
        if string.find(message, "Rend") then self:SetRole(4, 3); self.Line = 3 end -- yellow

        -- Last set traits
        if string.find(message, "Invincible") then self:SetRole(1, 1); self.Line = 1 end -- blue
        if string.find(message, "Continuous Blood Rage") then self:SetRole(4, 2); self.Line = 2 end -- red
        if string.find(message, "Born For Combat") then self:SetRole(4, 3); self.Line = 3 end -- yellow

    elseif self.playerClass == Turbine.Gameplay.Class.Captain then

        -- Captain
        if string.find(message, "Valiant Strike") then self:SetRole(2, 1); self.Line = 1 end -- blue
        if string.find(message, "Shadow's Lament") then self:SetRole(4, 2); self.Line = 2 end -- red
        if string.find(message, "Elendil's Roar") then self:SetRole(1, 3); self.Line = 3 end -- yellow

        -- Last set traits
        if string.find(message, "Inspiriting Presence") then self:SetRole(2, 1); self.Line = 1 end -- blue
        if string.find(message, "Master of War") then self:SetRole(4, 2); self.Line = 2 end -- red
        if string.find(message, "Exemplar") then self:SetRole(1, 3); self.Line = 3 end -- yellow

    --elseif self.playerClass == Turbine.Gameplay.Class.Corsair then


    elseif self.playerClass == Turbine.Gameplay.Class.Guardian then

        -- Guardian
        if string.find(message, "Litany of Defiance") then self:SetRole(1, 1); self.Line = 1 end -- blue
        if string.find(message, "Overwhelm") then self:SetRole(4, 2); self.Line = 2 end -- red
        if string.find(message, "Take to Heart") then self:SetRole(1, 3); self.Line = 3 end -- yellow

        -- Last set traits
        if string.find(message, "Shield the People") then self:SetRole(1, 1); self.Line = 1 end -- blue
        if string.find(message, "Prey on the Weak") then self:SetRole(4, 2); self.Line = 2 end -- red
        if string.find(message, "Incapacitation") then self:SetRole(1, 3); self.Line = 3 end -- yellow

    elseif self.playerClass == Turbine.Gameplay.Class.Hunter then

        -- Hunter
        if string.find(message, "Barrage") then self:SetRole(4, 1); self.Line = 1 end -- blue
        if string.find(message, "Pinning Shot") then self:SetRole(4, 2); self.Line = 2 end -- red
        if string.find(message, "Decoy") then self:SetRole(4, 3); self.Line = 3 end -- yellow

        -- Last set traits
        if string.find(message, "Intensified Fire") then self:SetRole(4, 1); self.Line = 1 end -- blue
        if string.find(message, "Marksman") then self:SetRole(4, 2); self.Line = 2 end -- red
        if string.find(message, "Sturdy Traps") then self:SetRole(4, 3); self.Line = 3 end -- yellow
        
    elseif self.playerClass == Turbine.Gameplay.Class.LoreMaster then

        -- LoreMaster
        if string.find(message, "Sic 'em!") then self:SetRole(3, 1); self.Line = 1 end -- blue
        if string.find(message, "Ents go to War") then self:SetRole(4, 2); self.Line = 2 end -- red
        if string.find(message, "Test of Will") then self:SetRole(3, 3); self.Line = 3 end -- yellow

        -- Last set traits
        if string.find(message, "Tutelage of the Brown Wizard") then self:SetRole(3, 1); self.Line = 1 end -- blue
        if string.find(message, "Flame of Anor") then self:SetRole(4, 2); self.Line = 2 end -- red
        if string.find(message, "Force of Will") then self:SetRole(3, 3); self.Line = 3 end -- yellow

    elseif self.playerClass == Turbine.Gameplay.Class.Minstrel then

        -- Minstrel
        if string.find(message, "Soliloquy Of Spirit") then self:SetRole(2, 1); self.Line = 1 end -- blue
        if string.find(message, "Call to Fate") then self:SetRole(4, 2); self.Line = 2 end -- red

        -- Last set traits
        if string.find(message, "Chord of Legend") then self:SetRole(2, 1); self.Line = 1 end -- blue
        if string.find(message, "Encore") then self:SetRole(4, 2); self.Line = 2 end -- red

    elseif self.playerClass == Turbine.Gameplay.Class.RuneKeeper then

        -- Runekeeper
        if string.find(message, "Writ of Health") then self:SetRole(2, 1); self.Line = 1 end -- blue
        if string.find(message, "Smouldering Wrath") then self:SetRole(4, 2); self.Line = 2 end -- red
        if string.find(message, "Writ of Lightning") then self:SetRole(4, 3); self.Line = 3 end -- yellow

        -- Last set traits
        if string.find(message, "Our Fates Entwined") then self:SetRole(2, 1); self.Line = 1 end -- blue
        if string.find(message, "Seething Truth") then self:SetRole(4, 2); self.Line = 2 end -- red
        if string.find(message, "Static Surge") then self:SetRole(4, 3); self.Line = 3 end -- yellow

    elseif self.playerClass == Turbine.Gameplay.Class.Warden then

        -- Warden
        if string.find(message, "Warning Shot") then self:SetRole(1, 1); self.Line = 1 end -- blue
        if string.find(message, "Recklessness") then self:SetRole(4, 2); self.Line = 2 end -- red
        --if string.find(message, "Improved Hampering Javelin") then self:SetRole(4, 3); self.Line = 3 end -- yellow

        -- Last set traits
        if string.find(message, "Desperate Combat") then self:SetRole(1, 1); self.Line = 1 end -- blue
        if string.find(message, "Martial Fury") then self:SetRole(4, 2); self.Line = 2 end -- red
        --if string.find(message, "Snap Shot") then self:SetRole(4, 3); self.Line = 3 end -- yellow
        
    end
end

function RoleHandler:GetRoleSpecificItems(apply)
    if true then return end
    -- Roles
    -- 0 undefined
    -- 1 tank
    -- 2 heal
    -- 3 support
    -- 4 dps
    
    if self.Role == 1 then

    elseif self.Role == 2 then
        
    elseif self.Role == 3 then

    elseif self.Role == 4 then
    end
end

function RoleHandler:Unload()
    SaveTable(Turbine.DataScope.Character, "PrimeCharacterRoleData", self.LoadStatus)
end