TargetEventHandling = class(Turbine.UI.Control)
function TargetEventHandling:Constructor()
    Turbine.UI.Control.Constructor(self)

    self.player = Turbine.Gameplay.LocalPlayer.GetInstance()

    self.targetCallbackCount = 0
    self.targetInfoCallbackCount = 0
    self.targetHealthCallbackCount = 0
    self.targetTemporaryHealthCallbackCount = 0
    self.targetPowerCallbackCount = 0
    self.targetEffectCallbackCount = 0

    self.target = nil

    self.effectIDs = {}
    self.callbacks = {}
    self.targetCallbacks = {}
    self.targetEffectCallbacks = {}

    self.TargetUpdate = function(sender, event, args)
        if type(event)=="function" then
            event(args)
        else
            if type(event)=="table" then
                for i=1,#event do
                    if type(event[i])=="function" then
                        event[i](args)
                    end
                end
            end
        end
    end
end

function TargetEventHandling:TargetEvents()
    self:Clear()

    self.target = self.player:GetTarget()

    if self.target == self.player then
        self.TargetUpdate(self, self.TargetInfo, nil)
        self:Clear()
    end

    if self.target then 
        
        self:SetWantsUpdates(true)

        if self.targetHealthCallbackCount > 0 then
            self:AddHealthCallbackInternal()
        end

        if self.targetTemporaryHealthCallbackCount > 0 then
            self:AddTemporaryHealthCallbackInternal()
        end

        if self.targetPowerCallbackCount > 0 then
            self:AddPowerCallbackInternal()
        end

        if self.target.GetEffects then
            if self.targetInfoCallbackCount > 0 then
                self.TargetUpdate(self, self.TargetInfo, {  ["Name"] = self.target:GetName(), ["Level"] = self.target:GetLevel(),
                                                            ["Morale"] = self.target:GetMorale(), ["MaxMorale"] = self.target:GetMaxMorale(), 
                                                            ["TemporaryMorale"] = self.target:GetTemporaryMorale(), ["TemporaryMaxMorale"] = self.target:GetMaxTemporaryMorale(), 
                                                            ["Power"] = self.target:GetPower(), ["MaxPower"] = self.target:GetMaxPower()})
            end

            if self.targetEffectCallbackCount > 0 then
                self.targetEffects = self.target:GetEffects()
                for i = 1, self.targetEffects:GetCount() do
                    local effect = self.targetEffects:Get(i)
                    self.effectIDs[effect:GetID()] = true
                    self.TargetUpdate(self, self.EffectInfo, effect)
                end
                self:AddEffectCallbackInternal()
            end
        else
            if self.targetInfoCallbackCount > 0 then
                self.TargetUpdate(self, self.TargetInfo, {["Name"] = self.target:GetName()})
            end
        end
    else
        if self.targetInfoCallbackCount > 0 then
            self.TargetUpdate(self, self.TargetInfo, nil)
        end
    end
end

function TargetEventHandling:Update()
    if self.target and self.target.GetMorale and self.target:GetMorale() > 0 then
        if self.targetHealthCallbackCount > 0 then
            self:AddHealthCallbackInternal()
        end

        if self.targetEffectCallbackCount > 0 then
            self.targetEffects = self.target:GetEffects()
            for i = 1, self.targetEffects:GetCount() do
                local effect = self.targetEffects:Get(i)
                self.effectIDs[effect:GetID()] = true
                self.TargetUpdate(self, self.EffectInfo, effect)
            end
            self:AddEffectCallbackInternal()
        end

        if self.targetInfoCallbackCount > 0 then
            self.TargetUpdate(self, self.TargetInfo, {  ["Name"] = self.target:GetName(), ["Level"] = self.target:GetLevel(),
                                                        ["Morale"] = self.target:GetMorale(), ["MaxMorale"] = self.target:GetMaxMorale(), 
                                                        ["TemporaryMorale"] = self.target:GetTemporaryMorale(), ["TemporaryMaxMorale"] = self.target:GetMaxTemporaryMorale(), 
                                                        ["Power"] = self.target:GetPower(), ["MaxPower"] = self.target:GetMaxPower()})
            if self.targetHealthCallbackCount > 0 then
                self.TargetUpdate(self, self.MoraleInfo, self.target:GetMorale())
            end

            if self.targetTemporaryHealthCallbackCount > 0 then
                self.TargetUpdate(self, self.TemporaryMoraleInfo, self.target:GetTemporaryMorale())
            end

            if self.targetPowerCallbackCount > 0 then
                self.TargetUpdate(self, self.PowerInfo, self.target:GetPower())
            end
        end
        self:SetWantsUpdates(false)
    end
end

function TargetEventHandling:Clear()
    self:RemoveHealthCallbackInternal()
    self:RemoveTemporaryHealthCallbackInternal()
    self:RemovePowerCallbackInternal()
    self:RemoveEffectCallbackInternal()

    self.target = nil
    self.targetEffects = nil
    self.effectIDs = nil

    collectgarbage()

    self.effectIDs = {}
end

-----------------------------------------------------------------------------------------
-- Health callbacks
-----------------------------------------------------------------------------------------

function TargetEventHandling:AddHealthCallback(func)
    
    self:HandleTargetCallbackInternal(true)

    self.targetCallbackCount = self.targetCallbackCount + 1
    self.targetHealthCallbackCount = self.targetHealthCallbackCount + 1

    self:AddHealthCallbackInternal()

    return AddCallback(self, "MoraleInfo", func)
end


function TargetEventHandling:RemoveHealthCallback(callback) 
    if callback then
        self.targetCallbackCount = self.targetCallbackCount - 1
        self.targetHealthCallbackCount = self.targetHealthCallbackCount - 1

        if self.targetHealthCallbackCount == 0 then
            self:RemoveHealthCallbackInternal()
        end

        if self.targetCallbackCount == 0 then
            self:HandleTargetCallbackInternal(false)
        end

        RemoveCallback(self, "MoraleInfo", callback)
    end
end

function TargetEventHandling:AddHealthCallbackInternal()
    if self.target and not(self.targetCallbacks["MoraleChanged"]) then
        self.targetCallbacks["MoraleChanged"] = AddCallback(self.target, "MoraleChanged", function(sender, args)
                self.TargetUpdate(self, self.MoraleInfo, self.target:GetMorale())
            end)
    end
end

function TargetEventHandling:RemoveHealthCallbackInternal()
    if self.targetCallbacks["MoraleChanged"] then
        RemoveCallback(self.target, "MoraleChanged", self.targetCallbacks["MoraleChanged"])
        self.targetCallbacks["MoraleChanged"] = nil
    end
end

-----------------------------------------------------------------------------------------
-- Temp callbacks
-----------------------------------------------------------------------------------------

function TargetEventHandling:AddTemporaryHealthCallback(func)
    
    self:HandleTargetCallbackInternal(true)

    self.targetCallbackCount = self.targetCallbackCount + 1
    self.targetTemporaryHealthCallbackCount = self.targetTemporaryHealthCallbackCount + 1

    self:AddTemporaryHealthCallbackInternal()

    return AddCallback(self, "TemporaryMoraleInfo", func)
end


function TargetEventHandling:RemoveTemporaryHealthCallback(callback) 
    if callback then
        self.targetCallbackCount = self.targetCallbackCount - 1
        self.targetTemporaryHealthCallbackCount = self.targetTemporaryHealthCallbackCount - 1

        if self.targetTemporaryHealthCallbackCount == 0 then
            self:RemoveTemporaryHealthCallbackInternal()
        end

        if self.targetCallbackCount == 0 then
            self:HandleTargetCallbackInternal(false)
        end

        RemoveCallback(self, "TemporaryMoraleInfo", callback)
    end
end

function TargetEventHandling:AddTemporaryHealthCallbackInternal()
    if self.target and not(self.targetCallbacks["TemporaryMoraleChanged"]) then
        self.targetCallbacks["TemporaryMoraleChanged"] = AddCallback(self.target, "TemporaryMoraleChanged", function(sender, args)
                self.TargetUpdate(self, self.TemporaryMoraleInfo, self.target:GetTemporaryMorale())
            end)
    end
end

function TargetEventHandling:RemoveTemporaryHealthCallbackInternal()
    if self.targetCallbacks["TemporaryMoraleChanged"] then
        RemoveCallback(self.target, "TemporaryMoraleChanged", self.targetCallbacks["TemporaryMoraleChanged"])
        self.targetCallbacks["TemporaryMoraleChanged"] = nil
    end
end

-----------------------------------------------------------------------------------------
-- Power callbacks
-----------------------------------------------------------------------------------------

function TargetEventHandling:AddPowerCallback(func)
    
    self:HandleTargetCallbackInternal(true)

    self.targetCallbackCount = self.targetCallbackCount + 1
    self.targetPowerCallbackCount = self.targetPowerCallbackCount + 1

    self:AddPowerCallbackInternal()

    return AddCallback(self, "PowerInfo", func)
end


function TargetEventHandling:RemovePowerCallback(callback) 
    if callback then
        self.targetCallbackCount = self.targetCallbackCount - 1
        self.targetPowerCallbackCount = self.targetPowerCallbackCount - 1

        if self.targetPowerCallbackCount == 0 then
            self:RemovePowerCallbackInternal()
        end

        if self.targetCallbackCount == 0 then
            self:HandleTargetCallbackInternal(false)
        end

        RemoveCallback(self, "PowerInfo", callback)
    end
end

function TargetEventHandling:AddPowerCallbackInternal()
    if self.target and not(self.targetCallbacks["PowerChanged"]) then
        self.targetCallbacks["PowerChanged"] = AddCallback(self.target, "PowerChanged", function(sender, args)
                self.TargetUpdate(self, self.PowerInfo, self.target:GetPower())
            end)
    end
end

function TargetEventHandling:RemovePowerCallbackInternal()
    if self.targetCallbacks["PowerChanged"] then
        RemoveCallback(self.target, "PowerChanged", self.targetCallbacks["PowerChanged"])
        self.targetCallbacks["PowerChanged"] = nil
    end
end


-----------------------------------------------------------------------------------------
-- Target info callbacks
-----------------------------------------------------------------------------------------

function TargetEventHandling:AddTargetInfoCallback(callback)
    self:HandleTargetCallbackInternal(true)

    self.targetCallbackCount = self.targetCallbackCount + 1
    self.targetInfoCallbackCount = self.targetInfoCallbackCount + 1

    return AddCallback(self, "TargetInfo", callback)
end

function TargetEventHandling:RemoveTargetInfoCallback(callback)
    if callback then
        self.targetCallbackCount = self.targetCallbackCount - 1
        self.targetInfoCallbackCount = self.targetInfoCallbackCount - 1

        if self.targetCallbackCount == 0 then
            self:HandleTargetCallbackInternal(false)
        end

        RemoveCallback(self, "TargetInfo", callback)
    end
end

function TargetEventHandling:HandleTargetCallbackInternal(value)
    if value and not(self.callbacks["TargetChanged"]) then
        self.callbacks["TargetChanged"] = AddCallback(self.player, "TargetChanged", function(sender, args) self:TargetEvents() end)
    elseif not(value) and self.callbacks["TargetChanged"] then
        RemoveCallback(self.player, "TargetChanged", self.callbacks["TargetChanged"])
        self.callbacks["TargetChanged"] = nil
    end
end

-----------------------------------------------------------------------------------------
-- Effect callbacks
-----------------------------------------------------------------------------------------


function TargetEventHandling:AddEffectCallback(func)
    
    self:HandleTargetCallbackInternal(true)

    self.targetCallbackCount = self.targetCallbackCount + 1
    self.targetEffectCallbackCount = self.targetEffectCallbackCount + 1

    self:AddEffectCallbackInternal()

    return AddCallback(self, "EffectInfo", func)
end


function TargetEventHandling:RemoveEffectCallback(callback) 
    if callback then
        self.targetCallbackCount = self.targetCallbackCount - 1
        self.targetEffectCallbackCount = self.targetEffectCallbackCount - 1

        if self.targetEffectCallbackCount == 0 then
            self:RemoveEffectCallbackInternal()
        end

        if self.targetCallbackCount == 0 then
            self:HandleTargetCallbackInternal(false)
        end

        RemoveCallback(self, "EffectInfo", callback)
    end
end

function TargetEventHandling:AddEffectCallbackInternal()
    if self.target and self.targetEffects and not(self.targetEffectCallbacks["EffectAdded"]) then
        self.targetEffectCallbacks["EffectAdded"] = AddCallback(self.targetEffects, "EffectAdded", function(sender, args)
            if self.targetEffects then
                local effect = self.targetEffects:Get(args.Index)
                if not self.effectIDs[effect:GetID()] then
                    self.TargetUpdate(self, self.EffectInfo, 
                    {
                        Name = effect:GetName(),
                        StartTime = effect:GetStartTime(),
                        Category = effect:GetCategory(), 
                        ID = effect:GetID(),
                        Description = effect:GetDescription(), 
                        Duration = effect:GetDuration(), 
                        Icon = effect:GetIcon(), 
                        IsCurable = effect:IsCurable(), 
                        IsDebuff = effect:IsDebuff()})
                    self.effectIDs[effect:GetID()] = true
                end
            end
        end)
    end
end

function TargetEventHandling:RemoveEffectCallbackInternal()
    if self.targetEffectCallbacks["EffectAdded"] then
        RemoveCallback(self.targetEffects, "EffectAdded", self.targetEffectCallbacks["EffectAdded"])
        self.targetEffectCallbacks["EffectAdded"] = nil
    end
end


function TargetEventHandling:Unload()
    self:RemoveHealthCallback()
    self:HandleTargetCallback(false)
    self:Clear()
    self:SetParent(nil)
end

function TargetEventHandling:GetActives()
    return self.targetCallbackCount, self.targetInfoCallbackCount, self.targetHealthCallbackCount, self.targetTemporaryHealthCallbackCount, self.targetPowerCallbackCount, self.targetEffectCallbackCount
end