function ConvertEntry(entry)
    if type(entry) == "number" then 
        return ("<num>" .. tostring(entry)) 
    end
    if type(entry) == "table" then return ConvertTable(entry) end
    return entry
end

local seperator = (tonumber("1,0") == 1) and "," or "."

function RecoverEntry(entry)
    if type(entry) == "string" then 
        if string.find(entry, "<num>") then
            --return tonumber(string.sub(entry, string.find(entry, ">") + 1, string.len(entry)))
            local neg, number1, number2 = string.match(entry, "<num>([-]?)(%d+)[,.]?(%d?)")
            if number1 then
                if number2 ~= "" then
                    return tonumber(string.format("%s%s%s%s", neg, number1, seperator, number2))
                else
                    return tonumber(string.format("%s%s", neg, number1))
                end
            else
                return 0
            end
        else
            return entry
        end
    elseif type(entry) == "table" then
        return RecoverTable(entry)
    end
    return entry
end

function ConvertTable(T)
    if T == nil then return nil end
    local Copy = {}
    for key, value in pairs(T) do 
        Copy[ConvertEntry(key)] = ConvertEntry(value)
    end
    return Copy
end

function RecoverTable(T)
    if T == nil then return nil end
    local Copy = {}
    for key, value in pairs(T) do 
        Copy[RecoverEntry(key)] = RecoverEntry(value)
    end
    return Copy
end

function SaveTable(Scope, FileName, Table)
    Turbine.PluginData.Save(Scope, FileName, ConvertTable(Table), nil)
end

function LoadTable(Scope, FileName)
    local result, data = pcall(Turbine.PluginData.Load, Scope, FileName, nil)
    if result then
        return RecoverTable(data)
    else
        ChatOutput("Failed to load file: " .. tostring(FileName))
        ChatOutput("Attempting to create new instance of file")
    end
end

-- Duplicate functions to ensure the non callback ones pass ni. to the callback 
function SaveTableCallback(Scope, FileName, Table, Callback)
    Turbine.PluginData.Save(Scope, FileName, ConvertTable(Table), Callback)
end

function LoadTableCallback(Scope, FileName, Callback)
    local updatedCallback = function(data)
        Callback(RecoverTable(data))
    end
    local result, data = pcall(Turbine.PluginData.Load, Scope, FileName, updatedCallback)
    if result then
        return data
    else
        ChatOutput("Failed to load file: " .. tostring(FileName))
        ChatOutput("Attempting to create new instance of file")
    end
end


----------------------------------------------
-- Pickle.lua
-- A table serialization utility for lua
-- Steve Dekorte, http://www.dekorte.com, Apr 2000
-- Freeware
----------------------------------------------

function pickle(t)
    return Pickle:clone():pickle_(t)
end

Pickle = {
clone = function (t) local nt={}; for i, v in pairs(t) do nt[i]=v end return nt end 
}

function Pickle:pickle_(root)
    if type(root) ~= "table" then 
        error("can only pickle tables, not ".. type(root).."s")
    end
    self._tableToRef = {}
    self._refToTable = {}
    local savecount = 0
    self:ref_(root)
    local s = ""

    while table.getn(self._refToTable) > savecount do
        savecount = savecount + 1
        local t = self._refToTable[savecount]
        s = s.."{\n"
        for i, v in pairs(t) do
            s = string.format("%s[%s]=%s,\n", s, self:value_(i), self:value_(v))
        end
        s = s.."},\n"
    end

    return string.format("{%s}", s)
    end

    function Pickle:value_(v)
    local vtype = type(v)
    if     vtype == "string" then return string.format("%q", v)
    elseif vtype == "number" then return v
    elseif vtype == "boolean" then return tostring(v)
    elseif vtype == "table" then return "{"..self:ref_(v).."}"
    else --error("pickle a "..type(v).." is not supported")
    end  
    end

    function Pickle:ref_(t)
    local ref = self._tableToRef[t]
    if not ref then 
        if t == self then error("can't pickle the pickle class") end
        table.insert(self._refToTable, t)
        ref = table.getn(self._refToTable)
        self._tableToRef[t] = ref
    end
    return ref
end

----------------------------------------------
-- unpickle
----------------------------------------------

function unpickle(s)
    if type(s) ~= "string" then
      error("can't unpickle a "..type(s)..", only strings")
    end
    local gentables = loadstring("return "..s)
    local tables = gentables()
    
    for tnum = 1, #tables do
      local t = tables[tnum]
      local tcopy = {}; for i, v in pairs(t) do tcopy[i] = v end
      for i, v in pairs(tcopy) do
        local ni, nv
        if type(i) == "table" then ni = tables[i[1]] else ni = i end
        if type(v) == "table" then nv = tables[v[1]] else nv = v end
        t[i] = nil
        t[ni] = nv
      end
    end
    return tables[1]
  end