Iambored = class(Turbine.UI.Lotro.Window)
function Iambored:Constructor()
    local width = 300
    local height = 300
    local x = Settings.LOX - 125
    local y = Settings.LOY
    local font = Settings.LOFont
    local colour = Turbine.UI.Color(1, 1, 1)
    Turbine.UI.Lotro.Window.Constructor(self)
    self:SetSize(width, height)
    self:SetPosition(x, y)
    self:SetWantsKeyEvents(true)
    self:SetVisible(true)
    self:SetText("I am bored!")

    -- Add hint color box
    local hintColorBox = ColourBox(self, (width - 50) / 2, 40)
    hintColorBox:SetMouseVisible(false)
    hintColorBox:SetColour(Turbine.UI.Color(0, 1, 0))
    self.hintColourBox = hintColorBox

    self.roundTime = 0.5

    -- Add "Play!" button
    self.playButton = Turbine.UI.Lotro.Button()
    self.playButton:SetParent(self)
    self.playButton:SetPosition((width - 50) / 2 - 20, 50)
    self.playButton:SetSize(100, 15)
    self.playButton:SetText("Play!")
    self.playButton:SetVisible(true)

    -- Click function for the play button
    self.playButton.Click = function()
        self:StartGame()
    end

    -- Score label
    self.scoreLabel = Turbine.UI.Label()
    self.scoreLabel:SetParent(self)
    self.scoreLabel:SetPosition(width/2-50, 100)
    self.scoreLabel:SetSize(100, 20)
    self.scoreLabel:SetForeColor(Turbine.UI.Color(1, 1, 1))
    self.scoreLabel:SetVisible(true)

    self.timerBar = Turbine.UI.Control()
    self.timerBar:SetParent(self)
    self.timerBar:SetPosition(10, 90)
    self.timerBar:SetSize(self:GetWidth() - 20, 3)
    self.timerBar:SetBackColor(Turbine.UI.Color.DarkGreen)

    self.points = 0
    self.score = 0
    self.multiplier = 1



    -- Add control that fills the rest of the available space
    local controlTop = hintColorBox:GetTop() + hintColorBox:GetHeight() + 10 -- 10 is a margin
    local controlHeight = height - controlTop - 10 -- 10 is a margin
    self.control = Turbine.UI.Control()
    self.control:SetParent(self)
    self.control:SetPosition(width / 2 - 180 / 2, controlTop)
    self.control:SetSize(width, controlHeight)

    -- Add color boxes to the control
    local boxSize = 50
    local gridTop = 10 -- Top margin for the grid of color boxes
    self.gridBoxes = {}
    for i = 1, 3 do
        for j = 1, 3 do
            local x = (i - 1) * boxSize + (i - 1) * 10 -- 10 is a margin
            local y = (j - 1) * boxSize + (j - 1) * 10 -- 10 is a margin
            local colourBox = ColourBox(self.control, x, y + gridTop, self)
            colourBox:SetBackColor(Turbine.UI.Color(1, 0, 0))
            table.insert(self.gridBoxes, colourBox)
        end
    end
end

function Iambored:Update()
    local currentTime = Turbine.Engine.GetGameTime()
    self.timerBar:SetWidth((self:GetWidth() - 20) * (1 - (currentTime - self.startGameTime) / self.roundTime))
    if self.startGameTime + self.roundTime < currentTime then
        self:StopGame()
    end
end

function Iambored:StartGame()
    -- Initialize variables
    self.startGameTime = Turbine.Engine.GetGameTime()
    self.points = 0
    self.score = 0
    self.multiplier = 1
    self.roundTime = 3
    self.InGame = true

    self.scoreLabel:SetText("Score: 0")

    -- Hide the play button
    self.playButton:SetVisible(false)

    -- Enable updates
    self:SetWantsUpdates(true)

    -- Set the initial round
    self:SetRound()
end

function Iambored:StopGame()
    -- Disable updates
    self:SetWantsUpdates(false)

    -- Show the play button
    self.playButton:SetVisible(true)

    -- Reset the hint color box
    self.hintColourBox:SetBackColor(self.hintColor)

    self.InGame = false
end



function Iambored:SetRound()
    -- List of available colors
    local colorList = {
        Turbine.UI.Color(1, 0, 0),       -- Red
        Turbine.UI.Color(0, 1, 0),       -- Green
        Turbine.UI.Color(0, 0, 1),       -- Blue
        Turbine.UI.Color(1, 1, 0),       -- Yellow
        Turbine.UI.Color(1, 0, 1),       -- Magenta
        Turbine.UI.Color(0, 1, 1),       -- Cyan
        Turbine.UI.Color(1, 0.5, 0),     -- Orange
        Turbine.UI.Color(0, 0.5, 1),     -- Light Blue
        Turbine.UI.Color(0.5, 0.8, 0.3),     -- Lime Green
        Turbine.UI.Color(1, 0, 0.5),     -- Pink
        Turbine.UI.Color(0.5, 0, 1),     -- Purple
        Turbine.UI.Color(0, 0.5, 0.5)      -- Teal
    }

    -- Pick a random box as the hint box
    local hintIndex = math.random(1, 9)
    local newHint
    repeat
        newHint = colorList[math.random(1, #colorList)]
    until newHint ~= self.hintColor
    self.hintColor = newHint
    self.hintColourBox:SetBackColor(self.hintColor)

    -- Set colors for the grid boxes
    for key, value in pairs(self.gridBoxes) do
        if key == hintIndex then
            value:SetBackColor(self.hintColor)
        else
            local randomColor
            repeat
                randomColor = colorList[math.random(1, #colorList)]
            until randomColor ~= self.hintColor
            value:SetBackColor(randomColor)
        end
    end
end

function Iambored:CheckColour(colour)
    if self.InGame then
        if colour.R == self.hintColor.R and colour.G == self.hintColor.G and colour.B == self.hintColor.B then
            -- Correct color clicked
            self.startGameTime = Turbine.Engine.GetGameTime()
            self.points = self.points + 1
            self.score = self.score + (1 * self.multiplier)

            if self.points % 3 == 0 then
                self.multiplier = self.multiplier + 0.5
            end

            self.scoreLabel:SetText("Score: " .. tostring(self.score))

            self.roundTime = math.max(self.roundTime - 0.1, 1)
            self:SetRound()
        else
            -- Incorrect color clicked
            self:StopGame()
        end
    end
end


ColourBox = class(Turbine.UI.Control)
function ColourBox:Constructor(parent, x, y, window)
    Turbine.UI.Control.Constructor(self)

    self:SetParent(parent)
    self:SetSize(50, 50)
    self:SetPosition(x, y)

    self:SetMouseVisible(true)

    self.MouseClick = function()
        window:CheckColour(self:GetBackColor())
    end
end

function ColourBox:SetColour(colour)
    self:SetBackColor(colour)
end