FightHandler = class(Turbine.UI.Control)
function FightHandler:Constructor()
    self.player = Turbine.Gameplay.LocalPlayer.GetInstance()
end

function FightHandler:Initialise(fightLines)
    self:Clean()
    self.InstanceTier = Settings.InstanceTier
    self.InstanceID =  Settings.InstanceVersion

    self.FightLines = fightLines
    
    self.partyNames = {
        [self.player:GetName()] = true,
    }

    self.party = self.player:GetParty()
    if self.party then
        local GroupSize = self.party:GetMemberCount()
        for i = 1, GroupSize, 1 do
            self.partyNames[self.party:GetMember(i):GetName()] = true
        end
    end

    if not(self.chatCallback) then
        self.chatCallback = AddCallback(ChatEventHandler, "Say", function(line)
            self:Parse(line)
        end)
    end

    if not(self.defeatCallback) then
        self.defeatCallback = AddCallback(ChatEventHandler, "Retreat", function(args)
            if self.InFight then
                self:HanldeEnd("", args.Name)
            end
        end)
    end

    if not(self.killCallback) then
        self.killCallback = AddCallback(ChatEventHandler, "Defeated", function(args)
            if self.InFight and self.partyNames[args.Source] then
                self:HanldeEnd("", nil, args.Name)
            end
        end)
    end
end

function FightHandler:Clean()
    self.InFight = false

    self.BossID = nil
    self.BossName = nil
    self.PhaseLines = nil
    self.EndLines = nil

    self.CurrentPhase = nil
    self.MaxPhases = nil

    self.StartTime = nil
    self.PhaseStartTime = nil

    if self.chatCallback then
        RemoveCallback(ChatEventHandler, "Say", self.chatCallback)
    end

    if self.defeatCallback then
        self.defeatCallback = RemoveCallback(ChatEventHandler, "Retreat", self.defeatCallback)
    end

    if self.killCallback then
        RemoveCallback(ChatEventHandler, "Defeated", self.killCallback)
    end

    collectgarbage()
end

function FightHandler:Parse(line)
    local name, cleanedLine = self:PareTextLine(line)
    if self.FightLines[name] and self.FightLines[name][cleanedLine] then
        if (not(self.InFight) or (self.InFight and not(self.player:IsInCombat()))) then
            self:HandleStart(name, cleanedLine)
        end
    elseif self.InFight then
        self:HandleFight(cleanedLine)
    end
end

function FightHandler:HandleStart(name, cleanedLine)
    --Find if the fight is starting
    if (not(self.InFight) or (self.InFight and not(self.player:IsInCombat()))) and self.FightLines[name] and self.FightLines[name][cleanedLine] then
        self:SetupStart(name, cleanedLine)
        return
    end

    self:HandleFight(cleanedLine)
end

function FightHandler:SetupStart(name, cleanedLine)
    -- [1] Boss ID [2] Phases lines [3] End Line
    local StartTemp = self.FightLines[name][cleanedLine]
    self.BossID = StartTemp[1]
    self.PhaseLines = StartTemp[2]
    self.PhaseCount = tablelength(self.PhaseLines)
    self.EndLines = StartTemp[3]
    self.BossName = StartTemp[4]
    self.MaxPhases = tablelength(self.PhaseLines)
    self.StartTime = Turbine.Engine.GetGameTime()
    self.PhaseStartTime = Turbine.Engine.GetGameTime()
    self.InFight = true
    self.CurrentPhase = 1
    self.BossCount = StartTemp[5] or 1
    self.SuccessCount = 0
    local succesLines = 0
    for key, value in pairs(StartTemp[3]) do
        if value[3] then
            succesLines = succesLines + 1
        end
    end
    self.LinesOnly = succesLines == self.BossCount

    if combatTimer then
        combatTimer:ChangeMode(true)
    end

    if InstanceController then
        InstanceController:FightStart({["Boss_Name"] = self.BossName, ["Boss_Tier"] = self.InstanceTier})
        InstanceController:FightPhase({["Boss_Name"] = self.BossName, ["Boss_Tier"] = self.InstanceTier, ["Phase"] = self.CurrentPhase})
    end

    ChatOutput("<rgb=#f79000>Started encounter \"" .. self.BossName .. "\" on Tier " .. self.InstanceTier .." difficulty, good luck!")
end

function FightHandler:HandleFight(cleanedLine)
    -- Return if the fight didn't start and not ongoing
    if not(self.InFight) then return end

    -- We are in a fight

    -- Check for phase ending
    if self.PhaseLines and self.PhaseLines[cleanedLine] then

        -- [1] Boss ID [2] Phase #
        local phaseTemp = self.PhaseLines[cleanedLine]

        if self.BossID == phaseTemp[1] and self.CurrentPhase < phaseTemp[2] then
            self.CurrentPhase = phaseTemp[2]
            local PhaseDuration = Turbine.Engine.GetGameTime() - self.PhaseStartTime
            ChatOutput("<rgb=#f79000>Completed phase " .. self.CurrentPhase - 1 .. " in " .. TimeDeltaToString(PhaseDuration))
            local effectsWindow = EventWindows:GetEffectWindow()
            if effectsWindow then
                effectsWindow:Message(8, "<rgb=#0008CFF>Phase ".. self.CurrentPhase .."</rgb>",nil,false)
            end
            self.PhaseStartTime = Turbine.Engine.GetGameTime()
            if InstanceController then
                InstanceController:FightPhase({["Boss_Name"] = self.BossName, ["Boss_Tier"] = self.InstanceTier, ["Phase"] = self.CurrentPhase})
            end
            return
        end
    end

    self:HanldeEnd(cleanedLine)
end

function FightHandler:HanldeEnd(cleanedLine, defeat, bossDefeat)
    -- Check for fight ending
    local detectWipe = defeat ~= nil --and tablelength(self.EndLines) < 2

    if self.EndLines and self.EndLines[cleanedLine] or detectWipe or bossDefeat then

        -- [1] Boss ID [2] Boss Name [3] Success
        local EndTemp
        if self.EndLines then
            EndTemp = self.EndLines[cleanedLine]
        end

        if detectWipe or EndTemp and self.BossID == EndTemp[1] or bossDefeat then 

            if bossDefeat then
                if self.LinesOnly then return end
                if not(string.find(bossDefeat, self.BossName)) then
                    return
                end
            end

            local tempData = StatEventHandling:ReturnBossData(self.InstanceID, self.BossName, self.InstanceTier)
            local FightDuration = Turbine.Engine.GetGameTime() - self.StartTime
            
            -- We win
            if EndTemp and EndTemp[3] or bossDefeat then
                self.SuccessCount = self.SuccessCount + 1
                if self.SuccessCount >= self.BossCount then
                    if tempData["FirstCompletion"] < 0 then
                        if Achievements then
                            Achievements:AddItem(0x410DB85B, "Achievement Earned", "First Completion", string.format("Defeated %s", self.BossName), self.InstanceTier)
                        end
                        ChatOutput("<rgb=#f79000>Congratulations on your first successful clear on Tier " .. self.InstanceTier .. " difficulty! You have wiped ".. tempData["Wipes"] .. " times before this completion!")
                    end

                    if FightDuration < tempData["BestTime"] then
                        ChatOutput("<rgb=#f79000>Completed encounter " .. self.BossName .. " in " .. TimeDeltaToString(FightDuration) .. ", new best time! You have completed this encounter ".. tempData["Completions"] + 1 .. " times on Tier " .. self.InstanceTier .. " difficulty!")
                    else
                        ChatOutput("<rgb=#f79000>Completed encounter " .. self.BossName .. " in " .. TimeDeltaToString(FightDuration) .. ", your best time is: ".. TimeDeltaToString(tempData["BestTime"]) .."! You have completed this encounter ".. tempData["Completions"] + 1 .. " times on Tier " .. self.InstanceTier .. " difficulty!")
                    end
                    StatEventHandling:SaveBossData(self.InstanceID, self.BossName, self.InstanceTier, FightDuration, true, self.player)

                    if InstanceController then
                        local _, accData = StatEventHandling:GetData()
                        InstanceController:FightEnd({["Completion"] = true, ["Boss_Completion"] = true, ["Boss_Completion_Time"] = TimeDeltaToString(FightDuration), ["Boss_Tier"] = self.InstanceTier, ["Boss_Wipes"] = accData["BossData"][self.InstanceID][self.BossName][self.InstanceTier]["Wipes"]})
                    end
                else
                    return
                end
            end

            -- We Lost
            if detectWipe or EndTemp and not(EndTemp[3]) and FightDuration > 5 then
                ChatOutput("<rgb=#f79000>Wiped after " .. TimeDeltaToString(FightDuration) .. ", you have wiped " .. tempData["Wipes"] + 1 .. " times on Tier " .. self.InstanceTier .. " difficulty.")
                StatEventHandling:SaveBossData(self.InstanceID, self.BossName, self.InstanceTier, FightDuration, false, self.player)

                if InstanceController then
                    local _, accData = StatEventHandling:GetData()
                    InstanceController:FightEnd({["Boss_Completion"] = false, ["Boss_Completion_Time"] = TimeDeltaToString(FightDuration), ["Boss_Tier"] = self.InstanceTier, ["Boss_Wipes"] = accData["BossData"][self.InstanceID][self.BossName][self.InstanceTier]["Wipes"]})
                end
            end
            
            if combatTimer then
                combatTimer:ChangeMode(false)
            end
            if EventWindows:GetTimerWindow() then EventWindows:GetTimerWindow():Clear() end
            self.InFight = false
        end
    end
end

function FightHandler:FightStatus()
    return self.InFight
end

function FightHandler:PareTextLine(message)
    local name, _, line = string.match(message, ">(.*)<\\Select> (.*), ''(.*)''")
    if not(name) and message then
        return "", message:gsub("\n", "")
    end
    return name, line
end