InstanceHub = class(Turbine.UI.Control)
function InstanceHub:Constructor()
    Turbine.UI.Control.Constructor(self)
    self.InstanceSwitch = function(sender, event, args)
        if type(event)=="function" then
            event(args)
        else
            if type(event)=="table" then
                for i = 1, #event do
                    if type(event[i])=="function" then
                        event[i](args)
                    end
                end
            end
        end
    end

    self:CreateTables()

    self.mplus = MythicPlus()
    --self.mplusViewer = MythicPlusViewer(self.nameTable, self.mplus:GetDungeonData())
    --self.button = MythicButton(self.mplusViewer)
--
    self.TEQStats = LoadTable(Turbine.DataScope.Character, "PrimeTEQData") or  {
        ["PearPoints"] = 0,
    }

    self.fightHandler = Instances.FightHandler()

    if Turbine.Gameplay.LocalPlayer.GetInstance():GetAlignment() == 2 then
        Settings.EttenmoorsState = true
        self.switchAction[4]()
        Settings.InstanceVersion = 4
    end

    if Settings.EttenmoorsState and Turbine.Gameplay.LocalPlayer.GetInstance():GetAlignment() < 2 then
        self.switchAction[5]()
        Settings.InstanceVersion = 5
    end
    --ChatOutput(Settings.InstanceVersion)
    
    self:ExecuteInstanceSwitch()

    if Settings.DelvingTier > 0 and Settings.MissionQuest ~= "" and Settings.DelvingMissionActive then
        self:DelvingHandling(true)
    end

    self.questCompleteCallback = AddCallback(ChatEventHandler, "QuestOrDeedCompleted", function(args)
        if Settings.DelvingMissionActive and string.match(args.Name, "Mission: (.*)\n") == Settings.MissionQuest then
            self:UpdateDelvingData(true)
            self:DelvingHandling(false)
        end
    end)
end

function InstanceHub:EnterNewInstance(line, id, tier)
    Settings.InstanceState = true
    Settings.InstanceVersion = id
    if tier then
        Settings.InstanceTier = tier
    elseif line then
        line = string.gsub(line, "+", "")
        Settings.InstanceTier = string.sub(line, string.len(line), string.len(line))
    else
        Settings.InstanceTier = "?"
    end

    --ChatOutput(id)
    if Target and Settings.InstanceState then 
        ChatOutput("<rgb=#FF0000>Target window is enabled, this might impact performance on some raid bosses!</rgb>")
    end

    self:ExecuteInstanceSwitch()
    if PrimeUIHandlerObject then PrimeUIHandlerObject:RecheckPermissions() end
end

function InstanceHub:GetInitialData()
    return {
        ["State"] = Settings.InstanceState, 
        ["Instance_Name"] = self.nameTable[Settings.InstanceVersion], 
        ["Instance_Tier"] = Settings.InstanceTier
    }
end

function InstanceHub:ExecuteInstanceSwitch()
    if self.switchAction[Settings.InstanceVersion] then self.switchAction[Settings.InstanceVersion]() end
    
    if Settings.InstanceState then
        self.mplus:LoadDungeon(Settings.InstanceVersion, Settings.InstanceTier)

        PluginsManager:LoadRaidPlugins()

        if Auras then Auras:HandleRaidLoads(true) end
    end

    self.InstanceSwitch(self, self.InstanceEvent, {["PvMP_State"] = Settings.EttenmoorsState, ["State"] = Settings.InstanceState, ["Instance_Name"] = self.nameTable[Settings.InstanceVersion], ["Instance_Tier"] = Settings.InstanceTier})
end

function InstanceHub:ParsePlus(line)
    self.mplus:Parse(line)
end

function InstanceHub:DelvingHandling(active)
    if active then
        --if not(Settings.DelvingMissionActive) then
            Settings.MissionStartTime = Turbine.Engine.GetGameTime()
        --end
        Settings.DelvingMissionActive = true
        Settings.InstanceState = true
        self.InstanceSwitch(self, self.DelvingEvent, {["DelvingMissionActive"] = Settings.DelvingMissionActive, ["Mission_Name"] = Settings.MissionQuest, ["Delving_Tier"] = Settings.DelvingTier})
    else
        self.InstanceSwitch(self, self.DelvingEvent, {["DelvingMissionActive"] = Settings.DelvingMissionActive, ["Mission_Name"] = Settings.MissionQuest, ["Delving_Tier"] = Settings.DelvingTier})
        Settings.DelvingTier = 0
        Settings.DelvingMissionActive = false
        Settings.InstanceState = false       
    end

    if PrimeUIHandlerObject then PrimeUIHandlerObject:RecheckPermissions() end
end

function InstanceHub:CheckInstanceState(line)
    if string.find(line, "malice spreads through the area") then
        if Settings.DelvingTier > 0 and Settings.MissionQuest ~= "" then
            self:DelvingHandling(true)
        end
    end

    if string.find(line, "Failed: Tier") then
        self:UpdateDelvingData(false)
        self:DelvingHandling(false)
    end

    if string.find(line, "Quest") then
        local strippedLine = ""
        if string.find(line, "New Quest: Mission:") then
            if self.delvingMissions[line] then
                Settings.MissionQuest = self.delvingMissionsNames[self.delvingMissions[line]]
            end
        elseif string.find(line, ": Delving") then
            -- New Quest: Tier 3: Delving Bloodstone
            --Failed: Tier 3: Delving Bloodstone
            -- A formidable malice spreads through the area... 5
            Settings.DelvingTier = tonumber(string.match(line, "%d+"))
        elseif string.find(line, "Tier") then
            if string.find(line, "- ") then
                strippedLine = string.sub(line, 1, string.find(line, "- ") - 2)
                if self.questLines[strippedLine] then self:EnterNewInstance(line, self.questLines[strippedLine]) end
            end

            if string.match(line, "(.*) %(Tier [1-5]%)") then
                local instance, tier = string.match(string.gsub(line, "+", ""), "(.*) %(Tier (.*)%)")
                if self.questLines[instance] then self:EnterNewInstance(instance, self.questLines[instance], tier) end
            end


        elseif string.find(line, "Solo") or string.find(line, "Duo") then
            local instance = string.match(line, "(.*)%-%- Solo") or string.match(line, "(.*)%-%- Duo")

            if instance then
                if self.questLines[instance] then self:EnterNewInstance(instance, self.questLines[instance], 0) end
            end
        end

        if self.questLines[line] then 
            self:EnterNewInstance(line, self.questLines[line], "?") 
        end
    end

    if string.find(line, "Entered the") then
        if Settings.EttenmoorsState and not(string.find(line, "Ettenmoors" )) then
            Settings.EttenmoorsState = false
            if self.KillTracker then self.KillTracker:SetVisible(false) end
            PluginsManager:UnloadEttenmoorsPlugins()
            if Auras then Auras:HandlePvpLoads(false) end
            if PrimeUIHandlerObject then PrimeUIHandlerObject:RecheckPermissions() end
        end

        if string.find(string.match(line, "Entered the (.*) channel."), "-") and Settings.InstanceVersion > 0 then
            if Settings.EEnabled and Settings.InstanceState then
                --effectsWindow:Message(8, "<rgb=#0008CFF>Leaving so soon?</rgb>",nil,false)
            end
            Settings.InstanceState = false
            Settings.InstanceVersion = 0
            Settings.InstanceTier = 0
            self:ExecuteInstanceSwitch()
            PluginsManager:UnloadRaidPlugins()
            if Auras then Auras:HandleRaidLoads(false) end
            self.mplus:LoadDungeon(0, 0)
            if PrimeUIHandlerObject then PrimeUIHandlerObject:RecheckPermissions() end
            

            self.InstanceSwitch(self, self.InstanceEvent, {["PvMP_State"] = Settings.EttenmoorsState, ["State"] = Settings.InstanceState, ["Instance_Name"] = self.nameTable[Settings.InstanceVersion], ["Instance_Tier"] = Settings.InstanceTier})
        end

        if combatTimer then
            combatTimer:ChangeMode(false)
        end
    end

    if string.find(line, "Entered the Ettenmoors") and not(Settings.EttenmoorsState) then
        Settings.EttenmoorsState = true

        if Settings.InstanceState then 
            PluginsManager:UnloadRaidPlugins() 
            if Auras then Auras:HandleRaidLoads(false) end
        end
        Settings.InstanceState = false
        if Turbine.Gameplay.LocalPlayer.GetInstance():GetAlignment() == 1 then
            self.switchAction[5]()
            Settings.InstanceVersion = 5
        elseif Turbine.Gameplay.LocalPlayer.GetInstance():GetAlignment() == 2 then
            Settings.InstanceVersion = 4
            self.switchAction[4]()
        end
        if self.KillTracker then self.KillTracker:SetVisible(true) end
        PluginsManager:LoadEttenmoorsPlugins()
        if Auras then Auras:HandlePvpLoads(true) end
    end

    if string.find(line, "resets") then
        -- DG
        if string.find(line, "Dâr Narbugud resets in") then
            Settings.InstanceState = true
            if Settings.EEnabled and Settings.InstanceVersion ~= 2 then
                --effectsWindow:Message(8, "<rgb=#0008CFF>Welcome to Dâr Narbugud! </rgb>",nil,false)
            end
            self:EnterNewInstance(line, 2)
        end

        -- Rift
        if string.find(line, "The Rift of Nûrz Ghâshu resets in") then
            Settings.InstanceState = true
            if Settings.EEnabled and Settings.InstanceVersion ~= 3 then
                --effectsWindow:Message(8, "<rgb=#0008CFF>Welcome to The Rift of Nûrz Ghâshu! </rgb>",nil,false)
            end
            self:EnterNewInstance(line, 3)
        end

        -- Throne
        if string.find(line, "Throne of the Dread Terror resets in") then
            Settings.InstanceState = true
            if Settings.EEnabled and Settings.InstanceVersion ~= 8 then
                --effectsWindow:Message(8, "<rgb=#0008CFF>Welcome to Throne! </rgb>",nil,false)
            end
            self:EnterNewInstance(line, 8)
        end

        -- Filikul
        if string.find(line, "Filikul resets in") then
            Settings.InstanceState = true
            if Settings.EEnabled and Settings.InstanceVersion ~= 8 then
                --effectsWindow:Message(8, "<rgb=#0008CFF>Welcome to Filikul!</rgb>",nil,false)
            end
            self:EnterNewInstance(line, 30)
        end

        -- Watcher
        if string.find(line, "The Vile Maw resets in") then
            Settings.InstanceState = true
            if Settings.EEnabled and Settings.InstanceVersion ~= 8 then
                --effectsWindow:Message(8, "<rgb=#0008CFF>Welcome to The Vile Maw!</rgb>",nil,false)
            end
            self:EnterNewInstance(line, 31)
        end

    end
end

function InstanceHub:CreateTables()

    --[[
        Instances
        Warg Pens of Dol Guldur
        Sammath Gûl

        Dark Delvings 21
        Fil Gashan 22
        Skumfil 23
        Forges of Khazad-dûm 24
        The Grand Stair 25
        The Sixteenth Hall 26

        Arboretum
        Barrows
        Fortress

        The Depths of Kidzul-kâlah
        The Harrowing of Morgul
        Eithel Gwaur, the Filth-well 
        Gath Daeroval, the Shadow-roost
        Gorthad Nûr, the Deep-barrow 

        Bâr Nírnaeth, the Houses of Lamentation 
        Ghashan-kútot, the Halls of Black Lore 
        The Fallen Kings 
        
        Shakalush, the Stair Battle

        Raids
        Barad Guldur
        Remmorchant, the Net of Darkness
        Amdân Dammul, the Bloody Threshold 
        Filikul
        The Vile Maw
    ]]

    self.questLines = {

        -- MM
        ["New Quest: Remmorchant, the Net of Darkness "] = 1,
        ["New Quest: Amdân Dammul, the Bloody Threshold "] = 7,
        ["New Quest: The Depths of Kidzul-kâlah "] = 9,
        ["New Quest: Shakalush, the Stair Battle "] = 10,
        ["New Quest: The Harrowing of Morgul"] = 11,
        ["New Quest: Bâr Nírnaeth, the Houses of Lamentation "] = 12,
        ["New Quest: Eithel Gwaur, the Filth-well "] = 13,
        ["New Quest: Gath Daeroval, the Shadow-roost "] = 14,
        ["New Quest: Ghashan-kútot, the Halls of Black Lore "] = 15,
        ["New Quest: Gorthad Nûr, the Deep-barrow "] = 16,
        ["New Quest: The Fallen Kings "] = 17,

        ["New Quest: Agoroth, the Narrowdelve "] = 32,
        ["New Quest: Woe of the Willow "] = 33,

        ["New Quest: Askâd-mazal, the Chamber of Shadows "] = 36,

        -- Mirkwood
        ["New Quest: Barad Guldur: The Deep Assault"] = 18,
        ["New Quest: Dark Deeds in Sammath Gûl"] = 19,
        ["New Quest: Strange Meat"] = 20,

        -- Moria
        ["New Quest: Challenge: Gurvand (Daily)"] = 21,
        ["New Quest: Challenge: General Talug (Daily)"] = 22,
        ["New Quest: Challenge: Hwandrin (Daily)"] = 23,
        ["New Quest: Challenge: Kranklûk (Daily)"] = 24,
        ["New Quest: Challenge: Igash (Daily)"] = 25,
        ["New Quest: Challenge: The Lost One (Daily)"] = 26,

        -- Garth Aragwen
        ["New Quest: Challenge: Grimbark"] = 27,
        ["New Quest: Challenge: Ivar"] = 28,
        ["New Quest: Challenge: Dúnlang"] = 29,   

        -- Filikul 30
        -- Vile maw 31

        -- Festival
        ["New Quest: Boss from the Vaults: Naruhel"] = 34,
        ["New Quest: Bee's Big Business"] = 35,

        -- ROots of fangorn
        ["New Quest: Instance: The Roots of Fangorn"] = 37,

        -- Fall of Khazad-Dum
        ["New Quest: The Fall of Khazad-dûm "] = 38,

        ["New Quest: Assault on Dhúrstrok "] = 39,
        ["New Quest: Den of Pughlak "] = 40,
        ["New Quest: Thikil-gundu, the Steel Keep"] = 41,
        ["New Quest: Caverns of Thrumfall "] = 42,
        ["New Quest: Glimmerdeep "] = 43,

        ["New Quest: Defenders of the Fortress"] = 44,
        ["New Quest: Uruk-necklaces"] = 45,

        --Annuminas
        ["New Quest: Glinghant Corrupted"] = 46,
        ["New Quest: End Dolvaethor's Menace"] = 47,
        ["New Quest: Balhest, Lord of Ost Elendil"] = 48,

        -- Barrows
        ["New Quest: Sambrog"] = 49,
        ["New Quest: Thadúr the Ravager"] = 50,
        ["New Quest: Gaerdring and Gaerthel"] = 51,
    
        -- Fornost
        ["New Quest: Raid: Spider Squasher"] = 52,
        ["New Quest: Raid: Giant-slayer"] = 53,
        ["New Quest: Raid: Drake-wrangler"] = 54,
        ["New Quest: Raid: Thorog Defeated"] = 55,

        ["New Quest: Adkhât-zahhar, the Houses of Rest "] = 56,
        
        ["New Quest: The Hiddenhoard of Abnankâra "] = 57,

        -- OD
        ["New Quest: Instance: The Lost Temple"] = 58,
        ["New Quest: Challenge: Toes in the Water"] = 59,
        ["New Quest: Challenge: The Power of Gortheron"] = 60,
        ["New Quest: Challenge: Face Your Fears"] = 61,
        ["New Quest: Challenge: Sâri-surma -- Survival"] = 62,
        ["New Quest: Challenge: The Unquenched Flame"] = 63,
        ["New Quest: Challenge: The Town of Stoneheight"] = 64,

        -- Erebor
        ["New Quest: Challenge: An Army from the East"] = 65,
        ["New Quest: Iorbar's Peak - Challenge"] = 66,
        ["New Quest: Challenge: Rapid Succession"] = 67,
        ["New Quest: The Battle for Erebor - Challenge"] = 68,
        ["New Quest: Challenge: The Bells of Dale"] = 69,
        ["New Quest: The Fires of Smaug - Challenge"] = 70,
        ["New Quest: Challenge: Webs of the Scuttledells"] = 71,
        
        -- Firework
        ["New Quest: A Flurry of Fireworks"] = 72,

        -- Before the Shadow
        ["New Quest: Doom of Caras Gelebren"] = 73,
        ["New Quest: Sarch Vorn, the Black Grave "] = 74,

        -- 
        ["New Quest: Sant Lhoer, the Poison Gardens "] = 75,
        ["New Quest: Sagroth, Lair of Vermin "] = 76,
        ["New Quest: Thaurisgar, the Vile Apothecary "] = 77,
        ["New Quest: Gwathrenost, the Witch-king's Citadel "] = 78,
        -- Highest 78

    }

    self.nameTable = {
        [0] = "All",
        [1] = "Remmorchant",
        [2] = "Dâr Narbugud",
        [3] = "The Rift of Nûrz Ghâshu",
        [4] = "Monster Player",
        [5] = "The Free Peoples",
        [6] = "The Ettenmoors",
        [7] = "Amdân Dammul",
        [8] = "Throne of the Dread Terror",
        [9] = "The Depths of Kidzul-kâlah",
        [10] = "Shakalush, the Stair Battle",
        [11] = "The Harrowing of Morgul",
        [12] = "Bâr Nírnaeth, the Houses of Lamentation",
        [13] = "Eithel Gwaur, the Filth-well",
        [14] = "Gath Daeroval, the Shadow-roost",
        [15] = "Ghashan-kútot, the Halls of Black Lore",
        [16] = "Gorthad Nûr, the Deep-barrow",
        [17] = "The Fallen Kings",
        [18] = "Barad Guldur",
        [19] = "Sammath Gûl",
        [20] = "Warg Pens of Dol Guldur",
        [21] = "Dark Delvings",
        [22] = "Fil Gashan",
        [23] = "Skumfil",
        [24] = "Forges of Khazad-dûm",
        [25] = "The Grand Stair",
        [26] = "The Sixteenth Hall",
        [27] = "Arboretum",
        [28] = "Barrows",
        [29] = "Fortress",
        [30] = "Filikul",
        [31] = "The Vile Maw",
        [32] = "Agoroth, the Narrowdelve",
        [33] = "Woe of the Willow",
        [34] = "Boss from the Vaults: Naruhel",
        [35] = "Bee's Big Business",
        [36] = "Askâd-mazal, the Chamber of Shadows",
        [37] = "Roots of Fangorn",
        [38] = "The Fall of Khazad-dûm",
        [39] = "Assault on Dhúrstrok",
        [40] = "Den of Pughlak",
        [41] = "Thikil-gundu",
        [42] = "Thrumfall",
        [43] = "Glimmerdeep",
        [44] = "Sâri-surma",
        [45] = "Sword-halls",
        [46] = "Glinghant",
        [47] = "Haudh Valandil",
        [48] = "Ost Elendil",
        [49] = "Sambrog",
        [50] = "Thadúr",
        [51] = "Great-barrow",
        [52] = "Spider Wing",
        [53] = "Giant Wing",
        [54] = "Drake Wing",
        [55] = "Thorog",
        [56] = "Adkhât-zahhar, the Houses of Rest",
        [57] = "The Hiddenhoard of Abnankâra",
        [58] = "Lost Temple",
        [59] = "Ost Dunhoth - Disease and Poison",
        [60] = "Ost Dunhoth - Gortheron",
        [61] = "Ost Dunhoth - Wound and Fear",
        [62] = "Sâri-surma",
        [63] = "The Northcotton Farm",
        [64] = "Stoneheight",
        [65] = "Flight to the Lonely Mountain",
        [66] = "Iorbar's Peak",
        [67] = "Seat of the Great Goblin",
        [68] = "The Battle for Erebor",
        [69] = "The Bells of Dale",
        [70] = "The Fires of Smaug",
        [71] = "Webs of Scuttledells",
        [72] = "A Flurry of Fireworks",
        [73] = "Doom of Caras Gelebren",
        [74] = "Sarch Vorn",
        [75] = "Sant Lhoer, the Poison Gardens",
        [76] = "Sagroth, Lair of Vermin",
        [77] = "Thaurisgar, the Vile Apothecary",
        [78] = "Gwathrenost, the Witch-king's Citadel",
    }

    self.delvingMissions = {
        ["New Quest: Mission: A Plague of Men"] = 1,
        ["New Quest: Mission: Brigands in Nímbarth"] = 2,
        ["New Quest: Mission: Caranost Command Post"] = 3,
        ["New Quest: Mission: Caranost Courtyard"] = 4,
        ["New Quest: Mission: Crown of Caranost"] = 5,
        ["New Quest: Mission: Down on the Farm"] = 6,
        ["New Quest: Mission: Haunting at Dol Ernil"] = 7,
        ["New Quest: Mission: Jorthkyn and Hounds"] = 8,
        ["New Quest: Mission: Siege of Sírlond"] = 9,
        ["New Quest: Mission: Steps of Caranost"] = 10,
        ["New Quest: Mission: Stirring of Spirits"] = 11,
        ["New Quest: Mission: The Bridge to Caranost"] = 12,
        ["New Quest: Mission: The Nettinglade"] = 13,
        ["New Quest: Mission: Siege-breaking"] = 14,
        ["New Quest: Mission: Tharbad Tower Courtyard"] = 15,
        ["New Quest: Mission: The Ruins of Amon Firn"] = 16,
        ["New Quest: Mission: The Tearful Tomb"] = 17,
        ["New Quest: Mission: Before They Bond"] = 18,
        ["New Quest: Mission: The Ruins of Gaervarad"] = 19,
        ["New Quest: Mission: Signs and Sigils"] = 20,
        ["New Quest: Mission: A Day on the Farm"] = 21,
        ["New Quest: Mission: Dangerous Deliveries"] = 22,
        ["New Quest: Mission: Not as Many Heads"] = 23,
        ["New Quest: Mission: Leadership Direction"] = 24,
        ["New Quest: Mission: The Jaws of Death"] = 25,
        ["New Quest: Mission: Aberrations of Nature"] = 26,
        ["New Quest: Mission: Infiltration"] = 27,
        ["New Quest: Mission: Stirring on Sarn Ford"] = 28,
        ["New Quest: Mission: Patrol the Township"] = 29,
        ["New Quest: Mission: Terrible Turtle Trouble"] = 30,
        ["New Quest: Mission: Mother of Wolves"] = 31,
        ["New Quest: Mission: Secrets and Plans"] = 32,
        ["New Quest: Mission: Among the Watch-towers"] = 33,
        ["New Quest: Mission: Into the Pits"] = 34,
        ["New Quest: Mission: Bite the Hand"] = 35,
        ["New Quest: Mission: Sifting Through Time"] = 36,
        ["New Quest: Mission: The Hand, Out!"] = 37,
        ["New Quest: Mission: The Tombs of Gond Orchal"] = 38,
        ["New Quest: Mission: Shadows in the Greyflood"] = 39,
        ["New Quest: Mission: Troubled Waters"] = 40,
        ["New Quest: Mission: The Accursed Flame"] = 41,
        ["New Quest: Mission: Tharbad Tower Ascent"] = 42,

        ["All"] = 999,
    }

    self.delvingMissionsNames = {
        [1] = "A Plague of Men",
        [2] = "Brigands in Nímbarth",
        [3] = "Caranost Command Post",
        [4] = "Caranost Courtyard",
        [5] = "Crown of Caranost",
        [6] = "Down on the Farm",
        [7] = "Haunting at Dol Ernil",
        [8] = "Jorthkyn and Hounds",
        [9] = "Siege of Sírlond",
        [10] = "Steps of Caranost",
        [11] = "Stirring of Spirits",
        [12] = "The Bridge to Caranost",
        [13] = "The Nettinglade",
        [14] = "Siege-breaking",
        [15] = "Tharbad Tower Courtyard",
        [16] = "The Ruins of Amon Firn",
        [17] = "The Tearful Tomb",
        [18] = "Before They Bond",
        [19] = "The Ruins of Gaervarad",
        [20] = "Signs and Sigils",
        [21] = "A Day on the Farm",
        [22] = "Dangerous Deliveries",
        [23] = "Not as Many Heads",
        [24] = "Leadership Direction",
        [25] = "The Jaws of Death",
        [26] = "Aberrations of Nature",
        [27] = "Infiltration",
        [28] = "Stirring on Sarn Ford",
        [29] = "Patrol the Township",
        [30] = "Terrible Turtle Trouble",
        [31] = "Mother of Wolves",
        [32] = "Secrets and Plans",
        [33] = "Among the Watch-towers",
        [34] = "Into the Pits",
        [35] = "Bite the Hand",
        [36] = "Sifting Through Time",
        [37] = "The Hand, Out!",
        [38] = "The Tombs of Gond Orchal",
        [39] = "Shadows in the Greyflood",
        [40] = "Troubled Waters",
        [41] = "The Accursed Flame",
        [42] = "Tharbad Tower Ascent",


        [999] = "All",
    }

    -- Go away >:(
    self.pearStrings = {
        [10] = {0x4113B26F, "Pear Start", "A good start, Pear Man will watch your career with great interest."},
        [20] = {0x4113B26F, "To Pear or not to Pear", "To Pear or not to Pear is a dumb question, you should always Pear!"},
        [30] = {0x4113B26F, "Surrender your mind", "To surrender your mind to the Pear is to achieve true happiness!"},
        [40] = {0x4113B26F, "Path of the Pear", "Following the path of the Pear is to achieve true meaning in life!"},
        [50] = {0x4113B26F, "Pear Notice", "The Pear Man is noticing your good work!"},
        [80] = {0x4113B26F, "Rotten Apple", "A rotten apple spoils the barrel, but the Pear is not affected."},
        [100] = {0x4113B26F, "A Pear Day", "A Pear a day keeps the darkness and dread of the all-consuming void away!"},
        [130] = {0x4113B26F, "Pear side of life", "Always look on the Pear side of life!"},
        [150] = {0x4113B26F, "Pear Pathway", "The Pear side is a pathway to many abilities, some considered to be unnatural."},
        [180] = {0x4113B26F, "Pearternal", "Civilizations rise and fall but Pear is eternal"},
        [200] = {0x4113B26F, "Inner Pear", "To find your inner Pear is to find balance in your mind, soul and body."},
        [230] = {0x4113B26F, "The Pear Side", "Love is a powerful force, but nothing compared to the power of the Pear side"},
        [250] = {0x4113B26F, "Teachings of the Pear", "The teachings of the Pear give power to only the worthy!"},
        [300] = {0x4113B26F, "The way of the Pear", "The way of the Pear is a noble pursuit, you are making good progess!"},
        [350] = {0x4113B26F, "Knowledge of Pear", "The pathway to true power can only be walked with the knowledge of the Pear."},
        [400] = {0x4113B26F, "Master the way", "To master the way of the Pear one has to master all aspects of life."},
        [420] = {0x4113B26F, "Master the Pear", "You have truly mastered the way of the Pear, you have become a true Pear Warrior. You have earned the respect of the Pear Man!"}
    }

    self.switchAction = {
        -- DefaultState
        [0] = 
        function()
    
        end,
        -- Remmorchant
        [1] = 
        function()

            --Player alerts
            self.fightHandler:Initialise(self:GetRaidInfo()[Settings.InstanceVersion])
        end,
        -- Ettenmoors Creep
        [4] = 
        function()
            
            self.switchAction[6]()
        end,
        -- Ettenmoors Freep
        [5] =
        function()
            self.switchAction[6]()
        end,
        -- Ettenmoors both factions
        [6] =
        function()
            if self.KillTracker == nil then
                self.KillTracker = KillCounter(self)
            end
        end,
        [7] =
        function()
            if Settings.EEnabled then
                --effectsWindow:Message(8, "<rgb=#0008CFF>Welcome to the Bloody Threshold! </rgb>",nil,false)
            end
            --TDamage["Thrash"] = {55, "Thrash"}
            --TDamage[""]
            self.fightHandler:Initialise(self:GetRaidInfo()[Settings.InstanceVersion])
            -- Let's see how they handle bardhrukh, or as my boys call him... THE KILLBEAST! -- Start on boss 1 + X seconds
            --Gorgar the Ruthless says  Look at that! The dwarves killed the Killbeast! -- kill on boss 1
            -- Good warg. To battle! -- Start boss 2
            --The warg chieftain is victorious! I guess you get to live, mongrel! -- Wipe on boss 2
            -- Prince Durin shouts Well done, heroes! -- Kill on boss 2
        end,
        -- Throne
        [8] =
        function()
            self.fightHandler:Initialise(self:GetRaidInfo()[Settings.InstanceVersion])
        end,
        [38] = function()
            self.fightHandler:Initialise(self:GetRaidInfo()[Settings.InstanceVersion])
        end,
        [57] = 
        function()
            self.fightHandler:Initialise(self:GetRaidInfo()[Settings.InstanceVersion])
        end,
        [75] =         
        function()
            self.fightHandler:Initialise(self:GetRaidInfo()[Settings.InstanceVersion])
        end,
        [78] =         
        function()
            self.fightHandler:Initialise(self:GetRaidInfo()[Settings.InstanceVersion])
        end,
        [999] =
        function()

        end,
    }
end

function InstanceHub:Unload()
    Settings.MythicPlusX, Settings.MythicPlusY = self.mplus:GetPosition()
    SaveTable(Turbine.DataScope.Character, "PrimeTEQData", self.TEQStats)
end

function InstanceHub:GetDungeonNameTable()
    return self.nameTable
end

function InstanceHub:GetDelvingTable()
    return self.delvingMissionsNames
end

function InstanceHub:FightStart(data)
    self.InstanceSwitch(self, self.FightEventStart, data)
end

function InstanceHub:FightEnd(data)
    self.InstanceSwitch(self, self.FightEventEnd, data)
end

function InstanceHub:FightPhase(data)
    self.InstanceSwitch(self, self.FightEventPhase, data)
end

function InstanceHub:PvpInfoDump(data)
    self.InstanceSwitch(self, self.PvpInfo, data)
end

function InstanceHub:FightStatus()
    if self.fightHandler then
        return self.fightHandler:FightStatus()
    end
end

function InstanceHub:GetPvpData()
    if self.KillTracker then
        return self.KillTracker:GetInfo()
    else
        return {}
    end
end

function InstanceHub:GetMPlusTable()
    return self.mplus:GetDungeonData()
end

function InstanceHub:GetKillTracker()
    return self.KillTracker
end

function InstanceHub:GetFightNames()
    local nameTable = {}
    for instanceID, bossTable in pairs(self:GetRaidInfo()) do
        for bossStartTrigger, bossInfo in pairs(bossTable) do
            for startString, bossInfoTable in pairs(bossInfo) do
                nameTable[bossInfoTable[4]] = true 
            end
        end
    end
    return nameTable
end

function InstanceHub:UpdateDelvingData(completed)
    -- Setup variables
    local mission = Settings.MissionQuest
    local delvingTier = Settings.DelvingTier
    local startTime = Settings.MissionStartTime
    local endTime = Turbine.Engine.GetGameTime()
    local time = endTime - startTime
    local timeString = FormatTimeStringNoMilli(time)

    -- Null checks
    if not(self.TEQStats[mission]) then
        self.TEQStats[mission] = {
            ["PearPoints"] = 0,
        }
    end

    if not(self.TEQStats[mission][delvingTier]) then
        self.TEQStats[mission][delvingTier] = {
            ["Completions"] = 0,
            ["TotalTime"] = 0,
            ["Fails"] = 0,
            ["Fastest"] = -1,
            ["Slowest"] = -1,
            ["Tries"] = 0,
        }
    end

    -- Completion types
    if completed then
        self.TEQStats[mission][delvingTier]["Completions"] = self.TEQStats[mission][delvingTier]["Completions"] + 1

        --Chat Message completion + time
        ChatOutput(string.format("%sYou have completed the delving: %s in %s. You have completed this mission on this delving tier %d times. %s", "<rgb=#0008CFF>", mission, timeString, self.TEQStats[mission][delvingTier]["Completions"], "</rgb>"))

        -- Update pear points if first completion of this delving tier on this mission
        if self.TEQStats[mission][delvingTier]["Completions"] == 1 then
            local currentHighestTier = self.TEQStats[mission]["PearPoints"]
            if currentHighestTier < delvingTier then
                self.TEQStats["PearPoints"] = self.TEQStats["PearPoints"] + (delvingTier - currentHighestTier)

                -- Pear points update
                ChatOutput(string.format("%sYou have gained %d Pear Points, bringing your Top End Quester Score to %d!%s", "<rgb=#f79000>", (delvingTier - currentHighestTier), self.TEQStats["PearPoints"], "</rgb>"))
    
                self:PearValueCheck(self.TEQStats["PearPoints"], (delvingTier - currentHighestTier))
            end
        end
        
        -- General times
        self.TEQStats[mission][delvingTier]["Tries"] = self.TEQStats[mission][delvingTier]["Tries"] + 1
        self.TEQStats[mission][delvingTier]["TotalTime"] = self.TEQStats[mission][delvingTier]["TotalTime"] + time

        local averageTimeString = FormatTimeStringNoMilli(self.TEQStats[mission][delvingTier]["TotalTime"] / self.TEQStats[mission][delvingTier]["Completions"])

        -- Fastest time
        if self.TEQStats[mission][delvingTier]["Fastest"] == -1 or self.TEQStats[mission][delvingTier]["Fastest"] > time then
            self.TEQStats[mission][delvingTier]["Fastest"] = time

            -- Fastest time update
            ChatOutput(string.format("%sYour time %s is your fastest completion for this delving on this tier! Your average completion time is %s.%s", "<rgb=#0008CFF>", timeString, averageTimeString, "</rgb>"))
        else
            -- Average time update
            ChatOutput(string.format("%sYour average completion time for this mission on this delving tier is %s.%s", "<rgb=#0008CFF>", averageTimeString, "</rgb>"))
        end

        -- Slowest time
        if self.TEQStats[mission][delvingTier]["Slowest"] == -1 or self.TEQStats[mission][delvingTier]["Slowest"] < time then
            self.TEQStats[mission][delvingTier]["Slowest"] = time
        end
    else
        self.TEQStats[mission][delvingTier]["Fails"] = self.TEQStats[mission][delvingTier]["Fails"] + 1

        -- Chat message failure
        ChatOutput(string.format("%sTotal time for this undertaking was %s.%s", "<rgb=#0008CFF>", timeString, "</rgb>"))
    end
end

function InstanceHub:PearValueCheck(PearPoints, delvingTier)
    for index, message in spairs(self.pearStrings, function(t, a, b) return a < b end) do
        if index > (PearPoints - delvingTier) and index <= PearPoints then
            ChatOutput(string.format("%s%s%s", "<rgb=#f79000>", message[3], "</rgb>"))
            if Achievements then
                Achievements:AddItem(message[1], "Pear Points Earned", message[2], message[3], PearPoints)
            end
            break
        end
    end

    -- 10 per mission 42 missions
end

function InstanceHub:GetRaidInfo()  
    return {
        [1] = {
            -- Boss 1
            ["Bratha Tasakh, Lady of Many Venoms"] = {
                ["No, no! I cannot be the last! The Ungoledain will not fall this day!"] = 
                {
                    1,
                    {
                    ["This will not do. Perhaps this may work...."] = {1, 2}
                    },
                    {
                        ["No! Erk... Felgúr.... You were right.... The Ungoledain die with me...."] = {1, "Bratha Tasakh", true}
                    },
                    "Bratha Tasakh",
                    1
                    
                },
            },
            -- Boss 3
            ["Captain Zabothak"] = {

                ["For the Pale Herald!"] = 
                {
                    3,
                    {
                        ["It is over, Zabothak... and you have failed."] = {3, 2}
                    },
                    {
                        ["You are... too late...."] = {3, "Zabothak & Rukhor", true}
                    },
                    "Zabothak & Rukhor",
                    1
                }
            },
            -- Boss 4
            ["Legolas"] = {
                ["You must wield the Gilgalar, my friend. Hold off Ungwetári for as long as you can!"] = 
                {
                    4,
                    nil,
                    {
                        ["Ungwetári means to flee! We cannot allow her to escape, my friends!"] = {4, "Shelob", true}
                    },
                    "Shelob",
                    1
                }
            }
        },
        [7] = {
            ["Gorgar the Ruthless"] = {
                -- Boss 1
                ["Let's see how they handle Bardhrukh, or as my boys call him... THE KILLBEAST!"] = 
                {
                    1, 
                    nil,
                    {
                        ["What a pathetic showing by the dwarves! At least the beasts will eat well tonight! Hah!"] = {1, "The Killbeast", false},
                        ["Look at that! The dwarves killed the Killbeast!"] = {1, "The Killbeast", true}
                    },
                    "The Killbeast",
                    1
                },

                -- Boss 2
                ["Good warg. To battle!"] = {
                    2,
                    {
                        ["ENOUGH! I AM NO ONE'S MOUNT!"] = {2, 2}
                    },
                    {
                        ["The warg chieftain is victorious! I guess you get to live, mongrel!"] = {2, "Gorgar & Ránulur", false},
                        ["None can stand against Gorgar the Ruthless!"] = {2, "Gorgar & Ránulur", false},
                        ["Prince Durin shouts Well done, heroes!"] = {2, "Gorgar & Ránulur", true},
                        ["At last... my suffering... is over...."] = {2, "Gorgar & Ránulur", true},
                    },
                    "Gorgar & Ránulur",
                    1
                }
            },
        },
        -- [[
        [8] = {
            ["Rakothas"] = {
                -- Boss 1
                ["I shall delight in delivering them to their deaths!"] = {
                    1,
                    {
                        ["You cannot stand against me..."] = {1, 2}
                    },
                    {
                        ["I am but a hammer before a field of nails!"] = {1, "Rakothas", false}
                    },
                    "Rakothas",
                    1
                }
            }
        },
        --]]
        [9] = {
            ["Gultháva, Maiden of the Gladden"] = {
                -- Boss 1
                ["You must prove you are strong enough to triumph over the shadows below."] = {
                    1,
                    nil,
                    {
                        ["You are indeed strong. Please, help me remove the shadows that corrupt this place."] = {1, "Gultháva", true},
                    },
                    "Gultháva",
                    1
                }
            }
        },
        [38] = {
            ["Commander Formáth Firmhand"] = {
                ["This is it, dwarves! All we must do is survive! For Durin! For Náin!"] = {
                    1,
                    {
                        ["Retreat! He will slaughter us!"] = {1, 2},
                        ["Fall back!"] = {1, 3},
                        ["Blast! He keeps pushing us back!"] = {1, 4},
                        ["We are quickly running out of ground!"] = {1, 5},
                    },
                    {
                        ["We have no more time. Go, cross the bridge and live! I will hold him off!"] = {1, "Durin's Bane", true},
                    },
                    "Durin's Bane",
                    1
                }
            }
        },
        [57] = {
            ["Threkvegg"] = {
                --["It was by the will of Hrungrod that we, the heirs of Thyrstáth, sought to forge an alliance with the Dragon."] = {
                ["We shall cast you down to the frozen deeps!"] = {
                    1,
                    nil,
                    {
                        ["Our sacrifice must not be in vain...."] = {1, "Kvethár, Threkvegg & Armód", true},
                        ["We have failed you...."] = {1, "Kvethár, Threkvegg & Armód", true},
                        ["It seems we chose... poorly...."] = {1, "Kvethár, Threkvegg & Armód", true},
                        ["Vengeance for Hrungrod! All hail the Frost-heart!"] = {1, "Kvethár, Threkvegg & Armód", false},
                    },
                    "Kvethár, Threkvegg & Armód",
                    3
                },
            },
            ["Thrall-lord Dushtalbúk"] = {
                ["Death to Durin! Glory to the Frost-heart!"] = {
                    2,
                    {
                        ["Defend the Frost-heart! Let none escape!"] = {2, 2},
                        ["The Frost-horde will devour you!"] = {2, 3},
                        ["Kill them or it'll be the lash for the lot of you!"] = {2, 4},
                        ["I am Dushtalbúk and I. SERVE. NO ONE!"] = {2, 5},
                    },
                    {
                        ["But... I... am the... War... lord... I..."] = {2, "Thrall-lord Dushtalbúk", true},
                        ["Urbhármokh will never be yours!"] = {2, "Thrall-lord Dushtalbúk", false},
                    },
                    "Thrall-lord Dushtalbúk",
                    1
                },
            },
            ["Durin"] = {
                ["My kin and kindred! The moment is at hand!"] = {
                    3,
                    {
                        ["None can withstand the Frost-horde!"] = {3, 2},
                        ["The Orcs of Gundabad answer to me!"] = {3, 3},
                        ["Feast, my children!"] = {3, 4},
                        ["It is time you faced your fear!"] = {3, 5},
                    },
                    {
                        ["You are beaten, Hrímil."] = {3, "Hrímil Frost-heart", true},
                    },
                    "Hrímil Frost-heart",
                    1
                },
            },
        },
        [75] = {
            ["Tarkka-oka"] = {
                ["Time to see what Oganuin's sorceries have wrought!"] = {
                    1,
                    {},
                    {
                        ["May the garden... reclaim me...."] = {1, "Cráthair & Tarkka-oka", true},
                        ["It was not... enough, Oganuin."] = {1, "Cráthair & Tarkka-oka", true},
                    },
                    "Cráthair & Tarkka-oka",
                    2
                }
            },
            ["Grand Cultivator Oganuin"] = {
                ["Guards!"] = {
                    2,
                    {
                        ["Impertinent fools! You drag me from my rituals? You slay my assistants?"] = {2, 2},
                        ["Come forth, my children! Show them what glories I have created!"] = {2, 3},
                        ["Come forth my creations!"] = {2, 4},
                        ["Come forth my children!"] = {2, 5},
                    },
                    {
                        ["Bury me in... the garden...."] = {2, "Grand Cultivator Oganuin", true},
                    },
                    "Grand Cultivator Oganuin",
                    1
                }
            }
        },
        [78] = {
            ["Gurkrak"] = {
                ["None may oppose the Witch-queen of Angmar!"] = {
                    1,
                    {
                        --["Heal me, sluggards, or I'll have your heads!"] = {1, 2},
                    },
                    {
                        ["I... failed you... my queen..."] = {1, "Gurkrak", true}
                    },
                    "Gurkrak",
                    1,
                }
            },
            ["Castellan Obáshurz"] = {
                ["It seems I shall have to entertain you myself!"] = {
                    2,
                    {
                        ["'I tire of these hollow pleasantries. Perhaps a more formal welcome is in order!"] = {2, 2},
                    },
                    {
                        ["How... rude...."] = {2, "Castellan Obáshurz", true},
                    },
                    "Castellan Obáshurz",
                    1,
                }
            },
            ["High Chieftain Claghórd"] = {
                ["Let us hope you've got more fight in you than old Crannog!"] = {
                    3,
                    {
                        ["Do as you are told, Claghórd!"] = {3, 2},
                        --["We are... betrayed...."] = {3, 3},
                        ["My faithful servants, you have stood watch over this hall for long enough."] = {3, 3},
                    },
                    {
                        ["We are... betrayed...."] = {3, "High Priestess Ásachal & High Chieftain Claghórd", true},
                        ["But I... swore...."] = {3, "High Priestess Ásachal & High Chieftain Claghórd", true},
                    },
                    "High Priestess Ásachal & High Chieftain Claghórd",
                    2,
                }
            },
            [""] = {
                ["<rgb=#FF0000>Tauressar turns its malice upon you!</rgb>"] = {
                    4,
                    nil,
                    {

                    },
                    "Shard of Tauressar",
                    1,
                }
            }
        }
        --[[
        [32] = {
            -- boss 1
            ["Aigrith the Ashen"] = {
                ["Outsiders! You will be destroyed at the hands of Aigrith the Ashen! The Witch-King has commanded it!"] = {
                    1,
                    nil,
                    {
                        ["Useless Gauredain magic... not enough... I have failed you, my Master!"] = {1, "Aigrith the Ashen", true},
                        ["Argh! No! Gauradan... take my power, and keep your word to Angmar...."] = {1, "Aigrith the Ashen", true},
                        ["Ashes to ashes!"] = {1, "Aigrith the Ashen", false},
                    },
                    "Aigrith the Ashen"
                },


            },
            -- boss 2
            ["Rucrog Hoarpelt"] = {
                ["I have failed! Angmarim, I have one last boon to offer. Take it and gain victory!"] = {
                    2,
                    nil,
                    {
                        ["I have failed! Angmarim, I have one last boon to offer. Take it and gain victory!"] = {2, "Rucrog Hoarpelt", true},
                        ["Erk.... Your foul magic was not enough, Angmarim. I go to meet my ancestors...."] = {2, "Rucrog Hoarpelt", true}
                    },
                    "Rucrog Hoarpelt"
                }
            },
            -- Boss 3
            ["Captain Glamagon"] = {
                [Turbine.Gameplay.LocalPlayer.GetInstance():GetName() .. ", Lothrandir, be ready for anything!"] = {
                    3,
                    nil,
                    {
                        ["We have lost control! The ship is being torn apart!"] = {3, "Witch-King in Narrowdelve", true}
                    },
                    "Witch-King in Narrowdelve"
                }
            }

        },
        [33] = {
            -- Boss 1
            [""] = {
                ["Pesky vermin... these woods are ours!"] = {

                }
            }
        }
        --]]
    }

    --[[
        
        Gultháva, Maiden of the Gladden shouts, ''You must prove you are strong enough to triumph over the shadows below.''
        Gultháva, Maiden of the Gladden shouts, ''You are indeed strong. Please, help me remove the shadows that corrupt this place.''
    ]]
end