LootLockWindow = class(Turbine.UI.Window)
function LootLockWindow:Constructor(parent, loothandler)
    self.width = 700
    self.height = 650
    local font = Settings.LOFont
    --local colour = Turbine.UI.Color(1, 1, 1)

    self.loothandler = loothandler

    Turbine.UI.Window.Constructor(self)
    self:SetParent(parent)
    self:SetSize(self.width, self.height)
    self:SetPosition(0, 0)
    self:SetVisible(true)
    self:SetWantsKeyEvents(true)
    self:SetMouseVisible(false)
    --self:SetBackColor(colour)

    self.topButtonDictionary = {}

    self.ChangeTopButtonChange = function(button)
        for key, value in pairs(self.topButtonDictionary) do
            if value == button and button ~= self.active then
                key:SetBackColor(Turbine.UI.Color(0.125, 0.125, 0.125))
                self.active = button

            else
                if value == self.active then self.active = "" end
                key:SetBackColor(Turbine.UI.Color(0.25, 0.25, 0.25))
            end
        end

        if button == "Chests" then
            self.chestTreeview:SetVisible(true)
            self.searchBox:SetVisible(true)
        end

        if button == "Daily" then
            self.dailyView:SetVisible(true)
            self.dailyScrollbar:SetVisible(true)
        end

        if button == "Weekly" then
            self.weeklyView:SetVisible(true)
            self.weeklyScrollbar:SetVisible(true)
        end

        if button ~= "Chests" or self.active == "" then
            self.chestTreeview:SetVisible(false)
            self.searchBox:SetVisible(false)
        end

        if button ~= "Daily" or self.active == "" then
            self.dailyView:SetVisible(false)
            self.dailyScrollbar:SetVisible(false)
        end

        if button ~= "Weekly" or self.active == "" then
            self.weeklyView:SetVisible(false)
            self.weeklyScrollbar:SetVisible(false)
        end
    end

    self.chestButton = Turbine.UI.Label()
    self.chestButton:SetParent(self)
    self.chestButton:SetSize(math.ceil(self.width - 45) / 3, 30)
    self.chestButton:SetPosition(5, 0)
    self.chestButton:SetBackColor(Turbine.UI.Color(0.25, 0.25, 0.25))
    self.chestButton:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
    self.chestButton:SetFont(Turbine.UI.Lotro.Font.BookAntiquaBold24)
    self.chestButton:SetMouseVisible(true)
    self.chestButton:SetText("Chests")
    self.topButtonDictionary[self.chestButton] = "Chests"

    self.chestButton.MouseClick = function()
        self.ChangeTopButtonChange("Chests")
    end

    self.dailyButton = Turbine.UI.Label()
    self.dailyButton:SetParent(self)
    self.dailyButton:SetSize(math.ceil(self.width - 45) / 3, 30)
    self.dailyButton:SetPosition((self.width - 45) / 3 * 1 + 5, 0)
    self.dailyButton:SetBackColor(Turbine.UI.Color(0.25, 0.25, 0.25))
    self.dailyButton:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
    self.dailyButton:SetFont(Turbine.UI.Lotro.Font.BookAntiquaBold24)
    self.dailyButton:SetMouseVisible(true)
    self.dailyButton:SetText("Daily")
    self.topButtonDictionary[self.dailyButton] = "Daily"

    self.dailyButton.MouseClick = function()
        self.ChangeTopButtonChange("Daily")
    end

    self.weeklyButton = Turbine.UI.Label()
    self.weeklyButton:SetParent(self)
    self.weeklyButton:SetSize(math.ceil(self.width - 45) / 3 + 2, 30)
    self.weeklyButton:SetPosition((self.width - 45) / 3 * 2 + 5, 0)
    self.weeklyButton:SetBackColor(Turbine.UI.Color(0.25, 0.25, 0.25))
    self.weeklyButton:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
    self.weeklyButton:SetFont(Turbine.UI.Lotro.Font.BookAntiquaBold24)
    self.weeklyButton:SetMouseVisible(true)
    self.weeklyButton:SetText("Weekly")
    self.topButtonDictionary[self.weeklyButton] = "Weekly"

    self.weeklyButton.MouseClick = function()
        self.ChangeTopButtonChange("Weekly")
    end

    self:CreateChestView()

    self.dailyView, self.dailyScrollbar = self:CreateQuestView(self.loothandler:GetLootData()["LootDaily"])
    self.weeklyView, self.weeklyScrollbar = self:CreateQuestView(self.loothandler:GetLootData()["LootWeekly"])
    self.ChangeTopButtonChange("Chests")
end

function LootLockWindow:CreateQuestView(data)

    local questTreeview = Turbine.UI.TreeView()
    questTreeview:SetParent(self)
    questTreeview:SetSize(self.width - 10, self.height - 85)
    questTreeview:SetPosition(5, 60)
    questTreeview:SetVisible(false)
    questTreeview:SetWantsKeyEvents(true)
    questTreeview:SetMouseVisible(true)
    questTreeview:SetIndentationWidth(0)

    local questTreeviewScrollbar = Turbine.UI.Lotro.ScrollBar()
    questTreeviewScrollbar:SetOrientation(Turbine.UI.Orientation.Vertical)
    questTreeviewScrollbar:SetParent(self)
    questTreeviewScrollbar:SetPosition(2000 - 10, 46)
    questTreeviewScrollbar:SetSize(10, 46)
    questTreeview:SetVerticalScrollBar(questTreeviewScrollbar)

    local questTreeviewScrollbarHorizontal = Turbine.UI.Lotro.ScrollBar()
    questTreeviewScrollbarHorizontal:SetOrientation(Turbine.UI.Orientation.Horizontal)
    questTreeviewScrollbarHorizontal:SetParent(self)
    questTreeviewScrollbarHorizontal:SetSize(self.width - 10, 10)
    questTreeviewScrollbarHorizontal:SetPosition(5, 50)
    questTreeview:SetHorizontalScrollBar(questTreeviewScrollbarHorizontal)

    local questNodes = questTreeview:GetNodes()

    local chestHeader = Turbine.UI.TreeNode()
    chestHeader:SetBlendMode(Turbine.UI.BlendMode.AlphaBlend)
    chestHeader:SetBackColorBlendMode(Turbine.UI.BlendMode.AlphaBlend)
    --chestHeader:SetBackColor(Turbine.UI.Color(0.6, 1.0, 0.125, 0.125))
    chestHeader:SetSize(self.width, 30)
    chestHeader.name = "a"
    chestHeader.instance = false

    chestHeader.nameHeader = Turbine.UI.Label()
    chestHeader.nameHeader:SetParent(chestHeader)
    chestHeader.nameHeader:SetPosition(10, 0)
    chestHeader.nameHeader:SetSize(200, 30)
    chestHeader.nameHeader:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft)
    chestHeader.nameHeader:SetFont(Turbine.UI.Lotro.Font.BookAntiquaBold19)
    chestHeader.nameHeader:SetBlendMode(Turbine.UI.BlendMode.AlphaBlend)
    chestHeader.nameHeader:SetText("Quest Name")

    local names = {}
    local nameIndex = 0
    for chestName, lootData in pairs(data) do
        for playerName, _ in pairs(data[chestName]) do
            names[playerName] = true
        end
    end

    chestHeader:SetSize(math.max(tablelength(names) * 92 + 125, self.width - 10), 30)

    local nameList = {}
    for key, value in spairs(names, function(t, a, b) return a < b end) do
        chestHeader.divide = Turbine.UI.Control()
        chestHeader.divide:SetParent(chestHeader)
        chestHeader.divide:SetBlendMode(Turbine.UI.BlendMode.AlphaBlend)
        chestHeader.divide:SetBackColorBlendMode(Turbine.UI.BlendMode.AlphaBlend)
        chestHeader.divide:SetBackColor(Turbine.UI.Color(0.6, 0, 0, 0))
        chestHeader.divide:SetPosition(125 + 92 * nameIndex, 0)
        chestHeader.divide:SetSize(2, 30)

        chestHeader.nameLabel = Turbine.UI.Label()
        chestHeader.nameLabel:SetParent(chestHeader)
        chestHeader.nameLabel:SetPosition(137 + 92 * nameIndex, 0)
        chestHeader.nameLabel:SetSize(80, 30)
        chestHeader.nameLabel:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft)
        chestHeader.nameLabel:SetFont(Turbine.UI.Lotro.Font.BookAntiquaBold19)
        chestHeader.nameLabel:SetBlendMode(Turbine.UI.BlendMode.AlphaBlend)
        chestHeader.nameLabel:SetMultiline(false)
        chestHeader.nameLabel:SetText(tostring(string.sub(key, 1, 10)))

        nameList[key] = nameIndex
        nameIndex = nameIndex + 1
    end

    questNodes:Add(chestHeader)
    for questName, playerData in spairs(data, function(t, a, b) return a < b end) do
        local chestDivide = Turbine.UI.TreeNode()
        chestDivide:SetBlendMode(Turbine.UI.BlendMode.AlphaBlend)
        chestDivide:SetBackColorBlendMode(Turbine.UI.BlendMode.AlphaBlend)
        chestDivide:SetBackColor(Turbine.UI.Color(0.6, 0, 0, 0))
        chestDivide:SetSize(self.width, 2)
        chestDivide.name = "a"
        chestDivide.instance = false

        questNodes:Add(chestDivide)

        local chestItem = Turbine.UI.TreeNode()
        chestItem:SetBlendMode(Turbine.UI.BlendMode.AlphaBlend)
        chestItem:SetBackColorBlendMode(Turbine.UI.BlendMode.AlphaBlend)
        --chestItem:SetBackColor(Turbine.UI.Color(0.6, 1.0, 0.125, 0.125))
        chestItem:SetSize(self.width, 30)
        chestItem.name = questName

        chestItem.nameHeader = Turbine.UI.Label()
        chestItem.nameHeader:SetParent(chestItem)
        chestItem.nameHeader:SetPosition(10, 0)
        chestItem.nameHeader:SetSize(200, 30)
        chestItem.nameHeader:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft)
        chestItem.nameHeader:SetFont(Turbine.UI.Lotro.Font.BookAntiqua16)
        chestItem.nameHeader:SetBlendMode(Turbine.UI.BlendMode.AlphaBlend)
        chestItem.nameHeader:SetMultiline(false)
        chestItem.nameHeader:SetText(questName)

        for key, value in pairs(names) do
            chestItem.divide = Turbine.UI.Control()
            chestItem.divide:SetParent(chestItem)
            chestItem.divide:SetBlendMode(Turbine.UI.BlendMode.AlphaBlend)
            chestItem.divide:SetBackColorBlendMode(Turbine.UI.BlendMode.AlphaBlend)
            chestItem.divide:SetBackColor(Turbine.UI.Color(0.6, 0, 0, 0))
            chestItem.divide:SetPosition(125 + 92 * nameList[key], 0)
            chestItem.divide:SetSize(2, 30)
        end

        for playerName, chestData in pairs(playerData) do

            chestItem.nameHeader = Turbine.UI.Label()
            chestItem.nameHeader:SetParent(chestItem)
            chestItem.nameHeader:SetPosition(137 + 92 * nameList[playerName], 0)
            chestItem.nameHeader:SetSize(80, 30)
            chestItem.nameHeader:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft)
            chestItem.nameHeader:SetFont(Turbine.UI.Lotro.Font.FixedSys15)
            chestItem.nameHeader:SetBlendMode(Turbine.UI.BlendMode.AlphaBlend)
            chestItem.nameHeader:SetText(chestData)

        end               
        questNodes:Add(chestItem)
    end

    return questTreeview, questTreeviewScrollbarHorizontal
end

function LootLockWindow:CreateChestView()
    self.chestTreeview = Turbine.UI.TreeView()
    self.chestTreeview:SetParent(self)
    self.chestTreeview:SetSize(self.width - 10, self.height - 85)
    self.chestTreeview:SetPosition(5, 60)
    self.chestTreeview:SetVisible(true)
    self.chestTreeview:SetWantsKeyEvents(true)
    self.chestTreeview:SetMouseVisible(true)
    self.chestTreeview:SetIndentationWidth(0)

    self.chestTreeviewScrollbar = Turbine.UI.Lotro.ScrollBar()
    self.chestTreeviewScrollbar:SetOrientation( Turbine.UI.Orientation.Vertical )
    self.chestTreeviewScrollbar:SetParent( self )
    self.chestTreeviewScrollbar:SetPosition( 2000 - 10, 46 )
    self.chestTreeviewScrollbar:SetSize( 10,46 )
    self.chestTreeview:SetVerticalScrollBar( self.chestTreeviewScrollbar )

    self.chestNodes = self.chestTreeview:GetNodes()

    
    self.filtering = false

    self.filterFunction = function(object)
        if self.filtering and object.instance then 
            return not(string.find(string.lower(object.name), string.lower(self.filterKey)))
        elseif not(object.instance) then

        else
            return false
        end
    end

    self.sortFunction = function(object1, object2)
        if object1.name >= object2.name then 
            return 1
        else
            return -1
        end
    end

    self.searchBox = Turbine.UI.Lotro.TextBox()
    self.searchBox:SetParent(self)
    self.searchBox:SetSize(250, 20)
    self.searchBox:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
    self.searchBox:SetPosition(30, 40)
    self.searchBox:SetText("Instance Name...")

    self.searchBox.TextChanged = function()
        self.filterKey = self.searchBox:GetText()
        if string.len(self.filterKey) > 0 then
            self.filtering = true
            self.chestTreeview:Refresh()
        else
            self.filtering = false
            self.chestTreeview:Refresh()
        end
    end

    --self.chestTreeview:SetSortMethod(self.sortFunction)
    self.chestTreeview:SetFilter(self.filterFunction)
    self.chestTreeview:Refresh()

    for instanceName, lootData in spairs(self.loothandler:GetLootData()["LootChests"], function(t, a, b) return a < b end) do
        local instanceBar = Turbine.UI.TreeNode()
        instanceBar:SetBlendMode(Turbine.UI.BlendMode.AlphaBlend)
        instanceBar:SetBackColorBlendMode(Turbine.UI.BlendMode.AlphaBlend)
        --valuebar:SetBackColor(Turbine.UI.Color(0.6, 0.125, 0.125, 0.125))
        instanceBar:SetSize(self.width, 65)
        instanceBar.name = instanceName
        instanceBar.instance = true
    
        instanceBar.left = Turbine.UI.Control()
        instanceBar.left:SetParent(instanceBar)
        instanceBar.left:SetSize(80,80)
        instanceBar.left:SetBackground(BackgroundHandler:GetBackground(0x411C0E92))
        instanceBar.left:SetPosition(-5, -5)
        instanceBar.left:SetBlendMode(Turbine.UI.BlendMode.AlphaBlend)
    
        instanceBar.middle = Turbine.UI.Control()
        instanceBar.middle:SetParent(instanceBar)
        instanceBar.middle:SetSize(self.width - 155,80)
        instanceBar.middle:SetBackground(BackgroundHandler:GetBackground(0x411C0E94))
        instanceBar.middle:SetPosition(75, -5)
        instanceBar.middle:SetBlendMode(Turbine.UI.BlendMode.AlphaBlend)
    
        instanceBar.right = Turbine.UI.Control()
        instanceBar.right:SetParent(instanceBar)
        instanceBar.right:SetSize(80,80)
        instanceBar.right:SetBackground(BackgroundHandler:GetBackground(0x411C0E9D))
        instanceBar.right:SetPosition(self.width - 80, -5)
        instanceBar.right:SetBlendMode(Turbine.UI.BlendMode.AlphaBlend)
     
        instanceBar.label = Turbine.UI.Label()
        instanceBar.label:SetParent(instanceBar)
        instanceBar.label:SetSize(self.width, 70)
        instanceBar.label:SetPosition(30, 0)
        instanceBar.label:SetMouseVisible(false)
        instanceBar.label:SetBackColorBlendMode(Turbine.UI.BlendMode.AlphaBlend)
        --instanceBar.label:SetBackColor(Turbine.UI.Color(0.6, 0.125, 0.125, 0.125))
        instanceBar.label:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft)
        instanceBar.label:SetFont(Turbine.UI.Lotro.Font.TrajanProBold24)
        instanceBar.label:SetFontStyle(Turbine.UI.FontStyle.Outline)
        instanceBar.label:SetOutlineColor(Turbine.UI.Color(0, 0, 0))
        instanceBar.label:SetText(tostring(instanceName))

        local instanceNode = instanceBar:GetChildNodes()

        for tier, tierData in pairs(lootData) do
            local tierHeader = Turbine.UI.TreeNode()
            tierHeader:SetSize(self.width - 10, 30)
            tierHeader.name = tostring(tier)
            tierHeader.instance = false
            
            tierHeader.tierHeaderControl = Turbine.UI.Control()
            tierHeader.tierHeaderControl:SetParent(tierHeader)
            tierHeader.tierHeaderControl:SetSize(self.width - 10, 30)
        
            tierHeader.tierHeaderLeft = Turbine.UI.Control()
            tierHeader.tierHeaderLeft:SetParent(tierHeader)
            tierHeader.tierHeaderLeft:SetSize(34, 30)
            tierHeader.tierHeaderLeft:SetBlendMode(Turbine.UI.BlendMode.AlphaBlend)
            tierHeader.tierHeaderLeft:SetBackground(BackgroundHandler:GetBackground(0x4110A0E0))
        
            tierHeader.tierHeaderRight = Turbine.UI.Label()
            tierHeader.tierHeaderRight:SetParent(tierHeader)
            tierHeader.tierHeaderRight:SetPosition(self.width - 34 - 10, 0)
            tierHeader.tierHeaderRight:SetSize(34, 30)
            tierHeader.tierHeaderRight:SetBlendMode(Turbine.UI.BlendMode.AlphaBlend)
            tierHeader.tierHeaderRight:SetBackground(BackgroundHandler:GetBackground(0x4110A0DB))
        
            tierHeader.tierHeaderMiddle = Turbine.UI.Label()
            tierHeader.tierHeaderMiddle:SetParent(tierHeader)
            tierHeader.tierHeaderMiddle:SetPosition(34, 0)
            tierHeader.tierHeaderMiddle:SetSize(self.width - 68 - 10, 30)
            tierHeader.tierHeaderMiddle:SetBlendMode(Turbine.UI.BlendMode.AlphaBlend)
            tierHeader.tierHeaderMiddle:SetBackground(BackgroundHandler:GetBackground(0x4110A0D9))
        
            tierHeader.tierHeaderHeaderLabel = Turbine.UI.Label()
            tierHeader.tierHeaderHeaderLabel:SetParent(tierHeader)
            tierHeader.tierHeaderHeaderLabel:SetPosition(35, 0)
            tierHeader.tierHeaderHeaderLabel:SetSize(self.width - 10 - 100, 30)
            tierHeader.tierHeaderHeaderLabel:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft)
            tierHeader.tierHeaderHeaderLabel:SetFont(Turbine.UI.Lotro.Font.BookAntiquaBold22)
            tierHeader.tierHeaderHeaderLabel:SetBlendMode(Turbine.UI.BlendMode.AlphaBlend)
            tierHeader.tierHeaderHeaderLabel:SetText("Tier: " .. tostring(tier))
             
            tierHeader.tierHeaderHeaderLabel.MouseEnter = function()
                tierHeader.tierHeaderLeft:SetBackground(BackgroundHandler:GetBackground(0x4110A0DF))
                tierHeader.tierHeaderMiddle:SetBackground(BackgroundHandler:GetBackground(0x4110A0DD))
                tierHeader.tierHeaderRight:SetBackground(BackgroundHandler:GetBackground(0x4110A0DE))
            end
        
            tierHeader.tierHeaderHeaderLabel.MouseUp = function()
                tierHeader.tierHeaderLeft:SetBackground(BackgroundHandler:GetBackground(0x4110A0E0))
                tierHeader.tierHeaderMiddle:SetBackground(BackgroundHandler:GetBackground(0x4110A0D9))
                tierHeader.tierHeaderRight:SetBackground(BackgroundHandler:GetBackground(0x4110A0DB))
            end
        
            tierHeader.tierHeaderHeaderLabel.MouseDown = function()
                tierHeader.tierHeaderLeft:SetBackground(BackgroundHandler:GetBackground(0x4110A0D8))
                tierHeader.tierHeaderMiddle:SetBackground(BackgroundHandler:GetBackground(0x4110A0DA))
                tierHeader.tierHeaderRight:SetBackground(BackgroundHandler:GetBackground(0x4110A0DC))
            end
        
            tierHeader.tierHeaderHeaderLabel.MouseLeave = function()
                tierHeader.tierHeaderLeft:SetBackground(BackgroundHandler:GetBackground(0x4110A0E0))
                tierHeader.tierHeaderMiddle:SetBackground(BackgroundHandler:GetBackground(0x4110A0D9))
                tierHeader.tierHeaderRight:SetBackground(BackgroundHandler:GetBackground(0x4110A0DB))
            end
            
            instanceNode:Add(tierHeader)
            local tierNode = tierHeader:GetChildNodes()

            local chestHeader = Turbine.UI.TreeNode()
            chestHeader:SetBlendMode(Turbine.UI.BlendMode.AlphaBlend)
            chestHeader:SetBackColorBlendMode(Turbine.UI.BlendMode.AlphaBlend)
            --chestHeader:SetBackColor(Turbine.UI.Color(0.6, 1.0, 0.125, 0.125))
            chestHeader:SetSize(self.width, 30)
            chestHeader.name = "a"
            chestHeader.instance = false

            chestHeader.nameHeader = Turbine.UI.Label()
            chestHeader.nameHeader:SetParent(chestHeader)
            chestHeader.nameHeader:SetPosition(10, 0)
            chestHeader.nameHeader:SetSize(80, 30)
            chestHeader.nameHeader:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft)
            chestHeader.nameHeader:SetFont(Turbine.UI.Lotro.Font.BookAntiquaBold19)
            chestHeader.nameHeader:SetBlendMode(Turbine.UI.BlendMode.AlphaBlend)
            chestHeader.nameHeader:SetText("Chest Name")

            local names = {}
            local nameIndex = 0
            for chestName, lootData in pairs(tierData) do
                for playerName, _ in pairs(tierData[chestName]) do
                    names[playerName] = true
                end
            end

            local nameList = {}
            for key, value in pairs(names) do
                chestHeader.divide = Turbine.UI.Control()
                chestHeader.divide:SetParent(chestHeader)
                chestHeader.divide:SetBlendMode(Turbine.UI.BlendMode.AlphaBlend)
                chestHeader.divide:SetBackColorBlendMode(Turbine.UI.BlendMode.AlphaBlend)
                chestHeader.divide:SetBackColor(Turbine.UI.Color(0.6, 0, 0, 0))
                chestHeader.divide:SetPosition(125 + 92 * nameIndex, 0)
                chestHeader.divide:SetSize(2, 30)

                chestHeader.nameLabel = Turbine.UI.Label()
                chestHeader.nameLabel:SetParent(chestHeader)
                chestHeader.nameLabel:SetPosition(137 + 92 * nameIndex, 0)
                chestHeader.nameLabel:SetSize(80, 30)
                chestHeader.nameLabel:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft)
                chestHeader.nameLabel:SetFont(Turbine.UI.Lotro.Font.BookAntiquaBold19)
                chestHeader.nameLabel:SetBlendMode(Turbine.UI.BlendMode.AlphaBlend)
                chestHeader.nameLabel:SetMultiline(false)
                chestHeader.nameLabel:SetText(tostring(string.sub(key, 1, 10)))

                nameList[key] = nameIndex
                nameIndex = nameIndex + 1
            end

            tierNode:Add(chestHeader)

            local orderInfo = {}
            for chestName, lootData in pairs(tierData) do
                for playerName, chestData in pairs(tierData[chestName]) do
                    orderInfo[chestName] = chestData.Order
                end
            end

            for chestName, lootData in spairs(tierData, function(t, a, b) return orderInfo[a] < orderInfo[b] end) do
                local chestDivide = Turbine.UI.TreeNode()
                chestDivide:SetBlendMode(Turbine.UI.BlendMode.AlphaBlend)
                chestDivide:SetBackColorBlendMode(Turbine.UI.BlendMode.AlphaBlend)
                chestDivide:SetBackColor(Turbine.UI.Color(0.6, 0, 0, 0))
                chestDivide:SetSize(self.width, 2)
                chestDivide.name = "a"
                chestDivide.instance = false

                tierNode:Add(chestDivide)

                local chestItem = Turbine.UI.TreeNode()
                chestItem:SetBlendMode(Turbine.UI.BlendMode.AlphaBlend)
                chestItem:SetBackColorBlendMode(Turbine.UI.BlendMode.AlphaBlend)
                --chestItem:SetBackColor(Turbine.UI.Color(0.6, 1.0, 0.125, 0.125))
                chestItem:SetSize(self.width, 30)
                chestItem.name = chestName
    
                chestItem.nameHeader = Turbine.UI.Label()
                chestItem.nameHeader:SetParent(chestItem)
                chestItem.nameHeader:SetPosition(10, 0)
                chestItem.nameHeader:SetSize(120, 30)
                chestItem.nameHeader:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft)
                chestItem.nameHeader:SetFont(Turbine.UI.Lotro.Font.BookAntiqua16)
                chestItem.nameHeader:SetBlendMode(Turbine.UI.BlendMode.AlphaBlend)
                chestItem.nameHeader:SetMultiline(false)
                chestItem.nameHeader:SetText(chestName)

                for key, value in pairs(names) do
                    chestItem.divide = Turbine.UI.Control()
                    chestItem.divide:SetParent(chestItem)
                    chestItem.divide:SetBlendMode(Turbine.UI.BlendMode.AlphaBlend)
                    chestItem.divide:SetBackColorBlendMode(Turbine.UI.BlendMode.AlphaBlend)
                    chestItem.divide:SetBackColor(Turbine.UI.Color(0.6, 0, 0, 0))
                    chestItem.divide:SetPosition(125 + 92 * nameList[key], 0)
                    chestItem.divide:SetSize(2, 30)
                end

                for playerName, chestData in pairs(tierData[chestName]) do

                    chestItem.nameHeader = Turbine.UI.Label()
                    chestItem.nameHeader:SetParent(chestItem)
                    chestItem.nameHeader:SetPosition(137 + 92 * nameList[playerName], 0)
                    chestItem.nameHeader:SetSize(80, 30)
                    chestItem.nameHeader:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft)
                    chestItem.nameHeader:SetFont(Turbine.UI.Lotro.Font.FixedSys15)
                    chestItem.nameHeader:SetBlendMode(Turbine.UI.BlendMode.AlphaBlend)
                    if chestData.CompletionsRemaining == 0 then
                        chestItem.nameHeader:SetForeColor(Turbine.UI.Color(1, 0, 0))
                    end

                    if chestData.Type then
                        if chestData.Type == "favoured" then
                            chestItem.nameHeader:SetForeColor(Turbine.UI.Color(0, 1, 0))
                        elseif chestData.Type == "common" then
                            chestItem.nameHeader:SetForeColor(Turbine.UI.Color(1, 1, 0))
                        end
                        
                    end
                    chestItem.nameHeader:SetText(chestData.CompletionsRemaining)
                        

                end               
                tierNode:Add(chestItem)
            end
        end
    

        self.chestNodes:Add(instanceBar)
    end
end