import "PrimePlugins.RaidTools.NewVersionSetup.CombatTimerSetup"
import "PrimePlugins.RaidTools.NewVersionSetup.FinishSetup"
import "PrimePlugins.RaidTools.NewVersionSetup.SoundSetup"
import "PrimePlugins.RaidTools.NewVersionSetup.TargetSetup"
import "PrimePlugins.RaidTools.NewVersionSetup.DataSetup"
import "PrimePlugins.RaidTools.NewVersionSetup.WelcomeWagon"

FirstTimerOptions = class(Turbine.UI.Lotro.Window)
function FirstTimerOptions:Constructor()
    local width = 800
    local height = 600
    local x = Turbine.UI.Display.GetWidth() / 2 - width / 2
    local y = Turbine.UI.Display.GetHeight() / 2 - height / 2
    local colour = Turbine.UI.Color(1, 1, 1)
    Turbine.UI.Lotro.Window.Constructor(self)
    self:SetSize(width,height)
    self:SetPosition(x,y)
    self:SetText("Prime")
    self:SetVisible(true)

    -- Setup windows
    OptionControls = {}

    -- Welcome screen
    OptionControls[#OptionControls + 1] = FirstTimerWindowsWelcome(self)

    -- Data collection
    OptionControls[#OptionControls + 1] = FirstTimerData(self)

    -- Combat Timer
    OptionControls[#OptionControls + 1] = FirstTimerWindowsCombatTimer(self)

    -- Target window
    OptionControls[#OptionControls + 1] = FirstTimerWindowsTarget(self)

    -- Sound player
    OptionControls[#OptionControls + 1] = FirstTimerWindowsSound(self)

    -- Finish screen
    OptionControls[#OptionControls + 1] = FirstTimerWindowsFinish(self)

    OptionControls[1]:SetVisible(true)

    local OptionCount = #OptionControls
    local CurrentOptions = 1

    local NameLabel = Turbine.UI.Label()
    NameLabel:SetParent(self)
    NameLabel:SetSize(50, 20)
    NameLabel:SetPosition(375, 550)
    NameLabel:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
    NameLabel:SetMouseVisible(false)
    NameLabel:SetMarkupEnabled(true)
    NameLabel:SetFont(Turbine.UI.Lotro.Font.TrajanPro16)
    NameLabel:SetText(CurrentOptions .. " / " .. OptionCount)
    
    -- Create buttons
    local PrevButton = Turbine.UI.Lotro.Button()
    PrevButton:SetParent(self)
    PrevButton:SetSize(100, 15)
    PrevButton:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
    PrevButton:SetMouseVisible(true)
    PrevButton:SetPosition(275, 550)
    PrevButton:SetText("Previous")
    PrevButton:SetVisible(false)

    local NextButton = Turbine.UI.Lotro.Button()
    NextButton:SetParent(self)
    NextButton:SetSize(100, 15)
    NextButton:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
    NextButton:SetMouseVisible(true)
    NextButton:SetPosition(425, 550)
    NextButton:SetText("Next")

    
    local FinishButton = Turbine.UI.Lotro.Button()
    FinishButton:SetParent(self)
    FinishButton:SetVisible(false)
    FinishButton:SetSize(100, 15)
    FinishButton:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
    FinishButton:SetMouseVisible(true)
    FinishButton:SetPosition(425, 550)
    FinishButton:SetText("Finish")


    -- Handle state changes
    self.NewState = function()

        -- Handle the button state
        if CurrentOptions == 1 then
            PrevButton:SetVisible(false)
        elseif CurrentOptions > 1 then
            PrevButton:SetVisible(true)
        end

        if CurrentOptions == OptionCount then
            NextButton:SetVisible(false)
            FinishButton:SetVisible(true)
        elseif CurrentOptions < OptionCount then
            NextButton:SetVisible(true)
            FinishButton:SetVisible(false)
        end

        -- Handle the window state
        for key, value in pairs(OptionControls) do
            value:SetVisible(false)
        end
        OptionControls[CurrentOptions]:SetVisible(true)

        NameLabel:SetText(CurrentOptions .. " / " .. OptionCount)
    end

    -- Button Events
    PrevButton.Click = function()
        CurrentOptions = CurrentOptions - 1
        self.NewState()
    end

    NextButton.Click = function()
        CurrentOptions = CurrentOptions + 1
        self.NewState()
    end

    FinishButton.Click = function()
        Settings.FirstTime = false
        ReloadPlugin(true)
    end

    self.Closing = function()
        Settings.FirstTime = false
        ReloadPlugin(true)
    end
end

function CheckWindowTemplate(parent, setting, text, x, y)
    local cBox = Turbine.UI.Lotro.CheckBox()
    cBox:SetParent(parent)
    cBox:SetPosition(x, y)
    cBox:SetSize(300,20)
    cBox:SetChecked(setting)
    cBox:SetText(text)
    cBox:SetVisible(true)
    return cBox
end