NoteWindow=class(Turbine.UI.Window)
function NoteWindow:Constructor(noteData)
    Turbine.UI.Window.Constructor(self)

    assert(InstanceController ~= nil, "InstanceController is not loaded")
    assert(ChatEventHandler ~= nil, "Parser is not loaded")
    assert(rezTracker ~= nil, "Rez tracker is not loaded")
    assert(Updater ~= nil, "Update handler is not loaded")

    self.noteData = noteData
    self.tokens = {}
    self.timeEvents = {}
    self.startTime = 0
    self.nextTimeEvent = 2147483647
    self.timeTokensUsed = false
    self.timeIndex = 1
    self.dnfData = {}
    self.borderColour = UI.ColourTupleToTurbineColour(self.noteData.BorderColour)
    self.backgroundColour = UI.ColourTupleToTurbineColour(self.noteData.BackgroundColour)


    self.killTracking = KillTracking()

    self.classObjectToName = {
        [Turbine.Gameplay.Class.Burglar] = "Burglar",
        [Turbine.Gameplay.Class.Captain] = "Captain",
        [Turbine.Gameplay.Class.Champion] = "Champion",
  --      [Turbine.Gameplay.Class.Corsair] = "Corsair",
        [Turbine.Gameplay.Class.Guardian] = "Guardian",
        [Turbine.Gameplay.Class.Hunter] = "Hunter",
        [Turbine.Gameplay.Class.LoreMaster] = "LoreMaster",
        [Turbine.Gameplay.Class.Minstrel] = "Minstrel",
        [Turbine.Gameplay.Class.RuneKeeper] = "RuneKeeper",
        [Turbine.Gameplay.Class.Warden] = "Warden",
        [Turbine.Gameplay.Class.Beorning] = "Beorning",
        [Turbine.Gameplay.Class.Brawler] = "Brawler",
        [Turbine.Gameplay.Class.BlackArrow] = "BlackArrow",
        [Turbine.Gameplay.Class.Defiler] = "Defiler",
        [Turbine.Gameplay.Class.Reaver] = "Reaver",
        [Turbine.Gameplay.Class.Stalker] = "Stalker",
        [Turbine.Gameplay.Class.WarLeader] = "WarLeader",
        [Turbine.Gameplay.Class.Weaver] = "Weaver",
    } 

    self.stringToBool={
        ["true"]=true,
        ["false"]=false
    }

    self.player = Turbine.Gameplay.LocalPlayer.GetInstance()
    self.playerName = self.player:GetName()
    self.playerClass = self.classObjectToName[self.player:GetClass()]

    self:SetSize(self.noteData.Width , self.noteData.Height)
    self:SetPosition(self.noteData.X, self.noteData.Y)
    self:SetWantsKeyEvents(false)
    if self.noteData.InstanceLoad then 
        self:SetVisible(Settings.InstanceState) 
        self.instanceCallback = AddCallback(InstanceController, "InstanceEvent", function(args)
            self:SetVisible(args.State)
        end)
    else 
        self:SetVisible(self.noteData.Visible) 
    end
    --self:SetVisible(true) 

    self:SetMouseVisible(false)
    self:SetBackColor(self.borderColour)
    
    self.NoteBox = Turbine.UI.Label()
    self.NoteBox:SetParent(self)
    self.NoteBox:SetMouseVisible(false)
    self.NoteBox:SetSize(self.noteData.Width - self.noteData.BorderSize * 2, self.noteData.Height - self.noteData.BorderSize * 2)
    self.NoteBox:SetPosition(self.noteData.BorderSize, self.noteData.BorderSize)
    self.NoteBox:SetBackColor(self.backgroundColour)
    self.NoteBox:SetFont(Turbine.UI.Lotro.Font[self.noteData.Font])
    self.NoteBox:SetOutlineColor(Turbine.UI.Color.Black)
    self.NoteBox:SetFontStyle(Turbine.UI.FontStyle.Outline)
    self.NoteBox:SetMarkupEnabled(true)

    self.sayChatCallback = AddCallback(ChatEventHandler, "SayLine", function(message) self:ApplyEvent(message) end)
    self.defeatedChatCallback = AddCallback(ChatEventHandler, "Defeated", function(args) self.killTracking:ApplyKill(args.Name) end)
    self.fightEventStartCallbacks = AddCallback(InstanceController, "FightEventStart", function(args)
        self:ApplyEvent(args.Boss_Name) 
        self.startTime = Turbine.Engine.GetGameTime()
        if self.timeTokensUsed then
            if not(self.updateCallback) then
                self.updateCallback = AddCallback(Updater, "ExecuteUpdate", self.ExecuteUpdate)
            end
        end
    end)
    self.fightEventEndCallbacks = AddCallback(InstanceController, "FightEventEnd", function(args)
        self.killTracking:Clear()
        if self.updateCallback then RemoveCallback(Updater, "ExecuteUpdate", self.updateCallback) end
        self.updateCallback = nil
    end)
    self.dnfApplicationCallback = AddCallback(rezTracker, "DNFApplication", function(args) 
        self.dnfData[args.Name] = true
        self:AdjustDNFText()
    end)
    self.dnfRemovalCallback = AddCallback(rezTracker, "DNFRemoval", function(args)
        self.dnfData[args.Name] = false
        self:AdjustDNFText()
    end)

    self.ExecuteUpdate = function()
        --ChatOutput(self.startTime + self.nextTimeEvent)
        --ChatOutput(Turbine.Engine.GetGameTime())
        if self.startTime + self.nextTimeEvent <= Turbine.Engine.GetGameTime() then
            self:ApplyEvent(self.nextTimeEvent)
            local eventEnd = true
            for i = self.timeIndex, #self.timeEvents do
                if self.timeEvents[i] > self.nextTimeEvent then
                    self.nextTimeEvent = self.timeEvents[i]
                    self.timeIndex = i
                    eventEnd = false
                    break
                end
            end
            if eventEnd then
                self.timeIndex = 1
                if self.updateCallback then RemoveCallback(Updater, "ExecuteUpdate", self.updateCallback) end
                self.updateCallback = nil
            end
        end
    end

    self:CreateHandles()
    --self:AddText("#AddSpawn1:[A][6][Interrupt]{Rorhillion}{Job}")
    --self:AddText("#AddSpawn1:[T][6][Interrupt][PrimePlugins/RaidTools/Images/Kick-icon.jpg]{Rorhillion}{Job}")
    --self:AddText("#AddSpawn1:[T][6][Interrupt][PrimePlugins/RaidTools/Images/Kick-icon.jpg]{ }|Hunter|")
    --self:AddText("#x:[K][true][true]{Wild Tusker1Tusky}{Thieving Shrew1}{Biting Shrew2}{Thieving Shrew1}")
    --self:AddText("#x:[D][true][true]{Rorhillion}")
    --self:AddText("#AddSpawn2:[A,5,Interrupt]{Rolorion}{Job}")
    --self:AddText("TEST")
    --self:AddText("asdflkasjdf\nalsdjflaskjdf\nadfadf\nasdf")
    --self:ApplyEvent("AddSpawn1")
    --self:ApplyEvent("x")
    --self.killTracking:Clear()
    --self:ApplyEvent("x")
end

function NoteWindow:CreateHandles()
    -- Resizing handle
    self.ResizeHandle = Turbine.UI.Window()
    self.ResizeHandle:SetParent(self)
    self.ResizeHandle:SetVisible(true)
    self.ResizeHandle:SetSize(45, 45)
    self.ResizeHandle:SetOpacity(0.1)
    self.ResizeHandle:SetBackColor(self.backgroundColour)
    self.ResizeHandle:SetPosition(self:GetWidth() - self.ResizeHandle:GetWidth() - self.noteData.BorderSize, self:GetHeight() - self.ResizeHandle:GetHeight() - self.noteData.BorderSize)

    self.ResizeHandle.MouseDown = function(sender, args)
        sender.dragStartX = args.X
        sender.dragStartY = args.Y
        sender.dragging = true
    end
    
    self.ResizeHandle.MouseMove = function( sender, args )
        local width, height = self:GetSize()
    
        if sender.dragging then
            local newWidth = math.max(90, width + (args.X - sender.dragStartX))
            local newHeight = math.max(90, height + (args.Y - sender.dragStartY))
            self:SetSize(newWidth, newHeight)
            self.ResizeHandle:SetPosition(self:GetWidth() - self.ResizeHandle:GetWidth() - self.noteData.BorderSize, self:GetHeight() - self.ResizeHandle:GetHeight() - self.noteData.BorderSize)
            self:Layout()
        end
    end
    
    self.ResizeHandle.MouseUp = function( sender, args )
        sender.dragging = false
    end

    self.ResizeHandle.MouseEnter = function()
        self.ResizeHandle:SetOpacity(0.8)
    end

    self.ResizeHandle.MouseLeave = function()
        self.ResizeHandle:SetOpacity(0.1)
    end

    -- Reposition handle
    self.RepositionHandle = Turbine.UI.Window()
    self.RepositionHandle:SetParent(self)
    self.RepositionHandle:SetSize(45, 45)
    self.RepositionHandle:SetVisible(true)
    self.RepositionHandle:SetOpacity(0.1)
    self.RepositionHandle:SetBackColor(self.backgroundColour)
    self.RepositionHandle:SetPosition(0, 0)

    self.Moving = false

    self.RepositionHandle.MouseDown = function(sender, args)
        self.MoveX = args.X
        self.MoveY = args.Y
        self.Moving = true
    end
    
    self.RepositionHandle.MouseMove = function( sender, args )
        if self.Moving then
            local newX = self:GetLeft() - (self.MoveX - args.X)
            local newY = self:GetTop() - (self.MoveY - args.Y)

            self:SetPosition(math.min(Turbine.UI.Display.GetWidth() - self:GetWidth(), math.max( newX, 0)), math.min(Turbine.UI.Display.GetHeight() - self:GetHeight(), math.max( newY, 0)))
            end
    end
    
    self.RepositionHandle.MouseUp = function( sender, args )
        self.Moving = false
    end

    self.RepositionHandle.MouseEnter = function()
        self.RepositionHandle:SetOpacity(0.8)
    end

    self.RepositionHandle.MouseLeave = function()
        self.RepositionHandle:SetOpacity(0.1)
    end
end

function NoteWindow:Layout()
    self.NoteBox:SetSize(self.NoteBox:GetParent():GetWidth() - 2 * self.noteData.BorderSize, self.NoteBox:GetParent():GetHeight() - 2 * self.noteData.BorderSize)
end

function NoteWindow:AdjustDNFText()
    local text = self.NoteBox:GetText()
    for key, _ in pairs(self.dnfData) do
        if self.dnfData[key] then
            text = string.gsub(text, string.format("!%s", key), string.format("!<rgb=#00FF00>%s</rgb>", key))
        else
            text = string.gsub(text, string.format("!%s", key), string.format("!<rgb=#FF0000>%s</rgb>", key))
        end
    end
    --ChatOutput(text)
    self.NoteBox:SetText(text)
end

function NoteWindow:ClearText()
    self.NoteBox:SetText("")
end

function NoteWindow:Clear()
    self:ClearText()
    self.killTracking:Clear()
    self.tokens = {}
    self.dnfData = {}
    self.timeTokensUsed = false
    self.timeEvents = {}
    self.startTime = 0
    self.nextTimeEvent = 2147483647
    if self.updateCallback then RemoveCallback(Updater, "ExecuteUpdate", self.updateCallback) end
    self.updateCallback = nil
    self.timeIndex = 1
end

function NoteWindow:AddText(text)
    local firstLine = false
    local skipLine
    for k in string.gmatch(text, "([^\n]*)\n?") do
        local token = string.match(k, "#([%-%/%.%P]+):")
        if token then
            local minutes, seconds = string.match(token, "(%d+)%.(%d+)")
            if minutes and seconds then
                local timestamp = tonumber(minutes) * 60 + tonumber(seconds)
                table.insert(self.timeEvents, timestamp)
                token = timestamp
                self.timeTokensUsed = true
            end
            if not(self.tokens[token]) then
                self.tokens[token] = {} 
            end

            local noteEventTable =  {
                ["Names"] = {},
                ["Classes"] = {},
                ["Conditions"] = {}
            }

            local conditionIndex = 1
            for key, _ in string.gmatch(k, "%[([%-%/%.%P]+)%]") do
                noteEventTable.Conditions[conditionIndex] = key
                conditionIndex = conditionIndex + 1
            end
            if noteEventTable.Conditions[1] == "K" then
                self.NoteBox:AppendText("Kill Order:\n")
                firstLine = true
                skipLine = false
            end

            for key, _ in string.gmatch(k, "{([-<>%P]+)}") do
                if key ~= " " then
                    if noteEventTable.Conditions[1] == "K" then
                        table.insert(noteEventTable.Names, key)
                        --self.NoteBox:AppendText(tostring(string.match(tostring(key), "%D*")) .. "\n")
                        local name, number, nickname, icon = string.match(tostring(key), "([%D]+)(%d+)([-%P]*)<?([%-%/%.%P]*)>?")
                        if string.len(nickname) == 0 then
                            nickname = name
                        end
                        self.NoteBox:AppendText(string.format("%s - %d \n", tostring(nickname), number))
                    else
                        noteEventTable.Names[key] = true
                    end
                end
            end

            for key, _ in string.gmatch(k, "|(%w+)|") do
                noteEventTable.Classes[key] = true
            end

            if noteEventTable.Conditions[1] == "A" or noteEventTable.Conditions[1] == "a" or noteEventTable.Conditions[1] == "T"  then
                if noteEventTable.Names[self.playerName] or noteEventTable.Classes[self.playerClass] or
                (tablelength(noteEventTable.Names) == 0 and noteEventTable.Classes[self.playerClass]) or
                (tablelength(noteEventTable.Names) == 0 and tablelength(noteEventTable.Classes) == 0) 
                 then
                    skipLine = false
                    firstLine = true
                    self.NoteBox:AppendText(string.format("#%s: %s\n", tostring(token), tostring(noteEventTable.Conditions[3])))
                else
                    skipLine = true
                end
            elseif noteEventTable.Conditions[1] == "D"  then
                if rezTracker then
                    skipLine = false
                    firstLine = true
                    for key, value in pairs(noteEventTable.Names) do
                        if rezTracker:HasDNF(key) then
                            self.NoteBox:AppendText(string.format("!<rgb=#00FF00>%s</rgb>", key))
                            self.dnfData[key] = true
                        else
                            self.NoteBox:AppendText(string.format("!<rgb=#FF0000>%s</rgb>", key))
                            self.dnfData[key] = false
                        end
                    end
                    self.NoteBox:AppendText("\n")
                end
            end

            table.insert(self.tokens[token], noteEventTable)

        elseif k and string.len(k) > 0 then
            self.NoteBox:AppendText(k)
            self.NoteBox:AppendText("\n")
        end

        if k == "" and firstLine and not(skipLine) then
            self.NoteBox:AppendText("\n")
        end
    end

    if self.timeTokensUsed then
        table.sort(self.timeEvents, function(a, b) return a < b end)
        self.nextTimeEvent = self.timeEvents[1]
        self.timeIndex = 1
    end
    --self.NoteBox:AppendText(text)
    --self.NoteBox:AppendText("\n")
end

function NoteWindow:ApplyEvent(event)
    if self.tokens[event] then
        for key, value in pairs(self.tokens[event]) do
            if value.Names[self.playerName] or value.Classes[self.playerClass] or
            (tablelength(value.Names) == 0 and value.Classes[self.playerClass]) or
            (tablelength(value.Names) == 0 and tablelength(value.Classes) == 0)  or value.Conditions[1] == "K" then
                if value.Conditions[1] == "A" then
                    if EventWindows then
                        local icon = value.Conditions[4]
                        if tonumber(icon) then
                            icon = tonumber(icon)
                        end
                        EventWindows:NoteAlert(value.Conditions[3], tonumber(value.Conditions[2]), icon)
                    end
                elseif value.Conditions[1] == "a" then
                    if EventWindows then
                        local icon = value.Conditions[4]
                        if tonumber(icon) then
                            icon = tonumber(icon)
                        end
                        EventWindows:NoteSmallAlert(value.Conditions[3], tonumber(value.Conditions[2]), icon)
                    end
                elseif value.Conditions[1] == "T" then
                    if EventWindows then
                        local icon = value.Conditions[4]
                        if tonumber(icon) then
                            icon = tonumber(icon)
                        end
                        EventWindows:NoteTimer(value.Conditions[3], tonumber(value.Conditions[2]), icon, self.playerName)
                    end
                elseif value.Conditions[1] == "K" then
                    self.killTracking:Clear()
                    for _, names in pairs(value.Names) do
                        self.killTracking:AddItem(string.match(names, "([%D]+)(%d+)([-%P]*)<?([%-%/%.%P]*)>?"))
                    end
                    self.killTracking:ToggleHeader(self.stringToBool[value.Conditions[2]])
                    self.killTracking:SetVisible(self.stringToBool[value.Conditions[3]])
                end
            end
        end
    end
end

function NoteWindow:Unload(skipSaving)
    if self.instanceCallback then
        RemoveCallback(InstanceController, "InstanceEvent", self.instanceCallback)
        self.instanceCallback = nil
    end

    RemoveCallback(ChatEventHandler, "SayLine", self.sayChatCallback)
    self.sayChatCallback = nil

    RemoveCallback(ChatEventHandler, "Defeated", self.defeatedChatCallback)
    self.defeatedChatCallback = nil

    RemoveCallback(InstanceController, "FightEventStart", self.fightEventStartCallbacks)
    self.fightEventStartCallbacks = nil

    RemoveCallback(InstanceController, "FightEventEnd", self.fightEventEndCallbacks)
    self.fightEventEndCallbacks = nil

    RemoveCallback(rezTracker, "DNFApplication", self.dnfApplicationCallback)
    self.dnfApplicationCallback = nil

    RemoveCallback(rezTracker, "DNFRemoval", self.dnfRemovalCallback)
    self.dnfRemovalCallback = nil

    if not(skipSaving) then
        self.noteData.X, self.noteData.Y = self:GetPosition()
        self.noteData.Width, self.noteData.Height = self:GetSize() 
    end

    self:SetVisible(false)

    local controlList = self:GetControls()
    for i = 1, controlList:GetCount() do
        controlList:RemoveAt(1)
    end
    self:SetParent(nil)

    return self.noteData
end
