NoteHandling = class()
function NoteHandling:Constructor()
    self.noteData = self:ImportData()

    self.noteWindows = {}
    self:Load()
end

function NoteHandling:Load()
    for name, windowData in pairs(self.noteData.Windows) do
        if name == "Group" then
            self.noteWindows[name] = NoteWindow(windowData)
        end
    end
    self:SetNoteActive(self.noteData.CurrentNote)
end

function NoteHandling:ImportData()
    local data = LoadTable(Turbine.DataScope.Account, "PrimeNoteData")

    if not(data) then
        data = self:GetDefaultSettings()

        --data.Windows.Personal = self:AddNewWindow()
        --data.Windows.Personal.Visible = false
        data.Windows.Group = self:AddNewWindow()
    end

    return data
end

function NoteHandling:GetDefaultSettings()
    return {
        ["Windows"] = {},
        ["Notes"] = {},
        ["CurrentNote"] = ""
    }
end

function NoteHandling:AddNewWindow()
    return {
        X = 300,
        Y = 300,
        Width = 300,
        Height = 600,
        BackgroundColour = {[1] = 0.3, [2] = 0, [3] = 0,  [4] = 0},
        BorderColour = {[1] = 1, [2] = 0, [3] = 0,  [4] = 0},
        BorderSize = 1,
        Font = "Verdana16",
        InstanceLoad = false,
        Visible = false,
    }
end

function NoteHandling:ChangeNoteData(noteName, noteText)
    if self.noteData.Notes[noteName] then
        self.noteData.Notes[noteName] = noteText
    end
end

function NoteHandling:SetNoteActive(noteName)
    self.noteWindows.Group:Clear()
    self.noteData.CurrentNote = noteName
    if self.noteData.Notes[noteName] then
        self.noteWindows.Group:AddText(self.noteData.Notes[noteName])
    end
end

function NoteHandling:AddNewNote(noteName)
    if self.noteData then
        if self.noteData.Notes[noteName] then
            return false
        else
            self.noteData.Notes[noteName] = ""
            return true
        end
    end
end

function NoteHandling:GetNoteDataText(noteName)
    return self.noteData.Notes[noteName]
end

function NoteHandling:ClearNoteWindow()
    self.noteWindows.Group:Clear()
end

function NoteHandling:HanldeEvent(event)
    for key, value in pairs(self.noteWindows) do
       value:ApplyEvent(event)
    end
end

function NoteHandling:GetNoteData()
    return self.noteData.Windows
end

function NoteHandling:GetNoteTextData()
    return self.noteData.Notes
end

function NoteHandling:DeleteNote(noteName)
    self.noteData.Notes[noteName] = nil
end

function NoteHandling:Reload(data)
    for key, value in pairs(self.noteWindows) do
        if data then
            value:Unload(true)
        else
            self.noteData.Windows[key] = value:Unload()
        end
        self.noteWindows[key] = nil
    end

    if data then
        self.noteData.Windows.Group = data.Group
    end

    self:Load()
end

function NoteHandling:Unload()
    for key, value in pairs(self.noteWindows) do
        self.noteData.Windows[key] = value:Unload()
        self.noteWindows[key] = nil
    end

    SaveTable(Turbine.DataScope.Account, "PrimeNoteData", self.noteData)
end

function NoteHandling:AddText(text)
    self.noteWindows.Group:AddText(text)
end

function NoteHandling:ClearText()
    self.noteWindows.Group:ClearText()
end

function NoteHandling:Clear()
    self.noteWindows.Group:Clear()
end