ChatHandler = class(Turbine.UI.Control)
function ChatHandler:Constructor()
    Turbine.UI.Control.Constructor(self)
    
    assert(Settings ~= nil, "Settings are not loaded")

    local AlertsEnabled = Settings.AEnabled
    local SKTEnabled = Settings.SKTEnabled
    local SEnabled = Settings.SEnabled

    self.TyrantIcon = "PrimePlugins/RaidTools/Images/Tyrant.tga"
    self.KeenEyeIcon = "PrimePlugins/RaidTools/Images/KeenEyeIcon.tga"

    self.specialNames = {
        ["Rolorion"] = true,
        ["Rolthondor"] = true,
        ["Rogrim"] = true,
        ["Rotorin"] = true,
        ["Bazaquezufegs"] = true,
        ["Roloreorn"] = true,
        ["Tryggtrygg"] = true,
        ["Wyicknamoac"] = true,
        ["Snorpel"] = true,
        ["Rorrelm"] = true,
        ["Rodreon"] = true,
        ["Embalii"] = true,
        ["Embalia"] = true,
        ["Embalie"] = true,
        ["Embalir"] = true,
        ["Embalira"] = true,
        ["Embalin"] = true,
        ["Embalea"] = true,
        ["Flavorofthemonth"] = true,
        ["Leonmon"] = true,
        ["Leonthorn"] = true,
        ["Leonvil"] = true,
        ["Zokai"] = true,
        ["Wotsi"] = true,
        ["Tossed"] = true,
        ["Dirtar"] = true,
        ["Brantokh"] = true,
        ["Beeswarmarmourcrush"] = true,
        ["Testobject"] = true,
    }
    
    self.superSpecialNames = {
        ["Rolorion"] = true,
        ["Rolthondor"] = true,
        ["Rogrim"] = true,
        ["Rotorin"] = true,
        ["Bazaquezufegs"] = true,
        ["Roloreorn"] = true,
        ["Tryggtrygg"] = true,
        ["Wyicknamoac"] = true,
        ["Snorpel"] = true,
        ["Rorrelm"] = true,
        ["Rodreon"] = true,
        ["Testobject"] = true
    }

    self.ChatFunctions = {}

    self.playerCombatTriggers = {}

    self.player = Turbine.Gameplay.LocalPlayer.GetInstance()
    self.party = self.player:GetParty()

    self.callbacks = {}

    self.subscribers = {}

    self.ChatEvent = function(sender, event, args)
        if type(event)=="function" then
            event(args)
        else
            if type(event)=="table" then
                for i=1,#event do
                    if type(event[i])=="function" then
                        event[i](args)
                    end
                end
            end
        end
    end

    -- Set data
    self.leader = ""
    self.playerName = self.player:GetName()

    self.UsedChatChannels = {
        [Turbine.ChatType.Raid] = true,
        [Turbine.ChatType.Fellowship] = true,
        [Turbine.ChatType.Say] = true,
        [Turbine.ChatType.Quest] = true,
        [Turbine.ChatType.Standard] = true,
        [Turbine.ChatType.Death] = true,
        [Turbine.ChatType.EnemyCombat] = true,
        [Turbine.ChatType.Advancement] = true,
        [Turbine.ChatType.PlayerCombat] = true,
    }

    self.CheckCommandPermission = function(message, senderName)
        if senderName == self.leader or 
        ((self.playerName == self.leader or self.specialNames[self.playerName]) and 
        (string.find(message, "%[To Fellowship%]") or 
        string.find(message, "%[To Group%]") or 
        string.find(message, "%[To Raid%]"))) or 
        self.specialNames[senderName] or 
        GroupEffectsHandler:CheckGroupRat(senderName) or
        ((self.specialNames[self.playerName] or GroupEffectsHandler:CheckGroupRat(self.playerName)) and (string.find(message, "%[To Fellowship%]" or string.find(message, "%[To Group%]") or 
        string.find(message, "%[To Raid%]")))) then
            return(true)
        else
            return(false)
        end
    end

    self.cleanPartyMessage = function(message)

        local name, cleanedLine
        -- Fellowship
        if string.find(message, "%[To Fellowship%]") then
            cleanedLine = string.match(message, "%[To Fellowship%] (.*)")
        elseif string.find(message, "%[Fellowship%]") then
            name, cleanedLine = string.match(message, "%[Fellowship%] (.*): (.*)")
        -- Group
        elseif string.find(message, "%[To Group%]") then
            cleanedLine = string.match(message, "%[To Group%] (.*)")
        elseif string.find(message, "%[Group%]") then
            name, cleanedLine = string.match(message, "%[Group%] (.*): (.*)")
        -- Raid
        elseif string.find(message, "%[To Raid%]") then
            cleanedLine = string.match(message, "%[To Raid%] (.*)")
        elseif string.find(message, "%[Raid%]") then
            name, cleanedLine = string.match(message, "%[Raid%] (.*): (.*)")
        end

        if name then 
            return string.match(name, ">(.*)<\\Select>"), cleanedLine
        else
            return self.player:GetName(), cleanedLine
        end
    end

    -- Chat handling functions
    self.PartyChatHandling = function(message)
        -- Handle impossible messages :)
        if self.party == nil then return end

        -- Check name of chat sender
        --local senderName = self:ParseName(message)
        local senderName, cleanedLine = self.cleanPartyMessage(message)
        if cleanedLine then cleanedLine = cleanedLine:gsub("\n", "") end

        self.ChatEvent(self, self.Party, cleanedLine)
        self.ChatEvent(self, self.PartyInfo, {["Message"] = cleanedLine, ["Source"] = senderName})

        -- Handle raid commands
        if self.CheckCommandPermission(message, senderName) then   

            -- Countdown command        
            if string.find(message, "countdown") then
                local number = tonumber(string.sub(message, string.find(message, "countdown") + 10, string.len(message)))
                if number then                 
                    countdownWindow:Countdown(number)
                end
            end

            -- Timer command
            if string.find(message, "start timer") then
                local timerName = string.sub(message, string.find(message, "start timer") + 12, string.find(message, ",") - 1)
                local number = tonumber(string.sub(message, string.find(message, ",") + 2, string.len(message)))
                if number and SEnabled then skillCounters:NewTimer(number, timerName, true) end
            end
                
            -- Note commands
            if string.find(message, "note") then
                -- Add command
                if(string.find(message, "noteAdd")) then
                    NoteHandler:AddText(string.sub(message, string.find(message, "noteAdd") + 8, string.len(message)))
                end

                -- Clear command
                if(string.match(message, "noteClear")) then
                    NoteHandler:ClearText()
                end
            end

            -- Ready check events start
            if(string.find(message, " Initiate Readycheck")) then
                ReadyHandler:MakeNewReadyCheck()
            end

            -- Reload
            if(string.find(message, " Force Reload")) then
                ReloadPlugin()
            end
        end

        if self.superSpecialNames[senderName] or (self.superSpecialNames[self.player:GetName()] and (string.find(message, "%[To Fellowship%]") or string.find(message, "%[To Raid%]"))) then
            local cleanedLine = self.cleanPartyMessage(message):gsub("\n", "")

            if cleanedLine == "Curse of the Selector" then
                MouseCurse = UI.MouseDecoy()
            end

            if cleanedLine == "Flashbang" then
                FlashBang = UI.FlashBang()
            end

            if cleanedLine == "Wink" then
                Wink = UI.Wink()
            end

            if cleanedLine == "Helloooo" then
                DriveBy = UI.DriveBy()
            end

            if cleanedLine == "I like trains" then
                TrainBy = UI.TrainBy()
            end

            if cleanedLine == "Peekaboo" then
                Peekaboo = UI.Peekaboo()
            end

            if cleanedLine == "Make it rain" then
                MakeItRain = UI.MakeItRain()
            end
        end

        -- Amazing
        if senderName == "Arandour" or self.playerName == "Arandour" then
            if string.find(message, "Amazing") then
                Sounds:PlaySoundEffect(Sounds:GetSoundList()["Amazing"])
            end
        end


        -- Handle self ready
        --[[
        if ReadyHandler:ReadyCheckActive()  then
            if string.find(message, " Ready") and not(string.find(message, " Initiate Readycheck" )) then
                if self:ParseName(message) == "" then 
                    --ReadyHandler:ActionCheck()
                end
            end
        end
        --]]

        -- Handle readycheck udpates
        if ReadyHandler:ReadyCheckActive() then
            if string.find(message, " Ready") and not(string.find(message, " Initiate Readycheck" )) then
                local playerName = ""
                for word in string.gmatch(message, "%S+") do
                    if(word == "Not" or word == "Ready") then
                        break
                    end  
                    playerName = word
                end
                playerName = self:ParseName(message)
                if playerName == "" then
                    playerName = self.player:GetName()
                    ReadyHandler:ShowOverview()
                end

                if string.find(message, " Not Ready") then
                    ReadyHandler:ReadyUpdate(playerName, false)
                else
                    ReadyHandler:ReadyUpdate(playerName, true)
                end
            end
        end
    end

    self.ChatFunctions[Turbine.ChatType.Fellowship] = self.PartyChatHandling
    self.ChatFunctions[Turbine.ChatType.Raid] = self.PartyChatHandling

    self.SayChatHandling = function(message)
        local line, source = self:ParseTextLineOnly(message)
        self.ChatEvent(self, self.Say, message)
        self.ChatEvent(self, self.SayLine, line)
        self.ChatEvent(self, self.SayInfo, {["Source"] = source, ["Message"] = line})
    end

    self.ChatFunctions[Turbine.ChatType.Say] = self.SayChatHandling

    self.QuestChatHandling = function(message)
        -- Instance controller handling
        if InstanceController then InstanceController:CheckInstanceState(message) end

        -- Loot window handling
        if string.sub(message, 1, string.find(message, "%s") - 1) == "Completed:" then
            LootHandler:QuestParsing(string.sub(message, string.find(message, "%s") + 1, string.len(message) - 1))
        end

        if string.find(message, "New Quest") then
            local questName = string.match(message, "New Quest: (.*)")
            if questName then
                self.ChatEvent(self, self.QuestAccepted, {["Name"] = questName})
                return
            end
        end

        if string.find(message, "New Deed") then
            local deedName = string.match(message, "New Deed: (.*)")
            if deedName then
                self.ChatEvent(self, self.NewDeed, {["Name"] = deedName})
                return
            end
        end

        if string.find(message, "Completed:") then
            local questName = string.match(message, "Completed:\n(.*)\n")
            if questName then
                self.ChatEvent(self, self.QuestOrDeedCompleted, {["Name"] = questName})
                return
            end
        end

        if string.match(message, "(.*) %((%d+)/(%d+)%)") then
            local name, progress, total = string.match(message, "(.*) %((%d+)/(%d+)%)")
            self.ChatEvent(self, self.QuestOrDeedProgress, {["Name"] = name, ["Progress"] = progress, ["Total"] = total})
            return
        end
    end

    self.ChatFunctions[Turbine.ChatType.Quest] = self.QuestChatHandling

    -- Lootlock

    self.StandardChatHandling = function(message)
        self.ChatEvent(self, self.Standard, message)

        if string.find(message, "<rgb=#FF9900><Prime></rgb> ") then return end

        -- Instancecontroller chat handling
        if InstanceController then InstanceController:CheckInstanceState(message) end

        -- Loot window chat handling
        LootHandler:LootlockParsing(message)        
        --HandleNewLock(message)

        -- Handle party change reloading
        if string.find(message, "Fellowship") or string.find(message, "Raid") or string.find(message, "dismiss") or string.find(message, "raid") or string.find(message, "fellowship") then
            if string.find( message, " has joined your Fellowship." ) or 
            string.find( message, " has joined your Raid." ) or
            string.find( message, "You have joined a Fellowship." ) or
            string.find( message, "You have joined a Raid." ) or
            string.find( message, " has left your Fellowship." ) or
            string.find( message, " has left your Raid." ) or
            string.find( message, "You leave your Fellowship." ) or
            string.find( message, "You leave your Raid." ) or
            string.find( message, "You dismiss " ) or
            string.find( message, " from the Raid." ) or
            string.find( message, " has been dismissed from your Fellowship." ) or
            string.find( message, " has been dismissed from your Raid." ) or
            string.find( message, "You have been dismissed from your Raid." ) or
            string.find( message, "You have been dismissed from your Fellowship." ) or
            string.find( message, "Your Fellowship has been disbanded." ) or
            string.find( message, "Your Raid has been disbanded." ) or
            string.find( message, "You have joined the chat room for your raid, raid chat is now available." ) or
            string.find( message, "You have joined the chat room for your Fellowship, Fellowship chat is now available." ) or
            string.find( message, "the leader of the raid") or
            string.find( message, "the leader of the fellowship") then
                ReloadPlugin()
            end
        end

        -- Nice
        if string.find(message, "rolled (100): 69") and AlertsEnabled then
            local messageWindow = EventWindows:GetAlertWindow()
            if messageWindow then
                messageWindow:Message(3, "Nice!", nil, false)
            end
        end

        -- Handle track alerts
        if string.find(message, "You feel as though you are being followed...") and AlertsEnabled then
            local messageWindow = EventWindows:GetAlertWindow()
            if messageWindow then
                messageWindow:Message(3, "You are being tracked!", nil, false)
            end
            StatEventHandling:AddTrack()
        end
    end

    self.ChatFunctions[Turbine.ChatType.Standard] = self.StandardChatHandling

    --  You have freed Superior Uruk Guard from a daze!
    -- Charged Fulgurite Rune-stone has freed Superior Uruk Guard from a daze!
    -- Embalii has released Mordor Wight from being immobilized!

    self.DeathChatHandling = function(message)

        if string.find(message, "freed") then
            local culprit, target, ccType = string.match(message, "(.*) ha[sv]e? freed (.*) from a (.*)!")
            self.ChatEvent(self, self.CCBreak, {["Culprit"] = string.gsub(culprit, "You", self.player:GetName()), ["Target"] = string.gsub(target,"^[Tt]he ",""), ["Type"] = ccType})
        elseif string.find(message, "released") then
            local culprit, target = string.match(message, " (.*) ha[sv]e? released (.*) from being immobilized!")
            if culprit and target then
                self.ChatEvent(self, self.CCBreak, {["Culprit"] = string.gsub(culprit, "You", self.player:GetName()), ["Target"] = string.gsub(target,"^[Tt]he ",""), ["Type"] = "root"})
            end
        end

        -- Player death
        if string.find(message, "You succumb to your wounds.") then 
            self.ChatEvent(self, self.Retreat, {["Name"] = self.player:GetName()})
        elseif string.find(message, "Your fellow,") then
            local retreatName = string.match(message, "Your fellow, (.*) has been defeated!")
            if retreatName then
                self.ChatEvent(self, self.Retreat, {["Name"] = retreatName})
            end
        end

        InstanceController:ParsePlus(message)

        -- KB handling
        if string.find(message, "Your mighty blow defeated") then
            local kbName = string.gsub(string.match(message, "Your mighty blow defeated (.+)%."),"^[Tt]he ","")
            self.ChatEvent(self, self.Defeated, {["Source"] = self.player:GetName(), ["Name"] = kbName})
            StatEventHandling:AddKB(kbName)
        elseif string.find(message, "defeated") then
            local source, name = string.match(message, "(.*) defeated (.+)%.")
            if name then name = string.gsub(name, "^[Tt]he ","") end
            self.ChatEvent(self, self.Defeated, {["Source"] = source, ["Name"] =  name or string.match(message,"(.*) has been defeated.")})
        elseif string.find(message, "Your mighty blow topples") then
            local kbName = string.gsub(string.match(message, "Your mighty blow topples (.+)%."),"^[Tt]he ","")
            self.ChatEvent(self, self.Defeated, {["Source"] = self.player:GetName(), ["Name"] = kbName})
            StatEventHandling:AddKB(string.match(message, kbName), true)
        end

        -- Getting killed handling
        if string.find(message, " incapacitated you.") then
            local source = string.match(message, "(.+) incapacitated you.")
            StatEventHandling:AddDeath(source, false)
            self.ChatEvent(self, self.Defeated, {["Death_Source"] = source, ["Name"] =  self.player:GetName()})
        end

        -- Misadventure death handling
        if string.find(message, "You have been incapacitated by misadventure.") then
            StatEventHandling:AddDeath(nil, true)
            self.ChatEvent(self, self.Defeated, {["Death_Source"] = "Misadventure", ["Name"] =  self.player:GetName()})
        end
    end

    self.ChatFunctions[Turbine.ChatType.Death] = self.DeathChatHandling

    self.EnemyCombatChatHandling = function(message)
        --local source, _, skillName, rest = string.match(string.gsub(string.gsub(message,"<rgb=#......>(.*)</rgb>","%1"),"^%s*(.-)%s*$", "%1"),"^(.*) scored a (.*)hit with (.*) on (.*) for (.-) (.*)%.$")
        local cleanedMessage = string.gsub(string.gsub(message,"<rgb=#......>(.*)</rgb>","%1"),"^%s*(.-)%s*$", "%1")

        local source, _, skillName, rest = string.match(cleanedMessage,"^(.*) scored a (.*)hit with (.*) on (.*)%.$")
        if skillName == "Steadfast Barrage" then
            local messageWindow = EventWindows:GetAlertWindow()
            if messageWindow then
                messageWindow:Message(2, "Barrage from: <rgb=#FF0000>".. source .. "</rgb>", self.TyrantIcon, false)
            end
        end

        if skillName then
            if rest then
                local target, _ = string.match(rest,"^(.*) for (.*)$")
                if target then
                    self.ChatEvent(self, self.EnemyCombat, {["SkillName"] = skillName, ["Target"] = string.gsub(target, "^[Tt]he ", "")})
                else
                    self.ChatEvent(self, self.EnemyCombat, {["SkillName"] = skillName, ["Target"] = string.gsub(rest, "^[Tt]he ", "")})
                end
                return
            else
                self.ChatEvent(self, self.EnemyCombat, {["SkillName"] = skillName, ["Target"] = ""})
                return
            end
        end  

        if string.find(cleanedMessage, "tried to") then
            --Rolorion tried to use Warden's Taunt on the High Sorceress but she resisted the attempt.
            --The Angmarim tried to use a melee double attack on Rolorion but he parried the attempt.

            if string.find(cleanedMessage, " but it ") then
                local Source, SkillName, Target, FailType = string.match(cleanedMessage,"^(.*) tried to use (.*) on (.*) but it (.*) the attempt.$")

                if string.find(FailType, "was immune to") then
                    FailType = "immune"
                end

                if Target and Target == self.playerName then
                    self.ChatEvent(self, self.EnemyCombat, {["Source"] = Source, ["SkillName"] = SkillName, ["Target"] = string.gsub(Target, "^[Tt]he ", ""), ["Failed"] = true, ["FailType"] = FailType})
                    return
                end
            else
                local Source, SkillName, Target, FailType = string.match(cleanedMessage,"^(.*) tried to use (.*) on (.*) but s?he (.*) the attempt.$")

                if string.find(FailType, "was immune to") then
                    FailType = "immune"
                end
                
                if Target and Target == self.playerName then
                    self.ChatEvent(self, self.EnemyCombat, {["Source"] = Source, ["SkillName"] = SkillName, ["Target"] = string.gsub(Target, "^[Tt]he ", ""), ["Failed"] = true, ["FailType"] = "immune"})
                    return
                end
            end
        elseif string.find(cleanedMessage, "missed trying to use") then
            local Source, SkillName, Target = string.match(cleanedMessage, "^(.*) missed trying to use (.*) on (.*).$")

            if Target and Target == self.playerName then
                self.ChatEvent(self, self.EnemyCombat, {["Source"] = Source, ["SkillName"] = SkillName, ["Target"] = string.gsub(Target, "^[Tt]he ", ""), ["Failed"] = true, ["FailType"] = "missed"})
                return
            end
        end
    end

    self.ChatFunctions[Turbine.ChatType.EnemyCombat] = self.EnemyCombatChatHandling

    self.PlayerCombatChatHandler = function(message)
        local cleanedMessage = string.gsub(string.gsub(message,"<rgb=#......>(.*)</rgb>","%1"),"^%s*(.-)%s*$", "%1")

        --if self.tools then
        --    self.tools:ParseUpdate(cleanedMessage)
        --end

        --for key, value in pairs(self.subscribers) do key:ParseUpdate(cleanedMessage) end

     
        --Fighter of Shadow applied a heal to Rogrim restoring 34,476 points to Morale.
        --Mending Verse applied a heal to Rotorin restoring 2,946 points to Morale.
        --Rotorin applied a benefit with Mending Verse on Rotorin.
        --You have dispelled Vile Fury from The Watcher.
        if string.find(cleanedMessage, "hit") then
            --local source,_,skillName,target,_ = string.match(cleanedMessage,"^(.*) scored a (.*)hit with (.*) on (.*) for (.-) (.*)%.$")
            local source, _, skillName, rest = string.match(cleanedMessage,"^(.*) scored a (.*)hit with (.*) on (.*)%.$")

            if skillName then
                if rest then
                    local target, _ = string.match(rest,"^(.*) for (.*)$")
                    if target then
                        self.ChatEvent(self, self.PlayerCombat, {["SkillName"] = skillName, ["Target"] = string.gsub(target, "^[Tt]he ", "")})
                    else
                        self.ChatEvent(self, self.PlayerCombat, {["SkillName"] = skillName, ["Target"] = string.gsub(rest, "^[Tt]he ", "")})
                    end
                else
                    self.ChatEvent(self, self.PlayerCombat, {["SkillName"] = skillName, ["Target"] = ""})
                end
                return
            end  
        elseif string.find(cleanedMessage, "heal") then
            local skillName, _, target, _ = string.match(cleanedMessage,"^(.*) applied a (.*)heal to (.*) restoring (.*) points to Morale%.$")
            if skillName then
                self.ChatEvent(self, self.PlayerHeal, {["SkillName"] = skillName, ["Target"] = target})
                return
            end 
            
            -- source, type, skillName, target, number
            local _, _, skillName, target, _ = string.match(cleanedMessage, "^(.*) applied a (.*)heal with (.*) to (.*) restoring (.*) points to Morale%.$")
            if skillName then
                self.ChatEvent(self, self.PlayerHeal, {["SkillName"] = skillName, ["Target"] = target})
                return
            end
        elseif string.find(cleanedMessage, "benefit") then
            local source, _, skillName, target = string.match(cleanedMessage,"^(.*) applied a (.*)benefit with (.*) on (.*)%.$")
            if skillName then
                self.ChatEvent(self, self.PlayerBenefit, {["SkillName"] = skillName, ["Target"] = target, ["Source"] = source})
                return
            end 
        elseif string.find(cleanedMessage, "dispelled") then
            local effectName, target = string.match(cleanedMessage, "You have dispelled (.*) from (.*).")
            if effectName then
                self.ChatEvent(self, self.PlayerDispell, {["EffectName"] = effectName, ["Target"] = target})
                return
            end
        end
    end

    self.ChatFunctions[Turbine.ChatType.PlayerCombat] = self.PlayerCombatChatHandler

    self.AdvancementHandling = function(message)
        if string.find(message, "You have acquired") then
            Roles:ParseTraits(message)
        end
    end

    self.ChatFunctions[Turbine.ChatType.Advancement] = self.AdvancementHandling
--if true then return end
    -- Set parse function
    self.callbacks["Received"] = AddCallback( Turbine.Chat, "Received", function(sender, args)
        if self.ChatFunctions[args.ChatType] then
            self.ChatFunctions[args.ChatType](args.Message)
        end
    end)
end

function ChatHandler:RemoveCallbacks()
    for key, value in pairs(self.callbacks) do
        RemoveCallback(Turbine.Chat, key, value)
    end
end

function ChatHandler:SetPartyData()
    self.leader = self.party:GetLeader():GetName()
end

function ChatHandler:SubscribeTools(tools)
    self.tools = tools
end

function ChatHandler:TogglePlayerCombat()
end

function ChatHandler:Subscribe(object)
    self.subscribers[object] = true
end

function ChatHandler:UnSubscribe(object)
    self.subscribers[object] = false
end

function ChatHandler:ParseTextLineOnly(message)
    --message = message:gsub("\n[^\n]*$", "")
    --local _, _, line= string.match(message, ">(.*)<\\Select> (.*), '+(.*)''$")
    local source, totalMessage= string.match(message, ">(.*)<\\Select> shouts, (.*)")
    if not(totalMessage) then
        local source2, totalMessage2 = string.match(message, ">(.*)<\\Select> says, (.*)")
        source = source2
        totalMessage = totalMessage2
    end
    if totalMessage then
        local line = string.match(totalMessage, "''(.*)''")
        if line then
            return line, source or ""
        end
        line = string.match(totalMessage, "''(.*)'")
        if line then
            return line, source or ""
        end
        line = string.match(totalMessage, "'(.*)'")
        if line then
            return line, source or ""
        end
    end

    if string.find(message, "You say, ") then
        return string.match(message, "You say, '(.*)'") or "", ""
    else
        return string.gsub(message, "\n", ""), ""
    end
end

function ChatHandler:ParseName(message)
    local name = string.match(message, ">(.*)<\\Select>")
    return name or ""
end

function ChatHandler:ParsePartyName(message)
    local name = string.match(message, ">(.*)<\\Select>")
    return name or ""
end