AlertPrimeUI = class(Turbine.UI.Window)
function AlertPrimeUI:Constructor(parent, x, y, width, height, borderSize, scale, borderColour, backgroundColour, fontColour, font, alignment, text, durationFont, showDuration, durationAlignment, decimalThreshold, outlineColour)
    Turbine.UI.Window.Constructor(self)
    self:SetParent(parent)
    self:SetPosition(x, y)
    self:SetSize((width + 2 * borderSize) * scale, (height + 2 * borderSize) * scale)
    self:SetMouseVisible(false)
    self:SetBackColor(borderColour)
    self:SetVisible(false)

    self.outlineColour = Turbine.UI.Color(0,0,0)
    if outlineColour and outlineColour.A > 0 then
        self.outlineColour = outlineColour
    end
    
    self.height = height

    self.decimalThreshold = decimalThreshold

    self.innerbar = Turbine.UI.Label()
    self.innerbar:SetParent(self)
    self.innerbar:SetPosition(borderSize * scale + height, borderSize * scale)
    self.innerbar:SetMouseVisible(false)
    --self.innerbar:SetBlendMode(Turbine.UI.BlendMode.AlphaBlend)
    self.innerbar:SetBackColor(backgroundColour)
    self.innerbar:SetForeColor(fontColour)
    self.innerbar:SetOutlineColor(self.outlineColour)
    self.innerbar:SetFontStyle(Turbine.UI.FontStyle.Outline)
    self.innerbar:SetMarkupEnabled(true)
    self.innerbar:SetFont(font)
    self.innerbar:SetTextAlignment(alignment)
    self.innerbar:SetText(text)
    self.innerbar:SetSize(width * scale - height, height * scale)

    self.durationLabel = Turbine.UI.Label()
    self.durationLabel:SetParent(self)
    self.durationLabel:SetPosition(borderSize * scale, borderSize * scale)
    self.durationLabel:SetSize(height, height)
    self.durationLabel:SetMouseVisible(false)
    self.durationLabel:SetMultiline(false)
    --self.durationLabel:SetBlendMode(Turbine.UI.BlendMode.AlphaBlend)
    self.durationLabel:SetFontStyle(Turbine.UI.FontStyle.Outline)
    self.durationLabel:SetOutlineColor(self.outlineColour)
    self.durationLabel:SetForeColor(fontColour)
    self.durationLabel:SetFont(durationFont)
    self.durationLabel:SetTextAlignment(durationAlignment)
    self.durationLabel:SetText(text)
    self.durationLabel:SetVisible(showDuration)

    self.duration = 0


    self.ExecuteUpdate = function(delta)
        self.duration = self.duration - delta
        
        if self.duration < 1 then
            self:SetOpacity(self.duration + 0.3)
        end

        if self.duration > self.decimalThreshold then
            self.durationLabel:SetText(string.format("%i", self.duration))
        else
            self.durationLabel:SetText(string.format("%.1f", self.duration))
        end

        if self.duration <= 0 then
            self:SetVisible(false)
            if self.callback then RemoveCallback(Updater, "ExecuteUpdate", self.callback) end
            self.callback = nil
            if self.callbackFunction then
                self.callbackFunction()
            end
        end
    end

end

function AlertPrimeUI:SetAlert(time, text, icon, callbackFunction)
    self:SetVisible(true)
    self.innerbar:SetText(text)
    self:SetOpacity(1)
    self.duration = time
    self.callbackFunction = callbackFunction
    if icon then
        self.durationLabel:SetBackground(BackgroundHandler:GetBackground(tonumber(icon) or icon))
        self.durationLabel:SetStretchMode(2)
        self.durationLabel:SetStretchMode(1)
        self.durationLabel:SetSize(self.height, self.height)
    else
        self.durationLabel:SetStretchMode(nil)
        self.durationLabel:SetSize(self.height, self.height)
        self.durationLabel:SetBackground("")
    end
    if self.duration < self.decimalThreshold then
        self.durationLabel:SetText(string.format("%i", self.duration))
    else
        self.durationLabel:SetText(string.format("%.1f", self.duration))
    end

    if not(self.callback) then self.callback = AddCallback(Updater, "ExecuteUpdate", self.ExecuteUpdate) end
end

function AlertPrimeUI:Unload()
    self:SetVisible(false)
    local controlList = self:GetControls()
    for i = 1, controlList:GetCount() do
        controlList:RemoveAt(1)
    end
end

-----------------------------------------------------------------------------------------------------------------
-----------------------------------------------------------------------------------------------------------------
-----------------------------------------------------------------------------------------------------------------
-----------------------------------------------------------------------------------------------------------------

AlertGroupPrimeUI = class(Turbine.UI.Window)
function AlertGroupPrimeUI:Constructor(parent, x, y, width, height, borderSize, scale, borderColour, backgroundColour, alertBackgroundColour, alertBorderColour, fontColour, font, alignment, durationFont, showDuration, durationAlignment, decimalThreshold, outlineColour, alertHeight, alertBorderSize, reverseFill)
    Turbine.UI.Window.Constructor(self)
    self.width = width
    self.alertHeight = alertHeight

    self:SetParent(parent)
    self:SetPosition(x, y)
    self:SetSize((self.width + 2 * borderSize) * scale, (height + 2 * borderSize) * scale)
    self:SetMouseVisible(false)
    self:SetBackColor(borderColour)
    self:SetVisible(true)

    self.outlineColour = outlineColour
    self.decimalThreshold = decimalThreshold

    self.backgroundColour = backgroundColour
    self.fontColour = fontColour
    self.font = font 
    self.alignment = alignment
    self.durationFont = durationFont 
    self.showDuration = showDuration
    self.durationAlignment = durationAlignment
    self.decimalThreshold = decimalThreshold
    self.outlineColour = outlineColour
    self.alertBackgroundColour = alertBackgroundColour
    self.alertBorderColour = alertBorderColour
    self.alertBorderSize = alertBorderSize

    self.alertContainer = Turbine.UI.ListBox()
    self.alertContainer:SetParent(self)
    self.alertContainer:SetSize(self.width * scale, height * scale)
    self.alertContainer:SetPosition(borderSize, borderSize)
    --container:SetBackColor(GoldColor)
    self.alertContainer:SetReverseFill(reverseFill)
    self.alertContainer:SetMouseVisible(false)
    self.alertContainer:SetVisible(true)

    self.alerts = {}
    self.deleteList = {}
    self.deleteBool = false

    self.ExecuteUpdate = function(delta)
        for i = 1, self.alertContainer:GetItemCount() do
            local alert = self.alertContainer:GetItem(i)

            if alert:UpdateAlert(delta) then
                self.deleteList[alert] = true
                self.deleteBool = true
            end
        end

        if self.deleteBool then
            for key, _ in pairs(self.deleteList) do
                self.alertContainer:RemoveItem(key)
                self.deleteList[key] = nil
            end
            self.deleteList = {}
            self.deleteBool = false

            if self.alertContainer:GetItemCount() == 0 then
                if self.updateCallback then RemoveCallback(Updater, "ExecuteUpdate", self.updateCallback) end
                self.updateCallback = nil
            end
        end


    end
end

function AlertGroupPrimeUI:AddAlert(icon, duration, text, showIcon, callbackFunction)
    if not(self.startMode and self.startValue or not(self.startMode)) then return end
    self.alertContainer:AddItem(AlertGroupAlert(self.width, self.alertHeight, self.alertBackgroundColour, self.alertBorderColour, self.outlineColour, self.font, self.fontColour, self.durationFont, icon, text, duration, self.decimalThreshold, self.durationAlignment, self.alignment, self.alertBorderSize, showIcon, callbackFunction))
    if not(self.updateCallback) then self.updateCallback = AddCallback(Updater, "ExecuteUpdate", self.ExecuteUpdate) end
end

function AlertGroupPrimeUI:Unload()
    self:SetVisible(false)
    local controlList = self:GetControls()
    for i = 1, controlList:GetCount() do
        controlList:RemoveAt(1)
    end
end

function AlertGroupPrimeUI:SetupStartmode(value)
    self.startMode = value
end

function AlertGroupPrimeUI:ToggleStartValue(value)
    self.startValue = value
end

function AlertGroupPrimeUI:Clear()
    for i = 1, self.alertContainer:GetItemCount() do
        self.alertContainer:GetItem(i):SetVisible(false)
    end
    self.alertContainer:ClearItems()
    self.alerts = {}
    self.deleteList = {}
    if self.updateCallback then RemoveCallback(Updater, "ExecuteUpdate", self.updateCallback) end
    self.updateCallback = nil
end

AlertGroupAlert = class(Turbine.UI.Window)
function AlertGroupAlert:Constructor(width, height, backgroundColour, borderColour, outlineColour, font, fontColour, durationFont, icon, text, duration, decimalThreshold, durationAlignment, textAlignment, borderSize, showIcon, callbackFunction)
    Turbine.UI.Window.Constructor(self)
    self:SetSize(width, height)
    self:SetMouseVisible(false)
    self:SetBackColor(borderColour)
    self:SetVisible(true)

    self.duration = duration
    self.decimalThreshold = decimalThreshold
    self.callbackFunction = callbackFunction

    self.iconLabel = Turbine.UI.Label()
    self.iconLabel:SetParent(self)
    self.iconLabel:SetPosition(borderSize, borderSize)
    self.iconLabel:SetMouseVisible(false)
    --self.innerbar:SetBlendMode(Turbine.UI.BlendMode.AlphaBlend)

    if icon and showIcon then
        self.iconLabel:SetBackground(BackgroundHandler:GetBackground(tonumber(icon) or icon))
        self.iconLabel:SetStretchMode(2)
        self.iconLabel:SetStretchMode(1)
    end

    self.iconLabel:SetSize(height - 2 * borderSize, height - 2 * borderSize)
    self.iconLabel:SetForeColor(fontColour)
    self.iconLabel:SetTextAlignment(durationAlignment)
    self.iconLabel:SetFont(durationFont)
    self.iconLabel:SetOutlineColor(outlineColour)
    self.iconLabel:SetFontStyle(Turbine.UI.FontStyle.Outline)
    self.iconLabel:SetText(self.duration)
    self.iconLabel:SetMouseVisible(false)

    self.textLabel = Turbine.UI.Label()
    self.textLabel:SetParent(self)
    self.textLabel:SetPosition(height + borderSize, borderSize)
    self.textLabel:SetSize(width - height - 2 * borderSize, height - 2 * borderSize)
    self.textLabel:SetMouseVisible(false)
    --self.durationLabel:SetBlendMode(Turbine.UI.BlendMode.AlphaBlend)
    self.textLabel:SetTextAlignment(textAlignment)
    self.textLabel:SetForeColor(fontColour)
    self.textLabel:SetBackColor(backgroundColour)
    self.textLabel:SetFont(font)
    self.textLabel:SetOutlineColor(outlineColour)
    self.textLabel:SetFontStyle(Turbine.UI.FontStyle.Outline)
    self.textLabel:SetMarkupEnabled(true)
    self.textLabel:SetText(text)
    self.textLabel:SetStretchMode(1)
    self.textLabel:SetMouseVisible(false)
end

function AlertGroupAlert:UpdateAlert(delta)
    self.duration = self.duration - delta
    if self.duration <= 0 then 
        self:SetVisible(false)
        if self.callbackFunction then
            self.callbackFunction()
            self.callbackFunction = nil
        end
        return true
    end

    if self.duration < self.decimalThreshold then
        self.iconLabel:SetText(string.format("%.1f", self.duration))
    else
        self.iconLabel:SetText(string.format("%d", self.duration))
    end

    if self.duration < 1 then
        self:SetOpacity(self.duration + 0.3)
    end

end