CountdownWindowPrimeUI = class(Turbine.UI.Window)
function CountdownWindowPrimeUI:Constructor(parent, x, y, width, height, font, scale, fontColour, thresholdColour, threshold)
    Turbine.UI.Window.Constructor(self)
    self.width = width or 100
    self.height = height or 50
    self.x = x or 0
    self.y = y or 0
    self.cooldownTimer = -100
    self.lastNumber = -100
    self.minShow = 99999
    self.font = font or Turbine.UI.Lotro.Font.TrajanProBold36
    self.scale = scale or 1
    if fontColour and fontColour.A > 0 then
        self.fontColour = fontColour
    else
        self.fontColour = Turbine.UI.Color(.95,.67,.04)
    end

    if thresholdColour and thresholdColour.A > 0 then
        self.thresholdColour = thresholdColour
    else
        self.thresholdColour = Turbine.UI.Color(.40,.0,.0)
    end

    self.sounds = Sounds:GetSoundList()

    self.threshold = threshold or 5

    self:SetParent(parent)
    self:SetSize(self.width, self.height)
    self:SetPosition(self.x, self.y)
    self:SetZOrder(1)

    self:SetWantsKeyEvents(true)
    self:SetMouseVisible(false)
    self:SetVisible(true)

    self.countdownBox = Turbine.UI.Label()
    self.countdownBox:SetParent(self)
    self.countdownBox:SetSize(self.width, self.height)
    self.countdownBox:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
    self.countdownBox:SetFont(self.font)
    self.countdownBox:SetFontStyle(Turbine.UI.FontStyle.Outline)
    self.countdownBox:SetMouseVisible(false)
    self.countdownBox:SetVisible(false)

    self.GetCountdown = function()
        return(self.cooldownTimer)
    end

    self.ExecuteUpdate = function(delta)
        self.cooldownTimer = self.cooldownTimer - delta
        local wholeNumber = math.ceil(self.cooldownTimer)
    
        if wholeNumber ~= self.lastNumber then
            
            -- Handle < 5 colour change
            if(wholeNumber <= self.threshold) then
                self.countdownBox:SetForeColor(self.thresholdColour)
            else
                self.countdownBox:SetForeColor(self.fontColour)
            end

            if self.cooldownTimer <= self.minShow then
                self.countdownBox:SetVisible(true)

                if (wholeNumber <= 10) then
                    if self.sounds[tostring(wholeNumber)] ~= nil then
                        Sounds:PlaySoundEffect(self.sounds[tostring(wholeNumber)])
                    end
                end
            end
  
            if (wholeNumber >= 1) then
                self.countdownBox:SetText(wholeNumber)
            else
                if self.callbackFunction then 
                    self.callbackFunction()
                end
                self.countdownBox:SetVisible(false)
                RemoveCallback(Updater, "ExecuteUpdate", self.callback)
                self.callback = nil
            end
            
            self.lastNumber = wholeNumber
        end
    end

    self:SetStretchMode(1)
    self:SetSize(self.width * self.scale, self.height * self.scale)
end

function CountdownWindowPrimeUI:Countdown(duration, minShow, callbackFunction)
    self:SetZOrder(100)
    self.cooldownTimer = duration
    self.lastNumber = duration + 1
    self.countdownBox:SetText(math.ceil(self.cooldownTimer))
    self.callbackFunction = callbackFunction
    if minShow then
        self.minShow = minShow
    else
        self.minShow = self.lastNumber
    end
    if(math.ceil(self.cooldownTimer) <= self.threshold) then
        self.countdownBox:SetForeColor(self.thresholdColour)
    else
        self.countdownBox:SetForeColor(self.fontColour)
    end

    self.countdownBox:SetVisible(self.cooldownTimer <= self.minShow)

    if not(self.callback) then  self.callback = AddCallback(Updater, "ExecuteUpdate", self.ExecuteUpdate) end
end

function CountdownWindowPrimeUI:Unload()
    RemoveCallback(Updater, "ExecuteUpdate", self.callback)
    self.callback = nil
    local controlList = self:GetControls()
    for i = 1, controlList:GetCount() do
        controlList:RemoveAt(1)
    end
    self:SetParent(nil)
end