CounterListBox = class(Turbine.UI.Window)
function CounterListBox:Constructor(parent, width, height, itemHeight, x, y, textAlignment, textFont, textFontColour, countAlignment, countFont, countFontColour, itemBorderSize, itemBorderColour, itemBackgroundColour, textWidth, countWidth, events, hideWhileNotActive)
    Turbine.UI.Window.Constructor(self)
    self.width = width
    self.height = height
    self.itemHeight = itemHeight
    self.x = x
    self.y = y
    self.events = {}
    self.parent = parent
    self.hideWhileNotActive = hideWhileNotActive

    for key, value in pairs(events) do
        if not(self.events[value.Counter_Text]) then
            self.events[value.Counter_Text] = {}
        end
        if not(self.events[value.Counter_Text][value.Number]) then
            self.events[value.Counter_Text][value.Number] = {}
        end

        self.events[value.Counter_Text][value.Number][value.Event] = true
    end

    self.items = {}
    self.startMode = false
    self.startValue = false

    self:SetParent(parent)
    self:SetSize(self.width, self.height)
    self:SetPosition(self.x, self.y)
    self:SetVisible(true)
    self:SetMouseVisible(false)
    self:SetOpacity(1)

    self.ItemContainer = Turbine.UI.ListBox()
    self.ItemContainer:SetParent(self)
    self.ItemContainer:SetVisible(true)
    self.ItemContainer:SetSize(self.width, 1000)
    self.ItemContainer:SetPosition(0, 20)
    self.ItemContainer:SetMouseVisible(false)

    self.layoutTable = {
        ["textAlignment"] = textAlignment, 
        ["textFont"] = textFont, 
        ["countAlignment"] = countAlignment, 
        ["countFont"] = countFont,
        ["textFontColour"] = textFontColour,
        ["countFontColour"] = countFontColour,
        ["itemBorderColour"] = itemBorderColour, 
        ["itemBackgroundColour"] = itemBackgroundColour,
        ["itemBorderSize"] = itemBorderSize,
        ["textWidth"] = textWidth,
        ["countWidth"] = countWidth,
    }
end

function CounterListBox:AddItem(text, startCount, minVis, maxVis)
    if self.items[text] then return end
    self.items[text] = CounterItem(self.width, self.itemHeight, text, startCount, minVis, maxVis, self.layoutTable)
    if self.items[text]:IsItemVisible() then
        self.ItemContainer:AddItem(self.items[text])
    end
end

function CounterListBox:Trigger(text, increment, number)
    if self.items[text] and (self.startMode and self.startValue or not(self.startMode)) then
        local listState = self.items[text]:IsItemVisible()
        self.items[text]:ChangeCount(increment, number)
        if not(listState) and self.items[text]:IsItemVisible() then
            self.ItemContainer:AddItem(self.items[text])
        elseif listState and not(self.items[text]:IsItemVisible()) then
            self.ItemContainer:RemoveItem(self.items[text])
        end

        if self.events[text] and self.events[text][self.items[text]:GetCurrentCount()] then
            for key, _ in pairs(self.events[text][self.items[text]:GetCurrentCount()]) do
                self.parent:FireTrigger(key)
            end
        end
    end
end

function CounterListBox:SetupStartmode(value)
    self.startMode = value
    if value and not(self.startValue) then self.ItemContainer:SetVisible(not(self.hideWhileNotActive)) end
end

function CounterListBox:ToggleStartValue(value)
    self.startValue = value
    if self.startMode and self.hideWhileNotActive then self.ItemContainer:SetVisible(value) end
end

function CounterListBox:Reset()
    for _, value in pairs(self.items) do
        value:Reset()
        local listState = value:IsItemVisible()
        if not(listState) and value:IsItemVisible() then
            self.ItemContainer:AddItem(value)
        elseif listState and not(value:IsItemVisible()) then
            self.ItemContainer:RemoveItem(value)
        end
    end
end

function CounterListBox:Unload()
    self.ItemContainer:ClearItems()

    for key, value in pairs(self.items) do
        self.items[key]:Unload()
        self.items[key] = nil
    end
    self:SetParent(nil)
end

CounterItem = class(Turbine.UI.Window)
function CounterItem:Constructor(width, height, text, startCount, minVis, maxVis, layoutTable)
    Turbine.UI.Window.Constructor(self)
    self.layoutTable = layoutTable
    self.width = width  
    self.height = height 
    self.startCount = startCount
    self.currentCount = startCount
    self.minVis = minVis
    self.maxVis = maxVis

    self:SetSize(self.width, self.height)
    self:SetVisible(startCount >= self.minVis and startCount <=  self.maxVis)
    self:SetMouseVisible(false)
    self:SetBackColor(self.layoutTable.itemBorderColour)

    self.InnerControl = Turbine.UI.Control()
    self.InnerControl:SetParent(self)
    self.InnerControl:SetVisible(true)
    self.InnerControl:SetSize(self.width - self.layoutTable.itemBorderSize * 2, self.height - self.layoutTable.itemBorderSize * 2)
    self.InnerControl:SetPosition(self.layoutTable.itemBorderSize, self.layoutTable.itemBorderSize)
    self.InnerControl:SetMouseVisible(false)
    self.InnerControl:SetBackColor(self.layoutTable.itemBackgroundColour)

    self.TextLabel = Turbine.UI.Label()
    self.TextLabel:SetParent(self.InnerControl)
    self.TextLabel:SetVisible(true)
    self.TextLabel:SetSize(self.layoutTable.textWidth, self.height - self.layoutTable.itemBorderSize * 2)
    self.TextLabel:SetMouseVisible(false)
    self.TextLabel:SetForeColor(self.layoutTable.textFontColour)
    self.TextLabel:SetFont(self.layoutTable.textFont)
    self.TextLabel:SetTextAlignment(self.layoutTable.textAlignment)
    self.TextLabel:SetOutlineColor(Turbine.UI.Color.Black)
    self.TextLabel:SetFontStyle(Turbine.UI.FontStyle.Outline)
    self.TextLabel:SetMarkupEnabled(true)
    self.TextLabel:SetText(text)

    self.CountLabel = Turbine.UI.Label()
    self.CountLabel:SetParent(self.InnerControl)
    self.CountLabel:SetVisible(true)
    self.CountLabel:SetSize(self.layoutTable.countWidth, self.height - self.layoutTable.itemBorderSize * 2)
    self.CountLabel:SetMouseVisible(false)
    self.CountLabel:SetForeColor(self.layoutTable.countFontColour)
    self.CountLabel:SetFont(self.layoutTable.countFont)
    self.CountLabel:SetTextAlignment(self.layoutTable.countAlignment)
    self.CountLabel:SetOutlineColor(Turbine.UI.Color.Black)
    self.CountLabel:SetFontStyle(Turbine.UI.FontStyle.Outline)
    self.CountLabel:SetText(startCount)
end

function CounterItem:ChangeCount(increment, number)
    if increment then
        self.currentCount = self.currentCount + number
    else
        self.currentCount = self.currentCount - number
    end
    self.CountLabel:SetText(self.currentCount)
    self:SetVisible(self.currentCount >= self.minVis and self.currentCount <=  self.maxVis)
end

function CounterItem:Reset()
    self.currentCount = self.startCount
    self.CountLabel:SetText(self.currentCount)
    self:SetVisible(self.currentCount >= self.minVis and self.currentCount <=  self.maxVis)
end

function CounterItem:IsItemVisible()
    return (self.currentCount >= self.minVis and self.currentCount <=  self.maxVis)
end

function CounterItem:GetCurrentCount()
    return self.currentCount
end

function CounterItem:Unload()
    self:SetVisible(false)
    self:SetParent(nil)
end