PrimeUIEffectBox = class(Turbine.UI.Control)
function PrimeUIEffectBox:Constructor(effect, size, scale, spacing, border, borderColour, text, font, fontAlignment, visibitilityThreshold, showStack, stackFont, stackAlignment)
    Turbine.UI.Control.Constructor( self )
    self:SetSize(size, size)

    self.text = text

    self.name = effect:GetName()

    self.applicationTime = Turbine.Engine.GetGameTime()
    self.visibitilityThreshold = visibitilityThreshold or 999999

    self.duration = effect:GetDuration() - (Turbine.Engine.GetGameTime() - effect:GetStartTime())

    self.borderControl = Turbine.UI.Control()
    self.borderControl:SetParent(self)
    self.borderControl:SetPosition(spacing, spacing)
    self.borderControl:SetSize(scale * (size - spacing), scale * (size - spacing))
    self.borderControl:SetMouseVisible(false)
    self.borderControl:SetVisible(true)
    self.borderControl:SetBackColor(borderColour)

    self.iconLabel = Turbine.UI.Lotro.EffectDisplay()
    self.iconLabel:SetEffect(effect)
    self.iconLabel:SetParent(self.borderControl)
    self.iconLabel:SetPosition(border, border)
    self.iconLabel:SetSize(scale * (size - spacing - border), scale * (size - spacing - border))
    self.iconLabel:SetMouseVisible(false)
    self.iconLabel:SetVisible(true)

    self.CountDownLabel = Turbine.UI.Label()
    self.CountDownLabel:SetParent(self.iconLabel)
    self.CountDownLabel:SetSize(self.iconLabel:GetWidth(), self.iconLabel:GetHeight())
    self.CountDownLabel:SetMouseVisible(false)
    self.CountDownLabel:SetVisible(true)
    self.CountDownLabel:SetStretchMode(1)
    self.CountDownLabel:SetTextAlignment(fontAlignment)
    self.CountDownLabel:SetFont(font)
    self.CountDownLabel:SetFontStyle(Turbine.UI.FontStyle.Outline)
    self.CountDownLabel:SetOutlineColor(Turbine.UI.Color(0, 0, 0))
    self.CountDownLabel:SetMultiline(false)
    self.CountDownLabel:SetZOrder(100)

    if showStack then
        self.StackLabel = Turbine.UI.Label()
        self.StackLabel:SetParent(self.iconLabel)
        self.StackLabel:SetSize(self.iconLabel:GetWidth(), self.iconLabel:GetHeight())
        self.StackLabel:SetMouseVisible(false)
        self.StackLabel:SetVisible(true)
        self.StackLabel:SetStretchMode(1)
        self.StackLabel:SetTextAlignment(stackAlignment)
        self.StackLabel:SetFont(stackFont)
        self.StackLabel:SetFontStyle(Turbine.UI.FontStyle.Outline)
        self.StackLabel:SetOutlineColor(Turbine.UI.Color(0, 0, 0))
        self.StackLabel:SetMultiline(false)
        self.StackLabel:SetZOrder(100)
        self.StackLabel:SetText(string.match(effect:GetName(), "%d+"))
    end

    self.iconLabel:SetStretchMode(1)
    self.iconLabel:SetSize(scale * (size - spacing - border), scale * (size - spacing - border))
    self.lastDuration = math.ceil(self.duration)

    self:SetSize(size * scale, size * scale)

    self.ExecuteUpdate = function(delta)
        if self.duration > 999999 then return end
        self.duration = self.duration - delta
    
        if self.text then
    
            self.CountDownLabel:SetStretchMode(nil)
            self.CountDownLabel:SetStretchMode(1)
    
            if self.duration > 3600 then
                self.CountDownLabel:SetText(string.format("%dh", self.duration /3600))
            elseif self.duration > 120 then
                self.CountDownLabel:SetText(string.format("%dm",self.duration /60))
            elseif self.duration > 5 then
                self.CountDownLabel:SetText(string.format("%d", self.duration))
            elseif self.duration >= 0 then
                self.CountDownLabel:SetText(string.format("%.1f", self.duration))
            elseif self.duration < 0 then
                self.CountDownLabel:SetText("")
                self:SetVisible(false)
                if self.updateCallback then RemoveCallback(Updater, "ExecuteUpdate", self.updateCallback) end
            end
    
            self.CountDownLabel:SetVisible(self.visibitilityThreshold > self.duration)
        end
    end
    
    if self.text then
        if not(self.updateCallback) then self.updateCallback = AddCallback(Updater, "ExecuteUpdate", self.ExecuteUpdate) end
    end
end

function PrimeUIEffectBox:ExecuteUpdate(delta)

end

function PrimeUIEffectBox:GetValue()
    return self.duration
end

function PrimeUIEffectBox:GetName()
    return self.name
end

function PrimeUIEffectBox:GetApplicationTime()
    return self.applicationTime
end

function PrimeUIEffectBox:Unload()
    if self.updateCallback then RemoveCallback(Updater, "ExecuteUpdate", self.updateCallback) end
    self.updateCallback = nil
    self:SetVisible(false)
    self.iconLabel:SetEffect(nil)
    self.borderControl:SetParent(nil)
    self.iconLabel:SetParent(nil)
    self.CountDownLabel:SetParent(nil)
    local controlList = self:GetControls()
    for i = 1, controlList:GetCount() do
        controlList:RemoveAt(1)
    end
    self:SetParent(nil)
end