PrimeUIHandler = class(Turbine.UI.Control)
function PrimeUIHandler:Constructor()
    Turbine.UI.Control.Constructor(self)

    self.PRIMEUI_VERSION = 4

    self.tools = {}

    self.player = Turbine.Gameplay.LocalPlayer.GetInstance()

    self.ClassLut = {
        [Turbine.Gameplay.Class.Burglar] = "Burglar",
        [Turbine.Gameplay.Class.Captain] = "Captain",
        [Turbine.Gameplay.Class.Champion] = "Champion",
    --    [Turbine.Gameplay.Class.Corsair] = "Corsair",
        [Turbine.Gameplay.Class.Guardian] = "Guardian",
        [Turbine.Gameplay.Class.Hunter] = "Hunter",
        [Turbine.Gameplay.Class.LoreMaster] = "LoreMaster",
        [Turbine.Gameplay.Class.Minstrel] = "Minstrel",
        [Turbine.Gameplay.Class.RuneKeeper] = "RuneKeeper",
        [Turbine.Gameplay.Class.Warden] = "Warden",
        [Turbine.Gameplay.Class.Beorning] = "Beorning",
        [Turbine.Gameplay.Class.Brawler] = "Brawler",
        [Turbine.Gameplay.Class.BlackArrow] = "BlackArrow",
        [Turbine.Gameplay.Class.Defiler] = "Defiler",
        [Turbine.Gameplay.Class.Reaver] = "Reaver",
        [Turbine.Gameplay.Class.Stalker] = "Stalker",
        [Turbine.Gameplay.Class.WarLeader] = "WarLeader",
        [Turbine.Gameplay.Class.Weaver] = "Weaver",
    } 

    self.DungeonNameTable = InstanceController:GetDungeonNameTable()
end

function PrimeUIHandler:Initialize()
    self.UIData = LoadTable(Turbine.DataScope.Account, "PrimeUI") or {}
    self.CodeData = LoadTable(Turbine.DataScope.Account, "PrimeUIData") or {}

    self:Setup()
end

function PrimeUIHandler:Setup()
    for key, value in pairs(self.UIData) do
        if not(value.Version) or value.Version ~= self.PRIMEUI_VERSION or true then
            self:UpdateData(value)
        end
        if self:Permission(value) then
            --self.tools[value.Name] = PrimeUIObjects.PrimeUI(value.PrimeUI, value.Name, value.X, value.Y, value.AdvancedMode, value.OOCLoad, value.ICLoad, value.Anchor_Advanced_X, value.Anchor_Advanced_Y)
            -- [[
            local success, result = pcall(PrimeUIObjects.PrimeUI, value.PrimeUI, value.Name, value.X, value.Y, value.AdvancedMode, value.OOCLoad, value.ICLoad, value.Anchor_Advanced_X, value.Anchor_Advanced_Y, self.CodeData[value.Name])
            if success then
                self.tools[value.Name] = result
            else
                ChatOutput(string.format("Failed to properly load PrimeUI \"%s\".", value.Name))
            end--]]
        end
    end
end

function PrimeUIHandler:Permission(data)   
    if data.NeverLoad then return false end
    return not(data.NeverLoad) and
        (data.OnlyAllDelvings == nil or data.OnlyAllDelvings and Settings.DelvingMissionActive or data.OnlyAllDelvings == false) and
        (data.Delving == nil or data.Delving == "All" or data.Delving == Settings.MissionQuest and Settings.DelvingMissionActive) and
        (data.All_Delving_Tiers == nil or data.All_Delving_Tiers or data.All_Delving_Tiers == false and tostring(data.Delving_Tier) == tostring(Settings.DelvingTier)) and 
        (data.OnlyPvP and Settings.EttenmoorsState or not(data.OnlyPvP)) and
        (data.ClassLoad["All"] == true or data.ClassLoad[self.ClassLut[self.player:GetClass()]]) and 
        (data.RoleLoad["All"] or data.RoleLoad[Roles:GetRoleText()]) and 
        (data.LineLoad["All"] or data.LineLoad[Roles:GetLineText()]) and
        (data.InstanceLoad["All"] and not(data.AnyInstance) or Settings.InstanceState and data.AnyInstance or data.InstanceLoad[self.DungeonNameTable[Settings.InstanceVersion]] and not(data.AnyInstance)) and 
        (not(data.CharLoad) or data.CharacterLoad and data.CharacterLoad[self.player:GetName()]) and
        (data.AllTiers == nil or data.AllTiers or data.AllTiers == false and data.InstanceTier[Settings.InstanceTier])
end

function PrimeUIHandler:RecheckPermissions()
    for key, value in pairs(self.UIData) do
        local load = self:Permission(value)
        if load then
            -- Load if not loaded
            if not(self.tools[value.Name]) then
                local success, result = pcall(PrimeUIObjects.PrimeUI, value.PrimeUI, value.Name, value.X, value.Y, value.AdvancedMode, value.OOCLoad, value.ICLoad, value.Anchor_Advanced_X, value.Anchor_Advanced_Y, self.CodeData[value.Name])
                --self.tools[value.Name] = PrimeUIObjects.PrimeUI(value.PrimeUI, value.Name, value.X, value.Y, value.AdvancedMode, value.OOCLoad, value.ICLoad, value.Anchor_Advanced_X, value.Anchor_Advanced_Y)
                if success then
                    self.tools[value.Name] = result
                    self.tools[value.Name]:ReloadWithDelay()
                else
                    ChatOutput(string.format("Failed to properly load PrimeUI \"%s\".", value.Name))
                end
            else
                self.tools[value.Name]:ReloadWithDelay()
            end
        else
            -- Unload if loaded
            if self.tools[value.Name] then
                self.tools[value.Name]:UnloadObject()
                self.tools[value.Name]:DeleteObject()
                self.tools[value.Name] = nil
            end
        end
    end
end

function PrimeUIHandler:LoadUI()
    for key, value in pairs(self.tools) do       
        value:Setup()
    end
end

function PrimeUIHandler:AddUI(data)
    if not(self.tools[data.Name]) then
        if data.Version and data.Version > self.PRIMEUI_VERSION then
            ChatOutput("Plugin version too low for this PrimeUI")
            return false
        end
        table.insert(self.UIData, data)
        if not(data.Version) or data.Version ~= self.PRIMEUI_VERSION then
            self:UpdateData(data)
        end
        --self.tools[data.Name] = PrimeUIObjects.PrimeUI(data.PrimeUI, data.Name, data.X, data.Y, data.AdvancedMode, data.OOCLoad, data.ICLoad, data.Anchor_Advanced_X, data.Anchor_Advanced_Y)
        local success, result = pcall(PrimeUIObjects.PrimeUI, data.PrimeUI, data.Name, data.X, data.Y, data.AdvancedMode, data.OOCLoad, data.ICLoad, data.Anchor_Advanced_X, data.Anchor_Advanced_Y, self.CodeData[data.Name])
        if success then
            self.tools[data.Name] = result
        else
            ChatOutput(string.format("Failed to properly load PrimeUI \"%s\".", data.Name))
        end
        return true
    else
        ChatOutput("Failed to add PrimeUI: Rejected by handler")
        return false
    end
end

function PrimeUIHandler:UnloadUI()
    for key, value in pairs(self.tools) do
        self.CodeData[key] = value:UnloadObject(true)
    end
end

function PrimeUIHandler:GetData()
    return self.UIData
end

function PrimeUIHandler:Refresh(name, newData)
    if self.tools[name] then
        self.tools[name]:UnloadObject()
        self.tools[name]:Setup(newData)
    end
end

function PrimeUIHandler:RefreshAndReorder(name, newData)
    if self.tools[name] then
        self.tools[name]:UnloadObject()
        self.tools[name]:Setup(newData)
    end
end

function PrimeUIHandler:Move(name)
    if self.tools[name] then
        if self.tools[name].groupMode then
            local table = self.tools[name]:ToggleAdvancedMove()
            for key, value in pairs(self.UIData) do
                if value.Name == name then
                    self.UIData[key].PrimeUI = table
                    local X, Y = self.tools[name]:GetAdvancedPosition()
                    if X and Y then
                        self.UIData[key].Anchor_Advanced_X = X
                        self.UIData[key].Anchor_Advanced_Y = Y
                    end
                end
            end
        else
            local X, Y = self.tools[name]:ToggleMove()
            for key, value in pairs(self.UIData) do
                if value.Name == name then
                    self.UIData[key].X = X
                    self.UIData[key].Y = Y
                end
            end
        end
    end
end

function PrimeUIHandler:ChangeUIMetaData(newData)
    for key, value in pairs(self.UIData) do
        if newData.Name == value.Name then
            self.UIData[key] = newData
            if self:Permission(newData) then
                if self.tools[newData.Name] then
                    self:Refresh(newData.Name, self.UIData[key]["PrimeUI"])
                end
            elseif self.tools[newData.Name] then
                self.tools[newData.Name]:UnloadObject()
            end
            self.UIData[key] = newData
            break
        end
    end

    self:RecheckPermissions()
end

function PrimeUIHandler:Delete(name)
    for key, value in pairs(self.UIData) do
        if value.Name == name then
            if self.tools[name] then
                self.tools[name]:UnloadObject()
                self.tools[name]:DeleteObject()
                self.tools[name] = nil
            end
            self.UIData[key] = nil
        end
    end
end

function PrimeUIHandler:GetUI(name)
    for key, value in pairs(self.UIData) do
        if value.Name == name then
            return self.UIData[key]
        end
    end
end


function PrimeUIHandler:MoveAll()
    for key, value in pairs(self.UIData) do
        if self.tools[value.Name] then
            if self.tools[value.Name].groupMode then
                self.UIData[key].PrimeUI = self.tools[value.Name]:ToggleAdvancedMove()
            else
                local X, Y = self.tools[value.Name]:ToggleMove()
                self.UIData[key].X = X
                self.UIData[key].Y = Y
            end 
        end
    end
end

function PrimeUIHandler:Save()
    SaveTable(Turbine.DataScope.Account, "PrimeUI", self.UIData)
    SaveTable(Turbine.DataScope.Account, "PrimeUIData", self.CodeData)
end

function PrimeUIHandler:ReloadWithDelay()
end

function PrimeUIHandler:HideWindow(hide)
    for key, value in pairs(self.tools) do
        value:Hide(hide)
    end
end

function PrimeUIHandler:GetPrimeUICount()
    return tablelength(self.tools)
end


function PrimeUIHandler:GetTotalPrimeUICount()
    return tablelength(self.UIData)
end

function PrimeUIHandler:UpdateData(data)

    -- Data 1 changes
    if type(data.ClassLoad) ~= "table" then
        data.ClassLoad = {
            [data.ClassLoad] = true,
        }
    end

    if type(data.LineLoad) ~= "table" then
        data.LineLoad = {
            [data.LineLoad] = true,
        }
    end

    if type(data.RoleLoad) ~= "table" then
        data.RoleLoad = {
            [data.RoleLoad] = true,
        }
    end

    --3
    if type(data.InstanceLoad) ~= "table" then
        data.InstanceLoad = {
            [data.InstanceLoad] = true,
        }
    end

    if data.InstanceTier == nil then data.InstanceTier = {} end 
    
    if type(data.InstanceTier) ~= "table" then
        data.InstanceTier = {
            [data.InstanceTier] = true,
        }
    end

    data.Version = self.PRIMEUI_VERSION
end

function PrimeUIHandler:GetBaseData(Name, AdvancedMode)
    return {
        ["Version"] = self.PRIMEUI_VERSION,
        ["InstanceLoad"] = {["All"] = true},
        ["AnyInstance"] = false,
        ["RoleLoad"] = {["All"] = true,},
        ["ClassLoad"] = {["All"] = true,},
        ["CharLoad"] = false,
        ["ZoneLoad"] = false,
        ["GroupLoad"] = false,
        ["CharacterLoad"] = {},
        ["LineLoad"] = {["All"] = true,},
        ["ICLoad"] = true,
        ["OOCLoad"] = true,
        ["NeverLoad"] = false,
        ["OnlyPvP"] = false,
        ["OnlyAllDelvings"] = false,
        ["Delving"] = "All",
        ["Delving_Tier"] = 1,
        ["All_Delving_Tiers"] = true,
        ["X"] = 100,
        ["Y"] = 100,
        ["Anchor_Advanced_X"] = 0,
        ["Anchor_Advanced_Y"] = 0,
        ["Name"] = Name,
        ["Author"] = Turbine.Gameplay.LocalPlayer.GetInstance():GetName(),
        ["Edits"] = {},
        ["PrimeUI"] = {},
        ["AdvancedMode"] = AdvancedMode
    }
end