function PrimeUI:Alert(data, parent)
    local x = data.X
    local y = data.Y
    if not(self.groupMode) then
        x = self:ContentAlignement(data["Alignment"], (data["Width"] + 2 * data["BorderSize"]) * data["Scale"])
        y = self.currentHeight
    end

    local valuebar = AlertPrimeUI(parent, x, y, data.Width, data.Height, data.BorderSize, data.Scale, self:ColourTupleToTurbineColour(data.BorderColour), self:ColourTupleToTurbineColour(data.BackgroundColour), self:ColourTupleToTurbineColour(data.Font_Colour), self:FontToFont(data.Font), Turbine.UI.ContentAlignment[data.TextAlignment], data.Text, self:FontToFont(data.Duration_Font), data.Show_Duration, Turbine.UI.ContentAlignment[data.DurationTextAlignment], data.Decimal_Threshold, self:ColourTupleToTurbineColour(data.Outline_Colour))
    self.lookup[data.NickName or ""] = valuebar
    
    if self.groupMode then
        valuebar.moveBox = MoveBox(valuebar, data["Width"], data["Height"], data["NickName"] or "Alert")
        valuebar.savePosition = function()
            if valuebar.moveBox.MoveBox:IsVisible() then
                valuebar:SetVisible(true)
            else
                valuebar:SetVisible(false)
            end
            data["X"], data["Y"] = valuebar:GetPosition()
        end
    end

    valuebar.nameFunction = function(text, name, args)
        return ReplaceTokens(string.gsub(text, "<name>", NameHandlerObject:GetName(name, true, "PrimeUITimerGroup", "")), args or {})
    end

    if tablelength(data["EventTrigger"]) > 0 then
        table.insert(self.eventTriggerCallbacks, AddCallback(self, "EventTrigger", 
        function(event)
            local value = data["EventTrigger"][event]
            if value then
                valuebar:SetAlert(value.Countdown, valuebar.nameFunction(value.Text, self.name, self.args[event]), value["Icon"], value["Event_Keyword"] and function() self:FireTrigger(value["Event_Keyword"]) end)
            end
        end))
    end

    if tablelength(data["EffectAdded"]) > 0 then
        table.insert(self.effectAddedCallbacks, AddCallback(PlayerEffectsHandler, "EffectAdded", 
        function(effect)
            local value = data["EffectAdded"][effect:GetName()]
            if value then
                local time = value.Countdown
                if time == true then
                    time = effect:GetDuration() - (Turbine.Engine.GetGameTime() - effect:GetStartTime())
                end

                local icon
                if value["Show_Icon"] then
                    icon = value["Icon"]
                    if value["Icon"] == true then
                        icon = effect:GetIcon()
                    end
                end

                if value["Match_Icon"] then
                    if tostring(icon) ~= tostring(effect:GetIcon()) then return end
                end

                valuebar:SetAlert(time, valuebar.nameFunction(value.Text, self.name), icon, value["Event_Keyword"] and function() self:FireTrigger(value["Event_Keyword"]) end)
            end
        end))
    end

    if tablelength(data["EffectAdded_Target"]) > 0 then
        table.insert(self.effectTargetAddedCallbacks, TargetHandlerObject:AddEffectCallback( 
        function(args)
            local value = data["EffectAdded_Target"][args.Name]
            if value then
                local time = value.Countdown
                if time == true then
                    time = args.Duration - (Turbine.Engine.GetGameTime() - args.StartTime)
                end

                
                local icon
                if value["Show_Icon"] then
                    icon = value["Icon"]
                    if value["Icon"] == true then
                        icon = args.Icon
                    end
                end

                if value["Match_Icon"] then
                    if tostring(icon) ~= tostring(args.Icon) then return end
                end

                valuebar:SetAlert(time, value.Text, icon, value["Event_Keyword"] and function() self:FireTrigger(value["Event_Keyword"]) end)
            end
        end))
    end

    if tablelength(data["EffectAdded_Group"]) > 0 then
        table.insert(self.effectAddedGroupCallbacks, AddCallback(GroupEffectsHandler, "EffectAdded", 
        function(args)
            local value = data["EffectAdded_Group"][args.effect.Name]
            if value then
                local time = value.Countdown
                local effectArgs = args.effect
                if time == true then
                    time = effectArgs.Duration - (Turbine.Engine.GetGameTime() - effectArgs.StartTime)
                end

                local icon
                if value["Show_Icon"] then
                    icon = value["Icon"]
                    if value["Icon"] == true then
                        icon = effectArgs.Icon
                    end
                end

                if value["Match_Icon"] then
                    if tostring(icon) ~= tostring(effectArgs.Icon) then return end
                end

                valuebar:SetAlert(time, valuebar.nameFunction(value.Text, args.name), icon, value["Event_Keyword"] and function() self:FireTrigger(value["Event_Keyword"]) end)
            end
        end))
    end

    if tablelength(data["EffectRemoved"]) > 0 then
        table.insert(self.effectRemovedCallbacks, AddCallback(PlayerEffectsHandler, "EffectRemoved", 
        function(effect)
            local value = data["EffectRemoved"][effect:GetName()]
            if value then
                local time = value.Countdown
                if time == true then
                    time = effect:GetDuration() - (Turbine.Engine.GetGameTime() - effect:GetStartTime())
                end

                local icon
                if value["Show_Icon"] then
                    icon = value["Icon"]
                    if value["Icon"] == true then
                        icon = effect:GetIcon()
                    end
                end

                if value["Match_Icon"] then
                    if tostring(icon) ~= tostring(effect:GetIcon()) then return end
                end

                valuebar:SetAlert(time, valuebar.nameFunction(value.Text, self.name), icon, value["Event_Keyword"] and function() self:FireTrigger(value["Event_Keyword"]) end)
            end
        end))
    end

    if tablelength(data["Skills"]) > 0 then
        self.skills = true
        for key, value in pairs(data["Skills"]) do
            if SkillEventHandler:SkillExists(key) then
                -- Add the callback for this skill slot
                if not(self.skillCallbacks[key]) then self.skillCallbacks[key] = {} end
                table.insert(self.skillCallbacks[key], AddCallback(SkillEventHandler, key, function(skill) 
                    local time = value.Countdown
                    if time == true then
                        time = skill:GetResetTime() - Turbine.Engine.GetGameTime()
                    end
                    if (skill:GetResetTime() - Turbine.Engine.GetGameTime()) + 1 <= skill:GetCooldown() then
                        return
                    end
                    valuebar:SetAlert(time, value.Text, value["Icon"], value["Event_Keyword"] and function() self:FireTrigger(value["Event_Keyword"]) end)
                end))
            end
        end
    end

    if tablelength(data["Action"]) > 0 then
        table.insert(self.actionCallbacks, AddCallback(ActionHandlerObject, "Action", function(args)
            local value = data["Action"][args.Action]
            if value then
                valuebar:SetAlert(value.Countdown, value.Text, value["Icon"], value["Event_Keyword"] and function() self:FireTrigger(value["Event_Keyword"]) end)
            end
        end))
    end

    if tablelength(data["EnemyHits"]) > 0 then
        table.insert(self.enemyChatCallback, AddCallback(ChatEventHandler, "EnemyCombat", function(args)
            local value = data["EnemyHits"][args.SkillName]
            if value then
                valuebar:SetAlert(value.Countdown, valuebar.nameFunction(value.Text, args.Target), value["Icon"], value["Event_Keyword"] and function() self:FireTrigger(value["Event_Keyword"]) end)
            end
        end))

    end

    if tablelength(data["PlayerHits"]) > 0 then
        table.insert(self.playerChatCallback, AddCallback(ChatEventHandler, "PlayerCombat", function(args)
            local value = data["PlayerHits"][args.SkillName]
            if value then
                valuebar:SetAlert(value.Countdown, valuebar.nameFunction(value.Text, args.Target), value["Icon"], value["Event_Keyword"] and function() self:FireTrigger(value["Event_Keyword"]) end)
            end
        end))
    end

    if tablelength(data["StandardChat"]) > 0 then
        table.insert(self.standardChatCallback, AddCallback(ChatEventHandler, "Standard", function(message)
            local value = data["StandardChat"][message]
            if value then
                valuebar:SetAlert(value.Countdown, value.Text, value["Icon"], value["Event_Keyword"] and function() self:FireTrigger(value["Event_Keyword"]) end)
            end
        end))
    end

    if tablelength(data["PartyChat"]) > 0 then
        table.insert(self.partyChatCallback, AddCallback(ChatEventHandler, "Party", function(message)
            local value = data["PartyChat"][message]
            if value then
                valuebar:SetAlert(value.Countdown, value.Text, value["Icon"], value["Event_Keyword"] and function() self:FireTrigger(value["Event_Keyword"]) end)
            end
        end))
    end

    if tablelength(data["SayChat"]) > 0 then
        table.insert(self.sayChatCallback, AddCallback(ChatEventHandler, "SayLine", function(message)
            local value = data["SayChat"][message]
            if value then
                valuebar:SetAlert(value.Countdown, value.Text, value["Icon"], value["Event_Keyword"] and function() self:FireTrigger(value["Event_Keyword"]) end)
            end
        end))
    end

    if tablelength(data["PlayerHeal"]) > 0 then
        table.insert(self.playerHealCallbacks, AddCallback(ChatEventHandler, "PlayerHeal", function(args)
            local value = data["PlayerHeal"][args.SkillName]
            if value then
                valuebar:SetAlert(value.Countdown, valuebar.nameFunction(value.Text, args.Target), value["Icon"], value["Event_Keyword"] and function() self:FireTrigger(value["Event_Keyword"]) end)
            end
        end))
    end

    if tablelength(data["PlayerBenefit"]) > 0 then
        table.insert(self.playerBenefitCallbacks, AddCallback(ChatEventHandler, "PlayerBenefit", function(args)
            local value = data["PlayerBenefit"][args.SkillName]
            if value then
                valuebar:SetAlert(value.Countdown, valuebar.nameFunction(value.Text, args.Target), value["Icon"], value["Event_Keyword"] and function() self:FireTrigger(value["Event_Keyword"]) end)
            end
        end))
    end

    if tablelength(data["DNFApplied"]) > 0 then
        table.insert(self.dnfAppliedCallbacks, AddCallback(rezTracker, "DNFApplication", function(args)
            for key, value in pairs(data["DNFApplied"]) do
                local time = value.Countdown
                if time and time == true then
                    time = args.Time
                end
                valuebar:SetAlert(time, valuebar.nameFunction(value.Text, args.Name), value["Icon"], value["Event_Keyword"] and function() self:FireTrigger(value["Event_Keyword"]) end)
            end
        end))
    end

    if tablelength(data["DNFRemoved"]) > 0 then
        table.insert(self.dnfRemovedCallbacks, AddCallback(rezTracker, "DNFRemoval", function(args)
            for key, value in pairs(data["DNFRemoved"]) do
                valuebar:SetAlert(value.Countdown, valuebar.nameFunction(value.Text, args.Name), value["Icon"], value["Event_Keyword"] and function() self:FireTrigger(value["Event_Keyword"]) end)
            end
        end))
    end

    if tablelength(data["CCBreak"]) > 0 then
        for key, value in pairs(data["CCBreak"]) do
            if key == "" then
                valuebar.CCBreakAllNames = true
            end
        end
        table.insert(self.playerBenefitCallbacks, AddCallback(ChatEventHandler, "CCBreak", function(args)
            local value = data["CCBreak"][args.Target]
            if value then
                if value.CCType == args.Type or value.CCType == true then
                    local text = string.gsub(value.Text, "<Target>", NameHandlerObject:GetName(args.Target, true, "PrimeUIAlert", ""))
                    text = string.gsub(text, "<Type>", NameHandlerObject:GetName(args.Type, true, "PrimeUIAlert", ""))
                    text = string.gsub(text, "<Culprit>", NameHandlerObject:GetName(args.Culprit, true, "PrimeUIAlert", ""))
                    valuebar:SetAlert(value.Countdown, text, value.Icon, value["Event_Keyword"] and function() self:FireTrigger(value["Event_Keyword"]) end)
                end
            end
            if valuebar.CCBreakAllNames then
                local allValue = data["CCBreak"][""]
                if allValue.CCType == args.Type or allValue.CCType == true then
                    local text = string.gsub(allValue.Text, "<Target>", NameHandlerObject:GetName(args.Target, true, "PrimeUIAlert", ""))
                    text = string.gsub(text, "<Type>", NameHandlerObject:GetName(args.Type, true, "PrimeUIAlert", ""))
                    text = string.gsub(text, "<Culprit>", NameHandlerObject:GetName(args.Culprit, true, "PrimeUIAlert", ""))
                    valuebar:SetAlert(allValue.Countdown, text, allValue.Icon, allValue["Event_Keyword"] and function() self:FireTrigger(allValue["Event_Keyword"]) end)
                end
            end
        end))
    end
    
    self.currentHeight = self.currentHeight + data["Height"] + 2 * data["BorderSize"]

    table.insert(self.tools, valuebar)
end