function PrimeUI:BuffBar(data, parent)
    local x = self:ContentAlignement(data["Alignment"], data["Width"])
    local y = self.currentHeight
    if self.groupMode then
        x = data.X
        y = data.Y
    end

    local buffbar = ClassSpecificTools.DynamicSkillBar(parent, x, y, data["Width"], (data["BorderSize"] and data["BorderSize"] * 2 or 0) + 32 * data["Scale"], (data["BorderSize"] and data["BorderSize"] * 2 or 0) + 32 * data["Scale"], self:AlignmentToNumber(data["Alignment"]), true, false, data["Offset"], true)
    self.lookup[data.NickName or ""] = buffbar
    buffbar:SetFancySort(data.Smooth_Sorting)
    buffbar.SkillID = {}
    buffbar.UniqueNames = {}
    buffbar.uniqueTable = {}
    buffbar.uniqueTableIDs = {}

    if self.groupMode then
        buffbar.moveBox = MoveBox(buffbar, data["Width"], data["Width"], data["NickName"] or "Buffbar")
        buffbar.savePosition = function()
            data["X"], data["Y"] = buffbar:GetPosition()
        end
    end

    if tablelength(data["Effects"]) > 0 or data["AllEffects"] then
        buffbar.effectFunction = function(effect)
            local name = effect:GetName()
            if data["Effects"][name] or data["AllEffects"] then
                if (not(data["OnlyBuffs"]) and not(data["OnlyDebuffs"])) or (data["OnlyDebuffs"] and effect:IsDebuff()) or (data["OnlyBuffs"] and not(effect:IsDebuff())) then
                    local highlight = data["AllHighlight"]
                    local icon = 0
                    local value = data["Effects"][name]
                    local time
                    if value then
                        time = value["Time"]
                        highlight = highlight or value["Highlight"]
                        if value["Icon"] == true then 
                            icon = effect:GetIcon()
                        else
                            icon = value["Icon"] 
                        end

                        if value["Fit_Icon"] and value["Icon"] and value["Icon"] ~= true then
                            if effect:GetIcon() ~= tonumber(value["Icon"]) then 
                                return 
                            end
                        end

                        if time == true then
                            time = effect:GetDuration() - (Turbine.Engine.GetGameTime() - effect:GetStartTime())
                        end
                    else
                        icon = effect:GetIcon()
                    end

                    local ID = effect:GetID()
                    buffbar.SkillID[ID] = true
                    local label = ClassSpecificTools.ClassTimerLabel(buffbar, 0, 0, 32, 32, icon, false, false, nil, 1, 1, false, false, true, data["Scale"], name, true, self.alignments[data["TextAlignment"]], self.alignments[data["StackTextAlignment"]], highlight, self:FontToFont(data["Font_Timer"]), self:FontToFont(data["Font_Stack"]), self:ColourTupleToTurbineColour(data["Font_Timer_Colour"]), self:ColourTupleToTurbineColour(data["Font_Stack_Colour"]), data["BorderSize"], self:ColourTupleToTurbineColour(data["BorderColour"]), self:ColourTupleToTurbineColour(data["HighlightColour"]), data["HighlightSize"], data.Stack_X_Offset, data.Stack_Y_Offset, data.Duration_X_Offset, data.Duration_Y_Offset, value.Wiggle_Lower, value.Wiggle_Upper, value.Always_Wiggle)
                    buffbar:AddLabel(ID, label, time)
                    buffbar:SetLabel(label)
                    buffbar:ToggleLabel(ID, true, time or effect:GetDuration() - (Turbine.Engine.GetGameTime() - effect:GetStartTime()))
                end
            end
        end

        for i = 1, self.player:GetEffects():GetCount() do
            buffbar.effectFunction(self.player:GetEffects():Get(i))
        end

        table.insert(self.effectAddedCallbacks, AddCallback(PlayerEffectsHandler, "EffectAdded", buffbar.effectFunction))

        table.insert(self.effectRemovedCallbacks, AddCallback(PlayerEffectsHandler, "EffectRemoved", 
        function(effect)
            local ID = effect:GetID()
            if buffbar.SkillID[ID] then
                buffbar:RemoveLabel(ID)
            end

        end))
    end

    if tablelength(data["EffectAdded"]) > 0 then
        table.insert(self.effectAddedCallbacks, AddCallback(PlayerEffectsHandler, "EffectAdded", 
        function(effect)
            local name = effect:GetName()
            if data["EffectAdded"][name] then
                local highlight = data["AllHighlight"]
                local icon = 0
                local value = data["EffectAdded"][name]
                local time = value["Time"]
                if value then
                    highlight = highlight or value["Highlight"]
                    if value["Icon"] == true then 
                        icon = effect:GetIcon()
                    else
                        icon = value["Icon"] 
                    end

                    if value["Fit_Icon"] and value["Icon"] and value["Icon"] ~= true then
                        if effect:GetIcon() ~= tonumber(value["Icon"]) then 
                            return 
                        end
                    end

                    if time == true then
                        time = effect:GetDuration() - (Turbine.Engine.GetGameTime() - effect:GetStartTime())
                    end
                else
                    icon = effect:GetIcon()
                end

                local ID = effect:GetID()
                buffbar.SkillID[ID] = true
                buffbar:AddLabel(ID, ClassSpecificTools.ClassTimerLabel(buffbar, 0, 0, 32, 32, icon, false, false, nil, 1, 1, false, false, true, data["Scale"], name, true, self.alignments[data["TextAlignment"]], self.alignments[data["StackTextAlignment"]], highlight, self:FontToFont(data["Font_Timer"]), self:FontToFont(data["Font_Stack"]), self:ColourTupleToTurbineColour(data["Font_Timer_Colour"]), self:ColourTupleToTurbineColour(data["Font_Stack_Colour"]), data["BorderSize"], self:ColourTupleToTurbineColour(data["BorderColour"]), self:ColourTupleToTurbineColour(data["HighlightColour"]), data["HighlightSize"], data.Stack_X_Offset, data.Stack_Y_Offset, data.Duration_X_Offset, data.Duration_Y_Offset, value.Wiggle_Lower, value.Wiggle_Upper, value.Always_Wiggle), time)
                buffbar:ToggleLabel(ID, true, time)
            end
        end))
    end

    if tablelength(data["EffectAdded_Target"]) > 0 then
        table.insert(self.effectTargetAddedCallbacks, TargetHandlerObject:AddEffectCallback(
        function(args)
            local name = args.Name
            if data["EffectAdded_Target"][name] then
                local effectIcon = args.Icon
                local effectStartTime = args.StartTime
                local effectID = args.ID
                local effectDuration = args.Duration
                local gameTime = Turbine.Engine.GetGameTime()

                local highlight = data["AllHighlight"]
                local icon = nil
                local value = data["EffectAdded_Target"][name]
                if value then
                    highlight = highlight or value["Highlight"]
                    if value["Icon"] == true then 
                        icon = effectIcon
                    else
                        icon = value["Icon"] 
                    end

                    local time = value["Time"]
                    if time == true then
                        time = effectDuration - (gameTime - effectStartTime)
                    end

                    if value["Fit_Icon"] and value["Icon"] and value["Icon"] ~= true then
                        if effectIcon ~= tonumber(value["Icon"]) then 
                            return 
                        end
                    end

                    if value["Unique"] then
                        local id = string.match(name, "%D+") .. tostring(icon)
                        -- It does not exist yet so we create the entry and fall through
                        -- Or
                        -- We have seen it before but it has since expired so we update the timeings
                        if not(buffbar.uniqueTable[id]) or gameTime >= buffbar.uniqueTable[id] then
                            buffbar.uniqueTable[id] = gameTime + time - (gameTime - effectStartTime)
                            buffbar.uniqueTableIDs[id] = effectID

                        -- It does exist but is not expired yet so we remove the old label and add a new one
                        elseif gameTime < buffbar.uniqueTable[id] then
                            buffbar:RemoveLabel(buffbar.uniqueTableIDs[id])
                            buffbar:AddLabel(effectID, ClassSpecificTools.ClassTimerLabel(buffbar, 0, 0, 32, 32, icon, false, false, nil, 1, 1, false, false, true, data["Scale"], name, true, self.alignments[data["TextAlignment"]], self.alignments[data["StackTextAlignment"]], highlight, self:FontToFont(data["Font_Timer"]), self:FontToFont(data["Font_Stack"]), self:ColourTupleToTurbineColour(data["Font_Timer_Colour"]), self:ColourTupleToTurbineColour(data["Font_Stack_Colour"])), value["Time"] == true and time or time - (gameTime - effectStartTime))
                            buffbar:ToggleLabel(effectID, true, value["Time"] == true and time or time - (gameTime - effectStartTime))
                            buffbar.uniqueTable[id] = gameTime + time - (gameTime - effectStartTime)
                            buffbar.uniqueTableIDs[id] = effectID
                            return
                        end
                    end

                    buffbar:AddLabel(effectID, ClassSpecificTools.ClassTimerLabel(buffbar, 0, 0, 32, 32, icon, false, false, nil, 1, 1, false, false, true, data["Scale"], name, true, self.alignments[data["TextAlignment"]], self.alignments[data["StackTextAlignment"]], highlight, self:FontToFont(data["Font_Timer"]), self:FontToFont(data["Font_Stack"]), self:ColourTupleToTurbineColour(data["Font_Timer_Colour"]), self:ColourTupleToTurbineColour(data["Font_Stack_Colour"])), value["Time"] == true and time or time - (gameTime - effectStartTime))
                    buffbar:ToggleLabel(effectID, true, value["Time"] == true and time or time - (gameTime - effectStartTime))
                    return
                else
                    return
                end
            end
        end))

        if data["Reset_On_Retarget"] then
            table.insert(self.targetInfoCallbacks, TargetHandlerObject:AddTargetInfoCallback(function(value) 
                if value then
                    --if buffbar.name ~= value["Name"] then
                    buffbar.uniqueTable = nil
                    buffbar.uniqueTableIDs = nil
                    buffbar:Clear()
                    buffbar.uniqueTable = {}
                    buffbar.uniqueTableIDs = {}
                    --end
                    buffbar.name = value["Name"]
                else
                    buffbar.uniqueTable = nil
                    buffbar.uniqueTableIDs = nil
                    buffbar.name = ""
                    buffbar:Clear()
                    buffbar.uniqueTable = {}
                    buffbar.uniqueTableIDs = {}
                end
                
            end))
        end
    end

    if tablelength(data["Skills"]) > 0 then
        self.skills = true
        for key, value in pairs(data["Skills"]) do
            if SkillEventHandler:SkillExists(key) then
                -- Add the callback for this skill slot
                if not(self.skillCallbacks[key]) then self.skillCallbacks[key] = {} end
                table.insert(self.skillCallbacks[key], AddCallback(SkillEventHandler, key, function(skill) 
                    local icon = value["Icon"]
                    if value["Icon"] == true then
                        icon = skill:GetSkillInfo():GetIconImageID()
                    end

                    local time = value["Time"]
                    if value["Time"] == true then
                        time = skill:GetResetTime() - Turbine.Engine.GetGameTime()
                    end

                    if (skill:GetResetTime() - Turbine.Engine.GetGameTime()) + 1 <= skill:GetCooldown() and value.Time ~= true then
                        return
                    end

                    local highlight = data["AllHighlight"] or value["Highlight"]

                    if not(buffbar:LabelExists(key)) then
                        buffbar:AddLabel(key, ClassSpecificTools.ClassTimerLabel(buffbar, 0, 0, 32, 32, icon, false, false, nil, 1, 1, false, false, true, data["Scale"], key, true, self.alignments[data["TextAlignment"]], self.alignments[data["StackTextAlignment"]], highlight, self:FontToFont(data["Font_Timer"]), self:FontToFont(data["Font_Stack"]), self:ColourTupleToTurbineColour(data["Font_Timer_Colour"]), self:ColourTupleToTurbineColour(data["Font_Stack_Colour"]), data["BorderSize"], self:ColourTupleToTurbineColour(data["BorderColour"]), self:ColourTupleToTurbineColour(data["HighlightColour"]), data["HighlightSize"], data.Stack_X_Offset, data.Stack_Y_Offset, data.Duration_X_Offset, data.Duration_Y_Offset, value.Wiggle_Lower, value.Wiggle_Upper, value.Always_Wiggle), time)
                    end
                    buffbar:ToggleLabel(key, true, time)
                end))
            end
        end
    end

    if tablelength(data["PlayerHits"]) > 0 then
        --for key, value in pairs(data["PlayerHits"]) do
            table.insert(self.playerChatCallback, AddCallback(ChatEventHandler, "PlayerCombat", function(args)
                local value = data["PlayerHits"][args.SkillName]
                if value then  
                    local highlight = data["AllHighlight"] or value["Highlight"]
                    if not(buffbar:LabelExists(args.SkillName)) then
                        buffbar:AddLabel(args.SkillName, ClassSpecificTools.ClassTimerLabel(buffbar, 0, 0, 32, 32, value["Icon"], false, false, nil, 1, 1, false, false, true, data["Scale"], key, true, self.alignments[data["TextAlignment"]], self.alignments[data["StackTextAlignment"]], highlight, self:FontToFont(data["Font_Timer"]), self:FontToFont(data["Font_Stack"]), self:ColourTupleToTurbineColour(data["Font_Timer_Colour"]), self:ColourTupleToTurbineColour(data["Font_Stack_Colour"]), data["BorderSize"], self:ColourTupleToTurbineColour(data["BorderColour"]), self:ColourTupleToTurbineColour(data["HighlightColour"]), data["HighlightSize"], data.Stack_X_Offset, data.Stack_Y_Offset, data.Duration_X_Offset, data.Duration_Y_Offset, value.Wiggle_Lower, value.Wiggle_Upper, value.Always_Wiggle), value["Time"])
                    end
                    buffbar:ToggleLabel(args.SkillName, true, value["Time"])
                end
            end))
        --end
    end

    if tablelength(data["PlayerHeal"]) > 0 then
        --for key, value in pairs(data["PlayerHeal"]) do
            table.insert(self.playerHealCallbacks, AddCallback(ChatEventHandler, "PlayerHeal", function(args)
                local value = data["PlayerHeal"][args.SkillName]
                if value then  
                    local highlight = data["AllHighlight"] or value["Highlight"]
                    if not(buffbar:LabelExists(args.SkillName)) then
                        buffbar:AddLabel(args.SkillName, ClassSpecificTools.ClassTimerLabel(buffbar, 0, 0, 32, 32, value["Icon"], false, false, nil, 1, 1, false, false, true, data["Scale"], key, true, self.alignments[data["TextAlignment"]], self.alignments[data["StackTextAlignment"]], highlight, self:FontToFont(data["Font_Timer"]), self:FontToFont(data["Font_Stack"]), self:ColourTupleToTurbineColour(data["Font_Timer_Colour"]), self:ColourTupleToTurbineColour(data["Font_Stack_Colour"]), data["BorderSize"], self:ColourTupleToTurbineColour(data["BorderColour"]), self:ColourTupleToTurbineColour(data["HighlightColour"]), data["HighlightSize"], data.Stack_X_Offset, data.Stack_Y_Offset, data.Duration_X_Offset, data.Duration_Y_Offset, value.Wiggle_Lower, value.Wiggle_Upper, value.Always_Wiggle), value["Time"])
                    end
                    buffbar:ToggleLabel(args.SkillName, true, value["Time"])
                end
            end))
        --end
    end

    if tablelength(data["PlayerBenefit"]) > 0 then
        --for key, value in pairs(data["PlayerBenefit"]) do
            table.insert(self.playerBenefitCallbacks, AddCallback(ChatEventHandler, "PlayerBenefit", function(args)
                local value = data["PlayerBenefit"][args.SkillName]
                if value then  
                    local highlight = data["AllHighlight"] or value["Highlight"]
                    if not(buffbar:LabelExists(args.SkillName)) then
                        buffbar:AddLabel(args.SkillName, ClassSpecificTools.ClassTimerLabel(buffbar, 0, 0, 32, 32, value["Icon"], false, false, nil, 1, 1, false, false, true, data["Scale"], key, true, self.alignments[data["TextAlignment"]], self.alignments[data["StackTextAlignment"]], highlight, self:FontToFont(data["Font_Timer"]), self:FontToFont(data["Font_Stack"]), self:ColourTupleToTurbineColour(data["Font_Timer_Colour"]), self:ColourTupleToTurbineColour(data["Font_Stack_Colour"]), data["BorderSize"], self:ColourTupleToTurbineColour(data["BorderColour"]), self:ColourTupleToTurbineColour(data["HighlightColour"]), data["HighlightSize"], data.Stack_X_Offset, data.Stack_Y_Offset, data.Duration_X_Offset, data.Duration_Y_Offset, value.Wiggle_Lower, value.Wiggle_Upper, value.Always_Wiggle), value["Time"])
                    end
                    buffbar:ToggleLabel(args.SkillName, true, value["Time"])
                end
            end))
        --end
    end

    if tablelength(data["EnemyHits"]) > 0 then
        --for key, value in pairs(data["EnemyHits"]) do
            table.insert(self.enemyChatCallback, AddCallback(ChatEventHandler, "EnemyCombat", function(args)
                local value = data["EnemyHits"][args.SkillName]
                if value then  
                    local highlight = data["AllHighlight"] or value["Highlight"]
                    if not(buffbar:LabelExists(args.SkillName)) then
                        buffbar:AddLabel(args.SkillName, ClassSpecificTools.ClassTimerLabel(buffbar, 0, 0, 32, 32, value["Icon"], false, false, nil, 1, 1, false, false, true, data["Scale"], key, true, self.alignments[data["TextAlignment"]], self.alignments[data["StackTextAlignment"]], highlight, self:FontToFont(data["Font_Timer"]), self:FontToFont(data["Font_Stack"]), self:ColourTupleToTurbineColour(data["Font_Timer_Colour"]), self:ColourTupleToTurbineColour(data["Font_Stack_Colour"]), data["BorderSize"], self:ColourTupleToTurbineColour(data["BorderColour"]), self:ColourTupleToTurbineColour(data["HighlightColour"]), data["HighlightSize"], data.Stack_X_Offset, data.Stack_Y_Offset, data.Duration_X_Offset, data.Duration_Y_Offset, value.Wiggle_Lower, value.Wiggle_Upper, value.Always_Wiggle), value["Time"])
                    end
                    buffbar:ToggleLabel(args.SkillName, true, value["Time"])
                end
            end))
        --end
    end

    self.currentHeight = self.currentHeight + (data["BorderSize"] and data["BorderSize"] * 2 or 0) + 32 * data["Scale"]
    table.insert(self.tools, buffbar)

end