function PrimeUI:CircleTimer(data, parent)
    local valuebar = Turbine.UI.Control()
    local width = 132
    local height = 172
    valuebar:SetParent(parent)
    if self.groupMode then
        valuebar:SetPosition(data["X"], data["Y"])
        valuebar.moveBox = MoveBox(valuebar, width, height, data["NickName"] or "Circle Timer")
        valuebar.savePosition = function()
            data["X"], data["Y"] = valuebar:GetPosition()
        end
    else
        valuebar:SetPosition(self:ContentAlignement(data["Alignment"], width), self.currentHeight)
    end
    valuebar:SetSize(width, height)
    valuebar:SetMouseVisible(false)

    valuebar.nameFunction = function(text, name)
        return string.gsub(text, "<name>", NameHandlerObject:GetName(name, true, "PrimeUICirleTimer", ""))
    end

    local timer = UI.CircleTimer(valuebar, self:ColourTupleToTurbineColour(data["BackgroundRingColour"]), self:ColourTupleToTurbineColour(data["TimerRingColour"]), self:FontToFont(data["Text_Font"]), self:FontToFont(data["Timer_Font"]), data["Clockwise"], data["Show_Timer"], data.Scale)
    self.lookup[data.NickName or ""] = timer
    timer.targetMaxHealth = 0
    timer.TargetName = ""

    if tablelength(data["EventTrigger"]) > 0 then
        table.insert(self.eventTriggerCallbacks, AddCallback(self, "EventTrigger", 
        function(event)
            local value = data["EventTrigger"][event]
            if value then
                timer:StartTimer(value["Time"], valuebar.nameFunction(value["Message"], self.player:GetName()), value["Event_Keyword"] and function() self:FireTrigger(value["Event_Keyword"]) end)
            end
        end))
    end

    if tablelength(data["EffectAdded"]) > 0 then
        table.insert(self.effectAddedCallbacks, AddCallback(PlayerEffectsHandler, "EffectAdded", 
        function(effect)
            local name = effect:GetName()
            if data["EffectAdded"][name] then
                local value = data["EffectAdded"][name]
                local time = value["Time"]
                if value["Time"] == true then
                    time = effect:GetDuration() - (Turbine.Engine.GetGameTime() - effect:GetStartTime())
                end

                timer:StartTimer(time, valuebar.nameFunction(value["Message"], self.player:GetName()), value["Event_Keyword"] and function() self:FireTrigger(value["Event_Keyword"]) end)
            end
        end))
    end

    if tablelength(data["EffectAdded_Target"]) > 0 then
        table.insert(self.effectTargetAddedCallbacks, TargetHandlerObject:AddEffectCallback( 
        function(args)
            local name = args.Name
            if data["EffectAdded_Target"][name] ~= nil then
                local value = data["EffectAdded_Target"][name]
                local time = value["Time"]
                if value["Time"] == true then
                    time = args.Duration - (Turbine.Engine.GetGameTime() - args.StartTime)
                end

                timer:StartTimer(time, valuebar.nameFunction(value["Message"], timer.TargetName), value["Event_Keyword"] and function() self:FireTrigger(value["Event_Keyword"]) end) 
            end
        end))
    end

    if tablelength(data["EffectAdded_Group"]) > 0 then
        table.insert(self.effectAddedGroupCallbacks, AddCallback(GroupEffectsHandler, "EffectAdded", 
        function(args)
            if data["EffectAdded_Group"][args.effect.Name] ~= nil then
                local effectArgs = args.effect
                local value = data["EffectAdded_Group"][effectArgs.Name]
                local time = value["Time"]
                if value["Time"] == true then
                    time = effectArgs.Duration - (Turbine.Engine.GetGameTime() - effectArgs.StartTime)
                end


                timer:StartTimer(time, valuebar.nameFunction(value["Message"], args.name), value["Event_Keyword"] and function() self:FireTrigger(value["Event_Keyword"]) end)
            end
        end))
    end

    

    if tablelength(data["EffectRemoved"]) > 0 then
        table.insert(self.effectRemovedCallbacks, AddCallback(PlayerEffectsHandler, "EffectRemoved", 
        function(effect)
            local name = effect:GetName()
            if data["EffectRemoved"][name] ~= nil then
                local value = data["EffectRemoved"][name]
                local time = value["Time"]
                if value["Time"] == true then
                    time = effect:GetDuration() - (Turbine.Engine.GetGameTime() - effect:GetStartTime())
                end

                timer:StartTimer(time, valuebar.nameFunction(value["Message"], self.player:GetName()), value["Event_Keyword"] and function() self:FireTrigger(value["Event_Keyword"]) end)
            end
        end))
    end

    if tablelength(data["Skills"]) > 0 then
        self.skills = true
        for key, value in pairs(data["Skills"]) do
            if SkillEventHandler:SkillExists(key) then
                if not(self.skillCallbacks[key]) then self.skillCallbacks[key] = {} end
                table.insert(self.skillCallbacks[key], AddCallback(SkillEventHandler, key, function(skill)  
                    local time = value["Time"]
                    local maxTime = value["Time"]
                    if value["Time"] == true then
                        time = skill:GetResetTime() - Turbine.Engine.GetGameTime()
                    end

                    if (skill:GetResetTime() - Turbine.Engine.GetGameTime()) + 1 <= skill:GetCooldown() then
                        return
                    end

                    timer:StartTimer(time, valuebar.nameFunction(value["Message"], self.player:GetName()), value["Event_Keyword"] and function() self:FireTrigger(value["Event_Keyword"]) end)
                end))
            end
        end
    end

    if tablelength(data["Action"]) > 0 then
        table.insert(self.actionCallbacks, AddCallback(ActionHandlerObject, "Action", function(args)
            local value = data["Action"][args.Action]
            if value then
                local value = value
                timer:StartTimer(value["Time"], valuebar.nameFunction(value["Message"], self.name), value["Event_Keyword"] and function() self:FireTrigger(value["Event_Keyword"]) end)
            end
        end))
    end

    if tablelength(data["EnemyHits"]) > 0 then
        table.insert(self.enemyChatCallback, AddCallback(ChatEventHandler, "EnemyCombat", function(args)
            local value = data["EnemyHits"][args.SkillName]
            if value then
                local value = value
                timer:StartTimer(value["Time"], valuebar.nameFunction(value["Message"], args.Target), value["Event_Keyword"] and function() self:FireTrigger(value["Event_Keyword"]) end)
            end
        end))

    end

    if tablelength(data["PlayerHits"]) > 0 then
        table.insert(self.playerChatCallback, AddCallback(ChatEventHandler, "PlayerCombat", function(args)
            if data["PlayerHits"][args.SkillName] then
                local value = data["PlayerHits"][args.SkillName]
                timer:StartTimer(value["Time"], valuebar.nameFunction(value["Message"], args.Target), value["Event_Keyword"] and function() self:FireTrigger(value["Event_Keyword"]) end)
            end
        end))
    end

    if tablelength(data["PlayerHeal"]) > 0 then
        table.insert(self.playerHealCallbacks, AddCallback(ChatEventHandler, "PlayerHeal", function(args)
            if data["PlayerHeal"][args.SkillName] then
                local value = data["PlayerHeal"][args.SkillName]
                timer:StartTimer(value["Time"], valuebar.nameFunction(value["Message"], args.Target), value["Event_Keyword"] and function() self:FireTrigger(value["Event_Keyword"]) end)
            end
        end))
    end

    if tablelength(data["PlayerBenefit"]) > 0 then
        table.insert(self.playerBenefitCallbacks, AddCallback(ChatEventHandler, "PlayerBenefit", function(args)
            if data["PlayerBenefit"][args.SkillName] then
                local value = data["PlayerBenefit"][args.SkillName]
                timer:StartTimer(value["Time"], valuebar.nameFunction(value["Message"], args.Target), value["Event_Keyword"] and function() self:FireTrigger(value["Event_Keyword"]) end)
            end
        end))
    end

    if tablelength(data["StandardChat"]) > 0 then
        table.insert(self.standardChatCallback, AddCallback(ChatEventHandler, "Standard", function(message)
            if data["StandardChat"][message] then
                local value = data["StandardChat"][message]
                timer:StartTimer(value["Time"], valuebar.nameFunction(value["Message"], self.player:GetName()), value["Event_Keyword"] and function() self:FireTrigger(value["Event_Keyword"]) end)
            end
        end))
    end

    if tablelength(data["PartyChat"]) > 0 then
        table.insert(self.partyChatCallback, AddCallback(ChatEventHandler, "Party", function(message)
            if data["PartyChat"][message] then
                local value = data["PartyChat"][message]
                timer:StartTimer(value["Time"], valuebar.nameFunction(value["Message"], self.player:GetName()), value["Event_Keyword"] and function() self:FireTrigger(value["Event_Keyword"]) end)
            end
        end))
    end

    if tablelength(data["SayChat"]) > 0 then
        table.insert(self.sayChatCallback, AddCallback(ChatEventHandler, "SayLine", function(message)
            if data["SayChat"][message]  then
                local value = data["SayChat"][message]
                timer:StartTimer(value["Time"], valuebar.nameFunction(value["Message"], self.player:GetName()), value["Event_Keyword"] and function() self:FireTrigger(value["Event_Keyword"]) end)
            end
        end))
    end

    if tablelength(data["Target_HealthThreshold"]) > 0 or tablelength(data["EffectAdded_Target"]) > 0 or data["Reset_On_Retarget"] then
        table.insert(self.targetInfoCallbacks, TargetHandlerObject:AddTargetInfoCallback(function(value) 
            if value then
                if value["Name"] ~= timer.TargetName then
                    timer:Clear()
                end
                timer.targetMaxHealth = value["MaxMorale"]
                timer.TargetName = value["Name"]
            else
                timer.targetMaxHealth = 0
                timer.TargetName = ""
                timer:Clear()
            end
        end))
    end

    
    if tablelength(data["Target_HealthThreshold"]) > 0 then
        timer.targetHealthThresholdCooldowns = {}
        timer.targetHealthThresholdPercentage = {}
        local health = self.player:GetMorale() / self.player:GetMaxMorale() * 100
        for key, value in pairs(data["Target_HealthThreshold"]) do
            timer.targetHealthThresholdCooldowns[key .. tostring(value["Number"]) .. tostring(value["Operator"])] = Turbine.Engine.GetGameTime() - 1000
            timer.targetHealthThresholdPercentage[key .. tostring(value["Number"]) .. tostring(value["Operator"])] = true
        end


        table.insert(self.healthTargetCallbacks, TargetHandlerObject:AddHealthCallback(function(value)
            if timer.targetMaxHealth > 0 then
                local percentage = value / timer.targetMaxHealth * 100
                local time = Turbine.Engine.GetGameTime()
                for key, value in pairs(data["Target_HealthThreshold"]) do
                    if key == timer.TargetName then
                        if self.compareFunctions[value["Operator"]](percentage, value["Number"]) then
                            local id = key .. tostring(value["Number"]) .. tostring(value["Operator"])
                            if timer.targetHealthThresholdCooldowns[id] < time and timer.targetHealthThresholdPercentage[id] then
                                timer.targetHealthThresholdPercentage[id] = false
                                timer:StartTimer(value["Time"], valuebar.nameFunction(value["Message"], key), value["Event_Keyword"] and function() self:FireTrigger(value["Event_Keyword"]) end) 
                                timer.targetHealthThresholdCooldowns[id] = time + 6
                            end
                        else
                            timer.targetHealthThresholdPercentage[key .. tostring(value["Number"]) .. tostring(value["Operator"])] = true
                        end
                    end
                end
            end
        end))
        table.insert(self.combatCallbacks, AddCallback(self.player, "InCombatChanged", function()
            if not(self.player:IsInCombat()) then
                for key, value in pairs(data["Target_HealthThreshold"]) do
                    timer.targetHealthThresholdCooldowns[key .. tostring(value["Number"]) .. tostring(value["Operator"])] = Turbine.Engine.GetGameTime() - 1000
                    timer.targetHealthThresholdPercentage[key .. tostring(value["Number"]) .. tostring(value["Operator"])] = true
                end 
            end
        end))
    end

    table.insert(self.tools, timer)

    self.currentHeight = self.currentHeight + height

    table.insert(self.UI, valuebar)
end