function PrimeUI:Conditions(data, parent)
    local valuebar = Turbine.UI.Window()
    valuebar:SetParent(parent)

    valuebar.statusTable = {}
    valuebar.applicationTable = {}
    valuebar.visibilityValue = nil
    valuebar.locationValue = data["Location"] and data["Location"][LocationHandler:GetCurrentLocation()] ~= nil and 1 or 0

    valuebar.togglefunction = function()
        local tableValue = 0
        for key, value in pairs(valuebar.statusTable) do
            if value > 0 then
                if valuebar.applicationTable[key] then
                    if value >= valuebar.applicationTable[key] then
                        tableValue = tableValue + 1
                    end
                else
                    tableValue = tableValue + 1
                end
                
            end
        end
        tableValue = tableValue + valuebar.locationValue

        local visibilityValue = tableValue >= valuebar.minCount
        if valuebar.visibilityValue ~= visibilityValue then
            if visibilityValue then
                for keyword in string.gmatch(data.Activation_Triggers, "([^,]+)") do
                    self:FireTrigger(keyword)
                end
            else
                for keyword in string.gmatch(data.Deactivation_Triggers, "([^,]+)") do
                    self:FireTrigger(keyword)
                end
            end
        end
        valuebar.visibilityValue = visibilityValue
        --PrintTable(valuebar.statusTable)
        --ChatOutput(tostring(tableValue) .. " - " .. tostring(valuebar.minCount))
    end

    valuebar.UpdateItems = {}
    valuebar.UpdateCount = 0
    valuebar.SkillItems = {}
    valuebar.SkillValue = {}
    valuebar.EventItems = {}
    valuebar.EventValue = {}

    valuebar.updateCallback = function(delta)
        for key, value in pairs(valuebar.UpdateItems) do
            valuebar.UpdateItems[key] = valuebar.UpdateItems[key] - delta
            if valuebar.UpdateItems[key] < 0 then
                valuebar.UpdateItems[key] = nil
                valuebar.statusTable[key] = 0
                valuebar.UpdateCount = valuebar.UpdateCount  - 1
                valuebar.togglefunction()
            end
        end

        for key, value in pairs(valuebar.EventItems) do
            valuebar.EventItems[key] = valuebar.EventItems[key] - delta
            if valuebar.EventItems[key] < 0 then
                valuebar.EventItems[key] = nil
                valuebar.statusTable[key] = 0
                valuebar.UpdateCount = valuebar.UpdateCount  - 1
                valuebar.togglefunction()
            end
        end

        for key, value in pairs(valuebar.SkillItems) do
            if valuebar.SkillItems[key] < Turbine.Engine.GetGameTime() then
                valuebar.SkillItems[key] = nil
                valuebar.statusTable[key] = 0
                if valuebar.SkillValue[key] then
                    valuebar.statusTable[key] = 0
                else
                    valuebar.statusTable[key] = 1
                end
                valuebar.UpdateCount = valuebar.UpdateCount  - 1
                valuebar.togglefunction()
            end
        end

        if valuebar.UpdateCount == 0 then
            RemoveCallback(Updater, "ExecuteUpdate", valuebar.updateCallback)
        end
    end

    valuebar.Unload = function()

    end

    valuebar.count = 0
    if data["Conjuction"] then
        valuebar.minCount = tablelength(data["Effects"]) + tablelength(data["HealthThreshold"])
        + tablelength(data["Fight"])
        + (tablelength(data["FightPhase"]) > 0 and 1 or 0)
        + (tablelength(data["Location"]) > 0 and 1 or 0)
        + (tablelength(data["Resource"]) > 0 and 1 or 0)
        + tablelength(data["EffectAdded_Target"])
        + tablelength(data["Target_HealthThreshold"])
        + tablelength(data["Skill_Ready"])
        + (data["Has_Target"] ~= nil and data["Has_Target"] and 1 or 0)
        + tablelength(data["EventTrigger"])
    else
        valuebar.minCount = 1
    end

    if tablelength(data["Effects"]) > 0 then
        for key, value in pairs(data["Effects"]) do
            valuebar.applicationTable[string.format("Effects%s", key)] = value.Applications
            if value.Not_Applied == true then
                valuebar.statusTable[string.format("Effects%s", key)] = 1
            else
                valuebar.statusTable[string.format("Effects%s", key)] = 0
            end
        end

        for i = 1, self.player:GetEffects():GetCount(), 1 do
            local effect = self.player:GetEffects():Get(i)
            local value = data["Effects"][effect:GetName()]
            if value then
                if value.Not_Applied == true then
                    valuebar.statusTable[string.format("Effects%s", effect:GetName())] = valuebar.statusTable[string.format("Effects%s", effect:GetName())] - 1
                else
                    valuebar.statusTable[string.format("Effects%s", effect:GetName())] = valuebar.statusTable[string.format("Effects%s", effect:GetName())] + 1
                end
            end
        end

        table.insert(self.effectAddedCallbacks, AddCallback(PlayerEffectsHandler, "EffectAdded", 
        function(effect)
            local name = effect:GetName()
            if data["Effects"][name] ~= nil then
                local value = data["Effects"][name]
                if value.Not_Applied == true then
                    valuebar.statusTable[string.format("Effects%s", name)] = valuebar.statusTable[string.format("Effects%s", name)] - 1
                else
                    valuebar.statusTable[string.format("Effects%s", name)] = valuebar.statusTable[string.format("Effects%s", name)] + 1
                end
                valuebar.togglefunction()
            end
        end))

        table.insert(self.effectRemovedCallbacks, AddCallback(PlayerEffectsHandler, "EffectRemoved", 
        function(effect)
            local name = effect:GetName()
            if data["Effects"][name] ~= nil then
                local value = data["Effects"][effect:GetName()]
                if value.Not_Applied == true then
                    valuebar.statusTable[string.format("Effects%s", name)] = valuebar.statusTable[string.format("Effects%s", name)] + 1
                else
                    valuebar.statusTable[string.format("Effects%s", name)] = valuebar.statusTable[string.format("Effects%s", name)] - 1
                end

                valuebar.togglefunction()
            end
        end))
    end

    if tablelength(data["FightPhase"]) > 0 then
        table.insert(self.fightEventEndCallbacks, AddCallback(InstanceController, "FightEventEnd", function(args)
            for key, value in pairs(data["FightPhase"]) do
                if value.Boss_Name == args.Boss_Name and (value.Tier_Info == true or tostring(value.Tier_Info) == args.Boss_Tier) then
                    valuebar.statusTable[string.format("FightPhase%s%s", value.Boss_Name, tostring(value.Tier_Info))] = 0
                end
            end
        end))
    end

    if tablelength(data["FightPhase"]) > 0 then
        for key, value in pairs(data["FightPhase"]) do
            if value.Not_Applied == true then
                valuebar.statusTable[string.format("FightPhase%s%s", value.Boss_Name, tostring(value.Tier_Info))] = 1
            else
                valuebar.statusTable[string.format("FightPhase%s%s", value.Boss_Name, tostring(value.Tier_Info))] = 0
            end
        end
        table.insert(self.fightEventPhaseCallback, AddCallback(InstanceController, "FightEventPhase", function(args)
            for key, value in pairs(data["FightPhase"]) do
                if value.Boss_Name == args.Boss_Name and (value.Tier_Info == true or tostring(value.Tier_Info) == args.Boss_Tier) then
                    if value.Phase == args.Phase then
                        valuebar.statusTable[string.format("FightPhase%s%s", value.Boss_Name, tostring(value.Tier_Info))] = 1
                    else
                        valuebar.statusTable[string.format("FightPhase%s%s", value.Boss_Name, tostring(value.Tier_Info))] = 0
                    end
                end
            end
        end))
    end

    if tablelength(data["HealthThreshold"]) > 0 then
        local health = self.player:GetMorale() / self.player:GetMaxMorale() * 100
        valuebar.healthThresholdPercentage = {}
        for key, value in pairs(data["HealthThreshold"]) do
            local checkValue = (self.compareFunctions[value["Comparator"]](health, value.Number) and 1) or 0
            valuebar.statusTable[tostring(key) .. tostring(value["Comparator"])] = checkValue
            valuebar.healthThresholdPercentage[tostring(key) .. tostring(value["Comparator"])] = checkValue
        end
        table.insert(self.playerHealthCallbacks, AddCallback(self.player, "MoraleChanged", function()
            local percentage = self.player:GetMorale() / self.player:GetMaxMorale() * 100
            for key, value in pairs(data["HealthThreshold"]) do
                local id = tostring(key) .. tostring(value["Comparator"])
                if self.compareFunctions[value["Comparator"]](percentage, value.Number) then
                    if valuebar.healthThresholdPercentage[id] == 0 then
                        if valuebar.healthThresholdPercentage[id] == 0 then
                            valuebar.healthThresholdPercentage[id] = 1
                            valuebar.statusTable[tostring(key) .. tostring(value["Comparator"])] = valuebar.statusTable[tostring(key) .. tostring(value["Comparator"])] + 1
                        end
                    end
                else
                    if valuebar.healthThresholdPercentage[id] == 1 then
                        valuebar.healthThresholdPercentage[id] = 0
                        valuebar.statusTable[tostring(key) .. tostring(value["Comparator"])] = valuebar.statusTable[tostring(key) .. tostring(value["Comparator"])] - 1
                    end
                end
            end
            valuebar.togglefunction()        
        end))
    end

    if tablelength(data["Target_HealthThreshold"]) > 0 then
        valuebar.targetHealthThresholdCooldowns = {}
        valuebar.targetHealthThresholdPercentage = {}
        local health = self.player:GetMorale() / self.player:GetMaxMorale() * 100
        for key, value in pairs(data["Target_HealthThreshold"]) do
            valuebar.targetHealthThresholdCooldowns[key .. tostring(value["Number"]) .. tostring(value["Comparator"])] = Turbine.Engine.GetGameTime() - 1000
            valuebar.targetHealthThresholdPercentage[key .. tostring(value["Number"]) .. tostring(value["Comparator"])] = true
        end


        table.insert(self.healthTargetCallbacks, TargetHandlerObject:AddHealthCallback(function(value)
            if valuebar.targetMaxHealth > 0 then
                local percentage = value / valuebar.targetMaxHealth * 100
                for key, tableValue in pairs(data["Target_HealthThreshold"]) do
                    if key == valuebar.TargetName or key == "" then
                        if self.compareFunctions[tableValue["Comparator"]](percentage, tableValue["Number"]) then
                            local id = key .. tostring(tableValue["Number"]) .. tostring(tableValue["Comparator"])
                            valuebar.targetHealthThresholdPercentage[id] = false
                            valuebar.statusTable[string.format("%s%s", "TargetHealthThreshold", key)] = 1
                        else
                            valuebar.statusTable[string.format("%s%s", "TargetHealthThreshold", key)] = 0
                            valuebar.targetHealthThresholdPercentage[key .. tostring(tableValue["Number"]) .. tostring(tableValue["Comparator"])] = true
                        end
                    end
                end
                valuebar.togglefunction()    
            end
        end))

        table.insert(self.combatCallbacks, AddCallback(self.player, "InCombatChanged", function()
            if not(self.player:IsInCombat()) then
                for key, value in pairs(data["Target_HealthThreshold"]) do
                    valuebar.targetHealthThresholdCooldowns[key .. tostring(value["Number"]) .. tostring(value["Comparator"])] = Turbine.Engine.GetGameTime() - 1000
                    valuebar.targetHealthThresholdPercentage[key .. tostring(value["Number"]) .. tostring(value["Comparator"])] = true
                end 
            end
        end))
    end

    if tablelength(data["Resource"]) > 0 then
        for key, value in pairs(data["Resource"]) do
            valuebar.statusTable[value.Comparator .. value.Number] = 0
        end

        valuebar.resourceFunction = function(resourceNumber)
            for key, value in pairs(data["Resource"]) do
                valuebar.statusTable[value.Comparator .. value.Number] = self.compareFunctions[value.Comparator](resourceNumber, value.Number) and 1 or 0
            end
            valuebar.togglefunction()  
        end

        if self.playerClass == Turbine.Gameplay.Class.Beorning then
            valuebar.resourceFunction(self.ClassAttr:GetWrath())
            if not(self.attrCallbacks["WrathChanged"] ) then self.attrCallbacks["WrathChanged"] = {} end
            table.insert(self.attrCallbacks["WrathChanged"], AddCallback(self.ClassAttr, "WrathChanged", function(sender, args) valuebar.resourceFunction(self.ClassAttr:GetWrath()) end))
        elseif self.playerClass == Turbine.Gameplay.Class.Champion then
            valuebar.resourceFunction(self.ClassAttr:GetFervor())
            if not(self.attrCallbacks["FervorChanged"] ) then self.attrCallbacks["FervorChanged"] = {} end
            table.insert(self.attrCallbacks["FervorChanged"], AddCallback(self.ClassAttr, "FervorChanged", function(sender, args) valuebar.resourceFunction( self.ClassAttr:GetFervor() ) end))
        elseif self.playerClass == Turbine.Gameplay.Class.Hunter then
            valuebar.resourceFunction(self.ClassAttr:GetFocus())
            if not(self.attrCallbacks["FocusChanged"] ) then self.attrCallbacks["FocusChanged"] = {} end
            table.insert(self.attrCallbacks["FocusChanged"], AddCallback(self.ClassAttr, "FocusChanged", function(sender, args) valuebar.resourceFunction( self.ClassAttr:GetFocus() ) end))
        elseif self.playerClass == Turbine.Gameplay.Class.RuneKeeper then
            if not(self.attrCallbacks["AttunementChanged"] ) then self.attrCallbacks["AttunementChanged"] = {} end
            valuebar.resourceFunction(self.ClassAttr:GetAttunement())
            table.insert(self.attrCallbacks["AttunementChanged"], AddCallback(self.ClassAttr, "AttunementChanged", function(sender, args) valuebar.resourceFunction( self.ClassAttr:GetAttunement() ) end))
        end
    end

    if tablelength(data["Location"]) > 0 then
        table.insert(self.locationCallbacks, AddCallback(LocationHandler, "Location", function(args)
            valuebar.locationValue = data["Location"][args.Location] and 1 or 0
            valuebar.togglefunction()  
        end))
    end

    if tablelength(data["EffectAdded_Target"]) > 0 then
        table.insert(self.effectTargetAddedCallbacks, TargetHandlerObject:AddEffectCallback(
        function(args)
            local name = args.Name
            if data["EffectAdded_Target"][name] then
                local value = data["EffectAdded_Target"][name]

                local tableName = string.format("EffectAdded_Target%s", name)

                if not(valuebar.UpdateItems[tableName]) then
                    valuebar.statusTable[tableName] = 1
                end

                valuebar.UpdateItems[tableName] = args.Duration - (Turbine.Engine.GetGameTime() - args.StartTime)

                if valuebar.UpdateCount == 0 then
                    AddCallback(Updater, "ExecuteUpdate", valuebar.updateCallback)
                end
                valuebar.UpdateCount = valuebar.UpdateCount + 1
                valuebar.togglefunction()  
            end
        end))
    end

    if tablelength(data["EventTrigger"]) > 0 then
        for key, value in pairs(data["EventTrigger"]) do
            local eventName = string.format("EventTrigger%s", value.Condition_Name)
            if not(valuebar.statusTable[eventName]) then
                if value.Not_Applied then
                    valuebar.statusTable[eventName] = 1
                else
                    valuebar.statusTable[eventName] = 0
                end
            else
                if value.Not_Applied then
                    valuebar.statusTable[eventName] = 1
                end
            end
        end

        table.insert(self.eventTriggerCallbacks, AddCallback(self, "EventTrigger", 
        function(event)
            local value = data["EventTrigger"][event]
            if value then

                local eventName = string.format("EventTrigger%s", value.Condition_Name)
                if value.Not_Applied then
                    valuebar.statusTable[eventName] = 0
                else
                    valuebar.statusTable[eventName] = 1
                end

                
                if not(valuebar.EventItems[eventName]) then
                    if valuebar.UpdateCount == 0 then
                        AddCallback(Updater, "ExecuteUpdate", valuebar.updateCallback)
                    end
                    valuebar.UpdateCount = valuebar.UpdateCount + 1
                end
                
                valuebar.EventItems[eventName] = value.Time

                valuebar.togglefunction()
            end
        end))
    end

    if tablelength(data["EffectAdded_Target"]) > 0 and data["Reset_On_Retarget"] or tablelength(data["Target_HealthThreshold"]) > 0 or data["Has_Target"] then
        table.insert(self.targetInfoCallbacks, TargetHandlerObject:AddTargetInfoCallback(function(value) 
            if value then
                valuebar.targetMaxHealth = value["MaxMorale"]
                valuebar.TargetName = value["Name"]

                if data["Has_Target"] ~= nil and data["Has_Target"] then
                    valuebar.statusTable.TargetValue = 1
                end
            else
                valuebar.targetMaxHealth = -1
                valuebar.TargetName = ""

                if data["Has_Target"] ~= nil and data["Has_Target"] then
                    valuebar.statusTable.TargetValue = 0
                end
            end

            if valuebar.UpdateCount > 0 then
                for key, _ in pairs(valuebar.UpdateItems) do
                    valuebar.UpdateItems[key] = nil
                    valuebar.statusTable[key] = 0
                    valuebar.UpdateCount = valuebar.UpdateCount - 1
                end
    
                valuebar.UpdateItems = nil
                valuebar.UpdateItems = {}
    
                if valuebar.UpdateCount == 0 then
                    RemoveCallback(Updater, "ExecuteUpdate", valuebar.updateCallback) 
                end
            end
            valuebar.togglefunction()
        end))
    end

    if tablelength(data["Skill_Ready"]) > 0 then
        self.skills = true
        for key, value in pairs(data["Skill_Ready"]) do
            if SkillEventHandler:SkillExists(key) then

                if SkillEventHandler:GetSkill(key) then
                    local tableName = string.format("Skill_Ready%s", SkillEventHandler:GetSkill(key):GetSkillInfo():GetName())
                    valuebar.SkillValue[tableName] = value.Not_Applied
                    if SkillEventHandler:GetSkill(key):GetResetTime() == -1 then
                        if value.Not_Applied then
                            valuebar.statusTable[tableName] = 0
                        else
                            valuebar.statusTable[tableName] = 1
                        end
                    else
                        if value.Not_Applied then
                            valuebar.statusTable[tableName] = 1
                        else
                            valuebar.statusTable[tableName] = 0
                        end

                        valuebar.SkillItems[tableName] = SkillEventHandler:GetSkill(key):GetResetTime()
                        if valuebar.UpdateCount == 0 then
                            AddCallback(Updater, "ExecuteUpdate", valuebar.updateCallback)
                        end
                        valuebar.UpdateCount = valuebar.UpdateCount + 1
                    end
                end

                if not(self.skillCallbacks[key]) then self.skillCallbacks[key] = {} end
                table.insert(self.skillCallbacks[key], AddCallback(SkillEventHandler, key, function(skill) 
                    local tableName = string.format("Skill_Ready%s", skill:GetSkillInfo():GetName())

                    local resetTime = skill:GetResetTime()

                    -- Skill is ready
                    if resetTime == -1  then
                        -- We want to see it when it is on cooldown
                        if value.Not_Applied then
                            -- Set status to 0
                            valuebar.statusTable[tableName] = 0

                            -- If it was previously on cd we need to adjust
                            if valuebar.SkillItems[tableName] then
                                valuebar.SkillItems[tableName] = nil
                                valuebar.UpdateCount = valuebar.UpdateCount - 1
                                if valuebar.UpdateCount == 0 then
                                    RemoveCallback(Updater, "ExecuteUpdate", valuebar.updateCallback) 
                                end
                            end
                        else
                            -- Not on cd and we want to see it now
                            valuebar.statusTable[tableName] = 1
                        end
                    else
                        -- Skill is on cooldown
                        if value.Not_Applied then
                            -- We want to see it
                            valuebar.statusTable[tableName] = 1
                        else
                            -- We do not want to see it
                            valuebar.statusTable[tableName] = 0
                        end
                        
                        -- Set update procedure to track when we need to adjust
                        valuebar.SkillItems[tableName] = resetTime
                        if valuebar.UpdateCount == 0 then
                            AddCallback(Updater, "ExecuteUpdate", valuebar.updateCallback)
                        end
                        valuebar.UpdateCount = valuebar.UpdateCount + 1

                    end
                    valuebar.togglefunction()
                end))
            end
        end
    end

    valuebar.togglefunction()

    table.insert(self.tools, valuebar)
end