function PrimeUI:Countdown(data, parent)
    local valuebar = Turbine.UI.Control()
    valuebar:SetParent(parent)
    if self.groupMode then
        valuebar:SetPosition(data["X"], data["Y"])
        valuebar.moveBox = MoveBox(valuebar, ((data["Width"] or 100) * (data["Scale"] or 1)) or 100, ((data["Height"] or 50) * (data["Scale"] or 1)) or 50, data["NickName"] or "Countdown")
        valuebar.savePosition = function()
            data["X"], data["Y"] = valuebar:GetPosition()
        end
    else
        valuebar:SetPosition(self:ContentAlignement((data["Alignment"] or "Middle"), (data["Width"] or 100) * (data["Scale"] or 1)), self.currentHeight)
    end
    valuebar:SetSize((data["Width"] or 100) * (data["Scale"] or 1), (data["Height"] or 50) * (data["Scale"] or 1))
    valuebar.countdownWindow = CountdownWindowPrimeUI(valuebar, 0, 0, data["Width"], data["Height"], self:FontToFont(data["Font"]), data["Scale"], self:ColourTupleToTurbineColour(data["FontColour"]), self:ColourTupleToTurbineColour(data["ThresholdColour"]), data["Threshold"])
    self.lookup[data.NickName or ""] = valuebar.countdownWindow
    if tablelength(data["EventTrigger"]) > 0 then
        table.insert(self.eventTriggerCallbacks, AddCallback(self, "EventTrigger", 
        function(event)
            if data["EventTrigger"][event] then
                if type(data["EventTrigger"][event]) == "table" then
                    valuebar.countdownWindow:Countdown(data["EventTrigger"][event]["Countdown"], data["Visibility_Threshold"], data["EventTrigger"][event]["Event_Keyword"] and function() self:FireTrigger(data["EventTrigger"][event]["Event_Keyword"]) end)
                else
                    valuebar.countdownWindow:Countdown(data["EventTrigger"][event], data["Visibility_Threshold"])
                end
            end
        end))
    end

    if tablelength(data["EffectAdded"]) > 0 then
        table.insert(self.effectAddedCallbacks, AddCallback(PlayerEffectsHandler, "EffectAdded", 
        function(effect)
            local name = effect:GetName()
            local value = data["EffectAdded"][name]
            if value then
                if type(value) == "table" then
                    local time = value.Countdown
                    if time == true then
                        time = effect:GetDuration() - (Turbine.Engine.GetGameTime() - effect:GetStartTime())
                    end
                    valuebar.countdownWindow:Countdown(time, data["Visibility_Threshold"], value["Event_Keyword"] and (function() self:FireTrigger(value["Event_Keyword"]) end))
                else
                    if value == true then
                        value = effect:GetDuration() - (Turbine.Engine.GetGameTime() - effect:GetStartTime())
                    end
                    valuebar.countdownWindow:Countdown(value, data["Visibility_Threshold"])
                end
            end
        end))
    end

    if tablelength(data["EffectAdded_Target"]) > 0 then
        table.insert(self.effectTargetAddedCallbacks, TargetHandlerObject:AddEffectCallback(
        function(args)
            local name = args.Name
            local value = data["EffectAdded_Target"][name]
            if value then
                if type(value) == "table" then
                    local time = value.Countdown
                    if time == true then
                        time = args.Duration - (Turbine.Engine.GetGameTime() - args.StartTime)
                    end
                    valuebar.countdownWindow:Countdown(time, data["Visibility_Threshold"], value["Event_Keyword"] and (function() self:FireTrigger(value["Event_Keyword"]) end))
                else
                    if value == true then
                        value = args.Duration - (Turbine.Engine.GetGameTime() - args.StartTime)
                    end
                    valuebar.countdownWindow:Countdown(value, data["Visibility_Threshold"])
                end
            end
        end))
    end

    if tablelength(data["EffectAdded_Group"]) > 0 then
        table.insert(self.effectAddedGroupCallbacks, AddCallback(GroupEffectsHandler, "EffectAdded", 
        function(args)
            local value = data["EffectAdded_Target"][args.effect.Name]
            if value then
                local effectArgs = args.effect
                if type(value) == "table" then
                    local time = value.Countdown
                    if time == true then
                        time = effectArgs.Duration - (Turbine.Engine.GetGameTime() - effectArgs.StartTime)
                    end
                    valuebar.countdownWindow:Countdown(time, data["Visibility_Threshold"], value["Event_Keyword"] and (function() self:FireTrigger(value["Event_Keyword"]) end))
                else
                    if value == true then
                        value = effectArgs.Duration - (Turbine.Engine.GetGameTime() - effectArgs.StartTime)
                    end
                    valuebar.countdownWindow:Countdown(value, data["Visibility_Threshold"])
                end
            end
        end))
    end

    if tablelength(data["EffectRemoved"]) > 0 then
        table.insert(self.effectRemovedCallbacks, AddCallback(PlayerEffectsHandler, "EffectRemoved", 
        function(effect)
            local name = effect:GetName()
            local value = data["EffectRemoved"][name]
            if value then
                if type(value) == "table" then
                    local time = value.Countdown
                    if time == true then
                        time = effect:GetDuration() - (Turbine.Engine.GetGameTime() - effect:GetStartTime())
                    end
                    valuebar.countdownWindow:Countdown(time, data["Visibility_Threshold"], value["Event_Keyword"] and (function() self:FireTrigger(value["Event_Keyword"]) end))
                else
                    if value == true then
                        value = effect:GetDuration() - (Turbine.Engine.GetGameTime() - effect:GetStartTime())
                    end
                    valuebar.countdownWindow:Countdown(value, data["Visibility_Threshold"])
                end
            end
        end))
    end

    if tablelength(data["Action"]) > 0 then
        table.insert(self.actionCallbacks, AddCallback(ActionHandlerObject, "Action", function(args)
            local value = data["Action"][args.Action]
            if value then
                if type(value) == "table" then
                    valuebar.countdownWindow:Countdown(value.Countdown, data["Visibility_Threshold"], value["Event_Keyword"] and (function() self:FireTrigger(value["Event_Keyword"]) end))
                else
                    valuebar.countdownWindow:Countdown(value, data["Visibility_Threshold"])
                end
            end
        end))
    end

    if tablelength(data["EnemyHits"]) > 0 then
        table.insert(self.enemyChatCallback, AddCallback(ChatEventHandler, "EnemyCombat", function(args)
            local value = data["EnemyHits"][args.SkillName]
            if value then
                if type(value) == "table" then
                    valuebar.countdownWindow:Countdown(value.Countdown, data["Visibility_Threshold"], value["Event_Keyword"] and (function() self:FireTrigger(value["Event_Keyword"]) end))
                else
                    valuebar.countdownWindow:Countdown(value, data["Visibility_Threshold"])
                end
            end
        end))

    end

    if tablelength(data["PlayerHits"]) > 0 then
        table.insert(self.playerChatCallback, AddCallback(ChatEventHandler, "PlayerCombat", function(args)
            local value = data["PlayerHits"][args.SkillName]
            if value then
                if type(value) == "table" then
                    valuebar.countdownWindow:Countdown(value.Countdown, data["Visibility_Threshold"], value["Event_Keyword"] and (function() self:FireTrigger(value["Event_Keyword"]) end))
                else
                    valuebar.countdownWindow:Countdown(value, data["Visibility_Threshold"])
                end
            end
        end))
    end

    if tablelength(data["StandardChat"]) > 0 then
        table.insert(self.standardChatCallback, AddCallback(ChatEventHandler, "Standard", function(message)
            local value = data["StandardChat"][args.SkillName]
            if value then
                if type(value) == "table" then
                    valuebar.countdownWindow:Countdown(value.Countdown, data["Visibility_Threshold"], value["Event_Keyword"] and (function() self:FireTrigger(value["Event_Keyword"]) end))
                else
                    valuebar.countdownWindow:Countdown(value, data["Visibility_Threshold"])
                end
            end
        end))
    end

    if tablelength(data["PartyChat"]) > 0 then
        table.insert(self.partyChatCallback, AddCallback(ChatEventHandler, "Party", function(message)
            local value = data["PartyChat"][message]
            if value then
                if type(value) == "table" then
                    valuebar.countdownWindow:Countdown(value.Countdown, data["Visibility_Threshold"], value["Event_Keyword"] and (function() self:FireTrigger(value["Event_Keyword"]) end))
                else
                    valuebar.countdownWindow:Countdown(value, data["Visibility_Threshold"])
                end
            end
        end))
    end

    if tablelength(data["SayChat"]) > 0 then
        table.insert(self.sayChatCallback, AddCallback(ChatEventHandler, "SayLine", function(message)
            local value = data["SayChat"][message]
            if value then
                if type(value) == "table" then
                    valuebar.countdownWindow:Countdown(value.Countdown, data["Visibility_Threshold"], value["Event_Keyword"] and (function() self:FireTrigger(value["Event_Keyword"]) end))
                else
                    valuebar.countdownWindow:Countdown(value, data["Visibility_Threshold"])
                end
            end
        end))
    end

    if tablelength(data["PlayerBenefit"]) > 0 then
        table.insert(self.playerBenefitCallbacks, AddCallback(ChatEventHandler, "PlayerBenefit", function(args)
            local value = data["PlayerBenefit"][args.SkillName]
            if value then
                if type(value) == "table" then
                    valuebar.countdownWindow:Countdown(value.Countdown, data["Visibility_Threshold"], value["Event_Keyword"] and (function() self:FireTrigger(value["Event_Keyword"]) end))
                else
                    valuebar.countdownWindow:Countdown(value, data["Visibility_Threshold"])
                end
            end
        end))
    end

    if tablelength(data["PlayerHeal"]) > 0 then
        table.insert(self.playerHealCallbacks, AddCallback(ChatEventHandler, "PlayerHeal", function(args)
            local value = data["PlayerHeal"][args.SkillName]
            if value then
                if type(value) == "table" then
                    valuebar.countdownWindow:Countdown(value.Countdown, data["Visibility_Threshold"], value["Event_Keyword"] and (function() self:FireTrigger(value["Event_Keyword"]) end))
                else
                    valuebar.countdownWindow:Countdown(value, data["Visibility_Threshold"])
                end
            end
        end))
    end


    if tablelength(data["Skills"]) > 0 then
        self.skills = true
        for key, value in pairs(data["Skills"]) do
            if SkillEventHandler:SkillExists(key) then
                -- Add the callback for this skill slot
                if not(self.skillCallbacks[key]) then self.skillCallbacks[key] = {} end
                table.insert(self.skillCallbacks[key], AddCallback(SkillEventHandler, key, function(skill) 
                    local value = data["Skills"][key]
                    if value then
                        if type(value) == "table" then
                            local time = value.Countdown
                            if time == true then
                                time = skill:GetResetTime() - Turbine.Engine.GetGameTime()
                            end
                            if (skill:GetResetTime() - Turbine.Engine.GetGameTime()) + 1 <= skill:GetCooldown() then
                                return
                            end
                            valuebar.countdownWindow:Countdown(time, data["Visibility_Threshold"], value["Event_Keyword"] and (function() self:FireTrigger(value["Event_Keyword"]) end))
                        else
                            if value == true then
                                value = skill:GetResetTime() - Turbine.Engine.GetGameTime()
                            end
                            valuebar.countdownWindow:Countdown(value, data["Visibility_Threshold"])
                        end
                    end
                end))
            end
        end
    end

    if tablelength(data["Target_HealthThreshold"]) > 0 then
        valuebar.targetHealthThresholdCooldowns = {}
        valuebar.targetHealthThresholdPercentage = {}
        valuebar.targetMaxHealth = 0
        local health = self.player:GetMorale() / self.player:GetMaxMorale() * 100
        for key, value in pairs(data["Target_HealthThreshold"]) do
            valuebar.targetHealthThresholdCooldowns[key .. tostring(value["Number"]) .. tostring(value["Operator"])] = Turbine.Engine.GetGameTime() - 1000
            valuebar.targetHealthThresholdPercentage[key .. tostring(value["Number"]) .. tostring(value["Operator"])] = true
        end

        table.insert(self.targetInfoCallbacks, TargetHandlerObject:AddTargetInfoCallback(function(value) 
            if value then
                valuebar.targetMaxHealth = value["MaxMorale"]
                valuebar.TargetName = value["Name"]
            else
                valuebar.targetMaxHealth = 0
                valuebar.TargetName = ""
            end
        end))

        table.insert(self.healthTargetCallbacks, TargetHandlerObject:AddHealthCallback(function(value)
            if valuebar.targetMaxHealth > 0 then
                local percentage = value / valuebar.targetMaxHealth * 100
                local time = Turbine.Engine.GetGameTime()
                for key, value in pairs(data["Target_HealthThreshold"]) do
                    if key == valuebar.TargetName then
                        if self.compareFunctions[value["Operator"]](percentage, value["Number"]) then
                            local id = key .. tostring(value["Number"]) .. tostring(value["Operator"])
                            if valuebar.targetHealthThresholdCooldowns[id] < time and valuebar.targetHealthThresholdPercentage[id] then
                                valuebar.targetHealthThresholdPercentage[id] = false
                                valuebar.countdownWindow:Countdown(value["Countdown"], data["Visibility_Threshold"], value["Event_Keyword"] and (function() self:FireTrigger(value["Event_Keyword"]) end))
                                valuebar.targetHealthThresholdCooldowns[id] = time + 6
                            end
                        else
                            valuebar.targetHealthThresholdPercentage[key .. tostring(value["Number"]) .. tostring(value["Operator"])] = true
                        end
                    end
                end
            end
        end))
        table.insert(self.combatCallbacks, AddCallback(self.player, "InCombatChanged", function()
            if not(self.player:IsInCombat()) then
                for key, value in pairs(data["Target_HealthThreshold"]) do
                    valuebar.targetHealthThresholdCooldowns[key .. tostring(value["Number"]) .. tostring(value["Operator"])] = Turbine.Engine.GetGameTime() - 1000
                    valuebar.targetHealthThresholdPercentage[key .. tostring(value["Number"]) .. tostring(value["Operator"])] = true
                end 
            end
        end))
    end

    if not(data["Height"]) then
        data["Width"] = 100
        data["Height"] = 50
    end

    self.currentHeight = self.currentHeight + (data["Height"] or 125) * (data["Scale"] or 1)

    table.insert(self.tools, valuebar.countdownWindow)
    table.insert(self.UI, valuebar)
    
end