function PrimeUI:Counter(data, parent)
    local x = self:ContentAlignement(data["Alignment"], (data["Width"] + 2 * data["BorderSize"]) * data["Scale"])
    local y = self.currentHeight
    if self.groupMode then
        x = data["X"]
        y = data["Y"]
    end
    local valuebar = PrimeUITools.CounterListBox(parent, data.Width, data.Height, data.ItemHeight, x, y, Turbine.UI.ContentAlignment[data.TextAlignment], self:FontToFont(data.Text_Font), self:ColourTupleToTurbineColour(data.Text_Font_Colour), Turbine.UI.ContentAlignment[data.Count_TextAlignment], self:FontToFont(data.Count_Font), self:ColourTupleToTurbineColour(data.Count_Font_Colour), data.Item_BorderSize, self:ColourTupleToTurbineColour(data.Item_BorderColour), self:ColourTupleToTurbineColour(data.Item_BackgroundColour), data.Text_Width, data.Count_Width, data.Counter_Event, data.Hide_While_Not_Active)
    self.lookup[data.NickName or ""] = valuebar
    if self.groupMode then
        valuebar.moveBox = MoveBox(valuebar, data["Width"] * data["Scale"], data["Height"] * data["Scale"], data["NickName"] or "Counter")
        valuebar.savePosition = function()
            data["X"], data["Y"] = valuebar:GetPosition()
        end
    end

    if tablelength(data["Start_Keyword"]) > 0 then
        valuebar:SetupStartmode(true)
        table.insert(self.eventTriggerCallbacks, AddCallback(self, "EventTrigger", 
        function(event)
            if data["Start_Keyword"][event] then
                valuebar:ToggleStartValue(true)
            end
        end))
    end

    if tablelength(data["End_Keyword"]) > 0 then
        valuebar:SetupStartmode(true)
        table.insert(self.eventTriggerCallbacks, AddCallback(self, "EventTrigger", 
        function(event)
            if data["End_Keyword"][event] then
                valuebar:ToggleStartValue(false)
            end
        end))
    end

    if tablelength(data["Reset_Keyword"]) > 0 then
        table.insert(self.eventTriggerCallbacks, AddCallback(self, "EventTrigger", 
        function(event)
            if data["Reset_Keyword"][event] then
                valuebar:Reset()
            end
        end))
    end

    valuebar.UpdateItems = {}
    valuebar.ItemValue = {}
    valuebar.SkillID = {}
    valuebar.UpdateCount = 0

    valuebar.updateCallback = function(delta)
        for key, value in pairs(valuebar.UpdateItems) do
            valuebar.UpdateItems[key] = valuebar.UpdateItems[key] - delta
            if valuebar.UpdateItems[key] < 0 then
                local counterValue = valuebar.ItemValue[key]
                valuebar:Trigger(counterValue.Text, counterValue.Decrement, counterValue.Number)
                valuebar.ItemValue[key] = nil
                valuebar.UpdateItems[key] = nil
                valuebar.UpdateCount = valuebar.UpdateCount  - 1
            end
        end

        if valuebar.UpdateCount == 0 then
            RemoveCallback(Updater, "ExecuteUpdate", valuebar.updateCallback)
        end
    end

    valuebar.counterFunc = function(text, decrement, number, conditionName, timer)
        if conditionName and conditionName ~= "" then
            if not(valuebar.ItemValue[conditionName])  then
                valuebar.UpdateItems[conditionName] = timer
                valuebar.ItemValue[conditionName] = {["Text"] = text, ["Decrement"] = not(decrement), ["Number"] = number}
    
                if valuebar.UpdateCount == 0 then
                    AddCallback(Updater, "ExecuteUpdate", valuebar.updateCallback)
                end
                valuebar.UpdateCount = valuebar.UpdateCount + 1
                valuebar:Trigger(text, decrement, number)
            else
                valuebar:Trigger(text, decrement, valuebar.ItemValue[conditionName] + (number - valuebar.ItemValue[conditionName]))
                valuebar.ItemValue[conditionName] = number
                valuebar.UpdateItems[conditionName] = timer
            end
        else
            valuebar:Trigger(text, decrement, number)
        end
    end


    if tablelength(data["Corruption"]) > 0 then
        for _, itemData in pairs(data["Corruption"]) do
            valuebar:AddItem(itemData.Text, itemData.StartCount, itemData.MinVis, itemData.MaxVis)
        end
        table.insert(self.playerDispellCallback, AddCallback(ChatEventHandler, "PlayerDispell", function(args)
            for _, triggerData in pairs(data["Corruption"]) do
                if (triggerData.EffectName == "" or triggerData.EffectName == args.EffectName) and (triggerData.Target == "" or triggerData.Target == args.Target) then
                    valuebar.counterFunc(triggerData.Text, not(triggerData.Decrement), triggerData.Number, triggerData.Condition_Name, triggerData.Time)
                end
            end
        end))
    end

    if tablelength(data["PlayerHits"]) > 0 then
        for _, itemData in pairs(data["PlayerHits"]) do
            valuebar:AddItem(itemData.Text, itemData.StartCount, itemData.MinVis, itemData.MaxVis)
        end
        table.insert(self.playerChatCallback, AddCallback(ChatEventHandler, "PlayerCombat", function(args)
            local value = data["PlayerHits"][args.SkillName]
            if value then
                valuebar.counterFunc(value.Text, not(value.Decrement), value.Number, value.Condition_Name, value.Time)
            end
        end))
    end

    if tablelength(data["DefeatEvent"]) > 0 then
        for _, itemData in pairs(data["DefeatEvent"]) do
            valuebar:AddItem(itemData.Text, itemData.StartCount, itemData.MinVis, itemData.MaxVis)
        end
        table.insert(self.defeatEventCallback, AddCallback(ChatEventHandler, "Defeated", function(args)
            if data["DefeatEvent"][args.Name] then
                local value = data["DefeatEvent"][args.Name]
                if value then
                    valuebar.counterFunc(value.Text, not(value.Decrement), value.Number, value.Condition_Name, value.Time)
                end
            end
        end))
    end

    if tablelength(data["EventTrigger"]) > 0 then
        for _, itemData in pairs(data["EventTrigger"]) do
            valuebar:AddItem(itemData.Text, itemData.StartCount, itemData.MinVis, itemData.MaxVis)
        end
        table.insert(self.eventTriggerCallbacks, AddCallback(self, "EventTrigger", 
        function(event)
            local value = data["EventTrigger"][event]
            if value then
                valuebar.counterFunc(value.Text, not(value.Decrement), value.Number, value.Condition_Name, value.Time)
            end
        end))
    end

    if tablelength(data["Effects"]) > 0 then
        for _, itemData in pairs(data["Effects"]) do
            valuebar:AddItem(itemData.Text, itemData.StartCount, itemData.MinVis, itemData.MaxVis)
        end
        valuebar.effectFunction = function(effect)
            local name = effect:GetName()
            if data["Effects"][name] then
                local value = data["Effects"][name]
                valuebar.counterFunc(value.Text, not(value.Decrement), value.Number)
                local ID = effect:GetID()
                valuebar.SkillID[ID] = {["Number"] = value.Number, ["Decrement"] = value.Decrement, ["Text"] = value.Text}
            end
        end

        for i = 1, self.player:GetEffects():GetCount() do
            valuebar.effectFunction(self.player:GetEffects():Get(i))
        end

        table.insert(self.effectAddedCallbacks, AddCallback(PlayerEffectsHandler, "EffectAdded", valuebar.effectFunction))

        table.insert(self.effectRemovedCallbacks, AddCallback(PlayerEffectsHandler, "EffectRemoved", 
        function(effect)
            local ID = effect:GetID()
            if valuebar.SkillID[ID] then
                local value = valuebar.SkillID[ID]
                valuebar:Trigger(value.Text, value.Decrement, value.Number)
            end

        end))
    end

    if tablelength(data["EffectAdded"]) > 0 then
        for _, itemData in pairs(data["EffectAdded"]) do
            valuebar:AddItem(itemData.Text, itemData.StartCount, itemData.MinVis, itemData.MaxVis)
        end
        table.insert(self.effectAddedCallbacks, AddCallback(PlayerEffectsHandler, "EffectAdded", 
        function(effect)
            local name = effect:GetName()
            local value = data["EffectAdded"][name]
            if value then
                valuebar.counterFunc(value.Text, not(value.Decrement), value.Number, value.Condition_Name, value.Time and value.Time == true and (effect:GetDuration() - (Turbine.Engine.GetGameTime() - effect:GetStartTime())) or value.Time)
            end
        end))
    end

    if tablelength(data["EffectAdded_Target"]) > 0 then
        for _, itemData in pairs(data["EffectAdded_Target"]) do
            valuebar:AddItem(itemData.Text, itemData.StartCount, itemData.MinVis, itemData.MaxVis)
        end
        table.insert(self.effectTargetAddedCallbacks, TargetHandlerObject:AddEffectCallback(
        function(args)
            local name = args.Name
            local value = data["EffectAdded_Target"][name]
            if value then
                valuebar.counterFunc(value.Text, not(value.Decrement), value.Number, value.Condition_Name, value.Time and value.Time == true and (args.Duration - (Turbine.Engine.GetGameTime() - args.StartTime)) or value.Time)
            end
        end))
    end

    if data["Reset_On_Retarget"] then
        table.insert(self.targetInfoCallbacks, TargetHandlerObject:AddTargetInfoCallback(function(value) 
            if valuebar.UpdateCount > 0 then
                for key, _ in pairs(valuebar.UpdateItems) do
                    valuebar.UpdateItems[key] = nil
                    valuebar.ItemValue[key] = nil
                    valuebar.UpdateCount = valuebar.UpdateCount - 1
                end
    
                valuebar:Reset()
                
                valuebar.UpdateItems = {}
                valuebar.ItemValue = {}
    
                if valuebar.UpdateCount == 0 then
                    RemoveCallback(Updater, "ExecuteUpdate", valuebar.updateCallback) 
                end
            end
        end))
    end

    if tablelength(data["EffectAdded_Group"]) > 0 then
        for _, itemData in pairs(data["EffectAdded_Group"]) do
            valuebar:AddItem(itemData.Text, itemData.StartCount, itemData.MinVis, itemData.MaxVis)
        end
        table.insert(self.effectAddedGroupCallbacks, AddCallback(GroupEffectsHandler, "EffectAdded", 
        function(args)
            local value = data["EffectAdded_Group"][args.effect.Name]
            if value then
                local effectArgs = args.effect
                valuebar.counterFunc(value.Text, not(value.Decrement), value.Number, value.Condition_Name, value.Time and value.Time == true and (effectArgs.Duration - (Turbine.Engine.GetGameTime() - effectArgs.StartTime)) or value.Time)
            end
        end))
    end

    if tablelength(data["EffectRemoved"]) > 0 then
        for _, itemData in pairs(data["EffectRemoved"]) do
            valuebar:AddItem(itemData.Text, itemData.StartCount, itemData.MinVis, itemData.MaxVis)
        end
        table.insert(self.effectRemovedCallbacks, AddCallback(PlayerEffectsHandler, "EffectRemoved", 
        function(effect)
            local name = effect:GetName()
            local value = data["EffectRemoved"][name]
            if value then
                valuebar.counterFunc(value.Text, not(value.Decrement), value.Number, value.Condition_Name, value.Time and value.Time == true and (effect:GetDuration() - (Turbine.Engine.GetGameTime() - effect:GetStartTime())) or value.Time)
            end
        end))
    end

    if tablelength(data["EnemyHits"]) > 0 then
        for _, itemData in pairs(data["EnemyHits"]) do
            valuebar:AddItem(itemData.Text, itemData.StartCount, itemData.MinVis, itemData.MaxVis)
        end
        table.insert(self.enemyChatCallback, AddCallback(ChatEventHandler, "EnemyCombat", function(args)
            local value = data["EnemyHits"][args.SkillName]
            if value then
                valuebar.counterFunc(value.Text, not(value.Decrement), value.Number, value.Condition_Name, value.Time)
            end
        end))

    end

    if tablelength(data["PlayerHits"]) > 0 then
        for _, itemData in pairs(data["PlayerHits"]) do
            valuebar:AddItem(itemData.Text, itemData.StartCount, itemData.MinVis, itemData.MaxVis)
        end
        table.insert(self.playerChatCallback, AddCallback(ChatEventHandler, "PlayerCombat", function(args)
            local value = data["PlayerHits"][args.SkillName]
            if value then
                valuebar.counterFunc(value.Text, not(value.Decrement), value.Number, value.Condition_Name, value.Time)
            end
        end))
    end

    if tablelength(data["StandardChat"]) > 0 then
        for _, itemData in pairs(data["StandardChat"]) do
            valuebar:AddItem(itemData.Text, itemData.StartCount, itemData.MinVis, itemData.MaxVis)
        end
        table.insert(self.standardChatCallback, AddCallback(ChatEventHandler, "Standard", function(message)
            local value = data["StandardChat"][message]
            if value then
                valuebar.counterFunc(value.Text, not(value.Decrement), value.Number, value.Condition_Name, value.Time)
            end
        end))
    end

    if tablelength(data["PartyChat"]) > 0 then
        for _, itemData in pairs(data["PartyChat"]) do
            valuebar:AddItem(itemData.Text, itemData.StartCount, itemData.MinVis, itemData.MaxVis)
        end
        table.insert(self.partyChatCallback, AddCallback(ChatEventHandler, "Party", function(message)
            local value = data["PartyChat"][message]
            if value then
                valuebar.counterFunc(value.Text, not(value.Decrement), value.Number, value.Condition_Name, value.Time)
            end
        end))
    end

    if tablelength(data["SayChat"]) > 0 then
        for _, itemData in pairs(data["SayChat"]) do
            valuebar:AddItem(itemData.Text, itemData.StartCount, itemData.MinVis, itemData.MaxVis)
        end
        table.insert(self.sayChatCallback, AddCallback(ChatEventHandler, "SayLine", function(message)
            local value = data["SayChat"][message]
            if value then
                valuebar.counterFunc(value.Text, not(value.Decrement), value.Number, value.Condition_Name, value.Time)
            end
        end))
    end

    if tablelength(data["PlayerBenefit"]) > 0 then
        for _, itemData in pairs(data["PlayerBenefit"]) do
            valuebar:AddItem(itemData.Text, itemData.StartCount, itemData.MinVis, itemData.MaxVis)
        end
        table.insert(self.playerBenefitCallbacks, AddCallback(ChatEventHandler, "PlayerBenefit", function(args)
            local value = data["PlayerBenefit"][args.SkillName]
            if value then
                valuebar.counterFunc(value.Text, not(value.Decrement), value.Number, value.Condition_Name, value.Time)
            end
        end))
    end

    if tablelength(data["PlayerHeal"]) > 0 then
        for _, itemData in pairs(data["PlayerHeal"]) do
            valuebar:AddItem(itemData.Text, itemData.StartCount, itemData.MinVis, itemData.MaxVis)
        end
        table.insert(self.playerHealCallbacks, AddCallback(ChatEventHandler, "PlayerHeal", function(args)
            local value = data["PlayerHeal"][args.SkillName]
            if value then
                valuebar.counterFunc(value.Text, not(value.Decrement), value.Number, value.Condition_Name, value.Time)
            end
        end))
    end


    if tablelength(data["Skills"]) > 0 then
        self.skills = true
        for _, itemData in pairs(data["Skills"]) do
            valuebar:AddItem(itemData.Text, itemData.StartCount, itemData.MinVis, itemData.MaxVis)
        end
        for key, value in pairs(data["Skills"]) do
            if SkillEventHandler:SkillExists(key) then
                -- Add the callback for this skill slot
                if not(self.skillCallbacks[key]) then self.skillCallbacks[key] = {} end
                table.insert(self.skillCallbacks[key], AddCallback(SkillEventHandler, key, function(skill) 
                    local value = data["Skills"][key]
                    if value then
                        if type(value) == "table" then
                            local time = value.Countdown
                            if time == true then
                                time = skill:GetResetTime() - Turbine.Engine.GetGameTime()
                            end
                            if (skill:GetResetTime() - Turbine.Engine.GetGameTime()) + 1 <= skill:GetCooldown() then
                                return
                            end
                            valuebar.counterFunc(value.Text, not(value.Decrement), value.Number, value.Condition_Name, value.Time)
                        else
                            if value == true then
                                value = skill:GetResetTime() - Turbine.Engine.GetGameTime()
                            end
                            valuebar.counterFunc(value.Text, not(value.Decrement), value.Number, value.Condition_Name, value.Time)
                        end
                    end
                end))
            end
        end
    end

    if tablelength(data["Target_HealthThreshold"]) > 0 then
        for _, itemData in pairs(data["Target_HealthThreshold"]) do
            valuebar:AddItem(itemData.Text, itemData.StartCount, itemData.MinVis, itemData.MaxVis)
        end
        valuebar.targetHealthThresholdCooldowns = {}
        valuebar.targetHealthThresholdPercentage = {}
        valuebar.targetMaxHealth = 0
        local health = self.player:GetMorale() / self.player:GetMaxMorale() * 100
        for key, value in pairs(data["Target_HealthThreshold"]) do
            valuebar.targetHealthThresholdCooldowns[key .. tostring(value["Number"]) .. tostring(value["Operator"])] = Turbine.Engine.GetGameTime() - 1000
            valuebar.targetHealthThresholdPercentage[key .. tostring(value["Number"]) .. tostring(value["Operator"])] = true
        end

        table.insert(self.targetInfoCallbacks, TargetHandlerObject:AddTargetInfoCallback(function(value) 
            if value then
                valuebar.targetMaxHealth = value["MaxMorale"]
                valuebar.TargetName = value["Name"]
            else
                valuebar.targetMaxHealth = 0
                valuebar.TargetName = ""
            end
        end))

        table.insert(self.healthTargetCallbacks, TargetHandlerObject:AddHealthCallback(function(value)
            if valuebar.targetMaxHealth > 0 then
                local percentage = value / valuebar.targetMaxHealth * 100
                local time = Turbine.Engine.GetGameTime()
                for key, value in pairs(data["Target_HealthThreshold"]) do
                    if key == valuebar.TargetName then
                        if self.compareFunctions[value["Operator"]](percentage, value["Number"]) then
                            local id = key .. tostring(value["Number"]) .. tostring(value["Operator"])
                            if valuebar.targetHealthThresholdCooldowns[id] < time and valuebar.targetHealthThresholdPercentage[id] then
                                valuebar.targetHealthThresholdPercentage[id] = false
                                valuebar.counterFunc(value.Text, not(value.Decrement), value.Number, value.Condition_Name, value.Time)
                                valuebar.targetHealthThresholdCooldowns[id] = time + 6
                            end
                        else
                            valuebar.targetHealthThresholdPercentage[key .. tostring(value["Number"]) .. tostring(value["Operator"])] = true
                        end
                    end
                end
            end
        end))

        table.insert(self.combatCallbacks, AddCallback(self.player, "InCombatChanged", function()
            if not(self.player:IsInCombat()) then
                for key, value in pairs(data["Target_HealthThreshold"]) do
                    valuebar.targetHealthThresholdCooldowns[key .. tostring(value["Number"]) .. tostring(value["Operator"])] = Turbine.Engine.GetGameTime() - 1000
                    valuebar.targetHealthThresholdPercentage[key .. tostring(value["Number"]) .. tostring(value["Operator"])] = true
                end 
            end
        end))
    end    

    table.insert(self.tools, valuebar)

    self.currentHeight = self.currentHeight + data["Height"] + 2 * data["BorderSize"]

end