function PrimeUI:EventKeyword(data, parent)
    local valuebar = Turbine.UI.Control()
    valuebar:SetParent(parent)
    self.lookup[data.NickName or ""] = valuebar
    valuebar.Active = true

    if tablelength(data["Start_Keyword"]) > 0 then
        valuebar.Active = false
        table.insert(self.eventTriggerCallbacks, AddCallback(self, "EventTrigger", 
        function(event)
            if data["Start_Keyword"][event] then
                valuebar.Active = true
            end
        end))
    end

    if tablelength(data["End_Keyword"]) > 0 then
        table.insert(self.eventTriggerCallbacks, AddCallback(self, "EventTrigger", 
        function(event)
            if data["End_Keyword"][event] then
                valuebar.Active = false
            end
        end))
    end

    if tablelength(data["EventTrigger"]) > 0 then
        table.insert(self.eventTriggerCallbacks, AddCallback(self, "EventTrigger", 
        function(event)
            if valuebar.Active then
                local value = data["EventTrigger"][event]
                if value then
                    for keyword in string.gmatch(value["Event"], "([^,]+)") do
                        self:FireTrigger(keyword, value.Args and {})
                    end
                end
                
            end
        end))
    end

    if tablelength(data["EffectAdded"]) > 0 then
        table.insert(self.effectAddedCallbacks, AddCallback(PlayerEffectsHandler, "EffectAdded", 
        function(effect)
            if valuebar.Active then
                local name = effect:GetName()
                local value = data["EffectAdded"][name]
                if value then
                    for keyword in string.gmatch(value["Event"], "([^,]+)") do
                        self:FireTrigger(keyword, value.Args and {
                            ["Duration"] = effect:GetDuration(),
                            ["Category"] = effect:GetCategory(),
                            ["Icon"] = effect:GetIcon(),
                            ["IsCurable"] = effect:IsCurable(),
                            ["IsDebuff"] = effect:IsDebuff(),
                            ["IsDispellable"] = effect.IsDispellable and effect:IsDispellable(),
                            ["StartTime"] = effect:GetStartTime(),
                            ["Description"] = effect:GetDescription(),
                            ["Effectname"] = effect:GetName(),
                        })
                    end
                end
            end
        end))
    end

    if tablelength(data["EffectAdded_Target"]) > 0 then
        table.insert(self.effectTargetAddedCallbacks, TargetHandlerObject:AddEffectCallback(
        function(args)
            if valuebar.Active then
                local name = args.Name
                local value = data["EffectAdded_Target"][name]
                if value then
                    for keyword in string.gmatch(value["Event"], "([^,]+)") do
                        self:FireTrigger(keyword, value.Args and {
                            ["Duration"] = args.Duration,
                            ["Category"] = args.Category,
                            ["Icon"] = args.Icon,
                            ["IsCurable"] = args.IsCurable,
                            ["IsDebuff"] = args.IsDebuff,
                            ["StartTime"] = args.StartTime,
                            ["Description"] = args.Description,
                            ["Effectname"] = args.Name,
                        })
                    end
                end
            end
        end))
    end

    if tablelength(data["EffectAdded_Group"]) > 0 then
        table.insert(self.effectAddedGroupCallbacks, AddCallback(GroupEffectsHandler, "EffectAdded", 
        function(args)
            if valuebar.Active then
                local name = args.effect.Name
                local value = data["EffectAdded_Group"][name]
                if value then
                    for keyword in string.gmatch(value["Event"], "([^,]+)") do
                        self:FireTrigger(keyword, value.Args and {
                            ["Duration"] = args.effect.Duration,
                            ["Category"] = args.effect.Category,
                            ["Icon"] = args.effect.Icon,
                            ["IsCurable"] = args.effect.IsCurable,
                            ["IsDebuff"] = args.effect.IsDebuff,
                            --["IsDispellable"] = args.effect.IsDispellable and args.effect:IsDispellable(),
                            ["StartTime"] = args.effect.StartTime,
                            ["Description"] = args.effect.Description,
                            ["Name"] = args.name,
                            ["Effectname"] = args.effect.Name,
                        })
                    end
                end
            end
        end))
    end

    if tablelength(data["EffectRemoved"]) > 0 then
        table.insert(self.effectRemovedCallbacks, AddCallback(PlayerEffectsHandler, "EffectRemoved", 
        function(effect)
            if valuebar.Active then
                local name = effect:GetName()
                local value = data["EffectRemoved"][name]
                if value then
                    for keyword in string.gmatch(value["Event"], "([^,]+)") do
                        self:FireTrigger(keyword, value.Args and {
                            ["Duration"] = effect:GetDuration(),
                            ["Category"] = effect:GetCategory(),
                            ["Icon"] = effect:GetIcon(),
                            ["IsCurable"] = effect:IsCurable(),
                            ["IsDebuff"] = effect:IsDebuff(),
                            ["IsDispellable"] = effect.IsDispellable and effect:IsDispellable(),
                            ["StartTime"] = effect:GetStartTime(),
                            ["Description"] = effect:GetDescription(),
                            ["Effectname"] = effect:GetName(),
                        })
                    end
                end
            end
        end))
    end

    if tablelength(data["EnemyHits"]) > 0 then
        table.insert(self.enemyChatCallback, AddCallback(ChatEventHandler, "EnemyCombat", function(args)
            if valuebar.Active then
                local value = data["EnemyHits"][args.SkillName]
                if value then
                    for keyword in string.gmatch(value["Event"], "([^,]+)") do
                        self:FireTrigger(keyword, value.Args and args)
                    end
                end
            end
        end))

    end

    if tablelength(data["PlayerHits"]) > 0 then
        table.insert(self.playerChatCallback, AddCallback(ChatEventHandler, "PlayerCombat", function(args)
            if valuebar.Active then
                local value = data["PlayerHits"][args.SkillName]
                if value then
                    for keyword in string.gmatch(value["Event"], "([^,]+)") do
                        self:FireTrigger(keyword, value.Args and args)
                    end
                end
            end
        end))
    end

    if tablelength(data["StandardChat"]) > 0 then
        table.insert(self.standardChatCallback, AddCallback(ChatEventHandler, "Standard", function(message)
            if valuebar.Active then
                local value = data["StandardChat"][message]
                if value then
                    for keyword in string.gmatch(value["Event"], "([^,]+)") do
                        self:FireTrigger(keyword)
                    end
                end
            end
        end))
    end

    if tablelength(data["PartyChat"]) > 0 then
        table.insert(self.partyInfoChatCallback, AddCallback(ChatEventHandler, "PartyInfo", function(args)
            if valuebar.Active then
                local value = data["PartyChat"][args.Message]
                if value then
                    for keyword in string.gmatch(value["Event"], "([^,]+)") do
                        self:FireTrigger(keyword, value.Args and args)
                    end
                end
            end
        end))
    end

    if tablelength(data["SayChat"]) > 0 then
        table.insert(self.sayInfoChatCallback, AddCallback(ChatEventHandler, "SayInfo", function(args)
            if valuebar.Active then
                local value = data["SayChat"][args.Message]
                if value then
                    for keyword in string.gmatch(value["Event"], "([^,]+)") do
                        self:FireTrigger(keyword, value.Args and args)
                    end
                end
            end
        end))
    end

    if tablelength(data["PlayerBenefit"]) > 0 then
        table.insert(self.playerBenefitCallbacks, AddCallback(ChatEventHandler, "PlayerBenefit", function(args)
            if valuebar.Active then
                local value = data["PlayerBenefit"][args.SkillName]
                if value then
                    for keyword in string.gmatch(value["Event"], "([^,]+)") do
                        self:FireTrigger(keyword, value.Args and args)
                    end
                end
            end
        end))
    end

    if tablelength(data["PlayerHeal"]) > 0 then
        table.insert(self.playerHealCallbacks, AddCallback(ChatEventHandler, "PlayerHeal", function(args)
            if valuebar.Active then
                local value = data["PlayerHeal"][args.SkillName]
                if value then
                    for keyword in string.gmatch(value["Event"], "([^,]+)") do
                        self:FireTrigger(keyword, value.Args and args)
                    end
                end
            end
        end))
    end

    if tablelength(data["Action"]) > 0 then
        table.insert(self.actionCallbacks, AddCallback(ActionHandlerObject, "Action", function(args)
            if valuebar.Active then
                local value = data["Action"][args.Action]
                if value then
                    for keyword in string.gmatch(value["Event"], "([^,]+)") do
                        self:FireTrigger(keyword, value.Args and args)
                    end
                end
            end
        end))
    end

    if tablelength(data["Quest Accepted"]) > 0 then
        table.insert(self.QuestAcceptedCallbacks, AddCallback(ChatEventHandler, "QuestAccepted", function(args)
            if valuebar.Active then
                local value = data["Quest Accepted"][args.Name]
                if value then
                    for keyword in string.gmatch(value["Event"], "([^,]+)") do
                        self:FireTrigger(keyword, value.Args and args)
                    end
                end
            end
        end))
    end

    if tablelength(data["Quest or Deed Completed"]) > 0 then
        table.insert(self.QuestOrDeedCompletedCallbacks, AddCallback(ChatEventHandler, "QuestOrDeedCompleted", function(args)
            if valuebar.Active then
                local value = data["Quest or Deed Completed"][args.Name]
                if value then
                    for keyword in string.gmatch(value["Event"], "([^,]+)") do
                        self:FireTrigger(keyword, value.Args and args)
                    end
                end
            end
        end))
    end

    if tablelength(data["Quest or Deed Progress"]) > 0 then
        table.insert(self.QuestOrDeedProgress, AddCallback(ChatEventHandler, "QuestOrDeedProgress", function(args)
            if valuebar.Active then
                local value = data["Quest or Deed Progress"][args.SkillName]
                if value then
                    for keyword in string.gmatch(value["Event"], "([^,]+)") do
                        self:FireTrigger(keyword, value.Args and args)
                    end
                end
            end
        end))
    end

    if tablelength(data["New Deed"]) > 0 then
        table.insert(self.NewDeedCallbacks, AddCallback(ChatEventHandler, "NewDeed", function(args)
            if valuebar.Active then
                local value = data["New Deed"][args.Name]
                if value then
                    for keyword in string.gmatch(value["Event"], "([^,]+)") do
                        self:FireTrigger(keyword, value.Args and args)
                    end
                end
            end
        end))
    end


    if tablelength(data["Skills"]) > 0 then
        self.skills = true
        for key, value in pairs(data["Skills"]) do
            if SkillEventHandler:SkillExists(key) then
                -- Add the callback for this skill slot
                if not(self.skillCallbacks[key]) then self.skillCallbacks[key] = {} end
                table.insert(self.skillCallbacks[key], AddCallback(SkillEventHandler, key, function(skill) 
                    if valuebar.Active then
                        local value = data["Skills"][key]
                        if value then
                            if type(value) == "table" then
                                if (skill:GetResetTime() - Turbine.Engine.GetGameTime()) + 1 <= skill:GetCooldown() then
                                    return
                                end
                                for keyword in string.gmatch(value["Event"], "([^,]+)") do
                                    self:FireTrigger(keyword, value.Args and {
                                        ["Cooldown"] = skill:GetCooldown(),
                                        ["GetBaseCooldown"] = skill:GetBaseCooldown(),
                                        ["GetResetTime"] = skill:GetResetTime(),
                                        ["GetDescription"] = skill:GetSkillInfo():GetDescription(),
                                        ["GetName"] = skill:GetSkillInfo():GetName(),
                                        ["GetType"] = skill:GetSkillInfo():GetType(),
                                        ["GetIconImageID"] = skill:GetSkillInfo():GetIconImageID(),
                                    })
                                end
                            else
                                if (skill:GetResetTime() - Turbine.Engine.GetGameTime()) + 1 <= skill:GetCooldown() then
                                    return
                                end
                                for keyword in string.gmatch(value["Event"], "([^,]+)") do
                                    self:FireTrigger(keyword, value.Args and {
                                        ["Cooldown"] = skill:GetCooldown(),
                                        ["GetBaseCooldown"] = skill:GetBaseCooldown(),
                                        ["GetResetTime"] = skill:GetResetTime(),
                                        ["GetDescription"] = skill:GetSkillInfo():GetDescription(),
                                        ["GetName"] = skill:GetSkillInfo():GetName(),
                                        ["GetType"] = skill:GetSkillInfo():GetType(),
                                        ["GetIconImageID"] = skill:GetSkillInfo():GetIconImageID(),
                                    })
                                end
                            end
                        end
                    end
                end))
            end
        end
    end

    if tablelength(data["Target_HealthThreshold"]) > 0 then
        valuebar.targetHealthThresholdCooldowns = {}
        valuebar.targetHealthThresholdPercentage = {}
        valuebar.targetMaxHealth = 0
        local health = self.player:GetMorale() / self.player:GetMaxMorale() * 100
        for _, value in pairs(data["Target_HealthThreshold"]) do
            valuebar.targetHealthThresholdCooldowns[value.Target_Name .. tostring(value["Number"]) .. tostring(value["Operator"])] = Turbine.Engine.GetGameTime() - 1000
            valuebar.targetHealthThresholdPercentage[value.Target_Name .. tostring(value["Number"]) .. tostring(value["Operator"])] = true
        end

        table.insert(self.targetInfoCallbacks, TargetHandlerObject:AddTargetInfoCallback(function(value) 
            if value then
                valuebar.targetMaxHealth = value["MaxMorale"]
                valuebar.TargetName = value["Name"]
            else
                valuebar.targetMaxHealth = -1
                valuebar.TargetName = ""
            end
        end))

        table.insert(self.healthTargetCallbacks, TargetHandlerObject:AddHealthCallback(function(value)
            if valuebar.targetMaxHealth > 0 then
                local percentage = value / valuebar.targetMaxHealth * 100
                local time = Turbine.Engine.GetGameTime()
                for _, tableValue in pairs(data["Target_HealthThreshold"]) do
                    if tableValue.Target_Name == valuebar.TargetName or tableValue.Target_Name == "" then
                        if self.compareFunctions[tableValue["Operator"]](percentage, tableValue["Number"]) then
                            local id = tableValue.Target_Name .. tostring(tableValue["Number"]) .. tostring(tableValue["Operator"])
                            valuebar.targetHealthThresholdPercentage[id] = false
                            if valuebar.Active then
                                for keyword in string.gmatch(tableValue["Event"], "([^,]+)") do
                                    self:FireTrigger(keyword)
                                end
                            end
                        else
                            valuebar.targetHealthThresholdPercentage[tableValue.Target_Name .. tostring(tableValue["Number"]) .. tostring(tableValue["Operator"])] = true
                        end
                    end
                end
            end
        end))
        table.insert(self.combatCallbacks, AddCallback(self.player, "InCombatChanged", function()
            if not(self.player:IsInCombat()) then
                for _, value in pairs(data["Target_HealthThreshold"]) do
                    valuebar.targetHealthThresholdCooldowns[value.Target_Name .. tostring(value["Number"]) .. tostring(value["Operator"])] = Turbine.Engine.GetGameTime() - 1000
                    valuebar.targetHealthThresholdPercentage[value.Target_Name .. tostring(value["Number"]) .. tostring(value["Operator"])] = true
                end 
            end
        end))
    end    

    if tablelength(data["FightStart"]) > 0 then
        table.insert(self.fightEventStartCallbacks, AddCallback(InstanceController, "FightEventStart", function(args)
            if valuebar.Active then
                for key, value in pairs(data["FightStart"]) do
                    if (value.Boss_Name == args.Boss_Name or value.Boss_Name == "All") and (value.Tier_Info == true or tostring(value.Tier_Info) == args.Boss_Tier) then
                        for keyword in string.gmatch(value["Event"], "([^,]+)") do
                            self:FireTrigger(keyword, value.Args and args)
                        end
                    end
                end
            end
        end))
    end

    if tablelength(data["FightPhase"]) > 0 then
        table.insert(self.fightEventPhaseCallback, AddCallback(InstanceController, "FightEventPhase", function(args)
            if valuebar.Active then
                for key, value in pairs(data["FightPhase"]) do
                    if (value.Boss_Name == args.Boss_Name or value.Boss_Name == "All") and (value.Tier_Info == true or tostring(value.Tier_Info) == args.Boss_Tier) and (value.Phase == args.Phase or value.Phase == nil) then
                        for keyword in string.gmatch(value["Event"], "([^,]+)") do
                            self:FireTrigger(keyword, value.Args and args)
                        end
                    end
                end
            end
        end))
    end

    if tablelength(data["FightEnd"]) > 0 then
        table.insert(self.fightEventEndCallbacks, AddCallback(InstanceController, "FightEventEnd", function(args)
            if valuebar.Active then
                for key, value in pairs(data["FightEnd"]) do
                    if (value.Boss_Name == args.Boss_Name or value.Boss_Name == "All") and (value.Tier_Info == true or tostring(value.Tier_Info) == args.Boss_Tier) then
                        for keyword in string.gmatch(value["Event"], "([^,]+)") do
                            self:FireTrigger(keyword, value.Args and args)
                        end
                    end
                end
            end
        end))
    end

    table.insert(self.UI, valuebar)
end