function PrimeUI:Figures(data, parent)
    local valuebar = PrimeUIFigure(data, parent, self:ColourTupleToTurbineColour(data["Colour"]))
    self.lookup[data.NickName or ""] = valuebar
    local x = self:ContentAlignement(data["Alignment"], (data["Width"]))
    local y = self.currentHeight
    if self.groupMode then
        x = data["X"]
        y = data["Y"]
    end

    if self.groupMode then
        valuebar.moveBox = MoveBox(valuebar, 50, 50, data["NickName"] or "Figures")
        valuebar.savePosition = function()
            data["X"], data["Y"] = valuebar:GetPosition()
        end
    end

    if tablelength(data["EventTrigger"]) > 0 then
        table.insert(self.eventTriggerCallbacks, AddCallback(self, "EventTrigger", 
        function(event)
            if data["EventTrigger"][event]  then
                Sounds:PlaySoundEffect(valuebar.sounds[data["EventTrigger"][event]])
            end
        end))
    end

    if tablelength(data["EffectAdded"]) > 0 then
        table.insert(self.effectAddedCallbacks, AddCallback(PlayerEffectsHandler, "EffectAdded", 
        function(effect)
            local name = effect:GetName()
            if data["EffectAdded"][name] then
                valuebar:Toggle()
            end
        end))
    end

    if tablelength(data["EffectAdded_Target"]) > 0 then
        table.insert(self.effectTargetAddedCallbacks, TargetHandlerObject:AddEffectCallback(
        function(args)
            local name = args.Name
            if data["EffectAdded_Target"][name] ~= nil then
                valuebar:Toggle()
            end
        end))
    end

    if tablelength(data["EffectAdded_Group"]) > 0 then
        table.insert(self.effectAddedGroupCallbacks, AddCallback(GroupEffectsHandler, "EffectAdded", 
        function(args)
            if data["EffectAdded_Group"][args.effect.Name] then
                valuebar:Toggle()
            end
        end))
    end

    if tablelength(data["EffectRemoved"]) > 0 then
        table.insert(self.effectRemovedCallbacks, AddCallback(PlayerEffectsHandler, "EffectRemoved", 
        function(effect)
            local name = effect:GetName()
            if data["EffectRemoved"][name] ~= nil then
                valuebar:Toggle()
            end
        end))
    end

    if tablelength(data["EnemyHits"]) > 0 then
        table.insert(self.enemyChatCallback, AddCallback(ChatEventHandler, "EnemyCombat", function(args)
            local value = data["EnemyHits"][args.SkillName]
            if value then
                valuebar:Toggle()
            end
        end))

    end

    if tablelength(data["StandardChat"]) > 0 then
        table.insert(self.standardChatCallback, AddCallback(ChatEventHandler, "Standard", function(message)
            if data["StandardChat"][message] then
                valuebar:Toggle()
            end
        end))
    end

    if tablelength(data["PartyChat"]) > 0 then
        table.insert(self.partyChatCallback, AddCallback(ChatEventHandler, "Party", function(message)
            if data["PartyChat"][message] then
                valuebar:Toggle()
            end
        end))
    end

    if tablelength(data["SayChat"]) > 0 then
        table.insert(self.sayChatCallback, AddCallback(ChatEventHandler, "SayLine", function(message)
            if data["SayChat"][message]  then
                valuebar:Toggle()
            end
        end))
    end

    if tablelength(data["PlayerHits"]) > 0 then
        table.insert(self.playerChatCallback, AddCallback(ChatEventHandler, "PlayerCombat", function(args)
            local value = data["PlayerHits"][args.SkillName]
            if value then
                valuebar:Toggle()
            end
        end))
    end

    if tablelength(data["PlayerBenefit"]) > 0 then
        table.insert(self.playerBenefitCallbacks, AddCallback(ChatEventHandler, "PlayerBenefit", function(args)
            local value = data["PlayerBenefit"][args.SkillName]
            if value then
                valuebar:Toggle()
            end
        end))
    end

    if tablelength(data["PlayerHeal"]) > 0 then
        table.insert(self.playerHealCallbacks, AddCallback(ChatEventHandler, "PlayerHeal", function(args)
            local value = data["PlayerHeal"][args.SkillName]
            if value then
                valuebar:Toggle()
            end
        end))
    end

    if tablelength(data["HealthThreshold"]) > 0 then
        local health = self.player:GetMorale() / self.player:GetMaxMorale() * 100
        for key, value in pairs(data["HealthThreshold"]) do
            self.healthThresholdCooldowns[tostring(key) .. tostring(value["Operator"])] = Turbine.Engine.GetGameTime() - 1000
            self.healthThresholdPercentage[tostring(key) .. tostring(value["Operator"])] = not(self.compareFunctions[value["Operator"]](health, key))
        end
        table.insert(self.playerHealthCallbacks, AddCallback(self.player, "MoraleChanged", function()
            local percentage = self.player:GetMorale() / self.player:GetMaxMorale() * 100
            local time = Turbine.Engine.GetGameTime()
            for key, value in pairs(data["HealthThreshold"]) do
                if self.compareFunctions[value["Operator"]](percentage, key) then
                    local id = tostring(key) .. tostring(value["Operator"])
                    if self.healthThresholdCooldowns[id] < time and self.healthThresholdPercentage[id] then
                        self.healthThresholdPercentage[id] = false
                        valuebar:Toggle()
                        self.healthThresholdCooldowns[id] = time + 6
                    end
                else
                    self.healthThresholdPercentage[tostring(key) .. tostring(value["Operator"])] = true
                end
                
            end
        end))
    end

    if tablelength(data["Target_HealthThreshold"]) > 0 then
        valuebar.targetHealthThresholdCooldowns = {}
        valuebar.targetHealthThresholdPercentage = {}
        valuebar.targetMaxHealth = 0
        local health = self.player:GetMorale() / self.player:GetMaxMorale() * 100
        for key, value in pairs(data["Target_HealthThreshold"]) do
            valuebar.targetHealthThresholdCooldowns[key .. tostring(value["Number"]) .. tostring(value["Operator"])] = Turbine.Engine.GetGameTime() - 1000
            valuebar.targetHealthThresholdPercentage[key .. tostring(value["Number"]) .. tostring(value["Operator"])] = true
        end

        table.insert(self.targetInfoCallbacks, TargetHandlerObject:AddTargetInfoCallback(function(value) 
            if value then
                valuebar.targetMaxHealth = value["MaxMorale"]
                valuebar.TargetName = value["Name"]
            else
                valuebar.targetMaxHealth = 0
                valuebar.TargetName = ""
            end
        end))

        table.insert(self.healthTargetCallbacks, TargetHandlerObject:AddHealthCallback(function(value)
            if valuebar.targetMaxHealth > 0 then
                local percentage = value / valuebar.targetMaxHealth * 100
                local time = Turbine.Engine.GetGameTime()
                for key, value in pairs(data["Target_HealthThreshold"]) do
                    if key == valuebar.TargetName then
                        if self.compareFunctions[value["Operator"]](percentage, value["Number"]) then
                            local id = key .. tostring(value["Number"]) .. tostring(value["Operator"])
                            if valuebar.targetHealthThresholdCooldowns[id] < time and valuebar.targetHealthThresholdPercentage[id] then
                                valuebar.targetHealthThresholdPercentage[id] = false
                                valuebar:Toggle()
                                valuebar.targetHealthThresholdCooldowns[id] = time + 6
                            end
                        else
                            valuebar.targetHealthThresholdPercentage[key .. tostring(value["Number"]) .. tostring(value["Operator"])] = true
                        end
                    end
                end
            end
        end))
        table.insert(self.combatCallbacks, AddCallback(self.player, "InCombatChanged", function()
            if not(self.player:IsInCombat()) then
                for key, value in pairs(data["Target_HealthThreshold"]) do
                    valuebar.targetHealthThresholdCooldowns[key .. tostring(value["Number"]) .. tostring(value["Operator"])] = Turbine.Engine.GetGameTime() - 1000
                    valuebar.targetHealthThresholdPercentage[key .. tostring(value["Number"]) .. tostring(value["Operator"])] = true
                end 
            end
        end))
    end

    if tablelength(data["Skills"]) > 0 then
        self.skills = true
        for key, value in pairs(data["Skills"]) do
            if SkillEventHandler:SkillExists(key) then
                -- Add the callback for this skill slot
                if not(self.skillCallbacks[key]) then self.skillCallbacks[key] = {} end
                table.insert(self.skillCallbacks[key], AddCallback(SkillEventHandler, key, function(skill) 
                    valuebar:Toggle()
                end))
            end
        end
    end

    table.insert(self.tools, valuebar)
end