function PrimeUI:Icon(data, parent)
    local valuebar = Turbine.UI.Window()
    valuebar:SetParent(parent)
    if self.groupMode then
        valuebar:SetPosition(data["X"], data["Y"])
        valuebar.moveBox = MoveBox(valuebar, (data["Width"] + 2 * data["BorderSize"]) * data["Scale"], (data["Height"] + 2 * data["BorderSize"]) * data["Scale"], data["NickName"] or "Icon")
        valuebar.savePosition = function()
            data["X"], data["Y"] = valuebar:GetPosition()
        end
    else
        valuebar:SetPosition(self:ContentAlignement(data["Alignment"], (data["Width"] + 2 * data["BorderSize"]) * data["Scale"]), self.currentHeight)
    end
    valuebar:SetSize((data["Width"] + 2 * data["BorderSize"]) * data["Scale"], (data["Height"] + 2 * data["BorderSize"]) * data["Scale"])
    --valuebar:SetSize(data["Width"] + 2 * data["BorderSize"], data["Height"] + 2 * data["BorderSize"])
    valuebar:SetMouseVisible(false)
    valuebar:SetBackColor(self:ColourTupleToTurbineColour(data["BorderColour"]))
    valuebar:SetVisible(data.Visible_On_Load or data.Visible_On_Load == nil)

    valuebar.innerbar = Turbine.UI.Control()
    valuebar.innerbar:SetParent(valuebar)
    valuebar.innerbar:SetPosition(data["BorderSize"] * data["Scale"], data["BorderSize"]  * data["Scale"])
    valuebar.innerbar:SetBackground(BackgroundHandler:GetBackground(tonumber(data["Icon"]) or data["Icon"]))
    valuebar.innerbar:SetSize(data["Width"], data["Height"])
    valuebar.innerbar:SetMouseVisible(false)
    valuebar.innerbar:SetBlendMode(Turbine.UI.BlendMode.AlphaBlend)

    self.lookup[data.NickName or ""] = valuebar.innerbar

    if data["Text"] and string.len(data["Text"]) > 0 then
        valuebar.textbar = Turbine.UI.Label()
        valuebar.textbar:SetParent(valuebar)
        valuebar.textbar:SetPosition(data["BorderSize"] * data["Scale"], data["BorderSize"]  * data["Scale"])
        valuebar.textbar:SetSize(data["Width"], data["Height"])
        valuebar.textbar:SetMouseVisible(false)
        valuebar.textbar:SetBlendMode(Turbine.UI.BlendMode.AlphaBlend)
        valuebar.textbar:SetFont(self:FontToFont(data["Font"]))
        valuebar.textbar:SetForeColor(self:ColourTupleToTurbineColour(data["Font_Colour"]))
        valuebar.textbar:SetTextAlignment(self.alignments[data["TextAlignment"]])
        valuebar.textbar:SetText(data["Text"])
    end

    if data["Visibilty_Event"] and data["Invisibility_Event"] then
        table.insert(self.eventTriggerCallbacks, AddCallback(self, "EventTrigger", 
        function(event)
            if data["Visibilty_Event"][event] then
                valuebar.innerbar:SetVisible(true)
            end
            
            if data["Invisibility_Event"][event] then
                valuebar.innerbar:SetVisible(false)
            end
        end))
    end

    valuebar.innerbar:SetStretchMode(1)
    valuebar.innerbar:SetSize((data["Width"]) * data["Scale"], (data["Height"]) * data["Scale"])

    valuebar.statusTable = {}
    for key, value in pairs(data["Effects"]) do
        if value == false then
            valuebar.statusTable[key] = 1
        else
            valuebar.statusTable[key] = 0
        end
    end

    valuebar.togglefunction = function()
        local tableValue = 0
        for key, value in pairs(valuebar.statusTable) do
            if value > 0 then
                tableValue = tableValue + 1
            end
        end
        local visibilityValue = tableValue >= valuebar.minCount
        valuebar:SetVisible(visibilityValue)
        if valuebar.innerbar.highlight then valuebar.innerbar.highlight:Toggle(visibilityValue) end
    end

    valuebar.count = 0
    if data["Conjuction"] then
        valuebar.minCount = tablelength(data["Effects"]) + tablelength(data["HealthThreshold"])
    else
        valuebar.minCount = 1
    end

    if data["Highlight"] then
        valuebar.innerbar.highlight = ClassSpecificTools.HighlightFrame2(valuebar.innerbar, data["Scale"])
        valuebar.innerbar.highlight:Toggle(true) 
    end

    if data["AlwaysVisible"] then
        valuebar:SetVisible(true)
        if valuebar.innerbar.highlight then valuebar.innerbar.highlight:Toggle(true) end
    else
        for i = 1, self.player:GetEffects():GetCount(), 1 do
            local effect = self.player:GetEffects():Get(i)
            if data["Effects"][effect:GetName()] == true then
                valuebar.statusTable[effect:GetName()] = valuebar.statusTable[effect:GetName()] + 1
            elseif data["Effects"][effect:GetName()] == false then
                valuebar.statusTable[effect:GetName()] = valuebar.statusTable[effect:GetName()] - 1
            end
        end


    
        if tablelength(data["Effects"]) then
            table.insert(self.effectAddedCallbacks, AddCallback(PlayerEffectsHandler, "EffectAdded", 
            function(effect)
                local name = effect:GetName()
                if data["Effects"][name] ~= nil then
                    local value = data["Effects"][name]
                    if value == true then
                        valuebar.statusTable[name] = valuebar.statusTable[name] + 1
                    else
                        valuebar.statusTable[name] = valuebar.statusTable[name] - 1
                    end
                    valuebar.togglefunction()
                end
            end))
    
            table.insert(self.effectRemovedCallbacks, AddCallback(PlayerEffectsHandler, "EffectRemoved", 
            function(effect)
                local name = effect:GetName()
                if data["Effects"][name] ~= nil then
                    local value = data["Effects"][effect:GetName()]
                    if value == true then
                        valuebar.statusTable[name] = valuebar.statusTable[name] - 1
                    else
                        valuebar.statusTable[name] = valuebar.statusTable[name] + 1
                    end

                    valuebar.togglefunction()
                end
            end))
        end

        if tablelength(data["HealthThreshold"]) > 0 then
            local health = self.player:GetMorale() / self.player:GetMaxMorale() * 100
            valuebar.healthThresholdPercentage = {}
            for key, value in pairs(data["HealthThreshold"]) do
                local checkValue = (self.compareFunctions[value["Operator"]](health, key) and 1) or 0
                valuebar.statusTable[tostring(key) .. tostring(value["Operator"])] = checkValue
                valuebar.healthThresholdPercentage[tostring(key) .. tostring(value["Operator"])] = checkValue
            end
            table.insert(self.playerHealthCallbacks, AddCallback(self.player, "MoraleChanged", function()
                local percentage = self.player:GetMorale() / self.player:GetMaxMorale() * 100
                for key, value in pairs(data["HealthThreshold"]) do
                    local id = tostring(key) .. tostring(value["Operator"])
                    if self.compareFunctions[value["Operator"]](percentage, key) then
                        if valuebar.healthThresholdPercentage[id] == 0 then
                            if valuebar.healthThresholdPercentage[id] == 0 then
                                valuebar.healthThresholdPercentage[id] = 1
                                valuebar.statusTable[tostring(key) .. tostring(value["Operator"])] = valuebar.statusTable[tostring(key) .. tostring(value["Operator"])] + 1
                            end
                        end
                    else
                        if valuebar.healthThresholdPercentage[id] == 1 then
                            valuebar.healthThresholdPercentage[id] = 0
                            valuebar.statusTable[tostring(key) .. tostring(value["Operator"])] = valuebar.statusTable[tostring(key) .. tostring(value["Operator"])] - 1
                        end
                    end
                end
                valuebar.togglefunction()        
            end))
        end

        valuebar.togglefunction()
    end

    self.currentHeight = self.currentHeight +  ((data["Height"] + 2 * data["BorderSize"]) * data["Scale"])

    if valuebar.textbar then table.insert(self.UI, valuebar.textLabel) end
    table.insert(self.UI, valuebar.innerbar)
    table.insert(self.UI, valuebar)
end