function PrimeUI:SetupMoraleBar(data, parent)
    local valuebar = Turbine.UI.Window()
    valuebar:SetParent(parent)
    if self.groupMode then
        valuebar:SetPosition(data["X"], data["Y"])
        valuebar.moveBox = MoveBox(valuebar, data["Width"] + 2 * data["BorderSize"], data["Height"] + 2 * data["BorderSize"], data["NickName"] or  "Morale Bar")
        valuebar.savePosition = function()
            data["X"], data["Y"] = valuebar:GetPosition()
        end
    else
        valuebar:SetPosition(self:ContentAlignement(data["Alignment"], data["Width"] + 2 * data["BorderSize"]), self.currentHeight)
    end
    valuebar:SetSize(data["Width"] + 2 * data["BorderSize"], data["Height"] + 2 * data["BorderSize"])
    valuebar:SetMouseVisible(false)
    valuebar:SetVisible(true)
    valuebar:SetBackColor(self:ColourTupleToTurbineColour(data["BorderColour"]))

    -- args (parent, x, y, width, height, textFormat, textAlignment, barAlignment, barColours, barbackground)
    local moralebar = ClassSpecificTools.PermanentBar(valuebar, data["BorderSize"], data["BorderSize"], data["Width"], data["Height"], data["Text"], Turbine.UI.ContentAlignment[data.TextAlignment], data["Alignment"], self:ColourTupleToTurbineColour(data["Colours"]), "bar25", self:FontToFont(data["Font"]), self:ColourTupleToTurbineColour(data["Interval_Colour"]), data["Intervals"])
    moralebar:SetMaxValue(self.player:GetMaxMorale())
    moralebar:SetCurrentValue(self.player:GetMorale())
    moralebar:SetTempValue(self.player:GetTemporaryMorale())
    self.lookup[data.NickName or ""] = moralebar
    if data.Disable_Player_Vitals then
        Turbine.UI.Lotro.LotroUI.SetEnabled(Turbine.UI.Lotro.LotroUIElement.Vitals, false)
    else
        Turbine.UI.Lotro.LotroUI.SetEnabled(Turbine.UI.Lotro.LotroUIElement.Vitals, true)
    end

    valuebar.entityControl = Turbine.UI.Lotro.EntityControl()
    valuebar.entityControl:SetParent(valuebar)
    valuebar.entityControl:SetSize(data["Width"] + 2 * data["BorderSize"], data["Height"] + 2 * data["BorderSize"])
    valuebar.entityControl:SetEntity(self.player)

    if data["Smooth"] then
        table.insert(self.playerMoraleHandlerCallbacks, AddCallback(PlayerHandlerObject, "MoraleChanged", function(args) 
            moralebar:SetMaxValue(args.Player_Max_Morale)
            moralebar:SetCurrentValue(args.Player_Morale) 
        end))
    else
        table.insert(self.playerMoraleHandlerCallbacks, AddCallback(PlayerHandlerObject, "MoraleChanged", function(args) 
            moralebar:SetMaxValue(args.Player_Max_Morale)
            moralebar:SetCurrentValueImmediate(args.Player_Morale) 
        end))
    end
    --table.insert(self.playerMaxMoraleHandlerCallbacks, AddCallback(PlayerHandlerObject, "MaxMoraleChanged", function(args) moralebar:SetMaxValue(args.Player_Max_Morale) end))
    table.insert(self.playerTemporaryMoraleHandlerCallbacks, AddCallback(PlayerHandlerObject, "TemporaryMoraleChanged", function(args) moralebar:SetTempValue(args.Player_Temporary_Morale) end))

    moralebar:SetCurrentValue(self.player:GetMorale())
    moralebar:SetMaxValue(self.player:GetMaxMorale())

    if tablelength(data["Highlight_On_Effect"]) > 0 then
        valuebar.effectHighlightCount = 0
        valuebar.effectIDs = {}
        valuebar.highlightFrame = ClassSpecificTools.HighlightFrame3(valuebar, 0, 0, data["Width"] + 2 * data["BorderSize"],  data["Height"] + 2 * data["BorderSize"], self:ColourTupleToTurbineColour(data.Highlight_Colour), 2)

        valuebar.effectFunction = function(effect)
            if data["Highlight_On_Effect"][effect:GetName()] then
                valuebar.effectHighlightCount = valuebar.effectHighlightCount + 1
                valuebar.effectIDs[effect:GetID()] = true
                if  valuebar.effectHighlightCount == 1 then
                    valuebar.highlightFrame:ToggleHighlight(true)
                end
            end
        end


        for i = 1, self.player:GetEffects():GetCount() do
            valuebar.effectFunction(self.player:GetEffects():Get(i))
        end

        table.insert(self.effectAddedCallbacks, AddCallback(PlayerEffectsHandler, "EffectAdded", valuebar.effectFunction))

        table.insert(self.effectRemovedCallbacks, AddCallback(PlayerEffectsHandler, "EffectRemoved", 
        function(effect)
            local ID = effect:GetID()
            if valuebar.effectIDs[ID] then
                valuebar.effectHighlightCount = valuebar.effectHighlightCount - 1
                if  valuebar.effectHighlightCount == 0 then
                    valuebar.highlightFrame:ToggleHighlight(false)
                end
                valuebar.effectIDs[ID] = nil
            end

        end))
    end

    table.insert(self.UI, valuebar)
    table.insert(self.tools, moralebar)

    self.currentHeight = self.currentHeight + data["Height"] + 2 * data["BorderSize"]
end