function PrimeUI:NoteActivation(data, parent)
    local valuebar = Turbine.UI.Control()
    valuebar:SetParent(parent)
    valuebar:SetVisible(false)

    if tablelength(data["EffectAdded"]) > 0 then
        table.insert(self.effectAddedCallbacks, AddCallback(PlayerEffectsHandler, "EffectAdded", 
        function(effect)
            local name = effect:GetName()
            if data["EffectAdded"][name] ~= nil then
                NoteHandler:SetNoteActive(data["EffectAdded"][name])
            end
        end))
    end

    if tablelength(data["EffectAdded_Target"]) > 0 then
        table.insert(self.effectTargetAddedCallbacks, TargetHandlerObject:AddEffectCallback(
        function(args)
            local name = args.Name
            if data["EffectAdded_Target"][name] ~= nil then
                NoteHandler:SetNoteActive(data["EffectAdded_Target"][name])
            end
        end))
    end

    if tablelength(data["EffectAdded_Group"]) > 0 then
        table.insert(self.effectAddedGroupCallbacks, AddCallback(GroupEffectsHandler, "EffectAdded", 
        function(args)
            local name = args.effect.Name
            if data["EffectAdded_Group"][name] ~= nil then
                NoteHandler:SetNoteActive(data["EffectAdded_Group"][name])
            end
        end))
    end

    if tablelength(data["EffectRemoved"]) > 0 then
        table.insert(self.effectRemovedCallbacks, AddCallback(PlayerEffectsHandler, "EffectRemoved", 
        function(effect)
            local name = effect:GetName()
            if data["EffectRemoved"][name] ~= nil then
                NoteHandler:SetNoteActive(data["EffectRemoved"][name])
            end
        end))
    end

    if tablelength(data["EnemyHits"]) > 0 then
        table.insert(self.enemyChatCallback, AddCallback(ChatEventHandler, "EnemyCombat", function(args)
            local value = data["EnemyHits"][args.SkillName]
            if value then
                NoteHandler:SetNoteActive(value)
            end
        end))

    end

    if tablelength(data["StandardChat"]) > 0 then
        table.insert(self.standardChatCallback, AddCallback(ChatEventHandler, "Standard", function(message)
            if data["StandardChat"][message] then
                NoteHandler:SetNoteActive(data["StandardChat"][message])
            end
        end))
    end

    if tablelength(data["PartyChat"]) > 0 then
        table.insert(self.partyChatCallback, AddCallback(ChatEventHandler, "Party", function(message)
            if data["PartyChat"][message] then
                NoteHandler:SetNoteActive(data["PartyChat"][message])
            end
        end))
    end

    if tablelength(data["SayChat"]) > 0 then
        table.insert(self.sayChatCallback, AddCallback(ChatEventHandler, "SayLine", function(message)
            if data["SayChat"][message]  then
                NoteHandler:SetNoteActive(data["SayChat"][message])
            end
        end))
    end

    if tablelength(data["PlayerHits"]) > 0 then
        table.insert(self.playerChatCallback, AddCallback(ChatEventHandler, "PlayerCombat", function(args)
            local value = data["PlayerHits"][args.SkillName]
            if value then
                NoteHandler:SetNoteActive(value)
            end
        end))
    end

    if tablelength(data["PlayerBenefit"]) > 0 then
        table.insert(self.playerBenefitCallbacks, AddCallback(ChatEventHandler, "PlayerBenefit", function(args)
            local value = data["PlayerBenefit"][args.SkillName]
            if value then
                NoteHandler:SetNoteActive(value)
            end
        end))
    end

    if tablelength(data["PlayerHeal"]) > 0 then
        table.insert(self.playerHealCallbacks, AddCallback(ChatEventHandler, "PlayerHeal", function(args)
            local value = data["PlayerHeal"][args.SkillName]
            if value then
                NoteHandler:SetNoteActive(value)
            end
        end))
    end

    if tablelength(data["DefeatEvent"]) > 0 then
        table.insert(self.defeatEventCallback, AddCallback(ChatEventHandler, "Defeated", function(args)
            if data["DefeatEvent"][args.Name] then
                NoteHandler:SetNoteActive(data["DefeatEvent"][args.Name])
            end
        end))
    end

    if tablelength(data["FightStart"]) > 0 then
        table.insert(self.fightEventStartCallbacks, AddCallback(InstanceController, "FightEventStart", function(args)
            for key, value in pairs(data["FightStart"]) do
                if value.Boss_Name == args.Boss_Name and (value.Tier_Info == true or  tostring(value.Tier_Info) == args.Boss_Tier) then
                    NoteHandler:SetNoteActive(value.Note_Name)
                end
            end
        end))
    end

    if tablelength(data["FightPhase"]) > 0 then
        table.insert(self.fightEventPhaseCallback, AddCallback(InstanceController, "FightEventPhase", function(args)
            for key, value in pairs(data["FightPhase"]) do
                if value.Boss_Name == args.Boss_Name and (value.Tier_Info == true or tostring(value.Tier_Info) == args.Boss_Tier) and value.Phase == args.Phase then
                    NoteHandler:SetNoteActive(value.Note_Name)
                end
            end
        end))
    end

    if tablelength(data["FightEnd"]) > 0 then
        table.insert(self.fightEventEndCallbacks, AddCallback(InstanceController, "FightEventEnd", function(args)
            for key, value in pairs(data["FightEnd"]) do
                if value.Boss_Name == args.Boss_Name and (value.Tier_Info == true or  tostring(value.Tier_Info) == args.Boss_Tier) then
                    NoteHandler:SetNoteActive(value.Note_Name)
                end
            end
        end))
    end

    if tablelength(data["HealthThreshold"]) > 0 then
        local health = self.player:GetMorale() / self.player:GetMaxMorale() * 100
        for key, value in pairs(data["HealthThreshold"]) do
            self.healthThresholdCooldowns[tostring(key) .. tostring(value["Operator"])] = Turbine.Engine.GetGameTime() - 1000
            self.healthThresholdPercentage[tostring(key) .. tostring(value["Operator"])] = not(self.compareFunctions[value["Operator"]](health, key))
        end
        table.insert(self.playerHealthCallbacks, AddCallback(self.player, "MoraleChanged", function()
            local percentage = self.player:GetMorale() / self.player:GetMaxMorale() * 100
            local time = Turbine.Engine.GetGameTime()
            for key, value in pairs(data["HealthThreshold"]) do
                if self.compareFunctions[value["Operator"]](percentage, key) then
                    local id = tostring(key) .. tostring(value["Operator"])
                    if self.healthThresholdCooldowns[id] < time and self.healthThresholdPercentage[id] then
                        self.healthThresholdPercentage[id] = false
                        NoteHandler:SetNoteActive(value["Event"])
                        self.healthThresholdCooldowns[id] = time + 6
                    end
                else
                    self.healthThresholdPercentage[tostring(key) .. tostring(value["Operator"])] = true
                end
                
            end
        end))
    end

    if tablelength(data["Target_HealthThreshold"]) > 0 then
        valuebar.targetHealthThresholdCooldowns = {}
        valuebar.targetHealthThresholdPercentage = {}
        valuebar.targetMaxHealth = 0
        local health = self.player:GetMorale() / self.player:GetMaxMorale() * 100
        for key, value in pairs(data["Target_HealthThreshold"]) do
            valuebar.targetHealthThresholdCooldowns[key .. tostring(value["Number"]) .. tostring(value["Operator"])] = Turbine.Engine.GetGameTime() - 1000
            valuebar.targetHealthThresholdPercentage[key .. tostring(value["Number"]) .. tostring(value["Operator"])] = true
        end

        table.insert(self.targetInfoCallbacks, TargetHandlerObject:AddTargetInfoCallback(function(value) 
            if value then
                valuebar.targetMaxHealth = value["MaxMorale"]
                valuebar.TargetName = value["Name"]
            else
                valuebar.targetMaxHealth = 0
                valuebar.TargetName = ""
            end
        end))

        table.insert(self.healthTargetCallbacks, TargetHandlerObject:AddHealthCallback(function(value)
            if valuebar.targetMaxHealth > 0 then
                local percentage = value / valuebar.targetMaxHealth * 100
                local time = Turbine.Engine.GetGameTime()
                for key, value in pairs(data["Target_HealthThreshold"]) do
                    if key == valuebar.TargetName then
                        if self.compareFunctions[value["Operator"]](percentage, value["Number"]) then
                            local id = key .. tostring(value["Number"]) .. tostring(value["Operator"])
                            if valuebar.targetHealthThresholdCooldowns[id] < time and valuebar.targetHealthThresholdPercentage[id] then
                                valuebar.targetHealthThresholdPercentage[id] = false
                                NoteHandler:SetNoteActiveActive(value["Note_Name"])
                                valuebar.targetHealthThresholdCooldowns[id] = time + 6
                            end
                        else
                            valuebar.targetHealthThresholdPercentage[key .. tostring(value["Number"]) .. tostring(value["Operator"])] = true
                        end
                    end
                end
            end
        end))
        table.insert(self.combatCallbacks, AddCallback(self.player, "InCombatChanged", function()
            if not(self.player:IsInCombat()) then
                for key, value in pairs(data["Target_HealthThreshold"]) do
                    valuebar.targetHealthThresholdCooldowns[key .. tostring(value["Number"]) .. tostring(value["Operator"])] = Turbine.Engine.GetGameTime() - 1000
                    valuebar.targetHealthThresholdPercentage[key .. tostring(value["Number"]) .. tostring(value["Operator"])] = true
                end 
            end
        end))
    end

    if tablelength(data["Skills"]) > 0 then
        self.skills = true
        for key, value in pairs(data["Skills"]) do
            if SkillEventHandler:SkillExists(key) then
                -- Add the callback for this skill slot
                if not(self.skillCallbacks[key]) then self.skillCallbacks[key] = {} end
                table.insert(self.skillCallbacks[key], AddCallback(SkillEventHandler, key, function(skill) 
                    NoteHandler:SetNoteActive(data["Skills"][key])
                end))
            end
        end
    end

    table.insert(self.UI, valuebar)
end