function PrimeUI:SetupPetMoraleBar(data, parent)
    local valuebar = Turbine.UI.Window()
    valuebar:SetParent(parent)
    if self.groupMode then
        valuebar:SetPosition(data["X"], data["Y"])
        valuebar.moveBox = MoveBox(valuebar, data["Width"] + 2 * data["BorderSize"], data["Height"] + 2 * data["BorderSize"], data["NickName"] or  "Pet Morale Bar")
        valuebar.savePosition = function()
            data["X"], data["Y"] = valuebar:GetPosition()
        end
    else
        valuebar:SetPosition(self:ContentAlignement(data["Alignment"], data["Width"] + 2 * data["BorderSize"]), self.currentHeight)
    end
    valuebar:SetSize(data["Width"] + 2 * data["BorderSize"], data["Height"] + 2 * data["BorderSize"])
    valuebar:SetMouseVisible(false)
    valuebar:SetVisible(true)
    valuebar:SetBackColor(self:ColourTupleToTurbineColour(data["BorderColour"]))

    -- args (parent, x, y, width, height, textFormat, textAlignment, barAlignment, barColours, barbackground)
    local moralebar = ClassSpecificTools.PermanentBar(valuebar, data["BorderSize"], data["BorderSize"], data["Width"], data["Height"], data["Text"], Turbine.UI.ContentAlignment[data.TextAlignment], data["Alignment"], self:ColourTupleToTurbineColour(data["Colours"]), "bar25", self:FontToFont(data["Font"]), self:ColourTupleToTurbineColour(data["Interval_Colour"]), data["Intervals"])
    self.lookup[data.NickName or ""] = moralebar
    valuebar.entityControl = Turbine.UI.Lotro.EntityControl()
    valuebar.entityControl:SetParent(valuebar)
    valuebar.entityControl:SetSize(data["Width"] + 2 * data["BorderSize"], data["Height"] + 2 * data["BorderSize"])
    valuebar.entityControl:SetEntity(self.player:GetPet())

    if data["Smooth"] then
        table.insert(self.petMoraleCallbacks, AddCallback(PlayerHandlerObject, "PetMoraleChanged", function(args) 
            moralebar:SetCurrentValue(args.Pet_Morale) 
            moralebar:SetMaxValue(args.Pet_Max_Morale) 
        end))
    else
        table.insert(self.petMoraleCallbacks, AddCallback(PlayerHandlerObject, "PetMoraleChanged", function(args) 
            moralebar:SetCurrentValueImmediate(args.Pet_Morale) 
            moralebar:SetMaxValue(args.Pet_Max_Morale) 
        end))
    end
    table.insert(self.petMoraleCallbacks, AddCallback(PlayerHandlerObject, "PetMaxMoraleChanged", function(args) 
        moralebar:SetCurrentValueImmediate(args.Pet_Morale) 
        moralebar:SetMaxValue(args.Pet_Max_Morale) 
    end))

    table.insert(self.petTemporaryMoraleCallbacks, AddCallback(PlayerHandlerObject, "PetTemporaryMoraleChanged", function(args) 
        moralebar:SetTempValue(args.Pet_Temporary_Morale) 
    end))

    table.insert(self.petInfoCallbacks, AddCallback(PlayerHandlerObject, "PetInfo", function(args)
        valuebar:SetVisible(args.PetStatus)
        if args.PetStatus then
            valuebar.entityControl:SetEntity(self.player:GetPet())
        end
    end))

    local petInfo = PlayerHandlerObject:GetPetInfo()
    if petInfo and petInfo.PetStatus then
        moralebar:SetCurrentValue(petInfo.Pet_Morale) 
        moralebar:SetMaxValue(petInfo.Pet_Max_Morale) 
        moralebar:SetTempValue(petInfo.Pet_Temporary_Morale) 
    else
        valuebar:SetVisible(false)
    end

    table.insert(self.UI, valuebar)
    table.insert(self.tools, moralebar)

    self.currentHeight = self.currentHeight + data["Height"] + 2 * data["BorderSize"]
end