function PrimeUI:ResourceBar(data, parent)
    local boxes = 0
    if self.playerClass == Turbine.Gameplay.Class.Champion then
        boxes = 5
    elseif self.playerClass == Turbine.Gameplay.Class.Hunter then
        boxes = 9
    elseif self.playerClass == Turbine.Gameplay.Class.RuneKeeper then
        if data["RKLayout"] then
            boxes = 9
        else
            boxes = 20
        end
    elseif self.playerClass == Turbine.Gameplay.Class.Brawler then
        boxes = 9
    elseif self.playerClass == Turbine.Gameplay.Class.Guardian then
        boxes = 5
    elseif self.playerClass == Turbine.Gameplay.Class.Beorning then
        boxes = 5
    elseif self.playerClass == Turbine.Gameplay.Class.BlackArrow then
        boxes = 5
    elseif self.playerClass == Turbine.Gameplay.Class.Weaver then
        boxes = 5
    elseif self.playerClass == Turbine.Gameplay.Class.Warden then
        boxes = 3
    end

    local x = self:ContentAlignement(data["Alignment"], data["Width"] + data["BorderSize"] * 2)
    local y = self.currentHeight
    if self.groupMode then
        x = data.X
        y = data.Y
    end

    local resourceBar = ClassSpecificTools.ResourceBar(parent, data["Width"], data["Height"], x, y, boxes, 
    self:ColourTupleToTurbineColour(data["ForegroundColour"]), self:ColourTupleToTurbineColour(data["BackgroundColour"]), data["ShowText"],
                                data["BoxOffset"], data["BorderSize"], self:ColourTupleToTurbineColour(data["BorderColour"]), self:ColourTupleToTurbineColour(data["BoxBackgroundColour"]), data["Full_Border"], data["Animate"],
                                self:ColourTupleToTurbineColour(data["Font_Colour"]), self:FontToFont(data["Font"]), data["Change_On_Full"], self:ColourTupleToTurbineColour(data["Filled_Colour"]))
    self.lookup[data.NickName or ""] = resourceBar
    if self.groupMode then
        resourceBar.moveBox = MoveBox(resourceBar, data["Width"], data["Height"], data["NickName"] or "Resourcebar")
        resourceBar.savePosition = function()
            data["X"], data["Y"] = resourceBar:GetPosition()
        end
    end
    
    if self.playerClass == Turbine.Gameplay.Class.Champion then
        resourceBar:ChangeResource(self.ClassAttr:GetFervor())
        if not(self.attrCallbacks["FervorChanged"] ) then self.attrCallbacks["FervorChanged"] = {} end
        table.insert(self.attrCallbacks["FervorChanged"], AddCallback(self.ClassAttr, "FervorChanged", function(sender, args) resourceBar:ChangeResource( self.ClassAttr:GetFervor() ) end))
    elseif self.playerClass == Turbine.Gameplay.Class.Hunter then
        resourceBar:ChangeResource(self.ClassAttr:GetFocus())
        if not(self.attrCallbacks["FocusChanged"] ) then self.attrCallbacks["FocusChanged"] = {} end
        table.insert(self.attrCallbacks["FocusChanged"], AddCallback(self.ClassAttr, "FocusChanged", function(sender, args) resourceBar:ChangeResource( self.ClassAttr:GetFocus() ) end))
    elseif self.playerClass == Turbine.Gameplay.Class.RuneKeeper then
        if not(self.attrCallbacks["AttunementChanged"] ) then self.attrCallbacks["AttunementChanged"] = {} end
        if data["RKLayout"] then
            local Attunement = self.ClassAttr:GetAttunement()
            if Attunement > 10 then
                resourceBar:ChangeRKResource(math.abs( Attunement - 10), self:ColourTupleToTurbineColour(data["RK_BackgroundColour_Heal"]), self:ColourTupleToTurbineColour(data["RK_ForegroundColour_Heal"]))
            else
                resourceBar:ChangeRKResource(math.abs( Attunement - 10), self:ColourTupleToTurbineColour(data["RK_BackgroundColour_DPS"]), self:ColourTupleToTurbineColour(data["RK_ForegroundColour_DPS"]))
            end
            table.insert(self.attrCallbacks["AttunementChanged"], AddCallback(self.ClassAttr, "AttunementChanged", function(sender, args)
                local Attunement = self.ClassAttr:GetAttunement()
                if Attunement > 10 then
                    resourceBar:ChangeRKResource(math.abs( Attunement - 10), self:ColourTupleToTurbineColour(data["RK_BackgroundColour_Heal"]), self:ColourTupleToTurbineColour(data["RK_ForegroundColour_Heal"]))
                else
                    resourceBar:ChangeRKResource(math.abs( Attunement - 10), self:ColourTupleToTurbineColour(data["RK_BackgroundColour_DPS"]), self:ColourTupleToTurbineColour(data["RK_ForegroundColour_DPS"]))
                end         
            end))
            
        else
            resourceBar:ChangeResource(self.ClassAttr:GetAttunement())
            table.insert(self.attrCallbacks["AttunementChanged"], AddCallback(self.ClassAttr, "AttunementChanged", function(sender, args) resourceBar:ChangeResource( self.ClassAttr:GetAttunement() ) end))
        end
    elseif self.playerClass == Turbine.Gameplay.Class.Brawler then
        for i = 1, self.player:GetEffects():GetCount(), 1 do
            local effect = self.player:GetEffects():Get(i)
            if string.gsub(effect:GetName(), "%d", "") == "Battle Flow " then
                resourceBar:ChangeResource( tonumber(string.gsub(effect:GetName(), "%D", "")) )
            end
        end

        table.insert(self.effectAddedCallbacks, AddCallback(PlayerEffectsHandler, "EffectAdded", 
        function(effect)
            if string.gsub(effect:GetName(), "%d", "") == "Battle Flow " then
                resourceBar:ChangeResource( tonumber(string.gsub(effect:GetName(), "%D", "")) )
                self.battleFlowID = effect:GetID()
            end
        end))

        table.insert(self.effectRemovedCallbacks, AddCallback(PlayerEffectsHandler, "EffectRemoved", 
        function(effect)
            if self.battleFlowID == effect:GetID() then
                resourceBar:ChangeResource(0)
            end
        end))

    elseif self.playerClass == Turbine.Gameplay.Class.Warden then
        resourceBar.resourceTable = {
            ["Advanced Technique I"] = 1,
            ["Advanced Technique II"] = 2,
            ["Advanced Technique III"] = 3,
            ["Advanced Technique IV"] = 4,
            ["Advanced Technique V"] = 5,
        }
        for i = 1, self.player:GetEffects():GetCount(), 1 do
            local effect = self.player:GetEffects():Get(i)
            if resourceBar.resourceTable[effect:GetName()] then
                resourceBar:ChangeResource(resourceBar.resourceTable[effect:GetName()])
            end
        end

        table.insert(self.effectAddedCallbacks, AddCallback(PlayerEffectsHandler, "EffectAdded", 
        function(effect)
            if resourceBar.resourceTable[effect:GetName()] then
                resourceBar:ChangeResource( resourceBar.resourceTable[effect:GetName()] )
                resourceBar.ATID = effect:GetID()
            end
        end))

        table.insert(self.effectRemovedCallbacks, AddCallback(PlayerEffectsHandler, "EffectRemoved", 
        function(effect)
            if resourceBar.ATID == effect:GetID() then
                resourceBar:ChangeResource(0)
            end
        end))

    elseif self.playerClass == Turbine.Gameplay.Class.Guardian then
        table.insert(self.effectAddedCallbacks, AddCallback(PlayerEffectsHandler, "EffectAdded", 
        function(effect)
            if self.fortificationTable[effect:GetName()] then
                resourceBar:ChangeResource(self.fortificationTable[effect:GetName()])
                self.forticationID = effect:GetID()
            end
        end))

        table.insert(self.effectRemovedCallbacks, AddCallback(PlayerEffectsHandler, "EffectRemoved", 
        function(effect)
            if self.forticationID == effect:GetID() and self.fortificationTable[effect:GetName()] then
                resourceBar:ChangeResource(0)
            end
        end))
    elseif self.playerClass == Turbine.Gameplay.Class.Beorning then
        for i = 1, self.player:GetEffects():GetCount(), 1 do
            local effect = self.player:GetEffects():Get(i)
            if string.match(effect:GetName(), "Guarded %- Tier (.*)") then
                self.GuardedTierID = effect:GetID()
                resourceBar:ChangeResource(tonumber(string.match(effect:GetName(), "Guarded %- Tier (.*)")))
            end
        end

        table.insert(self.effectAddedCallbacks, AddCallback(PlayerEffectsHandler, "EffectAdded", 
        function(effect)
            if string.match(effect:GetName(), "Guarded %- Tier (.*)") then
                self.GuardedTierID = effect:GetID()
                resourceBar:ChangeResource(tonumber(string.match(effect:GetName(), "Guarded %- Tier (.*)")))
            end
        end))

        table.insert(self.effectRemovedCallbacks, AddCallback(PlayerEffectsHandler, "EffectRemoved", 
        function(effect)
            if self.GuardedTierID == effect:GetID() then
                resourceBar:ChangeResource(0)
            end
        end))
    elseif self.playerClass == Turbine.Gameplay.Class.BlackArrow then
        for i = 1, self.player:GetEffects():GetCount(), 1 do
            local effect = self.player:GetEffects():Get(i)
            if string.find(effect:GetName(), "Keen Eye") then
                self.keenEyeID = effect:GetID()
                if string.match(effect:GetName(), "Keen Eye %((.*)%)") then
                    resourceBar:ChangeResource(tonumber(string.match(effect:GetName(), "Keen Eye %((.*)%)")))
                elseif effect:GetName() == "Keen Eye" then
                    resourceBar:ChangeResource(1)
                end
            end
        end

        table.insert(self.effectAddedCallbacks, AddCallback(PlayerEffectsHandler, "EffectAdded", 
        function(effect)
            if string.find(effect:GetName(), "Keen Eye") then
                self.keenEyeID = effect:GetID()
                if string.match(effect:GetName(), "Keen Eye %((.*)%)") then
                    resourceBar:ChangeResource(tonumber(string.match(effect:GetName(), "Keen Eye %((.*)%)")))
                elseif effect:GetName() == "Keen Eye" then
                    resourceBar:ChangeResource(1)
                end
            end
        end))

        table.insert(self.effectRemovedCallbacks, AddCallback(PlayerEffectsHandler, "EffectRemoved", 
        function(effect)
            if self.keenEyeID == effect:GetID() then
                resourceBar:ChangeResource(0)
            end
        end))
    elseif self.playerClass == Turbine.Gameplay.Class.Weaver then
        for i = 1, self.player:GetEffects():GetCount(), 1 do
            local effect = self.player:GetEffects():Get(i)
            if string.gsub(effect:GetName(), "%d", "") == "Venom " then
                resourceBar:ChangeResource( tonumber(string.gsub(effect:GetName(), "%D", "")) )
            end
        end

        table.insert(self.effectAddedCallbacks, AddCallback(PlayerEffectsHandler, "EffectAdded", 
        function(effect)
            if string.gsub(effect:GetName(), "%d", "") == "Venom " then
                resourceBar:ChangeResource( tonumber(string.gsub(effect:GetName(), "%D", "")) )
                self.venomID = effect:GetID()
            end
        end))

        table.insert(self.effectRemovedCallbacks, AddCallback(PlayerEffectsHandler, "EffectRemoved", 
        function(effect)
            if self.venomID == effect:GetID() then
                resourceBar:ChangeResource(0)
            end
        end))
    end

    table.insert(self.tools, resourceBar)

    self.currentHeight = self.currentHeight + data["Height"]
end