function PrimeUI:SkillLabel(data, parent)
    local x = self:ContentAlignement(data["Alignment"], (data["Width"] + 2 * data["BorderSize"]) * data["Scale"])
    local y = self.currentHeight
    if self.groupMode then
        x = data["X"]
        y = data["Y"]
    end

    local valuebar = PrimeUISkillLabel(parent, x, y, data["Width"], data["Height"], data["Scale"], data["Icon"], self:ColourTupleToTurbineColour(data["BorderColour"]), data["BorderSize"], data["Clockwise"], data["Visibility_Threshold"], data["Hide_While_Not_Active"], data["Hide_While_Active"], data["Opacity_While_Active"], data["Opacity_While_Not_Active"], self:ColourTupleToTurbineColour(data["Highlight_Colour"]), data["Highlight_Size"], data["HighlightWhileOff"], data["Highlight"], data["Visual_Timer"])
    self.lookup[data.NickName or ""] = valuebar
    if self.groupMode then
        valuebar.moveBox = MoveBox(valuebar, data["Width"] * data["Scale"], data["Height"] * data["Scale"], data["NickName"] or "Skill label")
        valuebar.savePosition = function()
            data["X"], data["Y"] = valuebar:GetPosition()
        end
    end

    valuebar.SkillID = {}
    valuebar.UniqueNames = {}

    if tablelength(data["Highlight_On_Effect"]) > 0 then
        table.insert(self.effectAddedCallbacks, AddCallback(PlayerEffectsHandler, "EffectAdded", 
        function(effect)
            local name = effect:GetName()
            local value = data["Highlight_On_Effect"][name]
            if value then
                if type(value) == "table" then
                    local ID = effect:GetID()
                    valuebar.SkillID[ID] = true
                    valuebar:ToggleHighlight(not(value.Highlight_Value))
                else
                    
                    --????
                    --if value.Fit_Icon then
                    --    if data["Highlight_On_Effect"] ~= effect:GetIcon() then return end
                    --end

                    local ID = effect:GetID()
                    valuebar.SkillID[ID] = true
                    valuebar:ToggleHighlight(not(value["Invert_Highlight"]))
                end
            end
        end))

        table.insert(self.effectRemovedCallbacks, AddCallback(PlayerEffectsHandler, "EffectRemoved", 
        function(effect)
            local ID = effect:GetID()
            if valuebar.SkillID[ID] then
                local value = data["Highlight_On_Effect"][effect:GetName()]
                if value then
                    if value.Invert_Highlight ~= nil then
                        valuebar:ToggleHighlight(value["Invert_Highlight"])
                    elseif value.Highlight_Value ~= nil then
                        valuebar:ToggleHighlight(value.Highlight_Value)
                    end
                end
            end

        end))
    end

    if tablelength(data["EffectAdded"]) > 0 then
        table.insert(self.effectAddedCallbacks, AddCallback(PlayerEffectsHandler, "EffectAdded", 
        function(effect)
            local name = effect:GetName()
            local value = data["EffectAdded"][name]
            if value then
                if type(value) == "table" then
                    local time = value.Countdown
                    local maxTime = value.Countdown
                    if time == true then
                        time = effect:GetDuration() - (Turbine.Engine.GetGameTime() - effect:GetStartTime())
                        maxTime = effect:GetDuration()
                    end
                    valuebar:SetTimer(time, maxTime)
                else
                    local time = value
                    local maxTime = value
                    if time == true then
                        time = effect:GetDuration() - (Turbine.Engine.GetGameTime() - effect:GetStartTime())
                        maxTime = effect:GetDuration()
                    end
                    valuebar:SetTimer(time, maxTime)
                end
            end
        end))
    end

    if tablelength(data["EffectAdded_Target"]) > 0 then
        table.insert(self.effectTargetAddedCallbacks, TargetHandlerObject:AddEffectCallback( 
        function(args)
            local name = args.Name
            local value = data["EffectAdded_Target"][name]
            if value then
                if type(value) == "table" then
                    local time = value.Countdown
                    local maxTime = value.Countdown
                    if time == true then
                        time = args.Duration - (Turbine.Engine.GetGameTime() - args.StartTime)
                        maxTime = args.Duration
                    end
                    valuebar:SetTimer(time, maxTime)
                else
                    local time = value
                    local maxTime = value
                    if time == true then
                        time = args.Duration - (Turbine.Engine.GetGameTime() - args.StartTime)
                        maxTime = args.Duration
                    end
                    valuebar:SetTimer(time, maxTime)
                end
            end
        end))
    end

    if tablelength(data["EffectAdded_Group"]) > 0 then
        table.insert(self.effectAddedGroupCallbacks, AddCallback(GroupEffectsHandler, "EffectAdded", 
        function(args)
            local effectArgs = args.effect
            local name = effectArgs.Name
            local value = data["EffectAdded_Group"][name]
            if value then
                if type(value) == "table" then
                    local time = value.Countdown
                    local maxTime = value.Countdown
                    if time == true then
                        time = effectArgs.Duration - (Turbine.Engine.GetGameTime() - effectArgs.StartTime)
                        maxTime = effectArgs.Duration
                    end
                    valuebar:SetTimer(time, maxTime)
                else
                    local time = value
                    local maxTime = value
                    if time == true then
                        time = effectArgs.Duration - (Turbine.Engine.GetGameTime() - effectArgs.StartTime)
                        maxTime = effectArgs.Duration
                    end
                    valuebar:SetTimer(time, maxTime)
                end
            end
        end))
    end

    if tablelength(data["EffectRemoved"]) > 0 then
        table.insert(self.effectRemovedCallbacks, AddCallback(PlayerEffectsHandler, "EffectRemoved", 
        function(effect)
            local name = effect:GetName()
            local value = data["EffectRemoved"][name]
            if value then
                if type(value) == "table" then
                    local time = value.Countdown
                    local maxTime = value.Countdown
                    if time == true then
                        time = effect:GetDuration() - (Turbine.Engine.GetGameTime() - effect:GetStartTime())
                        maxTime = effect:GetDuration()
                    end
                    valuebar:SetTimer(time, maxTime)
                else
                    local time = value
                    local maxTime = value
                    if time == true then
                        time = effect:GetDuration() - (Turbine.Engine.GetGameTime() - effect:GetStartTime())
                        maxTime = effect:GetDuration()
                    end
                    valuebar:SetTimer(time, maxTime)
                end
            end
        end))
    end

    if tablelength(data["EnemyHits"]) > 0 then
        table.insert(self.enemyChatCallback, AddCallback(ChatEventHandler, "EnemyCombat", function(args)
            local value = data["EnemyHits"][args.SkillName]
            if value then
                if type(value) == "table" then
                    valuebar:SetTimer(value.Countdown, value.Countdown)
                else
                    valuebar:SetTimer(value, value)
                end
            end
        end))

    end

    if tablelength(data["PlayerHits"]) > 0 then
        table.insert(self.playerChatCallback, AddCallback(ChatEventHandler, "PlayerCombat", function(args)
            local value = data["PlayerHits"][args.SkillName]
            if value then
                if type(value) == "table" then
                    valuebar:SetTimer(value.Countdown, value.Countdown)
                else
                    valuebar:SetTimer(value, value)
                end
            end
        end))
    end

    if tablelength(data["PlayerBenefit"]) > 0 then
        table.insert(self.playerBenefitCallbacks, AddCallback(ChatEventHandler, "PlayerBenefit", function(args)
            local value = data["PlayerBenefit"][args.SkillName]
            if value then
                if type(value) == "table" then
                    valuebar:SetTimer(value.Countdown, value.Countdown)
                else
                    valuebar:SetTimer(value, value)
                end
            end
        end))
    end

    if tablelength(data["PlayerHeal"]) > 0 then
        table.insert(self.playerHealCallbacks, AddCallback(ChatEventHandler, "PlayerHeal", function(args)
            local value = data["PlayerHeal"][args.SkillName]
            if value then
                if type(value) == "table" then
                    valuebar:SetTimer(value.Countdown, value.Countdown)
                else
                    valuebar:SetTimer(value, value)
                end
            end
        end))
    end

    if tablelength(data["StandardChat"]) > 0 then
        table.insert(self.standardChatCallback, AddCallback(ChatEventHandler, "Standard", function(message)
            local value = data["StandardChat"][message]
            if value then
                if type(value) == "table" then
                    valuebar:SetTimer(value.Countdown, value.Countdown)
                else
                    valuebar:SetTimer(value, value)
                end
            end
        end))
    end

    if tablelength(data["PartyChat"]) > 0 then
        table.insert(self.partyChatCallback, AddCallback(ChatEventHandler, "Party", function(message)
            local value = data["PartyChat"][message]
            if value then
                if type(value) == "table" then
                    valuebar:SetTimer(value.Countdown, value.Countdown)
                else
                    valuebar:SetTimer(value, value)
                end
            end
        end))
    end

    if tablelength(data["SayChat"]) > 0 then
        table.insert(self.sayChatCallback, AddCallback(ChatEventHandler, "SayLine", function(message)
            local value = data["SayChat"][message]
            if value then
                if type(value) == "table" then
                    valuebar:SetTimer(value.Countdown, value.Countdown)
                else
                    valuebar:SetTimer(value, value)
                end
            end
        end))
    end


    if tablelength(data["Skills"]) > 0 then
        self.skills = true
        for key, value in pairs(data["Skills"]) do
            if SkillEventHandler:SkillExists(key) then
                -- Add the callback for this skill slot
                if not(self.skillCallbacks[key]) then self.skillCallbacks[key] = {} end
                table.insert(self.skillCallbacks[key], AddCallback(SkillEventHandler, key, function(skill) 
                    if type(value) == "table" then
                        local time = value.Countdown
                        local maxTime = value.Countdown
                        if time == true then
                            time = skill:GetResetTime() - Turbine.Engine.GetGameTime()
                            maxTime = skill:GetCooldown()
                        end
    
                        valuebar:SetTimer(time, maxTime)
                    else
                        local time = value
                        local maxTime = value
                        if value == true then
                            time = skill:GetResetTime() - Turbine.Engine.GetGameTime()
                            maxTime = skill:GetCooldown()
                        end
    
                        valuebar:SetTimer(time, maxTime)
                    end
                end))
            end
        end
    end

    self.currentHeight = self.currentHeight + data["Height"]

    table.insert(self.UI, valuebar)
end