function PrimeUI:Text(data, parent)
    local valuebar = Turbine.UI.Control()
    valuebar:SetParent(parent)
    if self.groupMode then
        valuebar:SetPosition(data["X"], data["Y"])
        valuebar.moveBox = MoveBox(valuebar, (data["Width"] + 2 * data["BorderSize"]) * data["Scale"], (data["Height"] + 2 * data["BorderSize"]) * data["Scale"], data["NickName"] or "Text")
        valuebar.savePosition = function()
            data["X"], data["Y"] = valuebar:GetPosition()
        end
    else
        valuebar:SetPosition(self:ContentAlignement(data["Alignment"], (data["Width"] + 2 * data["BorderSize"]) * data["Scale"]), self.currentHeight)
    end
    valuebar:SetSize((data["Width"] + 2 * data["BorderSize"]) * data["Scale"], (data["Height"] + 2 * data["BorderSize"]) * data["Scale"])
    valuebar:SetMouseVisible(false)
    valuebar:SetBackColor(self:ColourTupleToTurbineColour(data["BorderColour"]))
    valuebar:SetZOrder(100)

    valuebar.innerbar = Turbine.UI.Label()
    valuebar.innerbar:SetParent(valuebar)
    valuebar.innerbar:SetPosition(data["BorderSize"] * data["Scale"], data["BorderSize"] * data["Scale"])
    valuebar.innerbar:SetSize(data["Width"], data["Height"])
    valuebar.innerbar:SetMouseVisible(false)
    valuebar.innerbar:SetBlendMode(Turbine.UI.BlendMode.AlphaBlend)
    valuebar.innerbar:SetBackColor(self:ColourTupleToTurbineColour(data["BackgroundColour"]))
    valuebar.innerbar:SetForeColor(self:ColourTupleToTurbineColour(data["Font_Colour"]))
    valuebar.innerbar:SetFontStyle(Turbine.UI.FontStyle.Outline)
    valuebar.innerbar:SetOutlineColor(Turbine.UI.Color.Black)
    valuebar.innerbar:SetFont(self:FontToFont(data["Font"]))
    valuebar.innerbar:SetVisible(data.Visible_On_Load or data.Visible_On_Load == nil)
    valuebar.innerbar:SetTextAlignment(Turbine.UI.ContentAlignment[data.TextAlignment] or Turbine.UI.ContentAlignment.MiddleLeft)
    valuebar.innerbar:SetText(data["Text"])
    valuebar.innerbar:SetMarkupEnabled(true)

    valuebar.innerbar:SetStretchMode(1)
    valuebar.innerbar:SetSize(data["Width"] * data["Scale"], data["Height"] * data["Scale"])

    valuebar.baseText = data["Text"]
    valuebar.textTable = {}
    valuebar.instanceState = Settings.InstanceState

    self.lookup[data.NickName or ""] = valuebar.innerbar

    valuebar.statusTable = {}
    for key, value in pairs(data["Effects"]) do
        if value == false then
            valuebar.statusTable[key] = 1
        else
            valuebar.statusTable[key] = 0
        end
    end

    valuebar.togglefunction = function()
        local tableValue = 0
        for key, value in pairs(valuebar.statusTable) do
            if value > 0 then
                tableValue = tableValue + 1
            end
        end
        if data["Instance_Only"] and valuebar.instanceState then
            tableValue = tableValue + 1
        end
        local visibilityValue = tableValue >= valuebar.minCount
        valuebar:SetVisible(visibilityValue)
        if valuebar.innerbar.highlight then valuebar.innerbar.highlight:Toggle(visibilityValue) end
    end


    if data["Visibilty_Event"] and data["Invisibility_Event"] then
        table.insert(self.eventTriggerCallbacks, AddCallback(self, "EventTrigger", 
        function(event)
            if data["Visibilty_Event"][event] then
                valuebar.innerbar:SetVisible(true)
            end
            
            if data["Invisibility_Event"][event] then
                valuebar.innerbar:SetVisible(false)
            end
        end))
    end

    valuebar:SetVisible(false)
    valuebar.count = 0
    if data["Conjuction"] then
        valuebar.minCount = tablelength(data["Effects"]) + tablelength(data["HealthThreshold"]) + (data["Instance_Only"] and 1 or 0)
    else
        valuebar.minCount = 1
    end

    if data["Instance_Only"] and valuebar.instanceState then
        valuebar.count = valuebar.count + 1
    end

    if data["AlwaysVisible"] then
        valuebar:SetVisible(true)
    else
        for i = 1, self.player:GetEffects():GetCount(), 1 do
            local effect = self.player:GetEffects():Get(i)
            if data["Effects"][effect:GetName()] ~= nil then
                if data["Effects"][effect:GetName()] == true then
                    valuebar.statusTable[effect:GetName()] = valuebar.statusTable[effect:GetName()] + 1
                elseif data["Effects"][effect:GetName()] == false then
                    valuebar.statusTable[effect:GetName()] = valuebar.statusTable[effect:GetName()] - 1
                end
            end
        end


        if tablelength(data["Effects"]) > 0 then
            table.insert(self.effectAddedCallbacks, AddCallback(PlayerEffectsHandler, "EffectAdded", 
            function(effect)
                local name = effect:GetName()
                if data["Effects"][name] ~= nil then
                    local value = data["Effects"][effect:GetName()]
                    if value == true then
                        valuebar.statusTable[name] = valuebar.statusTable[name] + 1
                    else
                        valuebar.statusTable[name] = valuebar.statusTable[name] - 1
                    end
                    valuebar.togglefunction()
                end
            end))

            table.insert(self.effectRemovedCallbacks, AddCallback(PlayerEffectsHandler, "EffectRemoved", 
            function(effect)
                local name = effect:GetName()
                if data["Effects"][name] ~= nil then
                    local value = data["Effects"][effect:GetName()]
                    if value == true then
                        valuebar.statusTable[name] = valuebar.statusTable[name] - 1
                    else
                        valuebar.statusTable[name] = valuebar.statusTable[name] + 1
                    end
                    valuebar.togglefunction()
                end
            end))
        end

        if tablelength(data["HealthThreshold"]) > 0 then
            local health = self.player:GetMorale() / self.player:GetMaxMorale() * 100
            valuebar.healthThresholdPercentage = {}
            for key, value in pairs(data["HealthThreshold"]) do
                local checkValue = (self.compareFunctions[value["Operator"]](health, key) and 1) or 0
                valuebar.statusTable[tostring(key) .. tostring(value["Operator"])] = checkValue
                valuebar.healthThresholdPercentage[tostring(key) .. tostring(value["Operator"])] = checkValue
            end
            table.insert(self.playerHealthCallbacks, AddCallback(self.player, "MoraleChanged", function()
                local percentage = self.player:GetMorale() / self.player:GetMaxMorale() * 100
                local time = Turbine.Engine.GetGameTime()
                for key, value in pairs(data["HealthThreshold"]) do
                    local id = tostring(key) .. tostring(value["Operator"])
                    if self.compareFunctions[value["Operator"]](percentage, key) then
                        if valuebar.healthThresholdPercentage[id] == 0 then
                            if valuebar.healthThresholdPercentage[id] == 0 then
                                valuebar.healthThresholdPercentage[id] = 1
                                valuebar.statusTable[tostring(key) .. tostring(value["Operator"])] = valuebar.statusTable[tostring(key) .. tostring(value["Operator"])] + 1
                            end
                        end
                    else
                        if valuebar.healthThresholdPercentage[id] == 1 then
                            valuebar.healthThresholdPercentage[id] = 0
                            valuebar.statusTable[tostring(key) .. tostring(value["Operator"])] = valuebar.statusTable[tostring(key) .. tostring(value["Operator"])] - 1
                        end
                    end
                end
                valuebar.togglefunction()
            end))
        end
        valuebar.togglefunction()
    end

    if true then
        --[[
            Instances:
            State - In an instance or not
            Instance_Name - Name of the instance or all
            Instance_Tier - Tier of instance or \"\"

            Boss fights:
            Boss_Name - Name of the boss fight
            Boss_Tier - Tier of the boss fight
            Boss_Completion_Time - Fight duration (wipe or completion)
            Boss_Wipes - Amount of wipes in a fight

            Player:
            Player_Name - Name of the player
            Player_Class - Class of the player

            Stats:
            Death_Count_Character - Amount of deaths on character
            Death_Count_Account - Amount of deaths on account
            Death_Count_Misadventure_Character - Amount of Misadventure deaths on character
            Death_Count_Misadventure_Account - Amount of Misadventure deaths on account

        ]]

        -- Get keywords in text
        valuebar.keyWords = {}
        valuebar.keyWordCallbacksRequired = {}
        for capture in string.gmatch(valuebar.baseText, "<(.-)>") do
            valuebar.keyWords[capture] = true
            if self.keyWordCategories[capture] then
                valuebar.keyWordCallbacksRequired[self.keyWordCategories[capture]] = true
            end
        end

        valuebar.RoundingFunc = function(table)
            for key, value in pairs(table) do
                if valuebar.keyWords[key] and type(value) == "number" then
                    --ChatOutput(type(value))
                    table[key] = FormatBigNumberVitals(value)
                end
            end
            return table
        end

        valuebar.keyWordData = InstanceController:GetInitialData()
        local charData, accData = StatEventHandling:GetData()

        for key, value in pairs(PlayerHandlerObject:GetInfo()) do
            if string.find(key,"Temporary") and value > 0 or not(string.find(key,"Temporary"))  then 
                if type(value) == "number" then
                    valuebar.keyWordData[key] = FormatBigNumberVitals(value)
                else
                    valuebar.keyWordData[key] = value
                end
            end
        end

        for key, value in pairs(PlayerHandlerObject:GetPetInfo()) do
            if string.find(key,"Temporary") and value > 0 or not(string.find(key,"Temporary"))  then 
                if type(value) == "number" then
                    valuebar.keyWordData[key] = FormatBigNumberVitals(value)
                else
                    valuebar.keyWordData[key] = value
                end
            end
        end


        valuebar.keyWordData["Player_Name"] = NameHandlerObject:GetName(self.player:GetName(), true, "PrimeUIText", "")
        valuebar.keyWordData["Player_Class"] = self.classObjectsToName[self.player:GetClass()]
        valuebar.keyWordData["Death_Count_Character"] = tostring(charData["Death"])
        valuebar.keyWordData["Death_Count_Account"] = tostring(accData["Death"])
        valuebar.keyWordData["Death_Count_Misadventure_Character"] = tostring(charData["MisadventureDeaths"])
        valuebar.keyWordData["Death_Count_Misadventure_Account"] = tostring(accData["MisadventureDeaths"])

        
        for key, value in pairs(valuebar.keyWordData) do
            if key == "Instance_Tier" and value == 0 then valuebar.keyWordData[key] = nil end
            if key == "Boss_Tier" and value == 0 then valuebar.keyWordData[key] = nil end
            if key == "Instance_Name" and value == "All" then valuebar.keyWordData[key] = nil end
        end

        if Settings.DelvingMissionActive then
            valuebar.keyWordData["Mission_Name"] = Settings.MissionQuest
            valuebar.keyWordData["Delving_Tier"] = Settings.DelvingTier
        end

        self:SetText(valuebar.innerbar, valuebar.textTable, valuebar.keyWordData, valuebar.baseText, valuebar.keyWords)  

        table.insert(self.deathChatCallbacks, AddCallback(ChatEventHandler, "Death", function(args)
            local charData, accData = StatEventHandling:GetData()
            self:SetText(valuebar.innerbar, valuebar.textTable, 
            {["Death_Count_Character"] = tostring(charData["Death"]), 
            ["Death_Count_Account"] = tostring(accData["Death"]),
            ["Death_Count_Misadventure_Character"] = tostring(charData["MisadventureDeaths"]),
            ["Death_Count_Misadventure_Account"] = tostring(accData["MisadventureDeaths"])}, valuebar.baseText)   
        end))

        -- Instance callback
        table.insert(self.instanceEventCallbacks, AddCallback(InstanceController, "InstanceEvent", function(args)
            if args["Instance_Tier"] == 0 then args["Instance_Tier"] = nil end
            if args["Instance_Name"] == "All" then args["Instance_Name"] = nil end

            self:SetText(valuebar.innerbar, valuebar.textTable, args, valuebar.baseText, valuebar.keyWords)   
            
            if data["Instance_Only"] then
                if args["State"] and not(valuebar.instanceState) then
                    valuebar.count = valuebar.count + 1
                    valuebar.instanceState = true
                elseif valuebar.instanceState then
                    valuebar.count = valuebar.count - 1
                    valuebar.instanceState = false
                end
                valuebar:SetVisible(valuebar.count == valuebar.minCount)
            end
        end))

        -- Delving callback
        table.insert(self.delvingEventCallbacks, AddCallback(InstanceController, "DelvingEvent", function(args)
            if not(args["DelvingMissionActive"]) then
                args["Delving_Tier"] = nil
                args["Mission_Name"] = nil
            end

            self:SetText(valuebar.innerbar, valuebar.textTable, args, valuebar.baseText, valuebar.keyWords)   
            
            if data["Instance_Only"] then
                if args["DelvingMissionActive"] and not(valuebar.instanceState) then
                    valuebar.count = valuebar.count + 1
                    valuebar.instanceState = true
                elseif valuebar.instanceState then
                    valuebar.count = valuebar.count - 1
                    valuebar.instanceState = false
                end
                valuebar:SetVisible(valuebar.count == valuebar.minCount)
            end
        end))

        -- Fight callback
        table.insert(self.fightEventStartCallbacks, AddCallback(InstanceController, "FightEventStart", function(args)
            self:SetText(valuebar.innerbar, valuebar.textTable, args, valuebar.baseText, valuebar.keyWords)  
        end))

        table.insert(self.fightEventEndCallbacks, AddCallback(InstanceController, "FightEventEnd", function(args)
            self:SetText(valuebar.innerbar, valuebar.textTable, args, valuebar.baseText, valuebar.keyWords)  
        end))
        
        -- Morale callback
        if valuebar.keyWordCallbacksRequired["Player_Morale"] then
            table.insert(self.playerMoraleHandlerCallbacks, AddCallback(PlayerHandlerObject, "MoraleChanged", function(args)
                self:SetText(valuebar.innerbar, valuebar.textTable, valuebar.RoundingFunc(args), valuebar.baseText, valuebar.keyWords)  
            end))
        end

        -- Temporary Morale callback
        if valuebar.keyWordCallbacksRequired["Player_Temporary_Morale"] then
            table.insert(self.playerTemporaryMoraleHandlerCallbacks, AddCallback(PlayerHandlerObject, "TemporaryMoraleChanged", function(args)
                if args.Player_Temporary_Morale == 0 then args.Player_Temporary_Morale = nil; valuebar.textTable.Player_Temporary_Morale = nil end
                self:SetText(valuebar.innerbar, valuebar.textTable, valuebar.RoundingFunc(args), valuebar.baseText, valuebar.keyWords)  
            end))
        end

        -- Power Callback
        if valuebar.keyWordCallbacksRequired["Player_Power"] then
            table.insert(self.playerPowerHandlerCallbacks, AddCallback(PlayerHandlerObject, "PowerChanged", function(args)
                self:SetText(valuebar.innerbar, valuebar.textTable, valuebar.RoundingFunc(args), valuebar.baseText, valuebar.keyWords)  
            end))
        end

        -- Temporary Power Callback
        if valuebar.keyWordCallbacksRequired["Player_Temporary_Power"] then
            table.insert(self.playerTemporaryPowerHandlerCallbacks, AddCallback(PlayerHandlerObject, "TemporaryPowerChanged", function(args)
                self:SetText(valuebar.innerbar, valuebar.textTable, valuebar.RoundingFunc(args), valuebar.baseText, valuebar.keyWords)  
            end))
        end

        -- Level callback
        if valuebar.keyWordCallbacksRequired["Player_Temporary_Power"] then
            table.insert(self.playerLevelHandlerCallbacks, AddCallback(PlayerHandlerObject, "LevelChanged", function(args)
                self:SetText(valuebar.innerbar, valuebar.textTable, valuebar.RoundingFunc(args), valuebar.baseText, valuebar.keyWords)  
            end))
        end

        -- Pet morale
        if valuebar.keyWordCallbacksRequired["Pet_Morale"] then
            table.insert(self.petMoraleCallbacks, AddCallback(PlayerHandlerObject, "PetMoraleChanged", function(args)
                self:SetText(valuebar.innerbar, valuebar.textTable, valuebar.RoundingFunc(args), valuebar.baseText, valuebar.keyWords)  
            end))
        end

        -- pet temp morale
        if valuebar.keyWordCallbacksRequired["Pet_Temporary_Morale"] then
            table.insert(self.petTemporaryMoraleCallbacks, AddCallback(PlayerHandlerObject, "PetTemporaryMoraleChanged", function(args)
                self:SetText(valuebar.innerbar, valuebar.textTable, valuebar.RoundingFunc(args), valuebar.baseText, valuebar.keyWords)  
            end))
        end

        -- pet info
        if valuebar.keyWordCallbacksRequired["Pet_Morale"] or valuebar.keyWordCallbacksRequired["Pet_Temporary_Morale"] or valuebar.keyWordCallbacksRequired["Pet_Info"] then
            table.insert(self.petInfoCallbacks, AddCallback(PlayerHandlerObject, "PetInfo", function(args)
                if not(args.Status) then
                    valuebar.textTable.Pet_Morale = nil
                    valuebar.textTable.Pet_Temporary_Morale = nil 
                    valuebar.textTable.Pet_Name = nil 
                end
                self:SetText(valuebar.innerbar, valuebar.textTable, valuebar.RoundingFunc(args), valuebar.baseText, valuebar.keyWords) 
            end))
        end

        if valuebar.keyWordCallbacksRequired["Target_Morale"] then
            table.insert(self.healthTargetCallbacks, TargetHandlerObject:AddHealthCallback(function(value) self:SetText(valuebar.innerbar, valuebar.textTable, valuebar.RoundingFunc({["Target_Morale"] = value}), valuebar.baseText, valuebar.keyWords) end))
        end

        if valuebar.keyWordCallbacksRequired["Target_Temporary_Morale"] then
            table.insert(self.temporaryHealthTargetCallbacks, TargetHandlerObject:AddTemporaryHealthCallback(function(value) self:SetText(valuebar.innerbar, valuebar.textTable, valuebar.RoundingFunc({["Target_Temporary_Morale"] = value}), valuebar.baseText, valuebar.keyWords) end))
        end

        if valuebar.keyWordCallbacksRequired["Target_Morale"] or valuebar.keyWordCallbacksRequired["Target_Temporary_Morale"] or valuebar.keyWordCallbacksRequired["Target_Level"] then
            table.insert(self.temporaryHealthTargetCallbacks, 
            TargetHandlerObject:AddTargetInfoCallback(function(value) 
                if value then
                    local args = {}
                    args["Target_Morale"] = value["Morale"]
                    args["Target_Max_Morale"] = value["MaxMorale"]
                    args["Target_Name"] = NameHandlerObject:GetName(value["Name"], true, "PrimeUIText", "")
                    args["Target_Level"] = value["Level"]
                    args["Target_Temporary_Morale"] = value["TemporaryMorale"]
                    args["Target_Max_Temporary_Morale"] = value["TemporaryMaxMorale"]

                    self:SetText(valuebar.innerbar, valuebar.textTable, valuebar.RoundingFunc(args), valuebar.baseText, valuebar.keyWords)
                else
                    local args = {}
                    args["Target_Morale"] = ""
                    args["Target_Max_Morale"] = ""
                    args["Target_Name"] = ""
                    args["Target_Level"] = ""
                    args["Target_Temporary_Morale"] = ""
                    args["Target_Max_Temporary_Morale"] = ""

                    self:SetText(valuebar.innerbar, valuebar.textTable, args, valuebar.baseText, valuebar.keyWords)
                end
            end))
        end

        if valuebar.keyWordCallbacksRequired["PvP"] then
            self:SetText(valuebar.innerbar, valuebar.textTable, InstanceController:GetPvpData(), valuebar.baseText, valuebar.keyWords) 
            table.insert(self.pvpCallbacks, AddCallback(InstanceController, "PvpInfo", function(args)
                self:SetText(valuebar.innerbar, valuebar.textTable, args, valuebar.baseText, valuebar.keyWords) 
            end))
        end

        if valuebar.keyWordCallbacksRequired["Location"] then
            self:SetText(valuebar.innerbar, valuebar.textTable, {["Location"] = LocationHandler:GetCurrentLocation()}, valuebar.baseText, valuebar.keyWords) 
            table.insert(self.locationCallbacks, AddCallback(LocationHandler, "Location", function(args)
                self:SetText(valuebar.innerbar, valuebar.textTable, args, valuebar.baseText, valuebar.keyWords) 
            end))
        end
    end

    
    self.currentHeight = self.currentHeight + (data["Height"] + 2 * data["BorderSize"]) * data["Scale"]

    table.insert(self.UI, valuebar.innerbar)
    table.insert(self.UI, valuebar)
end

function PrimeUI:SetText(label, table, args, base, keyWords)
    for key, value in pairs(args) do
        table[key] = tostring(value)
    end

    local newText = base
    for key, value in pairs(keyWords) do
        if table[key] then
            newText = newText:gsub(string.format("<%s>", tostring(key)), table[key])
        end
    end
    
    --newText = newText:gsub("<.->%[.-%]", "")
    --local one, two = string.match(newText, "<([%P%:]+)>%[([%P%:]+)%]")

    newText = newText:gsub("<([%P%:%s%_%-]+)>%[([%P%:%s%_%-]+)%]", "")

    --newText = newText:gsub("%[[%p%w%s^%<%>%[%]]-%]<[%p%w%s^%<%>%[%]]->", "")
    newText = newText:gsub("%[([%P%:%s%_%-]+)%]<([%P%:%s%_%-]+)>", "")

    newText = newText:gsub("[%[%]]", "")
    
    --newText = newText:gsub("%]", "")
    
    newText = newText:gsub("<.->", "")

    label:SetText(newText)
end