function PrimeUI:TimerIcon(data, parent)
    local valuebar = Turbine.UI.Control()
    valuebar:SetParent(parent)
    if self.groupMode then
        valuebar:SetPosition(data["X"], data["Y"])
        valuebar.moveBox = MoveBox(valuebar, data["Width"] + 2 * data["BorderSize"], data["Height"] + 2 * data["BorderSize"], data["NickName"] or "Timer Icon")
        valuebar.savePosition = function()
            data["X"], data["Y"] = valuebar:GetPosition()
        end
    else
        valuebar:SetPosition(self:ContentAlignement(data["Alignment"], (data["Width"] + 2 * data["BorderSize"]) * data["Scale"]), self.currentHeight)
    end
    valuebar:SetSize(data["Width"] + 2 * data["BorderSize"], data["Height"] + 2 * data["BorderSize"])
    valuebar:SetMouseVisible(false)
    valuebar:SetBackColor(self:ColourTupleToTurbineColour(data["BorderColour"]))

    local innerbar = ClassSpecificTools.BleedTimerLabel(valuebar, data["BorderSize"], data["BorderSize"], 32, 32 * data["Scale"], data["Icon"], true, data["Clockwise"], 0, 1, 1, data["Hide_While_Not_Active"], data["Visual_Timer"], data["ShowText"], data["GrayOut"], data["Hide_While_Not_Active"], data["Visibility_Threshold"], self:FontToFont(data["Font"]), data["Hide_While_Active"])
    self.lookup[data.NickName or ""] = innerbar
    innerbar:SetVisible(not(data["Hide_While_Not_Active"]))
    --valuebar:SetStretchMode(1)
    innerbar:SetPosition(data["BorderSize"] * data["Scale"], data["BorderSize"] * data["Scale"])
    valuebar:SetSize((data["Width"] + 2 * data["BorderSize"]) * data["Scale"], (data["Height"] + 2 * data["BorderSize"]) * data["Scale"])

    --valuebar:SetVisible(false)
    if innerbar.highlight then innerbar.highlight:Toggle(false) end
    innerbar.highlightWhileOff = data["HighlightWhileOff"]

    if data["Highlight"] then
        innerbar.highlight = ClassSpecificTools.HighlightFrame2(innerbar, data["Scale"])
        innerbar.highlight:Toggle(data["HighlightWhileOff"])
    end

    if tablelength(data["EffectAdded"]) > 0 then
        table.insert(self.effectAddedCallbacks, AddCallback(PlayerEffectsHandler, "EffectAdded", 
        function(effect)
            local name = effect:GetName()
            local value = data["EffectAdded"][name]
            if value then
                if type(value) == "table" then
                    local time = value.Countdown
                    local maxTime = value.Countdown
                    if time == true then
                        time = effect:GetDuration() - (Turbine.Engine.GetGameTime() - effect:GetStartTime())
                        maxTime = effect:GetDuration()
                    end
                    innerbar:SetTimer(time, maxTime, value["Event_Keyword"] and (function() self:FireTrigger(value["Event_Keyword"]) end))
                    if innerbar.highlight then innerbar.highlight:Toggle(not(data["HighlightWhileOff"])) end
                    self.icons[innerbar] = true
                    if not(self.updateCallback) then self.updateCallback = AddCallback(Updater, "ExecuteUpdate", self.ExecuteUpdate) end 
                else
                    local time = value
                    local maxTime = value
                    if time == true then
                        time = effect:GetDuration() - (Turbine.Engine.GetGameTime() - effect:GetStartTime())
                        maxTime = effect:GetDuration()
                    end
                    innerbar:SetTimer(time, maxTime)
                    if innerbar.highlight then innerbar.highlight:Toggle(not(data["HighlightWhileOff"])) end
                    self.icons[innerbar] = true
                    if not(self.updateCallback) then self.updateCallback = AddCallback(Updater, "ExecuteUpdate", self.ExecuteUpdate) end
                end
            end
        end))
    end

    if tablelength(data["EffectAdded_Target"]) > 0 then
        table.insert(self.effectTargetAddedCallbacks, TargetHandlerObject:AddEffectCallback( 
        function(args)
            local name = args.Name
            local value = data["EffectAdded_Target"][name]
            if value then
                if type(value) == "table" then
                    local time = value.Countdown
                    local maxTime = value.Countdown
                    if time == true then
                        time = args.Duration - (Turbine.Engine.GetGameTime() - args.StartTime)
                        maxTime = args.Duration
                    end
                    innerbar:SetTimer(time, maxTime, value["Event_Keyword"] and (function() self:FireTrigger(value["Event_Keyword"]) end))
                    if innerbar.highlight then innerbar.highlight:Toggle(not(data["HighlightWhileOff"])) end
                    self.icons[innerbar] = true
                    if not(self.updateCallback) then self.updateCallback = AddCallback(Updater, "ExecuteUpdate", self.ExecuteUpdate) end
                else
                    local time = value
                    local maxTime = value
                    if time == true then
                        time = args.Duration - (Turbine.Engine.GetGameTime() - args.StartTime)
                        maxTime = args.Duration
                    end
                    innerbar:SetTimer(time, maxTime)
                    if innerbar.highlight then innerbar.highlight:Toggle(not(data["HighlightWhileOff"])) end
                    self.icons[innerbar] = true
                    if not(self.updateCallback) then self.updateCallback = AddCallback(Updater, "ExecuteUpdate", self.ExecuteUpdate) end
                end
            end
        end))
    end

    if tablelength(data["EffectAdded_Group"]) > 0 then
        table.insert(self.effectAddedGroupCallbacks, AddCallback(GroupEffectsHandler, "EffectAdded", 
        function(args)
            local effectArgs = args.effect
            local name = effectArgs.Name
            local value = data["EffectAdded_Group"][name]
            if value then
                if type(value) == "table" then
                    local time = value.Countdown
                    local maxTime = value.Countdown
                    if time == true then
                        time = effectArgs.Duration - (Turbine.Engine.GetGameTime() - effectArgs.StartTime)
                        maxTime = effectArgs.Duration
                    end
                    innerbar:SetTimer(time, maxTime, value["Event_Keyword"] and (function() self:FireTrigger(value["Event_Keyword"]) end))
                    if innerbar.highlight then innerbar.highlight:Toggle(not(data["HighlightWhileOff"])) end
                    self.icons[innerbar] = true
                    if not(self.updateCallback) then self.updateCallback = AddCallback(Updater, "ExecuteUpdate", self.ExecuteUpdate) end
                else
                    local time = value
                    local maxTime = value
                    if time == true then
                        time = effectArgs.Duration - (Turbine.Engine.GetGameTime() - effectArgs.StartTime)
                        maxTime = effectArgs.Duration
                    end
                    innerbar:SetTimer(time, maxTime)
                    if innerbar.highlight then innerbar.highlight:Toggle(not(data["HighlightWhileOff"])) end
                    self.icons[innerbar] = true
                    if not(self.updateCallback) then self.updateCallback = AddCallback(Updater, "ExecuteUpdate", self.ExecuteUpdate) end
                end
            end
        end))
    end

    if tablelength(data["EffectRemoved"]) > 0 then
        table.insert(self.effectRemovedCallbacks, AddCallback(PlayerEffectsHandler, "EffectRemoved", 
        function(effect)
            local name = effect:GetName()
            local value = data["EffectRemoved"][name]
            if value then
                if type(value) == "table" then
                    local time = value.Countdown
                    local maxTime = value.Countdown
                    if time == true then
                        time = effect:GetDuration() - (Turbine.Engine.GetGameTime() - effect:GetStartTime())
                        maxTime = effect:GetDuration()
                    end
                    innerbar:SetTimer(time, maxTime, value["Event_Keyword"] and (function() self:FireTrigger(value["Event_Keyword"]) end))
                    if innerbar.highlight then innerbar.highlight:Toggle(not(data["HighlightWhileOff"])) end
                    self.icons[innerbar] = true
                    if not(self.updateCallback) then self.updateCallback = AddCallback(Updater, "ExecuteUpdate", self.ExecuteUpdate) end

                else
                    local time = value
                    local maxTime = value
                    if time == true then
                        time = effect:GetDuration() - (Turbine.Engine.GetGameTime() - effect:GetStartTime())
                        maxTime = effect:GetDuration()
                    end
                    innerbar:SetTimer(time, maxTime)
                    if innerbar.highlight then innerbar.highlight:Toggle(not(data["HighlightWhileOff"])) end
                    self.icons[innerbar] = true
                    if not(self.updateCallback) then self.updateCallback = AddCallback(Updater, "ExecuteUpdate", self.ExecuteUpdate) end
                end
            end
        end))
    end

    if tablelength(data["EnemyHits"]) > 0 then
        table.insert(self.enemyChatCallback, AddCallback(ChatEventHandler, "EnemyCombat", function(args)
            local value = data["EnemyHits"][args.SkillName]
            if value then
                if type(value) == "table" then
                    innerbar:SetTimer(value.Countdown, value.Countdown, value["Event_Keyword"] and (function() self:FireTrigger(value["Event_Keyword"]) end))
                    if innerbar.highlight then innerbar.highlight:Toggle(not(data["HighlightWhileOff"])) end
                    self.icons[innerbar] = true
                    if not(self.updateCallback) then self.updateCallback = AddCallback(Updater, "ExecuteUpdate", self.ExecuteUpdate) end
                else
                    innerbar:SetTimer(value, value)
                    if innerbar.highlight then innerbar.highlight:Toggle(not(data["HighlightWhileOff"])) end
                    self.icons[innerbar] = true
                    if not(self.updateCallback) then self.updateCallback = AddCallback(Updater, "ExecuteUpdate", self.ExecuteUpdate) end
                end
            end
        end))

    end

    if tablelength(data["PlayerHits"]) > 0 then
        table.insert(self.playerChatCallback, AddCallback(ChatEventHandler, "PlayerCombat", function(args)
            local value = data["PlayerHits"][args.SkillName]
            if value then
                if type(value) == "table" then
                    innerbar:SetTimer(value.Countdown, value.Countdown, value["Event_Keyword"] and (function() self:FireTrigger(value["Event_Keyword"]) end))
                    if innerbar.highlight then innerbar.highlight:Toggle(not(data["HighlightWhileOff"])) end
                    self.icons[innerbar] = true
                    if not(self.updateCallback) then self.updateCallback = AddCallback(Updater, "ExecuteUpdate", self.ExecuteUpdate) end
                else
                    innerbar:SetTimer(value, value)
                    if innerbar.highlight then innerbar.highlight:Toggle(not(data["HighlightWhileOff"])) end
                    self.icons[innerbar] = true
                    if not(self.updateCallback) then self.updateCallback = AddCallback(Updater, "ExecuteUpdate", self.ExecuteUpdate) end
                end
            end
        end))
    end

    if tablelength(data["StandardChat"]) > 0 then
        table.insert(self.standardChatCallback, AddCallback(ChatEventHandler, "Standard", function(message)
            local value = data["StandardChat"][message]
            if value then
                if type(value) == "table" then
                    innerbar:SetTimer(value.Countdown, value.Countdown, value["Event_Keyword"] and (function() self:FireTrigger(value["Event_Keyword"]) end))
                    if innerbar.highlight then innerbar.highlight:Toggle(not(data["HighlightWhileOff"])) end
                    self.icons[innerbar] = true
                    if not(self.updateCallback) then self.updateCallback = AddCallback(Updater, "ExecuteUpdate", self.ExecuteUpdate) end
                else
                    innerbar:SetTimer(value, value)
                    if innerbar.highlight then innerbar.highlight:Toggle(not(data["HighlightWhileOff"])) end
                    self.icons[innerbar] = true
                    if not(self.updateCallback) then self.updateCallback = AddCallback(Updater, "ExecuteUpdate", self.ExecuteUpdate) end
                end
            end
        end))
    end

    if tablelength(data["PartyChat"]) > 0 then
        table.insert(self.partyChatCallback, AddCallback(ChatEventHandler, "Party", function(message)
            local value = data["PartyChat"][message]
            if value then
                if type(value) == "table" then
                    innerbar:SetTimer(value.Countdown, value.Countdown, value["Event_Keyword"] and (function() self:FireTrigger(value["Event_Keyword"]) end))
                    if innerbar.highlight then innerbar.highlight:Toggle(not(data["HighlightWhileOff"])) end
                    self.icons[innerbar] = true
                    if not(self.updateCallback) then self.updateCallback = AddCallback(Updater, "ExecuteUpdate", self.ExecuteUpdate) end
                else
                    innerbar:SetTimer(value, value)
                    if innerbar.highlight then innerbar.highlight:Toggle(not(data["HighlightWhileOff"])) end
                    self.icons[innerbar] = true
                    if not(self.updateCallback) then self.updateCallback = AddCallback(Updater, "ExecuteUpdate", self.ExecuteUpdate) end
                end
            end
        end))
    end

    if tablelength(data["SayChat"]) > 0 then
        table.insert(self.sayChatCallback, AddCallback(ChatEventHandler, "SayLine", function(message)
            local value = data["SayChat"][message]
            if value then
                if type(value) == "table" then
                    innerbar:SetTimer(value.Countdown, value.Countdown, value["Event_Keyword"] and (function() self:FireTrigger(value["Event_Keyword"]) end))
                    if innerbar.highlight then innerbar.highlight:Toggle(not(data["HighlightWhileOff"])) end
                    self.icons[innerbar] = true
                    if not(self.updateCallback) then self.updateCallback = AddCallback(Updater, "ExecuteUpdate", self.ExecuteUpdate) end
                else
                    innerbar:SetTimer(value, value)
                    if innerbar.highlight then innerbar.highlight:Toggle(not(data["HighlightWhileOff"])) end
                    self.icons[innerbar] = true
                    if not(self.updateCallback) then self.updateCallback = AddCallback(Updater, "ExecuteUpdate", self.ExecuteUpdate) end
                end
            end
        end))
    end

    if tablelength(data["PlayerHeal"]) > 0 then
        table.insert(self.playerHealCallbacks, AddCallback(ChatEventHandler, "PlayerHeal", function(args)
            local value = data["PlayerHeal"][args.SkillName]
            if value then
                if type(value) == "table" then
                    innerbar:SetTimer(value.Countdown, value.Countdown, value["Event_Keyword"] and (function() self:FireTrigger(value["Event_Keyword"]) end))
                    if innerbar.highlight then innerbar.highlight:Toggle(not(data["HighlightWhileOff"])) end
                    self.icons[innerbar] = true
                    if not(self.updateCallback) then self.updateCallback = AddCallback(Updater, "ExecuteUpdate", self.ExecuteUpdate) end
                else
                    innerbar:SetTimer(value, value)
                    if innerbar.highlight then innerbar.highlight:Toggle(not(data["HighlightWhileOff"])) end
                    self.icons[innerbar] = true
                    if not(self.updateCallback) then self.updateCallback = AddCallback(Updater, "ExecuteUpdate", self.ExecuteUpdate) end
                end
            end
        end))
    end

    if tablelength(data["PlayerBenefit"]) > 0 then
        table.insert(self.playerBenefitCallbacks, AddCallback(ChatEventHandler, "PlayerBenefit", function(args)
            local value = data["PlayerBenefit"][args.SkillName]
            if value then
                if type(value) == "table" then
                    innerbar:SetTimer(value.Countdown, value.Countdown, value["Event_Keyword"] and (function() self:FireTrigger(value["Event_Keyword"]) end))
                    if innerbar.highlight then innerbar.highlight:Toggle(not(data["HighlightWhileOff"])) end
                    self.icons[innerbar] = true
                    if not(self.updateCallback) then self.updateCallback = AddCallback(Updater, "ExecuteUpdate", self.ExecuteUpdate) end
                else
                    innerbar:SetTimer(value, value)
                    if innerbar.highlight then innerbar.highlight:Toggle(not(data["HighlightWhileOff"])) end
                    self.icons[innerbar] = true
                    if not(self.updateCallback) then self.updateCallback = AddCallback(Updater, "ExecuteUpdate", self.ExecuteUpdate) end
                end
            end
        end))
    end


    if tablelength(data["Skills"]) > 0 then
        self.skills = true
        for key, value in pairs(data["Skills"]) do
            if SkillEventHandler:SkillExists(key) then
                -- Add the callback for this skill slot
                if not(self.skillCallbacks[key]) then self.skillCallbacks[key] = {} end
                table.insert(self.skillCallbacks[key], AddCallback(SkillEventHandler, key, function(skill) 
                    if type(value) ==  "table" then
                        if innerbar.highlight then innerbar.highlight:Toggle(not(data["HighlightWhileOff"])) end
                        local time = value.Countdown
                        local maxTime = value.Countdown
                        if value.Countdown == true then
                            time = skill:GetResetTime() - Turbine.Engine.GetGameTime()
                            maxTime = skill:GetCooldown()
                        end
                        if (skill:GetResetTime() - Turbine.Engine.GetGameTime()) + 1 <= maxTime and value.Countdown ~= true then
                            return
                        end
                        innerbar:SetTimer(time, maxTime, value["Event_Keyword"] and (function() self:FireTrigger(value["Event_Keyword"]) end))
                        self.icons[innerbar] = true
                        if not(self.updateCallback) then self.updateCallback = AddCallback(Updater, "ExecuteUpdate", self.ExecuteUpdate) end
                    else
                        if innerbar.highlight then innerbar.highlight:Toggle(not(data["HighlightWhileOff"])) end
                        local time = value
                        local maxTime = value
                        if value == true then
                            time = skill:GetResetTime() - Turbine.Engine.GetGameTime()
                            maxTime = skill:GetCooldown()
                        end
                        if time + 1 <= maxTime then
                            return
                        end
                        innerbar:SetTimer(time, maxTime)
                        self.icons[innerbar] = true
                        if not(self.updateCallback) then self.updateCallback = AddCallback(Updater, "ExecuteUpdate", self.ExecuteUpdate) end
                    end
                end))
            end
        end
    end
    
    self.currentHeight = self.currentHeight + (data["Height"] + data["BorderSize"] * 2) * data["Scale"]

    table.insert(self.UI, valuebar)
    table.insert(self.tools, innerbar)
end