function PrimeUI:ValueBar(data, parent)
    local valuebar = Turbine.UI.Control()
    valuebar:SetParent(parent)
    if self.groupMode then
        valuebar:SetPosition(data["X"], data["Y"])
        valuebar.moveBox = MoveBox(valuebar, data["Width"] + 2 * data["BorderSize"], data["Height"] + 2 * data["BorderSize"], data["NickName"] or "valuebar")
        valuebar.savePosition = function()
            data["X"], data["Y"] = valuebar:GetPosition()
        end
    else
        valuebar:SetPosition(self:ContentAlignement(data["BarAlignment"], data["Width"] + 2 * data["BorderSize"]) + (data.OffsetX or 0), self.currentHeight)
    end
    valuebar:SetSize(data["Width"] + 2 * data["BorderSize"], data["Height"] + 2 * data["BorderSize"])
    valuebar:SetMouseVisible(false)
    valuebar:SetBackColor(self:ColourTupleToTurbineColour(data["BorderColour"]))

    -- args (parent, width, height, x, y, barAlignment, barColour, backgroundColour, maxValue, barName, vertical, afterAnimation)
    valuebar.bar = ClassSpecificTools.ValueBar(valuebar, data["Width"], data["Height"], data["BorderSize"], data["BorderSize"], 
    self:AlignmentToNumberSmall(data["Alignment"]), self:ColourTupleToTurbineColour(data["ForegroundColour"]), self:ColourTupleToTurbineColour(data["BackgroundColour"]), 20, data["Texture"], data["Vertical"], nil, data["Visibility_Threshold"], data.StopAutoUpdate)
    self.lookup[data.NickName or ""] = valuebar.bar
    if tablelength(data["EventTrigger"]) > 0 then
        table.insert(self.eventTriggerCallbacks, AddCallback(self, "EventTrigger", 
        function(event)
            local time = data["EventTrigger"][event]
            if time then
                if type(time) == "table" then
                    valuebar.bar:SetMaxValue(time.Countdown)
                    valuebar.bar:SetCurrentValue(time.Countdown)
                else
                    valuebar.bar:SetMaxValue(time)
                    valuebar.bar:SetCurrentValue(time)
                end
            end
        end))
    end

    if tablelength(data["EffectAdded"]) > 0 then
        table.insert(self.effectAddedCallbacks, AddCallback(PlayerEffectsHandler, "EffectAdded", 
        function(effect)
            local time = data["EffectAdded"][effect:GetName()]
            if time then
                if type(time) == "table" then
                    local maxTime = time.Countdown
                    local currTime = time.Countdown
                    if currTime == true then
                        currTime = effect:GetDuration() - (Turbine.Engine.GetGameTime() - effect:GetStartTime())
                        maxTime = effect:GetDuration()
                    end
                    valuebar.bar:SetMaxValue(maxTime)
                    valuebar.bar:SetCurrentValue(currTime)
                else
                    local maxTime = time
                    if time == true then
                        time = effect:GetDuration() - (Turbine.Engine.GetGameTime() - effect:GetStartTime())
                        maxTime = effect:GetDuration()
                    end
                    valuebar.bar:SetMaxValue(maxTime)
                    valuebar.bar:SetCurrentValue(time)
                end
            end
        end))
    end

    if tablelength(data["EffectAdded_Target"]) > 0 then
        table.insert(self.effectTargetAddedCallbacks, TargetHandlerObject:AddEffectCallback( 
        function(args)
            local time = data["EffectAdded_Target"][args.Name]
            if time then
                if type(time) == "table" then
                    local maxTime = time.Countdown
                    local currTime = time.Countdown
                    if currTime == true then
                        currTime = args.Duration - (Turbine.Engine.GetGameTime() - args.StartTime)
                        maxTime = args.Duration
                    end
                    valuebar.bar:SetMaxValue(maxTime)
                    valuebar.bar:SetCurrentValue(currTime)
                else
                    local maxTime = time
                    if time == true then
                        time = args.Duration - (Turbine.Engine.GetGameTime() - args.StartTime)
                        maxTime = args.Duration
                    end
                    valuebar.bar:SetMaxValue(maxTime)
                    valuebar.bar:SetCurrentValue(time)
                end
            end
        end))
    end

    if tablelength(data["EffectAdded_Group"]) > 0 then
        table.insert(self.effectAddedGroupCallbacks, AddCallback(GroupEffectsHandler, "EffectAdded", 
        function(args)
            local time = data["EffectAdded_Group"][args.effect.Name]
            if time then
                if type(time) == "table" then
                    local maxTime = time.Countdown
                    local currTime = time.Countdown
                    if currTime == true then
                        currTime = args.effect.Duration - (Turbine.Engine.GetGameTime() - args.effect.StartTime)
                        maxTime = args.effect.Duration
                    end
                    valuebar.bar:SetMaxValue(maxTime)
                    valuebar.bar:SetCurrentValue(currTime)
                else
                    local maxTime = time
                    if time == true then
                        time = args.effect.Duration - (Turbine.Engine.GetGameTime() - args.effect.StartTime)
                        maxTime = args.effect.Duration
                    end
                    valuebar.bar:SetMaxValue(maxTime)
                    valuebar.bar:SetCurrentValue(time)
                end
            end
        end))
    end

    if tablelength(data["EffectRemoved"]) > 0 then
        table.insert(self.effectRemovedCallbacks, AddCallback(PlayerEffectsHandler, "EffectRemoved", 
        function(effect)
            local time = data["EffectRemoved"][effect:GetName()]
            if time then
                if type(time) == "table" then
                    local maxTime = time.Countdown
                    local currTime = time.Countdown
                    if currTime == true then
                        currTime = effect:GetDuration() - (Turbine.Engine.GetGameTime() - effect:GetStartTime())
                        maxTime = effect:GetDuration()
                    end
                    valuebar.bar:SetMaxValue(maxTime)
                    valuebar.bar:SetCurrentValue(currTime)
                else
                    local maxTime = time
                    if time == true then
                        time = effect:GetDuration() - (Turbine.Engine.GetGameTime() - effect:GetStartTime())
                        maxTime = effect:GetDuration()
                    end
                    valuebar.bar:SetMaxValue(maxTime)
                    valuebar.bar:SetCurrentValue(time)
                end
            end
        end))
    end

    if tablelength(data["Skills"]) > 0 then
        self.skills = true
        for key, value in pairs(data["Skills"]) do
            if SkillEventHandler:SkillExists(key) then
                -- Add the callback for this skill slot
                if not(self.skillCallbacks[key]) then self.skillCallbacks[key] = {} end
                table.insert(self.skillCallbacks[key], AddCallback(SkillEventHandler, key, function(skill) 
                    if type(value) == "table" then
                        local time = value.Countdown
                        local maxTime = value.Countdown
                        if time == true then
                            time = skill:GetResetTime() - Turbine.Engine.GetGameTime()
                            maxTime = skill:GetCooldown()
                        end
                        if (skill:GetResetTime() - Turbine.Engine.GetGameTime()) + 1 <= maxTime then
                            return
                        end
                        valuebar.bar:SetMaxValue(maxTime)
                        valuebar.bar:SetCurrentValue(time)
                    else
                        local time = value
                        local maxTime = value
                        if time == true then
                            time = skill:GetResetTime() - Turbine.Engine.GetGameTime()
                            maxTime = skill:GetCooldown()
                        end
                        if time + 1 <= maxTime then
                            return
                        end
                        valuebar.bar:SetMaxValue(maxTime)
                        valuebar.bar:SetCurrentValue(time)
                    end
                end))
            end
        end
    end

    if tablelength(data["EnemyHits"]) > 0 then
        table.insert(self.enemyChatCallback, AddCallback(ChatEventHandler, "EnemyCombat", function(args)
            local value = data["EnemyHits"][args.SkillName]
            if value then
                if type(value) == "table" then
                    valuebar.bar:SetMaxValue(value.Countdown)
                    valuebar.bar:SetCurrentValue(value.Countdown)
                else
                    valuebar.bar:SetMaxValue(value)
                    valuebar.bar:SetCurrentValue(value)
                end
            end
        end))

    end

    if tablelength(data["PlayerHits"]) > 0 then
        table.insert(self.playerChatCallback, AddCallback(ChatEventHandler, "PlayerCombat", function(args)
            local value = data["PlayerHits"][args.SkillName]
            if value then
                if type(value) == "table" then
                    valuebar.bar:SetMaxValue(value.Countdown)
                    valuebar.bar:SetCurrentValue(value.Countdown)
                else
                    valuebar.bar:SetMaxValue(value)
                    valuebar.bar:SetCurrentValue(value)
                end
            end
        end))
    end

    if tablelength(data["PlayerHeal"]) > 0 then
        table.insert(self.playerHealCallbacks, AddCallback(ChatEventHandler, "PlayerHeal", function(args)
            local value = data["PlayerHeal"][args.SkillName]
            if value then
                if type(value) == "table" then
                    valuebar.bar:SetMaxValue(value.Countdown)
                    valuebar.bar:SetCurrentValue(value.Countdown)
                else
                    valuebar.bar:SetMaxValue(value)
                    valuebar.bar:SetCurrentValue(value)
                end
            end
        end))
    end

    if tablelength(data["PlayerBenefit"]) > 0 then
        table.insert(self.playerBenefitCallbacks, AddCallback(ChatEventHandler, "PlayerBenefit", function(args)
            local value = data["PlayerBenefit"][args.SkillName]
            if value then
                if type(value) == "table" then
                    valuebar.bar:SetMaxValue(value.Countdown)
                    valuebar.bar:SetCurrentValue(value.Countdown)
                else
                    valuebar.bar:SetMaxValue(value)
                    valuebar.bar:SetCurrentValue(value)
                end
            end
        end))
    end

    if tablelength(data["StandardChat"]) > 0 then
        table.insert(self.standardChatCallback, AddCallback(ChatEventHandler, "Standard", function(message)
            local value = data["StandardChat"][message]
            if value then
                if type(value) == "table" then
                    valuebar.bar:SetMaxValue(value.Countdown)
                    valuebar.bar:SetCurrentValue(value.Countdown)
                else
                    valuebar.bar:SetMaxValue(value)
                    valuebar.bar:SetCurrentValue(value)
                end
            end
        end))
    end

    if tablelength(data["PartyChat"]) > 0 then
        table.insert(self.partyChatCallback, AddCallback(ChatEventHandler, "Party", function(message)
            local value = data["PartyChat"][message]
            if value then
                if type(value) == "table" then
                    valuebar.bar:SetMaxValue(value.Countdown)
                    valuebar.bar:SetCurrentValue(value.Countdown)
                else
                    valuebar.bar:SetMaxValue(value)
                    valuebar.bar:SetCurrentValue(value)
                end
            end
        end))
    end

    if tablelength(data["SayChat"]) > 0 then
        table.insert(self.sayChatCallback, AddCallback(ChatEventHandler, "SayLine", function(message)
            local value = data["SayChat"][message]
            if value then
                if type(value) == "table" then
                    valuebar.bar:SetMaxValue(value.Countdown)
                    valuebar.bar:SetCurrentValue(value.Countdown)
                else
                    valuebar.bar:SetMaxValue(value)
                    valuebar.bar:SetCurrentValue(value)
                end
            end
        end))
    end
    
    self.currentHeight = self.currentHeight + data["Height"] + 2 * data["BorderSize"]

    table.insert(self.UI, valuebar)
    table.insert(self.tools, valuebar.bar)
end