PrimeUITextDropDown = class(Turbine.UI.Window)
function PrimeUITextDropDown:Constructor(parent, infoWindow)
    Turbine.UI.Window.Constructor(self)
    self.height = 400
    self.width = 400
    self.infoWindow = infoWindow
    if parent then
        self:SetParent(parent)
    end
    self:SetVisible(false)
    self:SetSize(self.width, self.height)
    self:SetMouseVisible(false)
    self:SetBackColor(Turbine.UI.Color(0, 0, 0))
    self.dropdown = UI.DropDownMenuControl(self, self.height, "Text keyword dropdown")
    self.dropdown.VisibleChanged = function()
    end
    self.dropdown.closeButton.MouseClick = function()
        self.dropdown:SetVisible(false)
        self:SetVisible(false)
    end
    self.VisibleChanged = function()
        self.dropdown:SetVisible(self:IsVisible())
        if self:IsVisible() then
            local x, y = self:GetMousePosition()
            local oldX, oldY = self:GetPosition()
            self:SetPosition((x + oldX) - 200, y + oldY + 5)
        end
        self:SetZOrder(2)
    end

    self.info = {
        -- Instance
        ["Instance"] = {
            ["Instance_Name"] = {"Name of the instance or all"},
            ["Instance_Tier"] = {"Tier of instance or \"\""},
            ["Boss_Name"] = {"Name of the boss fight"},
            ["Boss_Tier"] = {"Tier of the boss fight"},
            ["Boss_Completion_Time"] = {"Fight duration (wipe or completion)"},
            ["Boss_Wipes"] = {"Amount of wipes in a fight"},
        },

        -- Info
        ["Player Info"] = {
            ["Player_Name"] = {"Name of the player"},
            ["Player_Class"] = {"Class of the player"},
            ["Location"] = {"Current location of the player"},
        },

        -- Stats
        ["Player stats"] = {
            ["Death_Count_Character"] = {"Amount of deaths on character"},
            ["Death_Count_Account"] = {"Amount of deaths on account"},
            ["Death_Count_Misadventure_Character"] = {"Amount of Misadventure deaths on character"},
            ["Death_Count_Misadventure_Account"] = {"Amount of Misadventure deaths on account"},
        },

        -- Player
        ["Player"] = {
            ["Player_Morale"] = {"The current amount of morale of the player"},
            ["Player_Max_Morale"] = {"The maximum amount of morale of the player"},
            ["Player_Base_Max_Morale"] = {"The maximum amount of base morale of the player"},
            ["Player_Morale_Percentage"] = {"The percentage of current morale/max morale"},
            ["Player_Temporary_Morale"] = {"The current amount of temporary morale"},
            ["Player_Max_Temporary_Morale"] = {"The maximum amount of temporary morale"},
            ["Player_Power"] = {"The current amount of power of the player"},
            ["Player_Max_Power"] = {"The maximum amount of power of the player"},
            ["Player_Base_Max_Power"] = {"The maximum amount of base power of the player"},
            ["Player_Power_Percentage"] = {"The percentage of current power/max power"},
            ["Player_Temporary_Power"] = {"The current amount of temporary power"},
            ["Player_Max_Temporary_Power"] = {"The maximum amount of temporary power"},
            ["Player_Level"] = {"The current level of the player"},
            ["Player_Is_In_Combat"] = {"The player is in combat"},
        },

        -- Target
        ["Target"] = {
            ["Target_Name"] = {"Name of your current target or nothing if no target"},
            ["Target_Morale"] = {"Amount of morale of the target"},
            ["Target_Max_Morale"] = {"The amount of max morale of the target"},
            ["Target_Base_Max_Morale"] = {"The amount of max base morale of the target"},
            ["Target_Temporary_Morale"] = {"The amount of current temprorary morale of the target"},
            ["Target_Max_Temporary_Morale"] = {"The amount of maximum temprorary morale of the target"},
            ["Target_Power"] = {"The amount of current power of the target"},
            ["Target_Max_Power"] = {"The amount of maximum power of the target"},
            ["Target_Base_Max_Power"] = {"The amount of maximum base power of the target"},
            ["Target_Temporary_Power"] = {"The amount of current temporary power of the target"},
            ["Target_Max_Temporary_Power"] = {"The amount of maximum temporary power of the target"},
            ["Target_Level"] = {"The level of your target"},
        },

        -- Pet
        ["Pet"] = {
            ["Pet_Morale"] = {"The current morale of a pet if it exists"},
            ["Pet_Max_Morale"] = {"The maximum morale of a pet if it exists"},
            ["Pet_Morale_Percentage"] = {"The current morale/maximum morale percentage of a pet if it exists"},
            ["Pet_Temporary_Morale"] = {"The current temporary morale of a pet if it exists"},
            ["Pet_Max_Temporary_Morale"] = {"The max temporary morale of a pet if it exists"},
            ["Pet_Name"] = {"The name of a pet if it exists"},
        },

        -- Delving
        ["Delving"] = {
            ["Delving_Tier"] = {"The tier of a currently active delving"},
            ["Mission_Name"] = {"The name of the mission of the currently active delving"},
        },

        -- PvP
        ["PvP"] = {
            ["Killstreak"] = {"The currently active killstreak"},
            ["Daily_KBs"] = {"The amount of pvp KBs made this day"},
            ["Daily_Deaths"] = {"The amount of pvp deaths made this day"},
            ["Last_KB"] = {"The name of the last player you scored a killing blow on"},
            ["Last_KB_KC"] = {"The amount of killing blows you scored against your last target killed"},
            ["Last_KB_Deaths"] = {"The amount of deaths you suffered at the hand of your last target killed"},
        },   
        --[""] = {""},
    }

    for key, value in pairs(self.info) do
        self.dropdown:AddItem(string.format("Category: %s(%s)", key, tablelength(value)), function ()
            
        end)
        for keyword, keywordInfo in pairs(value) do
            self.dropdown:AddItem(keyword, function()
                self.textbox:SetText(string.format("%s%s%s%s", self.textbox:GetText(), "<", keyword, ">"))
            end, keywordInfo[1], self.infoWindow)
        end
    end
end

function PrimeUITextDropDown:SetTextbox(textbox)
    self.textbox = textbox
end