
PrimeUISkillLabel = class(Turbine.UI.Window)
function PrimeUISkillLabel:Constructor(parent, x, y, width, height, scale, icon, borderColour, borderSize, reverse, visibitilityThreshold, hideWhileOff, hideWhileOn, opacityOn, opacityOff, highlightColour, higlightSize, highlightWhileOff, highlight, visualTimer)
    Turbine.UI.Window.Constructor(self)

    self.width = width
    self.height = height
    self.borderSize = borderSize
    self.scale = scale
    self.reverse = reverse
    self.icon = icon
    self.shade = visualTimer
    self.visibitilityThreshold = visibitilityThreshold
    self.hideWhileOff = hideWhileOff
    self.hideWhileOn = hideWhileOn
    self.opacityOff = opacityOff or 1
    self.opacityOn = opacityOn or 1
    self.highlightWhileOff = highlightWhileOff
    self.highlight = highlight

    self.normalOpacity = normalOpacity
    self.usageOpacity = usageOpacity
    self.hideOnCreation = hideOnCreation

    self.textFunction = StringifyDuration

    self:SetParent(parent)
    self:SetPosition(x, y)
    self:SetSize(self.width + 2 * self.borderSize, self.height + 2 * self.borderSize)
    self:SetBackColor(borderColour)
    self:SetVisible(not(self.hideWhileOff))
    self:SetMouseVisible(false)
    self:SetOpacity(self.opacityOff)

    self.currentDuration = 0
    self.maxDuration = 0
    self.baseImageID = 0x41007E35

    self.IconLabel = Turbine.UI.Label()
    self.IconLabel:SetParent(self)
    self.IconLabel:SetSize(self.width, self.height)
    --self.IconLabel:SetStretchMode(1)
    self.IconLabel:SetBackground(BackgroundHandler:GetBackground(tonumber(self.icon) or self.icon))
    self.IconLabel:SetPosition(self.borderSize, self.borderSize)
    self.IconLabel:SetBlendMode(Turbine.UI.BlendMode.GrayScale)
    self.IconLabel:SetMouseVisible(false)
    

    if self.shade then
        self.elapsedOverlay = Turbine.UI.Control()
        self.elapsedOverlay:SetStretchMode( 1 )
        self.elapsedOverlay:SetMouseVisible( false )
        --self.elapsedOverlay:SetZOrder( 99 )
        self.elapsedOverlay:SetBackColor( Turbine.UI.Color( 1, 0, 0, 0 ) )
        self.elapsedOverlay:SetBackColorBlendMode( Turbine.UI.BlendMode.Overlay )
        self.elapsedOverlay:SetParent( self )
        self.elapsedOverlay:SetSize(self.width + 2 + self.borderSize, self.height + 2  + self.borderSize)
        self.elapsedOverlay:SetMouseVisible(false)
    end

    self.textLabel = Turbine.UI.Label()
    self.textLabel:SetParent(self)
    self.textLabel:SetSize(self.width  + self.borderSize, self.height  + self.borderSize)
    self.textLabel:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
    --self.textLabel:SetFont(Turbine.UI.Lotro.Font.TrajanProBold16)
    self.textLabel:SetFont(Turbine.UI.Lotro.Font.FixedSys15)
    self.textLabel:SetOutlineColor(Turbine.UI.Color(0,0,0))
    self.textLabel:SetFontStyle(Turbine.UI.FontStyle.Outline)
    self.textLabel:SetMouseVisible(false)
    --self.textLabel:SetPosition(-2, -2)
    self.textLabel:SetStretchMode(1)
    self.textLabel:SetMultiline(false)

    self.highlightFrame = ClassSpecificTools.HighlightFrame3(self, 0, 0, (self.width + 2 * self.borderSize) * self.scale, (self.height + 2 * self.borderSize) * self.scale, highlightColour, higlightSize)
    --self.highlightFrame:SetPosition(self.borderSize,  self.borderSize)
    self.highlightFrame:Toggle(self.highlightWhileOff)


    self:SetStretchMode(1)
    self:SetSize((self.width + 2 * self.borderSize) * self.scale, (self.height + 2 * self.borderSize) * self.scale)


    self.duration = 0
    --self.textLabel:SetText("0")

    self.ExecuteUpdate = function(delta)

        --Update time
        self.currentDuration = self.currentDuration - delta

        self:SetVisible(self.currentDuration <= self.visibitilityThreshold and not(self.hideWhileOn))

        if self.currentDuration < 0 then
            if self.callback then RemoveCallback(Updater, "ExecuteUpdate", self.callback) end
            self.callback = nil
            self.textLabel:SetStretchMode(nil)
            self.textLabel:SetStretchMode(1)
            self.textLabel:SetText("")
            if self.shade then
                self.elapsedOverlay:SetVisible(false)
                self.elapsedOverlay:SetOpacity(self.opacityOff)
            end
            self:SetOpacity(self.opacityOff)
            if self.highlight and self.highlightFrame then
                self.highlightFrame:Toggle(false)
            end

            if self.highlightWhileOff and self.highlightFrame then
                self.highlightFrame:Toggle(true)
            end
            

            self:SetVisible(not(self.hideWhileOff))
            return
        end

        -- Visual timer
        if self.shade then
            self.elapsedOverlay:SetVisible(true)
            local percentComplete = math.min(59, math.ceil(60 * self.currentDuration / self.maxDuration))
    
            if self.currentPercentage ~= percentComplete then 
    
                self.currentPercentage = percentComplete
    
                local imageId = self.baseImageID + self.currentPercentage            
    
                self.elapsedOverlay:SetStretchMode(3)
                self.elapsedOverlay:SetSize(32, 32)
                self.elapsedOverlay:SetBackground(BackgroundHandler:GetBackground(imageId))
                self.elapsedOverlay:SetStretchMode(1)
                self.elapsedOverlay:SetSize(self.width + 2 + self.borderSize, self.height + 2 + self.borderSize)
            end
        end

        -- Update timer label
        self.textLabel:SetStretchMode(nil)
        self.textLabel:SetStretchMode(1)
        self.textLabel:SetText(self.textFunction(self.currentDuration))
    end
end

function PrimeUISkillLabel:SetTimer(time, maxTime)
    self.currentDuration = time
    self.maxDuration = maxTime
    self:SetOpacity(self.opacityOn)

    if self.highlightWhileOff and self.highlightFrame then
        self.highlightFrame:Toggle(false)
    end

    if self.highlight and self.highlightFrame then
        self.highlightFrame:Toggle(true)
    end


    self:SetVisible(self.currentDuration <= self.visibitilityThreshold and not(self.hideWhileOn))

    if self.shade then
        self.elapsedOverlay:SetVisible(true)
        self.elapsedOverlay:SetOpacity(self.opacityOn)
        self.elapsedOverlay:SetStretchMode(3)
        self.elapsedOverlay:SetSize(32, 32)
        self.elapsedOverlay:SetBackground(BackgroundHandler:GetBackground(self.baseImageID + math.min(59, math.ceil(60 * self.currentDuration / self.maxDuration))))
        self.elapsedOverlay:SetStretchMode(1)
        self.elapsedOverlay:SetSize(self.width + 2 + self.borderSize, self.height + 2 + self.borderSize)
    end

    if not(self.callback) then self.callback = AddCallback(Updater, "ExecuteUpdate", self.ExecuteUpdate) end
end

function PrimeUISkillLabel:Unload()
    self:SetParent(nil)
end

function PrimeUISkillLabel:ToggleHighlight(value)
    self.highlightFrame:Toggle(value)
end