Timer = class(Turbine.UI.Window)
function Timer:Constructor(parent, width, height, effect, message, font, colourScheme, showDuration, optionalDuration, optionalIcon, hide, durationFont, spark, borderSize, borderColour, opacity, increasing, useWarning, warningThreshold)
    Turbine.UI.Window.Constructor(self)
    if parent then
        self:SetParent(parent)
    end

    self:SetVisible(true)
    self:SetMouseVisible(false)
    self:SetSize(width, height)
    self:SetOpacity(opacity or 1)
    self:SetWantsUpdates(false)

    self.backgroundColour = colourScheme[2]
    self.warningColour = colourScheme[6]
    self.useWarning = useWarning
    self.warningThreshold = warningThreshold
    self.borderSize = borderSize or 1
    self.maxTimerWidth = width - height - self.borderSize * 2
    self.borderColour = borderColour or Turbine.UI.Color(1, 0, 0, 0)
    self.timerHeight = height
    self.maxTimerHeight = self.timerHeight - 2 * self.borderSize
    self.showDuration = not(showDuration)
    self.hide = hide
    if self.hide == nil then self.hide = false end

    self.increasing = increasing

    self.sparkOffset = 32 - 20 / 32 * 32 - 3
    self.height = height
    self.message = message
    self.sparkEnabled = spark

    self.iconHeight = height - 2 * self.borderSize

    self.font = font
    self.durationFont = durationFont
    if not(self.durationFont) then
        self.durationFont = Turbine.UI.Lotro.Font.FixedSys15
    end

    self.textFunction = StringifyDuration
    

    self.borderWindow = Turbine.UI.Control()
    self.borderWindow:SetParent(self)
    self.borderWindow:SetVisible(true)
    self.borderWindow:SetMouseVisible(false)
    self.borderWindow:SetPosition(height, 0)
    self.borderWindow:SetSize(width - height, height)
    self.borderWindow:SetBackColor(self.borderColour)

    -- Background UI
    self.opacityWindow = Turbine.UI.Control()
    self.opacityWindow:SetParent(self)
    self.opacityWindow:SetVisible(true)
    self.opacityWindow:SetMouseVisible(false)
    self.opacityWindow:SetPosition(0, self.borderSize)
    self.opacityWindow:SetSize(width - self.borderSize, height - 2 * self.borderSize)
    self.opacityWindow:SetBackColor(colourScheme[1])

    if self.sparkEnabled then
        self.spark = Turbine.UI.Window()
        self.spark:SetParent(self)
        self.spark:SetVisible(true)
        self.spark:SetPosition(0, 0)
        self.spark:SetOpacity(0.4)
        self.spark:SetSize(32, 32)
        self.spark:SetBackColorBlendMode(Turbine.UI.BlendMode.Multiply)
        self.spark:SetBackground(BackgroundHandler:GetBackground("PrimePlugins/RaidTools/Images/barSpark.tga"))
        self.spark:SetStretchMode(1)
        self.spark:SetSize(height, height)
        self.spark:SetMouseVisible(false)
        self.spark:SetPosition(width + (32 / height) * 16, 0)
    end

    self.timerBar = Turbine.UI.Control()
    self.timerBar:SetParent(self.opacityWindow)
    self.timerBar:SetVisible(true)
    self.timerBar:SetPosition(height, 0)
    self.timerBar:SetSize(self.maxTimerWidth, self.maxTimerHeight)
    self.timerBar:SetBackColor(colourScheme[3])
    self.timerBar:SetMouseVisible(false)


    -- Text UI
    self.textWindow = Turbine.UI.Window()
    self.textWindow:SetParent(self)
    self.textWindow:SetVisible(true)
    self.textWindow:SetPosition(0, 0)
    self.textWindow:SetSize(width - self.borderSize, height)
    self.textWindow:SetMouseVisible(false)

    self.effectIcon = Turbine.UI.Label()
    self.effectIcon:SetParent(self.textWindow)
    self.effectIcon:SetBackground(BackgroundHandler:GetBackground(tonumber(optionalIcon) or optionalIcon))
    self.effectIcon:SetStretchMode(2)
    self.effectIcon:SetStretchMode(1)
    self.effectIcon:SetSize(self.height, self.height)
    self.effectIcon:SetMouseVisible(false)
    self.effectIcon:SetVisible(true)

    self.textLabel = Turbine.UI.Label()
    self.textLabel:SetParent(self.textWindow)
    self.textLabel:SetPosition(2 + height, 0)
    self.textLabel:SetSize(width - height - height - 10, height)
    self.textLabel:SetVisible(true)
    self.textLabel:SetMouseVisible(false)
    self.textLabel:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft)
    self.textLabel:SetForeColor(colourScheme[4])
    self.textLabel:SetFont(font)
    self.textLabel:SetOutlineColor(Turbine.UI.Color.Black)
    self.textLabel:SetFontStyle(Turbine.UI.FontStyle.Outline)
    self.textLabel:SetMultiline(false)
    self.textLabel:SetText(self.message)

    self.durationLabel = Turbine.UI.Label()
    self.durationLabel:SetParent(self.textWindow)
    self.durationLabel:SetPosition(height, 0)
    self.durationLabel:SetSize(width - height - 5, height)
    self.durationLabel:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleRight)
    self.durationLabel:SetMouseVisible(false)
    self.durationLabel:SetForeColor(colourScheme[4])
    self.durationLabel:SetFont(self.durationFont)
    self.durationLabel:SetOutlineColor(Turbine.UI.Color.Black)
    self.durationLabel:SetFontStyle(Turbine.UI.FontStyle.Outline)
    self.durationLabel:SetMultiline(false)

    if optionalDuration then
        self.maxDuration = optionalDuration or self.increasing and 999999
        self.HasMaxDuration = self.maxDuration < 999999 
        self.divisionFactor = math.round(self.maxTimerWidth / self.maxDuration)
        self.number = self.increasing and 0 or optionalDuration
        self.permanent = false or self.increasing
        self:UpdateColour()
    else
        self.maxDuration = effect:GetDuration()
        self.divisionFactor = math.round(self.maxTimerWidth / self.maxDuration)

        if self.maxDuration > 99999 then 
            self.permanent = true 
            self.number = Turbine.Engine.GetGameTime() - effect:GetStartTime()
            self.timerBar:SetSize(self.maxTimerWidth,  self.maxTimerHeight)
            if self.sparkEnabled then
                self.spark:SetVisible(false)
            end
        else
            self.number = effect:GetDuration() - (Turbine.Engine.GetGameTime() - effect:GetStartTime())
            self:UpdateColour()
        end
    end
end



function Timer:GetRemainingTime()
    if self.permanent then
        if self.maxDuration < 999999 then
            return self.maxDuration - self.number
        else
            return 999999
        end
        
    else
        return(self.number)
    end
end

function Timer:GetDuration()
    return(self.maxDuration)
end

function Timer:UpdateColour()
    if self.useWarning and self.number < self.warningThreshold then
        local timerLeft = (self.warningThreshold - self.number) / self.warningThreshold
        self.opacityWindow:SetBackColor(Turbine.UI.Color(
            self.backgroundColour.A,
            math.abs(self.warningColour.R - self.backgroundColour.R) * timerLeft, 
            math.abs(self.warningColour.G - self.backgroundColour.G) * timerLeft, 
            math.abs(self.warningColour.B - self.backgroundColour.B) * timerLeft))
    end

    --self.timerBar:SetWidth(math.ceil(self.number * self.divisionFactor))--, self.timerHeight - 2 * self.borderSize)
    self.timerBar:SetWidth(self.maxTimerWidth * (self.number / self.maxDuration))

    if self.sparkEnabled then
        self.spark:SetPosition(self.number / self.maxDuration * self.maxTimerWidth + (self.height / 32 * 16), 0)
        self.spark:SetOpacity(math.abs(math.sin(self.number) * 0.55) + 0.3)
    end
end

function Timer:SetTimer(duration, maxDuration)
    self.number = self.increasing and 0 or duration
    self.maxDuration = maxDuration
    self.HasMaxDuration = self.maxDuration < 999999 
    self.divisionFactor = math.round(self.maxTimerWidth / self.maxDuration)

    if self.number < 0 then
        self.durationLabel:SetVisible(false)
        if self.hide then self:SetVisible(false) end
        return true
    end
    
    self.durationLabel:SetVisible(true)
    self:SetVisible(true)
    self:UpdateColour()
    if self.useWarning and duration > self.warningThreshold then self.opacityWindow:SetBackColor(self.backgroundColour) end
end

function Timer:ChangeDuration(newNumber, newMax)
    if newMax then
        self.maxDuration = newMax
        self.HasMaxDuration = self.maxDuration < 999999 
    end
    self.divisionFactor = math.round(self.maxTimerWidth / self.maxDuration)
    self.number = self.increasing and 0 or newNumber
    self.durationLabel:SetVisible(true)
    self:SetVisible(true)
    self:UpdateColour()
    if newNumber > 7 then self.opacityWindow:SetBackColor(self.backgroundColour) end
end

function Timer:UpdateText(delta)
    if self.showDuration then return end
    if self.permanent then
        self.number = self.number + delta
        self.durationLabel:SetText(self.textFunction(self.number))
        if  self.HasMaxDuration then
            
            self:UpdateColour()
        end
        if self.number > self.maxDuration then
            self.durationLabel:SetVisible(false)
            if self.hide then self:SetVisible(false) end
            return true
        end
    else
        self.number = self.number - delta

        self.durationLabel:SetText(self.textFunction(self.number))

        self:UpdateColour()

        if self.number < 0 then
            self.durationLabel:SetVisible(false)
            if self.hide then self:SetVisible(false) end
            return true
        end
    end
end

function Timer:StopTimer()
    self.number = 0
    self.maxDuration = 0
    self.HasMaxDuration = self.maxDuration < 999999 
    self.divisionFactor = math.round(self.maxTimerWidth / self.maxDuration)
    self.durationLabel:SetVisible(true)
    self:SetVisible(true)
    self:UpdateColour()
    self.durationLabel:SetVisible(false)
    if self.hide then self:SetVisible(false) end
    return true
end

function Timer:HideText(value)
    self.durationLabel:SetVisible(value)
end

function Timer:Round(num, rounding)
    local mult = 10^(rounding or 0)
    local number = math.floor(num * mult + 0.5) / mult
    if (number % 1) == 0 and rounding ~= 0 then
        return tostring(number .. ".0")
    else
        return number
    end
end

function Timer:ChangeName(newName)
    self.textLabel:SetText(newName)
end

function Timer:ChangeFrontOpacity(opacity)
    self.timerBar:SetOpacity(opacity)
end

function Timer:ChangeBackOpacity(opacity)
    self.opacityWindow:SetOpacity(opacity)
end

function Timer:ChangeText(message)
    self.message = message
    self.textLabel:SetText(message)
end

function Timer:ChangeIcon(icon)
    if self.effectIcon then
        if icon then
            self.effectIcon:SetStretchMode(nil)
            self.effectIcon:SetSize(32, 32)
            if tonumber(icon) then
                self.effectIcon:SetBackground(BackgroundHandler:GetBackground(tonumber(icon)))
            else
                self.effectIcon:SetBackground(BackgroundHandler:GetBackground(icon))
            end
            self.effectIcon:SetStretchMode(1)
            self.effectIcon:SetSize(self.height, self.height)
        end
    end
end

function Timer:GetText()
    return self.message
end

function Timer:Unload()
    self:Destroy()
end

function Timer:Destroy()
    self:SetVisible(false)
    local controlList = self:GetControls()
    for i = 1, controlList:GetCount() do
        controlList:RemoveAt(1)
    end
    self:SetParent(nil)
end