TimerGroupWindow = class(Turbine.UI.Window)
function TimerGroupWindow:Constructor(parent, width, height, x, y, sortOrder, timerheight, timerFont, durationFont, timerColours, staticTimers, timerOffset, timerDirection, spark, borderSize, borderColour, opacity, Timer_Header, Header_Text, Header_Only_When_Active, Header_Background_Colour, filterList, useFilter, useWarning, warningThreshold)
    Turbine.UI.Window.Constructor(self)
    self.width = width
    self.height = height
    self.x = x
    self.y = y
    self.timerHeight = timerheight
    self.font = timerFont
    self.colourScheme = timerColours
    self.sortOrder = sortOrder
    self.staticTimers = staticTimers
    self.timerOffset = timerOffset
    self.durationFont = durationFont
    self.spark = spark
    self.borderSize = borderSize
    self.borderColour = borderColour
    self.opacity = opacity or 1
    self.Timer_Header = Timer_Header
    self.Header_Text = Header_Text 
    self.Header_Only_When_Active = Header_Only_When_Active
    self.filterList = filterList or {}
    self.useFilter = useFilter
    self.useWarning = useWarning
    self.warningThreshold = warningThreshold

    self:SetParent(parent)
    self:SetSize(self.width, self.height)
    self:SetPosition(self.x, self.y)
    self:SetMouseVisible(false)
    self:SetVisible(true)
    self:SetOpacity(self.opacity)


    self.Timers = {}
    self.resort = false
    self.UniqueTimers = {}
    self.TieringTimers = {}
    self.StaticTimers = {}
    self.ActiveStaticTimers = {}
    self.count = 0

    self.offset = self.Timer_Header and self.timerHeight or 0
    if self.Timer_Header then
        self.InnerControl = Turbine.UI.Control()
        self.InnerControl:SetParent(self)
        self.InnerControl:SetVisible(true)
        self.InnerControl:SetSize(self.width, self.timerHeight)
        self.InnerControl:SetMouseVisible(false)
        self.InnerControl:SetBackColor(self.borderColour)
        self.InnerControl:SetVisible(not(self.Header_Only_When_Active))
    
        self.TextLabel = Turbine.UI.Label()
        self.TextLabel:SetParent(self.InnerControl)
        self.TextLabel:SetVisible(true)
        self.TextLabel:SetPosition(self.borderSize, self.borderSize)
        self.TextLabel:SetSize(self.width - self.borderSize * 2, self.timerHeight - self.borderSize * 2)
        self.TextLabel:SetMouseVisible(false)
        --self.TextLabel:SetForeColor(self.layoutTable.textFontColour)
        self.TextLabel:SetFont(self.font)
        self.TextLabel:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
        self.TextLabel:SetOutlineColor(Turbine.UI.Color.Black)
        self.TextLabel:SetFontStyle(Turbine.UI.FontStyle.Outline)
        self.TextLabel:SetText(self.Header_Text)
        self.TextLabel:SetBackColor(Header_Background_Colour)
    end

    if timerDirection == "Down" then
        self.placeTimers = function(timer)
            timer:SetPosition(0, self.offset + self.sortCounter * (self.timerHeight + self.timerOffset) )
            self.sortCounter = self.sortCounter + 1
        end
    else
        self.placeTimers = function(timer)
            timer:SetPosition(0, self.offset + self.height - (self.sortCounter + 1) * (self.timerHeight + self.timerOffset)) 
            self.sortCounter = self.sortCounter + 1
        end
    end

    if sortOrder == "Alphabetical" then
        self.ResortList = function()
            self.sortCounter = 0
            for _,v in spairs(self.Timers, function(t,a,b) return t[b][1]:GetText() > t[a][1]:GetText() end) do
                self.placeTimers(v[1])
            end
    
            self.resort = false
        end
    elseif sortOrder == "Alphabetical Reverse" then
        self.ResortList = function()
            self.sortCounter = 0
            for _,v in spairs(self.Timers, function(t,a,b) return t[b][1]:GetText() < t[a][1]:GetText() end) do
                self.placeTimers(v[1])
            end
    
            self.resort = false
        end
    elseif sortOrder == "Low to High (Remaining duration)" then
        self.ResortList = function()
            self.sortCounter = 0
            for _,v in spairs(self.Timers, function(t,a,b) return t[b][1]:GetRemainingTime() > t[a][1]:GetRemainingTime() end) do
                self.placeTimers(v[1])
            end
    
            self.resort = false
        end
    elseif sortOrder == "High to Low (Max duration)" then
        self.ResortList = function()
            self.sortCounter = 0
            for _,v in spairs(self.Timers, function(t,a,b) return t[b][1]:GetDuration() > t[a][1]:GetDuration() end) do
                self.placeTimers(v[1])
            end
    
            self.resort = false
        end
    elseif sortOrder == "Low to High (Max duration)" then
        self.ResortList = function()
            self.sortCounter = 0
            for _,v in spairs(self.Timers, function(t,a,b) return t[b][1]:GetDuration() > t[a][1]:GetDuration() end) do
                self.placeTimers(v[1])
            end
    
            self.resort = false
        end
    else -- High to Low (Remaining duration)
        self.ResortList = function()
            self.sortCounter = 0
            for _,v in spairs(self.Timers, function(t,a,b) return t[b][1]:GetRemainingTime() < t[a][1]:GetRemainingTime() end) do
                self.placeTimers(v[1])
            end
    
            self.resort = false
        end
    end

    if self.staticTimers then
        self.ExecuteUpdate = function(delta)    
            for key, value in pairs(self.ActiveStaticTimers) do 
                local value = self.Timers[key]
                if value[1]:UpdateText(delta) then 
                    if value[4] then
                        value[4]()
                    end
                    self.ActiveStaticTimers[key] = nil
                end
            end
        
            if self.resort then 
                if tablelength(self.ActiveStaticTimers) == 0 then 
                    if self.updateCallback then RemoveCallback(Updater, "ExecuteUpdate", self.updateCallback) end
                    self.updateCallback = nil
                    return
                end
                self.ResortList() 
            end
        end
    else
        self.ExecuteUpdate = function(delta)    
            for key, value in pairs(self.Timers) do 
                if value[1]:UpdateText(delta) then 
                    if value[4] then
                        value[4]()
                        self.Timers[key][4] = nil
                    end
                    self.UniqueTimers[value[2]] = nil
                    self.TieringTimers[value[3]] = nil
                    self.Timers[key][1]:Destroy()
                    self.Timers[key] = nil
                    self.resort = true
                    self.count = self.count - 1
                end
            end
        
            if self.resort then 
                if self.count == 0 then 
                    if self.updateCallback then RemoveCallback(Updater, "ExecuteUpdate", self.updateCallback) end
                    self.updateCallback = nil
                    self.Timers = nil
                    self.UniqueTimers = nil
                    self.TieringTimers = nil
                    collectgarbage()
                    self.Timers = {}
                    self.UniqueTimers = {}
                    self.TieringTimers = {}
                    return
                end
                self.ResortList() 
            end
        end
    end
end

-- Add a timer
function TimerGroupWindow:NewTimer(duration, message, unprocessedMessage, unique, icon, player, static, override, callbackFunction, optionalDuration, increasing) 
    if not(self.startMode and self.startValue or not(self.startMode)) then return end -- Check if active at the moment
    if self.useFilter and not(self.filterList[player]) then return end -- In the filter so we early out
    local originalMessage = message
    local tieringID = string.gsub(originalMessage, "%d+", "") .. player .. icon
    if self.StaticTimers[string.gsub(unprocessedMessage, "%d+", "") .. icon] then
        return
    end
    if string.match(originalMessage, "%d+") then
        if self.TieringTimers[tieringID] then
            local tieringID = self.TieringTimers[tieringID]
            if self.Timers[tieringID] then
                self.Timers[tieringID][1]:ChangeDuration(duration, duration)
                self.Timers[tieringID][1]:ChangeName(message)
                self.Timers[tieringID][2] = duration
                self.resort = true
                return
            end
        end
    end

    local ID = #self.Timers + 1
    local uniqueID = originalMessage .. icon
    if unique or override then
        if self.UniqueTimers[uniqueID] then
            if override then
                self.Timers[self.UniqueTimers[uniqueID]][1]:SetTimer(duration, duration)
                self.resort = true
            end
            return
        else
            if duration and duration > 0 then
                self.UniqueTimers[uniqueID] = ID
            end
        end
    end

    if (duration == nil or duration < 0) and not(increasing) then return end
    self.Timers[ID] = {Timer(self, self.width, self.timerHeight, nil, message, self.font, self.colourScheme, true, not(static) and duration or 0, icon, nil, self.durationFont, self.spark, self.borderSize, self.borderColour, self.opacity, increasing, self.useWarning, self.warningThreshold), uniqueID, tieringID, callbackFunction}
    
    if optionalDuration then
        self.Timers[ID][1]:SetTimer(duration, optionalDuration)
    end

    if string.match(originalMessage, "%d+") then
        if not(self.TieringTimers[tieringID]) then
            self.TieringTimers[tieringID] = ID
        end
    end

    if static then
        self.StaticTimers[string.gsub(unprocessedMessage, "%d+", "") .. icon] = ID
    end

    --ID = ID + 1
    self.count = self.count + 1
    self.resort = true
    if static then
        self.ResortList()
    else
        if not(self.updateCallback) then self.updateCallback = AddCallback(Updater, "ExecuteUpdate", self.ExecuteUpdate) end
    end

    if self.Timer_Header and self.Header_Only_When_Active then
        self.InnerControl:SetVisible(self.count > 0)
    end

    return ID
end

function TimerGroupWindow:StartStaticTimer(time, originalMessage, message, name, icon, iconValue, unique, maxTime)
    if self.useFilter and not(self.filterList[name]) then return end -- In the filter so we early out
    local ID = string.gsub(originalMessage, "%d+", "") .. iconValue

    local timerID = self.StaticTimers[ID]

    if unique and self.Timers[timerID][1]:GetRemainingTime() > 0 then
        return
    end

    self.Timers[timerID][1]:SetTimer(time, maxTime or time)
    self.Timers[timerID][1]:ChangeIcon(icon)
    self.Timers[timerID][1]:ChangeName(message)

    self.ActiveStaticTimers[timerID] = true

    if not(self.updateCallback) then self.updateCallback = AddCallback(Updater, "ExecuteUpdate", self.ExecuteUpdate) end

    return timerID
end

function TimerGroupWindow:Clear()
    if self.staticTimers then
        for key, value in pairs(self.Timers) do
            value[1]:StopTimer()
        end
        if self.updateCallback then RemoveCallback(Updater, "ExecuteUpdate", self.updateCallback) end
        self.updateCallback = nil
    else
        self:Unload()
    end

end

function TimerGroupWindow:DeleteTimer(ID)
    if self.staticTimers then
        self.ActiveStaticTimers[ID] = nil
        self.Timers[ID][1]:StopTimer()
    else
        local value = self.Timers[ID]
        if value then
            self.UniqueTimers[value[2]] = nil
            self.TieringTimers[value[3]] = nil
            self.Timers[ID][1]:Destroy()
            self.Timers[ID] = nil
            self.resort = true
            self.count = self.count - 1
        end
    end

    if self.Timer_Header and self.Header_Only_When_Active then
        self.InnerControl:SetVisible(self.count > 0)
    end
end

function TimerGroupWindow:Unload()
    
    if self.updateCallback then RemoveCallback(Updater, "ExecuteUpdate", self.updateCallback) end
    self.updateCallback = nil

    for key, value in pairs(self.Timers) do
        self.Timers[key][1]:Unload()
        self.Timers[key] = nil
    end

    self.UniqueTimers = nil
    self.TieringTimers = nil
    self.Timers = nil

    self.resort = false
    self.Timers = {}
    self.UniqueTimers = {}
    self.TieringTimers = {}
end

function TimerGroupWindow:SetupStartmode(value)
    self.startMode = value
end

function TimerGroupWindow:ToggleStartValue(value)
    self.startValue = value
end