ReadyWindow = class(Turbine.UI.Window)
function ReadyWindow:Constructor(parent)
    Turbine.UI.Window.Constructor(self)
    --self:SetSize(32, 32)
    self:SetBackColor(Turbine.UI.Color(0, 0, 0))
    self:SetVisible(false)
    --self:SetPosition(math.floor( Turbine.UI.Display.GetWidth() / 2 - 400), math.floor( Turbine.UI.Display.GetHeight() / 2 - 200))
    self:SetMouseVisible(true)
    self:SetWantsKeyEvents(true)
    --self:SetText("ReadyCheck")
    --self:SetWantsUpdates(false)

    self.KeyDown = function(sender, args)
        if args["Action"] == 145 then
            if self:IsVisible() then self:SetVisible(false) end
        end
    end

    self.player = Turbine.Gameplay.LocalPlayer.GetInstance()
    self.party = self.player:GetParty()

    self.HeaderBackground = Turbine.UI.Control()
    self.HeaderBackground:SetSize(500, 20)
    self.HeaderBackground:SetParent(self)
    self.HeaderBackground:SetBackColorBlendMode(Turbine.UI.BlendMode.Overlay)
    self.HeaderBackground:SetBackColor(Turbine.UI.Color(0.5, 0.25, 0.25, 0.25))
    self.HeaderBackground:SetMouseVisible(false)

    self.IconLabel = Turbine.UI.Window()
    self.IconLabel:SetParent(self)
    self.IconLabel:SetPosition(0, 0)
    self.IconLabel:SetBackground("PrimePlugins/RaidTools/Images/Primelogo.tga")
    --self.IconLabel:SetBackground("PrimePlugins/RaidTools/Images/Blood.tga")
    self.IconLabel:SetMouseVisible(false)
    self.IconLabel:SetVisible(true)
    --self.IconLabel:SetStretchMode(2)
    self.IconLabel:SetStretchMode(nil)
    self.IconLabel:SetSize(32, 32)
    self.IconLabel:SetStretchMode(1)
    self.IconLabel:SetSize(20, 20)

    self.HeaderLabel = Turbine.UI.Label()
    self.HeaderLabel:SetParent(self)
    self.HeaderLabel:SetSize(200, 20)
    self.HeaderLabel:SetPosition(20, 0)
    self.HeaderLabel:SetFont(Turbine.UI.Lotro.Font.TrajanProBold16)
    self.HeaderLabel:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft)
    self.HeaderLabel:SetText(string.format("Prime Readycheck 0/%s", self.party and self.party:GetMemberCount() or 1))
    self.HeaderLabel:SetMouseVisible(false)

    self.TimerLabel = Turbine.UI.Label()
    self.TimerLabel:SetPosition(0, 20)
    self.TimerLabel:SetParent(self)
    self.TimerLabel:SetBackColor(Turbine.UI.Color(0, 0.3, 0))
    self.TimerLabel:SetMouseVisible(false)
    self.TimerLabel:SetSize(420, 3)

    self.CloseLabel = Turbine.UI.Label()
    self.CloseLabel:SetParent(self)
    self.CloseLabel:SetSize(20, 20)
    self.CloseLabel:SetBackColor(Turbine.UI.Color(0.3, 0, 0))

    self.CloseLabel.MouseClick = function()
        self:SetVisible(false)
    end

    self.Moving = false

    self.MouseDown = function(sender,args)
        self.MoveX = args.X
        self.MoveY = args.Y
        self.Moving = true
    end
  
    self.MouseUp = function()
        self.Moving = false
    end
  
    self.MouseMove = function(sender,args)
        if self.Moving then
          local newLeft = self:GetLeft() - (self.MoveX - args.X)
          local newTop = self:GetTop() - (self.MoveY - args.Y)
          self:SetPosition(newLeft,newTop)
        end
    end

    self.timer = 10

    self.ExecuteUpdate = function(delta)
        self.timer = self.timer - delta
    
        if self.timer < 0 then
            self:SetVisible(false)
            if self.updateCallback then RemoveCallback(Updater, "ExecuteUpdate", self.updateCallback) end 
            self.updateCallback = nil
        end
    end

    self.parent = parent

    self.player = Turbine.Gameplay.LocalPlayer.GetInstance()
    self.party = self.player:GetParty()
    self.PartyMembers = {}

    -- status - 0 -> undecided | 1 -> not ready | 2 -> ready
    self.PartyStatus = {}
    self.checkSum = 0

    self.maxCheckSum = self.party:GetMemberCount() * 2

    -- Setup window
    --for j = 1, 12, 1 do
    for i = 1, self.party:GetMemberCount(), 1 do
        local player = self.party:GetMember(i)
        self.PartyMembers[player:GetName()] = ReadyLabelName(player, self, i)
        self.PartyStatus[player:GetName()] = 0
    end
    --end

    self:SetSize(480, 20 * 3 + 25)
    self:SetPosition(math.floor( Turbine.UI.Display.GetWidth() / 2 - self:GetWidth() / 2), math.floor( Turbine.UI.Display.GetHeight() / 2 - self:GetHeight() / 2))
    self.CloseLabel:SetPosition(self:GetWidth() - 20, 0)

    -- Prepare the window for the new readyCheck
    self.MakeNewReadyCheck = function()
        self:SetVisible(true)
        self.IconLabel:SetStretchMode(nil)
        self.IconLabel:SetSize(32, 32)
        self.IconLabel:SetStretchMode(1)
        self.IconLabel:SetSize(20, 20)
    end
end

function ReadyWindow:ClearOldReadyCheck()
    self.TimerLabel:SetVisible(true)
    self.checkSum = 0
    for key, value in pairs(self.PartyMembers) do
        self.PartyMembers[key]:SetDefault()
        self.PartyStatus[key] = 0
    end
end

function ReadyWindow:SetTimerBar(width)
    self.TimerLabel:SetSize(420 * width, 3)
end

-- We got an update from a partymember handle it
function ReadyWindow:UpdateReadycheckMember(player, state)
    if self.PartyMembers[player] ~= nil and self.PartyStatus[player] < 1 then 
        self.PartyMembers[player]:UpdateReadyState(state)
        if state then
            self.PartyStatus[player] = 2
            self.checkSum = self.checkSum + 2
        else 
            self.PartyStatus[player] = 1 
            self.checkSum = self.checkSum + 1
        end
    end

    local readyValue = 0
    for key, value in pairs(self.PartyStatus) do
        if value == 2 then
            readyValue = readyValue + 1
        end
    end

    self.HeaderLabel:SetText(string.format("Prime Readycheck %d/%s", readyValue, self.party and self.party:GetMemberCount() or 1))

    if self.checkSum == self.maxCheckSum then
        self:OutputState()
        self.parent:ReadyDone()
    end
end

function ReadyWindow:OutputState()
    if self.checkSum == self.maxCheckSum then
        ChatOutput("<rgb=#00FF00>Everyone is Ready!</rgb>")
    else
        ChatOutput("Readycheck Timer Ended")
        for key, value in pairs(self.PartyStatus) do
            if value == 0 then self:UpdateReadycheckMember(key, false) end
            if value < 2 then ChatOutput("<rgb=#FF0000>" .. key .. "</rgb> is away") end
        end
    end
end

function ReadyWindow:SetReadyDone()
    self.TimerLabel:SetVisible(false)
    self.HeaderLabel:SetForeColor(Turbine.UI.Color(0, 1, 0))
    if not(self.updateCallback) then self.updateCallback = AddCallback(Updater, "ExecuteUpdate", self.ExecuteUpdate) end
end