ProfileHandler = class(Turbine.UI.Control)
function ProfileHandler:Constructor()
    Turbine.UI.Control.Constructor(self)

    self.changedThisLoad = false

    self.player = Turbine.Gameplay.LocalPlayer.GetInstance()
    self.profiles = LoadTable(Turbine.DataScope.Account, "PrimeProfiles")

    -- Check for existing profile
    if self.profiles == nil or self.profiles[self.player:GetName()] == nil then

        -- Make sure the table exists
        if self.profiles == nil then self.profiles = {} end

        -- Load current settings in profile
        local loadedSettings = LoadTable(Turbine.DataScope.Character, "PrimeSettings")

        -- Make fresh profile if no settings exist
        if loadedSettings == nil then
            self:GetDefaultSettings() 
            self.profiles[self.player:GetName()] = Settings
        else
            self.profiles[self.player:GetName()] = loadedSettings
        end
    end

    if self.profiles["ProfilePreference"] == nil then
        self.profiles["ProfilePreference"] = {
            [self.player:GetName()] = self.player:GetName()
        }
    elseif self.profiles["ProfilePreference"][self.player:GetName()] == nil then
        self.profiles["ProfilePreference"][self.player:GetName()] = self.player:GetName()
    end

    Settings = self.profiles[self.profiles["ProfilePreference"][self.player:GetName()]]

    self:UpdateSettings()

    self.accountData = LoadTable(Turbine.DataScope.Account, "PrimeAccountData")

    if self.accountData == nil then
        self.accountData = {}
        self.accountData["TimeZone"] = 1
    end

    AccountData = self.accountData
end

function ProfileHandler:GetPreferences()
    return (self.profiles["ProfilePreference"])
end

function ProfileHandler:ChangePreference(preference)
    self.changedThisLoad = true
    self.profiles["ProfilePreference"][self.player:GetName()] = preference
    Settings = self.profiles[self.profiles["ProfilePreference"][self.player:GetName()]]
end

function ProfileHandler:UpdateSettings()
    local tempSettings = Settings
    self:GetDefaultSettings()
    for key, value in pairs(tempSettings) do
        if Settings[key] ~=  nil then
            Settings[key] = value
        end
    end

    if plugin:GetVersion() ~= tempSettings.Version then
        Settings.FirstTime = true
    end
    Settings.Version = plugin:GetVersion()
end

function ProfileHandler:SaveSettings()   
    if not(self.changedThisLoad) then
        self.profiles[self.profiles["ProfilePreference"][self.player:GetName()]] = Settings
    end
    SaveTable(Turbine.DataScope.Account, "PrimeProfiles", self.profiles)
    SaveTable(Turbine.DataScope.Account, "PrimeAccountData", AccountData)
end

function ProfileHandler:ResetProfile()
    if self.profiles["ProfilePreference"] == nil then
        self.profiles["ProfilePreference"] = {
            [self.player:GetName()] = self.player:GetName()
        }
    else
        self.profiles["ProfilePreference"][self.player:GetName()] = self.player:GetName()
    end

    StopReloading = true

    self:GetDefaultSettings()
    self.profiles[self.player:GetName()] = Settings
end

function ProfileHandler:GetDefaultSettings() 
    Settings = {}

    Settings.Reload = false
    Settings.Version = PluginVersion
    Settings.OptionsButton = 2

    -- CountdownWindow
    Settings.CWidth = 250
    Settings.CHeight = 125
    Settings.CX = math.floor(Turbine.UI.Display.GetWidth() / 2 - (Settings.CWidth /2))
    Settings.CY = math.floor(Turbine.UI.Display.GetHeight() / 2 - (Settings.CHeight))
    Settings.CFont = Turbine.UI.Lotro.Font.TrajanProBold36
    Settings.COpacity = 1

    -- EffectsWindow
    Settings.EWidth = 750
    Settings.EHeight = 125
    Settings.EX = math.floor(Turbine.UI.Display.GetWidth() / 2 - (Settings.EWidth/2) - 2)
    Settings.EY = math.floor(Turbine.UI.Display.GetHeight() / 3)
    Settings.EEnabled = true
    Settings.EFont = Turbine.UI.Lotro.Font.TrajanProBold36
    Settings.EL = true
    Settings.ER = true
    Settings.EIWidth = 100
    Settings.EIHeight = 100
    Settings.EOpacity = 1

    -- NoteWindow
    Settings.NoteWidth = 300
    Settings.NoteHeight = 450
    Settings.NoteX = 0
    Settings.NoteY = 0
    Settings.NoteOpacity = 1

    -- MessageWindow
    Settings.MWidth = 750
    Settings.MHeight = 125
    Settings.MX = math.floor(Turbine.UI.Display.GetWidth() / 2 - (Settings.MWidth/2) - 2)
    Settings.MY = math.floor(Turbine.UI.Display.GetHeight() / 3 - (Settings.MHeight))
    Settings.MEnabled = true
    Settings.MFont = Turbine.UI.Lotro.Font.TrajanProBold36
    Settings.ML = true
    Settings.MR = true
    Settings.MIWidth = 100
    Settings.MIHeight = 100
    Settings.MOpacity = 1
    Settings.Brickit = false

    -- Skill Counters
    Settings.SWidth = 250
    Settings.SHeight = 750
    Settings.SNewWidth = 200
    Settings.SNewHeight = 200
    Settings.SX = math.floor(Turbine.UI.Display.GetWidth() / 2 - (Settings.SNewWidth/2))
    Settings.SY = math.floor(Turbine.UI.Display.GetHeight() / 2)
    Settings.SX2 = math.floor(Turbine.UI.Display.GetWidth() / 2 - (Settings.SNewWidth/2))
    Settings.SY2 = math.floor(Turbine.UI.Display.GetHeight() / 2)
    Settings.SItemWidth = Settings.SWidth
    Settings.SItemHeight = Settings.SHeight
    Settings.SEnabled = true
    Settings.SFont = Turbine.UI.Lotro.Font.TrajanProBold24
    Settings.STimers = Turbine.UI.Lotro.Font.TrajanProBold36
    Settings.STimerWidth = Settings.SWidth
    Settings.STimerHeight = 25
    Settings.SOpacity = 1
    Settings.SNewTimer = 20
    Settings.SNewFont = Turbine.UI.Lotro.Font.TrajanPro14

    -- Skill Timers
    Settings.SPWidth2 = 200
    Settings.SPHeight2 = 500
    Settings.SPX2 = 0
    Settings.SPY2 = 0
    Settings.SPItemWidth = Settings.SPWidth
    Settings.SPItemHeight = Settings.SPHeight
    Settings.SPEnabled = true
    Settings.SPFont = Turbine.UI.Lotro.Font.TrajanProBold24
    Settings.SPTimers = Turbine.UI.Lotro.Font.TrajanProBold36
    Settings.SPNewFont = Turbine.UI.Lotro.Font.TrajanPro14
    Settings.SPTimerWidth = Settings.SPWidth
    Settings.SPTimerHeight = 32
    Settings.SPTimerHeight2 = 34
    Settings.SPOpacity = 1

    Settings.SPAX = 0
    Settings.SPAY = 0
    Settings.SPAWidth = 200
    Settings.SPAHeight = 300
    Settings.SPAEnabled = true

    Settings.SPCDEnabled = true
    Settings.SPActiveEnabled = true

    Settings.SkillTimerScale = 0.5

    Settings.SkilltimerSorting = 1
    Settings.ActivetimerSorting = 1

    Settings.SPInstance = true

    -- Skill trackers
    Settings.STSWidth = 200
    Settings.STSHeight = 200
    Settings.STSX = math.floor(Turbine.UI.Display.GetWidth() / 2 - (Settings.STSWidth/2))
    Settings.STSY = math.floor(Turbine.UI.Display.GetHeight() / 2 - (Settings.STSHeight / 2))
    Settings.STSSortingMethod = 1

    -- Log window
    Settings.LWidth = 250
    Settings.LHeight = 750
    Settings.LX = 0
    Settings.LY = 0
    Settings.LEnabled = false
    Settings.LOpacity = 1

    -- Combat Timer
    Settings.CTWidth = 100
    Settings.CTHeight = 100
    Settings.CTX = Turbine.UI.Display.GetWidth() / 3
    Settings.CTY = Turbine.UI.Display.GetHeight() - 300
    Settings.CTEnabled = true
    Settings.CTFont = Turbine.UI.Lotro.Font.TrajanProBold36
    Settings.CTOpacity = 1
    Settings.CTOverride = false
    Settings.CTInCombat = false

    -- Leader window
    Settings.LWEnabled = false
    Settings.LWOpacity = 1

    -- Loot window
    Settings.LOEnabled = true
    Settings.LOWidth = 750
    Settings.LOHeight = 750
    Settings.LOX = math.floor(Turbine.UI.Display.GetWidth() / 2 - (Settings.LOWidth /2))
    Settings.LOY =  math.floor(Turbine.UI.Display.GetHeight() / 2 - (Settings.LOHeight /2))
    Settings.LOFont = Turbine.UI.Lotro.Font.Verdana14
    Settings.LOOpacity = 1
    Settings.LOBFont = Turbine.UI.Lotro.Font.VerdanaBold16

    -- Options button
    Settings.OX = 500
    Settings.OY = Turbine.UI.Display.GetHeight() - 100

    -- RatTracker
    Settings.REnabled = false
    Settings.RWidth = 300
    Settings.RHeight = 20 * 17
    Settings.RX = 0
    Settings.RY = 0

    -- Skill Window
    Settings.SWWidth = 100
    Settings.SWX = 500
    Settings.SWY = 500
    Settings.SWVisible = true

    -- Alert Window
    Settings.AOffset = 10
    Settings.AWidth = 600
    Settings.AItemCount = 15
    Settings.AHeight = (Settings.AOffset + 32) * Settings.AItemCount 
    Settings.ABaseHeight = 32
    Settings.AX = math.floor(Turbine.UI.Display.GetWidth() / 2 - (Settings.AWidth/2)) + 100
    Settings.AY = math.floor(Turbine.UI.Display.GetHeight() / 3 - (Settings.AHeight))
    Settings.AEnabled = true
    Settings.AFont = Turbine.UI.Lotro.Font.TrajanProBold24
    Settings.AL = true
    Settings.AR = false
    Settings.AIWidth = 32
    Settings.AIHeight = 32
    Settings.AOpacity = 1

    -- Target window
    Settings.TWidth = 200
    Settings.THeight = 400
    Settings.THeight2 = 300
    Settings.TX = 500
    Settings.TY = 500
    Settings.TEnabled = true
    Settings.TEffects = true
    Settings.TOverride = false
    Settings.TShowAll = false
    Settings.TargetEffectsOld = false
    Settings.TargetWindowToken = 0
    Settings.NewTargetWindowEffects = {}

    Settings.ReloadData = false

    -- Instance controller
    Settings.InstanceState = false
    Settings.EttenmoorsState = false
    Settings.InstanceVersion = 0
    Settings.InstanceTier = 0

    -- Skill overlay
    Settings.SkillbarScale = 1
    Settings.SkillbarPopups = false
    Settings.SkilloverlayCDThreshold = 30
    Settings.SKPX = math.floor(Turbine.UI.Display.GetWidth() / 2 - (200/2))
    Settings.SKPY = math.floor(Turbine.UI.Display.GetHeight() / 3 + 300)

    -- Daily reset
    Settings.LastDailyReset = 0

    -- PvMP Tools
    Settings.KBVisible = true
    Settings.KBX = 500
    Settings.KBY = 0

    -- PvP Timers
    Settings.PPWidth = 200
    Settings.PPHeight = 200
    Settings.PPX = 700
    Settings.PPY = 500

    -- Skill Alerts
    Settings.SKAEnabled = false
    Settings.SKTEnabled = true

    -- First Time Setup
    Settings.FirstTime = true

    -- Sound Player
    Settings.Sound = true

    -- Target Boxes
    Settings.TBEnabled = true
    Settings.TBX = math.floor(Turbine.UI.Display.GetWidth() / 4)
    Settings.TBY = math.floor(Turbine.UI.Display.GetHeight() / 3)

    -- Swap reset
    Settings.SwapToken = 1
    Settings.SwapScale = 1

    -- Advanced
    Settings.Advanced = false
    Settings.UpdateDelay = 0.05
    Settings.IDKMode = false

    -- Update Tokens
    Settings.ChatAlertToken = 1
    Settings.SpecificEffectAlertToken = 1
    Settings.AllEffectAlertToken = 1
    Settings.SkillOverlayToken = 1

    -- Class Tools
    Settings.ClassToolsEnabled = false
    Settings.ClassToolsX = math.floor(Turbine.UI.Display.GetWidth() / 2 - 500 / 2)
    Settings.ClassToolsY = math.floor(Turbine.UI.Display.GetHeight() / 2 - 150/2)
    Settings.ClassToolsNoSkills = false
    Settings.ItemPositions = {}
    Settings.ClassToolsBaseWidth = 200

    -- Beorning
    Settings.ClassToolsBeorningFerocious = false
    Settings.ClassToolsBeorningSlash = true

    -- Captain
    Settings.ClassToolsCaptainPetSkill = true
    Settings.ClassToolsCaptainPetX = 11
    Settings.ClassToolsCaptainPetY = 2

    -- Guardian
    Settings.ClassToolsGuardianBreakRankLabel = true
    Settings.ClassToolsGuardianBreakRankXPrio = 7
    Settings.ClassToolsGuardianBreakRankYBar = 2

    -- Minstrel
    Settings.ClassToolsMinstrelAnthemLabels = true
    Settings.ClassToolsMinstrelAnthemTimers = true

    -- Warden
    Settings.ClassToolsWardenBleeds = {}
    Settings.ClassToolsWardenExtraTick = true
    Settings.ClassToolsWardenQS = true

    -- Plugins Manager
    Settings.AlwaysLoad = {}
    Settings.RaidLoad = {}
    Settings.EttenmoorsLoad = {}
    Settings.NoUnload = false

    -- Threshold bubble
    Settings.ThresX = 500
    Settings.ThresY = 500

    -- M+
    Settings.MythicPlusVisible = true
    Settings.MythicPlusX = 1400
    Settings.MythicPlusY = 0

    Settings.MythicPlusButtonX = Turbine.UI.Display.GetWidth() - 24
    Settings.MythicPlusButtonY = 0
    Settings.MythicPlusSeason = -1

    -- New bar settings
    Settings.BuffBarTexture = true
    Settings.BuffBarOpacity = 1
    Settings.BuffBarFrontTexture = nil
    Settings.BuffBarBackTexture = nil
    Settings.BuffBarBackOpacity = 0.4
    Settings.BuffBarFrontOpacity = 0.4
    Settings.BuffBarSorting = 1
    Settings.BuffBarWidth = 200

    Settings.BossTimersTexture = true
    Settings.BossTimersOpacity = 1
    Settings.BossTimersFrontTexture = nil
    Settings.BossTimersBackTexture = nil
    Settings.BossTimersBackOpacity = 0.6
    Settings.BossTimersFrontOpacity = 0.6

    Settings.SkillTimersTexture = true
    Settings.SkillTimersOpacity = 1
    Settings.SkillTimersFrontTexture = nil
    Settings.SkillTimersBackTexture = nil
    Settings.SkillTimersBackOpacity = 0.6
    Settings.SkillTimersFrontOpacity = 0.6

    Settings.SkillTrackersTexture = true
    Settings.SkillTrackersOpacity = 1
    Settings.SkillTrackersFrontTexture = nil
    Settings.SkillTrackersBackTexture = nil
    Settings.SkillTrackersBackOpacity = 0.6
    Settings.SkillTrackersFrontOpacity = 0.6

    Settings.TargetWindowTexture = true
    Settings.TargetWindowOpacity = 1
    Settings.TargetWindowFrontTexture = nil
    Settings.TargetWindowBackTexture = nil
    Settings.TargetWindowBackOpacity = 0.6
    Settings.TargetWindowFrontOpacity = 0.6

    -- Aura
    Settings.AirLoreAura = false
    Settings.BearMarkBlue = false
    Settings.BearMarkRed = false
    Settings.BearMarkBlueName = ""
    Settings.BearMarkRedName = ""

    -- Data collection
    Settings.CollectIconData = true -- old
    Settings.CollectData = false

    -- Diagnostics
    Settings.DiagnosticsEnabled = false

    -- kill tracker
    Settings.KLX = 1500
    Settings.KLY = 500

    -- Location
    Settings.Location = ""

    -- Missions
    Settings.MissionQuest = ""
    Settings.DelvingTier = 0
    Settings.DelvingMissionActive = false
    Settings.MissionStartTime = 0

end

Profiles = ProfileHandler()

--[[
LoadSettings = LoadTable(Turbine.DataScope.Account, "PrimeLoadSettings")

if LoadSettings == nil then 
    LoadSettings = {} 
    LoadSettings[Turbine.Gameplay.LocalPlayer.GetInstance():GetName()] = 1
end

if  not(LoadSettings[Turbine.Gameplay.LocalPlayer.GetInstance():GetName()]) then
    LoadSettings[Turbine.Gameplay.LocalPlayer.GetInstance():GetName()] = 1
end

LoadSettings.SaveSettings = true

if LoadSettings[Turbine.Gameplay.LocalPlayer.GetInstance():GetName()] == 1 then
    Settings = LoadTable(Turbine.DataScope.Character, "PrimeSettings")
else
    Settings = LoadTable(Turbine.DataScope.Account, "PrimeSettings")

    if Settings == nil then
        Settings = LoadTable(Turbine.DataScope.Character, "PrimeSettings")
    end
end
--]]

--[[
if Settings == nil or Settings.Reload == true then
    GetDefaultSettings()
end
--]]

function SaveData()
    if not(StopReloading) then
        ReloadData = {}
        if Settings.ReloadData then
            if Settings.AEnabled and rezTracker then  ReloadData.DNFS = rezTracker:GetDNFS() end
            if combatTimer then ReloadData.CombatTimer = combatTimer:GetTime() end
        end
        ReloadData.Countdown = countdownWindow:GetCountdown()
        SaveTable(Turbine.DataScope.Server, "PrimeReloadData", ReloadData)

        if optionsButton then Settings.OX, Settings.OY = optionsButton:GetPosition() end
        Settings.CX, Settings.CY = countdownWindow:GetPosition()
        --Settings.NoteX, Settings.NoteY = noteWindow:GetPosition()
        Settings.STSX, Settings.STSY = SkillUsageTracker:GetPosition()
        if SkillPopups then Settings.SKPX, Settings.SKPY = SkillPopups:GetPosition() end
        if Target ~= nil then Settings.TX, Settings.TY = Target:GetPosition() end
        if EventWindows:GetEffectWindow() ~= nil then Settings.EX, Settings.EY = EventWindows:GetEffectWindow():GetPosition() end
        if EventWindows:GetAlertWindow() ~= nil then Settings.AX, Settings.AY = EventWindows:GetAlertWindow():GetPosition() end
        if EventWindows:GetTimerWindow() ~= nil then Settings.SX2, Settings.SY2 = EventWindows:GetTimerWindow():GetPosition() end
        if skillTimers ~= nil then Settings.SPX2, Settings.SPY2 = skillTimers:GetPosition() end
        if activeTimers ~= nil then Settings.SPAX, Settings.SPAY = activeTimers:GetPosition() end
        if logWindow ~= nil then Settings.LX, Settings.LY = logWindow:GetPosition() end
        if combatTimer ~= nil then Settings.CTX, Settings.CTY = combatTimer:GetPosition() end
        --if ratTracker ~= nil then Settings.RX, Settings.RY = ratTracker:GetPosition() end
        --Settings.SWVisible = skillWindow:IsVisible()
        --Settings.SWX, Settings.SWY = skillWindow:GetPosition()
        if InstanceController:GetKillTracker() ~= nil then Settings.KBX, Settings.KBY = InstanceController:GetKillTracker():GetPosition() end
        if targetBox then Settings.TBX, Settings.TBY = targetBox:GetPosition() end
    end

    -- Target window settings handling
    SaveTable(Turbine.DataScope.Character, "PrimeRemovedTargetEffects", RemovedBasicEffects)
    SaveTable(Turbine.DataScope.Character, "PrimeAddedTargetEffects", AddedCharEffects) 
    
    -- Skill overlay
    SaveTable(Turbine.DataScope.Character, "PrimeSkillBars", SkillsHandler:GetSaveState())
    SaveTable(Turbine.DataScope.Character, "PrimeSkillBarsPositions", SkillsHandler:GetPositions())
    if SkillsHandler then SkillsHandler:Unload() end


    -- Buff window
    SaveTable(Turbine.DataScope.Character, "PrimeBuffWindows", BuffWindows)
    
    -- Save swaps
    SwappyWarner:SaveData()

    -- Save sounds
    SaveTable(Turbine.DataScope.Account, "PrimeSoundEffects", Sounds:GetSoundList())

    -- Save skill trackers
    SkillUsageTracker:SaveData()
end