BuffTracker = class(Turbine.UI.Window)
function BuffTracker:Constructor(Configuration)
    Turbine.UI.Window.Constructor(self)

    self.Config = Configuration
    self.width = self.Config.Width

    self.callback = nil

    if self.width == nil then 
        self.width = 200 
        BuffWindows[self.Config.Name].Width = 200
    end

    self.sortDirection = self.Config.sortDirection

    if self.sortDirection == nil then 
        self.sortDirection = 1 
        BuffWindows[self.Config.Name].sortDirection = 1
    end

    self.height = 200
    local x = self.Config.X
    local y = self.Config.Y

    local cooldownTimer = 0
    self.Timers = {}

    self.TimerHeight = 20

    self.ResortTimers = false

    self.itemCount = 0

    self.player = Turbine.Gameplay.LocalPlayer.GetInstance()


    --self.TargetEffects.self = nil 

    self:SetSize(self.width, self.height)
    self:SetPosition(x,y)
    self:SetVisible(true)
    self:SetWantsUpdates(true)
    --Updater:Subscribe(self)
    self:SetMouseVisible(false)
    self:SetOpacity(1)
    --self:SetBackColor(GoldColor)
    self.colour = nil

    if self.Config.SortMethod == nil then self.Config.SortMethod = 1 end

    self.font = Settings.SPNewFont
    --self.font = Turbine.UI.Lotro.Font.Verdana14
    --self.font = Turbine.UI.Lotro.Font.VerdanaBold16

    if self.Config.Buff then
        self.colour = Turbine.UI.Color(0,0.5,0)
    else
        self.colour = Turbine.UI.Color(0.5,0,0)
    end

    -- Colour scheme [1] outline [2] background [3] bar colour [4] text colour [5] outline colour [6] warning colour
    self.colourScheme = {Turbine.UI.Color(0.8,0,0,0), Turbine.UI.Color(0.8,0,0,0), self.colour, Turbine.UI.Color(1,1,1), Turbine.UI.Color(0,0,0), Turbine.UI.Color(0.6,0.5,0,0), Settings.BuffBarFrontTexture, Settings.BuffBarBackTexture, Settings.BuffBarFrontOpacity, Settings.BuffBarBackOpacity}

    local SortingMethod = self.Config.SortMethod

    self.TimerContainer = Turbine.UI.Control()
    self.TimerContainer:SetParent(self)
    self.TimerContainer:SetVisible(true)
    self.TimerContainer:SetSize(100, 200)
    self.TimerContainer:SetPosition(0, 0)
    self.TimerContainer:SetMouseVisible(false)

    local moveBox = MoveBox(self, self.width, 200, self.Config.Name)
    
    self.ToggleMove = function()
        BuffWindows[self.Config.Name].X = self:GetLeft()
        BuffWindows[self.Config.Name].Y = self:GetTop()
        moveBox:ToggleMove()
    end

    self.effectList = self.player:GetEffects()

    self.NewTimer = function(effect)
        if self.Config.Buff and effect:IsDebuff() or not(self.Config.Buff) and not(effect:IsDebuff()) then return end
        if self.Config.Mode == 1 and not(self.Config.List[effect:GetName()]) or self.Config.Mode == 2 and self.Config.List[effect:GetName()] then return end
        local duration = effect:GetDuration() - (Turbine.Engine.GetGameTime() - effect:GetStartTime())
    
    
        self.Timers[effect:GetID()] = EffectTimer(self.TimerContainer, self.width, self.TimerHeight, effect, effect:GetName(), self.font, self.colourScheme, true, self.Config.Name)
        
        if self.Config.sortDirection == 2 then
            self.Timers[effect:GetID()]:SetPosition(0, self.height - self.TimerHeight) 
        end
        self.itemCount = self.itemCount + 1
    
        if not(self.callback) then
            self.callback = AddCallback(Updater, "ExecuteUpdate", self.ExecuteUpdate)
        end
    
        self.ResortTimers = true
    
        duration = nil
        effect = nil
    end

    self.SetupTimers = function()
        for i = 1, self.effectList:GetCount(), 1 do
            self.NewTimer(self.effectList:Get(i))
        end 
    end

    self.Clear = function()
        for key, value in pairs(self.Timers) do 
            value:Destroy()
            self.Timers[key] = nil
            --table.remove(self.Timers, key )
            --PrintTable(self.Timers)
        end
        if Tablelength(self.Timers) > 0 then
            self.Clear()
        end
        self.Timers = nil
        collectgarbage()
        if self.callback then
            RemoveCallback(Updater, "ExecuteUpdate", self.callback)
            self.callback = nil
        end 
        self.Timers = {}
        self.itemCount = 0
    end

    if SortingMethod == 1 then
        self.TimerCompare = function(t, a, b)
            if t[b]:GetRemainingTime() == t[a]:GetRemainingTime() then
                return t[b]:GetText() > t[a]:GetText()
            else
                return t[b]:GetRemainingTime() < t[a]:GetRemainingTime()
            end
        end
    elseif SortingMethod == 2 then
        self.TimerCompare = function(t, a, b)
            if t[b]:GetRemainingTime() == t[a]:GetRemainingTime() then
                return t[b]:GetText() > t[a]:GetText()
            else
                return t[b]:GetRemainingTime() > t[a]:GetRemainingTime()
            end
        end
    end


    self.ResortList = function()
        local counter = 0

        collectgarbage()

        if self.sortDirection == 1 then
            for _,v in spairs(self.Timers,self.TimerCompare) do
                v:SetPosition(0, counter * self.TimerHeight) 
                counter = counter + 1
            end
        elseif self.sortDirection == 2 then
            for _,v in spairs(self.Timers, self.TimerCompare) do
                v:SetPosition(0, self.height - counter * self.TimerHeight - self.TimerHeight) 
                counter = counter + 1
            end
        end

        self.ResortTimers = false
    end

    self.ExecuteUpdate = function(delta)
        --We had an update
        for key, value in pairs(self.Timers) do 
            if value:UpdateText(delta) then 
                self.Timers[key]:Unload()
                self.Timers[key] = nil
                self.ResortTimers = true
                self.itemCount = self.itemCount - 1
            end
        end

        if self.ResortTimers then 
            if self.itemCount == 0 then 
                RemoveCallback(Updater, "ExecuteUpdate", self.callback)
                self.callback = nil
                self.Timers = nil
                collectgarbage()
                self.Timers = {}
                return
            end
            self.ResortList() 
        end
    end
    self.SetupTimers()

    self.RemoveTimer = function(effect)
        local id = effect:GetID()
        if self.Timers[id] ~= nil then 
            self.Timers[id]:Destroy()
            self.Timers[id] = nil
            self.itemCount = self.itemCount - 1
            self.ResortTimers = true
        end
    end


    self.addCallaback = AddCallback(PlayerEffectsHandler, "EffectAdded", self.NewTimer)
    self.removeCallback = AddCallback(PlayerEffectsHandler, "EffectRemoved", self.RemoveTimer)
end

function BuffTracker:Reset()
    self.Clear()
    self.SetupTimers()
end


function BuffTracker:Unload()
    RemoveCallback(PlayerEffectsHandler, "EffectAdded", self.addCallaback)
    RemoveCallback(PlayerEffectsHandler, "EffectRemoved", self.removeCallback)
    self:Clear()
    self:SetParent(nil)
end
