SkillTrackers = class(Turbine.UI.Window)
function SkillTrackers:Constructor()
    Turbine.UI.Window.Constructor(self)

    self.width = Settings.STSWidth
    self.height = Settings.STSHeight
    self.x = Settings.STSX
    self.y = Settings.STSY
    --self:SetBackColor(GetRandomColour())

    self.Timers = {}
    self.TimerCount = {}
    self.TimerLinks = {}
    self.skillItems = {}

    self.targettingAllowed = false

    self.TimerNames = {
        ["Fray the Edge"] = {10, 1, true},
        ["Engage"] = {10, 1, true},
        ["Wind-lore"] = {45, 1, true},
        ["Sign of Power: Righteousness"] = {20, 1, true}
    }

    self.TimerNames = LoadTable(Turbine.DataScope.Character, "PrimeSkillTracker")

    if self.TimerNames == nil then self.TimerNames = {} end
    
    self.resort = false
    self.count = 0
    self.sorting = Settings.STSSortingMethod

    self.player = Turbine.Gameplay.LocalPlayer.GetInstance()

    self.TimerHeight = Settings.SNewTimer
    self.font = Turbine.UI.Lotro.Font.VerdanaBold16

    -- Colour scheme [1] outline [2] background [3] bar colour [4] text colour [5] outline colour [6] warning colour
    --Turbine.UI.Color(1,0,0), Turbine.UI.Color(0.5,0,0), Turbine.UI.Color(0,0.125,0.5), Turbine.UI.Color(0,0.6,7), Turbine.UI.Color(0,0.0,0.8)
    self.colourScheme = {Turbine.UI.Color(0,0,0), Turbine.UI.Color(0,0,0), Turbine.UI.Color(0,170/255,170/255), Turbine.UI.Color(1,1,1), Turbine.UI.Color(0,0,0), Turbine.UI.Color(0.5,0,0), Settings.SkillTrackersFrontTexture, Settings.SkillTrackersBackTexture, Settings.SkillTrackersFrontOpacity, Settings.SkillTrackersBackOpacity}

    self:SetSize(self.width, self.height)
    self:SetPosition(self.x, self.y)
    --self:SetZOrder(1)

    self:SetMouseVisible(false)
    self:SetVisible(true)
    --self:SetWantsUpdates(true)
    --Updater:Subscribe(self)
  
    self.moveBox = MoveBox(self, self.width, self.height, "Skill tracker")

    self.Clear = function()
        for key, value in pairs(self.Timers) do 
            value[1]:Destroy()
            self.Timers[key] = nil
            --table.remove(self.Timers, key )
            --PrintTable(self.Timers)
        end
    
        if Tablelength(self.Timers) > 0 then
            self.Clear()
        end
        self.Timers = nil
        collectgarbage()
        self.Timers = {}
    end

    
    if self.sorting == 1 then
        -- Top to bottom
        self.ResortList = function()
            local counter = 0
            for _,v in spairs(self.Timers, function(t,a,b) return t[b][2] > t[a][2] end) do
                v[1]:SetPosition(0, counter * self.TimerHeight) 
                counter = counter + 1
            end
    
            self.resort = false
        end
    elseif self.sorting == 2 then
        -- Bottom to top
        self.ResortList = function()
            local counter = 0
            for _,v in spairs(self.Timers, function(t,a,b) return t[b][2] > t[a][2] end) do
                v[1]:SetPosition(0, self.height - counter * self.offset) 
                counter = counter + 1
            end
    
            self.resort = false
        end
    end

    self.skills = self.player:GetTrainedSkills()
    self.callbacks = {}

    self.ExecuteUpdate = function(delta)    
        --We had an update
        for key, value in pairs(self.Timers) do 
            --ChatOutput(value[2])
            value[2] = value[2] - delta
            value[1]:UpdateText(delta)
            if value[2] < 0 then 
                self.TimerCount[value[3]] = self.TimerCount[value[3]] - 1
                self.TimerLinks[value[3]][value[4]] = nil
                value[1]:Destroy()
                self.Timers[key] = nil
                self.resort = true
                self.count = self.count - 1
            end
        end
    
        if self.resort then 
            if self.count == 0 then 
                if self.updateCallback then if self.updateCallback then RemoveCallback(Updater, "ExecuteUpdate", self.updateCallback) end end
                self.updateCallback = nil 
                self.Timers = nil
                collectgarbage()
                self.Timers = {}
                return
            end
            self.ResortList() 
        end
    end

    self:SetCallbacks()
end

function SkillTrackers:RemoveCallbacks()
    for key, value in pairs(self.skillItems) do
        RemoveCallback(value, "ResetTimeChanged", self.callbacks[key])
    end
end

function SkillTrackers:SetCallbacks()
    for i = 1, self.skills:GetCount(), 1 do
        local item = self.skills:GetItem(i)--:GetSkillInfo()
        if self.TimerNames[item:GetSkillInfo():GetName()] then
            local skillInfo = item:GetSkillInfo()
            local timerData = self.TimerNames[skillInfo:GetName()]
            self.skillItems[skillInfo:GetName()] = item
            self.callbacks[skillInfo:GetName()] = AddCallback(item, "ResetTimeChanged", 
            function(sender, args)
                if item:GetResetTime() - Turbine.Engine.GetGameTime() > item:GetCooldown() -1 then
                    self:NewTimer(timerData[1], skillInfo:GetName(), timerData[2], skillInfo:GetIconImageID(), timerData[3])
                end
            end)
        end
    end
end

function SkillTrackers:ResetData()
    self:RemoveCallbacks()
    self:SetCallbacks()
end

function SkillTrackers:ToggleMove()
    self.moveBox:ToggleMove()
end

-- Add a timer
function SkillTrackers:NewTimer(duration, message, count, icon, target)

    local timerMessage = message
    local dictionaryString = message
    local playerTarget = nil 

    -- Do we want to look at the target for the name
    if target and self.targettingAllowed then
        --playerTarget = self.player:GetTarget()
        --if playerTarget then timerMessage = self.player:GetTarget():GetName() end
        dictionaryString = message .. timerMessage
    end

    -- Update amount of timers with of this kind
    if self.TimerCount[dictionaryString] then
        self.TimerCount[dictionaryString] = self.TimerCount[dictionaryString] + 1
    else
        self.TimerCount[dictionaryString] = 1
    end

    -- Handle invalid timers
    if self.TimerCount[dictionaryString] > count then
        local oldestID = 99999999
        for key, _ in pairs(self.TimerLinks[dictionaryString] ) do
            if oldestID > key then oldestID = key end
        end

        if oldestID ~= 99999999 then
            self.TimerLinks[dictionaryString][oldestID] = nil
            self.TimerCount[dictionaryString] = self.TimerCount[dictionaryString] - 1
            self.Timers[oldestID][1]:Destroy()
            self.Timers[oldestID] = nil
            self.count = self.count - 1
        end
    end


    -- Add new timer
    local timerID = #self.Timers + 1
    if self.TimerLinks[dictionaryString] == nil then self.TimerLinks[dictionaryString] = {} end
    self.TimerLinks[dictionaryString][timerID] = true
    self.Timers[timerID] = {EffectTimer(self, self.width, self.TimerHeight, nil, timerMessage, self.font, self.colourScheme, true, nil, duration, icon, playerTarget), duration, dictionaryString, timerID}
    
    -- Handle administration
    self.count = self.count + 1
    self.resort = true
    if not(self.updateCallback) then self.updateCallback = AddCallback(Updater, "ExecuteUpdate", self.ExecuteUpdate) end

    playerTarget = nil
    collectgarbage()
end

function SkillTrackers:SaveData()
    SaveTable(Turbine.DataScope.Character, "PrimeSkillTracker", self.TimerNames)
end

function SkillTrackers:AddTimerToTable(name, value)
    self.TimerNames[name] = value
    self:RemoveCallbacks()
    self:SetCallbacks()
end

function SkillTrackers:RemoveFromTable(name)
    self.TimerNames[name] = nil
end

function SkillTrackers:GetTimers()
    return self.TimerNames
end

---------------------------------------------------------------------------------------------------------------------------------------------------------
---------------------------------------------------------------------------------------------------------------------------------------------------------
---------------------------------------------------------------------------------------------------------------------------------------------------------
---------------------------------------------------------------------------------------------------------------------------------------------------------