function LoadTools()
    -- Loading reload data
    local reloadStateData = LoadTable(Turbine.DataScope.Server, "PrimeReloadData") 
    if reloadStateData == nil then reloadStateData = {} end

    -- Core handlers
    Sounds = SoundPlayer()
    Updater = Handlers.UpdateHandler()
    BackgroundHandler = Handlers.BackgroundHandler()
    ChatEventHandler = ChatHandler()
    PlayerEffectsHandler = Handlers.PlayerEffectEvents()
    GroupEffectsHandler = Handlers.GroupEffectEvents()
    EventWindows = Alerts.EventWindowsHandler()
    PlayerHandlerObject = Handlers.PlayerHandler()
    ActionHandlerObject = Handlers.ActionHandler()
    SkillEventHandler = Handlers.PlayerSkillEvents()
    StatEventHandling = StatsUtils.StatEventHandler()
    Roles = Handlers.RoleHandler()
    TargetHandlerObject = Handlers.TargetEventHandling()
    rezTracker = DNFTracker(reloadStateData) 
    LocationHandler = Handlers.LocationHandler()
    NameHandlerObject = Naming.NameHandler()

    -- Utility handlers
    LootHandler = LootUtils.LootHandler()
    PluginsManager = PluginManager()
    DataGrabberTool = Data.DataGrab()

    -- Popups
    Achievements = UI.AchievementListBox()

    -- Auras
    Auras = AuraCollection.AuraHandler()

    -- Instance handlers & tools
    InstanceController = Instances.InstanceHub()
    IWB = Instances.InstanceWindowButton()

    NoteHandler = Notes.NoteHandling()

    -- Player tools
    ClassTools = ClassSpecificTools.ClassToolHandler()
    countdownWindow = CountdownWindow()
    if reloadStateData ~= nil then
        local cd = reloadStateData.Countdown
        if reloadStateData.Countdown ~= nil and cd > 0 then
            countdownWindow:Countdown(cd)
        end
    end
    if Settings.CTEnabled then 
        combatTimer = Timers.CombatWindow(reloadStateData) 
    end
    if Settings.SPEnabled and false then 
        skillTimers = SkillTools.SkillTimers(0)
    end 
    if Settings.SPEnabled and false then 
        activeTimers = SkillTools.SkillTimers(1) 
    end
    SkillUsageTracker = SkillTools.SkillTrackers(false)
    SwappyWarner = SwapWarner()
    SkillsHandler = SkillTools.SkillBarHandler()
    if Settings.SkillOverlayToken == 1 then
        SkillsHandler:ConvertLegacySave()
        Settings.SkillOverlayToken = 2
    else
        SkillsHandler:CreateSkillBarFromDictionary()
    end
    if Settings.SkillbarPopups then
        SkillPopups =  SkillTools.SkillPopupWindow()
    end

    -- Group tools
    ReadyHandler = ReadyCheck.ReadyCheckHandler()

    -- Target Tools
    if Settings.TEnabled then 
        if Settings.TargetEffectsOld then
            ChatOutput("You are using the deprecated version of the target window")
            Target = TargetWindowTools.TargetWindowOld()
        else
            Target = TargetWindowTools.TargetWindow()
        end
    end

    -- PrimeUI
    PrimeUIHandlerObject = PrimeUITools.PrimeUIHandler()
    PrimeUIHandlerObject:Initialize()

    -- Buffbar loading
    BuffWindows = LoadTable(Turbine.DataScope.Character, "PrimeBuffWindows")
    BuffWindowObjects = {}

    if BuffWindows == nil then
        BuffWindows = {}
    end

    for key, value in pairs(BuffWindows) do
        local buff = SkillTools.BuffTracker(value)
        BuffWindowObjects[key] = buff
    end

    -- Options & Reloading
    reloadWindow = ReloadWindow()
    optionsButton = OptionsButton()
    RButton = ReloadButton()
    import "PrimePlugins.RaidTools.Options"
    if Settings.FirstTime then
        import "PrimePlugins.RaidTools.NewVersionSetup.FirstTimeSetup"
        Setup = NewVersionSetup.FirstTimerOptions()
    end

    -- Diagnostics
    if Settings.DiagnosticsEnabled then
        D = Diagnostics.DiagWindow()
    end
    if Settings.LWEnabled then 
        import "PrimePlugins.RaidTools.LeaderWindow"
        leaderWindow = LeaderWindow() 
    end

    if Settings.LEnabled then 
        import "PrimePlugins.RaidTools.LogWindow"
        logWindow = LogWindow() 
    end 

    -- Initialize commands
    SetCommands()
end

LoadTools()

function UnloadTools()
    if PlayerEffectsHandler then PlayerEffectsHandler:RemoveCallbacks() end
    if GroupEffectsHandler then GroupEffectsHandler:RemoveCallbacks() end
    if ChatEventHandler then ChatEventHandler:RemoveCallbacks() end
    if SwappyWarner then SwappyWarner:RemoveCallbacks() end
    if PlayerHandlerObject then PlayerHandlerObject:Unload() end
    if LocationHandler then LocationHandler:Unload() end

    if Target then Target:RemoveCallbacks() end
    --if Windows["Target Box"] then Windows["Target Box"]:RemoveCallbacks() end
    if combatTimer then combatTimer:RemoveCallbacks() end

    if Auras then Auras:Close() end
    if ClassTools then ClassTools:Unload() end
    if InstanceController then InstanceController:Unload() end
    DataGrabberTool:Save()
    for key, value in pairs(BuffWindowObjects) do
        value:Unload()
    end

    if EventWindows:GetAlertWindow() then EventWindows:GetAlertWindow():Unload() end
    if EventWindows:GetEffectWindow() then EventWindows:GetEffectWindow():Unload() end
    EventWindows:Unload()
    SkillEventHandler:RemoveCallbacks()
    if PrimeUIHandlerObject then 
        PrimeUIHandlerObject:UnloadUI()
        PrimeUIHandlerObject:Save() 
    end
    if IWB then IWB:Unload() end
    if Roles then Roles:Unload() end
    if LootHandler then LootHandler:Unload() end
    if NoteHandler then NoteHandler:Unload() end
    if WindowsHandlerController then WindowsHandlerController:Unload() end
    NameHandlerObject:Unload()
end