StatEventHandler = class()
function StatEventHandler:Constructor()
    self.player = Turbine.Gameplay.LocalPlayer.GetInstance()
    self:Load()
end

function StatEventHandler:Load()

    self.pluginStartTime = Turbine.Engine.GetGameTime()

    self.totalCharStats = LoadTable(Turbine.DataScope.Character, "PrimeCharStats") or self:GetBaseCharStats()
    self.sessionCharStats = self:GetBaseCharStats()

    self.totalAccountStats = LoadTable(Turbine.DataScope.Account, "PrimeAccountStats") or self:GetBaseAccountStats()
    self.sessionAccountStats = self:GetBaseAccountStats()

    for key, value in pairs(self:GetBaseAccountStats()) do
        if self.sessionAccountStats[key] ==  nil then
            self.sessionAccountStats[key] = value
        end
    end
end

function StatEventHandler:Unload()
    local pluginUptime = Turbine.Engine.GetGameTime() - self.pluginStartTime

    -- Daily pvmp stats
    if self.sessionCharStats.DailyDeaths > self.sessionCharStats["HighestDailyDeaths"] then self.sessionCharStats["HighestDailyDeaths"] = self.sessionCharStats.DailyDeaths end
    if self.sessionCharStats.DailyDeaths > self.sessionAccountStats["HighestDailyDeaths"] then self.sessionAccountStats["HighestDailyDeaths"] = self.sessionAccountStats.DailyDeaths end
    if self.sessionCharStats.DailyKB > self.sessionCharStats["HighestDailyKBs"] then self.sessionCharStats["HighestDailyKBs"] = self.sessionCharStats.DailyKB end
    if self.sessionCharStats.DailyKB > self.sessionAccountStats["HighestDailyKBs"] then self.sessionAccountStats["HighestDailyKBs"] = self.sessionAccountStats.DailyKB end

    if InstanceController:GetKillTracker() then
        if InstanceController:GetKillTracker():GetKillStreak() > self.sessionCharStats["HighestKillStreak"] then self.sessionCharStats["HighestKillStreak"] = InstanceController:GetKillTracker():GetKillStreak()  end
        if InstanceController:GetKillTracker():GetKillStreak() > self.sessionAccountStats["HighestKillStreak"] then self.sessionAccountStats["HighestKillStreak"] = InstanceController:GetKillTracker():GetKillStreak()  end
        if InstanceController:GetKillTracker():HighestKillCombo() > self.sessionCharStats["HighestKillCombo"] then self.sessionCharStats["HighestKillCombo"] = InstanceController:GetKillTracker():HighestKillCombo() end
        if InstanceController:GetKillTracker():HighestKillCombo() > self.sessionAccountStats["HighestKillCombo"] then self.sessionAccountStats["HighestKillCombo"] = InstanceController:GetKillTracker():HighestKillCombo() end
    end

    self.sessionCharStats["Time"] = self.sessionCharStats["Time"] + pluginUptime
    self.sessionAccountStats["Time"] = self.sessionAccountStats["Time"] + pluginUptime

    -- Handle char stats
    SaveTable(Turbine.DataScope.Character, "PrimeCharStats", self:GetCharData())

    -- Handle account stats
    SaveTable(Turbine.DataScope.Account, "PrimeAccountStats", self:GetAccountData())
end

function StatEventHandler:GetCharData()
    return CombineTables(self.totalCharStats, self.sessionCharStats)
end


function StatEventHandler:GetAccountData()
    return CombineTables(self.totalAccountStats, self.sessionAccountStats)
end

function StatEventHandler:GetData()
    return self:GetCharData(), self:GetAccountData()
end

function StatEventHandler:ResetDailyPvMPStats()
    self.sessionCharStats.DailyKB = 0
    self.sessionCharStats.DailyDeaths = 0
end

function StatEventHandler:KillStreakDeath(killstreak)
    if killstreak > self.sessionCharStats["HighestKillStreak"] then self.sessionCharStats["HighestKillStreak"] = killstreak end
    if killstreak > self.sessionAccountStats["HighestKillStreak"] then self.sessionAccountStats["HighestKillStreak"] = killstreak end
end

function StatEventHandler:AddReload()
    self.sessionCharStats["Reload"] = self.sessionCharStats["Reload"] + 1
    self.sessionAccountStats["Reload"] = self.sessionAccountStats["Reload"] + 1
end

function StatEventHandler:AddTrack()
    self.sessionCharStats["Tracks"] = self.sessionCharStats["Tracks"] + 1
    self.sessionAccountStats["Tracks"] = self.sessionAccountStats["Tracks"] + 1
end

function StatEventHandler:AddDeath(kb, misadventure)
    self.sessionCharStats["Death"] = self.sessionCharStats["Death"] + 1
    self.sessionAccountStats["Death"] = self.sessionAccountStats["Death"] + 1

    if not (kb or misadventure) then return end

    if kb then
        if not(self.sessionCharStats["KilledBy"]) then self.sessionCharStats["KilledBy"] = {} end
        if not(self.sessionAccountStats["KilledBy"]) then self.sessionAccountStats["KilledBy"] = {} end

        self.sessionCharStats["KilledBy"][kb] = (self.sessionCharStats["KilledBy"][kb] or 0) + 1
        self.sessionAccountStats["KilledBy"][kb] = (self.sessionAccountStats["KilledBy"][kb] or 0) + 1
    elseif misadventure then
        self.sessionCharStats["MisadventureDeaths"] = self.sessionCharStats["MisadventureDeaths"] + 1
        self.sessionAccountStats["MisadventureDeaths"] = self.sessionAccountStats["MisadventureDeaths"] + 1
    end

    if InstanceController:GetKillTracker() and Settings.EttenmoorsState then
        self.sessionCharStats.DailyDeaths = self.sessionCharStats.DailyDeaths + 1
        self.sessionAccountStats["PVPDeath"] = self.sessionAccountStats["PVPDeath"] + 1
        self.sessionCharStats["PVPDeath"] = self.sessionCharStats["PVPDeath"] + 1
        InstanceController:GetKillTracker():UpdateDeath()

        if Turbine.Gameplay.LocalPlayer.GetInstance():GetAlignment() == 2 then
            self.sessionAccountStats["CreepDeaths"] = self.sessionAccountStats["CreepDeaths"] + 1
        else
            self.sessionAccountStats["FreepDeaths"] = self.sessionAccountStats["FreepDeaths"] + 1
        end
    end
end

function StatEventHandler:AddKB(kb, pvp)
    if not kb then return end

    self.sessionCharStats["KB"][kb] = (self.sessionCharStats["KB"][kb] or 0) + 1
    self.sessionAccountStats["KB"][kb] = (self.sessionAccountStats["KB"][kb] or 0) + 1

    if pvp and Settings.EttenmoorsState then 
        self.sessionCharStats.DailyKB = self.sessionCharStats.DailyKB + 1
        if InstanceController:GetKillTracker() then
            InstanceController:GetKillTracker():UpdateKill(kb)
        end
        self.sessionAccountStats["PVPKB"] = self.sessionAccountStats["PVPKB"] + 1
        self.sessionCharStats["PVPKB"] = self.sessionCharStats["PVPKB"] + 1

        if Turbine.Gameplay.LocalPlayer.GetInstance():GetAlignment() == 2 then
            self.sessionAccountStats["CreepKBs"] = self.sessionAccountStats["CreepKBs"] + 1
        else
            self.sessionAccountStats["FreepKBs"] = self.sessionAccountStats["FreepKBs"] + 1
        end
    end
end


function StatEventHandler:SaveMythicPlusData(instanceID, name, value)
    if not(self.sessionAccountStats["MythicPlus"][name]) then self.sessionAccountStats["MythicPlus"][name] = {} end
    if not(self.sessionAccountStats["MythicPlus"][name][instanceID]) then self.sessionAccountStats["MythicPlus"][name][instanceID] = {} end
    table.insert(self.sessionAccountStats["MythicPlus"][name][instanceID], value)
end

function StatEventHandler:AddMythicPlusScore(instanceID, name, value)
    if not(self.sessionAccountStats["TEQ"][name]) then self.sessionAccountStats["TEQ"][name] = {} end
    if not(self.sessionAccountStats["TEQ"][name][instanceID]) then 
        self.sessionAccountStats["TEQ"][name][instanceID] = value 
    else
        if self.sessionAccountStats["TEQ"][name][instanceID]["Score"] < value["Score"] then
            self.sessionAccountStats["TEQ"][name][instanceID] = value 
        end
    end
end

function StatEventHandler:ReturnBossData(instanceID, bossName, instanceTier)
    if not(self.totalAccountStats["BossData"][instanceID]) then
        self.totalAccountStats["BossData"][instanceID] = {}
    end

    if not(self.totalAccountStats["BossData"][instanceID][bossName]) then
        self.totalAccountStats["BossData"][instanceID][bossName] = {}
    end

    if not(self.totalAccountStats["BossData"][instanceID][bossName][instanceTier]) then
        self.totalAccountStats["BossData"][instanceID][bossName][instanceTier] = {
            ["Wipes"] = 0,
            ["Completions"] = 0,
            ["BestTime"] = 99999999,
            ["FirstCompletion"] = -1,
        }
    end

    return(self.totalAccountStats["BossData"][instanceID][bossName][instanceTier])
end

function StatEventHandler:SaveBossData(instanceID, bossName, instanceTier, newTime, completion, player)

    -- Update completion/wipe count
    if completion then
        self.totalAccountStats["BossData"][instanceID][bossName][instanceTier]["Completions"] = self.totalAccountStats["BossData"][instanceID][bossName][instanceTier]["Completions"] + 1
    else
        self.totalAccountStats["BossData"][instanceID][bossName][instanceTier]["Wipes"] = self.totalAccountStats["BossData"][instanceID][bossName][instanceTier]["Wipes"] + 1
    end

    -- Update best time
    if self.totalAccountStats["BossData"][instanceID][bossName][instanceTier]["BestTime"] > newTime and completion then
        self.totalAccountStats["BossData"][instanceID][bossName][instanceTier]["BestTime"] = newTime
    end

    -- Update first completion
    if completion and self.totalAccountStats["BossData"][instanceID][bossName][instanceTier]["FirstCompletion"] < 0 then
        self.totalAccountStats["BossData"][instanceID][bossName][instanceTier]["FirstCompletion"] = Turbine.Engine.GetLocalTime()
        self.totalAccountStats["BossData"][instanceID][bossName][instanceTier]["FirstCompletionCount"] = self.totalAccountStats["BossData"][instanceID][bossName][instanceTier]["Wipes"] + 1
        self.totalAccountStats["BossData"][instanceID][bossName][instanceTier]["FirstCompletedCharacter"] = self.player:GetName()

        local group = ""
        local party = self.player:GetParty()
        if party then
            local members = {}
            for i = 1, party:GetMemberCount() do
                local member = party:GetMember(i)
                table.insert(members, member:GetName())
            end
            group = table.concat(members, ", ")
        end
        
        self.totalAccountStats["BossData"][instanceID][bossName][instanceTier]["FirstCompletedGroup"] = group
    end
end

function StatEventHandler:GetBaseAccountStats()
    return {
        Time = 0,
        KB = {},
        Reload = 0,
        Death = 0,
        KilledBy = {},
        DamageTaken = 0,
        DamageDealt = 0,
        Healing = 0,
        PVPKB = 0,
        PVPDeath = 0,
        MisadventureDeaths = 0,
        HighestDailyDeaths = 0,
        HighestDailyKBs = 0,
        HighestKillStreak = 0,
        HighestKillCombo = 0,
        CreepDeaths = 0,
        CreepKBs = 0,
        FreepDeaths = 0,
        FreepKBs = 0,
        Tracks = 0,
        BossData = {},
        TEQ = {},
        Version = 0
    }
end


function StatEventHandler:GetBaseCharStats()
    return {
        Time = 0,
        KB = {},
        Reload = 0,
        Death = 0,
        KilledBy = {},
        DamageTaken = 0,
        DamageDealt = 0,
        Healing = 0,
        DailyKB = 0,
        DailyDeaths = 0,
        PVPKB = 0,
        PVPDeath = 0,
        MisadventureDeaths = 0,
        HighestDailyDeaths = 0,
        HighestDailyKBs = 0,
        HighestKillStreak = 0,
        HighestKillCombo = 0,
        Tracks = 0,
        Version = 0
    }
end

