TargetWindowOld = class(Turbine.UI.Window)
function TargetWindowOld:Constructor()
    Turbine.UI.Window.Constructor(self)
    self.width = Settings.TWidth
    self.height = Settings.THeight2
    self.x = Settings.TX
    self.y = Settings.TY

    self.Timers = {}

    self.UpdateDelay = 0.05
    self.CheckDelay = 0.3
    self.TimeSinceLastCheck = 0
    self.UpdateToken = 1
    self.font = Settings.SPNewFont
    self.colourScheme = {Turbine.UI.Color(0,0,0), Turbine.UI.Color(0,0,0), Turbine.UI.Color(0,0.3,0), Turbine.UI.Color(1,1,1), Turbine.UI.Color(0,0,0), Turbine.UI.Color(0.3,0,0), Settings.TargetWindowFrontTexture, Settings.TargetWindowBackTexture, Settings.TargetWindowFrontOpacity, Settings.TargetWindowBackOpacity}

    self.PermDelay = 3

    self.TimerHeight = 20

    self.ResortTimers = false

    self.targetEffects = nil
    self.target = nil
    self.targetHealth = nil
    self.targetTime = Turbine.Engine.GetGameTime()
    self.targetEffectsID = {}

    self.ShowAll = Settings.TShowAll
    self.DoEffects = Settings.TEffects
    self.pvpEffects = Settings.SKTEnabled
    self.AlertsEnabled = Settings.AEnabled
    self.pvpAlerts = Settings.SKAEnabled

    self.oldGameTime = Turbine.Engine.GetGameTime()

    if Settings.TargetWindowToken < 1 or Settings.NewTargetWindowEffects == nil then
        Settings.NewTargetWindowEffects = SetEffects()
        Settings.TargetWindowToken = 1
    end

    self.targetEffectsDictionary = Settings.NewTargetWindowEffects

    self.callback = {}

    self.player = Turbine.Gameplay.LocalPlayer.GetInstance()

    self:SetSize(self.width, self.height)
    self:SetPosition(self.x, self.y)
    self:SetVisible(true)
    self:SetWantsUpdates(true)
    self:SetMouseVisible(false)
    self:SetOpacity(1)

    self.BackgroundWindow = Turbine.UI.Control()
    self.BackgroundWindow:SetParent(self)
    self.BackgroundWindow:SetSize(self.width , 30)
    self.BackgroundWindow:SetBackColor(Turbine.UI.Color(0,0,0))
    self.BackgroundWindow:SetOpacity(1)
    self.BackgroundWindow:SetVisible(true)
    self.BackgroundWindow:SetMouseVisible(false)

    self.OpacityBackground = Turbine.UI.Control()
    self.OpacityBackground:SetParent(self.BackgroundWindow)
    self.OpacityBackground:SetSize(self.width - 2, 30 - 2)
    self.OpacityBackground:SetPosition(1, 1)
    self.OpacityBackground:SetBackColor(Turbine.UI.Color(0.3, 0, 0, 0))
    self.OpacityBackground:SetVisible(true)
    self.OpacityBackground:SetMouseVisible(false)

    self.TopWindow = Turbine.UI.Window()
    self.TopWindow:SetParent(self)
    self.TopWindow:SetSize(self.width, 30)
    self.TopWindow:SetVisible(true)
    self.TopWindow:SetMouseVisible(false)

    self.NameLabel = Turbine.UI.Label()
    self.NameLabel:SetParent(self.TopWindow)
    self.NameLabel:SetSize(self.width, 15)
    self.NameLabel:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
    self.NameLabel:SetMouseVisible(false)
    self.NameLabel:SetMultiline(false)
    self.NameLabel:SetFont(Turbine.UI.Lotro.Font.LucidaConsole12)
    self.NameLabel:SetTop(2)

    self.DataLabel =  Turbine.UI.Label()
    self.DataLabel:SetParent(self.TopWindow)
    self.DataLabel:SetSize(self.width, 15)
    self.DataLabel:SetTop(15)
    self.DataLabel:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
    self.DataLabel:SetMouseVisible(false)
    self.DataLabel:SetMultiline(false)

    self.TimerContainer = Turbine.UI.ListBox()
    self.TimerContainer:SetParent(self)
    self.TimerContainer:SetVisible(true)
    self.TimerContainer:SetSize(self.width, 1000)
    self.TimerContainer:SetPosition(0, 30)
    self.TimerContainer:SetMouseVisible(false)

    self.SortMethod = function(a, b)
        if a:GetRemainingTime() == b:GetRemainingTime() then
            return a:GetText() < b:GetText()
        else
            return a:GetRemainingTime() > b:GetRemainingTime()
        end
    end

    local moveBox = MoveBox(self, self.width, self.height, "Target Window")
    
    self.ToggleMove = function()
        moveBox:ToggleMove()
    end

    -- Set callbacks
    self.callback["TargetChanged"] = AddCallback(self.player, "TargetChanged", function() self:TargetEvents() end)
    self.callback["InCombatChanged"] = AddCallback(self.player, "InCombatChanged", function() self:CombatUpdate() end)

    -- Handle initial situation
    self:CombatUpdate()
    self:TargetEvents()
    self:SetNameText()
end

-- Clean callbacks
function TargetWindowOld:RemoveCallbacks()
    RemoveCallback(self.player, "TargetChanged", self.callback["TargetChanged"])
    RemoveCallback(self.player, "InCombatChanged", self.callback["InCombatChanged"])
end

-- Looks at effects on targets and adds or removes timers based on conditions
function TargetWindowOld:UpdateEffects()
    if not(self.targetEffects) then return end

    for i = 1, self.targetEffects:GetCount() do

        -- Filter already handled IDs
        local currentEffect = self.targetEffects:Get(i)
        if not(self.targetEffectsID[currentEffect:GetID()]) then

            self.targetEffectsID[currentEffect:GetID()] = true 

            -- Check if we should add timer
            if self.ShowAll or self.targetEffectsDictionary[currentEffect:GetName()] then

                -- Only add id if not permanent effect
                if currentEffect:GetDuration() > 999999 then 
                    self.targetEffectsID[currentEffect:GetID()] = nil
                end

                self:NewTimer(currentEffect)
            end

        end
    end
end

-- Makes or updates timer based on effect
function TargetWindowOld:NewTimer(effect)
    local effectName = effect:GetName()
    local numberLessName = string.match(effectName, "%D+")
    local item = self.Timers[numberLessName]

    -- Check if timer already exists
    if item then
        -- Timer already exists
        if item.permanent then
            if effectName ~= item.effectName then
                item:ChangeName(effectName)
                item.effectName = effectName
                item:ChangeIcon(effect)
            end
            item.permanentTimer = 0
            return
        else
            if item.ID == effect:GetID() then
                return
            else
                item.ID = effect:GetID()
                item:ChangeIcon(effect)
                if effectName ~= item.effectName then
                    item:ChangeName(effectName)
                    item.effectName = effectName
                end
                -- Update timer for new effect 
                item:ChangeDuration(effect:GetDuration() - (Turbine.Engine.GetGameTime() - effect:GetStartTime()), effect:GetDuration())
                self.ResortTimers = true
                return
            end
        end
    end

    -- Haven't seen this effect before so we make a timer
    local timer = EffectTimer(nil, self.width, self.TimerHeight, effect, effectName, self.font, self.colourScheme, (effect:GetDuration() < 999999))
    timer.ID = effect:GetID()
    timer.effectName = effectName
    timer.permanentTimer = 0
    timer.permanent = (effect:GetDuration() > 999999)
    timer.numberLessName = numberLessName

    self.TimerContainer:AddItem(timer)
    self.Timers[numberLessName] = timer

    self.ResortTimers = true
end

-- Handles setting/resetting data of a target
function TargetWindowOld:TargetEvents()

    self:Clear()

    -- Start regathering info
    self.target = self.player:GetTarget()

    if self.target == self.player then
        self:Clear()
    end

    if self.target then 

        -- Make sure we can see the window now that we have a target
        self:SetVisible(true)
        self:SetWantsUpdates(true)

        self.targetName = self.target:GetName()

        -- Check if target can have effects
        if self.target.GetEffects then

            -- Set current data
            self.targetTime = Turbine.Engine.GetGameTime()
            self.targetHealth = self.target:GetMorale()
            -- Only update if looking at effects
            if self.DoEffects then
                self.targetEffects =  self.target:GetEffects()
                self.targetEffectsID = {}
            end

            -- Force first update
            self:CombatUpdate()
        end
    else
        -- No target -> Hide window
        self:SetWantsUpdates(false)
        self:SetVisible(false)
    end

    -- Handle label of window
    self:SetNameText()
end

-- Clear all timers
function TargetWindowOld:Clear()
    -- Destroy all timers in the table and remove them from table
    for key, value in pairs(self.Timers) do
        self.Timers[key] = nil
        value:SetVisible(false)
        self.TimerContainer:RemoveItem(value)
    end

    -- Reset all variables
    self.targetEffects = nil
    self.target = nil
    self.targetTime = 0
    self.targetHealth = 0
    self.targetEffectsID = nil
    self.Timers = nil
    self.targetName = nil

    collectgarbage()

    self.targetEffectsID = {}
    self.Timers = {}
end

-- Handle an update to combat state
function TargetWindowOld:CombatUpdate()
    if self.player:IsInCombat() then

        -- If we enter combat and have a target setup tracking
        if self.target then
            self:SetWantsUpdates(true)
            if self.target.GetMorale then
                self.targetHealth = self.target:GetMorale()
            end
        end

        -- Get time when we start tracking combat on target
        self.targetTime = Turbine.Engine.GetGameTime()
    end
end

-- Handle setting name text with or without correct data
function TargetWindowOld:SetNameText()
    -- Only update if we have a target
    if self.target then

        -- Check if we received data from the game
        if self.target.GetEffects and self.target:GetMaxMorale() ~= 0 then

            -- Get target morale
            local targetMorale = self.target:GetMorale()

            -- Get time between starting to track and now
            local delta = Turbine.Engine.GetGameTime() - self.targetTime

            -- Update if we have a health change and time since target is more than 1 second
            if self.targetHealth < targetMorale and delta < 1 then
                self.targetHealth = targetMorale
            end

            -- Format dps string
            if delta > 0 then
                self.DataLabel:SetText(string.format("%.2f%% DPS: %.2fK", (targetMorale / self.target:GetMaxMorale() * 100), (self.targetHealth - targetMorale) / delta / 1000))
            else
                self.DataLabel:SetText(string.format("%.2f%%", (targetMorale / self.target:GetMaxMorale() * 100)))
            end

            -- Set name
            self.NameLabel:SetText(self.targetName)
        else
            -- Only set name in case we haven't received data yet or will never receive
            self.NameLabel:SetText(self.targetName)
            self.DataLabel:SetText("")
        end
    else
        -- Set basic state
        self.NameLabel:SetText("-")
        self.DataLabel:SetText("")
    end
end

-- Update loop
function TargetWindowOld:Update()   
    -- What is the time?
    local currentGameTime = Turbine.Engine.GetGameTime()

    -- Get time passed since last handled update
    local delta = currentGameTime - self.oldGameTime

    -- See if we want to update the window
    if delta < self.UpdateDelay then return end

    -- Update effects
    if self.TimeSinceLastCheck > self.CheckDelay then
        if self.DoEffects then
            self:UpdateEffects()
        end
        
        self.TimeSinceLastCheck = 0

        --Update text
        self:SetNameText()
    else
        self.TimeSinceLastCheck = self.TimeSinceLastCheck + delta
    end

    for _, timer in pairs(self.Timers) do
        -- Check for permanent timer
        if timer.permanent then
            timer.permanentTimer = timer.permanentTimer + delta
            if timer.permanentTimer > self.PermDelay then
                self.Timers[timer.numberLessName] = nil
                timer:SetVisible(false)
                self.TimerContainer:RemoveItem(timer)
            end    
        else
            -- Update timer
            timer:UpdateText(delta)
            if timer:GetRemainingTime() <= 0 then
                self.Timers[timer.numberLessName] = nil
                timer:SetVisible(false)
                self.TimerContainer:RemoveItem(timer)
            end
        end
    end

    -- Resort timers
    if self.ResortTimers then
        self.TimerContainer:Sort(self.SortMethod)
    end

    self.oldGameTime = currentGameTime
end