DropDownMenuControl = class(Turbine.UI.Window)
function DropDownMenuControl:Constructor(parent, height, name)
    Turbine.UI.Window.Constructor(self)

    self.height = height
    self.width = 400
    self.itemWidth = 380
    self.hideable = false
    self.listboxHeight = self.height - 20
    
    if parent then
        self:SetParent(parent)
    end
    self:SetVisible(false)
    self:SetSize(self.width, self.height)
    self:SetMouseVisible(true)
    self:SetBackColor(Turbine.UI.Color(0, 0, 0))
    self:SetZOrder(100)

    self.headerLabel = Turbine.UI.Label()
    self.headerLabel:SetParent(self)
    self.headerLabel:SetPosition(5, 0)
    self.headerLabel:SetSize(self.width - 25, 20)
    self.headerLabel:SetMouseVisible(true)
    self.headerLabel:SetFont(Turbine.UI.Lotro.Font.BookAntiqua24)
    self.headerLabel:SetFontStyle(Turbine.UI.FontStyle.Outline)
    self.headerLabel:SetOutlineColor(Turbine.UI.Color.Black)
    self.headerLabel:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft)
    self.headerLabel:SetText(name or "Dropdown Menu")


    self.closeButton = Turbine.UI.Control()
    self.closeButton:SetParent(self)
    self.closeButton:SetPosition(self.width - 20, 0)
    self.closeButton:SetSize(20, 20)
    self.closeButton:SetMouseVisible(true)
    self.closeButton:SetBackColor(Turbine.UI.Color.Red)
    self.closeButton:SetZOrder(100)
    self.closeButton.MouseClick = function()
        self:SetVisible(false)
    end

    self.SelectionContainerPopOut = Turbine.UI.Control()
    self.SelectionContainerPopOut:SetParent(self)
    self.SelectionContainerPopOut:SetPosition(5, 20)
    self.SelectionContainerPopOut:SetSize(self.width - 10, self.listboxHeight - 10)
    self.SelectionContainerPopOut:SetMouseVisible(false)
    self.SelectionContainerPopOut:SetBackColor(Turbine.UI.Color(0.375, 0.375, 0.375))
    self.SelectionContainerPopOut:SetZOrder(100)

    self.PopOoutContainer = Turbine.UI.ListBox()
    self.PopOoutContainer:SetParent(self.SelectionContainerPopOut)
    self.PopOoutContainer:SetSize(380, self.listboxHeight - 10)
    self.PopOoutContainer:SetPosition(0, 0)
    self.PopOoutContainer:SetVisible(true)
    self.PopOoutContainer:SetBackColor(GetRandomColour())
    self.PopOoutContainer:SetOrientation(Turbine.UI.Orientation.Vertical)

    self.PopOoutContainerScrollbar = Turbine.UI.Lotro.ScrollBar()
    self.PopOoutContainerScrollbar:SetOrientation(Turbine.UI.Orientation.Vertical)
    self.PopOoutContainerScrollbar:SetParent(self.SelectionContainerPopOut)
    self.PopOoutContainerScrollbar:SetPosition(self.itemWidth, 0)
    self.PopOoutContainerScrollbar:SetSize(10, self.listboxHeight - 10)
    self.PopOoutContainerScrollbar:SetBackColor(Turbine.UI.Color(0.5, 0.5, 0.5))
    self.PopOoutContainer:SetVerticalScrollBar( self.PopOoutContainerScrollbar)

    self.VisibleChanged = function()
        if self:IsVisible() then
            local x, y = self:GetMousePosition()
            local oldX, oldY = self:GetPosition()
            self:SetPosition((x + oldX) - 200, y + oldY + 5)
        end
    end
end

function DropDownMenuControl:AddItem(name, func, infoBoxes, infoWindow, icon)
    local ListItem = Turbine.UI.Control()
    ListItem:SetSize(self.itemWidth, 24)
    ListItem:SetMouseVisible(true)
    ListItem:SetBackColor(Turbine.UI.Color(0.5, 0.5, 0.5))

    local BackGroundControl = Turbine.UI.Control()
    BackGroundControl:SetParent(ListItem)
    BackGroundControl:SetSize(self.itemWidth, 20)
    BackGroundControl:SetPosition(0, 2)
    BackGroundControl:SetMouseVisible(false)
    BackGroundControl:SetBackColor(Turbine.UI.Color(0.375, 0.375, 0.375))

    if icon then
        local IconControl = Turbine.UI.Control()
        IconControl:SetParent(ListItem)
        IconControl:SetSize(16, 16)
        IconControl:SetPosition(0, 2)
        IconControl:SetMouseVisible(false)
        IconControl:SetBackground(icon)
    end


    local NameLabel = Turbine.UI.Label()
    NameLabel:SetParent(ListItem)
    NameLabel:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft)
    NameLabel:SetFont(Turbine.UI.Lotro.Font.BookAntiquaBold18)
    NameLabel:SetFontStyle(Turbine.UI.FontStyle.Outline)
    NameLabel:SetOutlineColor(Turbine.UI.Color(0, 0, 0))
    NameLabel:SetSize(self.itemWidth, 20)
    NameLabel:SetPosition(icon and 16 or 0, 0)
    NameLabel:SetMouseVisible(false)
    NameLabel:SetText(name)

    if infoBoxes and infoWindow then
        ListItem.cctypeInfoShow = UI.InfoButton(NameLabel, self.itemWidth, infoBoxes, infoWindow)
    end
    
    ListItem.MouseClick = function()
        func()
    end

    ListItem.MouseEnter = function()
        BackGroundControl:SetBackColor(Turbine.UI.Color(0.5, 0.5, 0.5))
    end

    ListItem.MouseLeave = function()
        BackGroundControl:SetBackColor(Turbine.UI.Color(0.375, 0.375, 0.375))
    end

    self.PopOoutContainer:AddItem(ListItem)
end

function DropDownMenuControl:ToggleVisible()
    self:SetVisible(not(self:IsVisible()))
end