GraphWindow = class(Turbine.UI.Window)
function GraphWindow:Constructor(parent, data, x, y, title, width, height, borderWidth, titleHeight, itemWidth, dotWidth, lineWidth, itemColour, dotColour, dotSelectedColour, lineColour)
    Turbine.UI.Window.Constructor(self)

    -- Numerical values
    self.width = width or 800
    self.height = height or 320
    self.outerBorder = borderWidth or 2
    self.scrollbarHeight = 10
    self.titleHeight = titleHeight or 20
    self.listBoxHeight = self.height - 2 * self.outerBorder - self.scrollbarHeight - self.titleHeight
    self.itemWidth = itemWidth or 10
    self.lowestValue = 0
    self.highestValue = 10
    self.dotWidth = dotWidth or 5
    self.lineWidth = lineWidth or 2
    self.itemWidthAdjusted = self.itemWidth - self.dotWidth / 2
    self.itemWidthSquared = math.pow(self.itemWidthAdjusted, 2)
    self.listBoxWidth = self.width - 2 * self.outerBorder
    self.minItem = 0
    self.maxItem = math.floor(self.listBoxWidth / self.itemWidth)
    self.infoWindow = PrimePlugins.RaidTools.UI.InfoWindow()
    self.data = data or {}

    -- Colour values
    self.itemColour = itemColour or Turbine.UI.Color(1, 0, 0.5, 0.6)
    self.dotColour = dotColour or Turbine.UI.Color.NavajoWhite
    self.dotSelectedColour = dotSelectedColour or Turbine.UI.Color.DarkRed
    self.lineColour = lineColour or Turbine.UI.Color.White

    if parent then
        self:SetParent(parent)
    end
    self:SetVisible(true)
    self:SetPosition(x or 500, y or 500)
    self:SetSize(self.width, self.height)
    self:SetBackColor(Turbine.UI.Color.Black)

    self.titleBar = Turbine.UI.Label()
    self.titleBar:SetParent(self)
    self.titleBar:SetVisible(true)
    self.titleBar:SetSize(self.width - 2 * self.outerBorder, self.titleHeight)
    self.titleBar:SetBackColor(Turbine.UI.Color.Black)
    self.titleBar:SetPosition(self.outerBorder, self.outerBorder)
    self.titleBar:SetMouseVisible(false)
    self.titleBar:SetFontStyle(Turbine.UI.FontStyle.Outline)
    self.titleBar:SetOutlineColor(Turbine.UI.Color.Black)
    self.titleBar:SetFont(Turbine.UI.Lotro.Font.BookAntiquaBold18)
    self.titleBar:SetForeColor(Turbine.UI.Color.White)
    self.titleBar:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
    self.titleBar:SetText(title or "Graph title")


    self.graph = Turbine.UI.ListBox()
    self.graph:SetParent(self)
    self.graph:SetSize(self.listBoxWidth, self.listBoxHeight)
    self.graph:SetPosition(self.outerBorder, self.titleHeight + self.outerBorder)
    self.graph:SetBackColor(self.itemColour)
    self.graph:SetOrientation(Turbine.UI.Orientation.Horizontal)

    self.scrollbar = Turbine.UI.Lotro.ScrollBar()
    self.scrollbar:SetParent(self)
    self.scrollbar:SetPosition(self.outerBorder, self.listBoxHeight + self.outerBorder + self.titleHeight)
    self.scrollbar:SetSize(self.listBoxWidth, self.scrollbarHeight)
    self.scrollbar:SetBackColor(Turbine.UI.Color.Black)
    self.graph:SetHorizontalScrollBar(self.scrollbar)

    self:AddPositionItem()

    self.testData = {
        [1] = {1, 5},
        [2] = {2, 7},
        [3] = {3, 10},
        [4] = {4, 15},
        [5] = {5, 24.9},
        [6] = {6, 25},
        [7] = {7, 15},
        [8] = {8, 10},
        [9] = {9, 7},
        [10] = {10, 5},
    }

    self.testData = {}

    for i = 1, 1000 do
        self.testData[i] = {i, math.random(0, 5000)}
    end

    for _, value in pairs(self.data) do if value[2] > self.highestValue then self.highestValue = value[2] end end

    for key, value in ipairs(self.data) do 
        self:ConstructGraphItem(self.data[key - 1] and self.data[key - 1][2] or 0, value[2], value[1], key)
    end 

    self.leftCover = Turbine.UI.Window()
    self.leftCover:SetParent(self)
    self.leftCover:SetVisible(true)
    self.leftCover:SetSize(self.itemWidth, self.listBoxHeight)
    self.leftCover:SetBackColor(Turbine.UI.Color.Black)
    self.leftCover:SetPosition(self.outerBorder, self.outerBorder + self.titleHeight)
    self.leftCover:SetMouseVisible(false)

    self.rightCover = Turbine.UI.Window()
    self.rightCover:SetParent(self)
    self.rightCover:SetVisible(true)
    self.rightCover:SetSize(self.itemWidth, self.listBoxHeight)
    self.rightCover:SetBackColor(Turbine.UI.Color.Black)
    self.rightCover:SetPosition(self.listBoxWidth + self.outerBorder - self.itemWidth, self.outerBorder + self.titleHeight)
    self.rightCover:SetMouseVisible(false)
end

function GraphWindow:ConstructGraphItem(value, nextValue, x, ID)
    local listBoxItem = Turbine.UI.Window()
    listBoxItem:SetVisible(true)
    listBoxItem:SetSize(self.itemWidth, self.listBoxHeight)
    --listBoxItem:SetBackColor(self.itemColour)
    listBoxItem:SetMouseVisible(false)
    listBoxItem.ID = ID
    listBoxItem.X = x
    listBoxItem.Value = value

    local dotOneY = (1 - value / (self.highestValue - self.lowestValue)) * (self.listBoxHeight - self.dotWidth)
    local dotTwoY = (1 - (nextValue or 1) / (self.highestValue - self.lowestValue)) * (self.listBoxHeight - self.dotWidth)

    if math.abs(dotOneY - dotTwoY) <= self.dotWidth then
        listBoxItem.line = Turbine.UI.Window()
        listBoxItem.line:SetParent(listBoxItem)
        listBoxItem.line:SetSize(self.itemWidth, self.lineWidth)
        listBoxItem.line:SetPosition(0, dotOneY)
        listBoxItem.line:SetBackColor(self.lineColour)
        listBoxItem.line:SetVisible(true)
        listBoxItem.line:SetMouseVisible(false)

        listBoxItem.PositionChanged = function()
            if self.minItem < listBoxItem.ID and self.maxItem > listBoxItem.ID then
                listBoxItem:SetVisible(true)
            else
                listBoxItem:SetVisible(false)
            end
        end

        listBoxItem.dotOne = Turbine.UI.Control()
        listBoxItem.dotOne:SetParent(listBoxItem)
        listBoxItem.dotOne:SetSize(self.dotWidth, self.dotWidth)
        listBoxItem.dotOne:SetBackColor(self.dotColour)
        listBoxItem.dotOne:SetPosition(0, dotOneY)
        listBoxItem.dotOne:SetMouseVisible(false)
    
        listBoxItem.hoverBox = Turbine.UI.Control()
        listBoxItem.hoverBox:SetParent(listBoxItem)
        listBoxItem.hoverBox:SetSize(20, 20)
        listBoxItem.hoverBox:SetPosition(-5, dotOneY - 5)
    
        listBoxItem.hoverBox.MouseEnter = function()
            self.infoWindow:View("X: " ..listBoxItem.X .. "\nY: " .. listBoxItem.Value)
            listBoxItem.dotOne:SetBackColor(self.dotSelectedColour)
        end
    
        listBoxItem.hoverBox.MouseLeave = function()
            self.infoWindow:SetVisible(false)
            listBoxItem.dotOne:SetBackColor(self.dotColour)
        end


        self.graph:AddItem(listBoxItem)
        return 
    end

    local yDiff =  math.pow(dotTwoY - dotOneY, 2)
    local lineLength = math.sqrt(self.itemWidthSquared + yDiff)

    local firstProduct = math.pow(lineLength, 2) + self.itemWidthSquared - yDiff
    local secondProduct = 2 * lineLength * self.itemWidthAdjusted

    listBoxItem.lineAngle = math.deg(math.acos(firstProduct/secondProduct))

    local adjustment = (lineLength / math.sin(math.rad(90))) * math.sin(math.rad(180 - listBoxItem.lineAngle - 90))

    listBoxItem.line = Turbine.UI.Window()
    listBoxItem.line:SetParent(listBoxItem)
    listBoxItem.line:SetSize(lineLength, lineLength)
    listBoxItem.line:SetVisible(true)
    listBoxItem.line:SetMouseVisible(false)

    listBoxItem.lineTexture = Turbine.UI.Control()
    listBoxItem.lineTexture:SetParent(listBoxItem.line)
    listBoxItem.lineTexture:SetSize(lineLength, self.lineWidth)
    listBoxItem.lineTexture:SetBackColor(self.lineColour)
    listBoxItem.lineTexture:SetMouseVisible(false)

    -- [[
    if dotOneY < dotTwoY then
        listBoxItem.line:SetRotation({x=0,y=0,z=-listBoxItem.lineAngle})
        listBoxItem.line:SetPosition(self.itemWidth / 2, dotOneY - adjustment / 2  )
        listBoxItem.lineTexture:SetPosition(0, lineLength - self.lineWidth)
    else
        listBoxItem.line:SetRotation({x=0,y=0,z=listBoxItem.lineAngle})
        listBoxItem.line:SetPosition(self.itemWidth / 2, dotTwoY + adjustment / 2  )
    end
    --]]

    listBoxItem.dotOne = Turbine.UI.Control()
    listBoxItem.dotOne:SetParent(listBoxItem)
    listBoxItem.dotOne:SetSize(self.dotWidth, self.dotWidth)
    listBoxItem.dotOne:SetBackColor(self.dotColour)
    listBoxItem.dotOne:SetPosition(0, dotOneY)
    listBoxItem.dotOne:SetMouseVisible(false)

    listBoxItem.hoverBox = Turbine.UI.Control()
    listBoxItem.hoverBox:SetParent(listBoxItem)
    listBoxItem.hoverBox:SetSize(20, 20)
    listBoxItem.hoverBox:SetPosition(-5, dotOneY - 5)

    listBoxItem.hoverBox.MouseEnter = function()
        self.infoWindow:View("X: " ..listBoxItem.X .. "\nY: " .. listBoxItem.Value)
        listBoxItem.dotOne:SetBackColor(self.dotSelectedColour)
    end

    listBoxItem.hoverBox.MouseLeave = function()
        self.infoWindow:SetVisible(false)
        listBoxItem.dotOne:SetBackColor(self.dotColour)
    end

    listBoxItem.PositionChanged = function()
        if self.minItem < listBoxItem.ID and self.maxItem > listBoxItem.ID then
            listBoxItem:SetVisible(true)
            if dotOneY < dotTwoY then
                listBoxItem.line:SetRotation({x=0,y=0,z=-listBoxItem.lineAngle})
                listBoxItem.line:SetPosition(self.itemWidth / 2, dotOneY - adjustment / 2  )
                listBoxItem.lineTexture:SetPosition(0, lineLength - self.lineWidth)
            else
                listBoxItem.line:SetRotation({x=0,y=0,z=listBoxItem.lineAngle})
                listBoxItem.line:SetPosition(self.itemWidth / 2, dotTwoY + adjustment / 2  )
            end
        else
            listBoxItem:SetVisible(false)
        end
    end

    self.graph:AddItem(listBoxItem)
end

function GraphWindow:AddPositionItem()
    local listBoxItem = Turbine.UI.Window()
    listBoxItem:SetVisible(true)
    listBoxItem:SetSize(self.itemWidth, self.listBoxHeight)
    listBoxItem:SetBackColor(self.itemColour)
    listBoxItem:SetMouseVisible(false)
    listBoxItem:SetVisible(false)
    listBoxItem.PositionChanged = function()
        self.minItem = math.floor(math.abs((listBoxItem:GetLeft() - self.itemWidth)/self.itemWidth))
        self.maxItem = math.floor(math.abs((listBoxItem:GetLeft() - self.listBoxWidth)/self.itemWidth))
    end
    self.graph:AddItem(listBoxItem)
end

function GraphWindow:Unload()
    self.graph:ClearItems()
    self:SetParent(nil)
    local controlList = self:GetControls()
    for i = 1, controlList:GetCount() do
        controlList:RemoveAt(1)
    end
    for key, _ in pairs(self) do
        self[key] = nil
    end
    collectgarbage()
end