function Round(num, numDecimalPlaces)
    local mult = 10^(numDecimalPlaces or 0)
    local number = math.floor(num * mult + 0.5) / mult
    if (number % 1) == 0 and numDecimalPlaces ~= 0 then
        return tostring(number .. ".0")
    else
        return number
    end
end

function tablelength(T)
    local count = 0
    if T ~= nil then
        for _ in pairs(T) do count = count + 1 end
    end
    return count
end

function Tablelength(T)
    return tablelength(T)
end

function Compare(a, b)
    if a ~= nil and b ~= nil then
        return a[2] < b[2]
    end
end

math.round = function(number) return math.floor(number + 0.5) end

function ChatOutput(s) Turbine.Shell.WriteLine(string.format("%s%s", "<rgb=#FF9900><Prime></rgb> ", tostring(s))) end

function DirectChatOutput(s) Turbine.Shell.WriteLine(s) end

function Print(s) Turbine.Shell.WriteLine(tostring(s)) end

function GetRandomColour() return(Turbine.UI.Color(math.random(), math.random(), math.random())) end

function AddListItem(text, box, ItemSelectedLabel, extraText)
        
    local listItem = Turbine.UI.Control()
    listItem:SetVisible(true)
    listItem:SetSize( 200, 15 )
    listItem:SetMouseVisible(true)

    listItem.MouseClick = function()
        ItemSelectedLabel:SetText(text)
    end

    local ttttext = text

    if extraText ~= nil then
        ttttext = text .. " - " .. extraText
    end

    local NameLabel = Turbine.UI.Label()
    NameLabel:SetParent(listItem)
    NameLabel:SetSize(200, 15)
    NameLabel:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
    NameLabel:SetMouseVisible(false)
    --NameLabel:SetMultiline(false)
    NameLabel:SetTop(2)
    NameLabel:SetText(ttttext)

    box:AddItem( listItem )
end

function FormatBigNumber(number)
    if number == nil then return 0 end
    if number < 1000 then
        return string.format("%d", number)
    elseif number < 1e6 then
        return string.format("%.1fk", math.floor(number / 1e3))
    elseif number < 1e9 then
        return string.format("%.1fm", math.floor(number / 1e6))
    elseif number < 1e12 then 
        return string.format("%.1fb", math.floor(number / 1e9))
    else
        return string.format("%.1ft", math.floor(number / 1e12))
    end
end


function FormatBigNumberVitals2(number)
    if number == nil then return 0 end
    if number < 1000 then
        return string.format("%d", number)
    elseif number >= 1000 and number < 1000000 then
        return string.format("%.1fK", number / 1000)
    elseif number >= 1000000 and number < 1000000000 then
        return string.format("%.1fM", number / 1000000)
    elseif number >= 1e9  and number < 1000000000000 then 
        return string.format("%.1fB", number / 1000000000)
    elseif number >= 1e12  then
        return string.format("%.1fT", number / 1000000000000)
    else
        return number
    end
end

function FormatBigNumberVitals(number)
    if number == nil then return 0 end

    local suffixes = {"", "K", "M", "B", "T"}
    local index = 1

    while number >= 1000 and index < #suffixes do
        number = number / 1000
        index = index + 1
    end

    return string.format("%.1f%s", number, suffixes[index])
end


function spairs(t, order)
    if not(t) then return end
    -- collect the keys
    local keys = {}
    for k in pairs(t) do keys[#keys+1] = k end

    -- if order function given, sort by it by passing the table and keys a, b,
    -- otherwise just sort the keys 
    if order then
        table.sort(keys, function(a,b) return order(t, a, b) end)
    else
        table.sort(keys)
    end

    -- return the iterator function
    local i = 0
    return function()
        i = i + 1
        if keys[i] then
            return keys[i], t[keys[i]]
        end
    end
end

function GetLastReset()
    -- (7 * SecondsInDay)
    -- return(math.floor((Turbine.Engine.GetLocalTime() - BeginningOfPrimeTime)/SecondsInWeek) * SecondsInWeek + BeginningOfPrimeTime) 
    return(math.floor((Turbine.Engine.GetLocalTime() - 1578535200)/(7 * 86400)) * (7 * 86400) + 1578535200)
end

function GetLastDailyReset()
    --return(math.floor((Turbine.Engine.GetLocalTime() - BeginningOfPrimeTime)/SecondsInDay) * SecondsInDay + BeginningOfPrimeTime)
    -- 
    return(math.floor((Turbine.Engine.GetLocalTime() - 1578535200)/86400) * 86400 + 1578535200)
end

function GetLastT1Reset()
    local lastReset = GetLastReset()
    if lastReset + (86400 * 2) > Turbine.Engine.GetLocalTime() then
        return lastReset
    elseif lastReset + (86400 * 4) > Turbine.Engine.GetLocalTime() then
        return lastReset + 86400 * 2
    elseif lastReset + (86400 * 7) > Turbine.Engine.GetLocalTime() then 
        return lastReset + 86400 * 4
    end
end

function HandleLootUpdate()
    CheckLootTable()
    local lastWeeklyReset = GetLastReset()
    local lastT1Reset = GetLastT1Reset()
    local lastDailyReset = GetLastDailyReset()

    if GetLastReset() > LootLocks.LastReset then
        ClearWeeklyLootTable()
    else
        if GetLastT1Reset() > LootLocks.LastT1Reset then
            ClearT1RaidLootTable()
        end

        if GetLastDailyReset() > LootLocks.LastDailyReset then
            ClearDailyLootTable()
        end
    end
end

function ClearWeeklyLootTable()   
    -- Clear table
    LootLocks = {}

    -- Initialize values
    LootLocks.Locks = {}
    LootLocks.RaidLocks = {}
    LootLocks.WeeklyLocks = {}
    LootLocks.Characters = {}
    ClearT1RaidLootTable()
    ClearDailyLootTable()

    -- Set last resets
    LootLocks.LastReset = GetLastReset()

    ChatOutput("Cleared Weekly Locks")
end

function ClearT1RaidLootTable()
    LootLocks.T1RaidLocks = {}
    LootLocks.LastT1Reset = GetLastT1Reset()

    ChatOutput("Cleared T1 Raid Locks")
end

function ClearDailyLootTable()
    LootLocks.DailyLocks = {}
    LootLocks.T1Locks = {}
    LootLocks.LastDailyReset = GetLastDailyReset()

    ChatOutput("Cleared Daily Locks")
end

function CheckLootTable()
    local tempLoot = LootLocks

    LootLocks = {}
    
    -- Instances
    LootLocks.Locks = {}
    LootLocks.T1Locks = {}

    -- Raids
    LootLocks.RaidLocks = {}
    LootLocks.T1RaidLocks = {}

    -- Quests
    LootLocks.WeeklyLocks = {}
    LootLocks.DailyLocks = {}
    LootLocks.Characters = {}

    -- Reset times
    LootLocks.LastT1Reset = 1578535200
    LootLocks.LastDailyReset = 1578535200
    LootLocks.LastReset = 1578535200

    for key, value in pairs(LootLocks) do
        if tempLoot[key] ~= nil then
            LootLocks[key] = tempLoot[key]
        end
    end
end

function TimeDeltaToString(time)
    local minutes = math.floor(time / 60)

    local seconds = math.floor(time % 60)
    if seconds < 10 then
        seconds = "0" .. seconds
    end

    local milliSeconds = time - math.floor(time)
    
    if milliSeconds == 0 then milliSeconds = "0.0000" end

    return(minutes .. ":" .. seconds .. "." .. string.sub(milliSeconds, 3, 4))
end

function CombatTimeDeltaToString(time)
    return string.format("%01d:%02d", math.floor(time / 60), math.floor(time % 60))
end

function FormatTimeString(time)
    local minutes = math.floor(time / 60)
    local seconds = string.format("%02.f", math.floor(time % 60))
    local milliseconds = string.format("%02.f", math.floor(time * 100 % 100))

    return(minutes .. ":" .. seconds .. "." .. milliseconds)
end

function FormatTimeStringNoMilli(time)
    local minutes = math.floor(time / 60)
    local seconds = string.format("%02.f", math.floor(time % 60))

    return(minutes .. ":" .. seconds)
end


function AddCallback(object, event, callback)
    if (object[event] == nil) then
        object[event] = callback
    else
        if (type(object[event]) == "table") then
            table.insert(object[event], callback)
        else
            object[event] = {object[event], callback}
        end
    end
    return callback
end

function RemoveCallback(object, event, callback)
    if (object[event] == callback) then
        object[event] = nil
    else
        if (type(object[event]) == "table") then
            for i = 1, #object[event] do
                if (object[event][i] == callback) then
                    table.remove(object[event], i)
                    break
                end
            end
        end
    end
end

function math.clamp(value, lower, upper)
    return math.max(lower, math.min(upper, value))
end

function BoolToString(bool) if bool then return ("True") else return("False") end end

function IsNumber(x)
    if tonumber(x) then return true end
    return false
end

function StringifyDuration2(duration, round)
    if not(duration) then return "" end
    if round then
        if duration <= 60 then
            return string.format("%." .. round .. "f", duration)
        elseif duration <= 3600 then
            return string.format("%i:%02." .. round .. "f", duration / 60, duration % 60,duration % 60)
        elseif duration < 9000000 then
            return string.format("%i:%02d:%02." .. round .. "f", duration / 3600, (duration % 3600) / 60, duration % 60)
        else
            return ""
        end
    else
        if duration <= 60 then
            return string.format("%i", duration)
        elseif duration <= 3600 then
            return string.format("%i:%02d", duration / 60, duration % 60)
        elseif duration < 9000000 then
            return string.format("%i:%02d:%02d", duration / 3600, (duration % 3600) / 60, duration % 60)
        else
            return ""
        end
    end
end

function StringifyDuration(duration, round)
    if not duration then return "" end
    if round then
        if duration <= 60 then
            return string.format("%." .. round .. "f", duration)
        elseif duration <= 3600 then
            return string.format("%i:%02." .. round .. "f", duration / 60, duration % 60, duration % 60)
        else
            return string.format("%i:%02d:%02." .. round .. "f", duration / 3600, (duration % 3600) / 60, duration % 60)
        end
    else
        if duration <= 60 then
            return string.format("%i", duration)
        elseif duration <= 3600 then
            return string.format("%i:%02d", duration / 60, duration % 60)
        else
            return string.format("%i:%02d:%02d", duration / 3600, (duration % 3600) / 60, duration % 60)
        end
    end
end


function ReplaceTokens(str, tokens, count)
    local result = str:gsub("<(.-)>", function(token)
        local num = tonumber(token:match("^%d+$"))
        if num then
            return tostring(num + count)
        else
            return tokens[token] or ""
        end
    end)
    return result
end



function PrintTable(T)
    if T == nil or type(T) ~= "table" then return end

    for key, value in pairs(T) do
        if type(key) == "table" then 
            PrintTable(key) 
        else
            ChatOutput("Key: " .. tostring(key))
        end

        if type(value) == "table" then 
            PrintTable(value) 
        else
            ChatOutput("Value: " .. tostring(value))
        end
    end
end

function ColourTupleToTurbineColour(tuple)
    if tuple then
        if type(tuple[1]) == "table" then
            local colourTable = {}
            for key, value in pairs(tuple) do
                if value then
                    colourTable[key] = Turbine.UI.Color(value[1], value[2], value[3], value[4])
                else
                    colourTable[key] = nil
                end
            end

            return colourTable
        else
            return Turbine.UI.Color(tuple[1], tuple[2], tuple[3], tuple[4])
        end
    else
        return Turbine.UI.Color(0,0,0,0)
    end
end

function CombineTables(table1, table2)
    local result = {}

    -- Combine keys from table1
    for key, value in pairs(table1) do
        if type(value) == "table" and table2[key] and type(table2[key]) == "table" then
            -- Recursive combination if both values are tables
            result[key] = CombineTables(value, table2[key])
        elseif table2[key] then
            -- Add values if both keys exist
            result[key] = value + table2[key]
        else
            -- Copy value from table1 if no corresponding key in table2
            result[key] = value
        end
    end

    -- Add remaining keys from table2
    for key, value in pairs(table2) do
        if not table1[key] then
            -- Copy value from table2 if no corresponding key in table1
            result[key] = value
        end
    end

    return result
end