WindowHandler = class(Turbine.UI.Control)
function WindowHandler:Constructor()
    Turbine.UI.Control.Constructor(self)
    self:SetWantsKeyEvents(true)

    -- Window collection
    self.windows = {}
    self.handlers = {}
    self.tools = {}

    -- Check for reload data

    -- Setup player and party data
    self.player = Turbine.Gameplay.LocalPlayer.GetInstance()
    self.playerName = self.player:GetName()
    self.name = self.player:GetName()
    self.party = self.player:GetParty()

    -- Create parser
    self.handlers["ChatEventHandler"] = ChatEventHandler

    -- Create target Handler
    self.handlers["TargetHandler"] = TargetHandlerObject

    --Build current party to not break the plugin
    self.handlers["PlayerEffectHandler"] = PlayerEffectsHandler
    self.handlers["GroupEffectsHandler"] = GroupEffectsHandler

    -- Player Handler
    self.handlers["PlayerHandler"] = PlayerHandlerObject

    -- Action Handler
    self.handlers["ActionHandler"] = ActionHandlerObject

    -- Create the global upate handler
    self.handlers["Updater"] = Updater

    -- Countdown window
    self.windows["Countdown"] = countdownWindow

    -- Combat timer
    self.windows["Combat Timer"] = combatTimer
    self.handlers["DNF Tracker"] = rezTracker
    -- Skill & boss timers
    -- Boss Timers
    self.windows["Cooldown Timers"] = skillTimers
    self.windows["Active Timers"] = activeTimers
    self.windows["Skills Tracker"] = SkillUsageTracker

    -- Leader & Log Window
    self.windows["Leader Window"] = leaderWindow
    self.windows["Log Window"] = logWindow

    -- Options button & reload windows
    self.tools["Reload Window"] = reloadWindow
    self.tools["Options Button"] = optionsButton

    -- Improved ready check windows
    self.tools["Ready Handler"] = ReadyHandler

    -- Reload button
    self.tools["Reload Button"] = RButton

    -- Sound player
    self.windows["Sound Player"] = Sounds

    -- Swap window
    self.windows["Swap Warner"] = SwappyWarner

    -- Role Handler
    self.tools["Plugins Manager"] = PluginsManager

    -- Target Window
    self.windows["Target Window"] = Target

    -- Auras
    self.windows["Aura Handler"] = Auras

    -- Class tools
    self.windows["Class Tools"] = ClassTools

    -- Instance
    self.tools["Instance Window Button"] = IWB

    -- Create skill overlays from data
    self.windows["SkillBarHandler"] = SkillsHandler

    -- Handle first time use current version
    self.tools["FirstTimeSetup"] = Setup


    -- Import test env
    --import "PrimePlugins.RaidTools.TestFile"

    -- Set back to normal state
    Settings.ReloadData = false

    self.Hide = false

    self.ShowState = {}

    self.KeyDown = function(sender, args)
        if args.Action == 268435635 then
            if self.Hide then 
                self:ShowWindows()
            else
                self:HideWindows()
            end
        end
    end
end

function WindowHandler:ShowWindows()
    self.Hide = false
    for _, value in pairs(self.ShowState) do
        value:SetVisible(true)
    end
    self.ShowState = {}
    PrimeUIHandlerObject:HideWindow(true)
end

function WindowHandler:HideWindows()
    self.Hide = true
    for key, value in pairs(self.windows) do
        if value:IsVisible() then
            self.ShowState[key] = value
        end 
        value:SetVisible(false)
    end

    for key, value in pairs(self.tools) do
        if value.IsVisible then 
            if value:IsVisible() then
                self.ShowState[key] = value
            end
            value:SetVisible(false)
        end 
    end

    for key, value in pairs(self.handlers) do
        if value.IsVisible then 
            if value:IsVisible() then
                self.ShowState[key] = value
            end
            value:SetVisible(false)
        end 
    end
    PrimeUIHandlerObject:HideWindow(false)
end

function WindowHandler:CreatePatchWindow()
    if not(Patches) then
        Patches = PatchNotesWindow()
    else
        Patches:SetVisible(true)
    end
end

function WindowHandler:GetActiveTables()
    return self.windows, self.handlers, self.tools
end

function WindowHandler:Unload()
    self:SetParent(nil)
    local controlList = self:GetControls()
    for i = 1, controlList:GetCount() do
        controlList:RemoveAt(1)
    end
    for key, _ in pairs(self) do
        self[key] = nil
    end
    collectgarbage()
end

WindowsHandlerController = WindowHandler()

import "PrimePlugins.RaidTools.MoveHandler"