VitalWindow = class( Turbine.UI.Window)
function VitalWindow:Constructor(frameSettings, colourSettings)
    Turbine.UI.Window.Constructor(self)
	
    --self.layout = Turbine.UI.HorizontalLayout.LeftToRight
    self.frameSettings = frameSettings
    self.colourSettings = colourSettings

    local width = 254
    local height = 160 + 160 + 86

    local vitalWidth = 250 + 4
    local vitalHeight = 80 + 4
	
	self:SetPosition(self.frameSettings.FrameX, self.frameSettings.FrameY)
    self:SetSize(self.frameSettings.VitalWidth, self.frameSettings.HealthHeight + 320 )
    self:SetVisible(true)
    self:SetMouseVisible(false)


    if self.frameSettings.Aura then
        self.aura = Aura()
    end

    self.BuffBoxes = {}
    self.DebuffBoxes = {}

    self.effectAddedCallbacks = {}
    self.effectRemovedCallbacks = {}

    self.BuffResort = true
    self.DebuffResort = true

    self.UpdateCountdown = self.frameSettings.ShowCountdown

    self.Layout = Settings.Layout

    self.textSettings = Settings.Text

    self.BuffTracking = self.frameSettings.BuffTracking
    self.DebuffTracking = self.frameSettings.DebuffTracking

    self.IconScale = self.frameSettings.IconScale
    self.IconSize = 16 * self.IconScale

    self.debuffHeight = self.frameSettings.DebuffBoxHeight or 160

    --if self.Layout == 3 then self.debuffHeight = 150 end

    self.IconRow = 0
    self.IconRowHeight = self.debuffHeight - self.IconSize

    self.player = Turbine.Gameplay.LocalPlayer.GetInstance()
    self.ClassAttr = self.player:GetClassAttributes()
    self.ClassType = self.player:GetClass()

    self.playerCallbacks = {}
    self.attrCallbacks = {}
    self.effectCallbacks = {}

    self.ResourceText = self.frameSettings.ResourceText

    self.EffectPosition = Settings.EffectVersion

    self.oldGameTime = Turbine.Engine.GetGameTime()

    local offset = 0

    if self.ClassType == Turbine.Gameplay.Class.Champion or 
    self.ClassType == Turbine.Gameplay.Class.Hunter or 
    self.ClassType == Turbine.Gameplay.Class.RuneKeeper then
        if self.Layout == 1 then
            offset = 20
        elseif self.Layout == 2 then
            offset = 4
        end
    end
    
	self.vitalsFrame = Turbine.UI.Control()
	self.vitalsFrame:SetParent( self )
	self.vitalsFrame:SetPosition( 2, 2 )
    self.vitalsFrame:SetMouseVisible( true )
    self.vitalsFrame:SetSize(self.frameSettings.VitalWidth, self.frameSettings.HealthHeight)
    self.vitalsFrame:SetPosition(0, offset + self.frameSettings.HealthY)

    self.E = Turbine.UI.Lotro.EntityControl()
    self.E:SetParent (self.vitalsFrame)
    self.E:SetVisible (true)
    self.E:SetSize(self.frameSettings.VitalWidth, self.frameSettings.HealthHeight)
    self.E:SetEntity(self.player)
	
    self.moraleBar = VitalBar(self.frameSettings.HealthWidth - self.frameSettings.HealthBorder, self.frameSettings.HealthHeight, true, Turbine.UI.ContentAlignment.MiddleRight, nil, nil, self.frameSettings.ColourChanging, self.frameSettings.UseBackground)
    self.moraleBar:SetParent(self.vitalsFrame)

    -- Power bar
    self.powerBar = VitalBar(self.frameSettings.PowerWidth - self.frameSettings.HealthBorder, self.frameSettings.PowerHeight - self.frameSettings.HealthBorder, nil, nil, false, self.frameSettings.UseBackground)
    self.powerBar:SetPosition(self.frameSettings.PowerX, self.frameSettings.PowerY)
    self.powerBar:SetParent(self)
    local powerBackground
    local powerColour

    if self.ClassType == Turbine.Gameplay.Class.Beorning then
        powerBackground = PrimePlugins.RaidTools.UI.ColourTupleToTurbineColour(self.colourSettings["Wrath Background"])
        powerColour = PrimePlugins.RaidTools.UI.ColourTupleToTurbineColour(self.colourSettings.Wrath) 
    else
        powerBackground = PrimePlugins.RaidTools.UI.ColourTupleToTurbineColour(self.colourSettings["Power Background"])
        powerColour = PrimePlugins.RaidTools.UI.ColourTupleToTurbineColour(self.colourSettings.Power) 
    end

    self.powerBar:SetColours(powerBackground, powerColour, PrimePlugins.RaidTools.UI.ColourTupleToTurbineColour(self.colourSettings["Temporary Morale"]))

    self.powerBar:SetMouseVisible(false)
        
    self.healthText = Turbine.UI.Label()
    self.healthText:SetParent(self)
    self.healthText:SetForeColor(Turbine.UI.Color(1, 1, 1, 1))
    self.healthText:SetOutlineColor(Turbine.UI.Color(1, 0, 0, 0))
    self.healthText:SetFontStyle(Turbine.UI.FontStyle.Outline)
    self.healthText:SetFont(Turbine.UI.Lotro.Font[self.frameSettings.HealthFont])
    self.healthText:SetSize(self.frameSettings.VitalWidth, self.frameSettings.HealthHeight)
    self.healthText:SetMouseVisible(false)
    self.healthText:SetTextAlignment(Turbine.UI.ContentAlignment[self.frameSettings.HealthAlignment])
    self.healthText:SetPosition(self.frameSettings.HealthTextX, self.frameSettings.HealthTextY)
    self.healthText:SetZOrder(100)

    local colour
    local backgroundColor
    if self.frameSettings.UseBaseColour then
        colour = PrimePlugins.RaidTools.UI.ColourTupleToTurbineColour(self.colourSettings.Base)
        backgroundColor = PrimePlugins.RaidTools.UI.ColourTupleToTurbineColour(self.colourSettings.Base)
    else
        colour = PrimePlugins.RaidTools.UI.ColourTupleToTurbineColour(self.colourSettings[self.player:GetClass()])
        backgroundColor = PrimePlugins.RaidTools.UI.ColourTupleToTurbineColour(self.colourSettings[self.player:GetClass()])
    end
    backgroundColor = Turbine.UI.Color(self.frameSettings.BackgroundOpacity, backgroundColor.R, backgroundColor.G, backgroundColor.B)

    self.moraleBar:SetColours(backgroundColor, colour, PrimePlugins.RaidTools.UI.ColourTupleToTurbineColour(self.colourSettings["Temporary Morale"]))
	self.moraleBar:SetMouseVisible(false)
	
    if self.frameSettings.ShowName then
        self.text = Turbine.UI.Label()
        self.text:SetForeColor( Turbine.UI.Color( 1, 1, 1, 1 ) )
        self.text:SetOutlineColor( Turbine.UI.Color( 1, 0, 0, 0 ) )
        self.text:SetFontStyle( Turbine.UI.FontStyle.Outline )
        self.text:SetSize(self.frameSettings.VitalWidth - 2 * self.frameSettings.HealthBorder, self.frameSettings.HealthHeight)
        self.text:SetFont(Turbine.UI.Lotro.Font[self.frameSettings.NameFont])
        self.text:SetParent(self)
        self.text:SetZOrder(100)
        self.text:SetPosition(self.frameSettings.NameX, self.frameSettings.NameY)
        self.text:SetTextAlignment(Turbine.UI.ContentAlignment[self.frameSettings.NameAlignment])
        self.text:SetMouseVisible(false)
        self.text:SetText(self.player:GetName())
    end

    -- Set callbacks
    self.playerCallbacks["MoraleChanged"] = AddCallback(self.player, "MoraleChanged", function(sender, args) self:HandleMoraleChange() end)
    self.playerCallbacks["MaxMoraleChanged"] = AddCallback(self.player, "MaxMoraleChanged", function(sender, args) self:HandleMoraleChange() end)
    self.playerCallbacks["BaseMaxMoraleChanged"] = AddCallback(self.player, "BaseMaxMoraleChanged", function(sender, args) self:HandleMoraleChange() end)
    self.playerCallbacks["TemporaryMoraleChanged"] = AddCallback(self.player, "TemporaryMoraleChanged", function(sender, args) self:HandleMoraleChange() end)
    
    if self.ClassType == Turbine.Gameplay.Class.Beorning then  
        self.attrCallbacks["WrathChanged"] = AddCallback(self.ClassAttr, "WrathChanged", function(sender, args) self.powerBar:SetCurrentValue( self.ClassAttr:GetWrath() ) end)
    else
        self.playerCallbacks["PowerChanged"] = AddCallback(self.player, "PowerChanged", function(sender, args) self.powerBar:SetCurrentValue( self.player:GetPower() ) end)
        self.playerCallbacks["MaxPowerChanged"] = AddCallback(self.player, "MaxPowerChanged", function(sender, args) self.powerBar:SetMaxValue( self.player:GetMaxPower() ) end)
        self.playerCallbacks["BaseMaxPowerChanged"] = AddCallback(self.player, "BaseMaxPowerChanged", function(sender, args) self.powerBar:SetMaxValue( self.player:GetMaxPower() ) end)
    end

    -- Set base values
    self.moraleBar:SetCurrentValue( self.player:GetMorale() )
    self.moraleBar:SetMaxValue( self.player:GetMaxMorale() )
    self.moraleBar:SetMaxValue( self.player:GetMaxMorale() )
    self.moraleBar:SetTempValue( self.player:GetTemporaryMorale())
    self.moraleBar:SetMaxTempValue( self.player:GetMaxTemporaryMorale())
    self.healthText:SetText(self:CreateHealthString(self.player:GetMorale(), self.player:GetMaxMorale(), self.player:GetTemporaryMorale()))

    if self.ClassType == Turbine.Gameplay.Class.Beorning then
        self.powerBar:SetCurrentValue(self.ClassAttr:GetWrath())
        self.powerBar:SetMaxValue(100)
    else
        self.powerBar:SetCurrentValue(self.player:GetPower())
        self.powerBar:SetMaxValue(self.player:GetMaxPower())
        self.powerBar:SetMaxValue(self.player:GetMaxPower())
    end

    -- Add a movebox to the mix
    self.moveBox = PrimePlugins.RaidTools.MoveBox(self, width, height, "Vital Frame")

    if self.EffectPosition == 1 then

        self.buffBox = Turbine.UI.Control()
        self.buffBox:SetParent(self)
        self.buffBox:SetSize(self.frameSettings.VitalWidth, 160)	
        self.buffBox:SetMouseVisible(false)
        self.buffBox:SetPosition(self.frameSettings.BuffBoxX, self.frameSettings.BuffBoxY)

        self.IconRow = math.floor(self.frameSettings.VitalWidth / self.IconSize)

    elseif self.EffectPosition == 2 then

        self.buffBox = Turbine.UI.Window()
        self.buffBox:SetVisible(true)
        self.buffBox:SetPosition( 1300, 2 )
        self.buffBox:SetSize( 400, 160 );	
        self.buffBox:SetMouseVisible( false );	

        self.IconRow = math.floor(400 / self.IconSize)
    end

    self.debuffBox = Turbine.UI.Control()
	self.debuffBox:SetParent(self)
    self.debuffBox:SetSize(self.frameSettings.VitalWidth, self.frameSettings.DebuffBoxHeight)
    self.debuffBox:SetMouseVisible(false)
    self.debuffBox:SetPosition(self.frameSettings.DebuffBoxX, self.frameSettings.DebuffBoxY)

    -- Effects
    self.effectList = self.player:GetEffects()

    for i = 1, self.effectList:GetCount(), 1 do
        local effect =  self.effectList:Get(i)

        if effect:IsDebuff() and self.DebuffTracking then
            self.DebuffBoxes[effect:GetID()] = EffectBox(self.debuffBox, effect, self.IconSize, self.UpdateCountdown)
        elseif not(effect:IsDebuff()) and self.BuffTracking then
            self.BuffBoxes[effect:GetID()] = EffectBox(self.buffBox, effect, self.IconSize, self.UpdateCountdown)
        end
    end 

    self.effectCallbacks["EffectRemoved"] = AddCallback(self.effectList, "EffectRemoved", function(sender, args)
        if args.Effect then 
            local ID = args.Effect:GetID()
            if self.BuffBoxes[ID] then 
                self.BuffBoxes[ID]:SetVisible(false) 
                self.BuffBoxes[ID]:SetParent(nil) 
                self.BuffBoxes[ID] = nil
                self.BuffResort = true
            end
            if self.DebuffBoxes[ID] then 
                self.DebuffBoxes[ID]:SetVisible(false)
                self.DebuffBoxes[ID]:SetParent(nil)  
                self.DebuffBoxes[ID] = nil
                self.DebuffResort = true
            end
        end
    end)

    self.effectCallbacks["EffectAdded"] = AddCallback(self.effectList, "EffectAdded", function(sender, args)
        local effect = sender:Get(args.Index)
        if effect:IsDebuff() and self.DebuffTracking then
            self.DebuffBoxes[effect:GetID()] = EffectBox(self.debuffBox, effect, self.IconSize, self.UpdateCountdown, Turbine.UI.Lotro.Font[self.frameSettings.EffectFont])
            self.DebuffResort = true
        elseif not(effect:IsDebuff()) and self.BuffTracking then
            self.BuffBoxes[effect:GetID()] = EffectBox(self.buffBox, effect, self.IconSize, self.UpdateCountdown, Turbine.UI.Lotro.Font[self.frameSettings.EffectFont])
            self.BuffResort = true
        end
    end)

    self.Update = function()
        local currentGameTime = Turbine.Engine.GetGameTime()
        if currentGameTime == self.oldGameTime then
            return(true)
        end
        local delta = currentGameTime - self.oldGameTime

        for key, value in pairs(self.DebuffBoxes) do
            value:UpdateText(delta)
        end

        for key, value in pairs(self.BuffBoxes) do
            value:UpdateText(delta)
        end

        if self.BuffResort or self.DebuffResort then self.ResortList() end

        self.oldGameTime = currentGameTime
    end

    self.ResortList = function()
        if self.BuffResort then
            if self.EffectPosition == 1 then
                local counter = 0
                for _,v in spairs(self.BuffBoxes, 
                function(t,a,b) 
                    if t[b]:GetValue() == t[a]:GetValue() then
                        return a < b
                    else
                        return t[b]:GetValue() < t[a]:GetValue() 
                    end
                end) do
                    v:SetPosition((counter % self.IconRow) * self.IconSize, math.floor(counter / self.IconRow) * self.IconSize)
                    counter = counter + 1
                end
            elseif self.EffectPosition == 2 then
                local counter = 0
                for _,v in spairs(self.BuffBoxes, 
                function(t,a,b) 
                    if t[b]:GetValue() == t[a]:GetValue() then
                        return a < b
                    else
                        return t[b]:GetValue() < t[a]:GetValue() 
                    end
                end) do
                    v:SetPosition(400 - self.IconSize - (counter % self.IconRow) * self.IconSize, math.floor(counter / self.IconRow) * self.IconSize) 
                    counter = counter + 1	
                end
            end
            self.BuffResort = false
        end

        if self.DebuffResort then
            local counter = 0
            for _,v in spairs(self.DebuffBoxes, function(t,a,b) return t[b]:GetValue() > t[a]:GetValue() end) do
                v:SetPosition((counter % self.IconRow) * self.IconSize, self.IconRowHeight - math.floor(counter / self.IconRow) * self.IconSize) 
                counter = counter + 1
            end
            self.DebuffResort = false
        end
    end

    self:ResortList()
    self:SetWantsUpdates(true)

    local borderColour = Turbine.UI.Color(0, 0, 0)

    if self.frameSettings.ColourMode == 3 then 
        borderColour = Turbine.UI.Color(0.08, 0.08, 0.08)
    end

    
    
    
    
    if self.frameSettings.ResourceEnabled then
        if self.ClassType == Turbine.Gameplay.Class.Champion then
            self.RBar = ResourceBar(self, self.frameSettings.ResourceWidth, self.frameSettings.ResourceHeight, 5, self.ResourceText, self.frameSettings.ResourceY, borderColour)

            self.attrCallbacks["FervorChanged"] = AddCallback(self.ClassAttr, "FervorChanged", function(sender, args) self.RBar:ChangeResource(sender:GetFervor(), PrimePlugins.RaidTools.UI.ColourTupleToTurbineColour(self.colourSettings["Fervour Colour"])) end)
            
            self.RBar:ChangeResource(self.ClassAttr:GetFervor(), PrimePlugins.RaidTools.UI.ColourTupleToTurbineColour(self.colourSettings["Fervour Colour"]))

        elseif self.ClassType == Turbine.Gameplay.Class.Brawler then
            self.RBar = ResourceBar(self, self.frameSettings.ResourceWidth, self.frameSettings.ResourceHeight, 9, self.ResourceText, self.frameSettings.ResourceY, borderColour)

            for i = 1, self.player:GetEffects():GetCount(), 1 do
                local effect = self.player:GetEffects():Get(i)
                if string.gsub(effect:GetName(), "%d", "") == "Battle Flow " then
                    self.RBar:ChangeResource(tonumber(string.gsub(effect:GetName(), "%D", "")), PrimePlugins.RaidTools.UI.ColourTupleToTurbineColour(self.colourSettings["Fervour Colour"]))
                end
            end
    
            table.insert(self.effectAddedCallbacks, AddCallback(self.effectList, "EffectAdded", 
            function(sender, args)
                local effect = self.effectList:Get(args.Index)
                if string.gsub(effect:GetName(), "%d", "") == "Battle Flow " then
                    self.RBar:ChangeResource(tonumber(string.gsub(effect:GetName(), "%D", "")), PrimePlugins.RaidTools.UI.ColourTupleToTurbineColour(self.colourSettings["Fervour Colour"]))
                    self.battleFlowID = effect:GetID()
                end
            end))
    
            table.insert(self.effectRemovedCallbacks, AddCallback(self.effectList, "EffectRemoved", 
            function(sender, args)
                if self.battleFlowID == args.Effect:GetID() then
                    self.RBar:ChangeResource(0)
                end
            end))

        elseif self.ClassType == Turbine.Gameplay.Class.Hunter then
            self.RBar = ResourceBar(self, self.frameSettings.ResourceWidth, self.frameSettings.ResourceHeight, 9, self.ResourceText, self.frameSettings.ResourceY, borderColour)

            self.attrCallbacks["FocusChanged"] = AddCallback(self.ClassAttr, "FocusChanged", function(sender, args) self.RBar:ChangeResource(sender:GetFocus(), PrimePlugins.RaidTools.UI.ColourTupleToTurbineColour(self.colourSettings["Focus Colour"])) end)

            self.RBar:ChangeResource(self.ClassAttr:GetFocus(), PrimePlugins.RaidTools.UI.ColourTupleToTurbineColour(self.colourSettings["Focus Colour"]))

        elseif self.ClassType == Turbine.Gameplay.Class.RuneKeeper then

            self.RBar = ResourceBar(self, self.frameSettings.ResourceWidth, self.frameSettings.ResourceHeight, 9, self.ResourceText, self.frameSettings.ResourceY, borderColour)

            self.attrCallbacks["AttunementChanged"] = AddCallback(self.ClassAttr, "AttunementChanged", function(sender, args) 
                local Attunement = sender:GetAttunement()
                if Attunement > 10 then
                    self.RBar:ChangeResource(math.abs( Attunement - 10), PrimePlugins.RaidTools.UI.ColourTupleToTurbineColour(self.colourSettings["Defensive Attunement Colour"]))
                else
                    self.RBar:ChangeResource(math.abs( Attunement - 10), PrimePlugins.RaidTools.UI.ColourTupleToTurbineColour(self.colourSettings["Offensive Attunement Colour"]))
                end
            end)
            
            local Attunement = self.ClassAttr:GetAttunement()
            if Attunement > 10 then
                self.RBar:ChangeResource(math.abs( Attunement - 10), PrimePlugins.RaidTools.UI.ColourTupleToTurbineColour(self.colourSettings["Defensive Attunement Colour"]))
            else
                self.RBar:ChangeResource(math.abs( Attunement - 10), PrimePlugins.RaidTools.UI.ColourTupleToTurbineColour(self.colourSettings["Offensive Attunement Colour"]))
            end
        end
    end

    self:SetStretchMode(1)
    local UIWidth, UIHeight = self:GetSize()
    self:SetSize(UIWidth * self.frameSettings.Scale, UIHeight * self.frameSettings.Scale)
end

function VitalWindow:RemoveCallbacks()
    for key, value in pairs(self.playerCallbacks) do
        RemoveCallback(self.player, key, value)
        self.playerCallbacks[key] = nil
    end
    
    for key, value in pairs(self.effectCallbacks) do
        RemoveCallback(self.effectList, key, value)
        self.effectCallbacks[key] = nil
    end
    
    for key, value in pairs(self.attrCallbacks) do
        RemoveCallback(self.ClassAttr, key, value)
        self.attrCallbacks[key] = nil
    end

    for key, callback in pairs(self.effectAddedCallbacks) do
        RemoveCallback(self.effectList, "EffectAdded", callback)
        self.effectAddedCallbacks[key] = nil
    end
    
    for key, callback in pairs(self.effectRemovedCallbacks) do
        RemoveCallback(self.effectList, "EffectRemoved", callback)
        self.effectRemovedCallbacks[key] = nil
    end

    self.playerCallbacks = nil
    self.effectCallbacks = nil
    self.attrCallbacks = nil

    collectgarbage()

    self.playerCallbacks = {}
    self.attrCallbacks = {}
    self.effectCallbacks = {}
end

function VitalWindow:ToggleMove()
    self.moveBox:ToggleMove()
end

function VitalWindow:Unload()
    self:SetWantsUpdates(false)
    self:RemoveCallbacks()
    
    for key, value in pairs(self.BuffBoxes) do
        self.BuffBoxes[key]:Unload()
        self.BuffBoxes[key] = nil
    end

    for key, value in pairs(self.DebuffBoxes) do
        self.DebuffBoxes[key]:Unload()
        self.DebuffBoxes[key] = nil
    end

    self.BuffBoxes = nil
    self.DebuffBoxes = nil

    local controlList = self:GetControls()
    for i = 1, controlList:GetCount() do
        controlList:RemoveAt(1)
    end

    self:SetParent(nil)
end

function VitalWindow:CreateHealthString(current, max, temp) 
    if self.frameSettings.HealthText then
        local currentString = ""

        if self.frameSettings.OnlyShowCurrent then
            currentString = FormatBigNumber(math.floor( current + 0.5 ))
        else
            currentString = FormatBigNumber(math.floor( current + 0.5 ))  .. "/" .. FormatBigNumber(math.floor( max + 0.5 ))
        end


        local tempString = ""

        if temp > 0 then
            tempString = string.format(" (%s)", tostring(FormatBigNumber(math.ceil(temp))))
        end

        local percentageString = ""

        if self.frameSettings.PercentageShown then
            percentageString = " - " .. math.floor(current / max * 100) .. "%"
        else
            percentageString = ""
        end

        return string.format("%s%s%s", tostring(currentString), tostring(tempString), tostring(percentageString))
    end
end

function VitalWindow:HandleMoraleChange() 
    self.moraleBar:SetMaxValue(self.player:GetMaxMorale())
    self.moraleBar:SetCurrentValueImmediate(self.player:GetMorale()) 
    if self.frameSettings.Aura then self.aura:Adjust(self.player:GetMorale() / self.player:GetMaxMorale()) end
    self.moraleBar:SetTempValue(self.player:GetTemporaryMorale())
    self.healthText:SetText(self:CreateHealthString(self.player:GetMorale(), self.player:GetMaxMorale(), self.player:GetTemporaryMorale()))
end