AlertOptions = class(Turbine.UI.Control)
function AlertOptions:Constructor(parent)
    local width = 750
    local height = 500
    local font = Settings.LOFont
    local colour = Turbine.UI.Color(1, 1, 1)
    Turbine.UI.Control.Constructor(self)
    self:SetSize(width,height); -- sets the window size to 200 by 200 pixels
    self:SetParent(parent)
    self:SetPosition(0,0)
    self:SetWantsKeyEvents(true)
    self:SetVisible(false)

    self.BorderBackground = Turbine.UI.Control()
    self.BorderBackground:SetParent(self)
    self.BorderBackground:SetPosition(10 - 2, 10 - 2)
    self.BorderBackground:SetSize(730 + 4, 450 + 4)
    self.BorderBackground:SetBackColor(Turbine.UI.Color(0, 0, 0))

    self.Background = Turbine.UI.Control()
    self.Background:SetParent(self)
    self.Background:SetPosition(10, 10)
    self.Background:SetSize(730, 450)
    self.Background:SetBackColor(Turbine.UI.Color(0.250, 0.250, 0.250))

    self.IconOverview = self:iconOverview()
    self.EffectsOverview = self:HistoricalEffectOverview()

    --self.HistoricalEffectsTextLink:SetText(key)
    --self.HistoricalEffectsID = key

    self.SelectionWindows = {}


    -- Current alerts
    self:CurrentAlerts()

    self.ButtonBackground = Turbine.UI.Control()
    self.ButtonBackground:SetParent(self.Background)
    self.ButtonBackground:SetPosition(260, 3)
    self.ButtonBackground:SetSize(467, 49)
    self.ButtonBackground:SetBackColor(Turbine.UI.Color(0.375, 0.375, 0.375))

    self.ChatButton = self:SelectButton(self.ButtonBackground, 150, 20, 3, 3, "Chat (Standard)")
    self.ChatEnemyCombatButton = self:SelectButton(self.ButtonBackground, 150, 20, 156, 3, "Chat (Combat Enemy)")
    self.ChatPlayerCombatButton = self:SelectButton(self.ButtonBackground, 150, 20, 312, 3, "Chat (Combat Player)")
    self.SelfButton = self:SelectButton(self.ButtonBackground, 229, 20, 3, 26, "Effects (Self)")
    self.GroupButton = self:SelectButton(self.ButtonBackground, 229, 20, 235, 26, "Effects (Group)")

    self.SelectedBackground = Turbine.UI.Control()
    self.SelectedBackground:SetParent(self.Background)
    self.SelectedBackground:SetPosition(260, 55)
    self.SelectedBackground:SetSize(467, 389)
    self.SelectedBackground:SetBackColor(Turbine.UI.Color(0.375, 0.375, 0.375))

    self.rootNodes = self.CurrentAlertContainer:GetNodes()

    self:ChatAlerts(self.SelectedBackground)
    self:ChatPlayerCombatAlerts(self.SelectedBackground)
    self:ChatEnemyCombatAlerts(self.SelectedBackground)
    self:SelfEffectAlerts(self.SelectedBackground)
    self:GroupEffectAlerts(self.SelectedBackground)

    --self.CurrentAlertContainer:ExpandAll()
    --self:RefreshTreeview()
end

function AlertOptions:ToggleWindow(window)
    self:ClearAll()
    for key, value in pairs(self.SelectionWindows) do
        if window == key then
            value:SetVisible(true)
        else
            value:SetVisible(false)
        end
    end
end

function AlertOptions:ClearAll()
    self:ClearGroupEffect()
    self:ClearSelfEffect()
    self:ClearChatOptions()
    self:ClearPlayerCombat()
    self:ClearEnemyCombat()
end

-- Button for selecting specific category
function AlertOptions:SelectButton(parent, width, height, x, y, text, func)
    local Button = Turbine.UI.Button()
    Button:SetParent(parent)
    Button:SetPosition(x, y)
    Button:SetSize(width, height)
    Button:SetTextAlignment(Turbine.UI.ContentAlignment.TopCenter)
    Button:SetFont(Turbine.UI.Lotro.Font.BookAntiquaBold22)
    --Button:SetBackColor(Turbine.UI.Color(0.5, 0.5, 0.5))
    --Button:SetText(text)

    local ButtonBackground = Turbine.UI.Button()
    ButtonBackground:SetParent(Button)
    ButtonBackground:SetPosition(0, 0)
    ButtonBackground:SetSize(width, height)
    ButtonBackground:SetMouseVisible(false)
    ButtonBackground:SetBackColor(Turbine.UI.Color(0.5, 0.5, 0.5))

    --[[

    local middlePart = Turbine.UI.Control()
    middlePart:SetParent(Button)
    middlePart:SetPosition(11, 0)
    middlePart:SetSize(width - 11 - 12, 20)
    middlePart:SetBlendMode(Turbine.UI.BlendMode.AlphaBlend)
    middlePart:SetBackground(0x410F2E19)
    middlePart:SetMouseVisible(false)

    
    local leftAngle = Turbine.UI.Control()
    leftAngle:SetParent(Button)
    leftAngle:SetPosition(0, 0)
    leftAngle:SetSize(11, 20)
    leftAngle:SetBlendMode(Turbine.UI.BlendMode.AlphaBlend)
    leftAngle:SetBackground(0x410F2E18)
    leftAngle:SetMouseVisible(false)

    local rigthAngle = Turbine.UI.Control()
    rigthAngle:SetParent(Button)
    rigthAngle:SetPosition(width - 12, 0)
    rigthAngle:SetSize(12, 20)
    rigthAngle:SetBlendMode(Turbine.UI.BlendMode.AlphaBlend)
    rigthAngle:SetBackground(0x410F2E1A)
    rigthAngle:SetMouseVisible(false)
    --]]

    local textLabel = Turbine.UI.Button()
    textLabel:SetParent(Button)
    textLabel:SetPosition(5, -1)
    textLabel:SetSize(width, height)
    textLabel:SetTextAlignment(Turbine.UI.ContentAlignment.TopCenter)
    textLabel:SetFont(Turbine.UI.Lotro.Font.BookAntiquaBold22)
    textLabel:SetText(text)
    textLabel:SetMouseVisible(false)

    Button.MouseEnter = function()
        ButtonBackground:SetBackColor(Turbine.UI.Color(0.25, 0.25, 0.25))
    end

    Button.MouseLeave = function()
        ButtonBackground:SetBackColor(Turbine.UI.Color(0.5, 0.5, 0.5))
    end

    Button.MouseDown = function()
        ButtonBackground:SetBackColor(Turbine.UI.Color(0.1, 0.1, 0.1))
    end

    Button.MouseUp = function()
        ButtonBackground:SetBackColor(Turbine.UI.Color(0.25, 0.25, 0.25))
    end

    Button.Click = function()
        self:ToggleWindow(text)
    end
    
    return Button
end

-- Current alert listbox
function AlertOptions:CurrentAlerts()
    self.CurrentAlertBackground = Turbine.UI.Control()
    self.CurrentAlertBackground:SetParent(self.Background)
    self.CurrentAlertBackground:SetPosition(3, 3)
    self.CurrentAlertBackground:SetSize(250, 444)
    self.CurrentAlertBackground:SetBackColor(Turbine.UI.Color(0.375, 0.375, 0.375))

    self.CurrentLeft = Turbine.UI.Control()
    self.CurrentLeft:SetParent(self.CurrentAlertBackground)
    self.CurrentLeft:SetPosition(0, 20)
    self.CurrentLeft:SetSize(22, 26)
    self.CurrentLeft:SetBackground(BackgroundHandler:GetBackground(0x411BAF1E))
    self.CurrentLeft:SetBlendMode(Turbine.UI.BlendMode.AlphaBlend)

    self.CurrentRight = Turbine.UI.Control()
    self.CurrentRight:SetParent(self.CurrentAlertBackground)
    self.CurrentRight:SetPosition(self.CurrentAlertBackground:GetWidth() - 23, 20)
    self.CurrentRight:SetSize(23, 26)
    self.CurrentRight:SetBackground(BackgroundHandler:GetBackground(0x411BAF20))
    self.CurrentRight:SetBlendMode(Turbine.UI.BlendMode.AlphaBlend)

    self.CurrentMiddle = Turbine.UI.Control()
    self.CurrentMiddle:SetParent(self.CurrentAlertBackground)
    self.CurrentMiddle:SetPosition(22, 20)
    self.CurrentMiddle:SetSize(self.CurrentAlertBackground:GetWidth() - 40, 26)
    self.CurrentMiddle:SetBackground(BackgroundHandler:GetBackground(0x411BAF19))
    self.CurrentMiddle:SetBlendMode(Turbine.UI.BlendMode.AlphaBlend)

    self.CurrentLabel = Turbine.UI.Label()
    self.CurrentLabel:SetParent(self.Background)
    self.CurrentLabel:SetPosition(6, 26)
    self.CurrentLabel:SetSize(244, 20)
    self.CurrentLabel:SetTextAlignment(Turbine.UI.ContentAlignment.TopCenter)
    self.CurrentLabel:SetFont(Turbine.UI.Lotro.Font.BookAntiquaBold22)
    --self.CurrentLabel:SetBackColor(Turbine.UI.Color(0.5, 0.5, 0.5))
    self.CurrentLabel:SetText("Current Alerts")

    self.CurrentAlertContainer = Turbine.UI.TreeView()
    self.CurrentAlertContainer:SetParent(self.CurrentAlertBackground)
    self.CurrentAlertContainer:SetSize(self.CurrentAlertBackground:GetWidth() - 10, self.CurrentAlertBackground:GetHeight() - 46)
    self.CurrentAlertContainer:SetPosition(0, 46)
    self.CurrentAlertContainer:SetVisible(true)
    self.CurrentAlertContainer:SetWantsKeyEvents(true)
    self.CurrentAlertContainer:SetMouseVisible(true)
    self.CurrentAlertContainer:SetIndentationWidth( 0 )

    self.CurrentAlertContainerScrollbar = Turbine.UI.Lotro.ScrollBar()
    self.CurrentAlertContainerScrollbar:SetOrientation( Turbine.UI.Orientation.Vertical )
    self.CurrentAlertContainerScrollbar:SetParent( self.CurrentAlertBackground )
    self.CurrentAlertContainerScrollbar:SetPosition( self.CurrentAlertBackground:GetWidth() - 10, 46 )
    self.CurrentAlertContainerScrollbar:SetSize( 10, self.CurrentAlertBackground:GetHeight() - 46 )
    self.CurrentAlertContainer:SetVerticalScrollBar( self.CurrentAlertContainerScrollbar )

    --0x4100026B
    self.closeButton =  Turbine.UI.Button()
    self.closeButton:SetParent(self.CurrentAlertBackground)
    self.closeButton:SetPosition(self.CurrentAlertBackground:GetWidth() - 20, 0)
    self.closeButton:SetSize(20, 20)
    self.closeButton:SetBlendMode(Turbine.UI.BlendMode.AlphaBlend)
    self.closeButton:SetBackground(BackgroundHandler:GetBackground(0x4100026B))

    self.closeButton.Click = function()
        self.CurrentAlertContainer:CollapseAll()
    end

    self.UpdateButton = Turbine.UI.Lotro.Button()
    self.UpdateButton:SetParent(self.CurrentAlertBackground)
    self.UpdateButton:SetPosition(0, 0)
    self.UpdateButton:SetSize((self.CurrentAlertBackground:GetWidth() - 20) / 2, 20)
    self.UpdateButton:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
    self.UpdateButton:SetText("Update Base")
    self.UpdateButton.Click = function()
        EventWindows:UpdateBase()
    end
    
    self.ResetButton = Turbine.UI.Lotro.Button()
    self.ResetButton:SetParent(self.CurrentAlertBackground)
    self.ResetButton:SetPosition((self.CurrentAlertBackground:GetWidth() - 20) / 2, 0)
    self.ResetButton:SetSize((self.CurrentAlertBackground:GetWidth() - 20) / 2, 20)
    self.ResetButton:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
    self.ResetButton:SetText("Reset All")
    self.ResetButton.Click = function()
        EventWindows:ResetData()
    end

    --[[
    self.SearchBox = Turbine.UI.Lotro.TextBox()
    self.SearchBox:SetParent(self.CurrentAlertBackground)
    self.SearchBox:SetSize(self.CurrentAlertBackground:GetWidth(), 20)
    self.SearchBox:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
    self.SearchBox:SetMouseVisible(false)
    self.SearchBox:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
    self.SearchBox:SetPosition(0, 26)
    self.SearchBox:SetMultiline(true)

    self.CurrentAlertContainer:SetFilter(function(a)
        if a.SourceLabel then
            return a.SourceLabel:GetText() == "Azbauz"
        end

        if a.AlertMessageLabel then
            return string.find(a.AlertMessageLabel:GetText(), "Burn") ~= nil
        end

        return false
    end)
    --]]


end

-- Creation/Changing group effect alert UI
function AlertOptions:GroupEffectAlerts(parent)
    self.GroupEffectBackground = Turbine.UI.Control()
    self.GroupEffectBackground:SetParent(parent)
    self.GroupEffectBackground:SetPosition(0, 0)
    self.GroupEffectBackground:SetVisible(false)
    self.GroupEffectBackground:SetSize(parent:GetWidth(), parent:GetHeight())

    self.GroupEffectHeader = Turbine.UI.Label()
    self.GroupEffectHeader:SetParent(self.GroupEffectBackground)
    self.GroupEffectHeader:SetSize(200, 30)
    self.GroupEffectHeader:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft)
    self.GroupEffectHeader:SetFont(Turbine.UI.Lotro.Font.BookAntiquaBold24)
    self.GroupEffectHeader:SetFontStyle(Turbine.UI.FontStyle.Outline)
    self.GroupEffectHeader:SetMouseVisible(false)
    self.GroupEffectHeader:SetPosition(10, 0)
    self.GroupEffectHeader:SetVisible(true)
    self.GroupEffectHeader:SetText("Effects (Group)")

    self.GroupEffectNameLabel = Turbine.UI.Label()
    self.GroupEffectNameLabel:SetParent(self.GroupEffectBackground)
    self.GroupEffectNameLabel:SetSize(100, 30)
    self.GroupEffectNameLabel:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft)
    self.GroupEffectNameLabel:SetFont(Turbine.UI.Lotro.Font.BookAntiquaBold19)
    self.GroupEffectNameLabel:SetFontStyle(Turbine.UI.FontStyle.Outline)
    self.GroupEffectNameLabel:SetMouseVisible(false)
    self.GroupEffectNameLabel:SetPosition(10, 30)
    self.GroupEffectNameLabel:SetVisible(true)
    self.GroupEffectNameLabel:SetText("Effect Name")

    self.GroupEffectNameTextBox = Turbine.UI.Lotro.TextBox()
    self.GroupEffectNameTextBox:SetParent(self.GroupEffectBackground)
    self.GroupEffectNameTextBox:SetSize(300, 30)
    self.GroupEffectNameTextBox:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
    self.GroupEffectNameTextBox:SetMouseVisible(false)
    self.GroupEffectNameTextBox:SetPosition(100, 30)
    self.GroupEffectNameTextBox:SetMultiline(false)

    self.GroupEffectHistoryButton = Turbine.UI.Lotro.Button()
    self.GroupEffectHistoryButton:SetParent(self.GroupEffectBackground)
    self.GroupEffectHistoryButton:SetSize(70, 15)
    self.GroupEffectHistoryButton:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
    self.GroupEffectHistoryButton:SetMouseVisible(true)
    self.GroupEffectHistoryButton:SetPosition(400, 35)
    self.GroupEffectHistoryButton:SetText("Browse")

    self.GroupEffectHistoryButton.Click = function()
        self.EffectsOverview:SetVisible(true)
        self.HistoricalEffectsTextLink = self.GroupEffectNameTextBox
        --self.HistoricalEffectsID = key
    end

    
    self.GroupEffectInstanceLabel = Turbine.UI.Label()
    self.GroupEffectInstanceLabel:SetParent(self.GroupEffectBackground)
    self.GroupEffectInstanceLabel:SetSize(300, 30)
    self.GroupEffectInstanceLabel:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft)
    self.GroupEffectInstanceLabel:SetFont(Turbine.UI.Lotro.Font.BookAntiquaBold19)
    self.GroupEffectInstanceLabel:SetFontStyle(Turbine.UI.FontStyle.Outline)
    self.GroupEffectInstanceLabel:SetMouseVisible(false)
    self.GroupEffectInstanceLabel:SetPosition(10, 65)
    self.GroupEffectInstanceLabel:SetVisible(true)
    self.GroupEffectInstanceLabel:SetText("Only load for specific instance/role")

    self.GroupEffectInstanceButton = Turbine.UI.Lotro.Button()
    self.GroupEffectInstanceButton:SetParent(self.GroupEffectBackground)
    self.GroupEffectInstanceButton:SetSize(100, 15)
    self.GroupEffectInstanceButton:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
    self.GroupEffectInstanceButton:SetMouseVisible(true)
    self.GroupEffectInstanceButton:SetPosition(240, 70)
    self.GroupEffectInstanceButton:SetText("All")

    --self.GroupEffectInstanceMenu = Turbine.UI.ContextMenu()
    --self.GroupEffectInstanceMenuItems = self.GroupEffectInstanceMenu:GetItems()
    self.GroupEffectInstanceMenu = UI.DropDownMenuControl(self, 200, "Instance Dropdown Menu")
    for key, value in spairs(InstanceController:GetDungeonNameTable(), function(t, a, b) return t[a] < t[b] end) do
        self.GroupEffectInstanceMenu:AddItem(value, function()
            self.GroupEffectInstanceButton:SetText(value)
            self.GroupEffectInstanceMenu:SetVisible(false)
        end)
    end

    self.GroupEffectInstanceButton.Click = function()
        self.GroupEffectInstanceMenu:ToggleVisible()
    end

    self.GroupEffectRoleButton = Turbine.UI.Lotro.Button()
    self.GroupEffectRoleButton:SetParent(self.GroupEffectBackground)
    self.GroupEffectRoleButton:SetSize(100, 15)
    self.GroupEffectRoleButton:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
    self.GroupEffectRoleButton:SetMouseVisible(true)
    self.GroupEffectRoleButton:SetPosition(340, 70)
    self.GroupEffectRoleButton:SetText("All")

    self.GroupEffectRoleMenu = Turbine.UI.ContextMenu()
    self.GroupEffectRoleMenuItems = self.GroupEffectRoleMenu:GetItems()
    for key, value in spairs(Roles:GetAvailableRoles(), function(t, a, b) return t[a] < t[b] end) do
        local menuItem = Turbine.UI.MenuItem( value )
        self.GroupEffectRoleMenuItems:Add(menuItem)
        menuItem.Click = function()
            self.GroupEffectRoleButton:SetText(value)
        end
    end

    local menuItem = Turbine.UI.MenuItem( "All" )
    self.GroupEffectRoleMenuItems:Add(menuItem)
    menuItem.Click = function()
        self.GroupEffectRoleButton:SetText("All")
    end

    self.GroupEffectRoleButton.Click = function()
        self.GroupEffectRoleMenu:ShowMenu()
    end

    -- Alert
    self.GroupEffectAlertBackground = Turbine.UI.Control()
    self.GroupEffectAlertBackground:SetParent(self.GroupEffectBackground)
    self.GroupEffectAlertBackground:SetPosition(10, 100)
    self.GroupEffectAlertBackground:SetSize(parent:GetWidth() - 20, parent:GetHeight() - 260)
    self.GroupEffectAlertBackground:SetBackColor(Turbine.UI.Color(0.5, 0.5, 0.5))

    self.GroupEffectAlertCheckBox = Turbine.UI.Lotro.CheckBox()
    self.GroupEffectAlertCheckBox:SetParent(self.GroupEffectAlertBackground)
    self.GroupEffectAlertCheckBox:SetSize(200, 30)
    self.GroupEffectAlertCheckBox:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft)
    self.GroupEffectAlertCheckBox:SetFont(Turbine.UI.Lotro.Font.BookAntiquaBold24)
    self.GroupEffectAlertCheckBox:SetFontStyle(Turbine.UI.FontStyle.Outline)
    self.GroupEffectAlertCheckBox:SetMouseVisible(false)
    self.GroupEffectAlertCheckBox:SetPosition(3, 3)
    self.GroupEffectAlertCheckBox:SetVisible(true)
    self.GroupEffectAlertCheckBox:SetText("Alert on Trigger")

    self.PhraseLabel = Turbine.UI.Label()
    self.PhraseLabel:SetParent(self.GroupEffectAlertBackground)
    self.PhraseLabel:SetSize(100, 30)
    self.PhraseLabel:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft)
    self.PhraseLabel:SetFont(Turbine.UI.Lotro.Font.BookAntiquaBold19)
    self.PhraseLabel:SetFontStyle(Turbine.UI.FontStyle.Outline)
    self.PhraseLabel:SetMouseVisible(false)
    self.PhraseLabel:SetPosition(10, 35)
    self.PhraseLabel:SetVisible(true)
    self.PhraseLabel:SetText("Alert Text")

    self.GroupEffectAlertTextBox = Turbine.UI.Lotro.TextBox()
    self.GroupEffectAlertTextBox:SetParent(self.GroupEffectAlertBackground)
    self.GroupEffectAlertTextBox:SetSize(200, 30)
    self.GroupEffectAlertTextBox:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
    self.GroupEffectAlertTextBox:SetMouseVisible(false)
    self.GroupEffectAlertTextBox:SetMultiline(false)
    self.GroupEffectAlertTextBox:SetPosition(80, 35)

    self.GroupEffectAlertDurationLabel = Turbine.UI.Label()
    self.GroupEffectAlertDurationLabel:SetParent(self.GroupEffectAlertBackground)
    self.GroupEffectAlertDurationLabel:SetSize(200, 30)
    self.GroupEffectAlertDurationLabel:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft)
    self.GroupEffectAlertDurationLabel:SetFont(Turbine.UI.Lotro.Font.BookAntiquaBold19)
    self.GroupEffectAlertDurationLabel:SetFontStyle(Turbine.UI.FontStyle.Outline)
    self.GroupEffectAlertDurationLabel:SetMouseVisible(false)
    self.GroupEffectAlertDurationLabel:SetPosition(10, 65)
    self.GroupEffectAlertDurationLabel:SetVisible(true)
    self.GroupEffectAlertDurationLabel:SetText("Alert duration in seconds")

    self.GroupEffectAlertDurationTextBox = Turbine.UI.Lotro.TextBox()
    self.GroupEffectAlertDurationTextBox:SetParent(self.GroupEffectAlertBackground)
    self.GroupEffectAlertDurationTextBox:SetSize(50, 30)
    self.GroupEffectAlertDurationTextBox:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
    self.GroupEffectAlertDurationTextBox:SetMouseVisible(false)
    self.GroupEffectAlertDurationTextBox:SetPosition(185, 65)
    self.GroupEffectAlertDurationTextBox:SetMultiline(false)


    -- Countdown
    self.GroupEffectAlertCountdownCheckBox = Turbine.UI.Lotro.CheckBox()
    self.GroupEffectAlertCountdownCheckBox:SetParent(self.GroupEffectAlertBackground)
    self.GroupEffectAlertCountdownCheckBox:SetSize(200, 30)
    self.GroupEffectAlertCountdownCheckBox:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft)
    self.GroupEffectAlertCountdownCheckBox:SetFont(Turbine.UI.Lotro.Font.BookAntiquaBold19)
    self.GroupEffectAlertCountdownCheckBox:SetFontStyle(Turbine.UI.FontStyle.Outline)
    self.GroupEffectAlertCountdownCheckBox:SetMouseVisible(false)
    self.GroupEffectAlertCountdownCheckBox:SetPosition(250, 65)
    self.GroupEffectAlertCountdownCheckBox:SetVisible(true)
    self.GroupEffectAlertCountdownCheckBox:SetText("Alert duration shown")


    -- Sound effects
    self.GroupEffectAlertSoundLabel = Turbine.UI.Lotro.CheckBox()
    self.GroupEffectAlertSoundLabel:SetParent(self.GroupEffectAlertBackground)
    self.GroupEffectAlertSoundLabel:SetSize(200, 30)
    self.GroupEffectAlertSoundLabel:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft)
    self.GroupEffectAlertSoundLabel:SetFont(Turbine.UI.Lotro.Font.BookAntiquaBold19)
    self.GroupEffectAlertSoundLabel:SetFontStyle(Turbine.UI.FontStyle.Outline)
    self.GroupEffectAlertSoundLabel:SetMouseVisible(false)
    self.GroupEffectAlertSoundLabel:SetPosition(10, 90)
    self.GroupEffectAlertSoundLabel:SetVisible(true)
    self.GroupEffectAlertSoundLabel:SetText("Sound Effect")

    self.GroupEffectSoundeffectButton = Turbine.UI.Lotro.Button()
    self.GroupEffectSoundeffectButton:SetParent(self.GroupEffectAlertBackground)
    self.GroupEffectSoundeffectButton:SetSize(100, 15)
    self.GroupEffectSoundeffectButton:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
    self.GroupEffectSoundeffectButton:SetMouseVisible(true)
    self.GroupEffectSoundeffectButton:SetPosition(120, 97)
    self.GroupEffectSoundeffectButton:SetText("Horn")

    self.GroupEffectSoundEffectMenu = UI.DropDownMenuControl(self, 200, "Sound Menu")
    for key, value in spairs(Sounds:GetSoundList(), function(t, a, b) return a < b end) do
        self.GroupEffectSoundEffectMenu:AddItem(key, 
        function()
            self.GroupEffectSoundeffectButton:SetText(key)
            self.GroupEffectSoundEffectMenu:ToggleVisible()
        end)
    end

    self.GroupEffectSoundeffectButton.Click = function()
        self.GroupEffectSoundEffectMenu:ToggleVisible()
    end

    self.GroupEffectAlertIconLabel = Turbine.UI.Lotro.CheckBox()
    self.GroupEffectAlertIconLabel:SetParent(self.GroupEffectAlertBackground)
    self.GroupEffectAlertIconLabel:SetSize(50, 30)
    self.GroupEffectAlertIconLabel:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft)
    self.GroupEffectAlertIconLabel:SetFont(Turbine.UI.Lotro.Font.BookAntiquaBold19)
    self.GroupEffectAlertIconLabel:SetFontStyle(Turbine.UI.FontStyle.Outline)
    self.GroupEffectAlertIconLabel:SetMouseVisible(false)
    self.GroupEffectAlertIconLabel:SetPosition(240, 97)
    self.GroupEffectAlertIconLabel:SetVisible(true)
    self.GroupEffectAlertIconLabel:SetText("Icon")

    self.GroupEffectAlertIconBox = Turbine.UI.Control()
    self.GroupEffectAlertIconBox:SetParent(self.GroupEffectAlertBackground)
    self.GroupEffectAlertIconBox:SetPosition(290, 97)
    self.GroupEffectAlertIconBox:SetSize(32,32)
    self.GroupEffectAlertIconBox:SetBlendMode(Turbine.UI.BlendMode.AlphaBlend)
    self.GroupEffectAlertIconBox:SetBackground(BackgroundHandler:GetBackground(0x41007F41))
    self.GroupEffectAlertIconBox:SetVisible(true)

    self.GroupEffectAlertIconTextBox = Turbine.UI.Lotro.TextBox()
    self.GroupEffectAlertIconTextBox:SetParent(self.GroupEffectAlertBackground)
    self.GroupEffectAlertIconTextBox:SetSize(110, 30)
    self.GroupEffectAlertIconTextBox:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
    self.GroupEffectAlertIconTextBox:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
    self.GroupEffectAlertIconTextBox:SetMouseVisible(false)
    self.GroupEffectAlertIconTextBox:SetPosition(330, 97)
    self.GroupEffectAlertIconTextBox:SetMultiline(false)
    self.GroupEffectAlertIconTextBox:SetText(0x41007F41)

    self.GroupEffectAlertIconTextBox.TextChanged = function()
        --pcall(self.GroupEffectAlertIconBox:SetBackground(tonumber(self.GroupEffectAlertIconTextBox:GetText())))
    end

    self.GroupEffectAlertIconBox.MouseClick = function()
        if self.IconOverview then 
            self.IconOverview:SetVisible(true) 
            self.iconLink = self.GroupEffectAlertIconBox
            self.iconTextLink = self.GroupEffectAlertIconTextBox
        end
    end

    self.GroupEffectAlertIconBox.MouseDoubleClick = function()
        self.GroupEffectAlertIconBox:SetBackground(BackgroundHandler:GetBackground(0x41007F41))
        self.iconID = 0x41007F41
    end

    --Timer
    self.GroupEffectTimerBackground = Turbine.UI.Control()
    self.GroupEffectTimerBackground:SetParent(self.GroupEffectBackground)
    self.GroupEffectTimerBackground:SetPosition(10, 240)
    self.GroupEffectTimerBackground:SetSize(parent:GetWidth() - 20, 80)
    self.GroupEffectTimerBackground:SetBackColor(Turbine.UI.Color(0.5, 0.5, 0.5))

    self.GroupEffectTimerCheckBox = Turbine.UI.Lotro.CheckBox()
    self.GroupEffectTimerCheckBox:SetParent(self.GroupEffectTimerBackground)
    self.GroupEffectTimerCheckBox:SetSize(200, 30)
    self.GroupEffectTimerCheckBox:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft)
    self.GroupEffectTimerCheckBox:SetFont(Turbine.UI.Lotro.Font.BookAntiquaBold24)
    self.GroupEffectTimerCheckBox:SetFontStyle(Turbine.UI.FontStyle.Outline)
    self.GroupEffectTimerCheckBox:SetMouseVisible(false)
    self.GroupEffectTimerCheckBox:SetPosition(3, 3)
    self.GroupEffectTimerCheckBox:SetVisible(true)
    self.GroupEffectTimerCheckBox:SetText("Timer on Trigger")

    self.GroupEffectTimerText = Turbine.UI.Label()
    self.GroupEffectTimerText:SetParent(self.GroupEffectTimerBackground)
    self.GroupEffectTimerText:SetSize(100, 30)
    self.GroupEffectTimerText:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft)
    self.GroupEffectTimerText:SetFont(Turbine.UI.Lotro.Font.BookAntiquaBold19)
    self.GroupEffectTimerText:SetFontStyle(Turbine.UI.FontStyle.Outline)
    self.GroupEffectTimerText:SetMouseVisible(false)
    self.GroupEffectTimerText:SetPosition(195, 0)
    self.GroupEffectTimerText:SetVisible(true)
    self.GroupEffectTimerText:SetText("Timer Text")

    self.GroupEffectTimerTextBox = Turbine.UI.Lotro.TextBox()
    self.GroupEffectTimerTextBox:SetParent(self.GroupEffectTimerBackground)
    self.GroupEffectTimerTextBox:SetSize(170, 25)
    self.GroupEffectTimerTextBox:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
    self.GroupEffectTimerTextBox:SetMouseVisible(false)
    self.GroupEffectTimerTextBox:SetMultiline(false)
    self.GroupEffectTimerTextBox:SetPosition(270, 3)

    self.GroupEffectTimerDurationLabel = Turbine.UI.Label()
    self.GroupEffectTimerDurationLabel:SetParent(self.GroupEffectTimerBackground)
    self.GroupEffectTimerDurationLabel:SetSize(200, 30)
    self.GroupEffectTimerDurationLabel:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft)
    self.GroupEffectTimerDurationLabel:SetFont(Turbine.UI.Lotro.Font.BookAntiquaBold19)
    self.GroupEffectTimerDurationLabel:SetFontStyle(Turbine.UI.FontStyle.Outline)
    self.GroupEffectTimerDurationLabel:SetMouseVisible(false)
    self.GroupEffectTimerDurationLabel:SetPosition(10, 30)
    self.GroupEffectTimerDurationLabel:SetVisible(true)
    self.GroupEffectTimerDurationLabel:SetText("Timer duration in seconds")

    self.GroupEffectTimerDurationTextBox = Turbine.UI.Lotro.TextBox()
    self.GroupEffectTimerDurationTextBox:SetParent(self.GroupEffectTimerBackground)
    self.GroupEffectTimerDurationTextBox:SetSize(50, 30)
    self.GroupEffectTimerDurationTextBox:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
    self.GroupEffectTimerDurationTextBox:SetMouseVisible(false)
    self.GroupEffectTimerDurationTextBox:SetPosition(185, 30)
    self.GroupEffectTimerDurationTextBox:SetMultiline(false)

    self.GroupEffectTimerIconLabel = Turbine.UI.Lotro.CheckBox()
    self.GroupEffectTimerIconLabel:SetParent(self.GroupEffectTimerBackground)
    self.GroupEffectTimerIconLabel:SetSize(50, 30)
    self.GroupEffectTimerIconLabel:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft)
    self.GroupEffectTimerIconLabel:SetFont(Turbine.UI.Lotro.Font.BookAntiquaBold19)
    self.GroupEffectTimerIconLabel:SetFontStyle(Turbine.UI.FontStyle.Outline)
    self.GroupEffectTimerIconLabel:SetMouseVisible(false)
    self.GroupEffectTimerIconLabel:SetPosition(240, 30)
    self.GroupEffectTimerIconLabel:SetVisible(true)
    self.GroupEffectTimerIconLabel:SetText("Icon")

    self.GroupEffectTimerIconBox = Turbine.UI.Control()
    self.GroupEffectTimerIconBox:SetParent(self.GroupEffectTimerBackground)
    self.GroupEffectTimerIconBox:SetPosition(290, 30)
    self.GroupEffectTimerIconBox:SetSize(32,32)
    self.GroupEffectTimerIconBox:SetBlendMode(Turbine.UI.BlendMode.AlphaBlend)
    self.GroupEffectTimerIconBox:SetBackground(BackgroundHandler:GetBackground(0x41007F41))
    self.GroupEffectTimerIconBox:SetVisible(true)

    self.GroupEffectTimerIconTextBox = Turbine.UI.Lotro.TextBox()
    self.GroupEffectTimerIconTextBox:SetParent(self.GroupEffectTimerBackground)
    self.GroupEffectTimerIconTextBox:SetSize(110, 30)
    self.GroupEffectTimerIconTextBox:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
    self.GroupEffectTimerIconTextBox:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
    self.GroupEffectTimerIconTextBox:SetMouseVisible(false)
    self.GroupEffectTimerIconTextBox:SetPosition(330, 30)
    self.GroupEffectTimerIconTextBox:SetMultiline(false)
    self.GroupEffectTimerIconTextBox:SetText(0x41007F41)

    self.GroupEffectTimerIconTextBox.TextChanged = function()
        --pcall(self.GroupEffectTimerIconBox:SetBackground(tonumber(self.GroupEffectTimerIconTextBox:GetText())))
    end

    self.GroupEffectTimerIconBox.MouseClick = function()
        if self.IconOverview then 
            self.IconOverview:SetVisible(true) 
            self.iconLink = self.GroupEffectTimerIconBox
            self.iconTextLink = self.GroupEffectTimerIconTextBox
        end
    end

    self.GroupEffectTimerIconBox.MouseDoubleClick = function()
        self.GroupEffectTimerIconBox:SetBackground(BackgroundHandler:GetBackground(0x41007F41))
        self.iconID = 0x41007F41
    end

    self.GroupEffectTimerUnique = Turbine.UI.Lotro.CheckBox()
    self.GroupEffectTimerUnique:SetParent(self.GroupEffectTimerBackground)
    self.GroupEffectTimerUnique:SetSize(200, 30)
    self.GroupEffectTimerUnique:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft)
    self.GroupEffectTimerUnique:SetFont(Turbine.UI.Lotro.Font.BookAntiquaBold19)
    self.GroupEffectTimerUnique:SetFontStyle(Turbine.UI.FontStyle.Outline)
    self.GroupEffectTimerUnique:SetMouseVisible(false)
    self.GroupEffectTimerUnique:SetPosition(10, 50)
    self.GroupEffectTimerUnique:SetVisible(true)
    self.GroupEffectTimerUnique:SetText("Unique")

    self.GroupEffectTimerLoop = Turbine.UI.Lotro.CheckBox()
    self.GroupEffectTimerLoop:SetParent(self.GroupEffectTimerBackground)
    self.GroupEffectTimerLoop:SetSize(200, 30)
    self.GroupEffectTimerLoop:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft)
    self.GroupEffectTimerLoop:SetFont(Turbine.UI.Lotro.Font.BookAntiquaBold19)
    self.GroupEffectTimerLoop:SetFontStyle(Turbine.UI.FontStyle.Outline)
    self.GroupEffectTimerLoop:SetMouseVisible(false)
    self.GroupEffectTimerLoop:SetPosition(80, 50)
    self.GroupEffectTimerLoop:SetVisible(true)
    self.GroupEffectTimerLoop:SetText("Loop")

    self.GroupEffectIconFromEffect = Turbine.UI.Lotro.CheckBox()
    self.GroupEffectIconFromEffect:SetParent(self.GroupEffectBackground)
    self.GroupEffectIconFromEffect:SetSize(400, 20)
    self.GroupEffectIconFromEffect:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft)
    self.GroupEffectIconFromEffect:SetFont(Turbine.UI.Lotro.Font.BookAntiquaBold19)
    self.GroupEffectIconFromEffect:SetFontStyle(Turbine.UI.FontStyle.Outline)
    self.GroupEffectIconFromEffect:SetMouseVisible(false)
    self.GroupEffectIconFromEffect:SetPosition(10, 320)
    self.GroupEffectIconFromEffect:SetVisible(true)
    self.GroupEffectIconFromEffect:SetText("Use icon from effect (overwrites icon options)")

    self.GroupEffectTieringEffect = Turbine.UI.Lotro.CheckBox()
    self.GroupEffectTieringEffect:SetParent(self.GroupEffectBackground)
    self.GroupEffectTieringEffect:SetSize(400, 20)
    self.GroupEffectTieringEffect:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft)
    self.GroupEffectTieringEffect:SetFont(Turbine.UI.Lotro.Font.BookAntiquaBold19)
    self.GroupEffectTieringEffect:SetFontStyle(Turbine.UI.FontStyle.Outline)
    self.GroupEffectTieringEffect:SetMouseVisible(false)
    self.GroupEffectTieringEffect:SetPosition(10, 340)
    self.GroupEffectTieringEffect:SetVisible(true)
    self.GroupEffectTieringEffect:SetText("Tiering effect")

    self.GroupEffectTieringEffectLabel = Turbine.UI.Label()
    self.GroupEffectTieringEffectLabel:SetParent(self.GroupEffectBackground)
    self.GroupEffectTieringEffectLabel:SetSize(400, 50)
    self.GroupEffectTieringEffectLabel:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft)
    self.GroupEffectTieringEffectLabel:SetFont(Turbine.UI.Lotro.Font.Verdana12)
    self.GroupEffectTieringEffectLabel:SetFontStyle(Turbine.UI.FontStyle.Outline)
    self.GroupEffectTieringEffectLabel:SetMouseVisible(false)
    self.GroupEffectTieringEffectLabel:SetPosition(10, 345)
    self.GroupEffectTieringEffectLabel:SetVisible(true)
    self.GroupEffectTieringEffectLabel:SetText("(When checked do not specify the tier of effect e. g. \"Effect tier - 2\", you omit the 2. The tier of the effect will be added to the message automatically.")

    self.GroupEffectSaveButton = Turbine.UI.Lotro.Button()
    self.GroupEffectSaveButton:SetParent(self.GroupEffectBackground)
    self.GroupEffectSaveButton:SetSize(100, 15)
    self.GroupEffectSaveButton:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
    self.GroupEffectSaveButton:SetMouseVisible(true)
    self.GroupEffectSaveButton:SetPosition(170, 5)
    self.GroupEffectSaveButton:SetText("Save")

    self.GroupEffectRemoveButton = Turbine.UI.Lotro.Button()
    self.GroupEffectRemoveButton:SetParent(self.GroupEffectBackground)
    self.GroupEffectRemoveButton:SetSize(100, 15)
    self.GroupEffectRemoveButton:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
    self.GroupEffectRemoveButton:SetMouseVisible(true)
    self.GroupEffectRemoveButton:SetPosition(270, 5)
    self.GroupEffectRemoveButton:SetText("Remove")

    self.GroupEffectRemoveButton.Click = function()
        EventWindows:AddGroupEffectAlert(tostring(self.GroupEffectNameTextBox:GetText()), nil)
        self:RemoveNode(self.GroupEffectNode, tostring(self.GroupEffectNameTextBox:GetText()))
        self:ClearGroupEffect()
        ChatOutput("Removed Alert")
    end

    self.GroupEffectClearButton = Turbine.UI.Lotro.Button()
    self.GroupEffectClearButton:SetParent(self.GroupEffectBackground)
    self.GroupEffectClearButton:SetSize(100, 15)
    self.GroupEffectClearButton:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
    self.GroupEffectClearButton:SetMouseVisible(true)
    self.GroupEffectClearButton:SetPosition(370, 5)
    self.GroupEffectClearButton:SetText("Clear")

    self.GroupEffectClearButton.Click = function()
        self:ClearGroupEffect()
    end

    -- [[
    self.GroupEffectSaveButton.Click = function()
        -- Trigger
        local source = tostring(self.GroupEffectNameTextBox:GetText())
        local instanceName = tostring(self.GroupEffectInstanceButton:GetText())
        local roleName = tostring(self.GroupEffectRoleButton:GetText())

        -- Alert
        local alertYes = self.GroupEffectAlertCheckBox:IsChecked()
        local alertText = tostring(self.GroupEffectAlertTextBox:GetText())
        local alertDuration = tonumber(self.GroupEffectAlertDurationTextBox:GetText()) or 6
        local alertCountdown = self.GroupEffectAlertCountdownCheckBox:IsChecked()
        local alertSound = self.GroupEffectAlertSoundLabel:IsChecked()
        local alertSoundEffect = tostring(self.GroupEffectSoundeffectButton:GetText())
        local alertIconBox = nil
        local alertIconText = nil
        if self.GroupEffectAlertIconLabel:IsChecked() then
            alertIconBox = tonumber(self.GroupEffectAlertIconBox:GetBackground())
            alertIconText = tonumber(self.GroupEffectAlertIconTextBox:GetText())
        end

        -- Timer
        local timerYes = self.GroupEffectTimerCheckBox:IsChecked()
        local timerDuration = tonumber(self.GroupEffectTimerDurationTextBox:GetText()) or 6
        local timerBackgroundIcon = nil
        local timerBackgroundText = nil
        if self.GroupEffectTimerIconLabel:IsChecked() then
            timerBackgroundIcon = tonumber(self.GroupEffectTimerIconBox:GetBackground())
            timerBackgroundText = tonumber(self.GroupEffectTimerIconTextBox:GetText())
        end
        local timerText = tostring(self.GroupEffectTimerTextBox:GetText())
        local timerUnique = self.GroupEffectTimerUnique:IsChecked()
        local timerLoop = self.GroupEffectTimerLoop:IsChecked()

        local tieringEffect = self.GroupEffectTieringEffect:IsChecked()

        if self.GroupEffectIconFromEffect:IsChecked() then
            alertIconBox = true
            alertIconText = true
            timerBackgroundIcon = true
            timerBackgroundText = true
        end


        --1 duration, 2 message, 3 icon, 4 countdown, 5 soundeffect, 6 instanceID, 7 role, 8 timerduration, 9 timer text, 10 timer icon, 11 unique, 12 loop, 13 tieringEffect
        -- Valid trigger entry
        if source and instanceName and roleName then
            local alert = {}
            local timer = {}
            local restrict = {}
            if roleName ~= "All" then restrict[2] = Roles:NameToRoleID(roleName) end
            if instanceName ~= "All" then 
                local result = nil
                for key, value in pairs(InstanceController:GetDungeonNameTable()) do
                    if instanceName == value then
                        result = key
                        break
                    end
                end
                restrict[1] = result 
            end


            -- Valid Alert entry
            if alertYes and alertText and alertDuration then
                alert[1] = alertDuration
                alert[2] = alertText
                alert[3] = alertIconText
                alert[4] = alertCountdown
                ChatOutput("Added Alert Succesfully") 
                if alertSound then alert[5] = Sounds:GetSoundList()[alertSoundEffect] or alertSoundEffect end
            elseif alertYes then
                ChatOutput("Failed to add alert")
                if not(alertText) then ChatOutput("No Alert Text Specified") end
                if not(alertDuration) then ChatOutput("No Alert Duration Specified") end
            end

            --Valid Timer entry
            if timerYes and timerDuration and timerText then
                timer[1] = timerDuration
                timer[2] = timerText
                timer[3] = timerBackgroundText
                timer[4] = timerUnique
                timer[5] = timerLoop
                ChatOutput("Added Timer Succesfully")
            elseif timerYes then
                ChatOutput("Failed to add timer")
                if not(timerText) then ChatOutput("No Timer Text Specified") end
                if not(timerDuration) then ChatOutput("No Timer Duration Specified") end
            end

            restrict[3] = tieringEffect
            
            local completeInfo = {["Timers"] = {[1] = timer}, ["Alerts"] = {[1] = alert}, ["Restrictions"] = restrict}
            EventWindows:AddGroupEffectAlert(source, completeInfo)
            self:AddTreeNode(self.GroupEffectNode, self.CurrentAlertBackground:GetWidth() - 10, source, function() self:FillGroupEffectOptions(source, completeInfo) end, self.GroupEffectListBackground)
            self:RefreshTreeview()
            ChatOutput("Trigger added")
        else
            ChatOutput("Failed to add alert, check your input")
        end
    end


    self.SelectionWindows["Effects (Group)"] = self.GroupEffectBackground

    self.GroupEffectListBackground = Turbine.UI.TreeNode()
    self.GroupEffectListBackground:SetSize(self.CurrentAlertBackground:GetWidth() - 10, 30)
    
    self.GroupEffectListBoxHeader = Turbine.UI.Control()
    self.GroupEffectListBoxHeader:SetParent(self.GroupEffectListBackground)
    self.GroupEffectListBoxHeader:SetSize(self.CurrentAlertBackground:GetWidth() - 10, 30)

    self.GroupEffectListBoxHeaderLeft = Turbine.UI.Control()
    self.GroupEffectListBoxHeaderLeft:SetParent(self.GroupEffectListBoxHeader)
    self.GroupEffectListBoxHeaderLeft:SetSize(34, 30)
    self.GroupEffectListBoxHeaderLeft:SetBlendMode(Turbine.UI.BlendMode.AlphaBlend)
    self.GroupEffectListBoxHeaderLeft:SetBackground(BackgroundHandler:GetBackground(0x4110A0E0))

    self.GroupEffectListBoxHeaderRight = Turbine.UI.Label()
    self.GroupEffectListBoxHeaderRight:SetParent(self.GroupEffectListBoxHeader)
    self.GroupEffectListBoxHeaderRight:SetPosition(self.CurrentAlertBackground:GetWidth() - 34 - 10, 0)
    self.GroupEffectListBoxHeaderRight:SetSize(34, 30)
    self.GroupEffectListBoxHeaderRight:SetBlendMode(Turbine.UI.BlendMode.AlphaBlend)
    self.GroupEffectListBoxHeaderRight:SetBackground(BackgroundHandler:GetBackground(0x4110A0DB))

    self.GroupEffectListBoxHeaderMiddle = Turbine.UI.Label()
    self.GroupEffectListBoxHeaderMiddle:SetParent(self.GroupEffectListBoxHeader)
    self.GroupEffectListBoxHeaderMiddle:SetPosition(34, 0)
    self.GroupEffectListBoxHeaderMiddle:SetSize(self.CurrentAlertBackground:GetWidth() - 68 - 10, 30)
    self.GroupEffectListBoxHeaderMiddle:SetBlendMode(Turbine.UI.BlendMode.AlphaBlend)
    self.GroupEffectListBoxHeaderMiddle:SetBackground(BackgroundHandler:GetBackground(0x4110A0D9))

    self.GroupEffectListBoxHeaderLabel = Turbine.UI.Label()
    self.GroupEffectListBoxHeaderLabel:SetParent(self.GroupEffectListBackground)
    self.GroupEffectListBoxHeaderLabel:SetSize(self.CurrentAlertBackground:GetWidth() - 10, 30)
    self.GroupEffectListBoxHeaderLabel:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
    self.GroupEffectListBoxHeaderLabel:SetFont(Turbine.UI.Lotro.Font.BookAntiquaBold22)
    self.GroupEffectListBoxHeaderLabel:SetBlendMode(Turbine.UI.BlendMode.AlphaBlend)
    self.GroupEffectListBoxHeaderLabel:SetText("Effects (Group)")
     
    self.GroupEffectListBoxHeaderLabel.MouseEnter = function()
        self.GroupEffectListBoxHeaderLeft:SetBackground(BackgroundHandler:GetBackground(0x4110A0DF))
        self.GroupEffectListBoxHeaderMiddle:SetBackground(BackgroundHandler:GetBackground(0x4110A0DD))
        self.GroupEffectListBoxHeaderRight:SetBackground(BackgroundHandler:GetBackground(0x4110A0DE))
    end

    self.GroupEffectListBoxHeaderLabel.MouseUp = function()
        self.GroupEffectListBoxHeaderLeft:SetBackground(BackgroundHandler:GetBackground(0x4110A0E0))
        self.GroupEffectListBoxHeaderMiddle:SetBackground(BackgroundHandler:GetBackground(0x4110A0D9))
        self.GroupEffectListBoxHeaderRight:SetBackground(BackgroundHandler:GetBackground(0x4110A0DB))
    end

    self.GroupEffectListBoxHeaderLabel.MouseDown = function()
        self.GroupEffectListBoxHeaderLeft:SetBackground(BackgroundHandler:GetBackground(0x4110A0D8))
        self.GroupEffectListBoxHeaderMiddle:SetBackground(BackgroundHandler:GetBackground(0x4110A0DA))
        self.GroupEffectListBoxHeaderRight:SetBackground(BackgroundHandler:GetBackground(0x4110A0DC))
    end

    self.GroupEffectListBoxHeaderLabel.MouseLeave = function()
        self.GroupEffectListBoxHeaderLeft:SetBackground(BackgroundHandler:GetBackground(0x4110A0E0))
        self.GroupEffectListBoxHeaderMiddle:SetBackground(BackgroundHandler:GetBackground(0x4110A0D9))
        self.GroupEffectListBoxHeaderRight:SetBackground(BackgroundHandler:GetBackground(0x4110A0DB))
    end

    self.rootNodes:Add(self.GroupEffectListBackground)
 
    self.GroupEffectNode = self.GroupEffectListBackground:GetChildNodes()
    
    -- [[
    for key, value in spairs(EventWindows:GetCurrentGroupEffectData(), function(t, a, b) return a < b end) do
        self:AddTreeNode(self.GroupEffectNode, self.CurrentAlertBackground:GetWidth() - 10, key, function() self:FillGroupEffectOptions(key, value) end)
    end
    --]]
 
end

function AlertOptions:ClearGroupEffect()
    self.GroupEffectNameTextBox:SetText("")
    self.GroupEffectInstanceButton:SetText("All")
    self.GroupEffectRoleButton:SetText("All")

    -- Alert
    self.GroupEffectAlertCheckBox:SetChecked(false)
    self.GroupEffectAlertTextBox:SetText("")
    self.GroupEffectAlertDurationTextBox:SetText("")
    self.GroupEffectAlertCountdownCheckBox:SetChecked(false)
    self.GroupEffectAlertSoundLabel:SetChecked(false)
    self.GroupEffectSoundeffectButton:SetText("")
    self.GroupEffectAlertIconLabel:SetChecked(false)
    self.GroupEffectAlertIconBox:SetBackground(BackgroundHandler:GetBackground(0x41007F41))
    self.GroupEffectAlertIconTextBox:SetText(0x41007F41)

    -- Timer
    self.GroupEffectTimerCheckBox:SetChecked(false)
    self.GroupEffectTimerDurationTextBox:SetText("")
    self.GroupEffectTimerIconLabel:SetChecked(false)
    self.GroupEffectTimerIconBox:SetBackground(BackgroundHandler:GetBackground(0x41007F41))
    self.GroupEffectTimerIconTextBox:SetText(0x41007F41)
    self.GroupEffectTimerTextBox:SetText("")
    self.GroupEffectTimerUnique:SetChecked(false)
    self.GroupEffectTimerLoop:SetChecked(false)
    self.GroupEffectIconFromEffect:SetChecked(false)
    self.GroupEffectTieringEffect:SetChecked(false)
end

function AlertOptions:FillGroupEffectOptions(effectName, input)
    self:ToggleWindow("Effects (Group)")
   -- All effect added alerts {1 duration, 2 message, 3 icon, 4 countdown, 5 soundeffect, 6 5_countdown, 7 instanceID, 8 role}
   -- Trigger
   self.GroupEffectNameTextBox:SetText(tostring(effectName))

   local timer = input["Timers"][1]
   local alert = input["Alerts"][1]
   local restrict = input["Restrictions"]

   if restrict[1] then self.GroupEffectInstanceButton:SetText(tostring(InstanceController:GetDungeonNameTable()[tonumber(restrict[1])])) else self.GroupEffectInstanceButton:SetText("All") end
   if restrict[2] then self.GroupEffectRoleButton:SetText(Roles:GetAvailableRoles()[restrict[2]]) else self.GroupEffectRoleButton:SetText("All") end
   

   -- Alert
   self.GroupEffectAlertCheckBox:SetChecked(alert[1])

   -- Text
   if alert[2] then self.GroupEffectAlertTextBox:SetText(tostring(alert[2])) else self.GroupEffectAlertTextBox:SetText("") end

   -- duration
   self.GroupEffectAlertDurationTextBox:SetText(tonumber(alert[1]))

    -- Icon
    self.GroupEffectAlertIconLabel:SetChecked(alert[3])
    if alert[3] == true or timer[3] == true then 
        self.GroupEffectIconFromEffect:SetChecked(alert[3] or timer[3])
        self.GroupEffectAlertIconLabel:SetChecked(false)
    elseif alert[3] then
        self.GroupEffectAlertIconBox:SetBackground(BackgroundHandler:GetBackground(tonumber(alert[3])))
        self.GroupEffectAlertIconTextBox:SetText(tonumber(alert[3]))
        self.GroupEffectIconFromEffect:SetChecked(false)
    else
        self.GroupEffectAlertIconBox:SetBackground("")
        self.GroupEffectAlertIconTextBox:SetText("")
        self.GroupEffectIconFromEffect:SetChecked(false)
    end
    if alert[3] then self.GroupEffectAlertIconTextBox:SetText(tostring(alert[3])) else self.GroupEffectAlertIconTextBox:SetText("") end

   self.GroupEffectAlertCountdownCheckBox:SetChecked(alert[4])

   -- Sound effect
   self.GroupEffectAlertSoundLabel:SetChecked(alert[5])
   self.GroupEffectSoundeffectButton:SetText(alert[5])

   -- Timer
   -- [[
    -- [1] Duration [2] Message [3] Icon (Special/Game/None) [4] Countdown [5] SoundEffect [6] instanceID [7] role [8] timerduration, [9] timer text, [10] timer icon, [11] unique, [12] loop, [13] tiering effect}
   self.GroupEffectTimerCheckBox:SetChecked(timer[1])
   self.GroupEffectTimerDurationTextBox:SetText(tonumber(timer[1]))
   self.GroupEffectTimerIconLabel:SetChecked(timer[3])
   if timer[3] then
       self.GroupEffectTimerIconBox:SetBackground(BackgroundHandler:GetBackground(tonumber(timer[3])))
       self.GroupEffectTimerIconTextBox:SetText(tonumber(timer[3]))
   else
       self.GroupEffectTimerIconBox:SetBackground("")
       self.GroupEffectTimerIconTextBox:SetText("")
   end
   if timer[2] then self.GroupEffectTimerTextBox:SetText(tostring(timer[2])) else self.GroupEffectTimerTextBox:SetText("") end
   self.GroupEffectTimerUnique:SetChecked(timer[4])
   self.GroupEffectTimerLoop:SetChecked(timer[5])

   self.GroupEffectTieringEffect:SetChecked(restrict[3])

   --]]
end

-- Creation/Changing self effect alert UI
function AlertOptions:SelfEffectAlerts(parent)
    self.SelfEffectBackground = Turbine.UI.Control()
    self.SelfEffectBackground:SetParent(parent)
    self.SelfEffectBackground:SetPosition(0, 0)
    self.SelfEffectBackground:SetVisible(false)
    self.SelfEffectBackground:SetSize(parent:GetWidth(), parent:GetHeight())

    self.SelfEffectHeader = Turbine.UI.Label()
    self.SelfEffectHeader:SetParent(self.SelfEffectBackground)
    self.SelfEffectHeader:SetSize(200, 30)
    self.SelfEffectHeader:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft)
    self.SelfEffectHeader:SetFont(Turbine.UI.Lotro.Font.BookAntiquaBold24)
    self.SelfEffectHeader:SetFontStyle(Turbine.UI.FontStyle.Outline)
    self.SelfEffectHeader:SetMouseVisible(false)
    self.SelfEffectHeader:SetPosition(10, 0)
    self.SelfEffectHeader:SetVisible(true)
    self.SelfEffectHeader:SetText("Effects (Self)")

    self.SelfEffectNameLabel = Turbine.UI.Label()
    self.SelfEffectNameLabel:SetParent(self.SelfEffectBackground)
    self.SelfEffectNameLabel:SetSize(100, 30)
    self.SelfEffectNameLabel:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft)
    self.SelfEffectNameLabel:SetFont(Turbine.UI.Lotro.Font.BookAntiquaBold19)
    self.SelfEffectNameLabel:SetFontStyle(Turbine.UI.FontStyle.Outline)
    self.SelfEffectNameLabel:SetMouseVisible(false)
    self.SelfEffectNameLabel:SetPosition(10, 30)
    self.SelfEffectNameLabel:SetVisible(true)
    self.SelfEffectNameLabel:SetText("Effect Name")

    self.SelfEffectNameTextBox = Turbine.UI.Lotro.TextBox()
    self.SelfEffectNameTextBox:SetParent(self.SelfEffectBackground)
    self.SelfEffectNameTextBox:SetSize(300, 30)
    self.SelfEffectNameTextBox:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
    self.SelfEffectNameTextBox:SetMouseVisible(false)
    self.SelfEffectNameTextBox:SetPosition(100, 30)
    self.SelfEffectNameTextBox:SetMultiline(false)

    self.SelfEffectHistoryButton = Turbine.UI.Lotro.Button()
    self.SelfEffectHistoryButton:SetParent(self.SelfEffectBackground)
    self.SelfEffectHistoryButton:SetSize(70, 15)
    self.SelfEffectHistoryButton:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
    self.SelfEffectHistoryButton:SetMouseVisible(true)
    self.SelfEffectHistoryButton:SetPosition(400, 35)
    self.SelfEffectHistoryButton:SetText("Browse")

    self.SelfEffectHistoryButton.Click = function()
        self.EffectsOverview:SetVisible(true)
        self.HistoricalEffectsTextLink = self.SelfEffectNameTextBox
    end

    
    self.SelfEffectInstanceLabel = Turbine.UI.Label()
    self.SelfEffectInstanceLabel:SetParent(self.SelfEffectBackground)
    self.SelfEffectInstanceLabel:SetSize(300, 30)
    self.SelfEffectInstanceLabel:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft)
    self.SelfEffectInstanceLabel:SetFont(Turbine.UI.Lotro.Font.BookAntiquaBold19)
    self.SelfEffectInstanceLabel:SetFontStyle(Turbine.UI.FontStyle.Outline)
    self.SelfEffectInstanceLabel:SetMouseVisible(false)
    self.SelfEffectInstanceLabel:SetPosition(10, 65)
    self.SelfEffectInstanceLabel:SetVisible(true)
    self.SelfEffectInstanceLabel:SetText("Only load for specific instance/role")

    self.SelfEffectInstanceButton = Turbine.UI.Lotro.Button()
    self.SelfEffectInstanceButton:SetParent(self.SelfEffectBackground)
    self.SelfEffectInstanceButton:SetSize(100, 15)
    self.SelfEffectInstanceButton:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
    self.SelfEffectInstanceButton:SetMouseVisible(true)
    self.SelfEffectInstanceButton:SetPosition(240, 70)
    self.SelfEffectInstanceButton:SetText("All")

    self.SelfEffectInstanceMenu = UI.DropDownMenuControl(self, 200, "Instance Dropdown Menu")
    for key, value in spairs(InstanceController:GetDungeonNameTable(), function(t, a, b) return t[a] < t[b] end) do
        self.SelfEffectInstanceMenu:AddItem(value, function()
            self.SelfEffectInstanceButton:SetText(value)
            self.SelfEffectInstanceMenu:SetVisible(false)
        end)
    end

    self.SelfEffectInstanceButton.Click = function()
        self.SelfEffectInstanceMenu:ToggleVisible()
    end

    self.SelfEffectRoleButton = Turbine.UI.Lotro.Button()
    self.SelfEffectRoleButton:SetParent(self.SelfEffectBackground)
    self.SelfEffectRoleButton:SetSize(100, 15)
    self.SelfEffectRoleButton:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
    self.SelfEffectRoleButton:SetMouseVisible(true)
    self.SelfEffectRoleButton:SetPosition(340, 70)
    self.SelfEffectRoleButton:SetText("All")

    self.SelfEffectRoleMenu = Turbine.UI.ContextMenu()
    self.SelfEffectRoleMenuItems = self.SelfEffectRoleMenu:GetItems()
    for key, value in spairs(Roles:GetAvailableRoles(), function(t, a, b) return t[a] < t[b] end) do
        local menuItem = Turbine.UI.MenuItem( value )
        self.SelfEffectRoleMenuItems:Add(menuItem)
        menuItem.Click = function()
            self.SelfEffectRoleButton:SetText(value)
        end
    end

    local menuItem = Turbine.UI.MenuItem( "All" )
    self.SelfEffectRoleMenuItems:Add(menuItem)
    menuItem.Click = function()
        self.SelfEffectRoleButton:SetText("All")
    end

    self.SelfEffectRoleButton.Click = function()
        self.SelfEffectRoleMenu:ShowMenu()
    end

    -- Alert
    self.SelfEffectAlertBackground = Turbine.UI.Control()
    self.SelfEffectAlertBackground:SetParent(self.SelfEffectBackground)
    self.SelfEffectAlertBackground:SetPosition(10, 100)
    self.SelfEffectAlertBackground:SetSize(parent:GetWidth() - 20, parent:GetHeight() - 260)
    self.SelfEffectAlertBackground:SetBackColor(Turbine.UI.Color(0.5, 0.5, 0.5))

    self.SelfEffectAlertCheckBox = Turbine.UI.Lotro.CheckBox()
    self.SelfEffectAlertCheckBox:SetParent(self.SelfEffectAlertBackground)
    self.SelfEffectAlertCheckBox:SetSize(200, 30)
    self.SelfEffectAlertCheckBox:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft)
    self.SelfEffectAlertCheckBox:SetFont(Turbine.UI.Lotro.Font.BookAntiquaBold24)
    self.SelfEffectAlertCheckBox:SetFontStyle(Turbine.UI.FontStyle.Outline)
    self.SelfEffectAlertCheckBox:SetMouseVisible(false)
    self.SelfEffectAlertCheckBox:SetPosition(3, 3)
    self.SelfEffectAlertCheckBox:SetVisible(true)
    self.SelfEffectAlertCheckBox:SetText("Alert on Trigger")

    self.PhraseLabel = Turbine.UI.Label()
    self.PhraseLabel:SetParent(self.SelfEffectAlertBackground)
    self.PhraseLabel:SetSize(100, 30)
    self.PhraseLabel:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft)
    self.PhraseLabel:SetFont(Turbine.UI.Lotro.Font.BookAntiquaBold19)
    self.PhraseLabel:SetFontStyle(Turbine.UI.FontStyle.Outline)
    self.PhraseLabel:SetMouseVisible(false)
    self.PhraseLabel:SetPosition(10, 35)
    self.PhraseLabel:SetVisible(true)
    self.PhraseLabel:SetText("Alert Text")

    self.SelfEffectAlertTextBox = Turbine.UI.Lotro.TextBox()
    self.SelfEffectAlertTextBox:SetParent(self.SelfEffectAlertBackground)
    self.SelfEffectAlertTextBox:SetSize(200, 30)
    self.SelfEffectAlertTextBox:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
    self.SelfEffectAlertTextBox:SetMouseVisible(false)
    self.SelfEffectAlertTextBox:SetMultiline(false)
    self.SelfEffectAlertTextBox:SetPosition(80, 35)

    self.SelfEffectAlertDurationLabel = Turbine.UI.Label()
    self.SelfEffectAlertDurationLabel:SetParent(self.SelfEffectAlertBackground)
    self.SelfEffectAlertDurationLabel:SetSize(200, 30)
    self.SelfEffectAlertDurationLabel:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft)
    self.SelfEffectAlertDurationLabel:SetFont(Turbine.UI.Lotro.Font.BookAntiquaBold19)
    self.SelfEffectAlertDurationLabel:SetFontStyle(Turbine.UI.FontStyle.Outline)
    self.SelfEffectAlertDurationLabel:SetMouseVisible(false)
    self.SelfEffectAlertDurationLabel:SetPosition(10, 65)
    self.SelfEffectAlertDurationLabel:SetVisible(true)
    self.SelfEffectAlertDurationLabel:SetText("Alert duration in seconds")

    self.SelfEffectAlertDurationTextBox = Turbine.UI.Lotro.TextBox()
    self.SelfEffectAlertDurationTextBox:SetParent(self.SelfEffectAlertBackground)
    self.SelfEffectAlertDurationTextBox:SetSize(50, 30)
    self.SelfEffectAlertDurationTextBox:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
    self.SelfEffectAlertDurationTextBox:SetMouseVisible(false)
    self.SelfEffectAlertDurationTextBox:SetPosition(185, 65)
    self.SelfEffectAlertDurationTextBox:SetMultiline(false)


    -- Countdown
    self.SelfEffectAlertCountdownCheckBox = Turbine.UI.Lotro.CheckBox()
    self.SelfEffectAlertCountdownCheckBox:SetParent(self.SelfEffectAlertBackground)
    self.SelfEffectAlertCountdownCheckBox:SetSize(200, 30)
    self.SelfEffectAlertCountdownCheckBox:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft)
    self.SelfEffectAlertCountdownCheckBox:SetFont(Turbine.UI.Lotro.Font.BookAntiquaBold19)
    self.SelfEffectAlertCountdownCheckBox:SetFontStyle(Turbine.UI.FontStyle.Outline)
    self.SelfEffectAlertCountdownCheckBox:SetMouseVisible(false)
    self.SelfEffectAlertCountdownCheckBox:SetPosition(250, 65)
    self.SelfEffectAlertCountdownCheckBox:SetVisible(true)
    self.SelfEffectAlertCountdownCheckBox:SetText("Alert duration shown")


    self.SelfEffectAlertSoundCountdownCheckBox = Turbine.UI.Lotro.CheckBox()
    self.SelfEffectAlertSoundCountdownCheckBox:SetParent(self.SelfEffectAlertBackground)
    self.SelfEffectAlertSoundCountdownCheckBox:SetSize(200, 30)
    self.SelfEffectAlertSoundCountdownCheckBox:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft)
    self.SelfEffectAlertSoundCountdownCheckBox:SetFont(Turbine.UI.Lotro.Font.BookAntiquaBold19)
    self.SelfEffectAlertSoundCountdownCheckBox:SetFontStyle(Turbine.UI.FontStyle.Outline)
    self.SelfEffectAlertSoundCountdownCheckBox:SetMouseVisible(false)
    self.SelfEffectAlertSoundCountdownCheckBox:SetPosition(250, 5)
    self.SelfEffectAlertSoundCountdownCheckBox:SetVisible(true)
    self.SelfEffectAlertSoundCountdownCheckBox:SetText("Sound countdown from 5")

    -- Sound effects
    self.SelfEffectAlertSoundLabel = Turbine.UI.Lotro.CheckBox()
    self.SelfEffectAlertSoundLabel:SetParent(self.SelfEffectAlertBackground)
    self.SelfEffectAlertSoundLabel:SetSize(200, 30)
    self.SelfEffectAlertSoundLabel:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft)
    self.SelfEffectAlertSoundLabel:SetFont(Turbine.UI.Lotro.Font.BookAntiquaBold19)
    self.SelfEffectAlertSoundLabel:SetFontStyle(Turbine.UI.FontStyle.Outline)
    self.SelfEffectAlertSoundLabel:SetMouseVisible(false)
    self.SelfEffectAlertSoundLabel:SetPosition(10, 90)
    self.SelfEffectAlertSoundLabel:SetVisible(true)
    self.SelfEffectAlertSoundLabel:SetText("Sound Effect")

    self.SelfEffectSoundeffectButton = Turbine.UI.Lotro.Button()
    self.SelfEffectSoundeffectButton:SetParent(self.SelfEffectAlertBackground)
    self.SelfEffectSoundeffectButton:SetSize(100, 15)
    self.SelfEffectSoundeffectButton:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
    self.SelfEffectSoundeffectButton:SetMouseVisible(true)
    self.SelfEffectSoundeffectButton:SetPosition(120, 97)
    self.SelfEffectSoundeffectButton:SetText("Horn")

    self.SelfEffectSoundEffectMenu = UI.DropDownMenuControl(self, 200, "Sound Menu")
    for key, value in spairs(Sounds:GetSoundList(), function(t, a, b) return a < b end) do
        self.SelfEffectSoundEffectMenu:AddItem(key, 
        function()
            self.SelfEffectSoundeffectButton:SetText(key)
            self.SelfEffectSoundEffectMenu:ToggleVisible()
        end)
    end

    self.SelfEffectSoundeffectButton.Click = function()
        self.SelfEffectSoundEffectMenu:ToggleVisible()
    end

    self.SelfEffectAlertIconLabel = Turbine.UI.Lotro.CheckBox()
    self.SelfEffectAlertIconLabel:SetParent(self.SelfEffectAlertBackground)
    self.SelfEffectAlertIconLabel:SetSize(50, 30)
    self.SelfEffectAlertIconLabel:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft)
    self.SelfEffectAlertIconLabel:SetFont(Turbine.UI.Lotro.Font.BookAntiquaBold19)
    self.SelfEffectAlertIconLabel:SetFontStyle(Turbine.UI.FontStyle.Outline)
    self.SelfEffectAlertIconLabel:SetMouseVisible(false)
    self.SelfEffectAlertIconLabel:SetPosition(240, 97)
    self.SelfEffectAlertIconLabel:SetVisible(true)
    self.SelfEffectAlertIconLabel:SetText("Icon")

    self.SelfEffectAlertIconBox = Turbine.UI.Control()
    self.SelfEffectAlertIconBox:SetParent(self.SelfEffectAlertBackground)
    self.SelfEffectAlertIconBox:SetPosition(290, 97)
    self.SelfEffectAlertIconBox:SetSize(32,32)
    self.SelfEffectAlertIconBox:SetBlendMode(Turbine.UI.BlendMode.AlphaBlend)
    self.SelfEffectAlertIconBox:SetBackground(BackgroundHandler:GetBackground(0x41007F41))
    self.SelfEffectAlertIconBox:SetVisible(true)

    self.SelfEffectAlertIconTextBox = Turbine.UI.Lotro.TextBox()
    self.SelfEffectAlertIconTextBox:SetParent(self.SelfEffectAlertBackground)
    self.SelfEffectAlertIconTextBox:SetSize(110, 30)
    self.SelfEffectAlertIconTextBox:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
    self.SelfEffectAlertIconTextBox:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
    self.SelfEffectAlertIconTextBox:SetMouseVisible(false)
    self.SelfEffectAlertIconTextBox:SetPosition(330, 97)
    self.SelfEffectAlertIconTextBox:SetMultiline(false)
    self.SelfEffectAlertIconTextBox:SetText(0x41007F41)

    self.SelfEffectAlertIconTextBox.TextChanged = function()
        --pcall(self.SelfEffectAlertIconBox:SetBackground(tonumber(self.SelfEffectAlertIconTextBox:GetText())))
    end

    self.SelfEffectAlertIconBox.MouseClick = function()
        if self.IconOverview then 
            self.IconOverview:SetVisible(true) 
            self.iconLink = self.SelfEffectAlertIconBox
            self.iconTextLink = self.SelfEffectAlertIconTextBox
        end
    end

    self.SelfEffectAlertIconBox.MouseDoubleClick = function()
        self.SelfEffectAlertIconBox:SetBackground(BackgroundHandler:GetBackground(0x41007F41))
        self.iconID = 0x41007F41
    end

    --Timer
    self.SelfEffectTimerBackground = Turbine.UI.Control()
    self.SelfEffectTimerBackground:SetParent(self.SelfEffectBackground)
    self.SelfEffectTimerBackground:SetPosition(10, 240)
    self.SelfEffectTimerBackground:SetSize(parent:GetWidth() - 20, 80)
    self.SelfEffectTimerBackground:SetBackColor(Turbine.UI.Color(0.5, 0.5, 0.5))

    self.SelfEffectTimerCheckBox = Turbine.UI.Lotro.CheckBox()
    self.SelfEffectTimerCheckBox:SetParent(self.SelfEffectTimerBackground)
    self.SelfEffectTimerCheckBox:SetSize(200, 30)
    self.SelfEffectTimerCheckBox:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft)
    self.SelfEffectTimerCheckBox:SetFont(Turbine.UI.Lotro.Font.BookAntiquaBold24)
    self.SelfEffectTimerCheckBox:SetFontStyle(Turbine.UI.FontStyle.Outline)
    self.SelfEffectTimerCheckBox:SetMouseVisible(false)
    self.SelfEffectTimerCheckBox:SetPosition(3, 3)
    self.SelfEffectTimerCheckBox:SetVisible(true)
    self.SelfEffectTimerCheckBox:SetText("Timer on Trigger")

    self.SelfEffectTimerText = Turbine.UI.Label()
    self.SelfEffectTimerText:SetParent(self.SelfEffectTimerBackground)
    self.SelfEffectTimerText:SetSize(100, 30)
    self.SelfEffectTimerText:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft)
    self.SelfEffectTimerText:SetFont(Turbine.UI.Lotro.Font.BookAntiquaBold19)
    self.SelfEffectTimerText:SetFontStyle(Turbine.UI.FontStyle.Outline)
    self.SelfEffectTimerText:SetMouseVisible(false)
    self.SelfEffectTimerText:SetPosition(195, 0)
    self.SelfEffectTimerText:SetVisible(true)
    self.SelfEffectTimerText:SetText("Timer Text")

    self.SelfEffectTimerTextBox = Turbine.UI.Lotro.TextBox()
    self.SelfEffectTimerTextBox:SetParent(self.SelfEffectTimerBackground)
    self.SelfEffectTimerTextBox:SetSize(170, 25)
    self.SelfEffectTimerTextBox:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
    self.SelfEffectTimerTextBox:SetMouseVisible(false)
    self.SelfEffectTimerTextBox:SetMultiline(false)
    self.SelfEffectTimerTextBox:SetPosition(270, 3)

    self.SelfEffectTimerDurationLabel = Turbine.UI.Label()
    self.SelfEffectTimerDurationLabel:SetParent(self.SelfEffectTimerBackground)
    self.SelfEffectTimerDurationLabel:SetSize(200, 30)
    self.SelfEffectTimerDurationLabel:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft)
    self.SelfEffectTimerDurationLabel:SetFont(Turbine.UI.Lotro.Font.BookAntiquaBold19)
    self.SelfEffectTimerDurationLabel:SetFontStyle(Turbine.UI.FontStyle.Outline)
    self.SelfEffectTimerDurationLabel:SetMouseVisible(false)
    self.SelfEffectTimerDurationLabel:SetPosition(10, 30)
    self.SelfEffectTimerDurationLabel:SetVisible(true)
    self.SelfEffectTimerDurationLabel:SetText("Timer duration in seconds")

    self.SelfEffectTimerDurationTextBox = Turbine.UI.Lotro.TextBox()
    self.SelfEffectTimerDurationTextBox:SetParent(self.SelfEffectTimerBackground)
    self.SelfEffectTimerDurationTextBox:SetSize(50, 30)
    self.SelfEffectTimerDurationTextBox:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
    self.SelfEffectTimerDurationTextBox:SetMouseVisible(false)
    self.SelfEffectTimerDurationTextBox:SetPosition(185, 30)
    self.SelfEffectTimerDurationTextBox:SetMultiline(false)

    self.SelfEffectTimerIconLabel = Turbine.UI.Lotro.CheckBox()
    self.SelfEffectTimerIconLabel:SetParent(self.SelfEffectTimerBackground)
    self.SelfEffectTimerIconLabel:SetSize(50, 30)
    self.SelfEffectTimerIconLabel:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft)
    self.SelfEffectTimerIconLabel:SetFont(Turbine.UI.Lotro.Font.BookAntiquaBold19)
    self.SelfEffectTimerIconLabel:SetFontStyle(Turbine.UI.FontStyle.Outline)
    self.SelfEffectTimerIconLabel:SetMouseVisible(false)
    self.SelfEffectTimerIconLabel:SetPosition(240, 30)
    self.SelfEffectTimerIconLabel:SetVisible(true)
    self.SelfEffectTimerIconLabel:SetText("Icon")

    self.SelfEffectTimerIconBox = Turbine.UI.Control()
    self.SelfEffectTimerIconBox:SetParent(self.SelfEffectTimerBackground)
    self.SelfEffectTimerIconBox:SetPosition(290, 30)
    self.SelfEffectTimerIconBox:SetSize(32,32)
    self.SelfEffectTimerIconBox:SetBlendMode(Turbine.UI.BlendMode.AlphaBlend)
    self.SelfEffectTimerIconBox:SetBackground(BackgroundHandler:GetBackground(0x41007F41))
    self.SelfEffectTimerIconBox:SetVisible(true)

    self.SelfEffectTimerIconTextBox = Turbine.UI.Lotro.TextBox()
    self.SelfEffectTimerIconTextBox:SetParent(self.SelfEffectTimerBackground)
    self.SelfEffectTimerIconTextBox:SetSize(110, 30)
    self.SelfEffectTimerIconTextBox:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
    self.SelfEffectTimerIconTextBox:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
    self.SelfEffectTimerIconTextBox:SetMouseVisible(false)
    self.SelfEffectTimerIconTextBox:SetPosition(330, 30)
    self.SelfEffectTimerIconTextBox:SetMultiline(false)
    self.SelfEffectTimerIconTextBox:SetText(0x41007F41)

    self.SelfEffectTimerIconTextBox.TextChanged = function()
        --pcall(self.SelfEffectTimerIconBox:SetBackground(tonumber(self.SelfEffectTimerIconTextBox:GetText())))
    end

    self.SelfEffectTimerIconBox.MouseClick = function()
        if self.IconOverview then 
            self.IconOverview:SetVisible(true) 
            self.iconLink = self.SelfEffectTimerIconBox
            self.iconTextLink = self.SelfEffectTimerIconTextBox
        end
    end

    self.SelfEffectTimerIconBox.MouseDoubleClick = function()
        self.SelfEffectTimerIconBox:SetBackground(BackgroundHandler:GetBackground(0x41007F41))
        self.iconID = 0x41007F41
    end

    self.SelfEffectTimerUnique = Turbine.UI.Lotro.CheckBox()
    self.SelfEffectTimerUnique:SetParent(self.SelfEffectTimerBackground)
    self.SelfEffectTimerUnique:SetSize(200, 30)
    self.SelfEffectTimerUnique:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft)
    self.SelfEffectTimerUnique:SetFont(Turbine.UI.Lotro.Font.BookAntiquaBold19)
    self.SelfEffectTimerUnique:SetFontStyle(Turbine.UI.FontStyle.Outline)
    self.SelfEffectTimerUnique:SetMouseVisible(false)
    self.SelfEffectTimerUnique:SetPosition(10, 50)
    self.SelfEffectTimerUnique:SetVisible(true)
    self.SelfEffectTimerUnique:SetText("Unique")

    self.SelfEffectTimerLoop = Turbine.UI.Lotro.CheckBox()
    self.SelfEffectTimerLoop:SetParent(self.SelfEffectTimerBackground)
    self.SelfEffectTimerLoop:SetSize(200, 30)
    self.SelfEffectTimerLoop:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft)
    self.SelfEffectTimerLoop:SetFont(Turbine.UI.Lotro.Font.BookAntiquaBold19)
    self.SelfEffectTimerLoop:SetFontStyle(Turbine.UI.FontStyle.Outline)
    self.SelfEffectTimerLoop:SetMouseVisible(false)
    self.SelfEffectTimerLoop:SetPosition(80, 50)
    self.SelfEffectTimerLoop:SetVisible(true)
    self.SelfEffectTimerLoop:SetText("Loop")

    self.SelfEffectIconFromEffect = Turbine.UI.Lotro.CheckBox()
    self.SelfEffectIconFromEffect:SetParent(self.SelfEffectBackground)
    self.SelfEffectIconFromEffect:SetSize(400, 20)
    self.SelfEffectIconFromEffect:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft)
    self.SelfEffectIconFromEffect:SetFont(Turbine.UI.Lotro.Font.BookAntiquaBold19)
    self.SelfEffectIconFromEffect:SetFontStyle(Turbine.UI.FontStyle.Outline)
    self.SelfEffectIconFromEffect:SetMouseVisible(false)
    self.SelfEffectIconFromEffect:SetPosition(10, 320)
    self.SelfEffectIconFromEffect:SetVisible(true)
    self.SelfEffectIconFromEffect:SetText("Use icon from effect (overwrites icon options)")

    self.SelfEffectTieringEffect = Turbine.UI.Lotro.CheckBox()
    self.SelfEffectTieringEffect:SetParent(self.SelfEffectBackground)
    self.SelfEffectTieringEffect:SetSize(400, 20)
    self.SelfEffectTieringEffect:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft)
    self.SelfEffectTieringEffect:SetFont(Turbine.UI.Lotro.Font.BookAntiquaBold19)
    self.SelfEffectTieringEffect:SetFontStyle(Turbine.UI.FontStyle.Outline)
    self.SelfEffectTieringEffect:SetMouseVisible(false)
    self.SelfEffectTieringEffect:SetPosition(10, 340)
    self.SelfEffectTieringEffect:SetVisible(true)
    self.SelfEffectTieringEffect:SetText("Tiering effect")

    self.SelfEffectTieringEffectLabel = Turbine.UI.Label()
    self.SelfEffectTieringEffectLabel:SetParent(self.SelfEffectBackground)
    self.SelfEffectTieringEffectLabel:SetSize(400, 50)
    self.SelfEffectTieringEffectLabel:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft)
    self.SelfEffectTieringEffectLabel:SetFont(Turbine.UI.Lotro.Font.Verdana12)
    self.SelfEffectTieringEffectLabel:SetFontStyle(Turbine.UI.FontStyle.Outline)
    self.SelfEffectTieringEffectLabel:SetMouseVisible(false)
    self.SelfEffectTieringEffectLabel:SetPosition(10, 345)
    self.SelfEffectTieringEffectLabel:SetVisible(true)
    self.SelfEffectTieringEffectLabel:SetText("(When checked do not specify the tier of effect e. g. \"Effect tier - 2\", you omit the 2. The tier of the effect will be added to the message automatically.")


    self.SelfEffectSaveButton = Turbine.UI.Lotro.Button()
    self.SelfEffectSaveButton:SetParent(self.SelfEffectBackground)
    self.SelfEffectSaveButton:SetSize(100, 15)
    self.SelfEffectSaveButton:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
    self.SelfEffectSaveButton:SetMouseVisible(true)
    self.SelfEffectSaveButton:SetPosition(170, 5)
    self.SelfEffectSaveButton:SetText("Save")

    
    self.SelfEffectRemoveButton = Turbine.UI.Lotro.Button()
    self.SelfEffectRemoveButton:SetParent(self.SelfEffectBackground)
    self.SelfEffectRemoveButton:SetSize(100, 15)
    self.SelfEffectRemoveButton:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
    self.SelfEffectRemoveButton:SetMouseVisible(true)
    self.SelfEffectRemoveButton:SetPosition(270, 5)
    self.SelfEffectRemoveButton:SetText("Remove")

    self.SelfEffectRemoveButton.Click = function()
        EventWindows:AddSelfEffectAlert(tostring(self.SelfEffectNameTextBox:GetText()), nil)
        self:RemoveNode(self.SelfEffectNode, tostring(self.SelfEffectNameTextBox:GetText()))
        self:ClearSelfEffect()
        ChatOutput("Removed Alert")
    end

    self.SelfEffectClearButton = Turbine.UI.Lotro.Button()
    self.SelfEffectClearButton:SetParent(self.SelfEffectBackground)
    self.SelfEffectClearButton:SetSize(100, 15)
    self.SelfEffectClearButton:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
    self.SelfEffectClearButton:SetMouseVisible(true)
    self.SelfEffectClearButton:SetPosition(370, 5)
    self.SelfEffectClearButton:SetText("Clear")

    self.SelfEffectClearButton.Click = function()
        self:ClearSelfEffect()
    end


    -- [[
    self.SelfEffectSaveButton.Click = function()
        -- Trigger
        local source = tostring(self.SelfEffectNameTextBox:GetText())
        local instanceName = tostring(self.SelfEffectInstanceButton:GetText())
        local roleName = tostring(self.SelfEffectRoleButton:GetText())

        -- Alert
        local alertYes = self.SelfEffectAlertCheckBox:IsChecked()
        local alertText = tostring(self.SelfEffectAlertTextBox:GetText())
        local alertDuration = tonumber(self.SelfEffectAlertDurationTextBox:GetText()) or 6
        local alertCountdown = self.SelfEffectAlertCountdownCheckBox:IsChecked()
        local alertSoundCountdown = self.SelfEffectAlertSoundCountdownCheckBox:IsChecked()
        local alertSound = self.SelfEffectAlertSoundLabel:IsChecked()
        local alertSoundEffect = tostring(self.SelfEffectSoundeffectButton:GetText())
        local alertIconBox = nil
        local alertIconText = nil
        if self.SelfEffectAlertIconLabel:IsChecked() then
            alertIconBox = tonumber(self.SelfEffectAlertIconBox:GetBackground())
            alertIconText = tonumber(self.SelfEffectAlertIconTextBox:GetText())
        end

        -- Timer
        local timerYes = self.SelfEffectTimerCheckBox:IsChecked()
        local timerDuration = tonumber(self.SelfEffectTimerDurationTextBox:GetText()) or 6
        local timerBackgroundIcon = nil
        local timerBackgroundText = nil
        if self.SelfEffectTimerIconLabel:IsChecked() then
            timerBackgroundIcon = tonumber(self.SelfEffectTimerIconBox:GetBackground())
            timerBackgroundText = tonumber(self.SelfEffectTimerIconTextBox:GetText())
        end
        local timerText = tostring(self.SelfEffectTimerTextBox:GetText())
        local timerUnique = self.SelfEffectTimerUnique:IsChecked()
        local timerLoop = self.SelfEffectTimerLoop:IsChecked()

        local tieringEffect = self.SelfEffectTieringEffect:IsChecked()

        if self.SelfEffectIconFromEffect:IsChecked() then
            alertIconBox = true
            alertIconText = true
            timerBackgroundIcon = true
            timerBackgroundText = true
        end

        local newAlert = {}
         -- All effect added alerts {1 duration, 2 message, 3 icon, 4 countdown, 5 soundeffect, 6 5_countdown, 7 instanceID, 8 role 9 timer duration 10 timer text, 11 timer icon, 12 unique, 13 loop, 14 tiering effect}
        -- Valid trigger entry
        if source and instanceName and roleName then
            local alert = {}
            local timer = {}
            local restrict = {}
            if roleName ~= "All" then restrict[2] = Roles:NameToRoleID(roleName) end
            if instanceName ~= "All" then 
                local result = nil
                for key, value in pairs(InstanceController:GetDungeonNameTable()) do
                    if instanceName == value then
                        result = key
                        break
                    end
                end
                restrict[1] = result 
            end


            -- Valid Alert entry
            if alertYes and alertText and alertDuration then
                alert[1] = alertDuration
                alert[2] = alertText
                alert[3] = alertIconText
                alert[4] = alertCountdown
                ChatOutput("Added Alert Succesfully")
                if alertSound then alert[5] = Sounds:GetSoundList()[alertSoundEffect] or alertSoundEffect end
                alert[6] = alertSoundCountdown
            elseif alertYes then
                ChatOutput("Failed to add alert")
                if not(alertText) then ChatOutput("No Alert Text Specified") end
                if not(alertDuration) then ChatOutput("No Alert Duration Specified") end
            end

            --Valid Timer entry
            if timerYes and timerDuration and timerText then
                timer[1] = timerDuration
                timer[2] = timerText
                timer[3] = timerBackgroundText
                timer[4] = timerUnique
                timer[5] = timerLoop
                ChatOutput("Added Timer Succesfully")
            elseif timerYes then
                ChatOutput("Failed to add timer")
                if not(timerText) then ChatOutput("No Timer Text Specified") end
                if not(timerDuration) then ChatOutput("No Timer Duration Specified") end
            end

            restrict[3] = tieringEffect

            local completeInfo = {["Timers"] = {[1] = timer}, ["Alerts"] = {[1] = alert}, ["Restrictions"] = restrict}
            EventWindows:AddSelfEffectAlert(source, completeInfo)
            self:AddTreeNode(self.SelfEffectNode, self.CurrentAlertBackground:GetWidth() - 10, source, function() self:FillSelfEffectOptions(source, completeInfo) end, self.SelfEffectListBackground)
            self:RefreshTreeview()
            ChatOutput("Trigger added")
        else
            ChatOutput("Failed to add alert, check your input")
        end
    end


    self.SelectionWindows["Effects (Self)"] = self.SelfEffectBackground

    self.SelfEffectListBackground = Turbine.UI.TreeNode()
    self.SelfEffectListBackground:SetSize(self.CurrentAlertBackground:GetWidth() - 10, 30)

    self.SelfEffectListBackground = Turbine.UI.TreeNode()
    self.SelfEffectListBackground:SetSize(self.CurrentAlertBackground:GetWidth() - 10, 30)
    
    self.SelfEffectListBoxHeader = Turbine.UI.Control()
    self.SelfEffectListBoxHeader:SetParent(self.SelfEffectListBackground)
    self.SelfEffectListBoxHeader:SetSize(self.CurrentAlertBackground:GetWidth() - 10, 30)

    self.SelfEffectListBoxHeaderLeft = Turbine.UI.Control()
    self.SelfEffectListBoxHeaderLeft:SetParent(self.SelfEffectListBoxHeader)
    self.SelfEffectListBoxHeaderLeft:SetSize(34, 30)
    self.SelfEffectListBoxHeaderLeft:SetBlendMode(Turbine.UI.BlendMode.AlphaBlend)
    self.SelfEffectListBoxHeaderLeft:SetBackground(BackgroundHandler:GetBackground(0x4110A0E0))

    self.SelfEffectListBoxHeaderRight = Turbine.UI.Label()
    self.SelfEffectListBoxHeaderRight:SetParent(self.SelfEffectListBoxHeader)
    self.SelfEffectListBoxHeaderRight:SetPosition(self.CurrentAlertBackground:GetWidth() - 34 - 10, 0)
    self.SelfEffectListBoxHeaderRight:SetSize(34, 30)
    self.SelfEffectListBoxHeaderRight:SetBlendMode(Turbine.UI.BlendMode.AlphaBlend)
    self.SelfEffectListBoxHeaderRight:SetBackground(BackgroundHandler:GetBackground(0x4110A0DB))

    self.SelfEffectListBoxHeaderMiddle = Turbine.UI.Label()
    self.SelfEffectListBoxHeaderMiddle:SetParent(self.SelfEffectListBoxHeader)
    self.SelfEffectListBoxHeaderMiddle:SetPosition(34, 0)
    self.SelfEffectListBoxHeaderMiddle:SetSize(self.CurrentAlertBackground:GetWidth() - 68 - 10, 30)
    self.SelfEffectListBoxHeaderMiddle:SetBlendMode(Turbine.UI.BlendMode.AlphaBlend)
    self.SelfEffectListBoxHeaderMiddle:SetBackground(BackgroundHandler:GetBackground(0x4110A0D9))

    self.SelfEffectListBoxHeaderLabel = Turbine.UI.Label()
    self.SelfEffectListBoxHeaderLabel:SetParent(self.SelfEffectListBackground)
    self.SelfEffectListBoxHeaderLabel:SetSize(self.CurrentAlertBackground:GetWidth() - 10, 30)
    self.SelfEffectListBoxHeaderLabel:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
    self.SelfEffectListBoxHeaderLabel:SetFont(Turbine.UI.Lotro.Font.BookAntiquaBold22)
    self.SelfEffectListBoxHeaderLabel:SetBlendMode(Turbine.UI.BlendMode.AlphaBlend)
    self.SelfEffectListBoxHeaderLabel:SetText("Effects (Self)")
     
    self.SelfEffectListBoxHeaderLabel.MouseEnter = function()
        self.SelfEffectListBoxHeaderLeft:SetBackground(BackgroundHandler:GetBackground(0x4110A0DF))
        self.SelfEffectListBoxHeaderMiddle:SetBackground(BackgroundHandler:GetBackground(0x4110A0DD))
        self.SelfEffectListBoxHeaderRight:SetBackground(BackgroundHandler:GetBackground(0x4110A0DE))
    end

    self.SelfEffectListBoxHeaderLabel.MouseUp = function()
        self.SelfEffectListBoxHeaderLeft:SetBackground(BackgroundHandler:GetBackground(0x4110A0E0))
        self.SelfEffectListBoxHeaderMiddle:SetBackground(BackgroundHandler:GetBackground(0x4110A0D9))
        self.SelfEffectListBoxHeaderRight:SetBackground(BackgroundHandler:GetBackground(0x4110A0DB))
    end

    self.SelfEffectListBoxHeaderLabel.MouseDown = function()
        self.SelfEffectListBoxHeaderLeft:SetBackground(BackgroundHandler:GetBackground(0x4110A0D8))
        self.SelfEffectListBoxHeaderMiddle:SetBackground(BackgroundHandler:GetBackground(0x4110A0DA))
        self.SelfEffectListBoxHeaderRight:SetBackground(BackgroundHandler:GetBackground(0x4110A0DC))
    end

    self.SelfEffectListBoxHeaderLabel.MouseLeave = function()
        self.SelfEffectListBoxHeaderLeft:SetBackground(BackgroundHandler:GetBackground(0x4110A0E0))
        self.SelfEffectListBoxHeaderMiddle:SetBackground(BackgroundHandler:GetBackground(0x4110A0D9))
        self.SelfEffectListBoxHeaderRight:SetBackground(BackgroundHandler:GetBackground(0x4110A0DB))
    end


    self.rootNodes:Add(self.SelfEffectListBackground)
 
    self.SelfEffectNode = self.SelfEffectListBackground:GetChildNodes()
    
    -- [[
    for key, value in spairs(EventWindows:GetCurrentSelfEffectData(), function(t, a, b) return a < b end) do
        self:AddTreeNode(self.SelfEffectNode, self.CurrentAlertBackground:GetWidth() - 10, key, function() self:FillSelfEffectOptions(key, value) end)
    end
    --]]
 
end

function AlertOptions:ClearSelfEffect()
    self.SelfEffectNameTextBox:SetText("")
    self.SelfEffectInstanceButton:SetText("All")
    self.SelfEffectRoleButton:SetText("All")

    -- Alert
    self.SelfEffectAlertCheckBox:SetChecked(false)
    self.SelfEffectAlertTextBox:SetText("")
    self.SelfEffectAlertDurationTextBox:SetText("")
    self.SelfEffectAlertCountdownCheckBox:SetChecked(false)
    self.SelfEffectAlertSoundLabel:SetChecked(false)
    self.SelfEffectSoundeffectButton:SetText("")
    self.SelfEffectAlertIconLabel:SetChecked(false)
    self.SelfEffectAlertIconBox:SetBackground(BackgroundHandler:GetBackground(0x41007F41))
    self.SelfEffectAlertIconTextBox:SetText(0x41007F41)
    self.SelfEffectAlertSoundCountdownCheckBox:SetChecked(false)

    -- Timer
    self.SelfEffectTimerCheckBox:SetChecked(false)
    self.SelfEffectTimerDurationTextBox:SetText("")
    self.SelfEffectTimerIconLabel:SetChecked(false)
    self.SelfEffectTimerIconBox:SetBackground(BackgroundHandler:GetBackground(0x41007F41))
    self.SelfEffectTimerIconTextBox:SetText(0x41007F41)
    self.SelfEffectTimerTextBox:SetText("")
    self.SelfEffectTimerUnique:SetChecked(false)
    self.SelfEffectTimerLoop:SetChecked(false)
    self.SelfEffectIconFromEffect:SetChecked(false)
    self.SelfEffectTieringEffect:SetChecked(false)
end

function AlertOptions:FillSelfEffectOptions(effectName, input)
    self:ToggleWindow("Effects (Self)")
   -- All effect added alerts {1 duration, 2 message, 3 icon, 4 countdown, 5 soundeffect, 6 5_countdown, 7 instanceID, 8 role}
   -- Trigger
   self.SelfEffectNameTextBox:SetText(tostring(effectName))

   local timer = input["Timers"][1]
   local alert = input["Alerts"][1]
   local restrict = input["Restrictions"]

   if restrict[1] then self.SelfEffectInstanceButton:SetText(tostring(InstanceController:GetDungeonNameTable()[tonumber(restrict[1])])) else self.SelfEffectInstanceButton:SetText("All") end
   if restrict[2] then self.SelfEffectRoleButton:SetText(Roles:GetAvailableRoles()[restrict[2]]) else self.SelfEffectRoleButton:SetText("All") end
   

   -- Alert
   self.SelfEffectAlertCheckBox:SetChecked(alert[1])

   -- Text
   if alert[2] then self.SelfEffectAlertTextBox:SetText(tostring(alert[2])) else self.SelfEffectAlertTextBox:SetText("") end

   -- duration
   self.SelfEffectAlertDurationTextBox:SetText(tonumber(alert[1]))

    -- Icon
    self.SelfEffectAlertIconLabel:SetChecked(alert[3])
    if alert[3] == true or timer[3] == true then 
        self.SelfEffectIconFromEffect:SetChecked(alert[3] or timer[3])
        self.SelfEffectAlertIconLabel:SetChecked(false)
    elseif alert[3] then
        self.SelfEffectAlertIconBox:SetBackground(BackgroundHandler:GetBackground(tonumber(alert[3])))
        self.SelfEffectAlertIconTextBox:SetText(tonumber(alert[3]))
        self.SelfEffectIconFromEffect:SetChecked(false)
    else
        self.SelfEffectAlertIconBox:SetBackground("")
        self.SelfEffectAlertIconTextBox:SetText("")
        self.SelfEffectIconFromEffect:SetChecked(false)
    end
    if alert[3] then self.SelfEffectAlertIconTextBox:SetText(tostring(alert[3])) else self.SelfEffectAlertIconTextBox:SetText("") end

   self.SelfEffectAlertCountdownCheckBox:SetChecked(alert[4])

   -- Sound effect
   self.SelfEffectAlertSoundLabel:SetChecked(alert[5])
   self.SelfEffectSoundeffectButton:SetText(alert[5])
   self.SelfEffectAlertSoundCountdownCheckBox:SetChecked(alert[6])

   -- Timer
   -- [[
    -- All effect added alerts {1 duration, 2 message, 3 icon, 4 countdown, 5 soundeffect, 6 5_countdown, 7 instanceID, 8 role 9 timer duration 10 timer text, 11 timer icon, 12 unique, 13 loop, 14 tiering effect}
   self.SelfEffectTimerCheckBox:SetChecked(timer[1])
   self.SelfEffectTimerDurationTextBox:SetText(tonumber(timer[1]))
   self.SelfEffectTimerIconLabel:SetChecked(timer[3])
   if alert[3] then
       self.SelfEffectTimerIconBox:SetBackground(BackgroundHandler:GetBackground(tonumber(timer[3])))
       self.SelfEffectTimerIconTextBox:SetText(tonumber(timer[3]))
   else
       self.SelfEffectTimerIconBox:SetBackground("")
       self.SelfEffectTimerIconTextBox:SetText("")
   end
   if timer[2] then self.SelfEffectTimerTextBox:SetText(tostring(timer[2])) else self.SelfEffectTimerTextBox:SetText("") end
   self.SelfEffectTimerUnique:SetChecked(timer[4])
   self.SelfEffectTimerLoop:SetChecked(timer[5])

   self.SelfEffectTieringEffect:SetChecked(restrict[3])

   --]]
end

-- Creation/Changing chat alert UI
function AlertOptions:ChatAlerts(parent)

    self.ChatBackground = Turbine.UI.Control()
    self.ChatBackground:SetParent(parent)
    self.ChatBackground:SetVisible(false)
    self.ChatBackground:SetPosition(0, 0)
    self.ChatBackground:SetSize(parent:GetWidth(), parent:GetHeight())

    self.SelectionWindows["Chat (Standard)"] = self.ChatBackground

    -- Source
    self.SourceLabel = Turbine.UI.Label()
    self.SourceLabel:SetParent(self.ChatBackground)
    self.SourceLabel:SetSize(100, 30)
    self.SourceLabel:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft)
    self.SourceLabel:SetFont(Turbine.UI.Lotro.Font.BookAntiquaBold19)
    self.SourceLabel:SetFontStyle(Turbine.UI.FontStyle.Outline)
    self.SourceLabel:SetMouseVisible(false)
    self.SourceLabel:SetPosition(10, 30)
    self.SourceLabel:SetVisible(true)
    self.SourceLabel:SetText("Chat source")

    self.SourceTextBox = Turbine.UI.Lotro.TextBox()
    self.SourceTextBox:SetParent(self.ChatBackground)
    self.SourceTextBox:SetSize(300, 30)
    self.SourceTextBox:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
    self.SourceTextBox:SetMouseVisible(false)
    self.SourceTextBox:SetPosition(100, 30)
    self.SourceTextBox:SetMultiline(false)

    -- Key phrase
    self.PhraseLabel = Turbine.UI.Label()
    self.PhraseLabel:SetParent(self.ChatBackground)
    self.PhraseLabel:SetSize(200, 30)
    self.PhraseLabel:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft)
    self.PhraseLabel:SetFont(Turbine.UI.Lotro.Font.BookAntiquaBold19)
    self.PhraseLabel:SetFontStyle(Turbine.UI.FontStyle.Outline)
    self.PhraseLabel:SetMouseVisible(false)
    self.PhraseLabel:SetPosition(10, 70)
    self.PhraseLabel:SetVisible(true)
    self.PhraseLabel:SetText("Trigger Phrase")

    self.PhraseTextBox = Turbine.UI.Lotro.TextBox()
    self.PhraseTextBox:SetParent(self.ChatBackground)
    self.PhraseTextBox:SetSize(300, 60)
    self.PhraseTextBox:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
    self.PhraseTextBox:SetMouseVisible(false)
    self.PhraseTextBox:SetPosition(140, 70)
    self.PhraseTextBox:SetMultiline(true)

    self.ChatInstanceLabel = Turbine.UI.Label()
    self.ChatInstanceLabel:SetParent(self.ChatBackground)
    self.ChatInstanceLabel:SetSize(300, 30)
    self.ChatInstanceLabel:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft)
    self.ChatInstanceLabel:SetFont(Turbine.UI.Lotro.Font.BookAntiquaBold19)
    self.ChatInstanceLabel:SetFontStyle(Turbine.UI.FontStyle.Outline)
    self.ChatInstanceLabel:SetMouseVisible(false)
    self.ChatInstanceLabel:SetPosition(10, 125)
    self.ChatInstanceLabel:SetVisible(true)
    self.ChatInstanceLabel:SetText("Only load for specific instance/role")

    self.ChatInstanceButton = Turbine.UI.Lotro.Button()
    self.ChatInstanceButton:SetParent(self.ChatBackground)
    self.ChatInstanceButton:SetSize(100, 15)
    self.ChatInstanceButton:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
    self.ChatInstanceButton:SetMouseVisible(true)
    self.ChatInstanceButton:SetPosition(240, 133)
    self.ChatInstanceButton:SetText("All")

    self.ChatInstanceMenu = UI.DropDownMenuControl(self, 200, "Instance Dropdown Menu")
    for key, value in spairs(InstanceController:GetDungeonNameTable(), function(t, a, b) return t[a] < t[b] end) do
        self.ChatInstanceMenu:AddItem(value, function()
            self.ChatInstanceButton:SetText(value)
            self.ChatInstanceMenu:SetVisible(false)
        end)
    end

    self.ChatInstanceButton.Click = function()
        self.ChatInstanceMenu:ToggleVisible()
    end

    self.ChatRoleButton = Turbine.UI.Lotro.Button()
    self.ChatRoleButton:SetParent(self.ChatBackground)
    self.ChatRoleButton:SetSize(100, 15)
    self.ChatRoleButton:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
    self.ChatRoleButton:SetMouseVisible(true)
    self.ChatRoleButton:SetPosition(340, 133)
    self.ChatRoleButton:SetText("All")

    self.ChatRoleMenu = Turbine.UI.ContextMenu()
    self.ChatRoleMenuItems = self.ChatRoleMenu:GetItems()
    for key, value in spairs(Roles:GetAvailableRoles(), function(t, a, b) return t[a] < t[b] end) do
        local menuItem = Turbine.UI.MenuItem( value )
        self.ChatRoleMenuItems:Add(menuItem)
        menuItem.Click = function()
            self.ChatRoleButton:SetText(value)
        end
    end

    local menuItem = Turbine.UI.MenuItem( "All" )
    self.ChatRoleMenuItems:Add(menuItem)
    menuItem.Click = function()
        self.ChatRoleButton:SetText("All")
    end

    self.ChatRoleButton.Click = function()
        self.ChatRoleMenu:ShowMenu()
    end


    -- Alert
    self.ChatAlertBackground = Turbine.UI.Control()
    self.ChatAlertBackground:SetParent(self.ChatBackground)
    self.ChatAlertBackground:SetPosition(10, 160)
    self.ChatAlertBackground:SetSize(parent:GetWidth() - 20, parent:GetHeight() - 260)
    self.ChatAlertBackground:SetBackColor(Turbine.UI.Color(0.5, 0.5, 0.5))

    self.ChatAlertCheckBox = Turbine.UI.Lotro.CheckBox()
    self.ChatAlertCheckBox:SetParent(self.ChatAlertBackground)
    self.ChatAlertCheckBox:SetSize(200, 30)
    self.ChatAlertCheckBox:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft)
    self.ChatAlertCheckBox:SetFont(Turbine.UI.Lotro.Font.BookAntiquaBold24)
    self.ChatAlertCheckBox:SetFontStyle(Turbine.UI.FontStyle.Outline)
    self.ChatAlertCheckBox:SetMouseVisible(false)
    self.ChatAlertCheckBox:SetPosition(3, 3)
    self.ChatAlertCheckBox:SetVisible(true)
    self.ChatAlertCheckBox:SetText("Alert on Trigger")

    self.PhraseLabel = Turbine.UI.Label()
    self.PhraseLabel:SetParent(self.ChatAlertBackground)
    self.PhraseLabel:SetSize(100, 30)
    self.PhraseLabel:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft)
    self.PhraseLabel:SetFont(Turbine.UI.Lotro.Font.BookAntiquaBold19)
    self.PhraseLabel:SetFontStyle(Turbine.UI.FontStyle.Outline)
    self.PhraseLabel:SetMouseVisible(false)
    self.PhraseLabel:SetPosition(10, 35)
    self.PhraseLabel:SetVisible(true)
    self.PhraseLabel:SetText("Alert Text")

    self.ChatAlertTextBox = Turbine.UI.Lotro.TextBox()
    self.ChatAlertTextBox:SetParent(self.ChatAlertBackground)
    self.ChatAlertTextBox:SetSize(200, 30)
    self.ChatAlertTextBox:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
    self.ChatAlertTextBox:SetMouseVisible(false)
    self.ChatAlertTextBox:SetMultiline(false)
    self.ChatAlertTextBox:SetPosition(80, 35)

    self.ChatAlertDurationLabel = Turbine.UI.Label()
    self.ChatAlertDurationLabel:SetParent(self.ChatAlertBackground)
    self.ChatAlertDurationLabel:SetSize(200, 30)
    self.ChatAlertDurationLabel:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft)
    self.ChatAlertDurationLabel:SetFont(Turbine.UI.Lotro.Font.BookAntiquaBold19)
    self.ChatAlertDurationLabel:SetFontStyle(Turbine.UI.FontStyle.Outline)
    self.ChatAlertDurationLabel:SetMouseVisible(false)
    self.ChatAlertDurationLabel:SetPosition(10, 65)
    self.ChatAlertDurationLabel:SetVisible(true)
    self.ChatAlertDurationLabel:SetText("Alert duration in seconds")

    self.ChatAlertDurationTextBox = Turbine.UI.Lotro.TextBox()
    self.ChatAlertDurationTextBox:SetParent(self.ChatAlertBackground)
    self.ChatAlertDurationTextBox:SetSize(50, 30)
    self.ChatAlertDurationTextBox:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
    self.ChatAlertDurationTextBox:SetMouseVisible(false)
    self.ChatAlertDurationTextBox:SetPosition(185, 65)
    self.ChatAlertDurationTextBox:SetMultiline(false)


    -- Countdown
    self.ChatAlertCountdownCheckBox = Turbine.UI.Lotro.CheckBox()
    self.ChatAlertCountdownCheckBox:SetParent(self.ChatAlertBackground)
    self.ChatAlertCountdownCheckBox:SetSize(200, 30)
    self.ChatAlertCountdownCheckBox:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft)
    self.ChatAlertCountdownCheckBox:SetFont(Turbine.UI.Lotro.Font.BookAntiquaBold19)
    self.ChatAlertCountdownCheckBox:SetFontStyle(Turbine.UI.FontStyle.Outline)
    self.ChatAlertCountdownCheckBox:SetMouseVisible(false)
    self.ChatAlertCountdownCheckBox:SetPosition(250, 65)
    self.ChatAlertCountdownCheckBox:SetVisible(true)
    self.ChatAlertCountdownCheckBox:SetText("Alert duration shown")

    -- Sound effects
    self.ChatAlertSoundLabel = Turbine.UI.Lotro.CheckBox()
    self.ChatAlertSoundLabel:SetParent(self.ChatAlertBackground)
    self.ChatAlertSoundLabel:SetSize(200, 30)
    self.ChatAlertSoundLabel:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft)
    self.ChatAlertSoundLabel:SetFont(Turbine.UI.Lotro.Font.BookAntiquaBold19)
    self.ChatAlertSoundLabel:SetFontStyle(Turbine.UI.FontStyle.Outline)
    self.ChatAlertSoundLabel:SetMouseVisible(false)
    self.ChatAlertSoundLabel:SetPosition(10, 90)
    self.ChatAlertSoundLabel:SetVisible(true)
    self.ChatAlertSoundLabel:SetText("Sound Effect")

    self.ChatSoundeffectButton = Turbine.UI.Lotro.Button()
    self.ChatSoundeffectButton:SetParent(self.ChatAlertBackground)
    self.ChatSoundeffectButton:SetSize(100, 15)
    self.ChatSoundeffectButton:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
    self.ChatSoundeffectButton:SetMouseVisible(true)
    self.ChatSoundeffectButton:SetPosition(120, 97)
    self.ChatSoundeffectButton:SetText("Horn")

    self.ChatSoundEffectMenu = UI.DropDownMenuControl(self, 200, "Sound Menu")
    for key, value in spairs(Sounds:GetSoundList(), function(t, a, b) return a < b end) do
        self.ChatSoundEffectMenu:AddItem(key, 
        function()
            self.ChatSoundeffectButton:SetText(key)
            self.ChatSoundEffectMenu:ToggleVisible()
        end)
    end

    self.ChatSoundeffectButton.Click = function()
        self.ChatSoundEffectMenu:ToggleVisible()
    end

    self.ChatAlertIconLabel = Turbine.UI.Lotro.CheckBox()
    self.ChatAlertIconLabel:SetParent(self.ChatAlertBackground)
    self.ChatAlertIconLabel:SetSize(50, 30)
    self.ChatAlertIconLabel:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft)
    self.ChatAlertIconLabel:SetFont(Turbine.UI.Lotro.Font.BookAntiquaBold19)
    self.ChatAlertIconLabel:SetFontStyle(Turbine.UI.FontStyle.Outline)
    self.ChatAlertIconLabel:SetMouseVisible(false)
    self.ChatAlertIconLabel:SetPosition(240, 97)
    self.ChatAlertIconLabel:SetVisible(true)
    self.ChatAlertIconLabel:SetText("Icon")

    self.ChatAlertIconBox = Turbine.UI.Control()
    self.ChatAlertIconBox:SetParent(self.ChatAlertBackground)
    self.ChatAlertIconBox:SetPosition(290, 97)
    self.ChatAlertIconBox:SetSize(32,32)
    self.ChatAlertIconBox:SetBlendMode(Turbine.UI.BlendMode.AlphaBlend)
    self.ChatAlertIconBox:SetBackground(BackgroundHandler:GetBackground(0x41007F41))
    self.ChatAlertIconBox:SetVisible(true)

    self.ChatAlertIconTextBox = Turbine.UI.Lotro.TextBox()
    self.ChatAlertIconTextBox:SetParent(self.ChatAlertBackground)
    self.ChatAlertIconTextBox:SetSize(110, 30)
    self.ChatAlertIconTextBox:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
    self.ChatAlertIconTextBox:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
    self.ChatAlertIconTextBox:SetMouseVisible(false)
    self.ChatAlertIconTextBox:SetPosition(330, 97)
    self.ChatAlertIconTextBox:SetMultiline(false)
    self.ChatAlertIconTextBox:SetText(0x41007F41)

    self.ChatAlertIconTextBox.TextChanged = function()
        --pcall(self.ChatAlertIconBox:SetBackground(tonumber(self.ChatAlertIconTextBox:GetText())))
    end

    self.ChatAlertIconBox.MouseClick = function()
        if self.IconOverview then 
            self.IconOverview:SetVisible(true) 
            self.iconLink = self.ChatAlertIconBox
            self.iconTextLink = self.ChatAlertIconTextBox
        end
    end

    self.ChatAlertIconBox.MouseDoubleClick = function()
        self.ChatAlertIconBox:SetBackground(BackgroundHandler:GetBackground(0x41007F41))
        self.iconID = 0x41007F41
    end


    --Timer
    self.ChatTimerBackground = Turbine.UI.Control()
    self.ChatTimerBackground:SetParent(self.ChatBackground)
    self.ChatTimerBackground:SetPosition(10, 300)
    self.ChatTimerBackground:SetSize(parent:GetWidth() - 20, 80)
    self.ChatTimerBackground:SetBackColor(Turbine.UI.Color(0.5, 0.5, 0.5))

    self.ChatTimerCheckBox = Turbine.UI.Lotro.CheckBox()
    self.ChatTimerCheckBox:SetParent(self.ChatTimerBackground)
    self.ChatTimerCheckBox:SetSize(200, 30)
    self.ChatTimerCheckBox:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft)
    self.ChatTimerCheckBox:SetFont(Turbine.UI.Lotro.Font.BookAntiquaBold24)
    self.ChatTimerCheckBox:SetFontStyle(Turbine.UI.FontStyle.Outline)
    self.ChatTimerCheckBox:SetMouseVisible(false)
    self.ChatTimerCheckBox:SetPosition(3, 3)
    self.ChatTimerCheckBox:SetVisible(true)
    self.ChatTimerCheckBox:SetText("Timer on Trigger")

    self.ChatTimerText = Turbine.UI.Label()
    self.ChatTimerText:SetParent(self.ChatTimerBackground)
    self.ChatTimerText:SetSize(100, 30)
    self.ChatTimerText:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft)
    self.ChatTimerText:SetFont(Turbine.UI.Lotro.Font.BookAntiquaBold19)
    self.ChatTimerText:SetFontStyle(Turbine.UI.FontStyle.Outline)
    self.ChatTimerText:SetMouseVisible(false)
    self.ChatTimerText:SetPosition(195, 0)
    self.ChatTimerText:SetVisible(true)
    self.ChatTimerText:SetText("Timer Text")

    self.ChatTimerTextBox = Turbine.UI.Lotro.TextBox()
    self.ChatTimerTextBox:SetParent(self.ChatTimerBackground)
    self.ChatTimerTextBox:SetSize(170, 25)
    self.ChatTimerTextBox:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
    self.ChatTimerTextBox:SetMouseVisible(false)
    self.ChatTimerTextBox:SetMultiline(false)
    self.ChatTimerTextBox:SetPosition(270, 3)

    self.ChatTimerDurationLabel = Turbine.UI.Label()
    self.ChatTimerDurationLabel:SetParent(self.ChatTimerBackground)
    self.ChatTimerDurationLabel:SetSize(200, 30)
    self.ChatTimerDurationLabel:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft)
    self.ChatTimerDurationLabel:SetFont(Turbine.UI.Lotro.Font.BookAntiquaBold19)
    self.ChatTimerDurationLabel:SetFontStyle(Turbine.UI.FontStyle.Outline)
    self.ChatTimerDurationLabel:SetMouseVisible(false)
    self.ChatTimerDurationLabel:SetPosition(10, 30)
    self.ChatTimerDurationLabel:SetVisible(true)
    self.ChatTimerDurationLabel:SetText("Timer duration in seconds")

    self.ChatTimerDurationTextBox = Turbine.UI.Lotro.TextBox()
    self.ChatTimerDurationTextBox:SetParent(self.ChatTimerBackground)
    self.ChatTimerDurationTextBox:SetSize(50, 30)
    self.ChatTimerDurationTextBox:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
    self.ChatTimerDurationTextBox:SetMouseVisible(false)
    self.ChatTimerDurationTextBox:SetPosition(185, 30)
    self.ChatTimerDurationTextBox:SetMultiline(false)

    self.ChatTimerIconLabel = Turbine.UI.Lotro.CheckBox()
    self.ChatTimerIconLabel:SetParent(self.ChatTimerBackground)
    self.ChatTimerIconLabel:SetSize(50, 30)
    self.ChatTimerIconLabel:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft)
    self.ChatTimerIconLabel:SetFont(Turbine.UI.Lotro.Font.BookAntiquaBold19)
    self.ChatTimerIconLabel:SetFontStyle(Turbine.UI.FontStyle.Outline)
    self.ChatTimerIconLabel:SetMouseVisible(false)
    self.ChatTimerIconLabel:SetPosition(240, 30)
    self.ChatTimerIconLabel:SetVisible(true)
    self.ChatTimerIconLabel:SetText("Icon")

    self.ChatTimerIconBox = Turbine.UI.Control()
    self.ChatTimerIconBox:SetParent(self.ChatTimerBackground)
    self.ChatTimerIconBox:SetPosition(290, 30)
    self.ChatTimerIconBox:SetSize(32,32)
    self.ChatTimerIconBox:SetBlendMode(Turbine.UI.BlendMode.AlphaBlend)
    self.ChatTimerIconBox:SetBackground(BackgroundHandler:GetBackground(0x41007F41))
    self.ChatTimerIconBox:SetVisible(true)

    self.ChatTimerIconTextBox = Turbine.UI.Lotro.TextBox()
    self.ChatTimerIconTextBox:SetParent(self.ChatTimerBackground)
    self.ChatTimerIconTextBox:SetSize(110, 30)
    self.ChatTimerIconTextBox:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
    self.ChatTimerIconTextBox:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
    self.ChatTimerIconTextBox:SetMouseVisible(false)
    self.ChatTimerIconTextBox:SetPosition(330, 30)
    self.ChatTimerIconTextBox:SetMultiline(false)
    self.ChatTimerIconTextBox:SetText(0x41007F41)

    self.ChatTimerIconTextBox.TextChanged = function()
        --pcall(self.ChatTimerIconBox:SetBackground(tonumber(self.ChatTimerIconTextBox:GetText())))
    end

    self.ChatTimerIconBox.MouseClick = function()
        if self.IconOverview then 
            self.IconOverview:SetVisible(true) 
            self.iconLink = self.ChatTimerIconBox
            self.iconTextLink = self.ChatTimerIconTextBox
        end
    end

    self.ChatTimerIconBox.MouseDoubleClick = function()
        self.ChatTimerIconBox:SetBackground(BackgroundHandler:GetBackground(0x41007F41))
        self.iconID = 0x41007F41
    end

    self.ChatTimerUnique = Turbine.UI.Lotro.CheckBox()
    self.ChatTimerUnique:SetParent(self.ChatTimerBackground)
    self.ChatTimerUnique:SetSize(200, 30)
    self.ChatTimerUnique:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft)
    self.ChatTimerUnique:SetFont(Turbine.UI.Lotro.Font.BookAntiquaBold19)
    self.ChatTimerUnique:SetFontStyle(Turbine.UI.FontStyle.Outline)
    self.ChatTimerUnique:SetMouseVisible(false)
    self.ChatTimerUnique:SetPosition(10, 50)
    self.ChatTimerUnique:SetVisible(true)
    self.ChatTimerUnique:SetText("Unique")

    self.ChatTimerLoop = Turbine.UI.Lotro.CheckBox()
    self.ChatTimerLoop:SetParent(self.ChatTimerBackground)
    self.ChatTimerLoop:SetSize(200, 30)
    self.ChatTimerLoop:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft)
    self.ChatTimerLoop:SetFont(Turbine.UI.Lotro.Font.BookAntiquaBold19)
    self.ChatTimerLoop:SetFontStyle(Turbine.UI.FontStyle.Outline)
    self.ChatTimerLoop:SetMouseVisible(false)
    self.ChatTimerLoop:SetPosition(80, 50)
    self.ChatTimerLoop:SetVisible(true)
    self.ChatTimerLoop:SetText("Loop")

    self.ChatHeader = Turbine.UI.Label()
    self.ChatHeader:SetParent(self.ChatBackground)
    self.ChatHeader:SetSize(200, 30)
    self.ChatHeader:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft)
    self.ChatHeader:SetFont(Turbine.UI.Lotro.Font.BookAntiquaBold24)
    self.ChatHeader:SetFontStyle(Turbine.UI.FontStyle.Outline)
    self.ChatHeader:SetMouseVisible(false)
    self.ChatHeader:SetPosition(10, 0)
    self.ChatHeader:SetVisible(true)
    self.ChatHeader:SetText("Chat (Standard)")

    self.ChatSaveButton = Turbine.UI.Lotro.Button()
    self.ChatSaveButton:SetParent(self.ChatBackground)
    self.ChatSaveButton:SetSize(100, 15)
    self.ChatSaveButton:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
    self.ChatSaveButton:SetMouseVisible(true)
    self.ChatSaveButton:SetPosition(170, 5)
    self.ChatSaveButton:SetText("Save")

    self.ChatRemoveButton = Turbine.UI.Lotro.Button()
    self.ChatRemoveButton:SetParent(self.ChatBackground)
    self.ChatRemoveButton:SetSize(100, 15)
    self.ChatRemoveButton:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
    self.ChatRemoveButton:SetMouseVisible(true)
    self.ChatRemoveButton:SetPosition(270, 5)
    self.ChatRemoveButton:SetText("Remove")

    self.ChatRemoveButton.Click = function()
        EventWindows:AddChatAlert(tostring(self.SourceTextBox:GetText()), tostring(self.PhraseTextBox:GetText()), nil)
        self:RemoveChatNode(self.chatNode, tostring(self.SourceTextBox:GetText()), tostring(self.PhraseTextBox:GetText()))
        self:ClearChatOptions()
        ChatOutput("Removed Alert")
    end

    self.ChatClearButton = Turbine.UI.Lotro.Button()
    self.ChatClearButton:SetParent(self.ChatBackground)
    self.ChatClearButton:SetSize(100, 15)
    self.ChatClearButton:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
    self.ChatClearButton:SetMouseVisible(true)
    self.ChatClearButton:SetPosition(370, 5)
    self.ChatClearButton:SetText("Clear")

    self.ChatClearButton.Click = function()
        self:ClearChatOptions()
    end

    -- [[
    self.ChatSaveButton.Click = function()
        -- Trigger
        local source = tostring(self.SourceTextBox:GetText())
        local trigger = tostring(self.PhraseTextBox:GetText())
        local instanceName = tostring(self.ChatInstanceButton:GetText())
        local roleName = tostring(self.ChatRoleButton:GetText())

        -- Alert
        local alertYes = self.ChatAlertCheckBox:IsChecked()
        local alertText = tostring(self.ChatAlertTextBox:GetText())
        local alertDuration = tonumber(self.ChatAlertDurationTextBox:GetText()) or 6
        local alertCountdown = self.ChatAlertCountdownCheckBox:IsChecked()
        local alertSound = self.ChatAlertSoundLabel:IsChecked()
        local alertSoundEffect = tostring(self.ChatSoundeffectButton:GetText())
        local alertIconBox = nil
        local alertIconText = nil
        if self.ChatAlertIconLabel:IsChecked() then
            alertIconBox = tonumber(self.ChatAlertIconBox:GetBackground())
            alertIconText = tonumber(self.ChatAlertIconTextBox:GetText())
        end

        -- Timer
        local timerYes = self.ChatTimerCheckBox:IsChecked()
        local timerDuration = tonumber(self.ChatTimerDurationTextBox:GetText()) or 6
        local timerBackgroundIcon = nil
        local timerBackgroundText = nil
        if self.ChatTimerIconLabel:IsChecked() then
            timerBackgroundIcon = tonumber(self.ChatTimerIconBox:GetBackground())
            timerBackgroundText = tonumber(self.ChatTimerIconTextBox:GetText())
        end
        local timerText = tostring(self.ChatTimerTextBox:GetText())
        local timerUnique = self.ChatTimerUnique:IsChecked()
        local timerLoop = self.ChatTimerLoop:IsChecked()

        local newAlert = {}
        --1 duration, 2 message, 3 icon, 4 countdown, 5 soundeffect, 6 instanceID, 7 role, 8 timerduration, 9 timer text, 10 timer icon, 11 unique, 12 loop, 13 windowID
        -- Valid trigger entry
        if source and trigger and instanceName and roleName then
            local alert = {}
            local timer = {}
            local restrict = {}
            if roleName ~= "All" then restrict[2] = Roles:NameToRoleID(roleName) end
            if instanceName ~= "All" then 
                local result = nil
                for key, value in pairs(InstanceController:GetDungeonNameTable()) do
                    if instanceName == value then
                        result = key
                        break
                    end
                end
                restrict[1] = result 
            end


            -- Valid Alert entry
            if alertYes and alertText and alertDuration then
                alert[1] = alertDuration
                alert[2] = alertText
                alert[3] = alertIconText
                alert[4] = alertCountdown
                ChatOutput("Added Alert Succesfully")
                if alertSound then alert[5] = Sounds:GetSoundList()[alertSoundEffect] or alertSoundEffect end
            elseif alertYes then
                ChatOutput("Failed to add alert")
                if not(alertText) then ChatOutput("No Alert Text Specified") end
                if not(alertDuration) then ChatOutput("No Alert Duration Specified") end
            end

            --Valid Timer entry
            if timerYes and timerDuration and timerText then
                timer[1] = timerDuration
                timer[2] = timerText
                timer[3] = timerBackgroundText
                timer[4] = timerUnique
                timer[5] = timerLoop
                ChatOutput("Added Timer Succesfully")
            elseif timerYes then
                ChatOutput("Failed to add timer")
                if not(timerText) then ChatOutput("No Timer Text Specified") end
                if not(timerDuration) then ChatOutput("No Timer Duration Specified") end
            end

            restrict[3] = tieringEffect

            

            local completeInfo = {["Timers"] = {[1] = timer}, ["Alerts"] = {[1] = alert}, ["Restrictions"] = restrict}
            EventWindows:AddChatAlert(source, trigger, completeInfo)
            self:AddChatNode(self.chatNode, source, trigger, completeInfo)
            self:RefreshTreeview()
            ChatOutput("Trigger added")
        else
            ChatOutput("Failed to add alert, check your input")
        end
    end

    self.ChatListBackground = Turbine.UI.TreeNode()
    self.ChatListBackground:SetSize(self.CurrentAlertBackground:GetWidth() - 10, 30)

    self.ChatListBackground = Turbine.UI.TreeNode()
    self.ChatListBackground:SetSize(self.CurrentAlertBackground:GetWidth() - 10, 30)
    
    self.ChatListBoxHeader = Turbine.UI.Control()
    self.ChatListBoxHeader:SetParent(self.ChatListBackground)
    self.ChatListBoxHeader:SetSize(self.CurrentAlertBackground:GetWidth() - 10, 30)

    self.ChatListBoxHeaderLeft = Turbine.UI.Control()
    self.ChatListBoxHeaderLeft:SetParent(self.ChatListBoxHeader)
    self.ChatListBoxHeaderLeft:SetSize(34, 30)
    self.ChatListBoxHeaderLeft:SetBlendMode(Turbine.UI.BlendMode.AlphaBlend)
    self.ChatListBoxHeaderLeft:SetBackground(BackgroundHandler:GetBackground(0x4110A0E0))

    self.ChatListBoxHeaderRight = Turbine.UI.Label()
    self.ChatListBoxHeaderRight:SetParent(self.ChatListBoxHeader)
    self.ChatListBoxHeaderRight:SetPosition(self.CurrentAlertBackground:GetWidth() - 34 - 10, 0)
    self.ChatListBoxHeaderRight:SetSize(34, 30)
    self.ChatListBoxHeaderRight:SetBlendMode(Turbine.UI.BlendMode.AlphaBlend)
    self.ChatListBoxHeaderRight:SetBackground(BackgroundHandler:GetBackground(0x4110A0DB))

    self.ChatListBoxHeaderMiddle = Turbine.UI.Label()
    self.ChatListBoxHeaderMiddle:SetParent(self.ChatListBoxHeader)
    self.ChatListBoxHeaderMiddle:SetPosition(34, 0)
    self.ChatListBoxHeaderMiddle:SetSize(self.CurrentAlertBackground:GetWidth() - 68 - 10, 30)
    self.ChatListBoxHeaderMiddle:SetBlendMode(Turbine.UI.BlendMode.AlphaBlend)
    self.ChatListBoxHeaderMiddle:SetBackground(BackgroundHandler:GetBackground(0x4110A0D9))

    self.ChatListBoxHeaderLabel = Turbine.UI.Label()
    self.ChatListBoxHeaderLabel:SetParent(self.ChatListBackground)
    self.ChatListBoxHeaderLabel:SetSize(self.CurrentAlertBackground:GetWidth() - 10, 30)
    self.ChatListBoxHeaderLabel:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
    self.ChatListBoxHeaderLabel:SetFont(Turbine.UI.Lotro.Font.BookAntiquaBold22)
    self.ChatListBoxHeaderLabel:SetBlendMode(Turbine.UI.BlendMode.AlphaBlend)
    self.ChatListBoxHeaderLabel:SetText("Chat (Standard)")
     
    self.ChatListBoxHeaderLabel.MouseEnter = function()
        self.ChatListBoxHeaderLeft:SetBackground(BackgroundHandler:GetBackground(0x4110A0DF))
        self.ChatListBoxHeaderMiddle:SetBackground(BackgroundHandler:GetBackground(0x4110A0DD))
        self.ChatListBoxHeaderRight:SetBackground(BackgroundHandler:GetBackground(0x4110A0DE))
    end

    self.ChatListBoxHeaderLabel.MouseUp = function()
        self.ChatListBoxHeaderLeft:SetBackground(BackgroundHandler:GetBackground(0x4110A0E0))
        self.ChatListBoxHeaderMiddle:SetBackground(BackgroundHandler:GetBackground(0x4110A0D9))
        self.ChatListBoxHeaderRight:SetBackground(BackgroundHandler:GetBackground(0x4110A0DB))
    end

    self.ChatListBoxHeaderLabel.MouseDown = function()
        self.ChatListBoxHeaderLeft:SetBackground(BackgroundHandler:GetBackground(0x4110A0D8))
        self.ChatListBoxHeaderMiddle:SetBackground(BackgroundHandler:GetBackground(0x4110A0DA))
        self.ChatListBoxHeaderRight:SetBackground(BackgroundHandler:GetBackground(0x4110A0DC))
    end

    self.ChatListBoxHeaderLabel.MouseLeave = function()
        self.ChatListBoxHeaderLeft:SetBackground(BackgroundHandler:GetBackground(0x4110A0E0))
        self.ChatListBoxHeaderMiddle:SetBackground(BackgroundHandler:GetBackground(0x4110A0D9))
        self.ChatListBoxHeaderRight:SetBackground(BackgroundHandler:GetBackground(0x4110A0DB))
    end

    self.rootNodes:Add(self.ChatListBackground)

    self.chatNode = self.ChatListBackground:GetChildNodes()
    
    for key, value in spairs(EventWindows:GetCurrentChatData(), function(t, a, b) return a < b end) do

        local SourceNode = Turbine.UI.TreeNode()
        SourceNode:SetSize(self.CurrentAlertBackground:GetWidth() - 10, 20 + 2)

        SourceNode.SourceLabel = Turbine.UI.Label()
        SourceNode.SourceLabel:SetParent(SourceNode)
        SourceNode.SourceLabel:SetPosition(1,1)
        SourceNode.SourceLabel:SetSize(self.CurrentAlertBackground:GetWidth() - 10- 2, 20)
        SourceNode.SourceLabel:SetBlendMode(Turbine.UI.BlendMode.AlphaBlend)
        SourceNode.SourceLabel:SetFont(Turbine.UI.Lotro.Font.BookAntiquaBold24)
        SourceNode.SourceLabel:SetMouseVisible(true)

        SourceNode.LefLabel = Turbine.UI.Label()
        SourceNode.LefLabel:SetParent(SourceNode)
        SourceNode.LefLabel:SetPosition(0, 0)
        SourceNode.LefLabel:SetSize(30, 20)
        SourceNode.LefLabel:SetBlendMode(Turbine.UI.BlendMode.AlphaBlend)
        SourceNode.LefLabel:SetBackground(BackgroundHandler:GetBackground(0x4111D1FA))
        SourceNode.LefLabel:SetMouseVisible(false)

        SourceNode.RigthLabel = Turbine.UI.Label()
        SourceNode.RigthLabel:SetParent(SourceNode)
        SourceNode.RigthLabel:SetPosition(self.CurrentAlertBackground:GetWidth() - 157, 0)
        SourceNode.RigthLabel:SetSize(147, 20)
        SourceNode.RigthLabel:SetBlendMode(Turbine.UI.BlendMode.AlphaBlend)
        SourceNode.RigthLabel:SetBackground(BackgroundHandler:GetBackground(0x4111D1FA))
        SourceNode.RigthLabel:SetMouseVisible(false)

        SourceNode.MidLabel = Turbine.UI.Label()
        SourceNode.MidLabel:SetParent(SourceNode)
        SourceNode.MidLabel:SetPosition(30, 0)
        SourceNode.MidLabel:SetSize(self.CurrentAlertBackground:GetWidth() - 70, 20)
        SourceNode.MidLabel:SetBlendMode(Turbine.UI.BlendMode.AlphaBlend)
        SourceNode.MidLabel:SetBackground(BackgroundHandler:GetBackground(0x4111D201))
        SourceNode.MidLabel:SetMouseVisible(false)

        SourceNode.textLabel = Turbine.UI.Label()
        SourceNode.textLabel:SetParent(SourceNode)
        SourceNode.textLabel:SetPosition(1,1)
        SourceNode.textLabel:SetSize(self.CurrentAlertBackground:GetWidth() - 10- 2, 20)
        SourceNode.textLabel:SetBlendMode(Turbine.UI.BlendMode.AlphaBlend)
        SourceNode.textLabel:SetFont(Turbine.UI.Lotro.Font.BookAntiquaBold19)
        SourceNode.textLabel:SetMouseVisible(false)
        SourceNode.textLabel:SetText(key)

        SourceNode.SourceLabel.MouseEnter = function()
            SourceNode.LefLabel:SetBackground(BackgroundHandler:GetBackground(0x4111D1FB))
            SourceNode.RigthLabel:SetBackground(BackgroundHandler:GetBackground(0x4111D1FB))
            SourceNode.MidLabel:SetBackground(BackgroundHandler:GetBackground(0x4111D202))
        end

        SourceNode.SourceLabel.MouseLeave = function()
            SourceNode.LefLabel:SetBackground(BackgroundHandler:GetBackground(0x4111D1FA))
            SourceNode.RigthLabel:SetBackground(BackgroundHandler:GetBackground(0x4111D1FA))
            SourceNode.MidLabel:SetBackground(BackgroundHandler:GetBackground(0x4111D201))
        end

        SourceNode.SourceLabel.MouseDown = function()
            SourceNode.LefLabel:SetBackground(BackgroundHandler:GetBackground(0x4111D1FC))
            SourceNode.RigthLabel:SetBackground(BackgroundHandler:GetBackground(0x4111D1FC))
            SourceNode.MidLabel:SetBackground(BackgroundHandler:GetBackground(0x4111D203))
        end

        SourceNode.SourceLabel.MouseUp = function()
            SourceNode.LefLabel:SetBackground(BackgroundHandler:GetBackground(0x4111D1FA))
            SourceNode.RigthLabel:SetBackground(BackgroundHandler:GetBackground(0x4111D1FA))
            SourceNode.MidLabel:SetBackground(BackgroundHandler:GetBackground(0x4111D201))
        end

        SourceNode.name = key

        self.chatNode:Add(SourceNode)

        local KeyNodes = SourceNode:GetChildNodes()
        
        for phrase, alert in spairs(value, function(t, a, b) return a < b end) do
            local AlertNode = Turbine.UI.TreeNode()
            AlertNode:SetSize(self.CurrentAlertBackground:GetWidth() - 10, 20 + 2)

            AlertNode.SourceLabel = Turbine.UI.Label()
            AlertNode.SourceLabel:SetParent(AlertNode)
            AlertNode.SourceLabel:SetPosition(1,1)
            AlertNode.SourceLabel:SetSize(self.CurrentAlertBackground:GetWidth() - 10- 2, 20)
            AlertNode.SourceLabel:SetBlendMode(Turbine.UI.BlendMode.AlphaBlend)
            AlertNode.SourceLabel:SetFont(Turbine.UI.Lotro.Font.BookAntiquaBold16)
            AlertNode.SourceLabel:SetMouseVisible(true)
    
            AlertNode.LefLabel = Turbine.UI.Label()
            AlertNode.LefLabel:SetParent(AlertNode)
            AlertNode.LefLabel:SetPosition(0, 0)
            AlertNode.LefLabel:SetSize(30, 20)
            AlertNode.LefLabel:SetBlendMode(Turbine.UI.BlendMode.AlphaBlend)
            AlertNode.LefLabel:SetBackground(0x4111D1F9)
            AlertNode.LefLabel:SetMouseVisible(false)
    
            AlertNode.RigthLabel = Turbine.UI.Label()
            AlertNode.RigthLabel:SetParent(AlertNode)
            AlertNode.RigthLabel:SetPosition(self.CurrentAlertBackground:GetWidth() - 157, 0)
            AlertNode.RigthLabel:SetSize(147, 20)
            AlertNode.RigthLabel:SetBlendMode(Turbine.UI.BlendMode.AlphaBlend)
            AlertNode.RigthLabel:SetBackground(0x4111D1F9)
            AlertNode.RigthLabel:SetMouseVisible(false)
    
            AlertNode.MidLabel = Turbine.UI.Label()
            AlertNode.MidLabel:SetParent(AlertNode)
            AlertNode.MidLabel:SetPosition(30, 0)
            AlertNode.MidLabel:SetSize(self.CurrentAlertBackground:GetWidth() - 70, 20)
            AlertNode.MidLabel:SetBlendMode(Turbine.UI.BlendMode.AlphaBlend)
            AlertNode.MidLabel:SetBackground(0x4111D200)
            AlertNode.MidLabel:SetMouseVisible(false)
    
            AlertNode.textLabel = Turbine.UI.Label()
            AlertNode.textLabel:SetParent(AlertNode)
            AlertNode.textLabel:SetPosition(1,1)
            AlertNode.textLabel:SetSize(self.CurrentAlertBackground:GetWidth() - 10- 2, 20)
            AlertNode.textLabel:SetBlendMode(Turbine.UI.BlendMode.AlphaBlend)
            AlertNode.textLabel:SetFont(Turbine.UI.Lotro.Font.BookAntiquaBold19)
            AlertNode.textLabel:SetMouseVisible(false)
            AlertNode.textLabel:SetText(phrase)
    
            AlertNode.SourceLabel.MouseDown = function()
                AlertNode.LefLabel:SetBackground(0x4111D1FE)
                AlertNode.RigthLabel:SetBackground(0x4111D1FE)
                AlertNode.MidLabel:SetBackground(0x4111D205)
            end
    
            AlertNode.SourceLabel.MouseUp = function()
                AlertNode.LefLabel:SetBackground(0x4111D1F9)
                AlertNode.RigthLabel:SetBackground(0x4111D1F9)
                AlertNode.MidLabel:SetBackground(0x4111D200)
            end

            AlertNode.name = phrase

            KeyNodes:Add(AlertNode)

            AlertNode.SourceLabel.MouseClick = function()
                self.SelectedItem = AlertNode.AlertMessageLabel
                self:FillChatOptions(key, phrase, alert)
            end
        end
    end
    --]]
end

function AlertOptions:ClearChatOptions()
    self.SourceTextBox:SetText("")
    self.PhraseTextBox:SetText("")
    self.ChatInstanceButton:SetText("All")
    self.ChatRoleButton:SetText("All")

    -- Alert
    self.ChatAlertCheckBox:SetChecked(false)
    self.ChatAlertTextBox:SetText("")
    self.ChatAlertDurationTextBox:SetText("")
    self.ChatAlertCountdownCheckBox:SetChecked(false)
    self.ChatAlertSoundLabel:SetChecked(false)
    self.ChatSoundeffectButton:SetText("")
    self.ChatAlertIconLabel:SetChecked(false)
    self.ChatAlertIconBox:SetBackground(BackgroundHandler:GetBackground(0x41007F41))
    self.ChatAlertIconTextBox:SetText(0x41007F41)

    -- Timer
    self.ChatTimerCheckBox:SetChecked(false)
    self.ChatTimerDurationTextBox:SetText("")
    self.ChatTimerIconLabel:SetChecked(false)
    self.ChatTimerIconBox:SetBackground(BackgroundHandler:GetBackground(0x41007F41))
    self.ChatTimerIconTextBox:SetText(0x41007F41)
    self.ChatTimerTextBox:SetText("")
    self.ChatTimerUnique:SetChecked(false)
    self.ChatTimerLoop:SetChecked(false)
end

function AlertOptions:FillChatOptions(source, phrase, input)
    self:ToggleWindow("Chat (Standard)")
     --1 duration, 2 message, 3 icon, 4 countdown, 5 soundeffect, 6 instanceID, 7 role, 8 timerduration, 9 timer text, 10 timer icon, 11 unique, 12 loop, 13 windowID
    -- Trigger

    local timer = input["Timers"][1]
    local alert = input["Alerts"][1]
    local restrict = input["Restrictions"]

    self.SourceTextBox:SetText(tostring(source))
    self.PhraseTextBox:SetText(tostring(phrase))
    if restrict[1] then self.ChatInstanceButton:SetText(tostring(InstanceController:GetDungeonNameTable()[tonumber(restrict[1])])) else self.ChatInstanceButton:SetText("All") end
    if restrict[2] then self.ChatRoleButton:SetText(Roles:GetAvailableRoles()[restrict[2]]) else self.ChatRoleButton:SetText("All") end
    

    -- Alert
    self.ChatAlertCheckBox:SetChecked(alert[1])
    if alert[2] then self.ChatAlertTextBox:SetText(tostring(alert[2])) else self.ChatAlertTextBox:SetText("") end
    self.ChatAlertDurationTextBox:SetText(tonumber(alert[1]))
    self.ChatAlertCountdownCheckBox:SetChecked(alert[4])
    self.ChatAlertSoundLabel:SetChecked(alert[5])
    self.ChatSoundeffectButton:SetText(alert[5])
    self.ChatAlertIconLabel:SetChecked(alert[3])
    if alert[3] then
        self.ChatAlertIconBox:SetBackground(BackgroundHandler:GetBackground(tonumber(alert[3])))
        self.ChatAlertIconTextBox:SetText(tonumber(alert[3]))
    else
        self.ChatAlertIconBox:SetBackground("")
        self.ChatAlertIconTextBox:SetText("")
    end
    if alert[3] then self.ChatAlertIconTextBox:SetText(tostring(alert[3])) else self.ChatAlertIconTextBox:SetText("") end

    -- Timer
    self.ChatTimerCheckBox:SetChecked(timer[1])
    self.ChatTimerDurationTextBox:SetText(tonumber(timer[1]))
    self.ChatTimerIconLabel:SetChecked(timer[3])
    if timer[3] then
        self.ChatTimerIconBox:SetBackground(BackgroundHandler:GetBackground(tonumber(timer[3])))
        self.ChatTimerIconTextBox:SetText(tonumber(timer[3]))
    else
        self.ChatTimerIconBox:SetBackground("")
        self.ChatTimerIconTextBox:SetText("")
    end
    if timer[2] then self.ChatTimerTextBox:SetText(tostring(timer[2])) else self.ChatTimerTextBox:SetText("") end
    self.ChatTimerUnique:SetChecked(timer[4])
    self.ChatTimerLoop:SetChecked(timer[5])
end

-- Creation/Changing chat player combat alert UI
function AlertOptions:ChatPlayerCombatAlerts(parent)

    self.ChatPlayerCombatBackground = Turbine.UI.Control()
    self.ChatPlayerCombatBackground:SetParent(parent)
    self.ChatPlayerCombatBackground:SetVisible(false)
    self.ChatPlayerCombatBackground:SetPosition(0, 0)
    self.ChatPlayerCombatBackground:SetSize(parent:GetWidth(), parent:GetHeight())

    self.SelectionWindows["Chat (Combat Player)"] = self.ChatPlayerCombatBackground

    -- Key phrase
    self.ChatPlayerCombatPhraseLabel = Turbine.UI.Label()
    self.ChatPlayerCombatPhraseLabel:SetParent(self.ChatPlayerCombatBackground)
    self.ChatPlayerCombatPhraseLabel:SetSize(200, 30)
    self.ChatPlayerCombatPhraseLabel:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft)
    self.ChatPlayerCombatPhraseLabel:SetFont(Turbine.UI.Lotro.Font.BookAntiquaBold19)
    self.ChatPlayerCombatPhraseLabel:SetFontStyle(Turbine.UI.FontStyle.Outline)
    self.ChatPlayerCombatPhraseLabel:SetMouseVisible(false)
    self.ChatPlayerCombatPhraseLabel:SetPosition(10, 70)
    self.ChatPlayerCombatPhraseLabel:SetVisible(true)
    self.ChatPlayerCombatPhraseLabel:SetText("Hit trigger")

    self.ChatPlayerCombatPhraseTextBox = Turbine.UI.Lotro.TextBox()
    self.ChatPlayerCombatPhraseTextBox:SetParent(self.ChatPlayerCombatBackground)
    self.ChatPlayerCombatPhraseTextBox:SetSize(300, 30)
    self.ChatPlayerCombatPhraseTextBox:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
    self.ChatPlayerCombatPhraseTextBox:SetMouseVisible(false)
    self.ChatPlayerCombatPhraseTextBox:SetPosition(140, 70)
    self.ChatPlayerCombatPhraseTextBox:SetMultiline(true)

    self.ChatPlayerCombatInstanceLabel = Turbine.UI.Label()
    self.ChatPlayerCombatInstanceLabel:SetParent(self.ChatPlayerCombatBackground)
    self.ChatPlayerCombatInstanceLabel:SetSize(300, 30)
    self.ChatPlayerCombatInstanceLabel:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft)
    self.ChatPlayerCombatInstanceLabel:SetFont(Turbine.UI.Lotro.Font.BookAntiquaBold19)
    self.ChatPlayerCombatInstanceLabel:SetFontStyle(Turbine.UI.FontStyle.Outline)
    self.ChatPlayerCombatInstanceLabel:SetMouseVisible(false)
    self.ChatPlayerCombatInstanceLabel:SetPosition(10, 125)
    self.ChatPlayerCombatInstanceLabel:SetVisible(true)
    self.ChatPlayerCombatInstanceLabel:SetText("Only load for specific instance/role")

    self.ChatPlayerCombatInstanceButton = Turbine.UI.Lotro.Button()
    self.ChatPlayerCombatInstanceButton:SetParent(self.ChatPlayerCombatBackground)
    self.ChatPlayerCombatInstanceButton:SetSize(100, 15)
    self.ChatPlayerCombatInstanceButton:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
    self.ChatPlayerCombatInstanceButton:SetMouseVisible(true)
    self.ChatPlayerCombatInstanceButton:SetPosition(240, 133)
    self.ChatPlayerCombatInstanceButton:SetText("All")

    self.ChatPlayerCombatInstanceMenu = UI.DropDownMenuControl(self, 200, "Instance Dropdown Menu")
    for key, value in spairs(InstanceController:GetDungeonNameTable(), function(t, a, b) return t[a] < t[b] end) do
        self.ChatPlayerCombatInstanceMenu:AddItem(value, function()
            self.ChatPlayerCombatInstanceButton:SetText(value)
            self.ChatPlayerCombatInstanceMenu:SetVisible(false)
        end)
    end

    self.ChatPlayerCombatInstanceButton.Click = function()
        self.ChatPlayerCombatInstanceMenu:ToggleVisible()
    end

    self.ChatPlayerCombatRoleButton = Turbine.UI.Lotro.Button()
    self.ChatPlayerCombatRoleButton:SetParent(self.ChatPlayerCombatBackground)
    self.ChatPlayerCombatRoleButton:SetSize(100, 15)
    self.ChatPlayerCombatRoleButton:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
    self.ChatPlayerCombatRoleButton:SetMouseVisible(true)
    self.ChatPlayerCombatRoleButton:SetPosition(340, 133)
    self.ChatPlayerCombatRoleButton:SetText("All")

    self.ChatPlayerCombatRoleMenu = Turbine.UI.ContextMenu()
    self.ChatPlayerCombatRoleMenuItems = self.ChatPlayerCombatRoleMenu:GetItems()
    for key, value in spairs(Roles:GetAvailableRoles(), function(t, a, b) return t[a] < t[b] end) do
        local menuItem = Turbine.UI.MenuItem( value )
        self.ChatPlayerCombatRoleMenuItems:Add(menuItem)
        menuItem.Click = function()
            self.ChatPlayerCombatRoleButton:SetText(value)
        end
    end

    local menuItem = Turbine.UI.MenuItem( "All" )
    self.ChatPlayerCombatRoleMenuItems:Add(menuItem)
    menuItem.Click = function()
        self.ChatPlayerCombatRoleButton:SetText("All")
    end

    self.ChatPlayerCombatRoleButton.Click = function()
        self.ChatPlayerCombatRoleMenu:ShowMenu()
    end


    -- Alert
    self.ChatPlayerCombatAlertBackground = Turbine.UI.Control()
    self.ChatPlayerCombatAlertBackground:SetParent(self.ChatPlayerCombatBackground)
    self.ChatPlayerCombatAlertBackground:SetPosition(10, 160)
    self.ChatPlayerCombatAlertBackground:SetSize(parent:GetWidth() - 20, parent:GetHeight() - 260)
    self.ChatPlayerCombatAlertBackground:SetBackColor(Turbine.UI.Color(0.5, 0.5, 0.5))

    self.ChatPlayerCombatAlertCheckBox = Turbine.UI.Lotro.CheckBox()
    self.ChatPlayerCombatAlertCheckBox:SetParent(self.ChatPlayerCombatAlertBackground)
    self.ChatPlayerCombatAlertCheckBox:SetSize(200, 30)
    self.ChatPlayerCombatAlertCheckBox:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft)
    self.ChatPlayerCombatAlertCheckBox:SetFont(Turbine.UI.Lotro.Font.BookAntiquaBold24)
    self.ChatPlayerCombatAlertCheckBox:SetFontStyle(Turbine.UI.FontStyle.Outline)
    self.ChatPlayerCombatAlertCheckBox:SetMouseVisible(false)
    self.ChatPlayerCombatAlertCheckBox:SetPosition(3, 3)
    self.ChatPlayerCombatAlertCheckBox:SetVisible(true)
    self.ChatPlayerCombatAlertCheckBox:SetText("Alert on Trigger")

    self.ChatPlayerCombatPhraseLabel = Turbine.UI.Label()
    self.ChatPlayerCombatPhraseLabel:SetParent(self.ChatPlayerCombatAlertBackground)
    self.ChatPlayerCombatPhraseLabel:SetSize(100, 30)
    self.ChatPlayerCombatPhraseLabel:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft)
    self.ChatPlayerCombatPhraseLabel:SetFont(Turbine.UI.Lotro.Font.BookAntiquaBold19)
    self.ChatPlayerCombatPhraseLabel:SetFontStyle(Turbine.UI.FontStyle.Outline)
    self.ChatPlayerCombatPhraseLabel:SetMouseVisible(false)
    self.ChatPlayerCombatPhraseLabel:SetPosition(10, 35)
    self.ChatPlayerCombatPhraseLabel:SetVisible(true)
    self.ChatPlayerCombatPhraseLabel:SetText("Alert Text")

    self.ChatPlayerCombatAlertTextBox = Turbine.UI.Lotro.TextBox()
    self.ChatPlayerCombatAlertTextBox:SetParent(self.ChatPlayerCombatAlertBackground)
    self.ChatPlayerCombatAlertTextBox:SetSize(200, 30)
    self.ChatPlayerCombatAlertTextBox:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
    self.ChatPlayerCombatAlertTextBox:SetMouseVisible(false)
    self.ChatPlayerCombatAlertTextBox:SetMultiline(false)
    self.ChatPlayerCombatAlertTextBox:SetPosition(80, 35)

    self.ChatPlayerCombatAlertDurationLabel = Turbine.UI.Label()
    self.ChatPlayerCombatAlertDurationLabel:SetParent(self.ChatPlayerCombatAlertBackground)
    self.ChatPlayerCombatAlertDurationLabel:SetSize(200, 30)
    self.ChatPlayerCombatAlertDurationLabel:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft)
    self.ChatPlayerCombatAlertDurationLabel:SetFont(Turbine.UI.Lotro.Font.BookAntiquaBold19)
    self.ChatPlayerCombatAlertDurationLabel:SetFontStyle(Turbine.UI.FontStyle.Outline)
    self.ChatPlayerCombatAlertDurationLabel:SetMouseVisible(false)
    self.ChatPlayerCombatAlertDurationLabel:SetPosition(10, 65)
    self.ChatPlayerCombatAlertDurationLabel:SetVisible(true)
    self.ChatPlayerCombatAlertDurationLabel:SetText("Alert duration in seconds")

    self.ChatPlayerCombatAlertDurationTextBox = Turbine.UI.Lotro.TextBox()
    self.ChatPlayerCombatAlertDurationTextBox:SetParent(self.ChatPlayerCombatAlertBackground)
    self.ChatPlayerCombatAlertDurationTextBox:SetSize(50, 30)
    self.ChatPlayerCombatAlertDurationTextBox:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
    self.ChatPlayerCombatAlertDurationTextBox:SetMouseVisible(false)
    self.ChatPlayerCombatAlertDurationTextBox:SetPosition(185, 65)
    self.ChatPlayerCombatAlertDurationTextBox:SetMultiline(false)


    -- Countdown
    self.ChatPlayerCombatAlertCountdownCheckBox = Turbine.UI.Lotro.CheckBox()
    self.ChatPlayerCombatAlertCountdownCheckBox:SetParent(self.ChatPlayerCombatAlertBackground)
    self.ChatPlayerCombatAlertCountdownCheckBox:SetSize(200, 30)
    self.ChatPlayerCombatAlertCountdownCheckBox:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft)
    self.ChatPlayerCombatAlertCountdownCheckBox:SetFont(Turbine.UI.Lotro.Font.BookAntiquaBold19)
    self.ChatPlayerCombatAlertCountdownCheckBox:SetFontStyle(Turbine.UI.FontStyle.Outline)
    self.ChatPlayerCombatAlertCountdownCheckBox:SetMouseVisible(false)
    self.ChatPlayerCombatAlertCountdownCheckBox:SetPosition(250, 65)
    self.ChatPlayerCombatAlertCountdownCheckBox:SetVisible(true)
    self.ChatPlayerCombatAlertCountdownCheckBox:SetText("Alert duration shown")

    -- Sound effects
    self.ChatPlayerCombatAlertSoundLabel = Turbine.UI.Lotro.CheckBox()
    self.ChatPlayerCombatAlertSoundLabel:SetParent(self.ChatPlayerCombatAlertBackground)
    self.ChatPlayerCombatAlertSoundLabel:SetSize(200, 30)
    self.ChatPlayerCombatAlertSoundLabel:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft)
    self.ChatPlayerCombatAlertSoundLabel:SetFont(Turbine.UI.Lotro.Font.BookAntiquaBold19)
    self.ChatPlayerCombatAlertSoundLabel:SetFontStyle(Turbine.UI.FontStyle.Outline)
    self.ChatPlayerCombatAlertSoundLabel:SetMouseVisible(false)
    self.ChatPlayerCombatAlertSoundLabel:SetPosition(10, 90)
    self.ChatPlayerCombatAlertSoundLabel:SetVisible(true)
    self.ChatPlayerCombatAlertSoundLabel:SetText("Sound Effect")

    self.ChatPlayerCombatSoundeffectButton = Turbine.UI.Lotro.Button()
    self.ChatPlayerCombatSoundeffectButton:SetParent(self.ChatPlayerCombatAlertBackground)
    self.ChatPlayerCombatSoundeffectButton:SetSize(100, 15)
    self.ChatPlayerCombatSoundeffectButton:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
    self.ChatPlayerCombatSoundeffectButton:SetMouseVisible(true)
    self.ChatPlayerCombatSoundeffectButton:SetPosition(120, 97)
    self.ChatPlayerCombatSoundeffectButton:SetText("Horn")

    self.ChatPlayerCombatSoundEffectMenu = UI.DropDownMenuControl(self, 200, "Sound Menu")
    for key, value in spairs(Sounds:GetSoundList(), function(t, a, b) return a < b end) do
        self.ChatPlayerCombatSoundEffectMenu:AddItem(key, 
        function()
            self.ChatPlayerCombatSoundeffectButton:SetText(key)
            self.ChatPlayerCombatSoundEffectMenu:ToggleVisible()
        end)
    end

    self.ChatPlayerCombatSoundeffectButton.Click = function()
        self.ChatPlayerCombatSoundEffectMenu:ToggleVisible()
    end

    self.ChatPlayerCombatAlertIconLabel = Turbine.UI.Lotro.CheckBox()
    self.ChatPlayerCombatAlertIconLabel:SetParent(self.ChatPlayerCombatAlertBackground)
    self.ChatPlayerCombatAlertIconLabel:SetSize(50, 30)
    self.ChatPlayerCombatAlertIconLabel:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft)
    self.ChatPlayerCombatAlertIconLabel:SetFont(Turbine.UI.Lotro.Font.BookAntiquaBold19)
    self.ChatPlayerCombatAlertIconLabel:SetFontStyle(Turbine.UI.FontStyle.Outline)
    self.ChatPlayerCombatAlertIconLabel:SetMouseVisible(false)
    self.ChatPlayerCombatAlertIconLabel:SetPosition(240, 97)
    self.ChatPlayerCombatAlertIconLabel:SetVisible(true)
    self.ChatPlayerCombatAlertIconLabel:SetText("Icon")

    self.ChatPlayerCombatAlertIconBox = Turbine.UI.Control()
    self.ChatPlayerCombatAlertIconBox:SetParent(self.ChatPlayerCombatAlertBackground)
    self.ChatPlayerCombatAlertIconBox:SetPosition(290, 97)
    self.ChatPlayerCombatAlertIconBox:SetSize(32,32)
    self.ChatPlayerCombatAlertIconBox:SetBlendMode(Turbine.UI.BlendMode.AlphaBlend)
    self.ChatPlayerCombatAlertIconBox:SetBackground(BackgroundHandler:GetBackground(0x41007F41))
    self.ChatPlayerCombatAlertIconBox:SetVisible(true)

    self.ChatPlayerCombatAlertIconTextBox = Turbine.UI.Lotro.TextBox()
    self.ChatPlayerCombatAlertIconTextBox:SetParent(self.ChatPlayerCombatAlertBackground)
    self.ChatPlayerCombatAlertIconTextBox:SetSize(110, 30)
    self.ChatPlayerCombatAlertIconTextBox:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
    self.ChatPlayerCombatAlertIconTextBox:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
    self.ChatPlayerCombatAlertIconTextBox:SetMouseVisible(false)
    self.ChatPlayerCombatAlertIconTextBox:SetPosition(330, 97)
    self.ChatPlayerCombatAlertIconTextBox:SetMultiline(false)
    self.ChatPlayerCombatAlertIconTextBox:SetText(0x41007F41)

    self.ChatPlayerCombatAlertIconTextBox.TextChanged = function()
        --pcall(self.ChatPlayerCombatAlertIconBox:SetBackground(tonumber(self.ChatPlayerCombatAlertIconTextBox:GetText())))
    end

    self.ChatPlayerCombatAlertIconBox.MouseClick = function()
        if self.IconOverview then 
            self.IconOverview:SetVisible(true) 
            self.iconLink = self.ChatPlayerCombatAlertIconBox
            self.iconTextLink = self.ChatPlayerCombatAlertIconTextBox
        end
    end

    self.ChatPlayerCombatAlertIconBox.MouseDoubleClick = function()
        self.ChatPlayerCombatAlertIconBox:SetBackground(BackgroundHandler:GetBackground(0x41007F41))
        self.iconID = 0x41007F41
    end


    --Timer
    self.ChatPlayerCombatTimerBackground = Turbine.UI.Control()
    self.ChatPlayerCombatTimerBackground:SetParent(self.ChatPlayerCombatBackground)
    self.ChatPlayerCombatTimerBackground:SetPosition(10, 300)
    self.ChatPlayerCombatTimerBackground:SetSize(parent:GetWidth() - 20, 80)
    self.ChatPlayerCombatTimerBackground:SetBackColor(Turbine.UI.Color(0.5, 0.5, 0.5))

    self.ChatPlayerCombatTimerCheckBox = Turbine.UI.Lotro.CheckBox()
    self.ChatPlayerCombatTimerCheckBox:SetParent(self.ChatPlayerCombatTimerBackground)
    self.ChatPlayerCombatTimerCheckBox:SetSize(200, 30)
    self.ChatPlayerCombatTimerCheckBox:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft)
    self.ChatPlayerCombatTimerCheckBox:SetFont(Turbine.UI.Lotro.Font.BookAntiquaBold24)
    self.ChatPlayerCombatTimerCheckBox:SetFontStyle(Turbine.UI.FontStyle.Outline)
    self.ChatPlayerCombatTimerCheckBox:SetMouseVisible(false)
    self.ChatPlayerCombatTimerCheckBox:SetPosition(3, 3)
    self.ChatPlayerCombatTimerCheckBox:SetVisible(true)
    self.ChatPlayerCombatTimerCheckBox:SetText("Timer on Trigger")

    self.ChatPlayerCombatTimerText = Turbine.UI.Label()
    self.ChatPlayerCombatTimerText:SetParent(self.ChatPlayerCombatTimerBackground)
    self.ChatPlayerCombatTimerText:SetSize(100, 30)
    self.ChatPlayerCombatTimerText:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft)
    self.ChatPlayerCombatTimerText:SetFont(Turbine.UI.Lotro.Font.BookAntiquaBold19)
    self.ChatPlayerCombatTimerText:SetFontStyle(Turbine.UI.FontStyle.Outline)
    self.ChatPlayerCombatTimerText:SetMouseVisible(false)
    self.ChatPlayerCombatTimerText:SetPosition(195, 0)
    self.ChatPlayerCombatTimerText:SetVisible(true)
    self.ChatPlayerCombatTimerText:SetText("Timer Text")

    self.ChatPlayerCombatTimerTextBox = Turbine.UI.Lotro.TextBox()
    self.ChatPlayerCombatTimerTextBox:SetParent(self.ChatPlayerCombatTimerBackground)
    self.ChatPlayerCombatTimerTextBox:SetSize(170, 25)
    self.ChatPlayerCombatTimerTextBox:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
    self.ChatPlayerCombatTimerTextBox:SetMouseVisible(false)
    self.ChatPlayerCombatTimerTextBox:SetMultiline(false)
    self.ChatPlayerCombatTimerTextBox:SetPosition(270, 3)

    self.ChatPlayerCombatTimerDurationLabel = Turbine.UI.Label()
    self.ChatPlayerCombatTimerDurationLabel:SetParent(self.ChatPlayerCombatTimerBackground)
    self.ChatPlayerCombatTimerDurationLabel:SetSize(200, 30)
    self.ChatPlayerCombatTimerDurationLabel:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft)
    self.ChatPlayerCombatTimerDurationLabel:SetFont(Turbine.UI.Lotro.Font.BookAntiquaBold19)
    self.ChatPlayerCombatTimerDurationLabel:SetFontStyle(Turbine.UI.FontStyle.Outline)
    self.ChatPlayerCombatTimerDurationLabel:SetMouseVisible(false)
    self.ChatPlayerCombatTimerDurationLabel:SetPosition(10, 30)
    self.ChatPlayerCombatTimerDurationLabel:SetVisible(true)
    self.ChatPlayerCombatTimerDurationLabel:SetText("Timer duration in seconds")

    self.ChatPlayerCombatTimerDurationTextBox = Turbine.UI.Lotro.TextBox()
    self.ChatPlayerCombatTimerDurationTextBox:SetParent(self.ChatPlayerCombatTimerBackground)
    self.ChatPlayerCombatTimerDurationTextBox:SetSize(50, 30)
    self.ChatPlayerCombatTimerDurationTextBox:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
    self.ChatPlayerCombatTimerDurationTextBox:SetMouseVisible(false)
    self.ChatPlayerCombatTimerDurationTextBox:SetPosition(185, 30)
    self.ChatPlayerCombatTimerDurationTextBox:SetMultiline(false)

    self.ChatPlayerCombatTimerIconLabel = Turbine.UI.Lotro.CheckBox()
    self.ChatPlayerCombatTimerIconLabel:SetParent(self.ChatPlayerCombatTimerBackground)
    self.ChatPlayerCombatTimerIconLabel:SetSize(50, 30)
    self.ChatPlayerCombatTimerIconLabel:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft)
    self.ChatPlayerCombatTimerIconLabel:SetFont(Turbine.UI.Lotro.Font.BookAntiquaBold19)
    self.ChatPlayerCombatTimerIconLabel:SetFontStyle(Turbine.UI.FontStyle.Outline)
    self.ChatPlayerCombatTimerIconLabel:SetMouseVisible(false)
    self.ChatPlayerCombatTimerIconLabel:SetPosition(240, 30)
    self.ChatPlayerCombatTimerIconLabel:SetVisible(true)
    self.ChatPlayerCombatTimerIconLabel:SetText("Icon")

    self.ChatPlayerCombatTimerIconBox = Turbine.UI.Control()
    self.ChatPlayerCombatTimerIconBox:SetParent(self.ChatPlayerCombatTimerBackground)
    self.ChatPlayerCombatTimerIconBox:SetPosition(290, 30)
    self.ChatPlayerCombatTimerIconBox:SetSize(32,32)
    self.ChatPlayerCombatTimerIconBox:SetBlendMode(Turbine.UI.BlendMode.AlphaBlend)
    self.ChatPlayerCombatTimerIconBox:SetBackground(BackgroundHandler:GetBackground(0x41007F41))
    self.ChatPlayerCombatTimerIconBox:SetVisible(true)

    self.ChatPlayerCombatTimerIconTextBox = Turbine.UI.Lotro.TextBox()
    self.ChatPlayerCombatTimerIconTextBox:SetParent(self.ChatPlayerCombatTimerBackground)
    self.ChatPlayerCombatTimerIconTextBox:SetSize(110, 30)
    self.ChatPlayerCombatTimerIconTextBox:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
    self.ChatPlayerCombatTimerIconTextBox:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
    self.ChatPlayerCombatTimerIconTextBox:SetMouseVisible(false)
    self.ChatPlayerCombatTimerIconTextBox:SetPosition(330, 30)
    self.ChatPlayerCombatTimerIconTextBox:SetMultiline(false)
    self.ChatPlayerCombatTimerIconTextBox:SetText(0x41007F41)

    self.ChatPlayerCombatTimerIconTextBox.TextChanged = function()
        --pcall(self.ChatPlayerCombatTimerIconBox:SetBackground(tonumber(self.ChatPlayerCombatTimerIconTextBox:GetText())))
    end

    self.ChatPlayerCombatTimerIconBox.MouseClick = function()
        if self.IconOverview then 
            self.IconOverview:SetVisible(true) 
            self.iconLink = self.ChatPlayerCombatTimerIconBox
            self.iconTextLink = self.ChatPlayerCombatTimerIconTextBox
        end
    end

    self.ChatPlayerCombatTimerIconBox.MouseDoubleClick = function()
        self.ChatPlayerCombatTimerIconBox:SetBackground(BackgroundHandler:GetBackground(0x41007F41))
        self.iconID = 0x41007F41
    end

    self.ChatPlayerCombatTimerUnique = Turbine.UI.Lotro.CheckBox()
    self.ChatPlayerCombatTimerUnique:SetParent(self.ChatPlayerCombatTimerBackground)
    self.ChatPlayerCombatTimerUnique:SetSize(200, 30)
    self.ChatPlayerCombatTimerUnique:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft)
    self.ChatPlayerCombatTimerUnique:SetFont(Turbine.UI.Lotro.Font.BookAntiquaBold19)
    self.ChatPlayerCombatTimerUnique:SetFontStyle(Turbine.UI.FontStyle.Outline)
    self.ChatPlayerCombatTimerUnique:SetMouseVisible(false)
    self.ChatPlayerCombatTimerUnique:SetPosition(10, 50)
    self.ChatPlayerCombatTimerUnique:SetVisible(true)
    self.ChatPlayerCombatTimerUnique:SetText("Unique")

    self.ChatPlayerCombatTimerLoop = Turbine.UI.Lotro.CheckBox()
    self.ChatPlayerCombatTimerLoop:SetParent(self.ChatPlayerCombatTimerBackground)
    self.ChatPlayerCombatTimerLoop:SetSize(200, 30)
    self.ChatPlayerCombatTimerLoop:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft)
    self.ChatPlayerCombatTimerLoop:SetFont(Turbine.UI.Lotro.Font.BookAntiquaBold19)
    self.ChatPlayerCombatTimerLoop:SetFontStyle(Turbine.UI.FontStyle.Outline)
    self.ChatPlayerCombatTimerLoop:SetMouseVisible(false)
    self.ChatPlayerCombatTimerLoop:SetPosition(80, 50)
    self.ChatPlayerCombatTimerLoop:SetVisible(true)
    self.ChatPlayerCombatTimerLoop:SetText("Loop")

    self.ChatPlayerCombatHeader = Turbine.UI.Label()
    self.ChatPlayerCombatHeader:SetParent(self.ChatPlayerCombatBackground)
    self.ChatPlayerCombatHeader:SetSize(200, 30)
    self.ChatPlayerCombatHeader:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft)
    self.ChatPlayerCombatHeader:SetFont(Turbine.UI.Lotro.Font.BookAntiquaBold24)
    self.ChatPlayerCombatHeader:SetFontStyle(Turbine.UI.FontStyle.Outline)
    self.ChatPlayerCombatHeader:SetMouseVisible(false)
    self.ChatPlayerCombatHeader:SetPosition(10, 0)
    self.ChatPlayerCombatHeader:SetVisible(true)
    self.ChatPlayerCombatHeader:SetText("Chat (Combat Player)")

    self.ChatPlayerCombatSaveButton = Turbine.UI.Lotro.Button()
    self.ChatPlayerCombatSaveButton:SetParent(self.ChatPlayerCombatBackground)
    self.ChatPlayerCombatSaveButton:SetSize(60, 15)
    self.ChatPlayerCombatSaveButton:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
    self.ChatPlayerCombatSaveButton:SetMouseVisible(true)
    self.ChatPlayerCombatSaveButton:SetPosition(210, 5)
    self.ChatPlayerCombatSaveButton:SetText("Save")

    self.ChatPlayerCombatRemoveButton = Turbine.UI.Lotro.Button()
    self.ChatPlayerCombatRemoveButton:SetParent(self.ChatPlayerCombatBackground)
    self.ChatPlayerCombatRemoveButton:SetSize(100, 15)
    self.ChatPlayerCombatRemoveButton:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
    self.ChatPlayerCombatRemoveButton:SetMouseVisible(true)
    self.ChatPlayerCombatRemoveButton:SetPosition(270, 5)
    self.ChatPlayerCombatRemoveButton:SetText("Remove")

    self.ChatPlayerCombatRemoveButton.Click = function()
        EventWindows:AddChatPlayerCombatAlert(tostring(self.ChatPlayerCombatPhraseTextBox:GetText()), nil)
        self:RemoveNode(self.ChatPlayerCombatNode, tostring(self.ChatPlayerCombatPhraseTextBox:GetText()))
        self:ClearPlayerCombat()
        ChatOutput("Removed Alert")
    end

    self.ChatPlayerCombatClearButton = Turbine.UI.Lotro.Button()
    self.ChatPlayerCombatClearButton:SetParent(self.ChatPlayerCombatBackground)
    self.ChatPlayerCombatClearButton:SetSize(100, 15)
    self.ChatPlayerCombatClearButton:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
    self.ChatPlayerCombatClearButton:SetMouseVisible(true)
    self.ChatPlayerCombatClearButton:SetPosition(370, 5)
    self.ChatPlayerCombatClearButton:SetText("Clear")

    self.ChatPlayerCombatClearButton.Click = function()
        self:ClearPlayerCombat()
    end

    -- [[
    self.ChatPlayerCombatSaveButton.Click = function()
        -- Trigger
        local trigger = tostring(self.ChatPlayerCombatPhraseTextBox:GetText())
        local instanceName = tostring(self.ChatPlayerCombatInstanceButton:GetText())
        local roleName = tostring(self.ChatPlayerCombatRoleButton:GetText())

        -- Alert
        local alertYes = self.ChatPlayerCombatAlertCheckBox:IsChecked()
        local alertText = tostring(self.ChatPlayerCombatAlertTextBox:GetText())
        local alertDuration = tonumber(self.ChatPlayerCombatAlertDurationTextBox:GetText()) or 6
        local alertCountdown = self.ChatPlayerCombatAlertCountdownCheckBox:IsChecked()
        local alertSound = self.ChatPlayerCombatAlertSoundLabel:IsChecked()
        local alertSoundEffect = tostring(self.ChatPlayerCombatSoundeffectButton:GetText())
        local alertIconBox = nil
        local alertIconText = nil
        if self.ChatPlayerCombatAlertIconLabel:IsChecked() then
            alertIconBox = tonumber(self.ChatPlayerCombatAlertIconBox:GetBackground())
            alertIconText = tonumber(self.ChatPlayerCombatAlertIconTextBox:GetText())
        end

        -- Timer
        local timerYes = self.ChatPlayerCombatTimerCheckBox:IsChecked()
        local timerDuration = tonumber(self.ChatPlayerCombatTimerDurationTextBox:GetText()) or 6
        local timerBackgroundIcon = nil
        local timerBackgroundText = nil
        if self.ChatPlayerCombatTimerIconLabel:IsChecked() then
            timerBackgroundIcon = tonumber(self.ChatPlayerCombatTimerIconBox:GetBackground())
            timerBackgroundText = tonumber(self.ChatPlayerCombatTimerIconTextBox:GetText())
        end
        local timerText = tostring(self.ChatPlayerCombatTimerTextBox:GetText())
        local timerUnique = self.ChatPlayerCombatTimerUnique:IsChecked()
        local timerLoop = self.ChatPlayerCombatTimerLoop:IsChecked()

        local newAlert = {}
        --1 duration, 2 message, 3 icon, 4 countdown, 5 soundeffect, 6 instanceID, 7 role, 8 timerduration, 9 timer text, 10 timer icon, 11 unique, 12 loop, 13 windowID
        -- Valid trigger entry
        if trigger and instanceName and roleName then
            local alert = {}
            local timer = {}
            local restrict = {}
            if roleName ~= "All" then restrict[2] = Roles:NameToRoleID(roleName) end
            if instanceName ~= "All" then 
                local result = nil
                for key, value in pairs(InstanceController:GetDungeonNameTable()) do
                    if instanceName == value then
                        result = key
                        break
                    end
                end
                restrict[1] = result 
            end


            -- Valid Alert entry
            if alertYes and alertText and alertDuration then
                alert[1] = alertDuration
                alert[2] = alertText
                alert[3] = alertIconText
                alert[4] = alertCountdown
                ChatOutput("Added Alert Succesfully")
                if alertSound then alert[5] = Sounds:GetSoundList()[alertSoundEffect] or alertSoundEffect end
            elseif alertYes then
                ChatOutput("Failed to add alert")
                if not(alertText) then ChatOutput("No Alert Text Specified") end
                if not(alertDuration) then ChatOutput("No Alert Duration Specified") end
            end

            --Valid Timer entry
            if timerYes and timerDuration and timerText then
                timer[1] = timerDuration
                timer[2] = timerText
                timer[3] = timerBackgroundText
                timer[4] = timerUnique
                timer[5] = timerLoop
                ChatOutput("Added Timer Succesfully")
            elseif timerYes then
                ChatOutput("Failed to add timer")
                if not(timerText) then ChatOutput("No Timer Text Specified") end
                if not(timerDuration) then ChatOutput("No Timer Duration Specified") end
            end

            restrict[3] = tieringEffect

            local completeInfo = {["Timers"] = {[1] = timer}, ["Alerts"] = {[1] = alert}, ["Restrictions"] = restrict}
            EventWindows:AddChatPlayerCombatAlert(trigger, completeInfo)
            self:AddTreeNode(self.ChatPlayerCombatNode, self.CurrentAlertBackground:GetWidth() - 10, trigger, function() self:FillChatPlayerCombatOptions(trigger, completeInfo) end, self.ChatPlayerCombatListBackground)
            self:RefreshTreeview()
            ChatOutput("Trigger added")
        else
            ChatOutput("Failed to add alert, check your input")
        end
    end

    self.ChatPlayerCombatListBackground = Turbine.UI.TreeNode()
    self.ChatPlayerCombatListBackground:SetSize(self.CurrentAlertBackground:GetWidth() - 10, 30)

    self.ChatPlayerCombatListBackground = Turbine.UI.TreeNode()
    self.ChatPlayerCombatListBackground:SetSize(self.CurrentAlertBackground:GetWidth() - 10, 30)
    
    self.ChatPlayerCombatListBoxHeader = Turbine.UI.Control()
    self.ChatPlayerCombatListBoxHeader:SetParent(self.ChatPlayerCombatListBackground)
    self.ChatPlayerCombatListBoxHeader:SetSize(self.CurrentAlertBackground:GetWidth() - 10, 30)

    self.ChatPlayerCombatListBoxHeaderLeft = Turbine.UI.Control()
    self.ChatPlayerCombatListBoxHeaderLeft:SetParent(self.ChatPlayerCombatListBoxHeader)
    self.ChatPlayerCombatListBoxHeaderLeft:SetSize(34, 30)
    self.ChatPlayerCombatListBoxHeaderLeft:SetBlendMode(Turbine.UI.BlendMode.AlphaBlend)
    self.ChatPlayerCombatListBoxHeaderLeft:SetBackground(BackgroundHandler:GetBackground(0x4110A0E0))

    self.ChatPlayerCombatListBoxHeaderRight = Turbine.UI.Label()
    self.ChatPlayerCombatListBoxHeaderRight:SetParent(self.ChatPlayerCombatListBoxHeader)
    self.ChatPlayerCombatListBoxHeaderRight:SetPosition(self.CurrentAlertBackground:GetWidth() - 34 - 10, 0)
    self.ChatPlayerCombatListBoxHeaderRight:SetSize(34, 30)
    self.ChatPlayerCombatListBoxHeaderRight:SetBlendMode(Turbine.UI.BlendMode.AlphaBlend)
    self.ChatPlayerCombatListBoxHeaderRight:SetBackground(BackgroundHandler:GetBackground(0x4110A0DB))

    self.ChatPlayerCombatListBoxHeaderMiddle = Turbine.UI.Label()
    self.ChatPlayerCombatListBoxHeaderMiddle:SetParent(self.ChatPlayerCombatListBoxHeader)
    self.ChatPlayerCombatListBoxHeaderMiddle:SetPosition(34, 0)
    self.ChatPlayerCombatListBoxHeaderMiddle:SetSize(self.CurrentAlertBackground:GetWidth() - 68 - 10, 30)
    self.ChatPlayerCombatListBoxHeaderMiddle:SetBlendMode(Turbine.UI.BlendMode.AlphaBlend)
    self.ChatPlayerCombatListBoxHeaderMiddle:SetBackground(BackgroundHandler:GetBackground(0x4110A0D9))

    self.ChatPlayerCombatListBoxHeaderLabel = Turbine.UI.Label()
    self.ChatPlayerCombatListBoxHeaderLabel:SetParent(self.ChatPlayerCombatListBackground)
    self.ChatPlayerCombatListBoxHeaderLabel:SetSize(self.CurrentAlertBackground:GetWidth() - 10, 30)
    self.ChatPlayerCombatListBoxHeaderLabel:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
    self.ChatPlayerCombatListBoxHeaderLabel:SetFont(Turbine.UI.Lotro.Font.BookAntiquaBold22)
    self.ChatPlayerCombatListBoxHeaderLabel:SetBlendMode(Turbine.UI.BlendMode.AlphaBlend)
    self.ChatPlayerCombatListBoxHeaderLabel:SetText("Chat (Combat Player)")
     
    self.ChatPlayerCombatListBoxHeaderLabel.MouseEnter = function()
        self.ChatPlayerCombatListBoxHeaderLeft:SetBackground(BackgroundHandler:GetBackground(0x4110A0DF))
        self.ChatPlayerCombatListBoxHeaderMiddle:SetBackground(BackgroundHandler:GetBackground(0x4110A0DD))
        self.ChatPlayerCombatListBoxHeaderRight:SetBackground(BackgroundHandler:GetBackground(0x4110A0DE))
    end

    self.ChatPlayerCombatListBoxHeaderLabel.MouseUp = function()
        self.ChatPlayerCombatListBoxHeaderLeft:SetBackground(BackgroundHandler:GetBackground(0x4110A0E0))
        self.ChatPlayerCombatListBoxHeaderMiddle:SetBackground(BackgroundHandler:GetBackground(0x4110A0D9))
        self.ChatPlayerCombatListBoxHeaderRight:SetBackground(BackgroundHandler:GetBackground(0x4110A0DB))
    end

    self.ChatPlayerCombatListBoxHeaderLabel.MouseDown = function()
        self.ChatPlayerCombatListBoxHeaderLeft:SetBackground(BackgroundHandler:GetBackground(0x4110A0D8))
        self.ChatPlayerCombatListBoxHeaderMiddle:SetBackground(BackgroundHandler:GetBackground(0x4110A0DA))
        self.ChatPlayerCombatListBoxHeaderRight:SetBackground(BackgroundHandler:GetBackground(0x4110A0DC))
    end

    self.ChatPlayerCombatListBoxHeaderLabel.MouseLeave = function()
        self.ChatPlayerCombatListBoxHeaderLeft:SetBackground(BackgroundHandler:GetBackground(0x4110A0E0))
        self.ChatPlayerCombatListBoxHeaderMiddle:SetBackground(BackgroundHandler:GetBackground(0x4110A0D9))
        self.ChatPlayerCombatListBoxHeaderRight:SetBackground(BackgroundHandler:GetBackground(0x4110A0DB))
    end


    self.rootNodes:Add(self.ChatPlayerCombatListBackground)

    self.ChatPlayerCombatNode = self.ChatPlayerCombatListBackground:GetChildNodes()
    -- [[
    for key, value in spairs(EventWindows:GetCurrentChatPlayerCombatData(), function(t, a, b) return a < b end) do
        self:AddTreeNode(self.ChatPlayerCombatNode, self.CurrentAlertBackground:GetWidth() - 10, key, function() self:FillChatPlayerCombatOptions(key, value) end)
    end
    --]]
end

function AlertOptions:ClearPlayerCombat()
    self.ChatPlayerCombatPhraseTextBox:SetText("")
    self.ChatPlayerCombatInstanceButton:SetText("All")
    self.ChatPlayerCombatRoleButton:SetText("All")

    -- Alert
    self.ChatPlayerCombatAlertCheckBox:SetChecked(false)
    self.ChatPlayerCombatAlertTextBox:SetText("")
    self.ChatPlayerCombatAlertDurationTextBox:SetText("")
    self.ChatPlayerCombatAlertCountdownCheckBox:SetChecked(false)
    self.ChatPlayerCombatAlertSoundLabel:SetChecked(false)
    self.ChatPlayerCombatSoundeffectButton:SetText("")
    self.ChatPlayerCombatAlertIconLabel:SetChecked(false)
    self.ChatPlayerCombatAlertIconBox:SetBackground(BackgroundHandler:GetBackground(0x41007F41))
    self.ChatPlayerCombatAlertIconTextBox:SetText(0x41007F41)

    -- Timer
    self.ChatPlayerCombatTimerCheckBox:SetChecked(false)
    self.ChatPlayerCombatTimerDurationTextBox:SetText("")
    self.ChatPlayerCombatTimerIconLabel:SetChecked(false)
    self.ChatPlayerCombatTimerIconBox:SetBackground(BackgroundHandler:GetBackground(0x41007F41))
    self.ChatPlayerCombatTimerIconTextBox:SetText(0x41007F41)
    self.ChatPlayerCombatTimerTextBox:SetText("")
    self.ChatPlayerCombatTimerUnique:SetChecked(false)
    self.ChatPlayerCombatTimerLoop:SetChecked(false)
end

function AlertOptions:FillChatPlayerCombatOptions(source, input)
    self:ToggleWindow("Chat (Combat Player)")
     --1 duration, 2 message, 3 icon, 4 countdown, 5 soundeffect, 6 instanceID, 7 role, 8 timerduration, 9 timer text, 10 timer icon, 11 unique, 12 loop, 13 windowID

    local timer = input["Timers"][1]
    local alert = input["Alerts"][1]
    local restrict = input["Restrictions"]

    -- Trigger
    self.ChatPlayerCombatPhraseTextBox:SetText(tostring(source))
    if restrict[1] then self.ChatPlayerCombatInstanceButton:SetText(tostring(InstanceController:GetDungeonNameTable()[tonumber(restrict[1])])) else self.ChatPlayerCombatInstanceButton:SetText("All") end
    if restrict[2] then self.ChatPlayerCombatRoleButton:SetText(Roles:GetAvailableRoles()[restrict[2]]) else self.ChatPlayerCombatRoleButton:SetText("All") end
    

    -- Alert
    self.ChatPlayerCombatAlertCheckBox:SetChecked(alert[1])
    if alert[2] then self.ChatPlayerCombatAlertTextBox:SetText(tostring(alert[2])) else self.ChatPlayerCombatAlertTextBox:SetText("") end
    self.ChatPlayerCombatAlertDurationTextBox:SetText(tonumber(alert[1]))
    self.ChatPlayerCombatAlertCountdownCheckBox:SetChecked(alert[4])
    self.ChatPlayerCombatAlertSoundLabel:SetChecked(alert[5])
    self.ChatPlayerCombatSoundeffectButton:SetText(alert[5])
    self.ChatPlayerCombatAlertIconLabel:SetChecked(alert[3])
    if alert[3] then
        self.ChatPlayerCombatAlertIconBox:SetBackground(BackgroundHandler:GetBackground(tonumber(alert[3])))
        self.ChatPlayerCombatAlertIconTextBox:SetText(tonumber(alert[3]))
    else
        self.ChatPlayerCombatAlertIconBox:SetBackground("")
        self.ChatPlayerCombatAlertIconTextBox:SetText("")
    end
    if alert[3] then self.ChatPlayerCombatAlertIconTextBox:SetText(tostring(alert[3])) else self.ChatPlayerCombatAlertIconTextBox:SetText("") end

    -- Timer
    self.ChatPlayerCombatTimerCheckBox:SetChecked(timer[1])
    self.ChatPlayerCombatTimerDurationTextBox:SetText(tonumber(timer[1]))
    self.ChatPlayerCombatTimerIconLabel:SetChecked(timer[3])
    if timer[3] then
        self.ChatPlayerCombatTimerIconBox:SetBackground(BackgroundHandler:GetBackground(tonumber(timer[3])))
        self.ChatPlayerCombatTimerIconTextBox:SetText(tonumber(timer[3]))
    else
        self.ChatPlayerCombatTimerIconBox:SetBackground("")
        self.ChatPlayerCombatTimerIconTextBox:SetText("")
    end
    if timer[2] then self.ChatPlayerCombatTimerTextBox:SetText(tostring(timer[2])) else self.ChatPlayerCombatTimerTextBox:SetText("") end
    self.ChatPlayerCombatTimerUnique:SetChecked(timer[4])
    self.ChatPlayerCombatTimerLoop:SetChecked(timer[5])
end

-- Creation/Changing chat player combat alert UI
function AlertOptions:ChatEnemyCombatAlerts(parent)

    self.ChatEnemyCombatBackground = Turbine.UI.Control()
    self.ChatEnemyCombatBackground:SetParent(parent)
    self.ChatEnemyCombatBackground:SetVisible(false)
    self.ChatEnemyCombatBackground:SetPosition(0, 0)
    self.ChatEnemyCombatBackground:SetSize(parent:GetWidth(), parent:GetHeight())

    self.SelectionWindows["Chat (Combat Enemy)"] = self.ChatEnemyCombatBackground

    -- Key phrase
    self.ChatEnemyCombatPhraseLabel = Turbine.UI.Label()
    self.ChatEnemyCombatPhraseLabel:SetParent(self.ChatEnemyCombatBackground)
    self.ChatEnemyCombatPhraseLabel:SetSize(200, 30)
    self.ChatEnemyCombatPhraseLabel:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft)
    self.ChatEnemyCombatPhraseLabel:SetFont(Turbine.UI.Lotro.Font.BookAntiquaBold19)
    self.ChatEnemyCombatPhraseLabel:SetFontStyle(Turbine.UI.FontStyle.Outline)
    self.ChatEnemyCombatPhraseLabel:SetMouseVisible(false)
    self.ChatEnemyCombatPhraseLabel:SetPosition(10, 70)
    self.ChatEnemyCombatPhraseLabel:SetVisible(true)
    self.ChatEnemyCombatPhraseLabel:SetText("Hit trigger")

    self.ChatEnemyCombatPhraseTextBox = Turbine.UI.Lotro.TextBox()
    self.ChatEnemyCombatPhraseTextBox:SetParent(self.ChatEnemyCombatBackground)
    self.ChatEnemyCombatPhraseTextBox:SetSize(300, 30)
    self.ChatEnemyCombatPhraseTextBox:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
    self.ChatEnemyCombatPhraseTextBox:SetMouseVisible(false)
    self.ChatEnemyCombatPhraseTextBox:SetPosition(140, 70)
    self.ChatEnemyCombatPhraseTextBox:SetMultiline(true)

    self.ChatEnemyCombatInstanceLabel = Turbine.UI.Label()
    self.ChatEnemyCombatInstanceLabel:SetParent(self.ChatEnemyCombatBackground)
    self.ChatEnemyCombatInstanceLabel:SetSize(300, 30)
    self.ChatEnemyCombatInstanceLabel:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft)
    self.ChatEnemyCombatInstanceLabel:SetFont(Turbine.UI.Lotro.Font.BookAntiquaBold19)
    self.ChatEnemyCombatInstanceLabel:SetFontStyle(Turbine.UI.FontStyle.Outline)
    self.ChatEnemyCombatInstanceLabel:SetMouseVisible(false)
    self.ChatEnemyCombatInstanceLabel:SetPosition(10, 125)
    self.ChatEnemyCombatInstanceLabel:SetVisible(true)
    self.ChatEnemyCombatInstanceLabel:SetText("Only load for specific instance/role")

    self.ChatEnemyCombatInstanceButton = Turbine.UI.Lotro.Button()
    self.ChatEnemyCombatInstanceButton:SetParent(self.ChatEnemyCombatBackground)
    self.ChatEnemyCombatInstanceButton:SetSize(100, 15)
    self.ChatEnemyCombatInstanceButton:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
    self.ChatEnemyCombatInstanceButton:SetMouseVisible(true)
    self.ChatEnemyCombatInstanceButton:SetPosition(240, 133)
    self.ChatEnemyCombatInstanceButton:SetText("All")

    self.ChatEnemyCombatInstanceMenu = UI.DropDownMenuControl(self, 200, "Instance Dropdown Menu")
    for key, value in spairs(InstanceController:GetDungeonNameTable(), function(t, a, b) return t[a] < t[b] end) do
        self.ChatEnemyCombatInstanceMenu:AddItem(value, function()
            self.ChatEnemyCombatInstanceButton:SetText(value)
            self.ChatEnemyCombatInstanceMenu:SetVisible(false)
        end)
    end

    self.ChatEnemyCombatInstanceButton.Click = function()
        self.ChatPlayerCombatInstanceMenu:ToggleVisible()
    end

    self.ChatEnemyCombatRoleButton = Turbine.UI.Lotro.Button()
    self.ChatEnemyCombatRoleButton:SetParent(self.ChatEnemyCombatBackground)
    self.ChatEnemyCombatRoleButton:SetSize(100, 15)
    self.ChatEnemyCombatRoleButton:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
    self.ChatEnemyCombatRoleButton:SetMouseVisible(true)
    self.ChatEnemyCombatRoleButton:SetPosition(340, 133)
    self.ChatEnemyCombatRoleButton:SetText("All")

    self.ChatEnemyCombatRoleMenu = Turbine.UI.ContextMenu()
    self.ChatEnemyCombatRoleMenuItems = self.ChatEnemyCombatRoleMenu:GetItems()
    for key, value in spairs(Roles:GetAvailableRoles(), function(t, a, b) return t[a] < t[b] end) do
        local menuItem = Turbine.UI.MenuItem( value )
        self.ChatEnemyCombatRoleMenuItems:Add(menuItem)
        menuItem.Click = function()
            self.ChatEnemyCombatRoleButton:SetText(value)
        end
    end

    local menuItem = Turbine.UI.MenuItem( "All" )
    self.ChatEnemyCombatRoleMenuItems:Add(menuItem)
    menuItem.Click = function()
        self.ChatEnemyCombatRoleButton:SetText("All")
    end

    self.ChatEnemyCombatRoleButton.Click = function()
        self.ChatEnemyCombatRoleMenu:ShowMenu()
    end


    -- Alert
    self.ChatEnemyCombatAlertBackground = Turbine.UI.Control()
    self.ChatEnemyCombatAlertBackground:SetParent(self.ChatEnemyCombatBackground)
    self.ChatEnemyCombatAlertBackground:SetPosition(10, 160)
    self.ChatEnemyCombatAlertBackground:SetSize(parent:GetWidth() - 20, parent:GetHeight() - 260)
    self.ChatEnemyCombatAlertBackground:SetBackColor(Turbine.UI.Color(0.5, 0.5, 0.5))

    self.ChatEnemyCombatAlertCheckBox = Turbine.UI.Lotro.CheckBox()
    self.ChatEnemyCombatAlertCheckBox:SetParent(self.ChatEnemyCombatAlertBackground)
    self.ChatEnemyCombatAlertCheckBox:SetSize(200, 30)
    self.ChatEnemyCombatAlertCheckBox:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft)
    self.ChatEnemyCombatAlertCheckBox:SetFont(Turbine.UI.Lotro.Font.BookAntiquaBold24)
    self.ChatEnemyCombatAlertCheckBox:SetFontStyle(Turbine.UI.FontStyle.Outline)
    self.ChatEnemyCombatAlertCheckBox:SetMouseVisible(false)
    self.ChatEnemyCombatAlertCheckBox:SetPosition(3, 3)
    self.ChatEnemyCombatAlertCheckBox:SetVisible(true)
    self.ChatEnemyCombatAlertCheckBox:SetText("Alert on Trigger")

    self.ChatEnemyCombatPhraseLabel = Turbine.UI.Label()
    self.ChatEnemyCombatPhraseLabel:SetParent(self.ChatEnemyCombatAlertBackground)
    self.ChatEnemyCombatPhraseLabel:SetSize(100, 30)
    self.ChatEnemyCombatPhraseLabel:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft)
    self.ChatEnemyCombatPhraseLabel:SetFont(Turbine.UI.Lotro.Font.BookAntiquaBold19)
    self.ChatEnemyCombatPhraseLabel:SetFontStyle(Turbine.UI.FontStyle.Outline)
    self.ChatEnemyCombatPhraseLabel:SetMouseVisible(false)
    self.ChatEnemyCombatPhraseLabel:SetPosition(10, 35)
    self.ChatEnemyCombatPhraseLabel:SetVisible(true)
    self.ChatEnemyCombatPhraseLabel:SetText("Alert Text")

    self.ChatEnemyCombatAlertTextBox = Turbine.UI.Lotro.TextBox()
    self.ChatEnemyCombatAlertTextBox:SetParent(self.ChatEnemyCombatAlertBackground)
    self.ChatEnemyCombatAlertTextBox:SetSize(200, 30)
    self.ChatEnemyCombatAlertTextBox:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
    self.ChatEnemyCombatAlertTextBox:SetMouseVisible(false)
    self.ChatEnemyCombatAlertTextBox:SetMultiline(false)
    self.ChatEnemyCombatAlertTextBox:SetPosition(80, 35)

    self.ChatEnemyCombatAlertDurationLabel = Turbine.UI.Label()
    self.ChatEnemyCombatAlertDurationLabel:SetParent(self.ChatEnemyCombatAlertBackground)
    self.ChatEnemyCombatAlertDurationLabel:SetSize(200, 30)
    self.ChatEnemyCombatAlertDurationLabel:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft)
    self.ChatEnemyCombatAlertDurationLabel:SetFont(Turbine.UI.Lotro.Font.BookAntiquaBold19)
    self.ChatEnemyCombatAlertDurationLabel:SetFontStyle(Turbine.UI.FontStyle.Outline)
    self.ChatEnemyCombatAlertDurationLabel:SetMouseVisible(false)
    self.ChatEnemyCombatAlertDurationLabel:SetPosition(10, 65)
    self.ChatEnemyCombatAlertDurationLabel:SetVisible(true)
    self.ChatEnemyCombatAlertDurationLabel:SetText("Alert duration in seconds")

    self.ChatEnemyCombatAlertDurationTextBox = Turbine.UI.Lotro.TextBox()
    self.ChatEnemyCombatAlertDurationTextBox:SetParent(self.ChatEnemyCombatAlertBackground)
    self.ChatEnemyCombatAlertDurationTextBox:SetSize(50, 30)
    self.ChatEnemyCombatAlertDurationTextBox:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
    self.ChatEnemyCombatAlertDurationTextBox:SetMouseVisible(false)
    self.ChatEnemyCombatAlertDurationTextBox:SetPosition(185, 65)
    self.ChatEnemyCombatAlertDurationTextBox:SetMultiline(false)


    -- Countdown
    self.ChatEnemyCombatAlertCountdownCheckBox = Turbine.UI.Lotro.CheckBox()
    self.ChatEnemyCombatAlertCountdownCheckBox:SetParent(self.ChatEnemyCombatAlertBackground)
    self.ChatEnemyCombatAlertCountdownCheckBox:SetSize(200, 30)
    self.ChatEnemyCombatAlertCountdownCheckBox:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft)
    self.ChatEnemyCombatAlertCountdownCheckBox:SetFont(Turbine.UI.Lotro.Font.BookAntiquaBold19)
    self.ChatEnemyCombatAlertCountdownCheckBox:SetFontStyle(Turbine.UI.FontStyle.Outline)
    self.ChatEnemyCombatAlertCountdownCheckBox:SetMouseVisible(false)
    self.ChatEnemyCombatAlertCountdownCheckBox:SetPosition(250, 65)
    self.ChatEnemyCombatAlertCountdownCheckBox:SetVisible(true)
    self.ChatEnemyCombatAlertCountdownCheckBox:SetText("Alert duration shown")

    -- Sound effects
    self.ChatEnemyCombatAlertSoundLabel = Turbine.UI.Lotro.CheckBox()
    self.ChatEnemyCombatAlertSoundLabel:SetParent(self.ChatEnemyCombatAlertBackground)
    self.ChatEnemyCombatAlertSoundLabel:SetSize(200, 30)
    self.ChatEnemyCombatAlertSoundLabel:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft)
    self.ChatEnemyCombatAlertSoundLabel:SetFont(Turbine.UI.Lotro.Font.BookAntiquaBold19)
    self.ChatEnemyCombatAlertSoundLabel:SetFontStyle(Turbine.UI.FontStyle.Outline)
    self.ChatEnemyCombatAlertSoundLabel:SetMouseVisible(false)
    self.ChatEnemyCombatAlertSoundLabel:SetPosition(10, 90)
    self.ChatEnemyCombatAlertSoundLabel:SetVisible(true)
    self.ChatEnemyCombatAlertSoundLabel:SetText("Sound Effect")

    self.ChatEnemyCombatSoundeffectButton = Turbine.UI.Lotro.Button()
    self.ChatEnemyCombatSoundeffectButton:SetParent(self.ChatEnemyCombatAlertBackground)
    self.ChatEnemyCombatSoundeffectButton:SetSize(100, 15)
    self.ChatEnemyCombatSoundeffectButton:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
    self.ChatEnemyCombatSoundeffectButton:SetMouseVisible(true)
    self.ChatEnemyCombatSoundeffectButton:SetPosition(120, 97)
    self.ChatEnemyCombatSoundeffectButton:SetText("Horn")

    self.ChatEnemyCombatSoundEffectMenu = UI.DropDownMenuControl(self, 200, "Sound Menu")
    for key, value in spairs(Sounds:GetSoundList(), function(t, a, b) return a < b end) do
        self.ChatEnemyCombatSoundEffectMenu:AddItem(key, 
        function()
            self.ChatEnemyCombatSoundeffectButton:SetText(key)
            self.ChatEnemyCombatSoundEffectMenu:ToggleVisible()
        end)
    end

    self.ChatEnemyCombatSoundeffectButton.Click = function()
        self.ChatEnemyCombatSoundEffectMenu:ToggleVisible()
    end

    self.ChatEnemyCombatAlertIconLabel = Turbine.UI.Lotro.CheckBox()
    self.ChatEnemyCombatAlertIconLabel:SetParent(self.ChatEnemyCombatAlertBackground)
    self.ChatEnemyCombatAlertIconLabel:SetSize(50, 30)
    self.ChatEnemyCombatAlertIconLabel:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft)
    self.ChatEnemyCombatAlertIconLabel:SetFont(Turbine.UI.Lotro.Font.BookAntiquaBold19)
    self.ChatEnemyCombatAlertIconLabel:SetFontStyle(Turbine.UI.FontStyle.Outline)
    self.ChatEnemyCombatAlertIconLabel:SetMouseVisible(false)
    self.ChatEnemyCombatAlertIconLabel:SetPosition(240, 97)
    self.ChatEnemyCombatAlertIconLabel:SetVisible(true)
    self.ChatEnemyCombatAlertIconLabel:SetText("Icon")

    self.ChatEnemyCombatAlertIconBox = Turbine.UI.Control()
    self.ChatEnemyCombatAlertIconBox:SetParent(self.ChatEnemyCombatAlertBackground)
    self.ChatEnemyCombatAlertIconBox:SetPosition(290, 97)
    self.ChatEnemyCombatAlertIconBox:SetSize(32,32)
    self.ChatEnemyCombatAlertIconBox:SetBlendMode(Turbine.UI.BlendMode.AlphaBlend)
    self.ChatEnemyCombatAlertIconBox:SetBackground(BackgroundHandler:GetBackground(0x41007F41))
    self.ChatEnemyCombatAlertIconBox:SetVisible(true)

    self.ChatEnemyCombatAlertIconTextBox = Turbine.UI.Lotro.TextBox()
    self.ChatEnemyCombatAlertIconTextBox:SetParent(self.ChatEnemyCombatAlertBackground)
    self.ChatEnemyCombatAlertIconTextBox:SetSize(110, 30)
    self.ChatEnemyCombatAlertIconTextBox:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
    self.ChatEnemyCombatAlertIconTextBox:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
    self.ChatEnemyCombatAlertIconTextBox:SetMouseVisible(false)
    self.ChatEnemyCombatAlertIconTextBox:SetPosition(330, 97)
    self.ChatEnemyCombatAlertIconTextBox:SetMultiline(false)
    self.ChatEnemyCombatAlertIconTextBox:SetText(0x41007F41)

    self.ChatEnemyCombatAlertIconTextBox.TextChanged = function()
        --pcall(self.ChatEnemyCombatAlertIconBox:SetBackground(tonumber(self.ChatEnemyCombatAlertIconTextBox:GetText())))
    end

    self.ChatEnemyCombatAlertIconBox.MouseClick = function()
        if self.IconOverview then 
            self.IconOverview:SetVisible(true) 
            self.iconLink = self.ChatEnemyCombatAlertIconBox
            self.iconTextLink = self.ChatEnemyCombatAlertIconTextBox
        end
    end

    self.ChatEnemyCombatAlertIconBox.MouseDoubleClick = function()
        self.ChatEnemyCombatAlertIconBox:SetBackground(BackgroundHandler:GetBackground(0x41007F41))
        self.iconID = 0x41007F41
    end


    --Timer
    self.ChatEnemyCombatTimerBackground = Turbine.UI.Control()
    self.ChatEnemyCombatTimerBackground:SetParent(self.ChatEnemyCombatBackground)
    self.ChatEnemyCombatTimerBackground:SetPosition(10, 300)
    self.ChatEnemyCombatTimerBackground:SetSize(parent:GetWidth() - 20, 80)
    self.ChatEnemyCombatTimerBackground:SetBackColor(Turbine.UI.Color(0.5, 0.5, 0.5))

    self.ChatEnemyCombatTimerCheckBox = Turbine.UI.Lotro.CheckBox()
    self.ChatEnemyCombatTimerCheckBox:SetParent(self.ChatEnemyCombatTimerBackground)
    self.ChatEnemyCombatTimerCheckBox:SetSize(200, 30)
    self.ChatEnemyCombatTimerCheckBox:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft)
    self.ChatEnemyCombatTimerCheckBox:SetFont(Turbine.UI.Lotro.Font.BookAntiquaBold24)
    self.ChatEnemyCombatTimerCheckBox:SetFontStyle(Turbine.UI.FontStyle.Outline)
    self.ChatEnemyCombatTimerCheckBox:SetMouseVisible(false)
    self.ChatEnemyCombatTimerCheckBox:SetPosition(3, 3)
    self.ChatEnemyCombatTimerCheckBox:SetVisible(true)
    self.ChatEnemyCombatTimerCheckBox:SetText("Timer on Trigger")

    self.ChatEnemyCombatTimerText = Turbine.UI.Label()
    self.ChatEnemyCombatTimerText:SetParent(self.ChatEnemyCombatTimerBackground)
    self.ChatEnemyCombatTimerText:SetSize(100, 30)
    self.ChatEnemyCombatTimerText:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft)
    self.ChatEnemyCombatTimerText:SetFont(Turbine.UI.Lotro.Font.BookAntiquaBold19)
    self.ChatEnemyCombatTimerText:SetFontStyle(Turbine.UI.FontStyle.Outline)
    self.ChatEnemyCombatTimerText:SetMouseVisible(false)
    self.ChatEnemyCombatTimerText:SetPosition(195, 0)
    self.ChatEnemyCombatTimerText:SetVisible(true)
    self.ChatEnemyCombatTimerText:SetText("Timer Text")

    self.ChatEnemyCombatTimerTextBox = Turbine.UI.Lotro.TextBox()
    self.ChatEnemyCombatTimerTextBox:SetParent(self.ChatEnemyCombatTimerBackground)
    self.ChatEnemyCombatTimerTextBox:SetSize(170, 25)
    self.ChatEnemyCombatTimerTextBox:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
    self.ChatEnemyCombatTimerTextBox:SetMouseVisible(false)
    self.ChatEnemyCombatTimerTextBox:SetMultiline(false)
    self.ChatEnemyCombatTimerTextBox:SetPosition(270, 3)

    self.ChatEnemyCombatTimerDurationLabel = Turbine.UI.Label()
    self.ChatEnemyCombatTimerDurationLabel:SetParent(self.ChatEnemyCombatTimerBackground)
    self.ChatEnemyCombatTimerDurationLabel:SetSize(200, 30)
    self.ChatEnemyCombatTimerDurationLabel:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft)
    self.ChatEnemyCombatTimerDurationLabel:SetFont(Turbine.UI.Lotro.Font.BookAntiquaBold19)
    self.ChatEnemyCombatTimerDurationLabel:SetFontStyle(Turbine.UI.FontStyle.Outline)
    self.ChatEnemyCombatTimerDurationLabel:SetMouseVisible(false)
    self.ChatEnemyCombatTimerDurationLabel:SetPosition(10, 30)
    self.ChatEnemyCombatTimerDurationLabel:SetVisible(true)
    self.ChatEnemyCombatTimerDurationLabel:SetText("Timer duration in seconds")

    self.ChatEnemyCombatTimerDurationTextBox = Turbine.UI.Lotro.TextBox()
    self.ChatEnemyCombatTimerDurationTextBox:SetParent(self.ChatEnemyCombatTimerBackground)
    self.ChatEnemyCombatTimerDurationTextBox:SetSize(50, 30)
    self.ChatEnemyCombatTimerDurationTextBox:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
    self.ChatEnemyCombatTimerDurationTextBox:SetMouseVisible(false)
    self.ChatEnemyCombatTimerDurationTextBox:SetPosition(185, 30)
    self.ChatEnemyCombatTimerDurationTextBox:SetMultiline(false)

    self.ChatEnemyCombatTimerIconLabel = Turbine.UI.Lotro.CheckBox()
    self.ChatEnemyCombatTimerIconLabel:SetParent(self.ChatEnemyCombatTimerBackground)
    self.ChatEnemyCombatTimerIconLabel:SetSize(50, 30)
    self.ChatEnemyCombatTimerIconLabel:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft)
    self.ChatEnemyCombatTimerIconLabel:SetFont(Turbine.UI.Lotro.Font.BookAntiquaBold19)
    self.ChatEnemyCombatTimerIconLabel:SetFontStyle(Turbine.UI.FontStyle.Outline)
    self.ChatEnemyCombatTimerIconLabel:SetMouseVisible(false)
    self.ChatEnemyCombatTimerIconLabel:SetPosition(240, 30)
    self.ChatEnemyCombatTimerIconLabel:SetVisible(true)
    self.ChatEnemyCombatTimerIconLabel:SetText("Icon")

    self.ChatEnemyCombatTimerIconBox = Turbine.UI.Control()
    self.ChatEnemyCombatTimerIconBox:SetParent(self.ChatEnemyCombatTimerBackground)
    self.ChatEnemyCombatTimerIconBox:SetPosition(290, 30)
    self.ChatEnemyCombatTimerIconBox:SetSize(32,32)
    self.ChatEnemyCombatTimerIconBox:SetBlendMode(Turbine.UI.BlendMode.AlphaBlend)
    self.ChatEnemyCombatTimerIconBox:SetBackground(BackgroundHandler:GetBackground(0x41007F41))
    self.ChatEnemyCombatTimerIconBox:SetVisible(true)

    self.ChatEnemyCombatTimerIconTextBox = Turbine.UI.Lotro.TextBox()
    self.ChatEnemyCombatTimerIconTextBox:SetParent(self.ChatEnemyCombatTimerBackground)
    self.ChatEnemyCombatTimerIconTextBox:SetSize(110, 30)
    self.ChatEnemyCombatTimerIconTextBox:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
    self.ChatEnemyCombatTimerIconTextBox:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
    self.ChatEnemyCombatTimerIconTextBox:SetMouseVisible(false)
    self.ChatEnemyCombatTimerIconTextBox:SetPosition(330, 30)
    self.ChatEnemyCombatTimerIconTextBox:SetMultiline(false)
    self.ChatEnemyCombatTimerIconTextBox:SetText(0x41007F41)

    self.ChatEnemyCombatTimerIconTextBox.TextChanged = function()
        --pcall(self.ChatEnemyCombatTimerIconBox:SetBackground(tonumber(self.ChatEnemyCombatTimerIconTextBox:GetText())))
    end

    self.ChatEnemyCombatTimerIconBox.MouseClick = function()
        if self.IconOverview then 
            self.IconOverview:SetVisible(true) 
            self.iconLink = self.ChatEnemyCombatTimerIconBox
            self.iconTextLink = self.ChatEnemyCombatTimerIconTextBox
        end
    end

    self.ChatEnemyCombatTimerIconBox.MouseDoubleClick = function()
        self.ChatEnemyCombatTimerIconBox:SetBackground(BackgroundHandler:GetBackground(0x41007F41))
        self.iconID = 0x41007F41
    end

    self.ChatEnemyCombatTimerUnique = Turbine.UI.Lotro.CheckBox()
    self.ChatEnemyCombatTimerUnique:SetParent(self.ChatEnemyCombatTimerBackground)
    self.ChatEnemyCombatTimerUnique:SetSize(200, 30)
    self.ChatEnemyCombatTimerUnique:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft)
    self.ChatEnemyCombatTimerUnique:SetFont(Turbine.UI.Lotro.Font.BookAntiquaBold19)
    self.ChatEnemyCombatTimerUnique:SetFontStyle(Turbine.UI.FontStyle.Outline)
    self.ChatEnemyCombatTimerUnique:SetMouseVisible(false)
    self.ChatEnemyCombatTimerUnique:SetPosition(10, 50)
    self.ChatEnemyCombatTimerUnique:SetVisible(true)
    self.ChatEnemyCombatTimerUnique:SetText("Unique")

    self.ChatEnemyCombatTimerLoop = Turbine.UI.Lotro.CheckBox()
    self.ChatEnemyCombatTimerLoop:SetParent(self.ChatEnemyCombatTimerBackground)
    self.ChatEnemyCombatTimerLoop:SetSize(200, 30)
    self.ChatEnemyCombatTimerLoop:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft)
    self.ChatEnemyCombatTimerLoop:SetFont(Turbine.UI.Lotro.Font.BookAntiquaBold19)
    self.ChatEnemyCombatTimerLoop:SetFontStyle(Turbine.UI.FontStyle.Outline)
    self.ChatEnemyCombatTimerLoop:SetMouseVisible(false)
    self.ChatEnemyCombatTimerLoop:SetPosition(80, 50)
    self.ChatEnemyCombatTimerLoop:SetVisible(true)
    self.ChatEnemyCombatTimerLoop:SetText("Loop")

    self.ChatEnemyCombatHeader = Turbine.UI.Label()
    self.ChatEnemyCombatHeader:SetParent(self.ChatEnemyCombatBackground)
    self.ChatEnemyCombatHeader:SetSize(230, 30)
    self.ChatEnemyCombatHeader:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft)
    self.ChatEnemyCombatHeader:SetFont(Turbine.UI.Lotro.Font.BookAntiquaBold24)
    self.ChatEnemyCombatHeader:SetFontStyle(Turbine.UI.FontStyle.Outline)
    self.ChatEnemyCombatHeader:SetMouseVisible(false)
    self.ChatEnemyCombatHeader:SetPosition(10, 0)
    self.ChatEnemyCombatHeader:SetVisible(true)
    self.ChatEnemyCombatHeader:SetText("Chat (Combat Enemy)")

    self.ChatEnemyCombatSaveButton = Turbine.UI.Lotro.Button()
    self.ChatEnemyCombatSaveButton:SetParent(self.ChatEnemyCombatBackground)
    self.ChatEnemyCombatSaveButton:SetSize(60, 15)
    self.ChatEnemyCombatSaveButton:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
    self.ChatEnemyCombatSaveButton:SetMouseVisible(true)
    self.ChatEnemyCombatSaveButton:SetPosition(210, 5)
    self.ChatEnemyCombatSaveButton:SetText("Save")

    self.ChatEnemyCombatRemoveButton = Turbine.UI.Lotro.Button()
    self.ChatEnemyCombatRemoveButton:SetParent(self.ChatEnemyCombatBackground)
    self.ChatEnemyCombatRemoveButton:SetSize(100, 15)
    self.ChatEnemyCombatRemoveButton:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
    self.ChatEnemyCombatRemoveButton:SetMouseVisible(true)
    self.ChatEnemyCombatRemoveButton:SetPosition(270, 5)
    self.ChatEnemyCombatRemoveButton:SetText("Remove")

    self.ChatEnemyCombatRemoveButton.Click = function()
        EventWindows:AddChatEnemyCombatAlert(tostring(self.ChatEnemyCombatPhraseTextBox:GetText()), nil)
        self:RemoveNode(self.ChatEnemyCombatNode, tostring(self.ChatEnemyCombatPhraseTextBox:GetText()))
        self:ClearEnemyCombat()
        ChatOutput("Removed Alert")
    end

    self.ChatEnemyCombatClearButton = Turbine.UI.Lotro.Button()
    self.ChatEnemyCombatClearButton:SetParent(self.ChatEnemyCombatBackground)
    self.ChatEnemyCombatClearButton:SetSize(100, 15)
    self.ChatEnemyCombatClearButton:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
    self.ChatEnemyCombatClearButton:SetMouseVisible(true)
    self.ChatEnemyCombatClearButton:SetPosition(370, 5)
    self.ChatEnemyCombatClearButton:SetText("Clear")

    self.ChatEnemyCombatClearButton.Click = function()
        self:ClearEnemyCombat()
    end

    -- [[
    self.ChatEnemyCombatSaveButton.Click = function()
        -- Trigger
        local trigger = tostring(self.ChatEnemyCombatPhraseTextBox:GetText())
        local instanceName = tostring(self.ChatEnemyCombatInstanceButton:GetText())
        local roleName = tostring(self.ChatEnemyCombatRoleButton:GetText())

        -- Alert
        local alertYes = self.ChatEnemyCombatAlertCheckBox:IsChecked()
        local alertText = tostring(self.ChatEnemyCombatAlertTextBox:GetText())
        local alertDuration = tonumber(self.ChatEnemyCombatAlertDurationTextBox:GetText()) or 6
        local alertCountdown = self.ChatEnemyCombatAlertCountdownCheckBox:IsChecked()
        local alertSound = self.ChatEnemyCombatAlertSoundLabel:IsChecked()
        local alertSoundEffect = tostring(self.ChatEnemyCombatSoundeffectButton:GetText())
        local alertIconBox = nil
        local alertIconText = nil
        if self.ChatEnemyCombatAlertIconLabel:IsChecked() then
            alertIconBox = tonumber(self.ChatEnemyCombatAlertIconBox:GetBackground())
            alertIconText = tonumber(self.ChatEnemyCombatAlertIconTextBox:GetText())
        end

        -- Timer
        local timerYes = self.ChatEnemyCombatTimerCheckBox:IsChecked()
        local timerDuration = tonumber(self.ChatEnemyCombatTimerDurationTextBox:GetText()) or 6
        local timerBackgroundIcon = nil
        local timerBackgroundText = nil
        if self.ChatEnemyCombatTimerIconLabel:IsChecked() then
            timerBackgroundIcon = tonumber(self.ChatEnemyCombatTimerIconBox:GetBackground())
            timerBackgroundText = tonumber(self.ChatEnemyCombatTimerIconTextBox:GetText())
        end
        local timerText = tostring(self.ChatEnemyCombatTimerTextBox:GetText())
        local timerUnique = self.ChatEnemyCombatTimerUnique:IsChecked()
        local timerLoop = self.ChatEnemyCombatTimerLoop:IsChecked()

        local newAlert = {}
        --1 duration, 2 message, 3 icon, 4 countdown, 5 soundeffect, 6 instanceID, 7 role, 8 timerduration, 9 timer text, 10 timer icon, 11 unique, 12 loop, 13 windowID
        -- Valid trigger entry
        if trigger and instanceName and roleName then
            local alert = {}
            local timer = {}
            local restrict = {}
            if roleName ~= "All" then restrict[2] = Roles:NameToRoleID(roleName) end
            if instanceName ~= "All" then 
                local result = nil
                for key, value in pairs(InstanceController:GetDungeonNameTable()) do
                    if instanceName == value then
                        result = key
                        break
                    end
                end
                restrict[1] = result 
            end


            -- Valid Alert entry
            if alertYes and alertText and alertDuration then
                alert[1] = alertDuration
                alert[2] = alertText
                alert[3] = alertIconText
                alert[4] = alertCountdown
                ChatOutput("Added Alert Succesfully")
                if alertSound then alert[5] = Sounds:GetSoundList()[alertSoundEffect] or alertSoundEffect end
            elseif alertYes then
                ChatOutput("Failed to add alert")
                if not(alertText) then ChatOutput("No Alert Text Specified") end
                if not(alertDuration) then ChatOutput("No Alert Duration Specified") end
            end

            --Valid Timer entry
            if timerYes and timerDuration and timerText then
                timer[1] = timerDuration
                timer[2] = timerText
                timer[3] = timerBackgroundText
                timer[4] = timerUnique
                timer[5] = timerLoop
                ChatOutput("Added Timer Succesfully")
            elseif timerYes then
                ChatOutput("Failed to add timer")
                if not(timerText) then ChatOutput("No Timer Text Specified") end
                if not(timerDuration) then ChatOutput("No Timer Duration Specified") end
            end

            restrict[3] = tieringEffect

            local completeInfo = {["Timers"] = {[1] = timer}, ["Alerts"] = {[1] = alert}, ["Restrictions"] = restrict}
            EventWindows:AddChatEnemyCombatAlert(trigger, completeInfo)
            self:AddTreeNode(self.ChatEnemyCombatNode, self.CurrentAlertBackground:GetWidth() - 10, trigger, function() self:FillChatEnemyCombatOptions(trigger, completeInfo) end, self.ChatEnemyCombatListBackground)
            self:RefreshTreeview()
            ChatOutput("Trigger added")
        else
            ChatOutput("Failed to add alert, check your input")
        end
    end

    self.ChatEnemyCombatListBackground = Turbine.UI.TreeNode()
    self.ChatEnemyCombatListBackground:SetSize(self.CurrentAlertBackground:GetWidth() - 10, 30)

    self.ChatEnemyCombatListBackground = Turbine.UI.TreeNode()
    self.ChatEnemyCombatListBackground:SetSize(self.CurrentAlertBackground:GetWidth() - 10, 30)
    
    self.ChatEnemyCombatListBoxHeader = Turbine.UI.Control()
    self.ChatEnemyCombatListBoxHeader:SetParent(self.ChatEnemyCombatListBackground)
    self.ChatEnemyCombatListBoxHeader:SetSize(self.CurrentAlertBackground:GetWidth() - 10, 30)

    self.ChatEnemyCombatListBoxHeaderLeft = Turbine.UI.Control()
    self.ChatEnemyCombatListBoxHeaderLeft:SetParent(self.ChatEnemyCombatListBoxHeader)
    self.ChatEnemyCombatListBoxHeaderLeft:SetSize(34, 30)
    self.ChatEnemyCombatListBoxHeaderLeft:SetBlendMode(Turbine.UI.BlendMode.AlphaBlend)
    self.ChatEnemyCombatListBoxHeaderLeft:SetBackground(BackgroundHandler:GetBackground(0x4110A0E0))

    self.ChatEnemyCombatListBoxHeaderRight = Turbine.UI.Label()
    self.ChatEnemyCombatListBoxHeaderRight:SetParent(self.ChatEnemyCombatListBoxHeader)
    self.ChatEnemyCombatListBoxHeaderRight:SetPosition(self.CurrentAlertBackground:GetWidth() - 34 - 10, 0)
    self.ChatEnemyCombatListBoxHeaderRight:SetSize(34, 30)
    self.ChatEnemyCombatListBoxHeaderRight:SetBlendMode(Turbine.UI.BlendMode.AlphaBlend)
    self.ChatEnemyCombatListBoxHeaderRight:SetBackground(BackgroundHandler:GetBackground(0x4110A0DB))

    self.ChatEnemyCombatListBoxHeaderMiddle = Turbine.UI.Label()
    self.ChatEnemyCombatListBoxHeaderMiddle:SetParent(self.ChatEnemyCombatListBoxHeader)
    self.ChatEnemyCombatListBoxHeaderMiddle:SetPosition(34, 0)
    self.ChatEnemyCombatListBoxHeaderMiddle:SetSize(self.CurrentAlertBackground:GetWidth() - 68 - 10, 30)
    self.ChatEnemyCombatListBoxHeaderMiddle:SetBlendMode(Turbine.UI.BlendMode.AlphaBlend)
    self.ChatEnemyCombatListBoxHeaderMiddle:SetBackground(BackgroundHandler:GetBackground(0x4110A0D9))

    self.ChatEnemyCombatListBoxHeaderLabel = Turbine.UI.Label()
    self.ChatEnemyCombatListBoxHeaderLabel:SetParent(self.ChatEnemyCombatListBackground)
    self.ChatEnemyCombatListBoxHeaderLabel:SetSize(self.CurrentAlertBackground:GetWidth() - 10, 30)
    self.ChatEnemyCombatListBoxHeaderLabel:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
    self.ChatEnemyCombatListBoxHeaderLabel:SetFont(Turbine.UI.Lotro.Font.BookAntiquaBold22)
    self.ChatEnemyCombatListBoxHeaderLabel:SetBlendMode(Turbine.UI.BlendMode.AlphaBlend)
    self.ChatEnemyCombatListBoxHeaderLabel:SetText("Chat (Combat Enemy)")
     
    self.ChatEnemyCombatListBoxHeaderLabel.MouseEnter = function()
        self.ChatEnemyCombatListBoxHeaderLeft:SetBackground(BackgroundHandler:GetBackground(0x4110A0DF))
        self.ChatEnemyCombatListBoxHeaderMiddle:SetBackground(BackgroundHandler:GetBackground(0x4110A0DD))
        self.ChatEnemyCombatListBoxHeaderRight:SetBackground(BackgroundHandler:GetBackground(0x4110A0DE))
    end

    self.ChatEnemyCombatListBoxHeaderLabel.MouseUp = function()
        self.ChatEnemyCombatListBoxHeaderLeft:SetBackground(BackgroundHandler:GetBackground(0x4110A0E0))
        self.ChatEnemyCombatListBoxHeaderMiddle:SetBackground(BackgroundHandler:GetBackground(0x4110A0D9))
        self.ChatEnemyCombatListBoxHeaderRight:SetBackground(BackgroundHandler:GetBackground(0x4110A0DB))
    end

    self.ChatEnemyCombatListBoxHeaderLabel.MouseDown = function()
        self.ChatEnemyCombatListBoxHeaderLeft:SetBackground(BackgroundHandler:GetBackground(0x4110A0D8))
        self.ChatEnemyCombatListBoxHeaderMiddle:SetBackground(BackgroundHandler:GetBackground(0x4110A0DA))
        self.ChatEnemyCombatListBoxHeaderRight:SetBackground(BackgroundHandler:GetBackground(0x4110A0DC))
    end

    self.ChatEnemyCombatListBoxHeaderLabel.MouseLeave = function()
        self.ChatEnemyCombatListBoxHeaderLeft:SetBackground(BackgroundHandler:GetBackground(0x4110A0E0))
        self.ChatEnemyCombatListBoxHeaderMiddle:SetBackground(BackgroundHandler:GetBackground(0x4110A0D9))
        self.ChatEnemyCombatListBoxHeaderRight:SetBackground(BackgroundHandler:GetBackground(0x4110A0DB))
    end


    self.rootNodes:Add(self.ChatEnemyCombatListBackground)

    self.ChatEnemyCombatNode = self.ChatEnemyCombatListBackground:GetChildNodes()
    -- [[
    for key, value in spairs(EventWindows:GetCurrentChatEnemyCombatData(), function(t, a, b) return a < b end) do
        self:AddTreeNode(self.ChatEnemyCombatNode, self.CurrentAlertBackground:GetWidth() - 10, key, function() self:FillChatEnemyCombatOptions(key, value) end)
    end
    --]]
end

function AlertOptions:ClearEnemyCombat()
    self.ChatEnemyCombatPhraseTextBox:SetText("")
    self.ChatEnemyCombatInstanceButton:SetText("All")
    self.ChatEnemyCombatRoleButton:SetText("All")

    -- Alert
    self.ChatEnemyCombatAlertCheckBox:SetChecked(false)
    self.ChatEnemyCombatAlertTextBox:SetText("")
    self.ChatEnemyCombatAlertDurationTextBox:SetText("")
    self.ChatEnemyCombatAlertCountdownCheckBox:SetChecked(false)
    self.ChatEnemyCombatAlertSoundLabel:SetChecked(false)
    self.ChatEnemyCombatSoundeffectButton:SetText("")
    self.ChatEnemyCombatAlertIconLabel:SetChecked(false)
    self.ChatEnemyCombatAlertIconBox:SetBackground(BackgroundHandler:GetBackground(0x41007F41))
    self.ChatEnemyCombatAlertIconTextBox:SetText(0x41007F41)

    -- Timer
    self.ChatEnemyCombatTimerCheckBox:SetChecked(false)
    self.ChatEnemyCombatTimerDurationTextBox:SetText("")
    self.ChatEnemyCombatTimerIconLabel:SetChecked(false)
    self.ChatEnemyCombatTimerIconBox:SetBackground(BackgroundHandler:GetBackground(0x41007F41))
    self.ChatEnemyCombatTimerIconTextBox:SetText(0x41007F41)
    self.ChatEnemyCombatTimerTextBox:SetText("")
    self.ChatEnemyCombatTimerUnique:SetChecked(false)
    self.ChatEnemyCombatTimerLoop:SetChecked(false)
end

function AlertOptions:FillChatEnemyCombatOptions(source, input)
    self:ToggleWindow("Chat (Combat Enemy)")
     --1 duration, 2 message, 3 icon, 4 countdown, 5 soundeffect, 6 instanceID, 7 role, 8 timerduration, 9 timer text, 10 timer icon, 11 unique, 12 loop, 13 windowID
    -- Trigger

    local timer = input["Timers"][1]
    local alert = input["Alerts"][1]
    local restrict = input["Restrictions"]

    self.ChatEnemyCombatPhraseTextBox:SetText(tostring(source))
    if restrict[1] then self.ChatEnemyCombatInstanceButton:SetText(tostring(InstanceController:GetDungeonNameTable()[tonumber(restrict[1])])) else self.ChatEnemyCombatInstanceButton:SetText("All") end
    if restrict[2] then self.ChatEnemyCombatRoleButton:SetText(Roles:GetAvailableRoles()[restrict[2]]) else self.ChatEnemyCombatRoleButton:SetText("All") end
    

    -- Alert
    self.ChatEnemyCombatAlertCheckBox:SetChecked(alert[1])
    if alert[2] then self.ChatEnemyCombatAlertTextBox:SetText(tostring(alert[2])) else self.ChatEnemyCombatAlertTextBox:SetText("") end
    self.ChatEnemyCombatAlertDurationTextBox:SetText(tonumber(alert[1]))
    self.ChatEnemyCombatAlertCountdownCheckBox:SetChecked(alert[4])
    self.ChatEnemyCombatAlertSoundLabel:SetChecked(alert[5])
    self.ChatEnemyCombatSoundeffectButton:SetText(alert[5])
    self.ChatEnemyCombatAlertIconLabel:SetChecked(alert[3])
    if alert[3] then
        self.ChatEnemyCombatAlertIconBox:SetBackground(BackgroundHandler:GetBackground(tonumber(alert[3])))
        self.ChatEnemyCombatAlertIconTextBox:SetText(tonumber(alert[3]))
    else
        self.ChatEnemyCombatAlertIconBox:SetBackground("")
        self.ChatEnemyCombatAlertIconTextBox:SetText("")
    end
    if alert[3] then self.ChatEnemyCombatAlertIconTextBox:SetText(tostring(alert[3])) else self.ChatEnemyCombatAlertIconTextBox:SetText("") end

    -- Timer
    self.ChatEnemyCombatTimerCheckBox:SetChecked(timer[1])
    self.ChatEnemyCombatTimerDurationTextBox:SetText(tonumber(timer[1]))
    self.ChatEnemyCombatTimerIconLabel:SetChecked(timer[3])
    if timer[3] then
        self.ChatEnemyCombatTimerIconBox:SetBackground(BackgroundHandler:GetBackground(tonumber(timer[3])))
        self.ChatEnemyCombatTimerIconTextBox:SetText(tonumber(timer[3]))
    else
        self.ChatEnemyCombatTimerIconBox:SetBackground("")
        self.ChatEnemyCombatTimerIconTextBox:SetText("")
    end
    if timer[2] then self.ChatEnemyCombatTimerTextBox:SetText(tostring(timer[2])) else self.ChatEnemyCombatTimerTextBox:SetText("") end
    self.ChatEnemyCombatTimerUnique:SetChecked(timer[4])
    self.ChatEnemyCombatTimerLoop:SetChecked(timer[5])
end

function AlertOptions:iconOverview()

    self.iconWindow = Turbine.UI.Window()
    self.iconWindow:SetSize(200, 200)
    self.iconWindow:SetPosition(300, 215)
    self.iconWindow:SetVisible(false)
    self.iconWindow:SetBackColor(Turbine.UI.Color(0, 0, 0))

    self.iconWindow.VisibleChanged = function()
        if self.iconWindow:IsVisible() then
            self.iconWindow:SetZOrder(100)
            self.iconWindow:SetPosition(self.iconWindow:GetMousePosition())
        end
    end

    self.IconWindowHeaderLabel = Turbine.UI.Label()
    self.IconWindowHeaderLabel:SetParent(self.iconWindow)
    self.IconWindowHeaderLabel:SetSize(200 - 2, 20 - 1)
    self.IconWindowHeaderLabel:SetPosition(1, 1)
    --self.IconWindowHeaderLabel:SetMouseVisible(false)
    --self.IconWindowHeaderLabel:SetTextAlignment(Turbine.UI.ContentAlignment.LeftCenter)
    self.IconWindowHeaderLabel:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
    self.IconWindowHeaderLabel:SetMultiline(true)
    self.IconWindowHeaderLabel:SetText("Icon selection")

    
    self.Moving = false

    self.IconWindowHeaderLabel.MouseDown = function(sender,args)
        self.MoveX = args.X
        self.MoveY = args.Y
        self.Moving = true
    end

    self.IconWindowHeaderLabel.MouseUp = function()
        self.Moving = false
    end

    self.IconWindowHeaderLabel.MouseMove = function(sender,args)
        if self.Moving then
            local newLeft = self.iconWindow:GetLeft() - (self.MoveX - args.X)
            local newTop = self.iconWindow:GetTop() - (self.MoveY - args.Y)
            self.iconWindow:SetPosition(newLeft,newTop)
        end
    end

    -- Close Button
    self.IconCloseBackground = Turbine.UI.Control()
    self.IconCloseBackground:SetParent(self.iconWindow)
    self.IconCloseBackground:SetPosition(200 - 20, 0)
    self.IconCloseBackground:SetSize(20, 20)
    self.IconCloseBackground:SetMouseVisible(true)
    self.IconCloseBackground:SetBackColor(Turbine.UI.Color(153/255, 0, 0))


    self.IconCloseBackground.MouseClick = function()
        self.iconWindow:SetVisible(false)
    end

    self.iconListBox = Turbine.UI.ListBox()
    self.iconListBox:SetSize(200 - 2, 180 - 2)
    self.iconListBox:SetPosition(1, 21)
    self.iconListBox:SetParent(self.iconWindow)
    self.iconListBox:SetMouseVisible(false)
    self.iconListBox:SetBackColor(Turbine.UI.Color(0.250, 0.250, 0.250))
    self.iconListBox:SetOrientation( Turbine.UI.Orientation.Horizontal )
    self.iconListBox:SetMaxColumns(6)

    self.iconListBoxScrollbar = Turbine.UI.Lotro.ScrollBar()
    self.iconListBoxScrollbar:SetOrientation(Turbine.UI.Orientation.Vertical)
    self.iconListBoxScrollbar:SetParent(self.iconWindow)
    self.iconListBoxScrollbar:SetPosition(192, 21)
    self.iconListBoxScrollbar:SetSize(8, 178)
    self.iconListBox:SetVerticalScrollBar(self.iconListBoxScrollbar)

    if DataGrabberTool then
        local duplicates = {}
        for key, value in spairs(DataGrabberTool:GetIcons(), function(t,a,b) return tostring(a) > tostring(b) end) do
            if duplicates[value] then
            else
                local icon = Turbine.UI.Control()
                icon:SetSize(32,32)
                icon:SetBlendMode(Turbine.UI.BlendMode.AlphaBlend)
                icon:SetBackground(BackgroundHandler:GetBackground(value))
                icon:SetMouseVisible(true)

                icon.MouseClick = function()
                    if self.iconLink then
                        self.iconLink:SetBackground(BackgroundHandler:GetBackground(value))
                        self.iconTextLink:SetText(value)
                        self.iconID = value
                        
                    end
                    self.iconWindow:SetVisible(false)

                end

                self.iconListBox:AddItem(icon)
                duplicates[value] = true


            end
        end

        duplicates = nil
    end

    return self.iconWindow
end

function AlertOptions:HistoricalEffectOverview()
    self.HistoricalEffectsWindow = Turbine.UI.Window()
    self.HistoricalEffectsWindow:SetSize(400, 200)
    self.HistoricalEffectsWindow:SetPosition(300, 215)
    self.HistoricalEffectsWindow:SetVisible(false)
    self.HistoricalEffectsWindow:SetBackColor(Turbine.UI.Color(0, 0, 0))

    self.HistoricalEffectsWindow.VisibleChanged = function()
        if self.HistoricalEffectsWindow:IsVisible() then
            self.HistoricalEffectsWindow:SetZOrder(100)
            self.HistoricalEffectsWindow:SetPosition(self.HistoricalEffectsWindow:GetMousePosition())
        end
    end

    self.HistoricalEffectsWindowHeaderLabel = Turbine.UI.Label()
    self.HistoricalEffectsWindowHeaderLabel:SetParent(self.HistoricalEffectsWindow)
    self.HistoricalEffectsWindowHeaderLabel:SetSize(400 - 2, 20 - 1)
    self.HistoricalEffectsWindowHeaderLabel:SetPosition(1, 1)
    --self.HistoricalEffectsWindowHeaderLabel:SetMouseVisible(false)
    --self.HistoricalEffectsWindowHeaderLabel:SetTextAlignment(Turbine.UI.ContentAlignment.LeftCenter)
    self.HistoricalEffectsWindowHeaderLabel:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
    self.HistoricalEffectsWindowHeaderLabel:SetMultiline(true)
    self.HistoricalEffectsWindowHeaderLabel:SetText("HistoricalEffects selection")

    
    self.Moving = false

    self.HistoricalEffectsWindowHeaderLabel.MouseDown = function(sender,args)
        self.MoveX = args.X
        self.MoveY = args.Y
        self.Moving = true
    end

    self.HistoricalEffectsWindowHeaderLabel.MouseUp = function()
        self.Moving = false
    end

    self.HistoricalEffectsWindowHeaderLabel.MouseMove = function(sender,args)
        if self.Moving then
            local newLeft = self.HistoricalEffectsWindow:GetLeft() - (self.MoveX - args.X)
            local newTop = self.HistoricalEffectsWindow:GetTop() - (self.MoveY - args.Y)
            self.HistoricalEffectsWindow:SetPosition(newLeft,newTop)
        end
    end

    -- Close Button
    self.HistoricalEffectsCloseBackground = Turbine.UI.Control()
    self.HistoricalEffectsCloseBackground:SetParent(self.HistoricalEffectsWindow)
    self.HistoricalEffectsCloseBackground:SetPosition(400 - 20, 0)
    self.HistoricalEffectsCloseBackground:SetSize(20, 20)
    self.HistoricalEffectsCloseBackground:SetMouseVisible(true)
    self.HistoricalEffectsCloseBackground:SetBackColor(Turbine.UI.Color(153/255, 0, 0))


    self.HistoricalEffectsCloseBackground.MouseClick = function()
        self.HistoricalEffectsWindow:SetVisible(false)
    end

    self.HistoricalEffectsListBox = Turbine.UI.ListBox()
    self.HistoricalEffectsListBox:SetSize(400 - 2, 180 - 2)
    self.HistoricalEffectsListBox:SetPosition(1, 21)
    self.HistoricalEffectsListBox:SetParent(self.HistoricalEffectsWindow)
    self.HistoricalEffectsListBox:SetMouseVisible(false)
    self.HistoricalEffectsListBox:SetBackColor(Turbine.UI.Color(0.250, 0.250, 0.250))
    self.HistoricalEffectsListBox:SetOrientation( Turbine.UI.Orientation.Horizontal )
    self.HistoricalEffectsListBox:SetMaxColumns(1)

    self.HistoricalEffectsListBoxScrollbar = Turbine.UI.Lotro.ScrollBar()
    self.HistoricalEffectsListBoxScrollbar:SetOrientation(Turbine.UI.Orientation.Vertical)
    self.HistoricalEffectsListBoxScrollbar:SetParent(self.HistoricalEffectsWindow)
    self.HistoricalEffectsListBoxScrollbar:SetPosition(400 - 8, 21)
    self.HistoricalEffectsListBoxScrollbar:SetSize(8, 178)
    self.HistoricalEffectsListBox:SetVerticalScrollBar(self.HistoricalEffectsListBoxScrollbar)

    if DataGrabberTool then
        local duplicates = {}
        local overviewFunction = function(effects)
            for key, value in spairs(effects, function(t,a,b) return a < b end) do
                if duplicates[value] then
                else
                    local HistoricalEffects = Turbine.UI.Control()
                    HistoricalEffects:SetSize(392,32)
                    HistoricalEffects:SetBlendMode(Turbine.UI.BlendMode.AlphaBlend)
                    --HistoricalEffects:SetBackground(BackgroundHandler:GetBackground(value[4]))
                    HistoricalEffects:SetMouseVisible(true)
    
                    HistoricalEffects.MouseEnter = function()
                        HistoricalEffects:SetBackColor(Turbine.UI.Color(0.5,0.5,0.5))
                    end
    
                    HistoricalEffects.MouseLeave = function()
                        HistoricalEffects:SetBackColor(nil)
                    end
    
                    local HistoricalEffectsIcon = Turbine.UI.Control()
                    HistoricalEffectsIcon:SetSize(32,32)
                    HistoricalEffectsIcon:SetParent(HistoricalEffects)
                    HistoricalEffectsIcon:SetBlendMode(Turbine.UI.BlendMode.AlphaBlend)
                    HistoricalEffectsIcon:SetBackground(BackgroundHandler:GetBackground(value[4]))
                    HistoricalEffectsIcon:SetMouseVisible(false)
    
                    local HistoricalEffectsName = Turbine.UI.Label()
                    HistoricalEffectsName:SetSize(392 - 32,32)
                    HistoricalEffectsName:SetParent(HistoricalEffects)
                    HistoricalEffectsName:SetPosition(32, 0)
                    HistoricalEffectsName:SetBlendMode(Turbine.UI.BlendMode.AlphaBlend)
                    HistoricalEffectsName:SetTextAlignment(Turbine.UI.ContentAlignment.TopLeft)
                    HistoricalEffectsName:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
                    local duration = ""
                    if value[3] < 999999 then
                        duration = " Dur: " .. tostring(value[3]) .. "s"
                    end
                    HistoricalEffectsName:SetText(key .. duration)
                    HistoricalEffectsName:SetMouseVisible(false)
    
    
                    local HistoricalEffectsName = Turbine.UI.Label()
                    HistoricalEffectsName:SetSize(392 - 32,32)
                    HistoricalEffectsName:SetParent(HistoricalEffects)
                    HistoricalEffectsName:SetPosition(32, 0)
                    HistoricalEffectsName:SetBlendMode(Turbine.UI.BlendMode.AlphaBlend)
                    HistoricalEffectsName:SetTextAlignment(Turbine.UI.ContentAlignment.BottomLeft)
                    HistoricalEffectsName:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
                    HistoricalEffectsName:SetText(tostring(value[2]))
                    HistoricalEffectsName:SetMouseVisible(false)
                    HistoricalEffectsName:SetMultiline(false)
    
                    HistoricalEffects.MouseClick = function()
                        if self.HistoricalEffectsTextLink then
                            --self.HistoricalEffectsLink:SetBackground(BackgroundHandler:GetBackground(value))
                            self.HistoricalEffectsTextLink:SetText(key)
                            self.HistoricalEffectsID = key
                            
                        end
                        self.HistoricalEffectsWindow:SetVisible(false)
    
                    end
    
                    self.HistoricalEffectsListBox:AddItem(HistoricalEffects)
                    duplicates[value] = true
    
    
                end
            end
    
            duplicates = nil
        end
        DataGrabberTool:GetEffects(overviewFunction)

    end

    return self.HistoricalEffectsWindow
end

function AlertOptions:AddTreeNode(NodeList, width, key, fillFunc, topNode)
    local node = UI.GetNiceTreeNode(width, key, 
    function()
        fillFunc()
    end)

    node.name = key
    NodeList:Add(node)

    if topNode then
        topNode:Collapse()
        topNode:Expand()
    end
end

function AlertOptions:AddChatNode(NodeList, source, trigger, value)
    for i = 1, NodeList:GetCount() do
        if NodeList:Get(i).name == source then 
            local nameNode = NodeList:Get(i):GetChildNodes()
            for j = 1, nameNode:GetCount() do
                if nameNode:Get(j).name == trigger then 
                    ChatOutput("Existing Node found")
                    return
                end
                --Add node

                local KeyNodes = NodeList:Get(i):GetChildNodes()
            
                local AlertNode = Turbine.UI.TreeNode()
                AlertNode:SetSize(self.CurrentAlertBackground:GetWidth() - 10, 20 + 2)

                AlertNode.SourceLabel = Turbine.UI.Label()
                AlertNode.SourceLabel:SetParent(AlertNode)
                AlertNode.SourceLabel:SetPosition(1,1)
                AlertNode.SourceLabel:SetSize(self.CurrentAlertBackground:GetWidth() - 10- 2, 20)
                AlertNode.SourceLabel:SetBlendMode(Turbine.UI.BlendMode.AlphaBlend)
                AlertNode.SourceLabel:SetFont(Turbine.UI.Lotro.Font.BookAntiquaBold16)
                AlertNode.SourceLabel:SetMouseVisible(true)
        
                AlertNode.LefLabel = Turbine.UI.Label()
                AlertNode.LefLabel:SetParent(AlertNode)
                AlertNode.LefLabel:SetPosition(0, 0)
                AlertNode.LefLabel:SetSize(30, 20)
                AlertNode.LefLabel:SetBlendMode(Turbine.UI.BlendMode.AlphaBlend)
                AlertNode.LefLabel:SetBackground(0x4111D1F9)
                AlertNode.LefLabel:SetMouseVisible(false)
        
                AlertNode.RigthLabel = Turbine.UI.Label()
                AlertNode.RigthLabel:SetParent(AlertNode)
                AlertNode.RigthLabel:SetPosition(self.CurrentAlertBackground:GetWidth() - 157, 0)
                AlertNode.RigthLabel:SetSize(147, 20)
                AlertNode.RigthLabel:SetBlendMode(Turbine.UI.BlendMode.AlphaBlend)
                AlertNode.RigthLabel:SetBackground(0x4111D1F9)
                AlertNode.RigthLabel:SetMouseVisible(false)
        
                AlertNode.MidLabel = Turbine.UI.Label()
                AlertNode.MidLabel:SetParent(AlertNode)
                AlertNode.MidLabel:SetPosition(30, 0)
                AlertNode.MidLabel:SetSize(self.CurrentAlertBackground:GetWidth() - 70, 20)
                AlertNode.MidLabel:SetBlendMode(Turbine.UI.BlendMode.AlphaBlend)
                AlertNode.MidLabel:SetBackground(0x4111D200)
                AlertNode.MidLabel:SetMouseVisible(false)
        
                AlertNode.textLabel = Turbine.UI.Label()
                AlertNode.textLabel:SetParent(AlertNode)
                AlertNode.textLabel:SetPosition(1,1)
                AlertNode.textLabel:SetSize(self.CurrentAlertBackground:GetWidth() - 10- 2, 20)
                AlertNode.textLabel:SetBlendMode(Turbine.UI.BlendMode.AlphaBlend)
                AlertNode.textLabel:SetFont(Turbine.UI.Lotro.Font.BookAntiquaBold19)
                AlertNode.textLabel:SetMouseVisible(false)
                AlertNode.textLabel:SetText(trigger)
        
                AlertNode.SourceLabel.MouseDown = function()
                    AlertNode.LefLabel:SetBackground(0x4111D1FE)
                    AlertNode.RigthLabel:SetBackground(0x4111D1FE)
                    AlertNode.MidLabel:SetBackground(0x4111D205)
                end
        
                AlertNode.SourceLabel.MouseUp = function()
                    AlertNode.LefLabel:SetBackground(0x4111D1F9)
                    AlertNode.RigthLabel:SetBackground(0x4111D1F9)
                    AlertNode.MidLabel:SetBackground(0x4111D200)
                end

                AlertNode.name = trigger

                KeyNodes:Add(AlertNode)

                AlertNode.SourceLabel.MouseClick = function()
                    self:FillChatOptions(source, trigger, value)
                end
                self.ChatListBackground:Collapse()
                self.ChatListBackground:Expand()
                NodeList:Get(i):Collapse()
                NodeList:Get(i):Expand()
                return

            end
        end
        -- Add nodelist + node
    end
    local SourceNode = Turbine.UI.TreeNode()
    SourceNode:SetSize(self.CurrentAlertBackground:GetWidth() - 10, 20 + 2)

    SourceNode.SourceLabel = Turbine.UI.Label()
    SourceNode.SourceLabel:SetParent(SourceNode)
    SourceNode.SourceLabel:SetPosition(1,1)
    SourceNode.SourceLabel:SetSize(self.CurrentAlertBackground:GetWidth() - 10- 2, 20)
    SourceNode.SourceLabel:SetBlendMode(Turbine.UI.BlendMode.AlphaBlend)
    SourceNode.SourceLabel:SetFont(Turbine.UI.Lotro.Font.BookAntiquaBold24)
    SourceNode.SourceLabel:SetMouseVisible(true)

    SourceNode.LefLabel = Turbine.UI.Label()
    SourceNode.LefLabel:SetParent(SourceNode)
    SourceNode.LefLabel:SetPosition(0, 0)
    SourceNode.LefLabel:SetSize(30, 20)
    SourceNode.LefLabel:SetBlendMode(Turbine.UI.BlendMode.AlphaBlend)
    SourceNode.LefLabel:SetBackground(BackgroundHandler:GetBackground(0x4111D1FA))
    SourceNode.LefLabel:SetMouseVisible(false)

    SourceNode.RigthLabel = Turbine.UI.Label()
    SourceNode.RigthLabel:SetParent(SourceNode)
    SourceNode.RigthLabel:SetPosition(self.CurrentAlertBackground:GetWidth() - 157, 0)
    SourceNode.RigthLabel:SetSize(147, 20)
    SourceNode.RigthLabel:SetBlendMode(Turbine.UI.BlendMode.AlphaBlend)
    SourceNode.RigthLabel:SetBackground(BackgroundHandler:GetBackground(0x4111D1FA))
    SourceNode.RigthLabel:SetMouseVisible(false)

    SourceNode.MidLabel = Turbine.UI.Label()
    SourceNode.MidLabel:SetParent(SourceNode)
    SourceNode.MidLabel:SetPosition(30, 0)
    SourceNode.MidLabel:SetSize(self.CurrentAlertBackground:GetWidth() - 70, 20)
    SourceNode.MidLabel:SetBlendMode(Turbine.UI.BlendMode.AlphaBlend)
    SourceNode.MidLabel:SetBackground(BackgroundHandler:GetBackground(0x4111D201))
    SourceNode.MidLabel:SetMouseVisible(false)

    SourceNode.textLabel = Turbine.UI.Label()
    SourceNode.textLabel:SetParent(SourceNode)
    SourceNode.textLabel:SetPosition(1,1)
    SourceNode.textLabel:SetSize(self.CurrentAlertBackground:GetWidth() - 10- 2, 20)
    SourceNode.textLabel:SetBlendMode(Turbine.UI.BlendMode.AlphaBlend)
    SourceNode.textLabel:SetFont(Turbine.UI.Lotro.Font.BookAntiquaBold19)
    SourceNode.textLabel:SetMouseVisible(false)
    SourceNode.textLabel:SetText(source)

    SourceNode.SourceLabel.MouseEnter = function()
        SourceNode.LefLabel:SetBackground(BackgroundHandler:GetBackground(0x4111D1FB))
        SourceNode.RigthLabel:SetBackground(BackgroundHandler:GetBackground(0x4111D1FB))
        SourceNode.MidLabel:SetBackground(BackgroundHandler:GetBackground(0x4111D202))
    end

    SourceNode.SourceLabel.MouseLeave = function()
        SourceNode.LefLabel:SetBackground(BackgroundHandler:GetBackground(0x4111D1FA))
        SourceNode.RigthLabel:SetBackground(BackgroundHandler:GetBackground(0x4111D1FA))
        SourceNode.MidLabel:SetBackground(BackgroundHandler:GetBackground(0x4111D201))
    end

    SourceNode.SourceLabel.MouseDown = function()
        SourceNode.LefLabel:SetBackground(BackgroundHandler:GetBackground(0x4111D1FC))
        SourceNode.RigthLabel:SetBackground(BackgroundHandler:GetBackground(0x4111D1FC))
        SourceNode.MidLabel:SetBackground(BackgroundHandler:GetBackground(0x4111D203))
    end

    SourceNode.SourceLabel.MouseUp = function()
        SourceNode.LefLabel:SetBackground(BackgroundHandler:GetBackground(0x4111D1FA))
        SourceNode.RigthLabel:SetBackground(BackgroundHandler:GetBackground(0x4111D1FA))
        SourceNode.MidLabel:SetBackground(BackgroundHandler:GetBackground(0x4111D201))
    end

    SourceNode.name = source

    self.chatNode:Add(SourceNode)

    local KeyNodes = SourceNode:GetChildNodes()
    
    local AlertNode = Turbine.UI.TreeNode()
    AlertNode:SetSize(self.CurrentAlertBackground:GetWidth() - 10, 20 + 2)

    AlertNode.SourceLabel = Turbine.UI.Label()
    AlertNode.SourceLabel:SetParent(AlertNode)
    AlertNode.SourceLabel:SetPosition(1,1)
    AlertNode.SourceLabel:SetSize(self.CurrentAlertBackground:GetWidth() - 10- 2, 20)
    AlertNode.SourceLabel:SetBlendMode(Turbine.UI.BlendMode.AlphaBlend)
    AlertNode.SourceLabel:SetFont(Turbine.UI.Lotro.Font.BookAntiquaBold16)
    AlertNode.SourceLabel:SetMouseVisible(true)

    AlertNode.LefLabel = Turbine.UI.Label()
    AlertNode.LefLabel:SetParent(AlertNode)
    AlertNode.LefLabel:SetPosition(0, 0)
    AlertNode.LefLabel:SetSize(30, 20)
    AlertNode.LefLabel:SetBlendMode(Turbine.UI.BlendMode.AlphaBlend)
    AlertNode.LefLabel:SetBackground(0x4111D1F9)
    AlertNode.LefLabel:SetMouseVisible(false)

    AlertNode.RigthLabel = Turbine.UI.Label()
    AlertNode.RigthLabel:SetParent(AlertNode)
    AlertNode.RigthLabel:SetPosition(self.CurrentAlertBackground:GetWidth() - 157, 0)
    AlertNode.RigthLabel:SetSize(147, 20)
    AlertNode.RigthLabel:SetBlendMode(Turbine.UI.BlendMode.AlphaBlend)
    AlertNode.RigthLabel:SetBackground(0x4111D1F9)
    AlertNode.RigthLabel:SetMouseVisible(false)

    AlertNode.MidLabel = Turbine.UI.Label()
    AlertNode.MidLabel:SetParent(AlertNode)
    AlertNode.MidLabel:SetPosition(30, 0)
    AlertNode.MidLabel:SetSize(self.CurrentAlertBackground:GetWidth() - 70, 20)
    AlertNode.MidLabel:SetBlendMode(Turbine.UI.BlendMode.AlphaBlend)
    AlertNode.MidLabel:SetBackground(0x4111D200)
    AlertNode.MidLabel:SetMouseVisible(false)

    AlertNode.textLabel = Turbine.UI.Label()
    AlertNode.textLabel:SetParent(AlertNode)
    AlertNode.textLabel:SetPosition(1,1)
    AlertNode.textLabel:SetSize(self.CurrentAlertBackground:GetWidth() - 10- 2, 20)
    AlertNode.textLabel:SetBlendMode(Turbine.UI.BlendMode.AlphaBlend)
    AlertNode.textLabel:SetFont(Turbine.UI.Lotro.Font.BookAntiquaBold19)
    AlertNode.textLabel:SetMouseVisible(false)
    AlertNode.textLabel:SetText(trigger)

    AlertNode.SourceLabel.MouseDown = function()
        AlertNode.LefLabel:SetBackground(0x4111D1FE)
        AlertNode.RigthLabel:SetBackground(0x4111D1FE)
        AlertNode.MidLabel:SetBackground(0x4111D205)
    end

    AlertNode.SourceLabel.MouseUp = function()
        AlertNode.LefLabel:SetBackground(0x4111D1F9)
        AlertNode.RigthLabel:SetBackground(0x4111D1F9)
        AlertNode.MidLabel:SetBackground(0x4111D200)
    end

    AlertNode.name = trigger

    KeyNodes:Add(AlertNode)

    AlertNode.SourceLabel.MouseClick = function()
        self:FillChatOptions(source, trigger, value)
    end

    self.ChatListBackground:Collapse()
    self.ChatListBackground:Expand()
    SourceNode:Collapse()
    SourceNode:Expand()
end

function AlertOptions:RemoveNode(NodeList, name)
    for i = 1, NodeList:GetCount() do
        if NodeList:Get(i).name == name then 
            NodeList:RemoveAt(i) 
            return
        end
    end
end

function AlertOptions:RemoveChatNode(NodeList, name, phrase)
    for i = 1, NodeList:GetCount() do
        if NodeList:Get(i).name == name then 
            local nameNode = NodeList:Get(i):GetChildNodes()
            for j = 1, nameNode:GetCount() do
                if nameNode:Get(j).name == phrase then 
                    nameNode:RemoveAt(j) 
                    if nameNode:GetCount() == 0 then
                        NodeList:Remove(NodeList:Get(i))
                    end
                    return
                end
            end
            return
        end
    end
end

function AlertOptions:RefreshTreeview()
    self.CurrentAlertContainer:Refresh()
end

AlertWindowOptions = class(Turbine.UI.Control)
function AlertWindowOptions:Constructor(parent)
    local width = 750
    local height = 500
    local font = Settings.LOFont
    local colour = Turbine.UI.Color(1, 1, 1)
    Turbine.UI.Control.Constructor(self)
    self:SetSize(width,height); -- sets the window size to 200 by 200 pixels
    self:SetParent(parent)
    self:SetPosition(0,0)
    self:SetWantsKeyEvents(true)
    self:SetVisible(false)

    self.BorderBackground = Turbine.UI.Control()
    self.BorderBackground:SetParent(self)
    self.BorderBackground:SetPosition(10 - 2, 10 - 2)
    self.BorderBackground:SetSize(730 + 4, 450 + 4)
    self.BorderBackground:SetBackColor(Turbine.UI.Color(0, 0, 0))

    self.Background = Turbine.UI.Control()
    self.Background:SetParent(self)
    self.Background:SetPosition(10, 10)
    self.Background:SetSize(730, 450)
    self.Background:SetBackColor(Turbine.UI.Color(0.250, 0.250, 0.250))

    import "PrimePlugins.RaidTools.EffectSettings"
    self.EffectWindowSettings = EffectSettings(self)
    self.TimerWindowSettings = Timers.BossTimerSettings(self)
    import "PrimePlugins.RaidTools.AlertSettings"
    self.GroupEffectWindowsSettings = AlertSettings(self)
    
    self.SelfButton = Turbine.UI.Lotro.Button()
    self.SelfButton:SetParent(self.Background)
    self.SelfButton:SetPosition(50, 0)
    self.SelfButton:SetSize(200, 20)
    self.SelfButton:SetTextAlignment(Turbine.UI.ContentAlignment.TopCenter)
    self.SelfButton:SetFont(Turbine.UI.Lotro.Font.BookAntiquaBold22)
    self.SelfButton:SetText("Self Alerts")

    self.AlertButton = Turbine.UI.Lotro.Button()
    self.AlertButton:SetParent(self.Background)
    self.AlertButton:SetPosition(250, 0)
    self.AlertButton:SetSize(200, 20)
    self.AlertButton:SetTextAlignment(Turbine.UI.ContentAlignment.TopCenter)
    self.AlertButton:SetFont(Turbine.UI.Lotro.Font.BookAntiquaBold22)
    self.AlertButton:SetText("Group Alerts")

    self.TimerButton = Turbine.UI.Lotro.Button()
    self.TimerButton:SetParent(self.Background)
    self.TimerButton:SetPosition(450, 0)
    self.TimerButton:SetSize(200, 20)
    self.TimerButton:SetTextAlignment(Turbine.UI.ContentAlignment.TopCenter)
    self.TimerButton:SetFont(Turbine.UI.Lotro.Font.BookAntiquaBold22)
    self.TimerButton:SetText("Timers")

    self.TimerButton.Click = function()
        self.EffectWindowSettings:SetVisible(false)
        self.GroupEffectWindowsSettings:SetVisible(false)
        self.TimerWindowSettings:SetVisible(true)
    end

    self.AlertButton.Click = function()
        self.EffectWindowSettings:SetVisible(false)
        self.GroupEffectWindowsSettings:SetVisible(true)
        self.TimerWindowSettings:SetVisible(false)
    end

    self.SelfButton.Click = function()
        self.EffectWindowSettings:SetVisible(true)
        self.GroupEffectWindowsSettings:SetVisible(false)
        self.TimerWindowSettings:SetVisible(false)
    end

end